// -*- java -*- to avoid jde mode, which is slow on this big a file on
// one of my boxes

/*
The contents of this file are subject to the THDL Open Community License
Version 1.0 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License on the THDL web site 
(http://www.thdl.org/).

Software distributed under the License is distributed on an "AS IS" basis, 
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the 
License for the specific terms governing rights and limitations under the 
License. 

The Initial Developer of this software is the Tibetan and Himalayan Digital
Library (THDL). Portions created by the THDL are Copyright 2003 THDL.
All Rights Reserved. 

Contributor(s): ______________________________________.
*/

package org.thdl.tib.text.ttt;

import org.thdl.util.ThdlOptions;

import junit.framework.TestCase;


/** Tests ACIP-to-Tibetan conversions using tsheg bars from real ACIP
 *  files.  Lots of tsheg bars.
 *
 *  <p>I took the publicly available ACIP texts from the Kangyur,
 *  Tengyur, and Sungbum and ran the ACIP=>Unicode converter on them
 *  early in the AM on Sunday October 26, 2003.  I did not run on
 *  English texts (that I know of), just 100% Tibetan.  Some of the
 *  100% Tibetan texts from the Sungbum and Tengyur (no more than 10
 *  or 20) had more than 250 lex errors in them, so they didn't get
 *  counted here -- fully, anyway.  These were in this list (which
 *  also contains at least one 100% English text): S0022M.ACT,
 *  S0023N3.ACT, S0034E.ACE, S0036F.ACE, S0038F.ACE, S0039F.ACE,
 *  S0041N.ACE, S0071A.ACE, S0192M.ACT, S0199A.ACE, S0200A.ACE,
 *  S0205A.INE, S0207I.ACT, S0212I.ACT, S0213M.ACT, S0270I.INC,
 *  S0291M.ACT, S0319M.ACT, S0986I2.INC, S0986I3.INC, SL6850I2.ACT,
 *  TD4105E.ACT.
 *
 *  <p>I had the converter output all tsheg bars using the
 *  org.thdl.tib.text.ttt.OutputAllTshegBars option.  I then ran 'sort
 *  -d' on that, piped it to 'uniq -c', and ran <code>awk -- '{y =
 *  sprintf("\"%s", $2); x = sprintf(" th(%10s\");\t// %d", y, $1);
 *  print x;}'</code> on that.  And you get the following.
 *
 *  <p>There were approximately 35,253,990 tsheg bars counted.  Note
 *  that I use the term <i>tsheg bar</i> to refer even to
 *  polysyllablic, non-native patches that literally occur between
 *  tsheg-like things.
 *
 *  <p>FIXME: The top 66 tsheg bars, those used 100,000 or more times
 *  apiece, account for 56% of total tsheg bars.  The top 449 tsheg
 *  bars, those used 10,000 times apiece, account for 90% of total
 *  tsheg bars.  This means that performance can be much improved by
 *  special-casing just 449 tsheg bars.
 *
 *  @author David Chandler */
public class LotsOfTshegBarsTest extends TestCase {

    /** Invokes a text UI and runs all this class's tests. */
	public static void main(String[] args) {
		junit.textui.TestRunner.run(LotsOfTshegBarsTest.class);
	}

    protected void setUp() {
        // We don't want to use options.txt:
        ThdlOptions.forTestingOnlyInitializeWithoutDefaultOptionsFile();

        // We don't want to load the TM or TMW font files ourselves:
        ThdlOptions.setUserPreference("thdl.rely.on.system.tmw.fonts", true);
        ThdlOptions.setUserPreference("thdl.rely.on.system.tm.fonts", true);
        ThdlOptions.setUserPreference("thdl.debug", true);
    }


    public LotsOfTshegBarsTest() { }

    private void th(String x) { PackageTest.tstHelper(x); }

    /** Tests some of the tsheg bars that appear only once (likely
     *  these are typos or non-native). */
    public void test1to1a() {
        th(       "''");	// 1
        th(       "00");	// 1
        th(      "004");	// 1
        th(     "004A");	// 1
        th(      "005");	// 1
        th(      "006");	// 1
        th(      "007");	// 1
        th(      "008");	// 1
        th(    "-008A");	// 1
        th(      "009");	// 1
        th(   "00BCOM");	// 1
        th(       "01");	// 1
        th(      "010");	// 1
        th(      "011");	// 1
        th(      "012");	// 1
        th(      "013");	// 1
        th(      "014");	// 1
        th(      "015");	// 1
        th(      "016");	// 1
        th(      "017");	// 1
        th(      "018");	// 1
        th(      "019");	// 1
        th(      "020");	// 1
        th(      "021");	// 1
        th(      "022");	// 1
        th(      "023");	// 1
        th(      "024");	// 1
        th(      "025");	// 1
        th(      "026");	// 1
        th(      "027");	// 1
        th(      "028");	// 1
        th(      "029");	// 1
        th(      "02A");	// 1
        th(      "030");	// 1
        th(      "031");	// 1
        th(      "032");	// 1
        th(      "033");	// 1
        th(      "034");	// 1
        th(      "035");	// 1
        th(      "036");	// 1
        th(      "037");	// 1
        th(      "038");	// 1
        th(      "039");	// 1
        th(     "039A");	// 1
        th(      "03A");	// 1
        th(      "03B");	// 1
        th(      "040");	// 1
        th(      "041");	// 1
        th(      "042");	// 1
        th(      "043");	// 1
        th(      "044");	// 1
        th(      "045");	// 1
        th(      "046");	// 1
        th(      "047");	// 1
        th(      "048");	// 1
        th(      "049");	// 1
        th(      "050");	// 1
        th(      "051");	// 1
        th(      "052");	// 1
        th(      "053");	// 1
        th(      "054");	// 1
        th(      "055");	// 1
        th(      "056");	// 1
        th(      "057");	// 1
        th(      "058");	// 1
        th(      "059");	// 1
        th(       "06");	// 1
        th(      "060");	// 1
        th(      "061");	// 1
        th(      "062");	// 1
        th(      "063");	// 1
        th(      "064");	// 1
        th(      "066");	// 1
        th(      "067");	// 1
        th(      "068");	// 1
        th(      "069");	// 1
        th(      "070");	// 1
        th(      "071");	// 1
        th(      "072");	// 1
        th(      "073");	// 1
        th(      "074");	// 1
        th(      "075");	// 1
        th(      "076");	// 1
        th(      "077");	// 1
        th(      "078");	// 1
        th(      "079");	// 1
        th(      "080");	// 1
        th(      "081");	// 1
        th(      "082");	// 1
        th(      "083");	// 1
        th(      "084");	// 1
        th(      "085");	// 1
        th(      "086");	// 1
        th(      "087");	// 1
        th(      "088");	// 1
        th(      "089");	// 1
        th(      "08B");	// 1
        th(      "090");	// 1
        th(      "091");	// 1
        th(      "092");	// 1
        th(      "093");	// 1
        th(      "094");	// 1
        th(      "095");	// 1
        th(      "096");	// 1
        th(      "097");	// 1
        th(      "098");	// 1
        th(      "099");	// 1
        th(      "0BA");	// 1
        th(   "0BSHAD");	// 1
        th(    "0DANG");	// 1
        th(   "0KYANG");	// 1
        th(      "0LA");	// 1
        th(       "0M");	// 1
        th(      "0PA");	// 1
        th(    "0PA'I");	// 1
        th(     "0PAR");	// 1
        th( "10000000");	// 1
        th("1000000000000");	// 1
        th(    "100'A");	// 1
        th(   "100BA1");	// 1
        th(   "100THO");	// 1
        th(    "101BA");	// 1
        th(   "101BA1");	// 1
        th(    "101DO");	// 1
        th(    "101NA");	// 1
        th(    "102NA");	// 1
        th(    "102NO");	// 1
        th(     "1033");	// 1
        th(    "103BA");	// 1
        th(    "103N4");	// 1
        th(    "103NA");	// 1
        th(    "103PO");	// 1
        th(    "104BA");	// 1
        th(   "104BA4");	// 1
        th(   "104PHO");	// 1
        th(    "105BA");	// 1
        th(    "105BO");	// 1
        th(   "105NA6");	// 1
        th(    "106MO");	// 1
        th(    "106N3");	// 1
        th(    "106NA");	// 1
        th(    "10766");	// 1
        th(    "107BA");	// 1
        th(   "107TZO");	// 1
        th(    "108NA");	// 1
        th(   "108NA3");	// 1
        th(   "108TSO");	// 1
        th(    "109BA");	// 1
        th(   "109BA2");	// 1
        th(   "109DZO");	// 1
        th(    "109NA");	// 1
        th(   "109NA6");	// 1
        th(     "10B4");	// 1
        th(     "10BA");	// 1
        th(    "10BA2");	// 1
        th(    "10BA5");	// 1
        th(     "10N4");	// 1
        th(     "10NA");	// 1
        th(    "10NA4");	// 1
        th(    "10THA");	// 1
        th(    "110BA");	// 1
        th(   "110BA6");	// 1
        th(    "110WO");	// 1
        th(    "11-15");	// 1
        th(    "111BA");	// 1
        th(    "111NA");	// 1
        th(   "111NA6");	// 1
        th(   "111ZHO");	// 1
        th(  "112-118");	// 1
        th(    "11231");	// 1
        th(    "112BA");	// 1
        th(    "112N1");	// 1
        th(    "112N3");	// 1
        th(   "112NA5");	// 1
        th(    "112ZO");	// 1
        th(     "1137");	// 1
        th(     "1138");	// 1
        th(    "113BA");	// 1
        th(    "113NA");	// 1
        th(   "113NA7");	// 1
        th(    "113'O");	// 1
        th(   "113tA1");	// 1
        th(    "114'A");	// 1
        th(    "114NA");	// 1
        th(    "114YO");	// 1
        th(    "115NA");	// 1
        th(    "115RO");	// 1
        th(     "116A");	// 1
        th(    "116LO");	// 1
        th(    "116NA");	// 1
        th(   "116NGA");	// 1
        th(      "117");	// 1
        th(    "117BA");	// 1
        th(    "117NA");	// 1
        th(   "117NA2");	// 1
        th(   "117NA3");	// 1
        th(   "117NA5");	// 1
        th(   "117SHO");	// 1
        th(    "11801");	// 1
        th(  "118-122");	// 1
        th(    "118B1");	// 1
        th(    "118B3");	// 1
        th(    "118BA");	// 1
        th(    "118SO");	// 1
        th(      "119");	// 1
        th(    "119BA");	// 1
        th(    "119HO");	// 1
        th(    "119NA");	// 1
        th(     "11DA");	// 1
        th(     "11NA");	// 1
        th(    "11NA2");	// 1
        th(    "120AO");	// 1
        th(    "120NA");	// 1
        th(   "121KRA");	// 1
        th(    "121NA");	// 1
        th(  "122-127");	// 1
        th(  "122KHRA");	// 1
        th(    "122N2");	// 1
        th(    "122NA");	// 1
        th(    "122PA");	// 1
        th(    "123BA");	// 1
        th(   "123GRA");	// 1
        th(    "123NA");	// 1
        th(    "124BA");	// 1
        th(    "124NA");	// 1
        th(   "124NA3");	// 1
        th(  "124NGRA");	// 1
        th(    "125B1");	// 1
        th(   "125BA1");	// 1
        th(   "125CRA");	// 1
        th(    "125NA");	// 1
        th(    "126BA");	// 1
        th(  "126CHRA");	// 1
        th(    "126NA");	// 1
        th(  "127-130");	// 1
        th(    "127BA");	// 1
        th(   "127BA1");	// 1
        th(   "127BA2");	// 1
        th(   "127BA6");	// 1
        th(   "127JRA");	// 1
        th(    "128B4");	// 1
        th(    "128NA");	// 1
        th(   "128NA2");	// 1
        th(   "128NA3");	// 1
        th(  "128NYRA");	// 1
        th(    "129B4");	// 1
        th(    "129BA");	// 1
        th(   "129NA4");	// 1
        th(   "129TRA");	// 1
        th(    "12BA1");	// 1
        th(    "12NA2");	// 1
        th(    "12NA3");	// 1
        th(    "12NA5");	// 1
        th(    "130N3");	// 1
        th(    "130N5");	// 1
        th(   "130NA3");	// 1
        th(   "130NA5");	// 1
        th(  "130NAPA");	// 1
        th(  "130THRA");	// 1
        th(  "131-136");	// 1
        th(   "131DRA");	// 1
        th(    "131NA");	// 1
        th(   "131NA4");	// 1
        th(    "132B2");	// 1
        th(    "132B3");	// 1
        th(   "132BA2");	// 1
        th(   "132BA3");	// 1
        th(   "132NA6");	// 1
        th(   "132NRA");	// 1
        th(    "133BA");	// 1
        th(   "133PRA");	// 1
        th(    "134BA");	// 1
        th(  "134PHRA");	// 1
        th(    "135BA");	// 1
        th(   "135BRA");	// 1
        th(  "136-138");	// 1
        th(    "136B7");	// 1
        th(   "136MRA");	// 1
        th(    "136NA");	// 1
        th(   "136NA2");	// 1
        th(    "137BA");	// 1
        th(   "137BA4");	// 1
        th(    "137NA");	// 1
        th(  "137TZRA");	// 1
        th(  "138-144");	// 1
        th(    "138BA");	// 1
        th(  "138TSRA");	// 1
        th(    "139BA");	// 1
        th(   "139BA2");	// 1
        th(  "139DZRA");	// 1
        th(    "13BA2");	// 1
        th(    "13NA4");	// 1
        th(    "13NAS");	// 1
        th(     "13PA");	// 1
        th(      "140");	// 1
        th(    "140BA");	// 1
        th(   "140BA4");	// 1
        th(    "140NA");	// 1
        th(   "140WRA");	// 1
        th(    "141BA");	// 1
        th(   "141NAS");	// 1
        th(  "141ZHRA");	// 1
        th(      "142");	// 1
        th(    "142BA");	// 1
        th(   "142NA1");	// 1
        th(   "142ZRA");	// 1
        th(    "143BA");	// 1
        th(    "143NA");	// 1
        th(   "143NA2");	// 1
        th(   "143'RA");	// 1
        th(  "144-148");	// 1
        th(    "144B1");	// 1
        th(   "144YRA");	// 1
        th(   "145RRA");	// 1
        th(      "146");	// 1
        th(    "146BA");	// 1
        th(   "146LRA");	// 1
        th(    "146N7");	// 1
        th(   "147907");	// 1
        th(    "147BA");	// 1
        th(    "147N6");	// 1
        th(    "147NA");	// 1
        th(   "147NA7");	// 1
        th(  "147SHRA");	// 1
        th(  "148-156");	// 1
        th(    "148B1");	// 1
        th(    "148BA");	// 1
        th(   "148DZA");	// 1
        th(   "148SRA");	// 1
        th(    "149BA");	// 1
        th(   "149HRA");	// 1
        th(    "149NA");	// 1
        th(    "14BA5");	// 1
        th(    "14NA2");	// 1
        th(    "14NA5");	// 1
        th(    "14PHA");	// 1
        th(   "150NA7");	// 1
        th(    "15-17");	// 1
        th(   "151NA7");	// 1
        th(    "152BA");	// 1
        th(    "153B3");	// 1
        th(    "153NA");	// 1
        th(    "154BA");	// 1
        th(    "154N7");	// 1
        th(    "156BA");	// 1
        th(    "156NA");	// 1
        th(   "156NA7");	// 1
        th(    "157BA");	// 1
        th(    "157NA");	// 1
        th(      "158");	// 1
        th(    "158B1");	// 1
        th(    "158NA");	// 1
        th(    "159NA");	// 1
        th(    "15BA4");	// 1
        th(    "15BA5");	// 1
        th(    "15BA6");	// 1
        th(    "15NA1");	// 1
        th(      "161");	// 1
        th(     "1623");	// 1
        th(    "162NA");	// 1
        th(   "162NA7");	// 1
        th(    "163BA");	// 1
        th(   "163BA2");	// 1
        th(    "163NA");	// 1
        th(    "165B3");	// 1
        th(    "166B7");	// 1
        th(   "167BA2");	// 1
        th(   "167NA2");	// 1
        th(   "168NA3");	// 1
        th(    "169B7");	// 1
        th(   "169BA2");	// 1
        th(    "169N7");	// 1
        th(     "16CA");	// 1
        th(     "16MA");	// 1
        th(      "1-7");	// 1
        th(    "170BA");	// 1
        th(   "170BA7");	// 1
        th(      "171");	// 1
        th(   "171BA3");	// 1
        th(      "172");	// 1
        th(    "17-22");	// 1
        th(    "172B1");	// 1
        th(    "172BA");	// 1
        th(    "172NA");	// 1
        th(    "173BA");	// 1
        th(    "174BA");	// 1
        th(    "175BA");	// 1
        th(    "175N4");	// 1
        th(    "175NA");	// 1
        th(   "175NA3");	// 1
        th(   "176BA2");	// 1
        th(      "178");	// 1
        th(    "178BA");	// 1
        th(    "178N5");	// 1
        th(    "178NA");	// 1
        th( "178NANYA");	// 1
        th(    "179B2");	// 1
        th(   "179BA1");	// 1
        th(   "179BA2");	// 1
        th(    "179NA");	// 1
        th(   "179NA6");	// 1
        th(     "17LA");	// 1
        th(     "17NA");	// 1
        th(    "17NA3");	// 1
        th(    "17NA5");	// 1
        th(    "17TZA");	// 1
        th(     "1801");	// 1
        th(   "180NA5");	// 1
        th(     "1815");	// 1
        th(    "181BA");	// 1
        th(    "181NA");	// 1
        th(    "182BA");	// 1
        th(    "182CA");	// 1
        th(      "183");	// 1
        th(    "184BA");	// 1
        th(      "185");	// 1
        th(   "185NA7");	// 1
        th(    "186N2");	// 1
        th(    "186NA");	// 1
        th(   "188BA2");	// 1
        th(   "188PAR");	// 1
        th(   "189BA3");	// 1
        th(   "189BA4");	// 1
        th(    "189NA");	// 1
        th(   "189NA4");	// 1
        th(     "18B7");	// 1
        th(     "18BA");	// 1
        th(    "18BA7");	// 1
        th(     "18N6");	// 1
        th(    "18NA3");	// 1
        th(    "18TSA");	// 1
        th(     "1901");	// 1
        th(   "19-20A");	// 1
        th(     "1939");	// 1
        th(   "194BA2");	// 1
        th(   "194NA1");	// 1
        th(     "1954");	// 1
        th(   "195BA7");	// 1
        th(    "195N2");	// 1
        th(   "195NA4");	// 1
        th(     "1961");	// 1
        th(     "1965");	// 1
        th(     "1966");	// 1
        th(     "1968");	// 1
        th(    "196B7");	// 1
        th(   "196BA7");	// 1
        th(    "196PA");	// 1
        th(     "1974");	// 1
        th(     "1975");	// 1
        th(     "1982");	// 1
        th(     "1984");	// 1
        th(     "1985");	// 1
        th(     "1986");	// 1
        th(     "1989");	// 1
        th(      "199");	// 1
        th(     "1991");	// 1
        th(     "1996");	// 1
        th(     "1997");	// 1
        th(     "1998");	// 1
        th(     "19BA");	// 1
        th(   "19BAm2");	// 1
        th(    "19BAR");	// 1
        th(    "19DZA");	// 1
        th(     "19NA");	// 1
        th(   "19NAm2");	// 1
        th(    "1BA14");	// 1
        th(    "1DANG");	// 1
        th(      "1KA");	// 1
        th(    "1MING");	// 1
        th(    "200BA");	// 1
        th(      "201");	// 1
        th(   "201NA1");	// 1
        th(   "201NA7");	// 1
        th(    "201PA");	// 1
        th(     "2028");	// 1
        th(    "203NA");	// 1
        th(      "204");	// 1
        th(     "2067");	// 1
        th(     "2081");	// 1
        th(    "209B7");	// 1
        th(     "20B5");	// 1
        th(     "20BA");	// 1
        th(     "20NA");	// 1
        th(    "20NA6");	// 1
        th(     "20WA");	// 1
        th(     "2105");	// 1
        th(     "2106");	// 1
        th(     "2109");	// 1
        th(     "2111");	// 1
        th(     "2112");	// 1
        th(     "2113");	// 1
        th(    "211B1");	// 1
        th(     "2122");	// 1
        th(   "213BA2");	// 1
        th(   "213NA4");	// 1
        th(   "215BA1");	// 1
        th(   "216BA5");	// 1
        th(   "216NA4");	// 1
        th(    "219N7");	// 1
        th(     "21GA");	// 1
        th(    "21NA5");	// 1
        th(    "21PA7");	// 1
        th(    "21ZHA");	// 1
        th(    "220N2");	// 1
        th(      "221");	// 1
        th(   "221NA6");	// 1
        th(    "222N5");	// 1
        th(   "222NA5");	// 1
        th(    "22-30");	// 1
        th(   "223BA7");	// 1
        th(     "2247");	// 1
        th(    "224B2");	// 1
        th(    "224B3");	// 1
        th(   "225BA7");	// 1
        th(    "227B1");	// 1
        th(      "229");	// 1
        th(     "22N2");	// 1
        th(    "22NA1");	// 1
        th(    "22NA2");	// 1
        th(    "22NA3");	// 1
        th(    "22NA5");	// 1
        th(     "22ZA");	// 1
        th(      "230");	// 1
        th(    "232B2");	// 1
        th(      "233");	// 1
        th(    "235B5");	// 1
        th(   "236BA6");	// 1
        th(    "236N3");	// 1
        th(     "23'A");	// 1
        th(     "23BA");	// 1
        th(     "23NA");	// 1
        th(    "23NA2");	// 1
        th(    "23NA6");	// 1
        th(   "240NA4");	// 1
        th(      "243");	// 1
        th(   "244NA3");	// 1
        th(    "246B1");	// 1
        th(   "247NA3");	// 1
        th(    "249B7");	// 1
        th(   "249BA7");	// 1
        th(    "249N4");	// 1
        th(    "249N5");	// 1
        th(    "249N7");	// 1
        th(    "249N8");	// 1
        th(   "249NA5");	// 1
        th(     "24B3");	// 1
        th(    "24BA2");	// 1
        th(    "24BA3");	// 1
        th(    "24BA6");	// 1
        th(    "24BAR");	// 1
        th(     "24NA");	// 1
        th(    "24NA1");	// 1
        th(    "24NAS");	// 1
        th(     "24YA");	// 1
        th(    "250B7");	// 1
        th(   "250BA7");	// 1
        th(   "250NA1");	// 1
        th(   "250NAS");	// 1
        th(    "250P4");	// 1
        th(    "251N7");	// 1
        th(    "252NA");	// 1
        th(   "252NA1");	// 1
        th(   "252NA6");	// 1
        th(    "253B1");	// 1
        th(    "253NA");	// 1
        th(    "253P1");	// 1
        th(      "254");	// 1
        th(   "254NA7");	// 1
        th(    "256N1");	// 1
        th(    "256N3");	// 1
        th(   "256NA3");	// 1
        th(   "257NA6");	// 1
        th(      "258");	// 1
        th(    "258B1");	// 1
        th(   "258BA1");	// 1
        th(    "258N1");	// 1
        th(   "258NA7");	// 1
        th(     "25BA");	// 1
        th(     "25NA");	// 1
        th(     "25RA");	// 1
        th(    "260B7");	// 1
        th(    "260NA");	// 1
        th(   "260NA1");	// 1
        th(   "260NA4");	// 1
        th(     "261N");	// 1
        th(    "261N1");	// 1
        th(   "262BA1");	// 1
        th(   "262BA7");	// 1
        th(  "262NGA1");	// 1
        th(    "263N1");	// 1
        th(    "264B4");	// 1
        th(   "264BAR");	// 1
        th(    "264N4");	// 1
        th(    "265N4");	// 1
        th(    "269N2");	// 1
        th(     "26BA");	// 1
        th(    "26BA9");	// 1
        th(     "26LA");	// 1
        th(     "26NA");	// 1
        th(     "270A");	// 1
        th(      "271");	// 1
        th(    "271N5");	// 1
        th(   "271NA1");	// 1
        th(   "271NAS");	// 1
        th(   "272BA6");	// 1
        th(   "274NA4");	// 1
        th(   "277NA1");	// 1
        th(     "2788");	// 1
        th(      "279");	// 1
        th(   "279NA6");	// 1
        th(     "-27B");	// 1
        th(     "27B7");	// 1
        th(     "27BA");	// 1
        th(     "27N6");	// 1
        th(     "27NA");	// 1
        th(    "27NA1");	// 1
        th(    "27SHA");	// 1
        th(      "280");	// 1
        th(   "280BA4");	// 1
        th(      "281");	// 1
        th(      "282");	// 1
        th(   "282NA2");	// 1
        th(     "2834");	// 1
        th(   "283BA2");	// 1
        th(   "284BA2");	// 1
        th(   "285BA6");	// 1
        th(   "285NA2");	// 1
        th(     "2867");	// 1
        th(   "287NA4");	// 1
        th(   "288NA2");	// 1
        th(     "28BA");	// 1
        th(     "28N2");	// 1
        th(    "28NA1");	// 1
        th(     "28SA");	// 1
        th(     "2905");	// 1
        th(   "290BA7");	// 1
        th(      "291");	// 1
        th(    "293B2");	// 1
        th(   "293BA2");	// 1
        th(    "296B1");	// 1
        th(   "297BA4");	// 1
        th(      "298");	// 1
        th(    "298B4");	// 1
        th(     "29B1");	// 1
        th(     "29BA");	// 1
        th(   "29DBYA");	// 1
        th(     "29HA");	// 1
        th(     "29N6");	// 1
        th(     "29NA");	// 1
        th(    "29NA3");	// 1
        th(     "2BA4");	// 1
        th(     "2BA5");	// 1
        th(    "2CHOS");	// 1
        th(    "2DANG");	// 1
        th(   "2GNYIS");	// 1
        th(     "2KHA");	// 1
        th(     "2LAS");	// 1
        th(     "2NA2");	// 1
        th(   "2NY2ID");	// 1
        th(      "2PO");	// 1
        th(  "2PUshPE");	// 1
        th(    "2TZAM");	// 1
        th(      "301");	// 1
        th(     "301A");	// 1
        th(   "302PA2");	// 1
        th(    "30-38");	// 1
        th(    "303N3");	// 1
        th(    "303N5");	// 1
        th(      "304");	// 1
        th(   "304BA4");	// 1
        th(    "304N1");	// 1
        th(    "304N5");	// 1
        th(    "304N6");	// 1
        th(    "305B2");	// 1
        th(   "305NA7");	// 1
        th(   "307BA4");	// 1
        th(    "308N3");	// 1
        th(   "309NA3");	// 1
        th(     "30AA");	// 1
        th(     "30BA");	// 1
        th(    "30BA6");	// 1
        th(     "30N5");	// 1
        th(     "30NA");	// 1
        th(    "30NA3");	// 1
        th(    "310B3");	// 1
        th(     "3115");	// 1
        th(    "311B3");	// 1
        th(    "312B7");	// 1
        th(   "312PA7");	// 1
        th(   "313NA3");	// 1
        th(      "315");	// 1
        th(   "315BA3");	// 1
        th(   "318NA5");	// 1
        th(   "319NA3");	// 1
        th(     "31B1");	// 1
        th(     "31BA");	// 1
        th(    "31BA6");	// 1
        th(     "31KI");	// 1
        th(     "31NA");	// 1
        th(    "31NA5");	// 1
        th(   "320BA3");	// 1
        th(   "320NA5");	// 1
        th(  "3227BA1");	// 1
        th(    "323N5");	// 1
        th(      "324");	// 1
        th(     "324N");	// 1
        th(     "32BA");	// 1
        th(    "32BA4");	// 1
        th(    "32KHI");	// 1
        th(     "32NA");	// 1
        th(    "32NA2");	// 1
        th(      "336");	// 1
        th(      "337");	// 1
        th(   "338NA3");	// 1
        th(     "33GI");	// 1
        th(     "33N6");	// 1
        th(     "33NA");	// 1
        th(    "33NA3");	// 1
        th(    "349N2");	// 1
        th(    "349N4");	// 1
        th(   "349NA2");	// 1
        th(     "34BA");	// 1
        th(    "34BA3");	// 1
        th(     "34BU");	// 1
        th(     "34NA");	// 1
        th(    "34NA3");	// 1
        th(    "34NGI");	// 1
        th(     "35CI");	// 1
        th(     "35NA");	// 1
        th(    "35NA1");	// 1
        th(    "35NA2");	// 1
        th(      "360");	// 1
        th(   "364BA3");	// 1
        th(   "365BA3");	// 1
        th(   "366BA3");	// 1
        th(      "368");	// 1
        th(      "369");	// 1
        th(     "36B7");	// 1
        th(    "36BA7");	// 1
        th(    "36CHI");	// 1
        th(     "36NA");	// 1
        th(    "36NA4");	// 1
        th(     "36PA");	// 1
        th(      "370");	// 1
        th(   "377NA7");	// 1
        th(      "378");	// 1
        th(      "379");	// 1
        th(    "379N1");	// 1
        th(   "379NA1");	// 1
        th(     "37BA");	// 1
        th(     "37JI");	// 1
        th(    "37NA2");	// 1
        th(      "381");	// 1
        th(    "38-46");	// 1
        th(   "384NA2");	// 1
        th(    "385N4");	// 1
        th(    "385N6");	// 1
        th(   "385NA6");	// 1
        th(     "38B7");	// 1
        th(     "38N3");	// 1
        th(     "38NA");	// 1
        th(    "38NA1");	// 1
        th(    "38NYI");	// 1
        th(      "390");	// 1
        th(      "393");	// 1
        th(      "395");	// 1
        th(    "397N2");	// 1
        th(    "397N7");	// 1
        th(      "398");	// 1
        th(      "399");	// 1
        th(     "39B5");	// 1
        th(     "39BA");	// 1
        th(     "39NA");	// 1
        th(     "39TI");	// 1
        th(    "3'BA'");	// 1
        th(     "3BA3");	// 1
        th(   "3BZLOG");	// 1
        th(      "3GA");	// 1
        th(   "3GSHAM");	// 1
        th(   "3KHYAD");	// 1
        th(     "3LHA");	// 1
        th(     "3LTA");	// 1
        th(     "3MED");	// 1
        th(      "3NA");	// 1
        th(     "3NA1");	// 1
        th(  "3NA'ANG");	// 1
        th(    "3NYAN");	// 1
        th(      "3PA");	// 1
        th(   "3PHYIR");	// 1
        th(   "3RTOGS");	// 1
        th(    "3TSAD");	// 1
        th(      "400");	// 1
        th(      "401");	// 1
        th(    "401B3");	// 1
        th(    "403B3");	// 1
        th(      "404");	// 1
        th(   "404NA1");	// 1
        th(     "4055");	// 1
        th(      "406");	// 1
        th(      "407");	// 1
        th(      "40A");	// 1
        th(     "-40B");	// 1
        th(     "40BA");	// 1
        th(    "40BA2");	// 1
        th(    "40NAS");	// 1
        th(    "40THI");	// 1
        th(   "411BA4");	// 1
        th(      "414");	// 1
        th(   "414NA1");	// 1
        th(    "415N2");	// 1
        th(   "416NA7");	// 1
        th(     "41B7");	// 1
        th(    "41BA2");	// 1
        th(    "41BA7");	// 1
        th(     "41DI");	// 1
        th(    "41NA6");	// 1
        th(      "421");	// 1
        th(   "422BA6");	// 1
        th(   "424BA4");	// 1
        th(      "429");	// 1
        th(    "429N2");	// 1
        th(    "429NU");	// 1
        th(    "42BA2");	// 1
        th(     "42N6");	// 1
        th(    "42NA6");	// 1
        th(     "42NI");	// 1
        th(    "430B6");	// 1
        th(    "430B7");	// 1
        th(     "4332");	// 1
        th(      "434");	// 1
        th(   "434NA4");	// 1
        th(      "435");	// 1
        th(   "437BA1");	// 1
        th(   "437BA4");	// 1
        th(   "439BA7");	// 1
        th(     "43BA");	// 1
        th(    "43BA1");	// 1
        th(    "43BAR");	// 1
        th(     "43NA");	// 1
        th(    "43NA2");	// 1
        th(     "43PI");	// 1
        th(      "449");	// 1
        th(    "44BA4");	// 1
        th(    "44PHI");	// 1
        th(      "451");	// 1
        th(      "453");	// 1
        th(      "455");	// 1
        th(    "455N2");	// 1
        th(    "455N7");	// 1
        th(     "45B3");	// 1
        th(     "45BI");	// 1
        th(    "45NA2");	// 1
        th(      "460");	// 1
        th(      "461");	// 1
        th(      "464");	// 1
        th(   "464BA6");	// 1
        th(    "46-51");	// 1
        th(  "4668BA2");	// 1
        th(   "466NA7");	// 1
        th(      "46A");	// 1
        th(     "46BA");	// 1
        th(    "46BA1");	// 1
        th(    "46BA2");	// 1
        th(     "46MI");	// 1
        th(      "475");	// 1
        th(    "476N7");	// 1
        th(    "476P3");	// 1
        th(     "47N9");	// 1
        th(    "47TZI");	// 1
        th(      "480");	// 1
        th(      "485");	// 1
        th(    "487P2");	// 1
        th(      "488");	// 1
        th(    "489N2");	// 1
        th(   "489NA3");	// 1
        th(     "48BA");	// 1
        th(    "48TSI");	// 1
        th(   "490495");	// 1
        th(      "496");	// 1
        th(    "497N2");	// 1
        th(    "497N7");	// 1
        th(      "499");	// 1
        th(     "49BA");	// 1
        th(    "49DZI");	// 1
        th(    "49NA5");	// 1
        th(      "4B4");	// 1
        th(      "4BA");	// 1
        th(    "4CHA4");	// 1
        th(    "4DANG");	// 1
        th(      "4NA");	// 1
        th(     "4NA2");	// 1
        th(     "4NA3");	// 1
        th(     "4NGA");	// 1
        th(    "4RTAG");	// 1
        th(     "4STE");	// 1
        th(      "501");	// 1
        th(    "503B1");	// 1
        th(      "506");	// 1
        th(    "50BA7");	// 1
        th(    "50NA2");	// 1
        th(     "50WI");	// 1
        th(      "510");	// 1
        th(      "511");	// 1
        th(      "513");	// 1
        th(      "514");	// 1
        th(      "515");	// 1
        th(    "51-55");	// 1
        th(      "516");	// 1
        th(    "51ZHI");	// 1
        th(      "522");	// 1
        th(      "527");	// 1
        th(    "52NA4");	// 1
        th(     "52ZI");	// 1
        th(      "535");	// 1
        th(      "536");	// 1
        th(      "537");	// 1
        th(     "5383");	// 1
        th(      "539");	// 1
        th(     "53'I");	// 1
        th(     "53N4");	// 1
        th(    "53NA5");	// 1
        th(      "541");	// 1
        th(      "545");	// 1
        th(      "546");	// 1
        th(      "549");	// 1
        th(    "54NA2");	// 1
        th(     "54YI");	// 1
        th(      "550");	// 1
        th(      "551");	// 1
        th(      "552");	// 1
        th(    "55-64");	// 1
        th(      "559");	// 1
        th(    "55NA2");	// 1
        th(     "55RI");	// 1
        th(      "562");	// 1
        th(      "563");	// 1
        th(      "564");	// 1
        th(      "565");	// 1
        th(     "56LI");	// 1
        th(      "573");	// 1
        th(     "57BA");	// 1
        th(     "57NA");	// 1
        th(    "57NA3");	// 1
        th(    "57SHI");	// 1
        th(     "58BA");	// 1
        th(     "58NA");	// 1
        th(     "58SI");	// 1
        th(      "597");	// 1
        th(     "59BA");	// 1
        th(     "59HI");	// 1
        th(     "59NA");	// 1
        th(    "59NA1");	// 1
        th(    "59NA6");	// 1
        th(      "5B4");	// 1
        th(      "5BA");	// 1
        th(     "5BA1");	// 1
        th(     "5BA2");	// 1
        th(     "5BA6");	// 1
        th(      "5CA");	// 1
        th(      "5N3");	// 1
        th(     "5NA4");	// 1
        th(     "60AI");	// 1
        th(     "60BA");	// 1
        th(    "60NA6");	// 1
        th(      "613");	// 1
        th(     "61BA");	// 1
        th(    "61NA6");	// 1
        th(      "622");	// 1
        th(     "62B1");	// 1
        th(    "62BA2");	// 1
        th(    "62KHE");	// 1
        th(    "62NA1");	// 1
        th(    "62NA2");	// 1
        th(     "63BA");	// 1
        th(     "63GE");	// 1
        th(     "63NA");	// 1
        th(    "64-68");	// 1
        th(     "64BA");	// 1
        th(    "64NGE");	// 1
        th(     "65BA");	// 1
        th(     "65CI");	// 1
        th(    "65NA5");	// 1
        th(    "66BA2");	// 1
        th(    "66CHE");	// 1
        th(     "66NA");	// 1
        th(   "671405");	// 1
        th(      "675");	// 1
        th(     "67JE");	// 1
        th(    "67NA5");	// 1
        th(    "68-69");	// 1
        th(    "68-72");	// 1
        th(      "689");	// 1
        th(     "68NA");	// 1
        th(    "68NYE");	// 1
        th(     "69BA");	// 1
        th(    "69BA3");	// 1
        th(    "69BA4");	// 1
        th(     "69N3");	// 1
        th(     "69NA");	// 1
        th(     "69TE");	// 1
        th(     "6BA1");	// 1
        th(     "6BA5");	// 1
        th(     "6CHA");	// 1
        th(     "6LHA");	// 1
        th(     "6NGA");	// 1
        th(       "'7");	// 1
        th(      "708");	// 1
        th(     "70BA");	// 1
        th(    "70NA2");	// 1
        th(    "70NA3");	// 1
        th(    "70THE");	// 1
        th(      "711");	// 1
        th(     "7-11");	// 1
        th(      "712");	// 1
        th(     "71BA");	// 1
        th(    "71BA3");	// 1
        th(     "71DE");	// 1
        th(     "71NA");	// 1
        th(    "71RA3");	// 1
        th(     "72NA");	// 1
        th(     "72NE");	// 1
        th(   "72-RAB");	// 1
        th(      "735");	// 1
        th(    "73BA4");	// 1
        th(     "73NA");	// 1
        th(     "73PE");	// 1
        th(      "747");	// 1
        th(     "74BA");	// 1
        th(     "74N5");	// 1
        th(     "74NA");	// 1
        th(    "74PHE");	// 1
        th(      "755");	// 1
        th(  "7568723");	// 1
        th(     "75BE");	// 1
        th(     "75NA");	// 1
        th(     "76B4");	// 1
        th(     "76BA");	// 1
        th(     "76ME");	// 1
        th(     "76N4");	// 1
        th(    "76NA4");	// 1
        th(     "77JA");	// 1
        th(     "77NA");	// 1
        th(    "77TZE");	// 1
        th(      "781");	// 1
        th(      "784");	// 1
        th(     "78BA");	// 1
        th(     "78NA");	// 1
        th(    "78TSE");	// 1
        th(   "796NA6");	// 1
        th(    "79DZE");	// 1
        th(      "7AO");	// 1
        th(       "7B");	// 1
        th(     "7BA7");	// 1
        th(  "7BRGYAD");	// 1
        th(  "7BRGYUD");	// 1
        th(   "7BSTAN");	// 1
        th(   "7BYANG");	// 1
        th(   "7BZHAD");	// 1
        th(     "7CES");	// 1
        th(     "7DAD");	// 1
        th(    "7DANG");	// 1
        th(     "7DER");	// 1
        th(     "7DGU");	// 1
        th(   "7DNGOS");	// 1
        th(    "7'DOD");	// 1
        th(     "7DON");	// 1
        th(    "7DPAL");	// 1
        th(    "7DRAG");	// 1
        th(   "7'DRES");	// 1
        th(     "7DRI");	// 1
        th(    "7DRIL");	// 1
        th(    "7DRUG");	// 1
        th(    "7DZAM");	// 1
        th(    "7'GAL");	// 1
        th(     "7GAR");	// 1
        th(    "7GNAS");	// 1
        th(   "7GNYID");	// 1
        th(    "7GROL");	// 1
        th(    "7GSAN");	// 1
        th(   "7GSANG");	// 1
        th(   "7GTZUG");	// 1
        th(    "7GUNG");	// 1
        th(   "7GZHAN");	// 1
        th(   "7GZUGS");	// 1
        th(  "7GZUNGS");	// 1
        th(      "7'I");	// 1
        th(      "7JA");	// 1
        th(     "7JAM");	// 1
        th(   "7'JIGS");	// 1
        th(   "7KHYAB");	// 1
        th(   "7KHYAD");	// 1
        th(   "7KLONG");	// 1
        th(     "7LAG");	// 1
        th(     "7LAM");	// 1
        th(     "7LAS");	// 1
        th(   "7LCANG");	// 1
        th(    "7LDAN");	// 1
        th(      "7LO");	// 1
        th(    "7LUGS");	// 1
        th(    "7LUNG");	// 1
        th(   "7MCHOD");	// 1
        th(     "7MDO");	// 1
        th(    "7MDUN");	// 1
        th(    "7MGON");	// 1
        th(   "7MKHAS");	// 1
        th(   "7MNYAM");	// 1
        th(   "7MTHA'");	// 1
        th(     "7MYA");	// 1
        th(      "7NA");	// 1
        th(    "7NANG");	// 1
        th(    "7NGAG");	// 1
        th(    "7NGED");	// 1
        th(     "7NOR");	// 1
        th(   "7NYANG");	// 1
        th(      "7PA");	// 1
        th(   "7PAndI");	// 1
        th(    "7PHYI");	// 1
        th(      "7PO");	// 1
        th(     "7REG");	// 1
        th(  "7RGYAGS");	// 1
        th(    "7RGYU");	// 1
        th(   "7RGYUN");	// 1
        th(     "7RIG");	// 1
        th(      "7RO");	// 1
        th(   "7RTAGS");	// 1
        th(    "7RTZA");	// 1
        th(    "7RUNG");	// 1
        th(     "7RVA");	// 1
        th(   "7SBYAR");	// 1
        th(    "7SDOM");	// 1
        th(     "7SGO");	// 1
        th(    "7SGRA");	// 1
        th(    "7SGYU");	// 1
        th(    "7SHER");	// 1
        th(     "7SHU");	// 1
        th(    "7SMRA");	// 1
        th(   "7SNANG");	// 1
        th(  "7SNGAGS");	// 1
        th(   "7SNGAR");	// 1
        th(   "7SNYAN");	// 1
        th(  "7SNYING");	// 1
        th(     "7SOG");	// 1
        th(   "7SPYIN");	// 1
        th(    "7SRAS");	// 1
        th(    "7SROG");	// 1
        th(    "7STES");	// 1
        th(   "7THAMS");	// 1
        th(    "7THUB");	// 1
        th(    "7TING");	// 1
        th(     "7TSE");	// 1
        th(     "7YAN");	// 1
        th(     "7YUL");	// 1
        th(     "7YUM");	// 1
        th(   "7ZANGS");	// 1
        th(    "7ZHAL");	// 1
        th(     "7ZHI");	// 1
        th(    "7ZHVA");	// 1
        th(    "7ZLOG");	// 1
        th(    "80-94");	// 1
        th(     "80NA");	// 1
        th(     "80WE");	// 1
        th(     "81NA");	// 1
        th(    "81NA5");	// 1
        th(    "81ZHE");	// 1
        th(     "82BA");	// 1
        th(     "82NA");	// 1
        th(    "82NA2");	// 1
        th(    "82NA5");	// 1
        th(     "82ZE");	// 1
        th(     "83BA");	// 1
        th(     "83'E");	// 1
        th(     "83NA");	// 1
        th(     "84BA");	// 1
        th(    "84BA4");	// 1
        th(     "84NA");	// 1
        th(     "84YE");	// 1
        th(     "85BA");	// 1
        th(     "85NA");	// 1
        th(    "85NA1");	// 1
        th(     "85RE");	// 1
        th(     "86LE");	// 1
        th(     "86NA");	// 1
        th(    "86NA4");	// 1
        th(    "87BA2");	// 1
        th(     "87NA");	// 1
        th(    "87SHE");	// 1
        th(      "881");	// 1
        th(     "88N1");	// 1
        th(     "88NA");	// 1
        th(    "88NA5");	// 1
        th(    "88PA1");	// 1
        th(     "88SE");	// 1
        th(     "89B1");	// 1
        th(    "89BA1");	// 1
        th(    "89BA3");	// 1
        th(    "89BA4");	// 1
        th(     "89HE");	// 1
        th(     "89NA");	// 1
        th(     "8BA6");	// 1
        th(     "8BAR");	// 1
        th(     "8NA4");	// 1
        th(     "8NA5");	// 1
        th(     "8NA6");	// 1
        th(     "8NYA");	// 1
        th(     "9087");	// 1
        th(     "90AE");	// 1
        th(    "90BA4");	// 1
        th(     "90NA");	// 1
        th(     "91BA");	// 1
        th(    "91BA3");	// 1
        th(     "91KE");	// 1
        th(     "91KO");	// 1
        th(    "92KHO");	// 1
        th(     "92NA");	// 1
        th(     "93B1");	// 1
        th(     "93BA");	// 1
        th(     "93GO");	// 1
        th(    "93NA3");	// 1
        th(   "94-112");	// 1
        th(     "94B2");	// 1
        th(     "94NA");	// 1
        th(    "94NA7");	// 1
        th(    "94NGO");	// 1
        th(     "95BA");	// 1
        th(     "95CO");	// 1
        th(    "96BA2");	// 1
        th(    "96CHO");	// 1
        th(     "96NA");	// 1
        th(    "96NA5");	// 1
        th(    "97BA4");	// 1
        th(     "97JO");	// 1
        th(     "97NA");	// 1
        th(     "98NA");	// 1
        th(    "98NA7");	// 1
        th(    "98NYO");	// 1
        th(      "997");	// 1
        th(    "99NA7");	// 1
        th(     "99TO");	// 1
        th(     "9BA7");	// 1
        th(    "9KYIS");	// 1
        th(      "9NA");	// 1
        th(     "9NAS");	// 1
        th(    "9RGYU");	// 1
        th(     "9SA1");	// 1
        th(      "9TA");	// 1
        th(     "9YIN");	// 1
        th(      "A':");	// 1
        th(      "A''");	// 1
        th(      "'A2");	// 1
        th(     "A''A");	// 1
        th(     "A'A'");	// 1
        th(      "AA2");	// 1
        th(      "AA4");	// 1
        th(   "A'A:AA");	// 1
        th(   "A'AAMA");	// 1
        th(     "AAAN");	// 1
        th(   "AA'ANG");	// 1
        th(    "AA'AS");	// 1
        th(   "A'A:BA");	// 1
        th("AABADANAMAnIKAdTHAKAN'AMABUtA");	// 1
        th("A'A:BADZRADHRi");	// 1
        th(  "AABAM'A");	// 1
        th(   "AABA'O");	// 1
        th(    "AABHA");	// 1
        th(    "AABHE");	// 1
        th(    "AABHI");	// 1
        th( "A'A:BItA");	// 1
        th(    "AABRA");	// 1
        th(    "AABYA");	// 1
        th(    "AACVA");	// 1
        th(  "AADBHI:");	// 1
        th(    "AADGI");	// 1
        th(     "AADI");	// 1
        th(   "A'AD'I");	// 1
        th(    "AADKU");	// 1
        th(    "AADNA");	// 1
        th(   "AADYEE");	// 1
        th(    "AADZA");	// 1
        th(   "AADZHA");	// 1
        th("A'ADZNY'A:");	// 1
        th(      "AAE");	// 1
        th(     "AAGA");	// 1
        th(   "AAGAKA");	// 1
        th(   "AAGN'I");	// 1
        th(   "AAGNI:");	// 1
        th( "AAGNIRGA");	// 1
        th(   "A'A:GU");	// 1
        th(     "AAHA");	// 1
        th(    "A'AHA");	// 1
        th( "A'A:HR'I");	// 1
        th("A'A:HRI:PRA");	// 1
        th(     "A'AI");	// 1
        th(  "AA'IKsh");	// 1
        th(  "AAINDRA");	// 1
        th(    "AAING");	// 1
        th(   "AA'IRA");	// 1
        th( "AAIT'ANG");	// 1
        th(     "AAKA");	// 1
        th(    "A'AKA");	// 1
        th(   "AAKA'I");	// 1
        th(  "A'A:KAR");	// 1
        th("AAKHANG+GA");	// 1
        th( "A'A:KHRE");	// 1
        th(    "AAK'O");	// 1
        th( "AA-Ksh'A");	// 1
        th( "AAKSHA'I");	// 1
        th(   "AAKshI");	// 1
        th(  "AA-KshI");	// 1
        th(  "AAKsh'I");	// 1
        th(  "AAKshnO");	// 1
        th( "AAKshOOm");	// 1
        th(  "AAKshRA");	// 1
        th(   "A'A:KU");	// 1
        th( "AAKY'A'I");	// 1
        th(   "A'A:LA");	// 1
        th(   "A'ALPA");	// 1
        th(  "AALP'A:");	// 1
        th(    "AALRA");	// 1
        th(     "AA'M");	// 1
        th("A'A:MAH'A");	// 1
        th(  "A'AM'AI");	// 1
        th(  "A'AMA'O");	// 1
        th(   "AAMB'A");	// 1
        th(   "AAMBAN");	// 1
        th(   "AAMBHA");	// 1
        th(    "AAM'O");	// 1
        th(    "AAMPA");	// 1
        th(  "AAmRGHA");	// 1
        th(   "A'A:MU");	// 1
        th(  "A'A:M'U");	// 1
        th(    "AAMYE");	// 1
        th( "A'ANANDA");	// 1
        th(  "AAn'ANG");	// 1
        th( "A'ANANSA");	// 1
        th("A'ANANTA'O");	// 1
        th("A'ANANTAS");	// 1
        th(   "AANA'O");	// 1
        th(    "AANBI");	// 1
        th(     "AAnd");	// 1
        th(    "AANdA");	// 1
        th(   "AANDRA");	// 1
        th( "AA-N+DRI");	// 1
        th("AANG-GA'I");	// 1
        th(   "AANGGI");	// 1
        th( "AANG+G'I");	// 1
        th( "AANG+GIR");	// 1
        th(  "AANGKIR");	// 1
        th(   "AANHAm");	// 1
        th(    "AANHI");	// 1
        th(     "AANt");	// 1
        th(   "A'ANTA");	// 1
        th(   "AA+NTA");	// 1
        th(  "AANT'AM");	// 1
        th(   "AANTAN");	// 1
        th(   "AANTAR");	// 1
        th(  "AANTARA");	// 1
        th(    "AANTI");	// 1
        th(  "AANTI'I");	// 1
        th(  "AANTI'O");	// 1
        th(   "AANTRA");	// 1
        th( "AANTSTHA");	// 1
        th(   "AAN+YA");	// 1
        th(  "AANYDZA");	// 1
        th(  "AANYTZI");	// 1
        th("A'A:PADMA");	// 1
        th("A'A:PIGHI");	// 1
        th(   "AAPSAR");	// 1
        th(    "AAPTI");	// 1
        th(     "AARA");	// 1
        th(   "AARA'O");	// 1
        th(  "AARBANA");	// 1
        th("AARBANTI'O");	// 1
        th(   "AARDHA");	// 1
        th(    "AARDI");	// 1
        th(   "AARDYA");	// 1
        th("AARDZRDZA");	// 1
        th(     "AARG");	// 1
        th(  "AARGE'I");	// 1
        th(   "AARGHA");	// 1
        th( "AARGHA'I");	// 1
        th(   "A'ARLA");	// 1
        th(    "AARMI");	// 1
        th(   "A'AR'O");	// 1
        th(    "AAR'O");	// 1
        th(    "AARTA");	// 1
        th(  "AAR-THA");	// 1
        th(  "AARTHA:");	// 1
        th(   "AARTHI");	// 1
        th("A'ARTZARYA");	// 1
        th( "A'A-R+YA");	// 1
        th(   "AA'RYA");	// 1
        th(   "AASBHU");	// 1
        th(   "AASHAM");	// 1
        th("A'A:SHANGKHA");	// 1
        th("A'A:SHLEshMA");	// 1
        th(  "AAshn'I");	// 1
        th(   "AASHNI");	// 1
        th(   "AASHNO");	// 1
        th(   "AAshtA");	// 1
        th(   "AAshtI");	// 1
        th("AASHVADTHA'I");	// 1
        th(  "AA-S+MA");	// 1
        th( "AASMADRA");	// 1
        th("AASMADYUT");	// 1
        th( "A'A:SOGS");	// 1
        th(   "AASTHI");	// 1
        th(  "AASTI'I");	// 1
        th("AA:SV'AH'A");	// 1
        th(    "AASYA");	// 1
        th(     "A'At");	// 1
        th(   "AAtA'I");	// 1
        th("A'A:TAKshA");	// 1
        th(  "AAT'ANG");	// 1
        th(  "A'ATA'O");	// 1
        th(   "AATHiI");	// 1
        th(  "AATIREE");	// 1
        th("AATMETZTSA");	// 1
        th(   "A'AT'O");	// 1
        th(    "AAttA");	// 1
        th(   "AA-ttA");	// 1
        th(    "AAttE");	// 1
        th(  "A'ATTHA");	// 1
        th("A'ATZARYA");	// 1
        th("AATZARYA'I");	// 1
        th( "A'ATZRYA");	// 1
        th(  "AATZTSA");	// 1
        th( "A'ATZTSA");	// 1
        th(  "AATZTSE");	// 1
        th("AATZTSINTE");	// 1
        th(      "AAU");	// 1
        th(    "AAU'I");	// 1
        th(   "AA'URA");	// 1
        th(     "AAUS");	// 1
        th( "AAUT'ANG");	// 1
        th(   "A'A:WA");	// 1
        th(     "AAYA");	// 1
        th(    "AA+YA");	// 1
        th(      "'AB");	// 1
        th(     "ABAR");	// 1
        th(    "ABHRA");	// 1
        th(    "ABHYA");	// 1
        th(    "ABM'A");	// 1
        th(    "ABOVE");	// 1
        th(      "ABR");	// 1
        th(   "ABRJOD");	// 1
        th(      "ABS");	// 1
        th(   "ABYA'I");	// 1
        th(   "ABYA'O");	// 1
        th(    "ABYED");	// 1
        th(   "ABZHAG");	// 1
        th(   "ABZHIN");	// 1
        th(     "ACAN");	// 1
        th(    "ACHEN");	// 1
        th(       "Ad");	// 1
        th(       "AD");	// 1
        th(    "ADANG");	// 1
        th(  "ADA+Y'A");	// 1
        th(    "AD'DU");	// 1
        th(      "ADE");	// 1
        th(    "ADKON");	// 1
        th(     "ADKU");	// 1
        th(  "ADYOMRA");	// 1
        th(    "ADYOO");	// 1
        th(  "ADZDZHA");	// 1
        th(      "^AE");	// 1
        th(      "AE2");	// 1
        th(      "AE4");	// 1
        th(  "AEENTRA");	// 1
        th(  "AEENTRI");	// 1
        th(     "AEER");	// 1
        th(     "AEES");	// 1
        th(   "AEES'O");	// 1
        th(    "AEHYA");	// 1
        th(    "AELVA");	// 1
        th( "AEN+YA'I");	// 1
        th( "AERKHE'I");	// 1
        th(  "AERnd'I");	// 1
        th(    "AERRA");	// 1
        th(    "AEshU");	// 1
        th(    "AGANG");	// 1
        th(      "AGG");	// 1
        th(     "AGNA");	// 1
        th(    "AGNAS");	// 1
        th(    "AGN'I");	// 1
        th(    "AGNIR");	// 1
        th(   "AGNYIS");	// 1
        th(    "AGRUB");	// 1
        th(  "AGSUNGS");	// 1
        th(   "AGZUGS");	// 1
        th(     "A'I'");	// 1
        th(      "AI4");	// 1
        th(  "A'I'A'O");	// 1
        th( "AI'I'ANG");	// 1
        th(   "A'IK'I");	// 1
        th(   "AIKshA");	// 1
        th(  "AIKsh'A");	// 1
        th(    "AIN'A");	// 1
        th(   "AInA'O");	// 1
        th(    "AINdA");	// 1
        th(    "AINDA");	// 1
        th(   "AINDHI");	// 1
        th(   "AInDRA");	// 1
        th(  "AIN+DRA");	// 1
        th("AINDRAN'I");	// 1
        th(   "AINDRE");	// 1
        th( "AI-N+DRI");	// 1
        th(   "AINDU:");	// 1
        th(     "AInE");	// 1
        th(  "AIn'I'O");	// 1
        th(   "AINK'A");	// 1
        th(    "AIN'O");	// 1
        th(   "AInTRi");	// 1
        th(      "AIP");	// 1
        th(   "AIRA'O");	// 1
        th( "A'IRTSAT");	// 1
        th(    "AIshA");	// 1
        th(   "A'IshI");	// 1
        th(    "AIshU");	// 1
        th(    "AIS'O");	// 1
        th(     "A'It");	// 1
        th( "A'IT'ANG");	// 1
        th("AiTHA'A'I");	// 1
        th(  "AiTHA'I");	// 1
        th( "AITH'I'I");	// 1
        th(  "AITHm'I");	// 1
        th(    "AIt'I");	// 1
        th(    "AITNU");	// 1
        th(   "A'It'O");	// 1
        th(    "AIt'O");	// 1
        th(    "AITSA");	// 1
        th(  "AITTHAm");	// 1
        th(   "AI-ttI");	// 1
        th(   "A'ITYA");	// 1
        th(    "AITZA");	// 1
        th("AITZATS'A'O");	// 1
        th("AITZTSU'O");	// 1
        th(      "AJI");	// 1
        th(    "A'JUG");	// 1
        th(   "AKHYAD");	// 1
        th(     "AKKA");	// 1
        th(     "AKRA");	// 1
        th(    "AKshA");	// 1
        th(     "AKUN");	// 1
        th(     "AKYA");	// 1
        th(   "AKYANG");	// 1
        th(     "ALAS");	// 1
        th(     "ALPA");	// 1
        th(     "ALPE");	// 1
        th(   "A'MA'O");	// 1
        th(   "AMBA'I");	// 1
        th(      "AMI");	// 1
        th(     "AM'I");	// 1
        th(   "AMKHA'");	// 1
        th(   "AMNYAM");	// 1
        th(     "'AMS");	// 1
        th(   "AMTHUN");	// 1
        th(    "AMUDI");	// 1
        th(       "An");	// 1
        th(      "A'N");	// 1
        th(     "ANAN");	// 1
        th(    "ANATA");	// 1
        th(      "AND");	// 1
        th(    "ANDHA");	// 1
        th(     "A'NG");	// 1
        th(    "ANGIG");	// 1
        th(    "ANGOR");	// 1
        th(    "ANHAm");	// 1
        th(      "ANI");	// 1
        th(   "ANIDRA");	// 1
        th(    "ANITA");	// 1
        th(     "ANNG");	// 1
        th(     "An'O");	// 1
        th(     "ANSA");	// 1
        th(     "ANTE");	// 1
        th(   "ANTI'I");	// 1
        th( "ANTSATHA");	// 1
        th(  "ANTSTHA");	// 1
        th(   "ANYDZA");	// 1
        th(   "ANYDZI");	// 1
        th(    "ANYID");	// 1
        th( "ANYTZA'I");	// 1
        th(      "'AO");	// 1
        th(      "AO3");	// 1
        th("AOmH'UmTR'AmHR'ImA'AmSOGS");	// 1
        th(     "AOmN");	// 1
        th(    "'AONG");	// 1
        th(  "AOO'ANG");	// 1
        th(    "AOOdI");	// 1
        th( "AOOdYA'N");	// 1
        th(  "AOORYAN");	// 1
        th(     "AOOT");	// 1
        th(    "AOOTA");	// 1
        th(   "AOOTTA");	// 1
        th(    "AORTU");	// 1
        th(  "AOshthA");	// 1
        th(  "AOshthO");	// 1
        th(    "AOT'O");	// 1
        th(     "AO'U");	// 1
        th("APAD-MO'I");	// 1
        th(    "APA'O");	// 1
        th(     "APAS");	// 1
        th(   "APHYIR");	// 1
        th(     "AP'I");	// 1
        th(      "APO");	// 1
        th(      "APS");	// 1
        th(     "A'RA");	// 1
        th(     "ARBA");	// 1
        th(    "ARDZA");	// 1
        th(    "ARHAm");	// 1
        th(      "ARI");	// 1
        th(     "ARJE");	// 1
        th(     "ARMA");	// 1
        th(  "ARTHA'I");	// 1
        th(   "ARTI'I");	// 1
        th(     "ARYA");	// 1
        th(   "A'R+YA");	// 1
        th(      "A'S");	// 1
        th(   "ASBYIN");	// 1
        th(   "ASBYOR");	// 1
        th(     "ASDE");	// 1
        th(    "ASDUD");	// 1
        th(    "ASHES");	// 1
        th(    "ASMAD");	// 1
        th(    "ASNGA");	// 1
        th( "ASTI'I'O");	// 1
        th(      "ASU");	// 1
        th(     "ASUM");	// 1
        th(    "AtA'O");	// 1
        th(      "ATH");	// 1
        th("ATHiTHI'I");	// 1
        th(    "ATHOB");	// 1
        th(   "A'THOB");	// 1
        th(     "At'O");	// 1
        th(    "A'T'O");	// 1
        th(     "ATZA");	// 1
        th(    "ATZAM");	// 1
        th("A'TZAR-YA");	// 1
        th("ATZARYA'I");	// 1
        th(      "AUA");	// 1
        th(      "AUB");	// 1
        th(     "AUBA");	// 1
        th(   "AUBDZA");	// 1
        th(    "AUBRA");	// 1
        th(   "AUDBHA");	// 1
        th(    "AUDDE");	// 1
        th(  "AUDDHYA");	// 1
        th(     "AUDE");	// 1
        th(    "AUDHA");	// 1
        th(  "AUD-PAL");	// 1
        th(  "AUDUM'A");	// 1
        th(  "AUDZDZA");	// 1
        th(    "AUGER");	// 1
        th(    "AUHAm");	// 1
        th(     "AUHU");	// 1
        th(    "AUHYA");	// 1
        th(    "AU''I");	// 1
        th(      "AUK");	// 1
        th(   "AU-KKE");	// 1
        th(   "AULL'A");	// 1
        th(      "AUm");	// 1
        th(     "AUnA");	// 1
        th(    "AUndA");	// 1
        th( "AUND'I'I");	// 1
        th(    "A'UNG");	// 1
        th( "AUNG+GER");	// 1
        th(  "AUNGGHA");	// 1
        th( "A'UNnA'O");	// 1
        th(      "AUP");	// 1
        th(    "AUPTA");	// 1
        th(   "AUPTAm");	// 1
        th(  "A'URDHA");	// 1
        th(  "AURDZ'A");	// 1
        th(   "AURGEL");	// 1
        th(   "A'URnU");	// 1
        th(   "AUshMA");	// 1
        th(  "AUSHMAD");	// 1
        th( "AUshnA'I");	// 1
        th( "AUshnIKA");	// 1
        th(    "AUSSI");	// 1
        th(    "AUT'A");	// 1
        th(   "AUTA'O");	// 1
        th( "AUTAPALA");	// 1
        th(   "AUTBAL");	// 1
        th(    "AUTHA");	// 1
        th("AUTIshm'A");	// 1
        th(   "AUT-KA");	// 1
        th(   "AU-TKU");	// 1
        th(   "A'Ut'O");	// 1
        th(    "AUT'O");	// 1
        th(     "AUTP");	// 1
        th(    "AUtPA");	// 1
        th(   "AUtPAL");	// 1
        th(  "AU-TPAL");	// 1
        th("A'UTPALA'I");	// 1
        th( "AUTPAL'I");	// 1
        th( "AUTPLA'I");	// 1
        th(  "AUTPL'I");	// 1
        th("AUTS'AHA:");	// 1
        th(   "A'UTTA");	// 1
        th(   "AUTTAM");	// 1
        th( "AUTZTSEE");	// 1
        th(  "AUTZTSI");	// 1
        th(  "AUTZTSU");	// 1
        th(    "A'U'U");	// 1
        th(      "AYA");	// 1
        th(    "AYANG");	// 1
        th(     "AYAS");	// 1
        th(     "AZAG");	// 1
        th(      "B0O");	// 1
        th(       "B1");	// 1
        th(   "B2YANG");	// 1
        th(   "B2ZHIN");	// 1
        th(       "B5");	// 1
        th(      "B6Y");	// 1
        th(       "B7");	// 1
        th(    "BA'2I");	// 1
        th(      "BA9");	// 1
        th(    "BA'AI");	// 1
        th(    "BA'AN");	// 1
        th(  "'BA'ANG");	// 1
        th(   "B'ABTI");	// 1
        th(     "^BAD");	// 1
        th(   "BADANG");	// 1
        th(  "BA:DANG");	// 1
        th(   "BADATA");	// 1
        th(   "BADDHA");	// 1
        th(    "BADHA");	// 1
        th(    "BADHI");	// 1
        th(   "BADH'U");	// 1
        th("B'ADH'U:LTA");	// 1
        th(  "BADHVEE");	// 1
        th(    "BAd'I");	// 1
        th(    "B'ADI");	// 1
        th(    "BA'DI");	// 1
        th(   "BA'DIR");	// 1
        th(   "BA'DIS");	// 1
        th(    "BADMA");	// 1
        th(   "BADREN");	// 1
        th(   "BADshA");	// 1
        th("BAd'UEER+YA");	// 1
        th(   "BADY'I");	// 1
        th(    "BADZE");	// 1
        th("BADZRAMU:ME");	// 1
        th("BADZRASATVA");	// 1
        th(   "BADZVA");	// 1
        th(  "BADZVOD");	// 1
        th(      "BAE");	// 1
        th(    "BAHES");	// 1
        th(  "BAHVANG");	// 1
        th(    "^BA'I");	// 1
        th("BA'IGZHUNG");	// 1
        th(   "BA'I'I");	// 1
        th( "BA'ILUGS");	// 1
        th(   "BA'IME");	// 1
        th(    "BA'IR");	// 1
        th(    "BA'IS");	// 1
        th(      "BAK");	// 1
        th(   "BAKCHA");	// 1
        th(   "B'AKKO");	// 1
        th(   "BAKshA");	// 1
        th( "B'AKSH'A");	// 1
        th(   "BAKshE");	// 1
        th(   "BAKshI");	// 1
        th(    "BAKTI");	// 1
        th( "B'AKTS'U");	// 1
        th(  "BAKVANG");	// 1
        th(     "BALA");	// 1
        th(  "B'ALK'A");	// 1
        th("BALK'A'R'A:");	// 1
        th(    "BALLI");	// 1
        th(   "BALNGS");	// 1
        th(     "BA'm");	// 1
        th(     "BAMA");	// 1
        th(     "BAME");	// 1
        th(    "BAM'O");	// 1
        th(    "BAMRA");	// 1
        th(     "BAMS");	// 1
        th(    "B'ANA");	// 1
        th(    "BANDA");	// 1
        th(    "BAndE");	// 1
        th(    "BAnDE");	// 1
        th(    "BANdE");	// 1
        th(   "BAN+DE");	// 1
        th(   "BANDER");	// 1
        th( "BANDHA'I");	// 1
        th("BANDH'ARTISHARTI");	// 1
        th(  "BAN-DHE");	// 1
        th(   "BANDHI");	// 1
        th(  "BANDHUS");	// 1
        th(   "BANDRA");	// 1
        th(   "BANDRE");	// 1
        th(   "BAN-DU");	// 1
        th(  "BANG+KA");	// 1
        th(  "BANSA'I");	// 1
        th(    "BANSI");	// 1
        th(   "BANSIL");	// 1
        th(  "BANSU'O");	// 1
        th(    "BANTE");	// 1
        th("B'ANYTZTS'U");	// 1
        th(     "BAOM");	// 1
        th(     "BAOS");	// 1
        th(    "BAPTI");	// 1
        th(     "^BAR");	// 1
        th(  "BAR'A'I");	// 1
        th(     "BARD");	// 1
        th(    "BARDA");	// 1
        th(   "BARDHA");	// 1
        th(    "BARDU");	// 1
        th(   "BARDZA");	// 1
        th(  "BARDZUS");	// 1
        th(    "BAR'I");	// 1
        th(   "BARKAm");	// 1
        th(   "BARKAM");	// 1
        th( "BARK'AR:");	// 1
        th(     "BARL");	// 1
        th(    "'BARm");	// 1
        th(  "BARnA'I");	// 1
        th(  "B'ARndA");	// 1
        th(  "BARnE'I");	// 1
        th(   "BARnnA");	// 1
        th(    "BAR'O");	// 1
        th(   "BAR'OS");	// 1
        th(    "'BARS");	// 1
        th(   "BARshA");	// 1
        th(  "BARsh'A");	// 1
        th(   "BARSHA");	// 1
        th(    "BARTA");	// 1
        th(   "BART'A");	// 1
        th(   "B'ARTI");	// 1
        th(    "BA:RU");	// 1
        th(    "BAR'U");	// 1
        th(  "B'AshPA");	// 1
        th(   "BAshtI");	// 1
        th(    "BAS'I");	// 1
        th(     "BASR");	// 1
        th(  "BASTRAM");	// 1
        th("BASV'AH'A");	// 1
        th(     "BATA");	// 1
        th(   "BATA'O");	// 1
        th("BATA+SA'I");	// 1
        th(   "BATBHU");	// 1
        th(  "BATRA'O");	// 1
        th(   "BAT+SA");	// 1
        th(   "B'ATSI");	// 1
        th(    "BAttE");	// 1
        th(   "BATZAN");	// 1
        th(  "BATZTSA");	// 1
        th(  "BATZTSI");	// 1
        th(   "BATZYA");	// 1
        th(     "BA'U");	// 1
        th(    "BAUNG");	// 1
        th(    "BAYNG");	// 1
        th( "'BA'ZHIG");	// 1
        th(       "BB");	// 1
        th(    "BBA'I");	// 1
        th(    "BBANG");	// 1
        th(    "'BBAS");	// 1
        th(   "BBCABS");	// 1
        th(    "BBCAS");	// 1
        th(     "BBEB");	// 1
        th(    "'BBEL");	// 1
        th(    "BBLAN");	// 1
        th(   "BBLUGS");	// 1
        th(     "BBRI");	// 1
        th(    "BBYED");	// 1
        th(    "BBYID");	// 1
        th(   "BBYUNG");	// 1
        th(     "BC'A");	// 1
        th(    "BCADS");	// 1
        th(     "BCAP");	// 1
        th(    "BCASD");	// 1
        th(     "BCCO");	// 1
        th(      "BCE");	// 1
        th(     "BCEL");	// 1
        th(   "BCENGS");	// 1
        th(     "BCHA");	// 1
        th(    "BCHA'");	// 1
        th(    "BCHAN");	// 1
        th(   "BCHANG");	// 1
        th(    "BCHAS");	// 1
        th(    "BCHED");	// 1
        th(    "BCHES");	// 1
        th(    "BCHIN");	// 1
        th(  "BCHINGS");	// 1
        th(    "BCHIS");	// 1
        th(     "BCHO");	// 1
        th(    "BCHOM");	// 1
        th(    "BCHUB");	// 1
        th(     "BCIB");	// 1
        th(     "BCOB");	// 1
        th(    "BCOMN");	// 1
        th(  "BCU'I'O");	// 1
        th(    "BCUIS");	// 1
        th(     "BCUt");	// 1
        th(     "BCWO");	// 1
        th(    "'BDAG");	// 1
        th(    "BDAGI");	// 1
        th(    "BDA'I");	// 1
        th(   "BDANGS");	// 1
        th(    "BDA'O");	// 1
        th(    "'BDAS");	// 1
        th(    "BDASG");	// 1
        th(    "BDDHA");	// 1
        th(   "'BDEBS");	// 1
        th(    "BDEGS");	// 1
        th(   "BDENAS");	// 1
        th(    "BDENG");	// 1
        th(    "BDE'U");	// 1
        th(     "BDGE");	// 1
        th(     "BDHA");	// 1
        th(    "BDH'A");	// 1
        th(    "BDHAD");	// 1
        th(    "BDH'U");	// 1
        th(     "BdIT");	// 1
        th(    "BDKAG");	// 1
        th(     "BDNE");	// 1
        th(   "BDRABS");	// 1
        th(    "BDRAG");	// 1
        th(   "BDRA'I");	// 1
        th(   "BDRA'O");	// 1
        th(    "BDRAS");	// 1
        th(     "BDRE");	// 1
        th(   "BDREGS");	// 1
        th(    "BDRIR");	// 1
        th(    "BDRIS");	// 1
        th(   "BDRUNG");	// 1
        th(  "BDRUNGS");	// 1
        th(    "BDRZA");	// 1
        th(    "BDUNI");	// 1
        th(   "BDVOGS");	// 1
        th(    "BDY'A");	// 1
        th(     "BDZA");	// 1
        th(    "BDZAG");	// 1
        th(    "BDZAM");	// 1
        th(    "BDZAN");	// 1
        th(    "BDZOD");	// 1
        th(   "BDZOGS");	// 1
        th(    "BDZON");	// 1
        th(   "BDZR'A");	// 1
        th("BDZRADh'U");	// 1
        th(    "BDZUD");	// 1
        th(  "BDZUNGS");	// 1
        th(  "BDZVA'I");	// 1
        th(      "BE:");	// 1
        th(      "BE2");	// 1
        th(      "BE3");	// 1
        th(      "BE4");	// 1
        th(    "BE'AM");	// 1
        th(    "BEd'U");	// 1
        th(   "BEd'UR");	// 1
        th(    "BEDZA");	// 1
        th( "BED:ZHES");	// 1
        th(     "BEED");	// 1
        th( "BEE-dA'U");	// 1
        th( "BEEd-NYU");	// 1
        th(  "BEE-d'U");	// 1
        th("BEEd'URLA");	// 1
        th("BEE-d'URYA");	// 1
        th("BEEd'UR-YA'I");	// 1
        th("BEED'URYA'I");	// 1
        th("BEEd'UR+YAS");	// 1
        th("BEEd'URYYA");	// 1
        th(   "BEEK'U");	// 1
        th("BEERNYDZA'I");	// 1
        th(   "BEEt'U");	// 1
        th("BEEt'UR+YA'I");	// 1
        th(     "'BEG");	// 1
        th(  "BEGCE'I");	// 1
        th( "BEGTZE'I");	// 1
        th(    "BE+HA");	// 1
        th(      "BEK");	// 1
        th(     "BEKA");	// 1
        th(    "BEKCE");	// 1
        th( "BEKTZE'I");	// 1
        th(      "BEm");	// 1
        th(    "BENDA");	// 1
        th(   "BEN+DE");	// 1
        th(   "BENDHA");	// 1
        th(  "BEN-DRA");	// 1
        th(   "BENTRA");	// 1
        th(     "'BER");	// 1
        th(  "BERGYUD");	// 1
        th(    "'BERL");	// 1
        th(   "BERY'A");	// 1
        th(     "'BES");	// 1
        th(  "BEshT'O");	// 1
        th(  "BESHTZA");	// 1
        th(    "BETTI");	// 1
        th(   "BGA'AM");	// 1
        th(     "BGAB");	// 1
        th(    "BGA'I");	// 1
        th(     "BGAN");	// 1
        th(     "BGAR");	// 1
        th(    "BGDAG");	// 1
        th(     "BGEG");	// 1
        th(    "BGLAB");	// 1
        th(   "BGLENG");	// 1
        th(    "BGLOS");	// 1
        th(    "BGOGS");	// 1
        th(    "BGO'I");	// 1
        th(   "BGOSSD");	// 1
        th(    "BGRA'");	// 1
        th(  "BGRA'AM");	// 1
        th(   "BGRABS");	// 1
        th(    "BGRAG");	// 1
        th(    "BGRAL");	// 1
        th(    "BGRAN");	// 1
        th(   "BGRA'O");	// 1
        th(    "BGRAS");	// 1
        th(    "BGRDS");	// 1
        th(  "BGRENGS");	// 1
        th(     "BGRI");	// 1
        th(    "BGRIB");	// 1
        th(    "BGRID");	// 1
        th(    "BGRIG");	// 1
        th(   "BGRIMS");	// 1
        th(   "BGRI'O");	// 1
        th(   "'BGROD");	// 1
        th("BGRODPA'I");	// 1
        th(   "BGRO'I");	// 1
        th(    "BGRON");	// 1
        th(   "BGRYAD");	// 1
        th(      "BGS");	// 1
        th(    "BGSAL");	// 1
        th(   "BGSANG");	// 1
        th(    "BGSAS");	// 1
        th(   "BGTEGS");	// 1
        th(  "BGTZONG");	// 1
        th(     "BGUG");	// 1
        th(    "BGUGS");	// 1
        th(    "BGU'O");	// 1
        th(    "BGYAB");	// 1
        th(    "BGYAN");	// 1
        th(   "BGYANG");	// 1
        th(   "'BGYED");	// 1
        th( "BGYIS'AM");	// 1
        th(   "BGYOGS");	// 1
        th(   "BGYONG");	// 1
        th(     "BGYS");	// 1
        th(   "BGYUGS");	// 1
        th(   "BGYU'O");	// 1
        th(    "BGYUS");	// 1
        th(   "BGZUGS");	// 1
        th(       "BH");	// 1
        th(    "BH'A:");	// 1
        th(     "BHA'");	// 1
        th(    "BHA'A");	// 1
        th(   "BHA'AM");	// 1
        th( "BHADANTA");	// 1
        th(  "BHAdd'A");	// 1
        th(   "BHADRE");	// 1
        th("BHAGSRB'A");	// 1
        th( "BHAKsh'A");	// 1
        th(     "BHAL");	// 1
        th(   "BH'ALA");	// 1
        th(    "BHA'm");	// 1
        th(    "BHAMS");	// 1
        th(  "BHANDHA");	// 1
        th(  "BHANDZI");	// 1
        th(   "BHANTI");	// 1
        th(    "BHA'O");	// 1
        th(  "BH'AshA");	// 1
        th( "BH'Ash'A");	// 1
        th(  "BH'AshI");	// 1
        th(    "BHATA");	// 1
        th(   "BHAtAS");	// 1
        th("BHAttARAKOTTAMASH'ILASH'ASA");	// 1
        th(  "BHATTRI");	// 1
        th(    "BHAWA");	// 1
        th("BHAWA'NYSH'U");	// 1
        th(  "BHDZDZA");	// 1
        th(    "BHE'I");	// 1
        th(   "BHET'A");	// 1
        th(      "BHi");	// 1
        th(    "BH'I:");	// 1
        th(     "BHI:");	// 1
        th("BHIDHARMMA");	// 1
        th(   "BHIDYA");	// 1
        th("BHIDZDZYA");	// 1
        th(     "BHIG");	// 1
        th(  "BHIKshA");	// 1
        th( "BHIKsh'A");	// 1
        th(  "BHIKshI");	// 1
        th(   "BHINDA");	// 1
        th(  "BHINGKA");	// 1
        th("BH'INTTA:'O");	// 1
        th(   "BHIR'O");	// 1
        th(   "BHIshE");	// 1
        th(   "BHISRA");	// 1
        th(   "BHITTA");	// 1
        th(  "BHNGAGA");	// 1
        th(    "BHNYA");	// 1
        th(  "BHNYDZA");	// 1
        th(     "BHOd");	// 1
        th(     "BHOD");	// 1
        th(   "BHOKTU");	// 1
        th(     "BHOL");	// 1
        th(     "BHOM");	// 1
        th(    "BHOtA");	// 1
        th(  "BHOtA'I");	// 1
        th( "BHR'ADZI");	// 1
        th(   "BHR'Am");	// 1
        th("BHRAMARA:'I");	// 1
        th(  "BHRANSI");	// 1
        th(  "BHRANSU");	// 1
        th(    "BHRAS");	// 1
        th( "BHRASDZA");	// 1
        th( "BHRASDZI");	// 1
        th(  "BHRAshA");	// 1
        th("BHRASHDZA");	// 1
        th(  "BHR'DZI");	// 1
        th(  "BHRiDGA");	// 1
        th(  "BHRiDHI");	// 1
        th(   "BHRIHU");	// 1
        th(   "BHRiNG");	// 1
        th(   "BHRING");	// 1
        th( "BHRiNGGA");	// 1
        th("BHRING+GA'I");	// 1
        th( "BHRINGKA");	// 1
        th(   "BHRiNY");	// 1
        th(    "BHRiS");	// 1
        th(  "BHRIshA");	// 1
        th(  "BHRTE'O");	// 1
        th(   "BHR'Un");	// 1
        th(  "BH'RY'A");	// 1
        th(  "BH'RYA'");	// 1
        th(     "BHSD");	// 1
        th(  "BHSNYEN");	// 1
        th(    "B+H'U");	// 1
        th(     "BHU'");	// 1
        th(     "BHUB");	// 1
        th(     "BHUD");	// 1
        th(  "BHUDDHI");	// 1
        th("BHUDDH'I'I");	// 1
        th("BHUDDHI'I");	// 1
        th(  "BHUDZRA");	// 1
        th(   "BHUGMA");	// 1
        th(  "BHUKshA");	// 1
        th(  "BHUKshI");	// 1
        th(    "BHUMI");	// 1
        th(    "BHUNG");	// 1
        th(  "BHUNGKA");	// 1
        th(  "BHUNN'A");	// 1
        th(   "BH'U'O");	// 1
        th(     "BHUR");	// 1
        th(    "BHUTA");	// 1
        th( "BHUTTE'O");	// 1
        th(    "BHU'U");	// 1
        th(   "BHVANG");	// 1
        th(      "BHY");	// 1
        th(  "BHY'AKA");	// 1
        th(    "BHYAM");	// 1
        th( "BHYA:P'U");	// 1
        th(   "BHYAS:");	// 1
        th(   "BHYASA");	// 1
        th("BHYA:SARBA");	// 1
        th("BHYA:SV'A");	// 1
        th(    "BH-YE");	// 1
        th(    "BHY'E");	// 1
        th(   "BHYEE:");	// 1
        th(     "BHYU");	// 1
        th(   "BHZUGS");	// 1
        th(     "B''I");	// 1
        th(    "BI-'A");	// 1
        th(     "BI'A");	// 1
        th(    "BIA'O");	// 1
        th(  "BIBHANS");	// 1
        th(    "BIBHU");	// 1
        th(   "BIBH'U");	// 1
        th(     "'BID");	// 1
        th(    "BIDHA");	// 1
        th(    "BIDTI");	// 1
        th(   "BId'UR");	// 1
        th(    "BIdYA");	// 1
        th(   "BI-DYA");	// 1
        th(  "BIDYAKA");	// 1
        th(    "BIDZA");	// 1
        th(     "BIED");	// 1
        th(    "'BIED");	// 1
        th(    "BIGHA");	// 1
        th( "BIGHANAN");	// 1
        th( "BIGHANTA");	// 1
        th(   "BIGHNA");	// 1
        th(  "BIGHNAN");	// 1
        th("BIGHNANTA");	// 1
        th(   "BIHDGA");	// 1
        th(    "'BI'I");	// 1
        th(    "BIK'A");	// 1
        th(  "BIKsh'A");	// 1
        th( "BIKshA'I");	// 1
        th(  "BIKshUG");	// 1
        th(   "BIKthU");	// 1
        th(     "BILA");	// 1
        th(   "BI-MBA");	// 1
        th("BIMBOOshthA:");	// 1
        th(   "BIM+PA");	// 1
        th( "BIM-PA'I");	// 1
        th(     "BIND");	// 1
        th("BING+GA'I");	// 1
        th(   "BINGKA");	// 1
        th(    "BINGS");	// 1
        th(  "BINGSHA");	// 1
        th(    "BIn'O");	// 1
        th(    "B'I'O");	// 1
        th(     "BI'O");	// 1
        th(     "B'IR");	// 1
        th(    "BIR'A");	// 1
        th(   "BIRJAD");	// 1
        th(   "BI-RYA");	// 1
        th(  "BIRYA'I");	// 1
        th(  "BI-RYKA");	// 1
        th(     "'BIS");	// 1
        th(    "BIshA");	// 1
        th(   "BISHA:");	// 1
        th(  "BIshA'I");	// 1
        th(   "BIshnU");	// 1
        th(   "BIshtA");	// 1
        th(  "BI-shtE");	// 1
        th( "BISHTSYA");	// 1
        th(   "BIshYA");	// 1
        th(   "BIST'A");	// 1
        th(      "BIt");	// 1
        th(   "BItA'O");	// 1
        th(    "BItnA");	// 1
        th(    "BITTA");	// 1
        th(   "BI-ttI");	// 1
        th( "BITZITSA");	// 1
        th(    "BIUNG");	// 1
        th(   "B'I-YA");	// 1
        th(  "BIZHI'I");	// 1
        th(      "BJA");	// 1
        th(     "BJA'");	// 1
        th(     "BJAG");	// 1
        th(     "BJAM");	// 1
        th(    "BJHIN");	// 1
        th(     "BJOM");	// 1
        th(    "BJORD");	// 1
        th(    "BJROD");	// 1
        th(     "BJUG");	// 1
        th(    "^BKA'");	// 1
        th(   "BKA'AM");	// 1
        th(    "BKAG'");	// 1
        th(     "BKAK");	// 1
        th(     "BKEN");	// 1
        th(     "BKES");	// 1
        th(     "BKHA");	// 1
        th(    "BKHA'");	// 1
        th(   "BKHEGS");	// 1
        th(     "BKHO");	// 1
        th(    "BKHOR");	// 1
        th(   "BKHRAN");	// 1
        th(   "BKHRID");	// 1
        th(   "BKHROL");	// 1
        th(    "BKHRU");	// 1
        th(   "BKHYED");	// 1
        th(   "BKHYIL");	// 1
        th(    "BKHYU");	// 1
        th(     "BKIG");	// 1
        th(    "BKLAB");	// 1
        th(    "BKLAd");	// 1
        th(  "BKLONGS");	// 1
        th(     "BKLU");	// 1
        th(   "BKLUGS");	// 1
        th(     "BKO'");	// 1
        th(    "BKOGS");	// 1
        th(    "^BKRA");	// 1
        th(    "'BKRA");	// 1
        th(  "BKRA'AM");	// 1
        th(   "BKRAGS");	// 1
        th(    "BKRAm");	// 1
        th(   "BKRA'O");	// 1
        th(   "^BKRAS");	// 1
        th(    "BKRED");	// 1
        th(    "BKREL");	// 1
        th(   "BKRENG");	// 1
        th(   "BKRE'O");	// 1
        th(   "'BKRES");	// 1
        th(    "BKshA");	// 1
        th(     "BKTO");	// 1
        th(    "BKUNG");	// 1
        th(    "BKURS");	// 1
        th(  "BKYANGS");	// 1
        th(   "BKYE'I");	// 1
        th(    "BKYEL");	// 1
        th(    "BKYEN");	// 1
        th(    "BKYID");	// 1
        th(    "BKYIG");	// 1
        th(    "BKYUS");	// 1
        th(    "BKZ'A");	// 1
        th(       "BL");	// 1
        th(     "^BLA");	// 1
        th(  "BLAKshA");	// 1
        th(   "BLAnGS");	// 1
        th(   "BLANGs");	// 1
        th(  "'BLANGS");	// 1
        th(   "BLCAGS");	// 1
        th(  "BLCI'AM");	// 1
        th(    "BLDAN");	// 1
        th(    "BLDAR");	// 1
        th(     "BLDE");	// 1
        th(   "BLDOGS");	// 1
        th(     "BLED");	// 1
        th(    "BLE'I");	// 1
        th(    "'BLEL");	// 1
        th(     "BLEN");	// 1
        th(    "BLENG");	// 1
        th(    "BLGAS");	// 1
        th(    "BLHAS");	// 1
        th(     "BLIL");	// 1
        th(   "BLKONG");	// 1
        th( "BLNGA'AM");	// 1
        th(     "^BLO");	// 1
        th(     "BLO'");	// 1
        th(     "BLO0");	// 1
        th(     "BLOA");	// 1
        th(     "BLOJ");	// 1
        th("BLOSKYANG");	// 1
        th(    "BLSAB");	// 1
        th(    "BLTA'");	// 1
        th(    "BLTAG");	// 1
        th(   "BLTAGS");	// 1
        th(     "BLTE");	// 1
        th(    "BLTIM");	// 1
        th(    "BLTOD");	// 1
        th(   "BLTOGS");	// 1
        th(     "BLTU");	// 1
        th(    "BLZOG");	// 1
        th(       "BM");	// 1
        th(     "Bm'A");	// 1
        th(      "BMA");	// 1
        th(     "BMED");	// 1
        th(      "BMI");	// 1
        th(    "BMYAL");	// 1
        th(    "BMYOD");	// 1
        th(   "BNA-HE");	// 1
        th(    "BNCAS");	// 1
        th(     "BNDE");	// 1
        th(    "BNDHA");	// 1
        th(     "BNED");	// 1
        th(     "BnEN");	// 1
        th(     "BNES");	// 1
        th(    "BNGAG");	// 1
        th(    "BNGAN");	// 1
        th(    "BNGAR");	// 1
        th(    "BNGON");	// 1
        th(   "BNGO'O");	// 1
        th(    "BNGOR");	// 1
        th(      "BNI");	// 1
        th(      "BNO");	// 1
        th(  "BNSNYEN");	// 1
        th(   "BNTA'O");	// 1
        th(     "BNUN");	// 1
        th(    "BNYAG");	// 1
        th(   "BNYAMS");	// 1
        th(    "BNYAN");	// 1
        th(    "BNYAS");	// 1
        th(  "BNYITZA");	// 1
        th(   "BNYTZA");	// 1
        th(   "BNYTZU");	// 1
        th(      "BO'");	// 1
        th(     "BO'A");	// 1
        th(    "BO'Am");	// 1
        th(    "'BOBS");	// 1
        th(      "BOd");	// 1
        th(   "BODHIS");	// 1
        th(   "BO'IMA");	// 1
        th(     "B'OK");	// 1
        th(    "'BONG");	// 1
        th(    "BONTA");	// 1
        th(      "BOO");	// 1
        th(     "B'OO");	// 1
        th(     "B'OR");	// 1
        th(     "BORU");	// 1
        th(    "BOSAm");	// 1
        th(   "BOSOGS");	// 1
        th(      "BOt");	// 1
        th(     "BOTA");	// 1
        th(     "BPA'");	// 1
        th(     "BPAG");	// 1
        th(     "BPAS");	// 1
        th(     "BPEN");	// 1
        th(   "BPHROS");	// 1
        th(   "BPHYIN");	// 1
        th(   "BPHYIR");	// 1
        th(   "BPHYIS");	// 1
        th(     "BPRO");	// 1
        th(    "BPYED");	// 1
        th(       "BR");	// 1
        th(    "'BR'A");	// 1
        th(  "'BRA'AM");	// 1
        th("BRABHAMATI");	// 1
        th(   "BRADZA");	// 1
        th(   "BRADZI");	// 1
        th( "BRADZNYA");	// 1
        th("BRADZNY'A");	// 1
        th(   "BRAHMA");	// 1
        th("BRAKshA'I");	// 1
        th(   "BRAL'I");	// 1
        th(    "BRAMS");	// 1
        th(   "BRAPTA");	// 1
        th(    "'BRAR");	// 1
        th(  "BRARTIR");	// 1
        th( "BRASHDZA");	// 1
        th(    "BRASL");	// 1
        th(   "'BRASL");	// 1
        th("BRATIshTHA");	// 1
        th(  "BRATITA");	// 1
        th( "BRATZTSA");	// 1
        th(   "BRBALS");	// 1
        th(   "'BRCES");	// 1
        th(    "BRCON");	// 1
        th(      "BRD");	// 1
        th(   "^BRDEG");	// 1
        th(    "BRDES");	// 1
        th(   "BRDE'U");	// 1
        th(     "BRDI");	// 1
        th(    "BRDMA");	// 1
        th(   "BRDONG");	// 1
        th(    "BRDUG");	// 1
        th(    "BRDUS");	// 1
        th(     "BRDZ");	// 1
        th(   "BRDZEN");	// 1
        th(   "BRDZIB");	// 1
        th(   "BRDZIN");	// 1
        th(  "BRDZING");	// 1
        th( "BRDZINGS");	// 1
        th(    "BRDZN");	// 1
        th(   "BRDZOB");	// 1
        th(   "BRDZOG");	// 1
        th(   "BRDZRA");	// 1
        th(  "BRDZRAS");	// 1
        th(   "BRDZUB");	// 1
        th(   "BRDZUR");	// 1
        th(   "BRE'AM");	// 1
        th(    "'BREB");	// 1
        th(    "'BREK");	// 1
        th(   "'BRELL");	// 1
        th(  "'BRELPA");	// 1
        th(    "'BRER");	// 1
        th(   "'BRERL");	// 1
        th(    "BRGAB");	// 1
        th(   "BRGAGS");	// 1
        th(    "BRGAM");	// 1
        th(    "BRGAR");	// 1
        th(   "BRGAYD");	// 1
        th( "BRGRANGS");	// 1
        th(   "BRGRYA");	// 1
        th(    "BRGUD");	// 1
        th(    "BRGUS");	// 1
        th(   "BRGYA:");	// 1
        th(   "BRGYA'");	// 1
        th(  "BRGYADS");	// 1
        th( "BRGYA-GO");	// 1
        th(    "BRGYD");	// 1
        th(    "BRGYI");	// 1
        th(   "BRGYOB");	// 1
        th(   "BRGYOD");	// 1
        th(  "BRGYUAD");	// 1
        th( "BRGYU'AM");	// 1
        th(    "BRHAN");	// 1
        th(    "''BRI");	// 1
        th("BR'iBRsh'A");	// 1
        th(     "BRiD");	// 1
        th(   "'BRIGS");	// 1
        th(    "BRi'I");	// 1
        th( "BRiK'Am:");	// 1
        th(   "BRIKsh");	// 1
        th( "BRiKshA:");	// 1
        th( "BRiKshA'");	// 1
        th(  "BRIKshA");	// 1
        th( "BRIKsh'A");	// 1
        th("BRIKsh'A:");	// 1
        th("BRiKshA'I");	// 1
        th("BRiKsh'AN");	// 1
        th("BRiKsh'ATRA");	// 1
        th("BRiKshATZTSA");	// 1
        th("BRIKshEE:");	// 1
        th("BRiKsh'NRA");	// 1
        th(     "BRim");	// 1
        th(  "'BRINGE");	// 1
        th("BRiNGNGH'A");	// 1
        th(  "'BRINGS");	// 1
        th(   "BRiNYA");	// 1
        th(  "BRINYAm");	// 1
        th(   "BRishI");	// 1
        th("BRIshthA:");	// 1
        th(   "BRIS'O");	// 1
        th(    "BRiTA");	// 1
        th(   "BRIT'O");	// 1
        th(   "BRITTA");	// 1
        th(  "BRiTTI:");	// 1
        th(    "BRI'U");	// 1
        th(    "BRJAB");	// 1
        th(    "BRJDO");	// 1
        th(    "BRJIG");	// 1
        th(  "BRJONGS");	// 1
        th(    "BRJUD");	// 1
        th(    "BRKA'");	// 1
        th(    "BRKAm");	// 1
        th(    "BRKEL");	// 1
        th(  "BRKO'AM");	// 1
        th(   "BRKO'O");	// 1
        th(   "BRKORU");	// 1
        th(    "BRKRU");	// 1
        th(    "BRKUB");	// 1
        th( "BRKUBA'I");	// 1
        th(   "BRKUMS");	// 1
        th("BRKYAENGS");	// 1
        th(   "BRKYAN");	// 1
        th(   "BRKYAS");	// 1
        th(   "BRKYNG");	// 1
        th(  "BRKYUNG");	// 1
        th(  "BRLA'AM");	// 1
        th(    "BRLAD");	// 1
        th(     "BRLG");	// 1
        th(  "BRLINGS");	// 1
        th(    "BRLON");	// 1
        th(   "BRnA'I");	// 1
        th(   "BRnA'O");	// 1
        th(    "BRnAS");	// 1
        th(     "BRnE");	// 1
        th(  "BRNGA'O");	// 1
        th(   "BRNGED");	// 1
        th(   "BRNGOL");	// 1
        th(    "BRNGU");	// 1
        th(  "BRNGUNG");	// 1
        th(    "BRn'I");	// 1
        th(    "BRNOD");	// 1
        th(    "BRNON");	// 1
        th(    "BRNUB");	// 1
        th(    "BRNUR");	// 1
        th(   "BRNYAG");	// 1
        th(  "BRNYA'I");	// 1
        th(   "BRNYAL");	// 1
        th(   "BRNYAR");	// 1
        th(  "BRNYEGS");	// 1
        th(   "BRNYON");	// 1
        th( "BRNYONGS");	// 1
        th(    "'BROm");	// 1
        th(   "'BROMS");	// 1
        th(   "BRONGS");	// 1
        th(    "BRRAL");	// 1
        th(   "BRRYAD");	// 1
        th(    "BRT'A");	// 1
        th(   "BRTAGm");	// 1
        th(    "BRTAL");	// 1
        th(    "BRTAR");	// 1
        th(   "BRTEAN");	// 1
        th(    "BRTEB");	// 1
        th(    "BRTEL");	// 1
        th(    "BRTGS");	// 1
        th(    "BRTIB");	// 1
        th(   "BRTIBS");	// 1
        th(   "BRTI'O");	// 1
        th(     "BRTN");	// 1
        th(     "BRTO");	// 1
        th(   "BRTOZN");	// 1
        th(   "BRTSON");	// 1
        th(     "BRTU");	// 1
        th(   "BRTUNG");	// 1
        th(  "BRTUNGS");	// 1
        th( "BRTZA'AM");	// 1
        th(   "BRTZAB");	// 1
        th(  "BRTZABS");	// 1
        th(  "BRTZAmS");	// 1
        th(   "BRTZEB");	// 1
        th(   "BRTZIB");	// 1
        th(   "BRTZIM");	// 1
        th(   "BRTZNL");	// 1
        th(   "BRTZOR");	// 1
        th(  "BRTZTSA");	// 1
        th(  "BRTZUNG");	// 1
        th(   "BRTZVA");	// 1
        th( "BRTZVA'I");	// 1
        th(   "BRTZWA");	// 1
        th(    "BRUGS");	// 1
        th(   "BR'U'I");	// 1
        th(     "BRUm");	// 1
        th(     "BRUM");	// 1
        th(   "'BRUNG");	// 1
        th(    "BRU'O");	// 1
        th(     "BRUR");	// 1
        th(    "BRYAD");	// 1
        th(    "BRYAN");	// 1
        th(    "BRYGA");	// 1
        th(  "BRYGANG");	// 1
        th(   "BS2TAN");	// 1
        th(    "BS'AB");	// 1
        th(   "BSAGAS");	// 1
        th(    "BSA'I");	// 1
        th(   "BSALDA");	// 1
        th(    "BSAMM");	// 1
        th(   "BSAMSO");	// 1
        th(  "^BSANGS");	// 1
        th(    "BSBAD");	// 1
        th(   "BSBANG");	// 1
        th(    "BSBAS");	// 1
        th(     "BSBE");	// 1
        th(     "BSBO");	// 1
        th(    "BSBOM");	// 1
        th(   "BSBOMS");	// 1
        th(    "BSBRE");	// 1
        th(    "BSBRU");	// 1
        th(    "BSBUB");	// 1
        th(   "BSBU'O");	// 1
        th(   "BSBYAD");	// 1
        th( "BSBYINGS");	// 1
        th(   "BSBYOD");	// 1
        th(   "BSBYUR");	// 1
        th(   "BSCAGS");	// 1
        th(  "BSCHAGS");	// 1
        th(   "BSDABS");	// 1
        th(    "BSDAG");	// 1
        th(   "BSDALS");	// 1
        th(   "^BSDAM");	// 1
        th(    "BSDEG");	// 1
        th(   "BSDEMS");	// 1
        th(  "BSDENGS");	// 1
        th(  "BSDGOGS");	// 1
        th(    "BSDGS");	// 1
        th(     "BSDI");	// 1
        th(   "BSDOBS");	// 1
        th(   "BSDOmS");	// 1
        th(    "BSDRU");	// 1
        th(   "BSDUAS");	// 1
        th(   "BSDUBA");	// 1
        th(   "BSDUGS");	// 1
        th(  "BSDUNGS");	// 1
        th(   "BSDZUN");	// 1
        th(    "BSGAB");	// 1
        th(   "BSGABS");	// 1
        th(    "BSGAS");	// 1
        th(     "BSGE");	// 1
        th(    "BSGES");	// 1
        th(    "BSGIG");	// 1
        th(    "BSGIL");	// 1
        th(    "BSGOL");	// 1
        th(  "BSGONGS");	// 1
        th(   "BSGRED");	// 1
        th(  "BSGRE'I");	// 1
        th(   "BSGREN");	// 1
        th(  "BSGROBS");	// 1
        th(  "BSGRONG");	// 1
        th( "BSGRONGS");	// 1
        th(  "BSGRO'O");	// 1
        th(  "^BSGRUB");	// 1
        th(  "BSGRUBO");	// 1
        th( "BSGRUBPA");	// 1
        th("BSGRUBPAR");	// 1
        th( "'BSGRUBS");	// 1
        th(   "BSGRUG");	// 1
        th(  "BSGUNGS");	// 1
        th(   "BSGURB");	// 1
        th(  "BSGYIBS");	// 1
        th(   "BSGYID");	// 1
        th(  "BSGYING");	// 1
        th( "BSGYONGS");	// 1
        th(  "BSGYUGS");	// 1
        th( "BSGYUNGS");	// 1
        th(  "^BSGYUR");	// 1
        th(  "BSGYURD");	// 1
        th( "BSGYURDA");	// 1
        th(  "BSGYURO");	// 1
        th(    "BSH'A");	// 1
        th(  "BSHA29D");	// 1
        th(   "'BSHAD");	// 1
        th(   "BSHADS");	// 1
        th(  "^BSHAGS");	// 1
        th(  "BSHAKAM");	// 1
        th(    "BshAt");	// 1
        th(    "BSHEL");	// 1
        th(     "BshI");	// 1
        th(     "BSHI");	// 1
        th(   "BSHING");	// 1
        th(    "BSHIR");	// 1
        th(    "BSHOG");	// 1
        th(   "BSHOMS");	// 1
        th(    "BSHRA");	// 1
        th(    "BSHRi");	// 1
        th(    "BshtI");	// 1
        th( "B'SHTZYA");	// 1
        th(  "BSHYAMS");	// 1
        th(    "BSIGS");	// 1
        th(     "BSIM");	// 1
        th(   "BSINGS");	// 1
        th(    "BSIUL");	// 1
        th(     "BSJI");	// 1
        th(  "BSKALDA");	// 1
        th(    "BSKAN");	// 1
        th(     "BSKE");	// 1
        th(    "BSKEL");	// 1
        th(    "BSKEM");	// 1
        th(   "BSKEYD");	// 1
        th(   "BSKOGS");	// 1
        th(    "BSKOO");	// 1
        th(  "BSKORDA");	// 1
        th(   "BSKRAN");	// 1
        th(   "BSKRES");	// 1
        th(   "BSKRIG");	// 1
        th(  "BSKRIMS");	// 1
        th(   "BSKROL");	// 1
        th(   "BSKRON");	// 1
        th(  "BSKRO'O");	// 1
        th(  "BSKRUBS");	// 1
        th(   "BSKUNG");	// 1
        th(   "BSKYEB");	// 1
        th(  "BSKYEBS");	// 1
        th(  "BSKYEED");	// 1
        th( "BSKYENGS");	// 1
        th(   "BSKYER");	// 1
        th(    "BSKYO");	// 1
        th(  "BSKYOMS");	// 1
        th( "BSKYONGS");	// 1
        th(   "BSKYUG");	// 1
        th(   "BSKYUL");	// 1
        th(    "BSLAL");	// 1
        th(    "BSLAM");	// 1
        th(   "BSLARS");	// 1
        th(    "BSLAS");	// 1
        th(     "BSLB");	// 1
        th(    "BSLEB");	// 1
        th(   "BSLOBS");	// 1
        th(   "BSLOMS");	// 1
        th(   "BSLUBS");	// 1
        th(   "BSLUNG");	// 1
        th(      "BSM");	// 1
        th(   "BSMYON");	// 1
        th(  "BSMYUNG");	// 1
        th(   "BSNAGL");	// 1
        th(    "BSNEL");	// 1
        th( "BSNGAMS3");	// 1
        th(  "BSNGANG");	// 1
        th(  "BSNGA'O");	// 1
        th(    "BSNGE");	// 1
        th( "BSNGO'AM");	// 1
        th("BSNGO'ANG");	// 1
        th(   "BSNGOD");	// 1
        th(   "BSNGOL");	// 1
        th(    "BSNGU");	// 1
        th(   "BSNGUB");	// 1
        th(    "BSNOM");	// 1
        th(   "BSNOMS");	// 1
        th(    "BSNUM");	// 1
        th(   "BSNUNG");	// 1
        th(   "BSNYAB");	// 1
        th(  "BSNYABS");	// 1
        th(    "BSNYD");	// 1
        th(   "BSNYIM");	// 1
        th(    "BSNYN");	// 1
        th(   "BSNYOM");	// 1
        th(  "BSNYONG");	// 1
        th(     "BSOB");	// 1
        th(   "BSONGS");	// 1
        th(    "BSO'O");	// 1
        th(    "BSPEL");	// 1
        th(     "BSPO");	// 1
        th(   "BSPOBS");	// 1
        th(   "BSPOMS");	// 1
        th(   "BSPONG");	// 1
        th(    "BSPOS");	// 1
        th(   "BSPRIM");	// 1
        th(   "BSPROS");	// 1
        th(   "BSPUNG");	// 1
        th(   "BSPYAN");	// 1
        th(   "BSPYED");	// 1
        th( "BSPYINGS");	// 1
        th(   "BSPYOD");	// 1
        th(   "BSRAGS");	// 1
        th(    "BSRAM");	// 1
        th(   "'BSREL");	// 1
        th(    "BSREN");	// 1
        th(   "BSRENG");	// 1
        th(   "BSRE'O");	// 1
        th(   "BSRI'O");	// 1
        th(   "BSRONG");	// 1
        th(   "BSRTEN");	// 1
        th( "'BSRUNGS");	// 1
        th(    "BSRVE");	// 1
        th(   "BSRYAS");	// 1
        th(   "BSTAGS");	// 1
        th(   "BSTA'I");	// 1
        th(    "BSTAL");	// 1
        th(    "BSTAn");	// 1
        th(   "^BSTAN");	// 1
        th(  "BSTANDA");	// 1
        th(   "BSTANM");	// 1
        th(  "BSTANTA");	// 1
        th(   "BSTA'O");	// 1
        th(    "BSTEG");	// 1
        th(    "BSTEM");	// 1
        th(    "BSTES");	// 1
        th(   "BSTH'A");	// 1
        th( "BSTI'ANG");	// 1
        th(    "BSTIB");	// 1
        th(    "BSTIG");	// 1
        th(    "BSTIS");	// 1
        th(   "BSTRAN");	// 1
        th(   "BSTSAN");	// 1
        th(   "BSTTEN");	// 1
        th(    "BSTUL");	// 1
        th(    "BSTUM");	// 1
        th(   "BSTZAD");	// 1
        th(   "BSTZAG");	// 1
        th(  "BSTZALD");	// 1
        th( "BSTZAL'I");	// 1
        th(   "BSTZAN");	// 1
        th(   "BSTZAS");	// 1
        th(   "BSTZOG");	// 1
        th(    "BSUGS");	// 1
        th(    "BSUMS");	// 1
        th(    "BSVOO");	// 1
        th(  "BSYENGS");	// 1
        th(    "BSYON");	// 1
        th(   "BTAGVS");	// 1
        th(   "BtANG0");	// 1
        th(     "BTAt");	// 1
        th(     "BTDA");	// 1
        th(     "BTEB");	// 1
        th(     "BTED");	// 1
        th(   "BTGEGS");	// 1
        th(     "BTGS");	// 1
        th(    "BTHAB");	// 1
        th(   "BTHAMS");	// 1
        th(   "BTHANG");	// 1
        th(    "BTHEG");	// 1
        th(     "BTHO");	// 1
        th(    "BTHOD");	// 1
        th(    "BTIGS");	// 1
        th(    "BTI'I");	// 1
        th(  "BTIKshU");	// 1
        th(     "BTIR");	// 1
        th(     "BTNG");	// 1
        th(     "BTOB");	// 1
        th(    "BTRAG");	// 1
        th(    "BTSAD");	// 1
        th(    "BTSAM");	// 1
        th(   "BTSANG");	// 1
        th(  "BTSANGS");	// 1
        th(   "BTSIMS");	// 1
        th(    "BTSOL");	// 1
        th(   "BTSONG");	// 1
        th(    "BTSOS");	// 1
        th(    "BTSUL");	// 1
        th(  "BTSUNGS");	// 1
        th(   "BTU:'O");	// 1
        th(    "BTZAB");	// 1
        th(    "BTZED");	// 1
        th( "BTZEETRI");	// 1
        th(   "BTZIGS");	// 1
        th(    "BTZIL");	// 1
        th(     "BTZN");	// 1
        th(   "BTZOMS");	// 1
        th(   "BTZO'O");	// 1
        th(    "BTZUB");	// 1
        th(    "BTZUL");	// 1
        th(    "BTZUS");	// 1
        th(   "BTZVAN");	// 1
        th(    "BTZVO");	// 1
        th(    "BTZWA");	// 1
        th(      "BU2");	// 1
        th(      "BU6");	// 1
        th(      "BUA");	// 1
        th(    "BU'AI");	// 1
        th(   "BUBH'U");	// 1
        th(     "'BUd");	// 1
        th( "BU-DADHA");	// 1
        th(   "BUDD'A");	// 1
        th(   "BUDDHE");	// 1
        th( "BUDDHI'O");	// 1
        th(   "BU+DHA");	// 1
        th(    "BUDHE");	// 1
        th(    "BUDHI");	// 1
        th(     "BUDI");	// 1
        th(  "BUDNY'A");	// 1
        th(   "BUGA'I");	// 1
        th( "BU'KHRUL");	// 1
        th(    "BUKRI");	// 1
        th(  "BUKsh'A");	// 1
        th(   "BUKshI");	// 1
        th(    "'BULO");	// 1
        th(    "BU'MA");	// 1
        th(   "BUMANG");	// 1
        th(  "BUM:AOM");	// 1
        th(     "BUMS");	// 1
        th(    "'BUMS");	// 1
        th(  "BUnA+YA");	// 1
        th(    "BUndA");	// 1
        th(   "BUN+YA");	// 1
        th(  "BUNYA'O");	// 1
        th(   "BUnYEE");	// 1
        th( "BUR-'ANG");	// 1
        th(  "BUR'ANG");	// 1
        th(    "BURnA");	// 1
        th(     "BUSH");	// 1
        th(  "BUshA'O");	// 1
        th(  "BUshtIm");	// 1
        th(    "BUtIK");	// 1
        th(   "BUtt'I");	// 1
        th(     "BU'U");	// 1
        th(   "'BUUNG");	// 1
        th(    "BUZNG");	// 1
        th(    "BVA'I");	// 1
        th(     "BVAL");	// 1
        th(   "BVANGS");	// 1
        th(    "BVDZA");	// 1
        th(      "BVI");	// 1
        th(     "BY3E");	// 1
        th(    "BY6ED");	// 1
        th(    "BYA'0");	// 1
        th(   "BYA'8I");	// 1
        th(   "'BYABS");	// 1
        th(  "BYADBHA");	// 1
        th(   "BYADZA");	// 1
        th(    "BYAED");	// 1
        th(    "BYAGN");	// 1
        th(  "BY'A'I'");	// 1
        th(    "'BYAL");	// 1
        th(    "BYAMD");	// 1
        th(   "BYAMS0");	// 1
        th(    "'BYAN");	// 1
        th(   "BY''AN");	// 1
        th(   "^BYANG");	// 1
        th(   "BYA'NG");	// 1
        th(   "BYANGG");	// 1
        th(  "BYA+NYA");	// 1
        th(  "BYANYCA");	// 1
        th( "BYANYDZA");	// 1
        th("BYANYDZAN");	// 1
        th(  "BYANYJA");	// 1
        th(   "BYA'O:");	// 1
        th(  "BYAPA'O");	// 1
        th(   "BYA'UG");	// 1
        th(     "^BYE");	// 1
        th(    "BYE3D");	// 1
        th(  "BYE'ANG");	// 1
        th(    "BYEDA");	// 1
        th(    "BYEDE");	// 1
        th(    "BYEDO");	// 1
        th(   "BYEDPA");	// 1
        th(  "BYEDPAS");	// 1
        th(    "BYEDS");	// 1
        th( "BYEDTHUN");	// 1
        th(     "BYEM");	// 1
        th(   "BYE'US");	// 1
        th(    "BYINR");	// 1
        th(    "'BYIR");	// 1
        th(  "BYI'U'I");	// 1
        th(     "BYNG");	// 1
        th(  "BYNYDZA");	// 1
        th(   "'BYOBS");	// 1
        th(   "'BYOGS");	// 1
        th(     "BYOM");	// 1
        th( "BYONGYIS");	// 1
        th(    "BYRID");	// 1
        th(      "BYS");	// 1
        th(     "BYTA");	// 1
        th(     "'BYU");	// 1
        th(     "BY'U");	// 1
        th(     "BYU'");	// 1
        th(   "BYUA'O");	// 1
        th("'BYUGNBA'I");	// 1
        th(    "BYU'I");	// 1
        th(    "'BYUM");	// 1
        th(    "BYUMS");	// 1
        th(    "BYUNB");	// 1
        th(   "BYUNBG");	// 1
        th("BYUNGBA'I");	// 1
        th("BYUNGGTZO");	// 1
        th( "'BYUNGNO");	// 1
        th(  "BYUNGYA");	// 1
        th(   "BYUNMG");	// 1
        th(      "BYY");	// 1
        th(     "BYYN");	// 1
        th(    "^BZA'");	// 1
        th(    "BZA'A");	// 1
        th(  "BZA'ANG");	// 1
        th(   "BZA'BA");	// 1
        th(  "BZEHNGS");	// 1
        th(     "BZEN");	// 1
        th(     "BZGI");	// 1
        th(    "BZHA'");	// 1
        th(   "BZHAGO");	// 1
        th(   "BZHA'I");	// 1
        th(   "BZHAmS");	// 1
        th(   "BZHGUS");	// 1
        th(   "BZHHIN");	// 1
        th(   "BZHI8N");	// 1
        th(   "BZHIBS");	// 1
        th(    "BZHId");	// 1
        th(   "BZHIGS");	// 1
        th(   "BZHI'i");	// 1
        th(    "BZHIK");	// 1
        th(    "BZHIM");	// 1
        th(  "BZHI'MA");	// 1
        th(    "BZhIN");	// 1
        th(   "'BZHIN");	// 1
        th(  "BZHINAU");	// 1
        th("BZHINDANG");	// 1
        th(  "BZHINDU");	// 1
        th(   "BZHINI");	// 1
        th(   "BZHINO");	// 1
        th(   "BZHINU");	// 1
        th(   "BZHIRU");	// 1
        th(    "BZHOM");	// 1
        th(   "BZHOMS");	// 1
        th(   "BZHUBS");	// 1
        th(   "BZHUGs");	// 1
        th( "BZHUG-SO");	// 1
        th(    "BZHUM");	// 1
        th(   "BZHUMS");	// 1
        th(    "BZIMS");	// 1
        th( "BZLA'O'I");	// 1
        th(   "BZLASA");	// 1
        th(    "BZLEG");	// 1
        th(    "BZLOD");	// 1
        th(  "BZLOGPA");	// 1
        th(     "BZLU");	// 1
        th(    "BZLUS");	// 1
        th(   "BZO'AM");	// 1
        th(     "BZOM");	// 1
        th(     "BZUB");	// 1
        th(    "BZU'I");	// 1
        th(     "BZUN");	// 1
        th(      "CA'");	// 1
        th(      "CA2");	// 1
        th(     "CADE");	// 1
        th(   "CADZIL");	// 1
        th(      "CAH");	// 1
        th(    "CAKYI");	// 1
        th(      "CAn");	// 1
        th(     "C'AN");	// 1
        th(     "CAN:");	// 1
        th( "CAN'BRAS");	// 1
        th(    "CANDA");	// 1
        th(    "CANDM");	// 1
        th(     "CANK");	// 1
        th(     "CANN");	// 1
        th(     "CANR");	// 1
        th(  "CANZHES");	// 1
        th(   "CARYAS");	// 1
        th(     "'CAS");	// 1
        th(     "CASN");	// 1
        th(     "CBUB");	// 1
        th(   "CBZHIN");	// 1
        th(       "CC");	// 1
        th(      "CE2");	// 1
        th(    "CEBAR");	// 1
        th(    "'CEBS");	// 1
        th(      "CED");	// 1
        th(      "CEE");	// 1
        th(     "CEHN");	// 1
        th(      "CEL");	// 1
        th(    "CEntA");	// 1
        th(     "'CER");	// 1
        th(    "CERYA");	// 1
        th(    "CES:-");	// 1
        th(     "CESE");	// 1
        th(     "CESO");	// 1
        th(     "CGIG");	// 1
        th(     "CGOM");	// 1
        th(   "CGZHAN");	// 1
        th(      "'CH");	// 1
        th(    "CH9OS");	// 1
        th(     "CH'A");	// 1
        th(     "CHA2");	// 1
        th(     "CHA4");	// 1
        th(   "'CHABS");	// 1
        th(   "CHA-dA");	// 1
        th(    "CHAEN");	// 1
        th(   "'CHAGN");	// 1
        th(  "^'CHANG");	// 1
        th(  "'CH'ANG");	// 1
        th(   "CHANNA");	// 1
        th(      "CHD");	// 1
        th(     "^CHE");	// 1
        th(     "CHE2");	// 1
        th(     "CHE4");	// 1
        th(   "'CHEBS");	// 1
        th("CHE'I'ANG");	// 1
        th(   "'CHENG");	// 1
        th(  "CHEN:LA");	// 1
        th(    "CHENM");	// 1
        th(    "CHENN");	// 1
        th(     "CHEO");	// 1
        th(   "'CHGAD");	// 1
        th(    "'CHI'");	// 1
        th(     "CHI2");	// 1
        th(     "CHI4");	// 1
        th(   "CHIBSS");	// 1
        th(    "'CHIM");	// 1
        th(   "CHINGs");	// 1
        th(     "CHO'");	// 1
        th(    "CHO0S");	// 1
        th(     "CHO3");	// 1
        th(    "CHOBS");	// 1
        th(   "'CHOLJ");	// 1
        th(   "'CHOMS");	// 1
        th(    "CHO'O");	// 1
        th( "CHOSKYES");	// 1
        th(    "CHOSM");	// 1
        th(   "'CHO'U");	// 1
        th(    "CHO'U");	// 1
        th(    "CHOWS");	// 1
        th(      "CHR");	// 1
        th(     "^CHU");	// 1
        th(     "CHU4");	// 1
        th(    "'CHUB");	// 1
        th(    "'CHUD");	// 1
        th(   "CHU'I'");	// 1
        th(   "CHUING");	// 1
        th( "CHUNGBAS");	// 1
        th(     "CHUP");	// 1
        th(    "CHU'U");	// 1
        th(     "CHYE");	// 1
        th(     "CHYU");	// 1
        th(      "C'I");	// 1
        th(      "CI'");	// 1
        th(      "CI2");	// 1
        th(      "CI4");	// 1
        th(    "CI8NG");	// 1
        th(   "CI'DRA");	// 1
        th(     "CIES");	// 1
        th(     "CIGL");	// 1
        th(     "CI'i");	// 1
        th(      "CII");	// 1
        th(    "'CI'I");	// 1
        th(     "'CIN");	// 1
        th(    "CINDA");	// 1
        th(    "CINGE");	// 1
        th(    "CINNG");	// 1
        th(    "CINTA");	// 1
        th(   "CINYID");	// 1
        th(    "CITRA");	// 1
        th(    "CITYA");	// 1
        th(     "CI'U");	// 1
        th(      "CLA");	// 1
        th(     "CLAD");	// 1
        th(    "CLAGS");	// 1
        th(    "CLEAR");	// 1
        th(    "CMGON");	// 1
        th(       "CN");	// 1
        th(      "CO3");	// 1
        th(    "CO'AM");	// 1
        th(     "'COD");	// 1
        th(     "COGS");	// 1
        th(     "COHS");	// 1
        th(  "COMPARE");	// 1
        th(    "CONGS");	// 1
        th(     "CO'O");	// 1
        th(     "CO'U");	// 1
        th(  "CPHYOGS");	// 1
        th(     "CTSE");	// 1
        th(     "'CUD");	// 1
        th(     "CUGS");	// 1
        th(      "CVA");	// 1
        th(     "CVAN");	// 1
        th(    "CZHIN");	// 1
        th(    "'CZIN");	// 1
        th(       "D'");	// 1
        th(     "D0ON");	// 1
        th(     "D3ES");	// 1
        th(      "D8U");	// 1
        th(      "D9O");	// 1
        th(      "^DA");	// 1
        th(     "'D'A");	// 1
        th(     "D'A:");	// 1
        th(    "DA0NG");	// 1
        th(      "DA2");	// 1
        th(      "DA4");	// 1
        th(     "DA'A");	// 1
        th(     "DAAG");	// 1
        th(      "dAB");	// 1
        th(  "'DA'BAR");	// 1
        th(     "DABS");	// 1
        th(  "DABYUNG");	// 1
        th(      "DAd");	// 1
        th(     "DAD'");	// 1
        th(     "DADG");	// 1
        th(    "DADHA");	// 1
        th(   "DADHYA");	// 1
        th(  "DA:DING");	// 1
        th(   "DADUNG");	// 1
        th(    "'DAG'");	// 1
        th(   "DAG'AM");	// 1
        th(  "DAG'ANG");	// 1
        th(     "DAGB");	// 1
        th(     "DAGG");	// 1
        th(    "DAGGA");	// 1
        th(    "d'AGI");	// 1
        th(    "DAGNG");	// 1
        th(     "DAGR");	// 1
        th( "DAGRGYAL");	// 1
        th(  "DAHA-'I");	// 1
        th(    "DA:'I");	// 1
        th(   "dA'ING");	// 1
        th(   "dA'I'O");	// 1
        th(      "DAK");	// 1
        th(     "DAK'");	// 1
        th(   "d'AKIS");	// 1
        th( "d'AKKI'I");	// 1
        th(   "DAKSHA");	// 1
        th( "DAKshAnI");	// 1
        th(   "DA-KTI");	// 1
        th(   "DAKTRI");	// 1
        th(     "D'AL");	// 1
        th(    "d'AMA");	// 1
        th(     "DAmA");	// 1
        th(     "DAMG");	// 1
        th(  "DAMPA'I");	// 1
        th(     "DAMR");	// 1
        th( "DAmshtRA");	// 1
        th("DAm-sh+tRA");	// 1
        th(    "DAN7G");	// 1
        th(  "D'AnA'I");	// 1
        th(    "DANBG");	// 1
        th(  "DANBHAI");	// 1
        th(   "DANBHU");	// 1
        th(    "DANDA");	// 1
        th(   "DAN-DA");	// 1
        th(  "DAnd'IS");	// 1
        th(   "DANDRA");	// 1
        th(   "DANDZA");	// 1
        th(    "d'ANG");	// 1
        th(    "DANG:");	// 1
        th(    "DANG'");	// 1
        th("DANG'BYUNG");	// 1
        th(  "DANGDON");	// 1
        th(    "DANGG");	// 1
        th("d'ANGGADA");	// 1
        th(    "DANGI");	// 1
        th( "DANGLDAN");	// 1
        th(  "DANGMED");	// 1
        th( "DANGNDGE");	// 1
        th(  "DANGNGS");	// 1
        th( "DANGSOGS");	// 1
        th(   "DANHAS");	// 1
        th(     "DANN");	// 1
        th(   "DANNAM");	// 1
        th(    "DANNG");	// 1
        th(     "DANT");	// 1
        th(   "dANT'A");	// 1
        th(   "D'ANTA");	// 1
        th(   "DANTAS");	// 1
        th(    "DAntI");	// 1
        th(  "DANtI'I");	// 1
        th(   "DANTYA");	// 1
        th("DANYTZA'I");	// 1
        th(    "D'A'O");	// 1
        th(     "D'AP");	// 1
        th(   "DARBAn");	// 1
        th(   "DARBAN");	// 1
        th(   "DARBHA");	// 1
        th(  "DAR-BHA");	// 1
        th(   "DARDZU");	// 1
        th(     "DARG");	// 1
        th("D'ARGHA'A'I");	// 1
        th(  "DA'RGIS");	// 1
        th(   "D'ARHI");	// 1
        th( "d'ARKANG");	// 1
        th(    "DARMA");	// 1
        th(  "DARPAnA");	// 1
        th( "DARSHARA");	// 1
        th( "D'AR-THA");	// 1
        th(      "dAS");	// 1
        th(     "DASG");	// 1
        th(   "DASH'I");	// 1
        th(    "dASSI");	// 1
        th(    "DASTA");	// 1
        th(      "DAT");	// 1
        th(    "-DATA");	// 1
        th(   "DATA'O");	// 1
        th(    "DAtDA");	// 1
        th(    "DATTA");	// 1
        th(  "DATZTSI");	// 1
        th(    "d'AUR");	// 1
        th( "d'AUR+YA");	// 1
        th("DA'URYA'I");	// 1
        th(       "DB");	// 1
        th(      "DBA");	// 1
        th(    "dBA'I");	// 1
        th(     "DBAM");	// 1
        th(    "DBANB");	// 1
        th(   "DBANG0");	// 1
        th(   "DBANGH");	// 1
        th( "DBANSEMS");	// 1
        th(    "DBARL");	// 1
        th(    "DBCAS");	// 1
        th(    "DBERN");	// 1
        th(   "DBGROG");	// 1
        th(   "DBGYES");	// 1
        th(      "DBI");	// 1
        th(     "DBIG");	// 1
        th(     "DBIN");	// 1
        th(    "DBLAS");	// 1
        th(  "DBO'ANG");	// 1
        th(     "DBOB");	// 1
        th(     "DBOD");	// 1
        th(   "DBPUGS");	// 1
        th(   "DBPYID");	// 1
        th(   "DBRA'I");	// 1
        th(    "DBRIL");	// 1
        th(    "DBRIR");	// 1
        th(     "DBRL");	// 1
        th(    "DBSRU");	// 1
        th(     "'DBU");	// 1
        th(      "DBY");	// 1
        th(    "DBYAG");	// 1
        th(   "DBYA'I");	// 1
        th(    "DBYAM");	// 1
        th("DBYANGSR'I");	// 1
        th(   "DBYASR");	// 1
        th(    "DBYEM");	// 1
        th(   "^DBYEN");	// 1
        th(  "DBYENGS");	// 1
        th(  "DBYIGNS");	// 1
        th(    "DBYIL");	// 1
        th( "DBYINGSU");	// 1
        th("DBYING-SU");	// 1
        th(   "DBYINS");	// 1
        th(   "DBYNGS");	// 1
        th(   "DBYOBS");	// 1
        th(  "DBYONGS");	// 1
        th(  "'DBYUNG");	// 1
        th(    "DBYUS");	// 1
        th(     "DCAB");	// 1
        th(     "DCES");	// 1
        th(   "DCHONG");	// 1
        th(    "DCING");	// 1
        th(      "DDA");	// 1
        th(    "DDAHA");	// 1
        th(     "DDAR");	// 1
        th(    "DDGA'");	// 1
        th(     "DDGE");	// 1
        th(    "dDH'A");	// 1
        th(     "DDHE");	// 1
        th(     "DDHO");	// 1
        th(      "DDO");	// 1
        th(     "'DDO");	// 1
        th(     "DDOS");	// 1
        th(  "DDRANGS");	// 1
        th(    "DDRIG");	// 1
        th(    "DDRUG");	// 1
        th(    "DDUNG");	// 1
        th(   "'DDZIN");	// 1
        th(      "d'E");	// 1
        th(      "DEA");	// 1
        th(   "'DE'AM");	// 1
        th(  "DE'ANGS");	// 1
        th(    "DEBAS");	// 1
        th(    "D'EBS");	// 1
        th(     "'DEd");	// 1
        th(    "DEDAG");	// 1
        th(     "DEDE");	// 1
        th(     "DE'E");	// 1
        th(    "DEEHA");	// 1
        th(     "DEEP");	// 1
        th("DEE:SV'AH'A");	// 1
        th(    "^DE'I");	// 1
        th(    "DE-'I");	// 1
        th(    "DE'I'");	// 1
        th(  "DE'I'AM");	// 1
        th( "DE'I'ANG");	// 1
        th(    "DE'ID");	// 1
        th("DE'IKHYAD");	// 1
        th( "DE'IRGYU");	// 1
        th(   "DE'I'U");	// 1
        th(  "DE'KHOR");	// 1
        th(     "DELA");	// 1
        th(    "DELAS");	// 1
        th(      "dEM");	// 1
        th(     "'DEM");	// 1
        th(    "DENAS");	// 1
        th(    "DE'NG");	// 1
        th( "DENYANOT");	// 1
        th(   "DENYID");	// 1
        th(   "DE'O'O");	// 1
        th(    "DERBI");	// 1
        th(    "DERGU");	// 1
        th(    "'DERN");	// 1
        th(  "DERTHAL");	// 1
        th(   "DERYAN");	// 1
        th(    "DESBS");	// 1
        th(    "DESHA");	// 1
        th(  "DESHTZA");	// 1
        th(    "DESNA");	// 1
        th(      "DET");	// 1
        th(   "DE'U'I");	// 1
        th(   "DGA'AR");	// 1
        th(   "DGA'BO");	// 1
        th( "DGAGDBYE");	// 1
        th(  "DGA'I'O");	// 1
        th(     "DGAM");	// 1
        th(  "DGANGVA");	// 1
        th(    "DGA'R");	// 1
        th(    "DG'AS");	// 1
        th(    "DGBRA");	// 1
        th(     "'DGE");	// 1
        th(     "DGE0");	// 1
        th(   "DGE'Am");	// 1
        th(     "DGEL");	// 1
        th(  "DGE'O'I");	// 1
        th(   "DGHA'I");	// 1
        th( "DGHATYYA");	// 1
        th(    "DGHOS");	// 1
        th(    "DGNAS");	// 1
        th(    "'DGOD");	// 1
        th(    "DGODS");	// 1
        th(     "DGOM");	// 1
        th(   "DGONGC");	// 1
        th(   "DGONGs");	// 1
        th(     "DGOR");	// 1
        th(  "DGOSNGS");	// 1
        th(    "DGOSO");	// 1
        th(    "DGRAB");	// 1
        th(   "DGRAGS");	// 1
        th(   "DGRANG");	// 1
        th( "DGRA'O:-");	// 1
        th(    "DGRES");	// 1
        th(     "DGRI");	// 1
        th(    "DGR'I");	// 1
        th(   "DGRI'I");	// 1
        th(    "DGRIS");	// 1
        th(    "DGROD");	// 1
        th(    "DGROR");	// 1
        th(    "DGROS");	// 1
        th(   "D'GRUB");	// 1
        th(   "DGRU'I");	// 1
        th(    "DGRUR");	// 1
        th(    "DGRVA");	// 1
        th(  "DGSHEGS");	// 1
        th(     "DGSO");	// 1
        th(    "DGSOD");	// 1
        th(    "DGSOL");	// 1
        th(    "DGUGS");	// 1
        th(    "DGUNS");	// 1
        th(    "DGVOS");	// 1
        th(    "DGYAR");	// 1
        th(   "DGYE'I");	// 1
        th(   "DGYI'I");	// 1
        th(    "DGYIL");	// 1
        th(   "DGYI'O");	// 1
        th(    "DGYOD");	// 1
        th(    "DGYUG");	// 1
        th(    "DGYUR");	// 1
        th(   "D'GYUR");	// 1
        th(      "dhA");	// 1
        th(    "DH'A:");	// 1
        th(    "DH'AD");	// 1
        th(    "dHA'I");	// 1
        th(   "DH'A'i");	// 1
        th(    "DH'AI");	// 1
        th(   "DHA-'I");	// 1
        th(   "DHA'I:");	// 1
        th("DH'AI:AOM");	// 1
        th(  "DHA'I'O");	// 1
        th(   "DHALLA");	// 1
        th(     "dhAm");	// 1
        th(     "DHAM");	// 1
        th(   "DHAM'A");	// 1
        th( "DHAM'A'I");	// 1
        th( "DHAMBHA'");	// 1
        th(  "DH'AN'O");	// 1
        th(   "DHANTU");	// 1
        th( "DH'AN+YA");	// 1
        th( "DHANYDZA");	// 1
        th(   "DHARAM");	// 1
        th(  "DHARDHA");	// 1
        th(    "DHARM");	// 1
        th(  "^DHARMA");	// 1
        th( "DHA-RMA:");	// 1
        th( "DHARM'A:");	// 1
        th(  "DHARMA:");	// 1
        th(  "DHARMA'");	// 1
        th( "DHARMA'A");	// 1
        th("DHARM'A'I");	// 1
        th("DHARMAK'I");	// 1
        th( "DHARM'AS");	// 1
        th( "DHARMASA");	// 1
        th("DHARM'A:SVA");	// 1
        th("DHARM'A:SVABH'A");	// 1
        th( "DHARM'AT");	// 1
        th("DHARM'I'A");	// 1
        th( "DHARMI'I");	// 1
        th( "DHARMM'A");	// 1
        th(  "DHA-RMO");	// 1
        th(   "DHARYA");	// 1
        th(  "DHA'UNY");	// 1
        th(      "dHE");	// 1
        th(      "DhE");	// 1
        th(     "dHEm");	// 1
        th(     "DHHA");	// 1
        th(      "dHI");	// 1
        th(  "DH'I:GU");	// 1
        th("DH'I:H'Um");	// 1
        th(  "DHIKshU");	// 1
        th(     "DHIm");	// 1
        th(   "DHI'NG");	// 1
        th(   "DHiNYA");	// 1
        th(   "DH'I'O");	// 1
        th(    "DH'IR");	// 1
        th(  "DHIRBHA");	// 1
        th(  "DH'I:SA");	// 1
        th( "DHIshshA");	// 1
        th(  "DH'ISSA");	// 1
        th( "DHISTHA'");	// 1
        th( "DH'I:YIG");	// 1
        th("DH'I:ZHES");	// 1
        th( "DHI:ZHES");	// 1
        th(   "DH'NYA");	// 1
        th(      "DhO");	// 1
        th(     "DH'O");	// 1
        th(     "DHOO");	// 1
        th(    "DHRAL");	// 1
        th(    "DHRAm");	// 1
        th(    "DHRAM");	// 1
        th( "DHRAmshI");	// 1
        th(    "DHRIG");	// 1
        th(   "DHRI'I");	// 1
        th(    "dhRiK");	// 1
        th(    "dHRIK");	// 1
        th(   "dHRIKA");	// 1
        th(   "DHRIKA");	// 1
        th(   "DHRiNG");	// 1
        th(  "DHRishi");	// 1
        th(    "DHRMA");	// 1
        th(   "DHRPAN");	// 1
        th(   "DHRU'O");	// 1
        th(    "DH'U:");	// 1
        th(   "DH'U'I");	// 1
        th(   "DH'UI:");	// 1
        th( "DH'UI:'I");	// 1
        th(    "DH'Um");	// 1
        th(     "DHUN");	// 1
        th(  "DH'UNYA");	// 1
        th(    "DHU'O");	// 1
        th(    "DH'UP");	// 1
        th(   "DHURBA");	// 1
        th(   "DHURBI");	// 1
        th(    "DHUTI");	// 1
        th(   "DHUt'O");	// 1
        th(   "DHUtRA");	// 1
        th( "DHUtRA'I");	// 1
        th(  "DHVADZA");	// 1
        th(    "DHVAm");	// 1
        th(   "DHV'AM");	// 1
        th(   "dHV'AN");	// 1
        th(    "DHVAN");	// 1
        th(   "DHVANI");	// 1
        th(  "DHVANSI");	// 1
        th(  "DHVANSU");	// 1
        th("DHVANYDZA");	// 1
        th(    "DHVAS");	// 1
        th(     "dHYA");	// 1
        th(   "DHY'AN");	// 1
        th(   "DHYA'N");	// 1
        th(    "DHYAR");	// 1
        th(     "DHYE");	// 1
        th(    "D'HYI");	// 1
        th(      "'di");	// 1
        th(      "DI2");	// 1
        th(      "DI4");	// 1
        th(     "'DI8");	// 1
        th(    "'DI9S");	// 1
        th( "'DI-'ANG");	// 1
        th("'DIAU-TPA");	// 1
        th(     "'DIB");	// 1
        th(    "DIBAN");	// 1
        th(      "did");	// 1
        th(      "DID");	// 1
        th(    "d'IDA");	// 1
        th(     "'DIE");	// 1
        th(    "d'IGA");	// 1
        th(     "DIHA");	// 1
        th(   "DI'I'O");	// 1
        th(      "dIK");	// 1
        th(     "dIKA");	// 1
        th(     "DIKA");	// 1
        th(   "DIKK'A");	// 1
        th(  "D'IKshA");	// 1
        th(   "DIKshI");	// 1
        th(   "DIKshU");	// 1
        th("DIKSRBEBHYA:");	// 1
        th(     "DILA");	// 1
        th(  "'DILTAR");	// 1
        th(     "'DIM");	// 1
        th(    "DI'NG");	// 1
        th(    "'DINI");	// 1
        th( "DIN+YA'I");	// 1
        th(    "'DIPA");	// 1
        th(      "dIR");	// 1
        th(   "'DIRAB");	// 1
        th(   "D'IRGA");	// 1
        th(  "D'IRGAS");	// 1
        th(  "DIRGH'A");	// 1
        th("D'IRGH'A'I");	// 1
        th( "D'IRGHAS");	// 1
        th("D'IRGHATA");	// 1
        th( "D'IRGT'O");	// 1
        th(  "'DIRNGS");	// 1
        th(      "dIT");	// 1
        th(    "DITSA");	// 1
        th(     "'DIU");	// 1
        th(     "DI'U");	// 1
        th(     "DJOS");	// 1
        th(   "DKA'AM");	// 1
        th(   "DKA'AN");	// 1
        th(  "DKA'A'O");	// 1
        th(     "DKAD");	// 1
        th(     "DKAL");	// 1
        th(    "^DKAR");	// 1
        th(    "DKHAm");	// 1
        th(    "DKHON");	// 1
        th(   "DKHYOD");	// 1
        th(   "DKHYUS");	// 1
        th(      "DKI");	// 1
        th(     "DKIL");	// 1
        th(     "DKIS");	// 1
        th(      "DKO");	// 1
        th(    "DKO0N");	// 1
        th(   "DKONDA");	// 1
        th(    "DKOOG");	// 1
        th(   "DKRA'I");	// 1
        th(    "DKREL");	// 1
        th(     "DKRO");	// 1
        th(    "DKRON");	// 1
        th(    "DKROS");	// 1
        th(     "DKRU");	// 1
        th(    "DKshA");	// 1
        th(    "DKshI");	// 1
        th(     "DKUG");	// 1
        th(    "DKUGS");	// 1
        th(    "DKUNG");	// 1
        th(    "DKVA'");	// 1
        th(    "DKYAS");	// 1
        th(    "DKYED");	// 1
        th(   "^DKYIL");	// 1
        th(    "DKYIN");	// 1
        th(     "DKYO");	// 1
        th(   "DKYOBS");	// 1
        th(    "DKYOG");	// 1
        th(    "DKYOL");	// 1
        th(    "DKYON");	// 1
        th(    "DKYOR");	// 1
        th(   "^DKYUS");	// 1
        th(      "DLA");	// 1
        th(      "DLI");	// 1
        th(     "DLIP");	// 1
        th(    "DLONG");	// 1
        th(   "DLTUNG");	// 1
        th( "dLURYA'I");	// 1
        th(       "DM");	// 1
        th(   "DMAL'A");	// 1
        th(    "DMBHI");	// 1
        th(     "DMED");	// 1
        th(     "DMEN");	// 1
        th(      "DMG");	// 1
        th(   "DMIGSZ");	// 1
        th(   "DMIHGS");	// 1
        th(    "DMItA");	// 1
        th(      "DMM");	// 1
        th(    "DMOGS");	// 1
        th(     "DMOL");	// 1
        th(     "DMON");	// 1
        th(     "DMTA");	// 1
        th(  "DMYALNA");	// 1
        th(   "DMYIBS");	// 1
        th(   "DMYIGS");	// 1
        th(   "DMYONG");	// 1
        th(     "DMYU");	// 1
        th(    "DMYUL");	// 1
        th(      "'DN");	// 1
        th(      "DnA");	// 1
        th(     "DNAD");	// 1
        th(     "DNAM");	// 1
        th(    "DNANG");	// 1
        th(     "DNAS");	// 1
        th(  "DNBHI'I");	// 1
        th(     "DndA");	// 1
        th(     "DndE");	// 1
        th(   "DNGA'I");	// 1
        th(     "DNGG");	// 1
        th(    "DNGOL");	// 1
        th(   "^DNGOS");	// 1
        th(   "DNGOS:");	// 1
        th(   "DNGOS0");	// 1
        th("DNGOSKHYAB");	// 1
        th(    "DNGSO");	// 1
        th(     "DNGU");	// 1
        th(    "DNGUD");	// 1
        th(   "'DNGUL");	// 1
        th(   "DNIBHA");	// 1
        th(      "DNO");	// 1
        th(     "DNOS");	// 1
        th(    "DNYED");	// 1
        th(      "DO0");	// 1
        th(      "DO3");	// 1
        th(     "DO99");	// 1
        th(   "DO'ANG");	// 1
        th(     "'DOB");	// 1
        th(    "^'DOD");	// 1
        th(     "DODN");	// 1
        th(     "DODS");	// 1
        th(     "DO'E");	// 1
        th(   "dOLGHI");	// 1
        th(     "DOMI");	// 1
        th(  "dOMZ+HI");	// 1
        th(    "DONDU");	// 1
        th(     "DONJ");	// 1
        th( "DONTHAMS");	// 1
        th(    "DONTO");	// 1
        th(    "'DOOD");	// 1
        th(     "DOON");	// 1
        th(    "DOO'O");	// 1
        th(     "DOPN");	// 1
        th(       "DP");	// 1
        th(    "DP''A");	// 1
        th(    "DPA''");	// 1
        th( "DPA'CHOS");	// 1
        th(    "^DPAL");	// 1
        th(    "'DPAL");	// 1
        th(    "DPAL0");	// 1
        th(  "DPALDAN");	// 1
        th(    "DPALI");	// 1
        th(    "DPALS");	// 1
        th(     "DPAM");	// 1
        th(    "DPA'S");	// 1
        th( "DPA'ZHES");	// 1
        th(     "'DPD");	// 1
        th(     "DPED");	// 1
        th(    "DPEER");	// 1
        th(     "DPEG");	// 1
        th(     "DPEM");	// 1
        th(  "DPE'O'I");	// 1
        th(     "DPGA");	// 1
        th(    "DPHA'");	// 1
        th(    "DPHAG");	// 1
        th(    "DPHAL");	// 1
        th(     "DPHE");	// 1
        th(    "DPHEN");	// 1
        th(    "DPHES");	// 1
        th(    "DPHID");	// 1
        th(    "DPHOD");	// 1
        th(    "DPHON");	// 1
        th(     "DPHU");	// 1
        th(  "DPHYAGS");	// 1
        th(  "DPHYANG");	// 1
        th( "DPHYANGS");	// 1
        th(  "DPHYIGS");	// 1
        th(   "DPHYIS");	// 1
        th(     "DP'O");	// 1
        th(     "DPOD");	// 1
        th(     "DPOR");	// 1
        th(     "DPOS");	// 1
        th(      "DPR");	// 1
        th(     "DPRA");	// 1
        th(     "DPUG");	// 1
        th(     "DPUM");	// 1
        th(     "DPUN");	// 1
        th(    "DPYA'");	// 1
        th(   "DPYEBS");	// 1
        th(   "DPYE'I");	// 1
        th(   "DPYING");	// 1
        th(  "DPYINGS");	// 1
        th(   "DPYOAD");	// 1
        th(   "DPYODD");	// 1
        th(   "DPYONG");	// 1
        th(  "DPYONGS");	// 1
        th(    "DPYOR");	// 1
        th(    "DPYOS");	// 1
        th(    "DPYUG");	// 1
        th(   "DPYUNG");	// 1
        th(      "'DR");	// 1
        th(    "DR'A:");	// 1
        th(   "DRA'AM");	// 1
        th("'DRA'ANGS");	// 1
        th(   "DR'A'i");	// 1
        th( "DRAKIshA");	// 1
        th(    "DR'Am");	// 1
        th(    "DR'AN");	// 1
        th( "'DRA'NGA");	// 1
        th(   "DRANGs");	// 1
        th(   "DR'A'O");	// 1
        th(   "DRA:'O");	// 1
        th( "'DRA'O'I");	// 1
        th(  "DRAS+YA");	// 1
        th(    "DRBAN");	// 1
        th(    "DRBTA");	// 1
        th(     "DRDL");	// 1
        th(  "'DRE'AM");	// 1
        th(     "DRGA");	// 1
        th(   "D'RGHI");	// 1
        th( "D'RGHI'O");	// 1
        th(     "DRGS");	// 1
        th(   "DRGYAL");	// 1
        th(   "DRGYAS");	// 1
        th(    "dRGYU");	// 1
        th(    "DRGYU");	// 1
        th(    "'DR'I");	// 1
        th(    "'DRIB");	// 1
        th(    "dRiKA");	// 1
        th(  "DRiKshA");	// 1
        th(    "'DRIO");	// 1
        th(  "DRI'O'I");	// 1
        th(    "DRKIS");	// 1
        th(  "DRLARNA");	// 1
        th(     "'DRN");	// 1
        th(    "'DR'O");	// 1
        th(     "DR'O");	// 1
        th(     "DROB");	// 1
        th(     "DROG");	// 1
        th(     "DROM");	// 1
        th(   "'DRO'O");	// 1
        th(   "DROO'O");	// 1
        th(  "DRsht'O");	// 1
        th(    "DRTOG");	// 1
        th(     "DRUA");	// 1
        th(    "DRUGM");	// 1
        th(    "DRUGO");	// 1
        th(     "DRUM");	// 1
        th(     "DRUN");	// 1
        th(    "'DRUN");	// 1
        th(  "'DRUNGS");	// 1
        th(    "DRUNS");	// 1
        th(    "DRUPS");	// 1
        th(     "DRUR");	// 1
        th(    "'DRUS");	// 1
        th(    "DRU'U");	// 1
        th(    "DRVAm");	// 1
        th(   "DRVANG");	// 1
        th(    "'DRVE");	// 1
        th(       "DS");	// 1
        th(     "DSAG");	// 1
        th(    "DSANG");	// 1
        th(      "DSD");	// 1
        th(     "DSER");	// 1
        th(     "DSGO");	// 1
        th(   "DshTZI");	// 1
        th(     "DSIM");	// 1
        th(    "DSKAD");	// 1
        th(   "DSKYAB");	// 1
        th(   "DSKYED");	// 1
        th(   "DSKYES");	// 1
        th(   "DSLONG");	// 1
        th(   "DSMRAS");	// 1
        th(   "DSNGON");	// 1
        th(      "DSO");	// 1
        th(     "DSOB");	// 1
        th(     "DSOD");	// 1
        th(     "DSOG");	// 1
        th(    "DSO'I");	// 1
        th(   "DSPONG");	// 1
        th(     "DSTE");	// 1
        th(      "DSU");	// 1
        th(     "DSUM");	// 1
        th(     "DSYU");	// 1
        th(    "DTA'I");	// 1
        th(     "DTEN");	// 1
        th(    "DTHEG");	// 1
        th(      "DTO");	// 1
        th(    "DTONG");	// 1
        th(    "DTRIG");	// 1
        th(   "DTSIGS");	// 1
        th(      "DTU");	// 1
        th(     "d+'U");	// 1
        th(      "DU'");	// 1
        th(      "DU4");	// 1
        th(    "DU'AD");	// 1
        th(   "'DU'AM");	// 1
        th(    "DUANG");	// 1
        th(      "dUD");	// 1
        th(   "DUDDHA");	// 1
        th(  "DU'GYUR");	// 1
        th(     "'DUI");	// 1
        th(   "DU'I'A");	// 1
        th( "DU'I'ANG");	// 1
        th(   "DU'JUG");	// 1
        th( "'DUKHANG");	// 1
        th(   "DUKshI");	// 1
        th(    "^'DUL");	// 1
        th(  "DU-MB'A");	// 1
        th(   "DUM-BA");	// 1
        th(  "DUM-B'A");	// 1
        th(   "dUMBHI");	// 1
        th(   "DUMLBA");	// 1
        th(    "DUMWA");	// 1
        th(     "DUNI");	// 1
        th(     "DUNS");	// 1
        th(     "dU'O");	// 1
        th(    "DURAB");	// 1
        th(   "DURBED");	// 1
        th(   "d'UR'I");	// 1
        th(  "d'U+RYA");	// 1
        th(  "d'UR-YA");	// 1
        th( "dURY+A'I");	// 1
        th(  "dURYA'I");	// 1
        th(  "DURYA'I");	// 1
        th( "D'URYARA");	// 1
        th(  "D'URYAS");	// 1
        th( "D'UR+Y'I");	// 1
        th(     "^DUS");	// 1
        th(    "'DUS:");	// 1
        th(    "DUSSE");	// 1
        th(    "DU-SU");	// 1
        th(      "dVA");	// 1
        th(     "dV'A");	// 1
        th(     "'DVA");	// 1
        th(     "DVAD");	// 1
        th( "DVADshnA");	// 1
        th(    "DV'Am");	// 1
        th(   "'DVA'O");	// 1
        th(    "DV'AR");	// 1
        th(     "DVED");	// 1
        th(     "'DVI");	// 1
        th(    "DVKAR");	// 1
        th(    "DVPER");	// 1
        th(    "'DVRA");	// 1
        th(    "DVRAG");	// 1
        th(     "DVRI");	// 1
        th(      "DVt");	// 1
        th(      "dVU");	// 1
        th(    "DVYA:");	// 1
        th(    "DVYAS");	// 1
        th(   "DV'YAS");	// 1
        th(       "DW");	// 1
        th(       "DY");	// 1
        th(      "dYA");	// 1
        th(     "D-YA");	// 1
        th(     "D+YA");	// 1
        th(     "DYA'");	// 1
        th(     "DYAL");	// 1
        th(    "DYALi");	// 1
        th(    "dYA'N");	// 1
        th(    "DY'AN");	// 1
        th(    "DYANG");	// 1
        th(   "DYANGS");	// 1
        th(     "DYAR");	// 1
        th(     "DYAS");	// 1
        th(     "DYAt");	// 1
        th(     "DYED");	// 1
        th(     "DYER");	// 1
        th(   "DYEshA");	// 1
        th(    "DYGAS");	// 1
        th(   "DYOTTA");	// 1
        th(       "DZ");	// 1
        th(     "DZA2");	// 1
        th(  "DZABBHA");	// 1
        th( "DZA-DZA:");	// 1
        th("DZA:DZA:AOm");	// 1
        th(    "DZAGS");	// 1
        th(   "DZAH'A");	// 1
        th( "DZAHNU'I");	// 1
        th(  "DZAHNUS");	// 1
        th( "DZA:'JAM");	// 1
        th(     "DZAL");	// 1
        th(    "'DZAL");	// 1
        th("DZAMBHALA");	// 1
        th(  "DZAMBHU");	// 1
        th(  "DZAM-BU");	// 1
        th(  "DZAMBUN");	// 1
        th(  "DZAMBUS");	// 1
        th( "DZA:MGON");	// 1
        th( "'DZAM-PU");	// 1
        th(  "DZA:NAS");	// 1
        th(   "DZANGS");	// 1
        th(  "DZANY'A");	// 1
        th( "DZANYA'I");	// 1
        th(   "DZ'A'O");	// 1
        th(   "DZARYA");	// 1
        th(    "DZ'AS");	// 1
        th(   "DZA:SA");	// 1
        th(  "DZ'ATME");	// 1
        th( "DZAYANTU");	// 1
        th("DZA:YONGS");	// 1
        th( "DZA:ZHES");	// 1
        th(    "DZDGA");	// 1
        th(   "DZDGHA");	// 1
        th(     "DZE2");	// 1
        th(     "DZE4");	// 1
        th(    "'DZEd");	// 1
        th(    "DZEEm");	// 1
        th(    "DZE'I");	// 1
        th(   "'DZENG");	// 1
        th(     "DZER");	// 1
        th(   "DZ'G'A");	// 1
        th(    "DZHAL");	// 1
        th("DZHEE:SV'AH'A");	// 1
        th(    "DZHES");	// 1
        th("DZHIKshIm");	// 1
        th(  "'DZHUGS");	// 1
        th(  "DZHUNGS");	// 1
        th(   "DZHYAS");	// 1
        th(    "'DZI^");	// 1
        th(     "DZI4");	// 1
        th( "'DZI'ANG");	// 1
        th(     "DZIG");	// 1
        th("DZIGHRA'O");	// 1
        th(    "DZIGS");	// 1
        th(   "DZIHVA");	// 1
        th("DZImshnU:");	// 1
        th(    "DZIN:");	// 1
        th(   "'DZIN0");	// 1
        th(   "'DZINB");	// 1
        th(  "'DZINDA");	// 1
        th(  "DZING'O");	// 1
        th(   "'DZINM");	// 1
        th(   "'DZINN");	// 1
        th(  "DZ'IRYA");	// 1
        th( "DZ'IR+YA");	// 1
        th("DZKIDBHYA:SAN");	// 1
        th(    "DZMBU");	// 1
        th( "DZM-BU'I");	// 1
        th(  "DZMBU'I");	// 1
        th(    "DZN'A");	// 1
        th("DZ'NGKshA");	// 1
        th(    "DZnOO");	// 1
        th(    "DZNRA");	// 1
        th(   "DZ'NTA");	// 1
        th(    "DZNY'");	// 1
        th(  "DZ-NY'A");	// 1
        th(  "DZN+Y'A");	// 1
        th(  "DZNYA'A");	// 1
        th( "DZNYA'AN");	// 1
        th(  "DZNY'AD");	// 1
        th(  "DZNYA'I");	// 1
        th(   "DZNYAN");	// 1
        th(  "DZNYANA");	// 1
        th( "DZNY'ANA");	// 1
        th( "DZNYANTU");	// 1
        th( "DZNYAPTA");	// 1
        th(   "DZNYAS");	// 1
        th(   "DZNY'I");	// 1
        th(    "DZNYO");	// 1
        th(   "DZNYON");	// 1
        th(   "DZNY'U");	// 1
        th(     "'DZO");	// 1
        th(   "'DZO'I");	// 1
        th(    "DZOKI");	// 1
        th(  "'DZONGS");	// 1
        th(     "DZOS");	// 1
        th(    "'DZOS");	// 1
        th(    "DZRAS");	// 1
        th(     "DZRi");	// 1
        th(    "DZR'i");	// 1
        th(     "DZRI");	// 1
        th(    "DZRUm");	// 1
        th(    "DZSRA");	// 1
        th(     "'DZU");	// 1
        th(     "DZ'U");	// 1
        th(     "DZU4");	// 1
        th("'DZUGSPAR");	// 1
        th(     "DZUM");	// 1
        th(     "DZUN");	// 1
        th(    "DZUNI");	// 1
        th(   "DZU:'O");	// 1
        th(  "DZUshYA");	// 1
        th(    "DZVA'");	// 1
        th(    "DZWAI");	// 1
        th(  "DZWAZHA");	// 1
        th(  "DZYAT'O");	// 1
        th(     "DZYE");	// 1
        th(  "DZYOO'O");	// 1
        th(      "'E2");	// 1
        th(      "'E4");	// 1
        th(      "E'A");	// 1
        th(    "EBPAR");	// 1
        th(    "E'CHI");	// 1
        th(    "ECHOS");	// 1
        th(    "EDANG");	// 1
        th(   "'EDZIN");	// 1
        th( "EEESBYAR");	// 1
        th(     "EGAL");	// 1
        th(    "EGNAS");	// 1
        th(   "E'GO'I");	// 1
        th(      "EGS");	// 1
        th(      "E'I");	// 1
        th(    "EKYEN");	// 1
        th(       "EL");	// 1
        th(     "ELEN");	// 1
        th(   "ELTUNG");	// 1
        th(     "EMDD");	// 1
        th(   "EMNGON");	// 1
        th(      "EMS");	// 1
        th(       "EN");	// 1
        th(      "'EN");	// 1
        th(    "EPA'I");	// 1
        th(   "EPHYIR");	// 1
        th(      "'ER");	// 1
        th(    "ERTAG");	// 1
        th(      "'ES");	// 1
        th(      "ESU");	// 1
        th(     "ETHA");	// 1
        th(   "EYPA'O");	// 1
        th(      "^GA");	// 1
        th(    "''GA'");	// 1
        th(   "''GA''");	// 1
        th(      "GA2");	// 1
        th(      "GA4");	// 1
        th(     "GAAB");	// 1
        th(    "GAANG");	// 1
        th(    "GA'AR");	// 1
        th(     "GAB:");	// 1
        th(    "'GABS");	// 1
        th(   "GADDHI");	// 1
        th( "GADG'A'I");	// 1
        th(  "GADGA'I");	// 1
        th(   "GADGHA");	// 1
        th(    "GADGO");	// 1
        th(    "GADHA");	// 1
        th(  "GADHA'I");	// 1
        th(    "GAD'I");	// 1
        th(    "'GAG'");	// 1
        th(    "'GAGL");	// 1
        th(    "G'A'i");	// 1
        th(  "GA:KHUS");	// 1
        th(  "GA'LANG");	// 1
        th(   "'GA-LO");	// 1
        th(   "GAMBHI");	// 1
        th( "GAM-BHIR");	// 1
        th(    "GAMLi");	// 1
        th(     "GAmS");	// 1
        th(     "G'AN");	// 1
        th(    "GAndA");	// 1
        th(  "GAn-d'A");	// 1
        th(    "GANdA");	// 1
        th(   "GANDA:");	// 1
        th(  "GAN-DHA");	// 1
        th(  "GANDHAE");	// 1
        th( "GANDHA'I");	// 1
        th(  "GANDHAR");	// 1
        th(  "GANDHAS");	// 1
        th( "GANDHE'I");	// 1
        th(  "GAN-DHI");	// 1
        th(  "GANDHOL");	// 1
        th(    "GAndi");	// 1
        th(   "GANd'I");	// 1
        th( "GANd'I'I");	// 1
        th(   "GAndIR");	// 1
        th(    "GANDO");	// 1
        th(   "GANG'A");	// 1
        th(  "GANG'A'");	// 1
        th("GANGA'G'I");	// 1
        th(    "GANGC");	// 1
        th( "GANG-GA'");	// 1
        th( "GANGG'A'");	// 1
        th(  "GANGGA'");	// 1
        th("GANGGA'A'I");	// 1
        th("GANG-G'AI");	// 1
        th( "GANGG'AI");	// 1
        th("GANGGA''I");	// 1
        th( "GANG+GAL");	// 1
        th("GANG+G'Am");	// 1
        th("GANG+G'AM");	// 1
        th( "GANGGARA");	// 1
        th("GANGG'ARA");	// 1
        th("GANGG'GA'I");	// 1
        th( "GANGG''I");	// 1
        th("GANGG'I'I");	// 1
        th(    "GANGH");	// 1
        th(   "GANGHA");	// 1
        th("GANGH'A'I");	// 1
        th( "GANG'-'I");	// 1
        th(  "GANG-LA");	// 1
        th(    "GANGs");	// 1
        th(  "GANGYIN");	// 1
        th("GANHARDHA");	// 1
        th(   "GANJIR");	// 1
        th(   "GAnSHI");	// 1
        th(    "GAntE");	// 1
        th(  "GAntE'I");	// 1
        th(   "GAnthI");	// 1
        th(    "GAntI");	// 1
        th(   "GAnt'I");	// 1
        th(  "GANtI'I");	// 1
        th(   "GANTSA");	// 1
        th(  "GA-NYIS");	// 1
        th(    "GAONG");	// 1
        th(    "GAR'A");	// 1
        th( "GARAKSHA");	// 1
        th(  "GAR-BHA");	// 1
        th( "GAR-B+HA");	// 1
        th( "GARBHA'I");	// 1
        th(    "GARBI");	// 1
        th(    "GARDE");	// 1
        th(   "GARDHA");	// 1
        th(   "GARGAS");	// 1
        th(  "GARGASA");	// 1
        th(    "GARVA");	// 1
        th(  "GARVHA:");	// 1
        th(     "GASA");	// 1
        th(     "GASG");	// 1
        th("GASMKHYEN");	// 1
        th(     "GASN");	// 1
        th(     "GASR");	// 1
        th(  "GASTYA:");	// 1
        th( "GASTYA'I");	// 1
        th(    "GATPA");	// 1
        th( "GATZTS'A");	// 1
        th(  "GATZTSE");	// 1
        th(     "GAUM");	// 1
        th(    "GA'UR");	// 1
        th( "GA-Y'A'I");	// 1
        th(   "GA-YEG");	// 1
        th(  "GA-YEMS");	// 1
        th(  "'GA-YES");	// 1
        th(   "GA-YIN");	// 1
        th(   "GA-YNG");	// 1
        th(    "GAYON");	// 1
        th(    "GAYOS");	// 1
        th(  "GA-YU'I");	// 1
        th(      "GBA");	// 1
        th(     "GBA'");	// 1
        th(    "GBAGS");	// 1
        th(    "GBA'I");	// 1
        th(     "GBAR");	// 1
        th(     "GBAS");	// 1
        th(    "GBCAS");	// 1
        th(    "GBDEN");	// 1
        th(     "GBHA");	// 1
        th(     "GBRA");	// 1
        th(   "GBRJOD");	// 1
        th(   "GBSAGS");	// 1
        th(  "GBSKYED");	// 1
        th(   "GBSTAN");	// 1
        th(  "GBSUNGS");	// 1
        th(    "GBU'I");	// 1
        th(  "GBZHI'I");	// 1
        th(       "GC");	// 1
        th(     "GCA'");	// 1
        th(    "GCADO");	// 1
        th(    "GCE4S");	// 1
        th(     "GCEM");	// 1
        th(    "^GCER");	// 1
        th(    "GCHAB");	// 1
        th(   "GCHAGS");	// 1
        th(    "GCHES");	// 1
        th(    "GCHOD");	// 1
        th(   "GCI'AM");	// 1
        th(    "GCIGG");	// 1
        th(  "GCIGGIS");	// 1
        th(    "GCIGO");	// 1
        th(   "GCIG'O");	// 1
        th(   "'GCIGS");	// 1
        th(    "GCIHG");	// 1
        th(     "GCIT");	// 1
        th(    "GCI'U");	// 1
        th(     "GCOB");	// 1
        th(    "GCOGS");	// 1
        th(     "GCUB");	// 1
        th(    "GCU'I");	// 1
        th(      "GDA");	// 1
        th(    "GDABS");	// 1
        th(     "GDAt");	// 1
        th(    "GDEBS");	// 1
        th(    "GDEMS");	// 1
        th(     "GDER");	// 1
        th(    "GDH'A");	// 1
        th(  "GDH'A'I");	// 1
        th(   "GDHI'O");	// 1
        th(     "GDIN");	// 1
        th(   "^GDING");	// 1
        th(  "GDINGBA");	// 1
        th(    "GDMAS");	// 1
        th(    "G'DOD");	// 1
        th(     "GDOG");	// 1
        th(     "GDRA");	// 1
        th(    "GDREN");	// 1
        th(    "GDRIM");	// 1
        th(    "GDRIN");	// 1
        th(  "GDRONGS");	// 1
        th(    "GDRUG");	// 1
        th(    "GDTIN");	// 1
        th(     "GDU'");	// 1
        th(     "GDUR");	// 1
        th(    "GDZAD");	// 1
        th(    "GDZAN");	// 1
        th(      "GE:");	// 1
        th(      "GE2");	// 1
        th(      "GE4");	// 1
        th(      "GEB");	// 1
        th(     "GEES");	// 1
        th(   "GENDRA");	// 1
        th(    "GENGS");	// 1
        th(     "'GER");	// 1
        th(    "'GERL");	// 1
        th(     "'GES");	// 1
        th(    "GESGS");	// 1
        th(     "GEUB");	// 1
        th(   "GE-YEL");	// 1
        th(      "GGA");	// 1
        th(     "GGAN");	// 1
        th(     "GGAS");	// 1
        th(    "GGCIG");	// 1
        th(     "GGIG");	// 1
        th(    "GGNAS");	// 1
        th(    "G'GOR");	// 1
        th(    "GGRUB");	// 1
        th(    "GGSER");	// 1
        th(     "GGSO");	// 1
        th(     "GGYI");	// 1
        th(    "GGYUR");	// 1
        th(   "GGZHAN");	// 1
        th(     "GH'A");	// 1
        th(    "GHABS");	// 1
        th(   "GHADHA");	// 1
        th(     "GHAm");	// 1
        th(   "GHANDE");	// 1
        th(  "GHANDHA");	// 1
        th("GHANDHASA");	// 1
        th(   "GHANTA");	// 1
        th(  "GHANY'O");	// 1
        th(   "GHASMA");	// 1
        th(     "GHEG");	// 1
        th(    "GHEGS");	// 1
        th(    "GHE'U");	// 1
        th( "GHIKsh'A");	// 1
        th(    "GHI'O");	// 1
        th(   "GHiSLA");	// 1
        th(    "GHNAm");	// 1
        th(    "GHndI");	// 1
        th(    "GHNYA");	// 1
        th(     "GHO:");	// 1
        th(     "GHOB");	// 1
        th(    "GHO'I");	// 1
        th(    "GHONG");	// 1
        th(     "GHOO");	// 1
        th(   "GHOshA");	// 1
        th(  "GHOsH'A");	// 1
        th(    "GHRAm");	// 1
        th(  "GHRBHAS");	// 1
        th(     "GHRi");	// 1
        th(    "GHR'I");	// 1
        th(    "GHRIm");	// 1
        th(     "GHUN");	// 1
        th(  "GHURBHA");	// 1
        th(     "GHUt");	// 1
        th(   "GHUt'I");	// 1
        th(   "GHUt'O");	// 1
        th(    "GHUtR");	// 1
        th(    "GHYAn");	// 1
        th(  "GHYAn'O");	// 1
        th(     "G''I");	// 1
        th(     "G'I:");	// 1
        th(      "GI0");	// 1
        th(      "GI2");	// 1
        th(      "GI4");	// 1
        th(    "GICUB");	// 1
        th(      "GID");	// 1
        th(     "GIDG");	// 1
        th(    "GI'DU");	// 1
        th(      "GIE");	// 1
        th(     "'GIG");	// 1
        th(     "GIGS");	// 1
        th(      "GII");	// 1
        th(    "'GI'I");	// 1
        th(     "GIIS");	// 1
        th(    "GIKHA");	// 1
        th(  "GIMCHOG");	// 1
        th(    "GIMLA");	// 1
        th(   "GInd'A");	// 1
        th(    "'GING");	// 1
        th(   "'GINGS");	// 1
        th(   "GINPRA");	// 1
        th(  "GIPHYIR");	// 1
        th(   "G'IRBA");	// 1
        th(      "GiS");	// 1
        th(    "GI'TA");	// 1
        th(    "'GI'U");	// 1
        th(     "GKA'");	// 1
        th(     "GKAG");	// 1
        th(   "GKHYAD");	// 1
        th(   "GKLING");	// 1
        th(     "GKLU");	// 1
        th(     "GKUN");	// 1
        th(    "GKUNG");	// 1
        th(    "GKYIS");	// 1
        th(     "GKYU");	// 1
        th(     "GL'A");	// 1
        th(  "GLE'ANG");	// 1
        th( "GLEGSBAM");	// 1
        th(     "GLEL");	// 1
        th(    "GLGAS");	// 1
        th(  "GLINGPA");	// 1
        th(     "GLIR");	// 1
        th(   "GLIRTI");	// 1
        th(    "GLSGS");	// 1
        th(    "GLTAN");	// 1
        th(    "GLTAR");	// 1
        th(   "GLTUNG");	// 1
        th(     "GLUB");	// 1
        th(     "GLUG");	// 1
        th(    "GLUGS");	// 1
        th(     "GLUM");	// 1
        th(     "GLUN");	// 1
        th(    "GLU'O");	// 1
        th(       "Gm");	// 1
        th(     "GMAG");	// 1
        th(     "GMAN");	// 1
        th(     "GMED");	// 1
        th(      "GMO");	// 1
        th(    "GMOGS");	// 1
        th(     "GMON");	// 1
        th(   "GMTSAN");	// 1
        th(      "GMU");	// 1
        th(    "GMYIS");	// 1
        th(       "GN");	// 1
        th(   "GNA'AM");	// 1
        th(  "GNA'ANG");	// 1
        th(    "GNAGN");	// 1
        th(     "GNAm");	// 1
        th( "GNAMKH'U");	// 1
        th(   "GNANGK");	// 1
        th(    "GNASO");	// 1
        th(    "GNBAS");	// 1
        th(    "GNDHA");	// 1
        th(     "GNEN");	// 1
        th(    "GNGA'");	// 1
        th(    "GNGAL");	// 1
        th(    "GNGAN");	// 1
        th(   "GNGG'A");	// 1
        th( "GNG+GA'I");	// 1
        th(    "GNGON");	// 1
        th(     "GNIS");	// 1
        th(    "GNIYS");	// 1
        th(     "GnnA");	// 1
        th(      "GnO");	// 1
        th(    "^GNOD");	// 1
        th(     "GNOR");	// 1
        th(     "GNRI");	// 1
        th(      "GNS");	// 1
        th(      "GNU");	// 1
        th(    "GNUNG");	// 1
        th(   "GNYA'A");	// 1
        th(   "GNYAIS");	// 1
        th(   "GNYDZI");	// 1
        th(    "GNYEB");	// 1
        th(    "GNYIA");	// 1
        th(  "GNYIDZA");	// 1
        th( "GNYIDZAR");	// 1
        th(   "GNYI'O");	// 1
        th(   ":GNYIS");	// 1
        th(   "'GNYIS");	// 1
        th("GNYISSNYOMS");	// 1
        th( "GNYISYIN");	// 1
        th(    "GNYJI");	// 1
        th(    "GNYOG");	// 1
        th(    "GNYON");	// 1
        th(   "GNYO'O");	// 1
        th(    "GNYOR");	// 1
        th(     "GNYU");	// 1
        th(   "GNYUIS");	// 1
        th(  "GNYUNGS");	// 1
        th(      "G'O");	// 1
        th(      "GO4");	// 1
        th(   "'GO'AM");	// 1
        th(     "GOBS");	// 1
        th(    "'GOBS");	// 1
        th(     "GODN");	// 1
        th(     "GOHA");	// 1
        th(      "GOm");	// 1
        th(     "'GOm");	// 1
        th(  "GOMNGON");	// 1
        th(   "^GONGS");	// 1
        th(    "'GO'O");	// 1
        th(   "GOOANG");	// 1
        th(    "GOO'I");	// 1
        th(   "GOOndI");	// 1
        th(     "GOOT");	// 1
        th(     "^GOR");	// 1
        th(   "GORSHA");	// 1
        th(   "GORSHI");	// 1
        th(    "GOshU");	// 1
        th(     "GPAD");	// 1
        th(    "GPA'I");	// 1
        th(     "GPAL");	// 1
        th(    "GPA'O");	// 1
        th(    "GPHOD");	// 1
        th(    "GPHUN");	// 1
        th(   "GPHYIN");	// 1
        th(     "GPOM");	// 1
        th(      "GPU");	// 1
        th(      "'GR");	// 1
        th(    "GR8UB");	// 1
        th(     "GRA:");	// 1
        th(  "GRA'ANG");	// 1
        th(    "'GRAB");	// 1
        th(   "GRAGGS");	// 1
        th(   "GRAGS0");	// 1
        th(    "GRAHA");	// 1
        th(   "GR'A'i");	// 1
        th(    "'GRAm");	// 1
        th( "'GRAMGYI");	// 1
        th(  "^GRANGS");	// 1
        th( "GRANGS34");	// 1
        th( "GRANGS41");	// 1
        th( "GRANGS42");	// 1
        th(  "GRANGS9");	// 1
        th( "GRANG-SU");	// 1
        th( "GRANTH'A");	// 1
        th(  "GRANTHI");	// 1
        th(   "GRA'OR");	// 1
        th(   "GRATHI");	// 1
        th(    "GRAVA");	// 1
        th(    "GRAVR");	// 1
        th(     "GRBA");	// 1
        th(   "GRBH'A");	// 1
        th(     "GRBU");	// 1
        th(    "GRBVA");	// 1
        th(   "GRDEGS");	// 1
        th(     "GRDO");	// 1
        th(   "GRDUNG");	// 1
        th(    "GRDZU");	// 1
        th(   "GRDZUN");	// 1
        th(    "'GREB");	// 1
        th(     "GREG");	// 1
        th(    "'GREK");	// 1
        th(   "'GREKL");	// 1
        th(   "''GREL");	// 1
        th(     "GREM");	// 1
        th(    "'GREO");	// 1
        th(  "GRE'U'O");	// 1
        th(   "GRE'UR");	// 1
        th(    "GRGAS");	// 1
        th(     "GRGS");	// 1
        th(     "GR'I");	// 1
        th(    "GR'I:");	// 1
        th(   "GRiDHI");	// 1
        th(     "GRIG");	// 1
        th(    "GRiHI");	// 1
        th(    "GRIHn");	// 1
        th(   "GRIHNE");	// 1
        th(    "GRi'I");	// 1
        th(   "'GRI'I");	// 1
        th(  "'GRINGS");	// 1
        th(    "GRJOD");	// 1
        th(   "GRNAGS");	// 1
        th(    "GRNAL");	// 1
        th(   "GRNIsh");	// 1
        th(   "GRNNGS");	// 1
        th(    "^'GRO");	// 1
        th(    "''GRO");	// 1
        th(      "GRO");	// 1
        th(  "'GRO'Am");	// 1
        th(   "GRO'AM");	// 1
        th(   "GRODHA");	// 1
        th(   "'GROLA");	// 1
        th(   "GRONYA");	// 1
        th(     "GRSA");	// 1
        th(    "GRTAG");	// 1
        th(    "GRTEN");	// 1
        th(   "GRTI'I");	// 1
        th(   "GRTOGS");	// 1
        th(   "GRTZER");	// 1
        th(   "GRTZOD");	// 1
        th(   "GRTZON");	// 1
        th(   "GRTZOR");	// 1
        th(   "GRTZUB");	// 1
        th(     "GRU'");	// 1
        th(    "GRUB0");	// 1
        th("GRUBMTHA'");	// 1
        th(   "'GRUGS");	// 1
        th(   "'GRU'I");	// 1
        th( "GRUNYTZA");	// 1
        th(    "GRU'O");	// 1
        th(     "GRUP");	// 1
        th(    "GRUtA");	// 1
        th(      "GRV");	// 1
        th(    "GRVA'");	// 1
        th( "GRVA'ANG");	// 1
        th(    "GRVAG");	// 1
        th(   "GRVAGS");	// 1
        th(    "GRVAL");	// 1
        th(    "GRV'I");	// 1
        th(     "GRVS");	// 1
        th(    "GRW'A");	// 1
        th(   "GR-WAR");	// 1
        th(    "GRYAL");	// 1
        th(    "GSAAL");	// 1
        th(    "GSAGS");	// 1
        th(    "GSA'I");	// 1
        th(    "GSAMS");	// 1
        th(    "GSDAL");	// 1
        th(   "GSDONG");	// 1
        th(     "GSDU");	// 1
        th(    "GSDUS");	// 1
        th(   "GSEHGS");	// 1
        th(     "GSEM");	// 1
        th(   "GSENGS");	// 1
        th(    "^GSER");	// 1
        th(    "GSERD");	// 1
        th(   "GSGROG");	// 1
        th(   "GSGRUB");	// 1
        th(    "GSHAd");	// 1
        th(   "GSHA'I");	// 1
        th(   "GSHA'O");	// 1
        th(    "'GSHE");	// 1
        th(   "GSHEBS");	// 1
        th(    "GSHEt");	// 1
        th(    "GSHIM");	// 1
        th(    "GSHUN");	// 1
        th(  "GSHUNGS");	// 1
        th(   "GSKABS");	// 1
        th(   "GSKYES");	// 1
        th(    "GSLAB");	// 1
        th(    "GSNAD");	// 1
        th(    "GSNAG");	// 1
        th(     "GSNG");	// 1
        th(   "GSNGAL");	// 1
        th(   "GSNYIS");	// 1
        th(     "^GSO");	// 1
        th(     "GSOd");	// 1
        th(    "^GSOL");	// 1
        th(    "GSPEL");	// 1
        th(     "GSRA");	// 1
        th(   "GSRANG");	// 1
        th(    "GSREL");	// 1
        th(   "GSRENG");	// 1
        th(    "GSROD");	// 1
        th(   "GSTZAL");	// 1
        th(     "GSUB");	// 1
        th(     "GSUE");	// 1
        th(    "GSUGN");	// 1
        th(    "^GSUM");	// 1
        th( "GSUMGYIS");	// 1
        th(    "GSUMM");	// 1
        th(    "GSUMO");	// 1
        th(   "^GSUNG");	// 1
        th(  "-:GSUNG");	// 1
        th( "GSUNGSPA");	// 1
        th(   "GSURAS");	// 1
        th(  "GS-YOGS");	// 1
        th(   "GTA'AM");	// 1
        th(    "GTAD2");	// 1
        th(    "GTADO");	// 1
        th(     "GTAL");	// 1
        th(     "GTAS");	// 1
        th(    "GTASN");	// 1
        th(     "GTED");	// 1
        th(     "GTEL");	// 1
        th(     "GTES");	// 1
        th(     "GTHA");	// 1
        th(     "GTHE");	// 1
        th(   "GTHONG");	// 1
        th(    "GTHOR");	// 1
        th(     "GTHU");	// 1
        th(   "GTHUNG");	// 1
        th(     "GTID");	// 1
        th(  "GTIMMUG");	// 1
        th(     "GTIN");	// 1
        th(   "GTINGS");	// 1
        th(     "GTIR");	// 1
        th(     "GTmS");	// 1
        th(   "GTO-dA");	// 1
        th(    "GTO'I");	// 1
        th(     "GTOM");	// 1
        th(     "GTOO");	// 1
        th(    "GTORS");	// 1
        th(    "GTRAN");	// 1
        th(    "GTSIG");	// 1
        th(   "GTSOGS");	// 1
        th(     "GTSU");	// 1
        th(   "GTSUGS");	// 1
        th(    "GTSUM");	// 1
        th(   "GTSUNG");	// 1
        th(     "GTUD");	// 1
        th(    "GTU'O");	// 1
        th(  "GTZANGA");	// 1
        th(    "GTZAS");	// 1
        th(    "GTZEB");	// 1
        th(   "GTZE'O");	// 1
        th(   "GTZHAN");	// 1
        th(    "GTZId");	// 1
        th(    "GTZIM");	// 1
        th(    "GTZIR");	// 1
        th(  "GTZO'AM");	// 1
        th(   "GTZOBS");	// 1
        th(    "GTZOM");	// 1
        th(    "GTZON");	// 1
        th(    "GTZOO");	// 1
        th(    "GTZUM");	// 1
        th(      "GU4");	// 1
        th(    "GU'Am");	// 1
        th(    "GUBHA");	// 1
        th(    "GUBHU");	// 1
        th(     "'GUD");	// 1
        th(    "GUDHA");	// 1
        th(      "GUH");	// 1
        th(   "GUH+YA");	// 1
        th(    "GULMA");	// 1
        th(      "GUn");	// 1
        th(     "GUnA");	// 1
        th(   "GUNDHA");	// 1
        th(    "'GUNG");	// 1
        th(   "GUNGAS");	// 1
        th(    "GUn'O");	// 1
        th(    "GUNTA");	// 1
        th(   "GUPT'A");	// 1
        th(    "GURTA");	// 1
        th(  "GUSHR'I");	// 1
        th(    "GUTPA");	// 1
        th(   "'GUYUR");	// 1
        th(    "GVA'I");	// 1
        th(     "GVAR");	// 1
        th(     "GVEL");	// 1
        th(     "GVOD");	// 1
        th(     "GVON");	// 1
        th(     "GVRA");	// 1
        th(      "'GY");	// 1
        th(      "G-Y");	// 1
        th(    "GY2IS");	// 1
        th(     "GY8I");	// 1
        th(    "G-Y'A");	// 1
        th(     "GY'A");	// 1
        th(     "GYAG");	// 1
        th(    "'GYAL");	// 1
        th(     "GYAm");	// 1
        th(  "'G-YANG");	// 1
        th(   "G-YA'O");	// 1
        th(   "GYA-OS");	// 1
        th(      "GYD");	// 1
        th(   "G-YEBS");	// 1
        th(    "'GYEG");	// 1
        th(   "'GYEGS");	// 1
        th(   "G-YEMS");	// 1
        th(    "'GYEN");	// 1
        th(    "GYE'O");	// 1
        th(    "'GYER");	// 1
        th(      "GYi");	// 1
        th(     "^GYI");	// 1
        th(    "GYIAS");	// 1
        th(    "GYIBA");	// 1
        th(   "GYIDON");	// 1
        th(     "GYIG");	// 1
        th(   "G-YING");	// 1
        th(   "'GYI'O");	// 1
        th( "GYIPHYIR");	// 1
        th(    "'GYIR");	// 1
        th(     "GYIs");	// 1
        th(   "GYIS'I");	// 1
        th(    "GYISS");	// 1
        th(  "GYItA'I");	// 1
        th(    "GYITD");	// 1
        th(     "'GYO");	// 1
        th(     "GY-O");	// 1
        th(   "G-YO9N");	// 1
        th(   "G-YOBS");	// 1
        th(   "G+YOGS");	// 1
        th(    "G-YOM");	// 1
        th(   "'GYO'O");	// 1
        th(    "GYPOS");	// 1
        th(      "GYR");	// 1
        th(    "'GYRR");	// 1
        th(     "GYSI");	// 1
        th(   "GYU'AM");	// 1
        th(    "GYUAR");	// 1
        th(     "GYUI");	// 1
        th(  "G-YUNGS");	// 1
        th(   "''GYUR");	// 1
        th(   "'GYUR'");	// 1
        th(   "GYURAD");	// 1
        th( "GYURPA'I");	// 1
        th( "GYURPA'O");	// 1
        th(   "'GYURR");	// 1
        th(   "'GYURS");	// 1
        th("'GYURTHAMS");	// 1
        th("'GYURZHING");	// 1
        th(      "GZA");	// 1
        th(    "GZAHN");	// 1
        th(     "GZAL");	// 1
        th(    "GZEHI");	// 1
        th(     "GZEN");	// 1
        th(    "GZHAA");	// 1
        th(    "GZHAB");	// 1
        th(   "GZHAGI");	// 1
        th(  "GZHAM'I");	// 1
        th(   "^GZHAN");	// 1
        th(   "'GZHAN");	// 1
        th(   "GZHAND");	// 1
        th(   "GZHANJ");	// 1
        th(   "GZHANO");	// 1
        th(   "GZHEE'");	// 1
        th(   "GZHENG");	// 1
        th(    "GZHGS");	// 1
        th(     "GZHi");	// 1
        th(    "GZHI'");	// 1
        th(  "GZHIMDO");	// 1
        th(   "GZHIMS");	// 1
        th(  "GZHINGS");	// 1
        th(   "GZHINI");	// 1
        th(   "GZHISU");	// 1
        th(   "GZHO'I");	// 1
        th( "GZHON-NU");	// 1
        th(   "GZHUBS");	// 1
        th(    "GZHUL");	// 1
        th(    "GZHUM");	// 1
        th(  "GZHUNMG");	// 1
        th(     "GZIB");	// 1
        th(   "GZIGS2");	// 1
        th(    "GZLAR");	// 1
        th(    "GZLOG");	// 1
        th(     "GZOM");	// 1
        th(     "GZUB");	// 1
        th(   "GZUG1S");	// 1
        th(    "GZUGC");	// 1
        th(   "GZUGGS");	// 1
        th(   "GZUGNG");	// 1
        th("GZUGSSDUG");	// 1
        th(  "GZUG-SU");	// 1
        th(    "GZUMS");	// 1
        th(   "GZUNGA");	// 1
        th(    "GZUNR");	// 1
        th(    "GZU'O");	// 1
        th(     "GZVA");	// 1
        th(    "GZZAN");	// 1
        th(       "hA");	// 1
        th(      "h'A");	// 1
        th(      "HA'");	// 1
        th(      "HA4");	// 1
        th(     "HA'A");	// 1
        th("HA:A'A:DZHI:H'Um");	// 1
        th(   "H'A'AM");	// 1
        th(     "'HAB");	// 1
        th(     "H'Am");	// 1
        th(     "'HAM");	// 1
        th(     "HAN:");	// 1
        th(   "H'AN'O");	// 1
        th(    "HANTA");	// 1
        th(  "HANYTZE");	// 1
        th(     "HARA");	// 1
        th(    "HARBA");	// 1
        th("H'ARshItA'O");	// 1
        th(  "H'ARTHA");	// 1
        th(    "HARTI");	// 1
        th(      "hAS");	// 1
        th(     "H'AS");	// 1
        th(   "HAST'A");	// 1
        th(    "HASTI");	// 1
        th(   "HASTIS");	// 1
        th(   "HASTNI");	// 1
        th(    "HA'Um");	// 1
        th(     "HAVA");	// 1
        th(   "HAVAGS");	// 1
        th(    "HA-YA");	// 1
        th(     "HAYG");	// 1
        th(    "HBSAM");	// 1
        th(     "HCOS");	// 1
        th(    "H'DMA");	// 1
        th(    "HDOGS");	// 1
        th(    "H'DRA");	// 1
        th(     "HDUL");	// 1
        th(      "HE2");	// 1
        th(      "HE4");	// 1
        th(     "HEBS");	// 1
        th(    "HEE'O");	// 1
        th(   "HEHUNG");	// 1
        th(    "HERE-");	// 1
        th(     "HE'U");	// 1
        th(  "HGSUNGS");	// 1
        th(       "Hi");	// 1
        th(   "HI'I'I");	// 1
        th(     "HIKA");	// 1
        th(    "HIKRA");	// 1
        th(     "HIKU");	// 1
        th(      "HIL");	// 1
        th(      "HIM");	// 1
        th(      "HIR");	// 1
        th(     "H'IS");	// 1
        th("HKshMLWRLAm");	// 1
        th("HKshMLWRYAm");	// 1
        th("H+Ksh+M+L+W+R+YAm");	// 1
        th(      "HLA");	// 1
        th(     "HL'A");	// 1
        th(    "HLAGS");	// 1
        th(    "HLDAN");	// 1
        th(      "HMI");	// 1
        th(      "HnA");	// 1
        th(     "HNAR");	// 1
        th(     "hNAS");	// 1
        th(     "HNGU");	// 1
        th(     "HNTE");	// 1
        th(     "H+NU");	// 1
        th(    "HNU'I");	// 1
        th(    "HNYID");	// 1
        th(      "H'O");	// 1
        th(      "HO3");	// 1
        th(  "HODZAS:");	// 1
        th(      "HOG");	// 1
        th( "HO:GNYIS");	// 1
        th("HO:HO:HO:");	// 1
        th(      "HOm");	// 1
        th(      "HOO");	// 1
        th(    "HOO'I");	// 1
        th(   "HO:PRA");	// 1
        th(     "'HOR");	// 1
        th("HOTR'mDZU");	// 1
        th(    "HO:YE");	// 1
        th(  "HO:ZHES");	// 1
        th(      "HPA");	// 1
        th(   "HPHANG");	// 1
        th(    "HPYIS");	// 1
        th(   "HPYOGS");	// 1
        th(    "HRAGS");	// 1
        th(     "HRAm");	// 1
        th(    "HR'Am");	// 1
        th(   "HRA'm:");	// 1
        th(      "HRE");	// 1
        th(    "H'RI:");	// 1
        th(     "HRiD");	// 1
        th(  "HR'I:'I");	// 1
        th(     "HRiL");	// 1
        th( "HR'I:LAS");	// 1
        th(     "HRIm");	// 1
        th(     "HRIM");	// 1
        th(    "HRIMS");	// 1
        th( "HR'I:PRA");	// 1
        th(   "HRishI");	// 1
        th("HR'I:shRtI:");	// 1
        th("HR'I:shRtR'I:");	// 1
        th("HR'I:shtI");	// 1
        th("HR'I:SNYIGS");	// 1
        th("HR'I:SOGS");	// 1
        th( "HRI:SOGS");	// 1
        th( "HR'I:YIG");	// 1
        th(     "HR'm");	// 1
        th(   "HRNAMS");	// 1
        th(     "HROD");	// 1
        th(     "HROG");	// 1
        th(      "HRU");	// 1
        th(   "HRVAGS");	// 1
        th(    "HRVAm");	// 1
        th(    "HSEMS");	// 1
        th(    "HSHES");	// 1
        th(    "hSOGS");	// 1
        th(   "hSPANG");	// 1
        th(   "HSTONG");	// 1
        th(      "HTA");	// 1
        th(      "HTI");	// 1
        th(     "HU'M");	// 1
        th(    "H'UNG");	// 1
        th(    "HU'NG");	// 1
        th(    "H'Unm");	// 1
        th(    "H'U'U");	// 1
        th(    "HU'Um");	// 1
        th(     "HVE:");	// 1
        th(      "HVI");	// 1
        th(     "HVOB");	// 1
        th(    "HVO'I");	// 1
        th(     "HVOR");	// 1
        th(    "H+YA:");	// 1
        th(     "HYAG");	// 1
        th(     "HYAL");	// 1
        th(     "HYAm");	// 1
        th(    "HYANG");	// 1
        th(     "HYAS");	// 1
        th(     "HYES");	// 1
        th(     "HYIN");	// 1
        th(    "HYSTA");	// 1
        th(    "HYUNG");	// 1
        th(     "HZES");	// 1
        th(      "^'I");	// 1
        th(      "'I2");	// 1
        th(      "'I4");	// 1
        th(       "IA");	// 1
        th(  "iATHi'I");	// 1
        th(  "iATHI'I");	// 1
        th(    "IBCOM");	// 1
        th(     "IBUR");	// 1
        th(     "IBYA");	// 1
        th(   "IBZHIN");	// 1
        th(    "ICHEN");	// 1
        th(     "IDGE");	// 1
        th(   "IDMIGS");	// 1
        th(      "IDO");	// 1
        th(      "IDU");	// 1
        th(      "'IG");	// 1
        th(    "I'GOD");	// 1
        th(       "II");	// 1
        th(   "III-01");	// 1
        th(   "III-03");	// 1
        th(   "III-04");	// 1
        th(   "III-06");	// 1
        th(   "III-08");	// 1
        th(   "III-09");	// 1
        th(   "III-10");	// 1
        th(  "III-100");	// 1
        th(  "III-101");	// 1
        th(  "III-103");	// 1
        th(  "III-104");	// 1
        th(  "III-105");	// 1
        th(  "III-106");	// 1
        th(  "III-107");	// 1
        th(  "III-108");	// 1
        th(  "III-109");	// 1
        th(  "III-111");	// 1
        th(  "III-113");	// 1
        th(  "III-114");	// 1
        th(  "III-115");	// 1
        th(  "III-116");	// 1
        th(  "III-118");	// 1
        th(  "III-119");	// 1
        th(   "III-12");	// 1
    }


    /** Tests some of the tsheg bars that appear only once (likely
     *  these are typos or non-native). */
    public void test1to1b() {
        th(  "III-120");	// 1
        th(  "III-122");	// 1
        th(  "III-123");	// 1
        th(  "III-124");	// 1
        th(  "III-125");	// 1
        th(  "III-126");	// 1
        th(  "III-127");	// 1
        th(  "III-128");	// 1
        th(   "III-13");	// 1
        th(  "III-130");	// 1
        th(  "III-131");	// 1
        th(  "III-133");	// 1
        th(  "III-134");	// 1
        th(  "III-135");	// 1
        th(  "III-136");	// 1
        th(  "III-137");	// 1
        th(  "III-138");	// 1
        th(  "III-139");	// 1
        th(  "III-140");	// 1
        th(  "III-141");	// 1
        th(  "III-142");	// 1
        th(  "III-143");	// 1
        th(  "III-144");	// 1
        th(  "III-146");	// 1
        th(  "III-147");	// 1
        th(  "III-148");	// 1
        th(  "III-149");	// 1
        th(   "III-15");	// 1
        th(  "III-150");	// 1
        th(  "III-152");	// 1
        th(  "III-153");	// 1
        th(  "III-154");	// 1
        th(  "III-155");	// 1
        th(  "III-156");	// 1
        th(  "III-157");	// 1
        th(  "III-159");	// 1
        th(   "III-16");	// 1
        th(  "III-160");	// 1
        th(  "III-161");	// 1
        th(  "III-163");	// 1
        th(  "III-164");	// 1
        th(  "III-165");	// 1
        th(  "III-166");	// 1
        th(  "III-168");	// 1
        th(  "III-169");	// 1
        th(   "III-17");	// 1
        th(  "III-170");	// 1
        th(  "III-171");	// 1
        th(  "III-172");	// 1
        th(  "III-173");	// 1
        th(  "III-174");	// 1
        th(  "III-175");	// 1
        th(  "III-176");	// 1
        th(  "III-177");	// 1
        th(  "III-178");	// 1
        th(  "III-179");	// 1
        th(   "III-18");	// 1
        th(  "III-181");	// 1
        th(  "III-182");	// 1
        th(  "III-183");	// 1
        th(  "III-184");	// 1
        th(  "III-186");	// 1
        th(  "III-187");	// 1
        th(  "III-188");	// 1
        th(  "III-189");	// 1
        th(   "III-19");	// 1
        th(  "III-190");	// 1
        th(  "III-191");	// 1
        th(  "III-192");	// 1
        th(  "III-193");	// 1
        th(  "III-194");	// 1
        th(  "III-195");	// 1
        th(  "III-196");	// 1
        th(  "III-197");	// 1
        th(  "III-198");	// 1
        th(  "III-199");	// 1
        th(  "III-200");	// 1
        th(  "III-201");	// 1
        th(  "III-202");	// 1
        th(  "III-203");	// 1
        th(  "III-205");	// 1
        th(  "III-206");	// 1
        th(  "III-207");	// 1
        th(  "III-208");	// 1
        th(  "III-209");	// 1
        th(   "III-21");	// 1
        th(  "III-210");	// 1
        th(  "III-212");	// 1
        th(  "III-213");	// 1
        th(  "III-214");	// 1
        th(  "III-215");	// 1
        th(  "III-217");	// 1
        th(  "III-218");	// 1
        th(  "III-219");	// 1
        th(   "III-22");	// 1
        th(  "III-220");	// 1
        th(  "III-221");	// 1
        th(  "III-222");	// 1
        th(  "III-223");	// 1
        th(  "III-225");	// 1
        th( "III-225B");	// 1
        th(  "III-226");	// 1
        th(  "III-227");	// 1
        th(  "III-228");	// 1
        th(  "III-229");	// 1
        th(   "III-23");	// 1
        th(  "III-230");	// 1
        th(  "III-231");	// 1
        th(  "III-233");	// 1
        th(  "III-234");	// 1
        th(  "III-235");	// 1
        th(  "III-236");	// 1
        th(  "III-237");	// 1
        th(  "III-238");	// 1
        th(   "III-24");	// 1
        th(  "III-240");	// 1
        th(  "III-241");	// 1
        th(  "III-243");	// 1
        th(  "III-245");	// 1
        th(  "III-246");	// 1
        th(  "III-247");	// 1
        th(   "III-25");	// 1
        th(  "III-251");	// 1
        th(   "III-26");	// 1
        th(  "III-261");	// 1
        th(  "III-279");	// 1
        th(   "III-28");	// 1
        th(  "III-283");	// 1
        th(  "III-289");	// 1
        th(   "III-29");	// 1
        th(  "III-291");	// 1
        th(  "III-298");	// 1
        th(   "III-30");	// 1
        th(  "III-302");	// 1
        th(  "III-304");	// 1
        th(  "III-309");	// 1
        th(   "III-31");	// 1
        th(  "III-313");	// 1
        th(   "III-33");	// 1
        th(  "III-330");	// 1
        th(  "III-334");	// 1
        th(  "III-337");	// 1
        th(   "III-34");	// 1
        th(  "III-340");	// 1
        th(  "III-343");	// 1
        th(  "III-348");	// 1
        th(   "III-35");	// 1
        th(  "III-355");	// 1
        th(  "III-357");	// 1
        th(   "III-36");	// 1
        th(  "III-361");	// 1
        th(   "III-37");	// 1
        th(  "III-375");	// 1
        th(  "III-379");	// 1
        th(   "III-38");	// 1
        th(  "III-385");	// 1
        th(   "III-39");	// 1
        th(  "III-399");	// 1
        th(   "III-40");	// 1
        th(  "III-403");	// 1
        th(   "III-41");	// 1
        th(  "III-410");	// 1
        th(  "III-419");	// 1
        th(   "III-42");	// 1
        th(  "III-432");	// 1
        th(  "III-434");	// 1
        th(   "III-44");	// 1
        th(   "III-45");	// 1
        th(   "III-46");	// 1
        th(   "III-47");	// 1
        th(   "III-48");	// 1
        th(   "III-49");	// 1
        th(   "III-50");	// 1
        th(   "III-51");	// 1
        th(   "III-52");	// 1
        th(   "III-53");	// 1
        th(   "III-54");	// 1
        th(   "III-55");	// 1
        th(   "III-56");	// 1
        th(   "III-57");	// 1
        th(   "III-59");	// 1
        th(   "III-60");	// 1
        th(   "III-61");	// 1
        th(   "III-62");	// 1
        th(   "III-63");	// 1
        th(   "III-64");	// 1
        th(   "III-67");	// 1
        th(   "III-68");	// 1
        th(   "III-70");	// 1
        th(   "III-71");	// 1
        th(   "III-72");	// 1
        th(   "III-74");	// 1
        th(   "III-75");	// 1
        th(   "III-76");	// 1
        th(   "III-78");	// 1
        th(   "III-79");	// 1
        th(   "III-81");	// 1
        th(   "III-83");	// 1
        th(   "III-84");	// 1
        th(   "III-85");	// 1
        th(   "III-87");	// 1
        th(   "III-88");	// 1
        th(   "III-89");	// 1
        th(   "III-90");	// 1
        th(   "III-91");	// 1
        th(   "III-94");	// 1
        th(   "III-96");	// 1
        th(   "III-97");	// 1
        th(   "III-98");	// 1
        th(   "III-99");	// 1
        th(      "IIN");	// 1
        th(    "IKYIS");	// 1
        th(    "ILTAR");	// 1
        th(      "ING");	// 1
        th(      "INP");	// 1
        th(     "'I'O");	// 1
        th(      "IPA");	// 1
        th(  "IPA'ANG");	// 1
        th(    "'IPRU");	// 1
        th(    "IRUNG");	// 1
        th(       "is");	// 1
        th(  "IS'DREN");	// 1
        th(   "ISKABS");	// 1
        th(   "ISNANG");	// 1
        th(      "ISO");	// 1
        th(      "ITE");	// 1
        th(      "IV-");	// 1
        th(    "IV-01");	// 1
        th(    "IV-02");	// 1
        th(    "IV-04");	// 1
        th(    "IV-05");	// 1
        th(    "IV-06");	// 1
        th(    "IV-08");	// 1
        th(    "IV-09");	// 1
        th(   "IV-100");	// 1
        th(   "IV-101");	// 1
        th(   "IV-102");	// 1
        th(   "IV-103");	// 1
        th(   "IV-104");	// 1
        th(   "IV-106");	// 1
        th(   "IV-107");	// 1
        th(   "IV-108");	// 1
        th(   "IV-109");	// 1
        th(   "IV-110");	// 1
        th(   "IV-111");	// 1
        th(   "IV-112");	// 1
        th(   "IV-113");	// 1
        th(   "IV-114");	// 1
        th(   "IV-115");	// 1
        th(   "IV-116");	// 1
        th(   "IV-117");	// 1
        th(    "IV-12");	// 1
        th(   "IV-120");	// 1
        th(   "IV-122");	// 1
        th(   "IV-123");	// 1
        th(   "IV-125");	// 1
        th(   "IV-126");	// 1
        th(   "IV-128");	// 1
        th(   "IV-129");	// 1
        th(    "IV-13");	// 1
        th(   "IV-130");	// 1
        th(   "IV-131");	// 1
        th(   "IV-132");	// 1
        th(   "IV-133");	// 1
        th(   "IV-135");	// 1
        th(   "IV-137");	// 1
        th(   "IV-139");	// 1
        th(    "IV-14");	// 1
        th(   "IV-140");	// 1
        th(   "IV-141");	// 1
        th(   "IV-142");	// 1
        th(   "IV-143");	// 1
        th(   "IV-144");	// 1
        th(   "IV-145");	// 1
        th(   "IV-146");	// 1
        th(   "IV-147");	// 1
        th(   "IV-148");	// 1
        th(   "IV-149");	// 1
        th(    "IV-15");	// 1
        th(   "IV-150");	// 1
        th(   "IV-151");	// 1
        th(   "IV-152");	// 1
        th(   "IV-153");	// 1
        th(   "IV-155");	// 1
        th(   "IV-157");	// 1
        th(   "IV-158");	// 1
        th(   "IV-159");	// 1
        th(    "IV-16");	// 1
        th(   "IV-160");	// 1
        th(   "IV-161");	// 1
        th(   "IV-162");	// 1
        th(   "IV-163");	// 1
        th(   "IV-164");	// 1
        th(   "IV-165");	// 1
        th(   "IV-166");	// 1
        th(   "IV-167");	// 1
        th(   "IV-168");	// 1
        th(   "IV-170");	// 1
        th(   "IV-172");	// 1
        th(   "IV-173");	// 1
        th(   "IV-174");	// 1
        th(   "IV-175");	// 1
        th(   "IV-176");	// 1
        th(   "IV-177");	// 1
        th(   "IV-178");	// 1
        th(   "IV-179");	// 1
        th(    "IV-18");	// 1
        th(   "IV-180");	// 1
        th(   "IV-181");	// 1
        th(   "IV-182");	// 1
        th(   "IV-183");	// 1
        th(   "IV-184");	// 1
        th(   "IV-185");	// 1
        th(   "IV-186");	// 1
        th(   "IV-187");	// 1
        th(   "IV-188");	// 1
        th(   "IV-189");	// 1
        th(   "IV-190");	// 1
        th(   "IV-191");	// 1
        th(   "IV-192");	// 1
        th(   "IV-193");	// 1
        th(   "IV-194");	// 1
        th(   "IV-195");	// 1
        th(   "IV-196");	// 1
        th(   "IV-197");	// 1
        th(   "IV-198");	// 1
        th(   "IV-199");	// 1
        th(    "IV-20");	// 1
        th(   "IV-200");	// 1
        th(   "IV-201");	// 1
        th(   "IV-202");	// 1
        th(   "IV-203");	// 1
        th(   "IV-204");	// 1
        th(   "IV-205");	// 1
        th(   "IV-207");	// 1
        th(   "IV-208");	// 1
        th(   "IV-209");	// 1
        th(    "IV-21");	// 1
        th(   "IV-210");	// 1
        th(    "IV-23");	// 1
        th(    "IV-24");	// 1
        th(    "IV-25");	// 1
        th(    "IV-26");	// 1
        th(    "IV-28");	// 1
        th(    "IV-30");	// 1
        th(    "IV-31");	// 1
        th(    "IV-32");	// 1
        th(    "IV-35");	// 1
        th(    "IV-36");	// 1
        th(    "IV-37");	// 1
        th(    "IV-38");	// 1
        th(    "IV-39");	// 1
        th(    "IV-40");	// 1
        th(    "IV-41");	// 1
        th(    "IV-42");	// 1
        th(    "IV-43");	// 1
        th(    "IV-44");	// 1
        th(    "IV-45");	// 1
        th(    "IV-46");	// 1
        th(    "IV-47");	// 1
        th(    "IV-48");	// 1
        th(    "IV-49");	// 1
        th(    "IV-51");	// 1
        th(    "IV-52");	// 1
        th(    "IV-53");	// 1
        th(    "IV-54");	// 1
        th(    "IV-55");	// 1
        th(    "IV-56");	// 1
        th(    "IV-59");	// 1
        th(    "IV-60");	// 1
        th(    "IV-62");	// 1
        th(    "IV-63");	// 1
        th(    "IV-64");	// 1
        th(    "IV-65");	// 1
        th(    "IV-68");	// 1
        th(    "IV-69");	// 1
        th(    "IV-70");	// 1
        th(    "IV-71");	// 1
        th(    "IV-72");	// 1
        th(    "IV-73");	// 1
        th(    "IV-74");	// 1
        th(    "IV-75");	// 1
        th(    "IV-76");	// 1
        th(    "IV-77");	// 1
        th(    "IV-78");	// 1
        th(    "IV-79");	// 1
        th(    "IV-80");	// 1
        th(    "IV-82");	// 1
        th(    "IV-83");	// 1
        th(    "IV-84");	// 1
        th(    "IV-85");	// 1
        th(    "IV-86");	// 1
        th(    "IV-87");	// 1
        th(    "IV-88");	// 1
        th(    "IV-89");	// 1
        th(    "IV-91");	// 1
        th(    "IV-92");	// 1
        th(    "IV-93");	// 1
        th(    "IV-94");	// 1
        th(    "IV-95");	// 1
        th(    "IV-96");	// 1
        th(    "IV-97");	// 1
        th(    "IV-98");	// 1
        th(    "IV-99");	// 1
        th(      "JAB");	// 1
        th(     "'JAm");	// 1
        th(    "^'JAM");	// 1
        th(    "'JAM0");	// 1
        th(  "'JAMA'I");	// 1
        th(  "'JAM-SU");	// 1
        th(     "J'AN");	// 1
        th(   "JANY'A");	// 1
        th(    "'JA'U");	// 1
        th(      "JE2");	// 1
        th(      "JE4");	// 1
        th(      "JED");	// 1
        th(      "JEE");	// 1
        th(      "JEG");	// 1
        th(      "JEN");	// 1
        th(      "JER");	// 1
        th(      "JHA");	// 1
        th(    "'JHOR");	// 1
        th(      "JI4");	// 1
        th(    "^'JIG");	// 1
        th(    "'J'IG");	// 1
        th(   "'JIGIS");	// 1
        th(   "^'JIGS");	// 1
        th(      "JIL");	// 1
        th(    "JINTU");	// 1
        th(      "^JO");	// 1
        th(      "JO3");	// 1
        th(     "'JOB");	// 1
        th(      "JOD");	// 1
        th(     "JOGS");	// 1
        th(      "JOL");	// 1
        th(    "'JOMD");	// 1
        th(    "'JOmS");	// 1
        th(     "JONG");	// 1
        th(    "'JONG");	// 1
        th(    "'JO'O");	// 1
        th(      "JOR");	// 1
        th(     "JPAR");	// 1
        th(   "JRGYAS");	// 1
        th(      "JTU");	// 1
        th(      "JU2");	// 1
        th(      "JU4");	// 1
        th(   "'JU'AM");	// 1
        th(    "'JUGN");	// 1
        th( "'JUGPA'I");	// 1
        th(    "'JU'I");	// 1
        th(   "JU'U'I");	// 1
        th(      "JYA");	// 1
        th(   "'JYNGS");	// 1
        th(   "''KA''");	// 1
        th(     "'KA'");	// 1
        th(     "K'A:");	// 1
        th(      "KA4");	// 1
        th(     "'KAB");	// 1
        th(    "KA:DE");	// 1
        th(   "KADHAR");	// 1
        th(    "K'A'E");	// 1
        th(     "KA'E");	// 1
        th(    "KAHnA");	// 1
        th(    "KAHNG");	// 1
        th(    "KA:JE");	// 1
        th(    "KAKHA");	// 1
        th(    "KAK'I");	// 1
        th( "K'AKYA'I");	// 1
        th(     "K'Am");	// 1
        th(     "KAMA");	// 1
        th(    "KAMBU");	// 1
        th(     "KAMI");	// 1
        th(    "KAmKA");	// 1
        th(    "KAMPI");	// 1
        th(     "KA'N");	// 1
        th(     "KAnA");	// 1
        th(    "KAN'A");	// 1
        th(    "KAndA");	// 1
        th(    "KANDA");	// 1
        th(   "KANDHA");	// 1
        th( "KANDHASA");	// 1
        th(   "KAndRI");	// 1
        th(  "KANDYEE");	// 1
        th(   "KANDZI");	// 1
        th(  "KAnthA:");	// 1
        th(   "KANthA");	// 1
        th(   "KANTHA");	// 1
        th(  "KANTH'A");	// 1
        th(  "KANTISA");	// 1
        th( "K'ANTYEE");	// 1
        th( "KANYDZIR");	// 1
        th(   "KANYPA");	// 1
        th(     "KA'R");	// 1
        th(     "KAR:");	// 1
        th(     "KAR7");	// 1
        th(   "K'ARA'");	// 1
        th(     "KARI");	// 1
        th(  "KARKAtA");	// 1
        th("KARKAtA'I");	// 1
        th(    "KARKE");	// 1
        th(    "KARKI");	// 1
        th(    "KARKO");	// 1
        th(     "KARM");	// 1
        th(   "^KARMA");	// 1
        th(   "K'ARMA");	// 1
        th(   "KA-RMA");	// 1
        th(  "KARMA'A");	// 1
        th(   "KARMAD");	// 1
        th(  "KARMAPA");	// 1
        th(   "KARMAR");	// 1
        th(  "KARMASA");	// 1
        th(    "KARMI");	// 1
        th(     "KARn");	// 1
        th(   "KARnAS");	// 1
        th(    "KARnI");	// 1
        th(   "KARnnA");	// 1
        th(    "KARnO");	// 1
        th(     "KARR");	// 1
        th(    "KARSH");	// 1
        th( "K'ARsH'A");	// 1
        th(  "KARsh'I");	// 1
        th( "K'ARshIT");	// 1
        th("K'ARshITA'O");	// 1
        th(     "KART");	// 1
        th( "KART'A'O");	// 1
        th(  "K'ARTHO");	// 1
        th(   "KARTRI");	// 1
        th(   "KARTYi");	// 1
        th(    "KARYA");	// 1
        th(  "KASHA'A");	// 1
        th( "KAshATRA");	// 1
        th(    "KAshi");	// 1
        th(  "KAshM'I");	// 1
        th( "KAshM'IS");	// 1
        th(  "KAshn'A");	// 1
        th(  "KASHR'I");	// 1
        th(  "KAshshA");	// 1
        th(  "KASHSHE");	// 1
        th(  "KAsht'I");	// 1
        th(  "KASHTZA");	// 1
        th( "KASHTZRA");	// 1
        th(   "KAshVI");	// 1
        th(   "KA:SKU");	// 1
        th("KASMIRA:A'I");	// 1
        th(   "KASS'A");	// 1
        th(    "KASTA");	// 1
        th(   "KA:STE");	// 1
        th(  "KASTH'U");	// 1
        th(      "KAt");	// 1
        th(   "KAT'Am");	// 1
        th(  "KA:THOG");	// 1
        th(   "KATPUR");	// 1
        th(  "KATSITI");	// 1
        th(    "KATTA");	// 1
        th(   "KATTRA");	// 1
        th(   "KATTTA");	// 1
        th(    "KATYA");	// 1
        th(   "K'ATYA");	// 1
        th(  "KATYA'I");	// 1
        th(  "K'ATY'I");	// 1
        th(  "KATZA'I");	// 1
        th(    "KAWAm");	// 1
        th(   "KAYANG");	// 1
        th(    "KAYGN");	// 1
        th(   "K'AYYA");	// 1
        th(      "KBA");	// 1
        th(    "KBVIL");	// 1
        th(     "KBYA");	// 1
        th(     "KCAR");	// 1
        th(       "KD");	// 1
        th(    "KDA'O");	// 1
        th(     "KDAR");	// 1
        th(   "KDPYID");	// 1
        th(      "KDU");	// 1
        th(      "KE2");	// 1
        th(      "KE4");	// 1
        th(      "KED");	// 1
        th(     "K'EE");	// 1
        th(      "KEm");	// 1
        th(      "KEM");	// 1
        th(     "KE'O");	// 1
        th(    "KEshA");	// 1
        th(    "KE'UR");	// 1
        th(     "KH0O");	// 1
        th(     "KH9O");	// 1
        th(     "^KHA");	// 1
        th(     "KHA2");	// 1
        th(     "KHA4");	// 1
        th(  "KHA'ANG");	// 1
        th(     "KHAd");	// 1
        th(    "'KHAD");	// 1
        th("KHAdGiDHRA");	// 1
        th(    "'KHAG");	// 1
        th(   "'KHA'I");	// 1
        th(   "KH'A'I");	// 1
        th(    "KHALA");	// 1
        th(   "KHAMSU");	// 1
        th(    "KHAnd");	// 1
        th( "KHANDHAS");	// 1
        th( "KHANDHIS");	// 1
        th(    "KHANI");	// 1
        th(  "KHANN'A");	// 1
        th("KHAO'DANG");	// 1
        th(   "KHAR'I");	// 1
        th("KHASIDDHI");	// 1
        th( "KHAtV'Am");	// 1
        th(  "KHA'U'I");	// 1
        th(    "KHAVA");	// 1
        th(     "KH'E");	// 1
        th(     "KHE2");	// 1
        th(     "KHE4");	// 1
        th(  "KHE'ANG");	// 1
        th(    "'KHED");	// 1
        th(     "KHEE");	// 1
        th( "KHENDR'A");	// 1
        th(    "'KHER");	// 1
        th(    "KHGAS");	// 1
        th(    "KHHAB");	// 1
        th(      "KHi");	// 1
        th(     "KHI2");	// 1
        th(     "KHI4");	// 1
        th(     "KHID");	// 1
        th(    "'KHID");	// 1
        th(     "KHIG");	// 1
        th(    "KHIGS");	// 1
        th(   "'KHIGS");	// 1
        th(  "KHI:K'A");	// 1
        th(     "KHIL");	// 1
        th(    "KHIMS");	// 1
        th( "KHINDHAS");	// 1
        th(    "KHING");	// 1
        th("KHINYTZAS");	// 1
        th(    "KHI'O");	// 1
        th(    "'KHIS");	// 1
        th(  "KHIshnU");	// 1
        th(     "KHMS");	// 1
        th(     "KHO2");	// 1
        th(     "KHO3");	// 1
        th(     "KHO4");	// 1
        th(   "KHO'AM");	// 1
        th(   "'KHOBA");	// 1
        th(    "KHOBS");	// 1
        th(  "KHOLBAS");	// 1
        th(    "'KHOM");	// 1
        th(   "''KHOR");	// 1
        th(    "^KHOS");	// 1
        th(    "KHO'U");	// 1
        th(  "KHRA'AM");	// 1
        th( "KHRA'ANG");	// 1
        th(   "'KHRAG");	// 1
        th(  "KHRAMSm");	// 1
        th(    "'KHRE");	// 1
        th(    "KHREB");	// 1
        th(    "KHREM");	// 1
        th(   "KHRE'O");	// 1
        th(    "KHRER");	// 1
        th(     "KHRi");	// 1
        th(   "KHRIBS");	// 1
        th(    "KHRId");	// 1
        th(   "'KHRId");	// 1
        th( "KHRI'I'I");	// 1
        th(   "KHRImS");	// 1
        th(   "KHRIMs");	// 1
        th(  "KHRIMSm");	// 1
        th(  "KHRIMSS");	// 1
        th(  "'KHRI'O");	// 1
        th(   "KHRISM");	// 1
        th(  "'KHRI'U");	// 1
        th(  "KHRI'US");	// 1
        th(   "KHRMAS");	// 1
        th(  "'KHRNGS");	// 1
        th( "KHRO'ANG");	// 1
        th(  "'KHROGS");	// 1
        th(  "'KHRO'I");	// 1
        th(   "^KHRON");	// 1
        th(   "'KHRON");	// 1
        th(   "'KHROS");	// 1
        th(   "KHRO'U");	// 1
        th(  "'KHRUBS");	// 1
        th("KHRU'I'U'AM");	// 1
        th(   "'KHRUM");	// 1
        th(  "'KHRU'O");	// 1
        th(   "'KHRVI");	// 1
        th(    "KHTRA");	// 1
        th(   "KHTYAD");	// 1
        th(     "KHU4");	// 1
        th(  "KHU'ANG");	// 1
        th(     "KHUd");	// 1
        th(    "KHUED");	// 1
        th(    "KHUIS");	// 1
        th(   "'KHUNG");	// 1
        th(   "KHUNGs");	// 1
        th(   "'KHURL");	// 1
        th(  "KHVA'AM");	// 1
        th(    "KHVAG");	// 1
        th(   "KHVANG");	// 1
        th(     "KHVO");	// 1
        th(    "KHY0D");	// 1
        th(    "KHYA'");	// 1
        th("KHY'ABNYA'O");	// 1
        th(  "KHYABPA");	// 1
        th(  "'KHYABS");	// 1
        th(    "KHYAd");	// 1
        th(   "^KHYAD");	// 1
        th( "KHYADPAR");	// 1
        th(   "'KHYAm");	// 1
        th(   "KHY'AM");	// 1
        th(   "KHY'AN");	// 1
        th(   "KHYAND");	// 1
        th(   "KHYA'O");	// 1
        th(    "KHYAP");	// 1
        th(  "KHY'ATA");	// 1
        th(  "KHYA'UR");	// 1
        th(     "KHYB");	// 1
        th(  "KHYE'AU");	// 1
        th(   "'KHYEB");	// 1
        th(  "KHYED'U");	// 1
        th(    "KHYEL");	// 1
        th(   "'KHYEL");	// 1
        th(   "KHYENG");	// 1
        th(  "KHYENGS");	// 1
        th(   "KHYHED");	// 1
        th(    "'KHYI");	// 1
        th(    "KHYI'");	// 1
        th(  "KHYI'AM");	// 1
        th(    "KHYIB");	// 1
        th(   "KHYIBS");	// 1
        th(   "'KHYID");	// 1
        th(    "KHYIG");	// 1
        th(  "'KHYIGS");	// 1
        th(   "^KHYIM");	// 1
        th(   "'KHYIN");	// 1
        th(   "KHYING");	// 1
        th(   "KHYI'O");	// 1
        th(   "'KHYIS");	// 1
        th(    "KHYOB");	// 1
        th(   "'KHYON");	// 1
        th(   "KHYO'O");	// 1
        th(    "'KHYU");	// 1
        th(  "'KHYUIG");	// 1
        th(   "'KHYUL");	// 1
        th(    "KHYUM");	// 1
        th(    "KHYUN");	// 1
        th( "'KHYUNGS");	// 1
        th(   "KHYU'O");	// 1
        th(   "'KHYUS");	// 1
        th(  "KHYUt'O");	// 1
        th(  "'KHYUYD");	// 1
        th(      "KI2");	// 1
        th(      "KI4");	// 1
        th(   "K'I'AM");	// 1
        th(     "K'Im");	// 1
        th(   "KI-MPA");	// 1
        th(    "KIMRA");	// 1
        th(   "KINGKA");	// 1
        th(    "KiNYA");	// 1
        th(    "K'I'O");	// 1
        th(     "KI'O");	// 1
        th("K'IRTHIKWADZIN");	// 1
        th(  "K'I-RTI");	// 1
        th("K'IRT'I'I");	// 1
        th("KiRTRAK'AT");	// 1
        th(    "KIshA");	// 1
        th(   "K'ITIS");	// 1
        th(    "KIT'O");	// 1
        th(    "KITRA");	// 1
        th(   "KI'U'I");	// 1
        th(     "KIVP");	// 1
        th(     "KIYS");	// 1
        th(     "KK'A");	// 1
        th(   "KK'A'I");	// 1
        th(    "KKA'I");	// 1
        th(     "KKHA");	// 1
        th(      "KKI");	// 1
        th(    "KKI'I");	// 1
        th(    "KKING");	// 1
        th(     "KKIS");	// 1
        th(      "KKO");	// 1
        th(      "KKU");	// 1
        th(    "KKYAD");	// 1
        th(   "KKYANG");	// 1
        th(       "KL");	// 1
        th(     "KLAd");	// 1
        th(     "KLAL");	// 1
        th(      "KLE");	// 1
        th(   "KLENGS");	// 1
        th(     "KLi:");	// 1
        th(   "KLIRTI");	// 1
        th(     "KLOM");	// 1
        th(    "KLO'O");	// 1
        th(   "KL'U'I");	// 1
        th(     "KLUL");	// 1
        th(     "KLUN");	// 1
        th(    "KLU'U");	// 1
        th(      "KmA");	// 1
        th(      "KNA");	// 1
        th(     "KndU");	// 1
        th( "KNGnnA'I");	// 1
        th(      "KNO");	// 1
        th(    "KNU'O");	// 1
        th(     "KNYA");	// 1
        th(      "KO:");	// 1
        th(      "KO3");	// 1
        th(      "KO4");	// 1
        th(    "KO'AM");	// 1
        th(    "KODKA");	// 1
        th(     "'KOG");	// 1
        th(     "KOKA");	// 1
        th(     "KOMS");	// 1
        th(  "KONG+KA");	// 1
        th(    "KONGS");	// 1
        th( "KONYATAm");	// 1
        th(  "KONYHAm");	// 1
        th(   "KOO'dI");	// 1
        th(   "KO'OHU");	// 1
        th( "KOO-nAdI");	// 1
        th( "KOO-n+dI");	// 1
        th( "KOOndINA");	// 1
        th( "KOOntA'I");	// 1
        th(  "KOOSH'A");	// 1
        th(  "KOOshAm");	// 1
        th(   "KOOSTU");	// 1
        th(   "KOOtnI");	// 1
        th(    "KOSTU");	// 1
        th(     "KOtA");	// 1
        th(    "KOTRA");	// 1
        th(    "KOttA");	// 1
        th(    "KPA'I");	// 1
        th(      "KPO");	// 1
        th(       "KR");	// 1
        th(     "K'RA");	// 1
        th(     "KRA'");	// 1
        th(    "KRAGS");	// 1
        th(   "KR'A'i");	// 1
        th(    "KRA'i");	// 1
        th(    "'KRAL");	// 1
        th(     "KRAm");	// 1
        th(   "KRAmTA");	// 1
        th(    "'KRAN");	// 1
        th(   "KRANGS");	// 1
        th(  "KR'ANTE");	// 1
        th(    "KRA'O");	// 1
        th(    "KRA'U");	// 1
        th(     "KREB");	// 1
        th(     "KRED");	// 1
        th(    "KRE'I");	// 1
        th(     "KREL");	// 1
        th(    "'KREL");	// 1
        th(     "KREN");	// 1
        th(    "'KREN");	// 1
        th(   "'KRENG");	// 1
        th(    "'KRES");	// 1
        th(    "KRGYA");	// 1
        th(   "KRGYUD");	// 1
        th(   "KRHIGS");	// 1
        th(     "KRi:");	// 1
        th(     "'KRI");	// 1
        th(   "KR'idA");	// 1
        th(    "KRIGS");	// 1
        th(   "KR'i'I");	// 1
        th(    "KRIMS");	// 1
        th(    "KRING");	// 1
        th(    "KRiNY");	// 1
        th(   "KRinYA");	// 1
        th(  "KRi+NYA");	// 1
        th(   "KRINYA");	// 1
        th(    "KRi'O");	// 1
        th(    "'KRIS");	// 1
        th(   "KRishn");	// 1
        th( "KRishn'A");	// 1
        th("KR'Ishn'A");	// 1
        th(  "KRIshnO");	// 1
        th(   "KRisht");	// 1
        th(   "KRit'O");	// 1
        th(   "KRKANG");	// 1
        th(     "KRnA");	// 1
        th(   "KRODHA");	// 1
        th(    "KROGS");	// 1
        th(    "KRO'I");	// 1
        th(    "'KROL");	// 1
        th(   "KR'ONG");	// 1
        th("K'RshU:'O");	// 1
        th(     "'KRU");	// 1
        th( "KRUNGDGA");	// 1
        th( "KRUNGNGA");	// 1
        th(   "KRUNGS");	// 1
        th(  "'KRUNGS");	// 1
        th( "KRUNYTZA");	// 1
        th(     "KRUS");	// 1
        th(       "KS");	// 1
        th(     "KSEY");	// 1
        th(     "Ksh'");	// 1
        th(     "KSHA");	// 1
        th(    "KSH'A");	// 1
        th(  "Ksh'A'I");	// 1
        th(   "KshANA");	// 1
        th(  "KshANTI");	// 1
        th(   "KshASA");	// 1
        th(  "KshATRA");	// 1
        th(    "KshE:");	// 1
        th(    "KshEm");	// 1
        th(    "KshES");	// 1
        th(  "KshETRA");	// 1
        th(    "KshI'");	// 1
        th(    "KshIM");	// 1
        th(     "KshM");	// 1
        th(    "KshMA");	// 1
        th(   "KshM'A");	// 1
        th(   "KshMA:");	// 1
        th(    "KshME");	// 1
        th(    "KshnA");	// 1
        th(     "KshO");	// 1
        th(    "KsH'O");	// 1
        th(   "KshthA");	// 1
        th(    "Ksh'U");	// 1
        th(     "KsHU");	// 1
        th(  "KshUDHA");	// 1
        th(     "KSUR");	// 1
        th(    "KSYED");	// 1
        th(   "KSYONG");	// 1
        th(      "KtA");	// 1
        th(     "K-TA");	// 1
        th(   "KT'ANG");	// 1
        th(      "KTE");	// 1
        th(     "KT'i");	// 1
        th(  "KTI'ANG");	// 1
        th(     "KTIS");	// 1
        th(     "KTRA");	// 1
        th(      "KTU");	// 1
        th(    "KTU'O");	// 1
        th(    "KTV'A");	// 1
        th(  "KTV'A'O");	// 1
        th(    "KTV'I");	// 1
        th(     "KTZI");	// 1
        th(      "KU4");	// 1
        th(     "KU8N");	// 1
        th(  "KUBBH'A");	// 1
        th(  "KUBDZA:");	// 1
        th(   "KUBH'A");	// 1
        th(     "KUDE");	// 1
        th(     "KUGS");	// 1
        th(     "KU'i");	// 1
        th(   "KUKshI");	// 1
        th(     "'KUM");	// 1
        th("KUM'ARATI'I");	// 1
        th(    "KUMBA");	// 1
        th(    "KUMBI");	// 1
        th(     "KUMS");	// 1
        th(   "KUN-DA");	// 1
        th(  "KUNdA'I");	// 1
        th(   "KUndAm");	// 1
        th(   "KUND'I");	// 1
        th(   "KUNDRA");	// 1
        th("KUNDZIKAS");	// 1
        th(    "KUNGS");	// 1
        th(   "KUNGYI");	// 1
        th(     "KUNN");	// 1
        th("KUN-NGA'I");	// 1
        th(    "KUntA");	// 1
        th(    "KUNTA");	// 1
        th(  "KUNTA'I");	// 1
        th(  "KUNYDZA");	// 1
        th(     "KURB");	// 1
        th(    "KURBA");	// 1
        th( "KURBA:'O");	// 1
        th("KURBNYSH'U");	// 1
        th("KURBNYTZTS'U");	// 1
        th(  "KURMA:O");	// 1
        th(  "KURMLA'");	// 1
        th("KURWANTI'O");	// 1
        th( "KURYA:'O");	// 1
        th(    "KUshA");	// 1
        th(      "KUt");	// 1
        th(    "KUTAS");	// 1
        th(    "KUttA");	// 1
        th(     "KV'A");	// 1
        th(     "KVA'");	// 1
        th(    "KVANG");	// 1
        th(   "KVANSU");	// 1
        th(     "KVAS");	// 1
        th(     "KVIB");	// 1
        th(     "KVIP");	// 1
        th(  "KVIPA'O");	// 1
        th(     "KVON");	// 1
        th(    "KVONG");	// 1
        th(    "KWA'E");	// 1
        th(     "'KYA");	// 1
        th(     "KYA:");	// 1
        th(    "KYA'A");	// 1
        th(   "^KYABS");	// 1
        th(    "KYAmS");	// 1
        th(    "KYAMS");	// 1
        th(    "KYAND");	// 1
        th(   "'KYANG");	// 1
        th(   "KYANGG");	// 1
        th("KYANG'JIG");	// 1
        th(   "KYANGM");	// 1
        th(  "KYAPA'O");	// 1
        th(    "'KYED");	// 1
        th(    "KYEGS");	// 1
        th(    "KYENG");	// 1
        th(    "KYE'U");	// 1
        th(   "KY'GOG");	// 1
        th(    "KYHIS");	// 1
        th(   "KYIBAR");	// 1
        th(   "KYId'A");	// 1
        th(  "KYIDBYE");	// 1
        th(   "KYIDON");	// 1
        th(    "KYIDS");	// 1
        th(     "KYIG");	// 1
        th( "KYI'GYUR");	// 1
        th(    "'KYIL");	// 1
        th(   "KYIMAS");	// 1
        th(    "KYImS");	// 1
        th( "KYIMTSAN");	// 1
        th(    "KYINA");	// 1
        th(   "KYINUS");	// 1
        th(   "KYI'OL");	// 1
        th(     "KYiS");	// 1
        th(    "^KYIS");	// 1
        th(  "KYITSIG");	// 1
        th(   "KYIYIS");	// 1
        th(      "KYN");	// 1
        th(     "'KYO");	// 1
        th(    "KYO'I");	// 1
        th(   "KYU'AM");	// 1
        th(     "KYUI");	// 1
        th(    "KYUNG");	// 1
        th(      "KYY");	// 1
        th(     "KYYI");	// 1
        th(     "KZHU");	// 1
        th(       "L'");	// 1
        th(       "L+");	// 1
        th(      "L2A");	// 1
        th(      "'LA");	// 1
        th(    "LA108");	// 1
        th(     "LA11");	// 1
        th(      "LA2");	// 1
        th(      "LAA");	// 1
        th(     "LAAM");	// 1
        th(   "LAA'NA");	// 1
        th(  "LA'ANGS");	// 1
        th(     "LAAS");	// 1
        th(   "LABDHE");	// 1
        th(   "LABDZI");	// 1
        th(  "LABYANG");	// 1
        th(    "LADAN");	// 1
        th(  "L'ADGAL");	// 1
        th(    "LA'DI");	// 1
        th(   "LA'DOD");	// 1
        th(     "LADS");	// 1
        th(  "LADZDZI");	// 1
        th(      "LAE");	// 1
        th(     "LA'E");	// 1
        th(     "^LAG");	// 1
        th(   "LAGANG");	// 1
        th(   "LAGCHI");	// 1
        th(    "LAGMA");	// 1
        th( "LAGSUNGS");	// 1
        th(   "LAKCHI");	// 1
        th(  "LAKCHIS");	// 1
        th( "LAKIshAm");	// 1
        th(   "LAKMIS");	// 1
        th(    "LAKNI");	// 1
        th( "LAKshA'I");	// 1
        th(  "LAKshAR");	// 1
        th(   "LAKshE");	// 1
        th( "LAKshM'A");	// 1
        th("LAKshM'I'I");	// 1
        th("LAKshMI'I");	// 1
        th("LAKshM'IS");	// 1
        th("LAKshM'ISA");	// 1
        th( "LAKshRIm");	// 1
        th(     "L'Am");	// 1
        th(     "L'AM");	// 1
        th(   "LAMBHE");	// 1
        th(     "LAMG");	// 1
        th(    "LAM'I");	// 1
        th(     "LAmS");	// 1
        th(   "LANBHA");	// 1
        th(   "LANdAR");	// 1
        th(   "LAndHA");	// 1
        th(  "LANDHAR");	// 1
        th( "LANDHARA");	// 1
        th(   "LANd'I");	// 1
        th(   "LANDRA");	// 1
        th( "LANDRA'I");	// 1
        th(    "LANGA");	// 1
        th( "LANGGA'I");	// 1
        th( "LANG-KA'");	// 1
        th( "LANGKARA");	// 1
        th(   "LANGKR");	// 1
        th(   "LANG'O");	// 1
        th(    "LANKA");	// 1
        th(   "LANTAS");	// 1
        th(   "LANTSA");	// 1
        th("LANYDZA'I");	// 1
        th("LANYIDZ'I");	// 1
        th(  "LANYTSA");	// 1
        th(   "LA'ONG");	// 1
        th(  "LA'ONGS");	// 1
        th(   "LAPDZA");	// 1
        th(    "LARMA");	// 1
        th(  "LARnnAM");	// 1
        th(      "LAs");	// 1
        th(     "^LAS");	// 1
        th(     "LAS:");	// 1
        th(     "LAS0");	// 1
        th(    "LASDE");	// 1
        th(    "LAsht");	// 1
        th(     "LASM");	// 1
        th(    "LASNI");	// 1
        th(   "LASOGS");	// 1
        th(   "LASPYI");	// 1
        th(     "LASU");	// 1
        th(   "LASYIN");	// 1
        th(     "LATR");	// 1
        th(   "LA'U'I");	// 1
        th(    "LA'UR");	// 1
        th(    "LAYIN");	// 1
        th(   "LAZHES");	// 1
        th(     "LB'A");	// 1
        th(     "LBAG");	// 1
        th(    "LBA'I");	// 1
        th(     "LBAR");	// 1
        th(   "LBARMA");	// 1
        th(    "LBENG");	// 1
        th(      "LBI");	// 1
        th(      "LBO");	// 1
        th(  "LBSKRUN");	// 1
        th(     "LBUB");	// 1
        th(    "LBUGS");	// 1
        th(   "LCAGES");	// 1
        th(    "LCA'I");	// 1
        th(    "LCAMS");	// 1
        th(   "^LCANG");	// 1
        th(  "LCE'ANG");	// 1
        th(     "LCED");	// 1
        th(     "LCEG");	// 1
        th(     "LCEM");	// 1
        th(    "LCE'U");	// 1
        th(   "LCHAGS");	// 1
        th(   "LCI'AM");	// 1
        th(     "LCIL");	// 1
        th(      "LCO");	// 1
        th(     "LCOD");	// 1
        th(     "LCUB");	// 1
        th(     "LDA'");	// 1
        th(   "LDA'AM");	// 1
        th(    "LDADN");	// 1
        th("LDANA'ANG");	// 1
        th(    "LDANE");	// 1
        th(    "LDEGS");	// 1
        th(    "LDGOG");	// 1
        th(    "LDIGS");	// 1
        th(    "LDI'I");	// 1
        th(   "L'DI'O");	// 1
        th(    "'LDIR");	// 1
        th(    "'LDOG");	// 1
        th(    "LDO'I");	// 1
        th(     "LDOL");	// 1
        th(     "LDOR");	// 1
        th(   "LDZAGS");	// 1
        th(      "LE2");	// 1
        th(      "LE4");	// 1
        th(   "LEBH'U");	// 1
        th(   "LEGCHI");	// 1
        th(    "^LEGS");	// 1
        th(    "'LEGS");	// 1
        th(    "LEGS0");	// 1
        th(    "LEGSA");	// 1
        th(    "L'E'I");	// 1
        th(   "LEMBHI");	// 1
        th(    "LENDA");	// 1
        th(  "LENDA'I");	// 1
        th(  "LENDR'A");	// 1
        th( "LENPRA'I");	// 1
        th(    "LENTA");	// 1
        th(   "LEN-TA");	// 1
        th(  "LENtA'I");	// 1
        th(   "LENTRA");	// 1
        th(  "LENTRAR");	// 1
        th(    "LE:'O");	// 1
        th(   "LE'U25");	// 1
        th(    "LE'U6");	// 1
        th(  "LE'U'AM");	// 1
        th(   "LE'UNG");	// 1
        th( "LE'U'O'I");	// 1
        th(    "LGAGS");	// 1
        th(     "LGAN");	// 1
        th(     "LGNA");	// 1
        th(      "LGS");	// 1
        th(   "LGZHAN");	// 1
        th(       "LH");	// 1
        th(    "LHA21");	// 1
        th(  "LHA'ANG");	// 1
        th(    "'LHAG");	// 1
        th(   "LHAGMA");	// 1
        th(   "LHA'IK");	// 1
        th(  "LHA'I'O");	// 1
        th(   "LHA-ME");	// 1
        th(   "LHANDA");	// 1
        th(    "LHA'U");	// 1
        th(    "LHAZG");	// 1
        th(    "LHEBS");	// 1
        th(    "LHE'I");	// 1
        th(    "LHENG");	// 1
        th(    "LHTAR");	// 1
        th(    "LHU7N");	// 1
        th(    "LHUBS");	// 1
        th(     "LHUm");	// 1
        th(    "'LHUN");	// 1
        th(    "LHYAD");	// 1
        th(      "LI2");	// 1
        th(      "LI4");	// 1
        th(    "LI'AM");	// 1
        th(     "LIHA");	// 1
        th(    "L'I'I");	// 1
        th(      "LIK");	// 1
        th(    "LIMBA");	// 1
        th(   "LIMBHI");	// 1
        th(    "LINDI");	// 1
        th(  "LINGA'I");	// 1
        th(   "LINGGA");	// 1
        th(  "LING-KA");	// 1
        th("LIN-NGA'I");	// 1
        th(    "L'ITA");	// 1
        th(   "LI'U'I");	// 1
        th(    "LI'UR");	// 1
        th(    "LI'US");	// 1
        th(      "LJA");	// 1
        th(     "LJAG");	// 1
        th(     "LJAR");	// 1
        th(     "LJEN");	// 1
        th(   "LJENGS");	// 1
        th(    "LJIGS");	// 1
        th(      "LJO");	// 1
        th(     "LJOR");	// 1
        th(    "LJUNG");	// 1
        th(     "LKAD");	// 1
        th(    "LKA'I");	// 1
        th(    "LKHRO");	// 1
        th(     "LKON");	// 1
        th(     "LKOS");	// 1
        th(    "LKSHA");	// 1
        th(    "LKshI");	// 1
        th(    "LKshU");	// 1
        th(     "LKUR");	// 1
        th(     "LKUS");	// 1
        th(      "LLA");	// 1
        th(      "LLO");	// 1
        th(     "LLUS");	// 1
        th(    "LMA'I");	// 1
        th(     "LMED");	// 1
        th(   "LMNYAM");	// 1
        th(       "LN");	// 1
        th(     "LNAG");	// 1
        th(    "LNANG");	// 1
        th(    "^LNGA");	// 1
        th(  "LNGA'Am");	// 1
        th(    "LNGAm");	// 1
        th(    "LNGAM");	// 1
        th(  "LNGANGS");	// 1
        th(  "LNGA-SE");	// 1
        th(   "LNGA'U");	// 1
        th(     "LNGI");	// 1
        th(   "LNGI'I");	// 1
        th(    "LNGIR");	// 1
        th(  "LNGKA'I");	// 1
        th(    "LNGOD");	// 1
        th(    "LNGOM");	// 1
        th(  "LNGONGS");	// 1
        th(    "LNGOS");	// 1
        th(     "LNGU");	// 1
        th(    "LNLGA");	// 1
        th(    "LNYAD");	// 1
        th(    "LNYAN");	// 1
        th(     "LNYO");	// 1
        th(      "L'O");	// 1
        th(      "LO'");	// 1
        th(      "LO3");	// 1
        th(     "LO'A");	// 1
        th(    "LOHAG");	// 1
        th(    "LONGs");	// 1
        th(   "'LONGS");	// 1
        th(  "LOTZA'A");	// 1
        th( "LOTZATSA");	// 1
        th("LO-TZ+TSA");	// 1
        th( "LOTZ-TSA");	// 1
        th("LOTZTSA'I");	// 1
        th(   "LPANGS");	// 1
        th(      "LPE");	// 1
        th(     "LPOS");	// 1
        th(      "LPR");	// 1
        th(    "LRNAM");	// 1
        th(      "LRO");	// 1
        th(     "LROL");	// 1
        th(     "LROS");	// 1
        th(     "LR'U");	// 1
        th(    "LSBUR");	// 1
        th(    "LSDUG");	// 1
        th(    "LSGYU");	// 1
        th(   "LSHUGS");	// 1
        th(      "LSU");	// 1
        th(    "LTAAR");	// 1
        th(  "LTADANG");	// 1
        th(    "LTA'U");	// 1
        th(    "LTE'I");	// 1
        th(    "LTEMS");	// 1
        th(     "LTEN");	// 1
        th(    "LTE'U");	// 1
        th(    "LTING");	// 1
        th(     "LTOB");	// 1
        th(    "LTO'O");	// 1
        th(   "LTOSPA");	// 1
        th(    "LTSUL");	// 1
        th(    "LTU'I");	// 1
        th(   "^LTUNG");	// 1
        th(   "LTUNGB");	// 1
        th(   "LTUNGS");	// 1
        th(    "LTUNN");	// 1
        th(     "LTUS");	// 1
        th(     "LTZA");	// 1
        th(   "LTZAGS");	// 1
        th(      "LU4");	// 1
        th(     "LUAS");	// 1
        th(     "LUGN");	// 1
        th(     "LUGs");	// 1
        th(    "L'U'I");	// 1
        th(     "LUKA");	// 1
        th(      "LUm");	// 1
        th(     "L'UM");	// 1
        th( "LU-MBI'I");	// 1
        th( "LUMB'I'I");	// 1
        th(     "LUMG");	// 1
        th(     "L'UN");	// 1
        th(    "LUndA");	// 1
        th(    "LUNTA");	// 1
        th(   "LU'U'I");	// 1
        th(     "LVAm");	// 1
        th(      "LVU");	// 1
        th(     "LVUN");	// 1
        th(    "LWA'I");	// 1
        th(    "LWANG");	// 1
        th(    "LYAmA");	// 1
        th(     "LYAR");	// 1
        th(      "LYI");	// 1
        th(      "LYU");	// 1
        th(    "LZHES");	// 1
        th(     "LZOG");	// 1
        th(       "M'");	// 1
        th(      "^MA");	// 1
        th(     "M''A");	// 1
        th(      "MA0");	// 1
        th(      "MA2");	// 1
        th(     "MA'A");	// 1
        th(    "MA'AL");	// 1
        th(   "MA:AOM");	// 1
        th(    "MABHI");	// 1
        th(     "M'AD");	// 1
        th(    "MAd'A");	// 1
        th("MA:d'AKKI");	// 1
        th(   "M'ADHA");	// 1
        th(   "MADHYA");	// 1
        th(    "MA'DI");	// 1
        th(    "MAD'I");	// 1
        th(  "MADMIGS");	// 1
        th(    "MAd'U");	// 1
        th(  "M'ADZA:");	// 1
        th( "MA-DZDZA");	// 1
        th( "MADZDZ'A");	// 1
        th( "MADZNY'A");	// 1
        th(     "MA'E");	// 1
        th(  "MAGHI'I");	// 1
        th(   "MAGRUB");	// 1
        th(   "MAH'AR");	// 1
        th(    "MAHdA");	// 1
        th(      "MAI");	// 1
        th("MA'IDNGOS");	// 1
        th(    "MAKAN");	// 1
        th(  "MAKshIm");	// 1
        th(  "MAL'ANG");	// 1
        th(   "MALCHA");	// 1
        th(    "MALUS");	// 1
        th(    "MALWA");	// 1
        th(     "M'AM");	// 1
        th(     "MA'M");	// 1
        th(     "MAmS");	// 1
        th(     "MAMS");	// 1
        th(     "MAn:");	// 1
        th(     "MAnA");	// 1
        th(    "M'AnA");	// 1
        th(    "M'ANA");	// 1
        th(  "MAn'A'I");	// 1
        th(   "MAnA'I");	// 1
        th(    "MAnAL");	// 1
        th(   "MAnd'A");	// 1
        th(   "MAnD'A");	// 1
        th( "MA-n+dAL");	// 1
        th(   "MANDAL");	// 1
        th(   "MAndAN");	// 1
        th(   "MANDAR");	// 1
        th( "MAN-D'AR");	// 1
        th(   "MAnDHA");	// 1
        th(    "MANDO");	// 1
        th(   "MANDZU");	// 1
        th(    "MA'NG");	// 1
        th(    "MANG'");	// 1
        th(   "MANGA'");	// 1
        th(   "MANGAL");	// 1
        th( "MANGALAm");	// 1
        th( "MANG'ANG");	// 1
        th(  "MANG-BA");	// 1
        th( "MA-NG+GA");	// 1
        th( "MANG-G'A");	// 1
        th("MANG-GA'I");	// 1
        th("MANG+GA'I");	// 1
        th("MANG-GA-LAM");	// 1
        th( "MANGHA'I");	// 1
        th(    "MANGL");	// 1
        th(  "MANG'UR");	// 1
        th(     "MANI");	// 1
        th(    "MANJI");	// 1
        th(     "MANL");	// 1
        th(    "MAN'O");	// 1
        th(  "MANP'AR");	// 1
        th(  "MANSA'I");	// 1
        th("M'AN:SVABH'A");	// 1
        th(   "MAntAR");	// 1
        th(   "MANTAR");	// 1
        th(   "MANTAS");	// 1
        th(   "MANthI");	// 1
        th(  "MAN-TRA");	// 1
        th(  "MAN-TRE");	// 1
        th(    "MANTU");	// 1
        th(    "MANYA");	// 1
        th(   "MA-NYA");	// 1
        th(  "MANYADZ");	// 1
        th( "MANY-DZU");	// 1
        th( "MANY+DZU");	// 1
        th( "MANYDZU:");	// 1
        th("MANYDZUGHO");	// 1
        th("MANYDZU'I");	// 1
        th("MANYDZUN'A");	// 1
        th( "MANYNDZU");	// 1
        th(   "MA'O'I");	// 1
        th(    "MA'ON");	// 1
        th(    "MA'OS");	// 1
        th(      "MAP");	// 1
        th(   "MA:PAR");	// 1
        th( "MA:PHYAG");	// 1
        th(      "mAR");	// 1
        th(    "M'ARA");	// 1
        th(  "MA-RD'A");	// 1
        th(   "MARDHA");	// 1
        th(  "M'A-RGA");	// 1
        th(  "MA-RGAD");	// 1
        th(    "MARKA");	// 1
        th(    "MARLA");	// 1
        th("M'ARTHASAm");	// 1
        th(   "MAR:YI");	// 1
        th(      "MAs");	// 1
        th(     "MA'S");	// 1
        th(     "MAS:");	// 1
        th(    "MA:SA");	// 1
        th("MA:SAMANTA");	// 1
        th("MA:SARBBA");	// 1
        th(  "MASHDZI");	// 1
        th( "MA:SHR'I");	// 1
        th(    "MASSA");	// 1
        th("MASSARBBA");	// 1
        th(  "MASSRBA");	// 1
        th(   "MASSVA");	// 1
        th(   "MASTU:");	// 1
        th(    "MA:SU");	// 1
        th(   "MATA'O");	// 1
        th(    "MA:TE");	// 1
        th(   "MATR'A");	// 1
        th( "M'ATZTSA");	// 1
        th(    "MA'UL");	// 1
        th(   "MA-YOG");	// 1
        th(     "MBA'");	// 1
        th(     "MBAR");	// 1
        th(     "MBAS");	// 1
        th(     "MBHE");	// 1
        th(    "MBI'I");	// 1
        th(    "MBRIS");	// 1
        th(    "MBU'I");	// 1
        th(    "MBYON");	// 1
        th(     "MCAD");	// 1
        th(     "MCED");	// 1
        th(     "MCHA");	// 1
        th(    "MCHAL");	// 1
        th(    "MCHEB");	// 1
        th(   "MCHE'U");	// 1
        th(  "MCHI'AM");	// 1
        th(    "MCHIK");	// 1
        th(   "-MCHIS");	// 1
        th(   "MCHOBS");	// 1
        th(   "MCHO'U");	// 1
        th(    "MCHU'");	// 1
        th(    "MCHUB");	// 1
        th(   "MCHUNG");	// 1
        th(   "MCHU'O");	// 1
        th(      "MCI");	// 1
        th(     "MCIL");	// 1
        th(      "MCO");	// 1
        th(     "MCOS");	// 1
        th(     "MCUD");	// 1
        th(    "MCU'I");	// 1
        th(      "MDA");	// 1
        th(     "MDAd");	// 1
        th(     "MDAM");	// 1
        th(    "MDAMS");	// 1
        th(    "MDA'O");	// 1
        th(    "MDASA");	// 1
        th(    "MDA'U");	// 1
        th(   "MDAZAD");	// 1
        th(     "MDES");	// 1
        th(   "MDEZOD");	// 1
        th(     "MDHA");	// 1
        th(    "MDHOD");	// 1
        th(     "MDHU");	// 1
        th(    "MDIGS");	// 1
        th(    "MDI'I");	// 1
        th(     "MDIS");	// 1
        th(     "^MDO");	// 1
        th(    "MDOBS");	// 1
        th(   "-:MDOG");	// 1
        th(     "MDOL");	// 1
        th(     "MDOM");	// 1
        th(    "MDOZD");	// 1
        th(    "MDRAD");	// 1
        th(  "MDRANGS");	// 1
        th(     "MDRI");	// 1
        th(     "MDRO");	// 1
        th(    "MDSAS");	// 1
        th(    "MDUDN");	// 1
        th(   "MDUNAN");	// 1
        th(   "MDUNGS");	// 1
        th(      "MDZ");	// 1
        th( "MDZA'ANG");	// 1
        th(    "MDZAB");	// 1
        th(  "MDZADNA");	// 1
        th(   "MDZADO");	// 1
        th(    "MDZAM");	// 1
        th(    "MDZAN");	// 1
        th(   "MDZA'O");	// 1
        th(     "MDZD");	// 1
        th(    "MDZEG");	// 1
        th(     "MDZI");	// 1
        th(   "MDZI'I");	// 1
        th(    "MDZOB");	// 1
        th(    "MDZOL");	// 1
        th(    "MDZON");	// 1
        th(    "MDZUD");	// 1
        th(    "MDZUN");	// 1
        th(   "MDZUNG");	// 1
        th("MDZUNYA-GDO");	// 1
        th(      "ME2");	// 1
        th(      "ME4");	// 1
        th(    "MEDA'");	// 1
        th( "MEDGZUGS");	// 1
        th(    "MEDPA");	// 1
        th(    "MEDRA");	// 1
        th(    "ME-DU");	// 1
        th(    "MEDYI");	// 1
        th(     "MEED");	// 1
        th(     "MEER");	// 1
        th(   "MEETRI");	// 1
        th(  "ME'I'AM");	// 1
        th(   "ME'I'O");	// 1
        th(    "MEKBA");	// 1
        th(   "MENTRE");	// 1
        th(  "MENYDZU");	// 1
        th(      "MET");	// 1
        th(      "MGA");	// 1
        th(     "MGA'");	// 1
        th(     "MGAD");	// 1
        th(    "MGANG");	// 1
        th(     "MGAS");	// 1
        th(     "MGEN");	// 1
        th(     "MGES");	// 1
        th(     "MG'I");	// 1
        th(     "MGIS");	// 1
        th(     "MGOG");	// 1
        th(    "MGOGS");	// 1
        th(    "MGOMS");	// 1
        th(    "^MGON");	// 1
        th(    "MGONG");	// 1
        th(   "MGONGS");	// 1
        th(    "MGO'O");	// 1
        th(     "MGRA");	// 1
        th(   "MGRAGS");	// 1
        th(    "MGRAN");	// 1
        th(    "MGREN");	// 1
        th(     "MGRI");	// 1
        th(    "MGRIB");	// 1
        th(   "'MGRON");	// 1
        th(   "MGU'AM");	// 1
        th(     "MGYA");	// 1
        th(   "MGYAGS");	// 1
        th(    "MGYES");	// 1
        th(     "MGYI");	// 1
        th(   "MGYIGS");	// 1
        th(    "MGYIS");	// 1
        th(    "MGYON");	// 1
        th(    "MGZAS");	// 1
        th(      "MHA");	// 1
        th(      "'MI");	// 1
        th(     "M''I");	// 1
        th(      "MI'");	// 1
        th(      "MI8");	// 1
        th(   "MI'A'I");	// 1
        th(  "M'I'ANG");	// 1
        th(    "MI'AS");	// 1
        th(      "MIB");	// 1
        th(   "MIBHAR");	// 1
        th(   "MI'GAL");	// 1
        th(    "MIGTA");	// 1
        th(  "MI'GYUR");	// 1
        th(     "MIHA");	// 1
        th(   "MI'I'O");	// 1
        th(    "MI'IR");	// 1
        th(      "MIJ");	// 1
        th( "MI'KHRUL");	// 1
        th(      "MIL");	// 1
        th(     "MI'M");	// 1
        th("MIMBOOshthA:");	// 1
        th(     "'MIN");	// 1
        th(     "M'IN");	// 1
        th(     "MINA");	// 1
        th(   "MINDRA");	// 1
        th(    "M'ING");	// 1
        th(    "MINGA");	// 1
        th(     "MINO");	// 1
        th(  "MI'PHRO");	// 1
        th(   "MIRIGS");	// 1
        th(  "MISSING");	// 1
        th(  "M'ISSVI");	// 1
        th(  "MI'THAD");	// 1
        th(  "MITHEBS");	// 1
        th(  "MI-TRAS");	// 1
        th(    "MITSA");	// 1
        th(   "MITSAN");	// 1
        th(      "MJA");	// 1
        th(     "MJAG");	// 1
        th(    "MJA'I");	// 1
        th(     "MJAM");	// 1
        th(    "MJANG");	// 1
        th(     "MJAS");	// 1
        th(    "MJE'I");	// 1
        th(     "MJEL");	// 1
        th(      "MJI");	// 1
        th(    "MJO'I");	// 1
        th(      "MJU");	// 1
        th(     "MJUm");	// 1
        th(     "MJUS");	// 1
        th(     "MJUT");	// 1
        th(    "MKANG");	// 1
        th(     "MKAS");	// 1
        th(    "MKH'A");	// 1
        th(  "MKHA'AM");	// 1
        th( "MKHA'ANG");	// 1
        th(   "MKH'AI");	// 1
        th(   "MKHANS");	// 1
        th("MKHA'NYID");	// 1
        th(   "'MKHAR");	// 1
        th(   "'MKHAS");	// 1
        th(   "MKHASN");	// 1
        th("MKHA'ZHES");	// 1
        th(   "MKHEBS");	// 1
        th(   "MKHEGS");	// 1
        th(    "MKHEL");	// 1
        th(  "MKHENGS");	// 1
        th(    "MKHES");	// 1
        th(    "MKHO'");	// 1
        th(    "MKHOD");	// 1
        th(    "MKHOM");	// 1
        th(    "MKHRA");	// 1
        th(   "MKHRAS");	// 1
        th(   "MKHREG");	// 1
        th(  "MKHRENG");	// 1
        th(   "MKHRID");	// 1
        th(   "MKHRO'");	// 1
        th(   "MKHROL");	// 1
        th(   "MKHROS");	// 1
        th(   "MKHRUG");	// 1
        th(   "MKHRUL");	// 1
        th(  "MKHY7UD");	// 1
        th(   "MKHYA'");	// 1
        th( "MKHYENDA");	// 1
        th(    "MKHYU");	// 1
        th(  "MKHYUNG");	// 1
        th(     "MKUN");	// 1
        th(   "MKYANG");	// 1
        th(   "MKYHEN");	// 1
        th(      "MLA");	// 1
        th(     "ML'A");	// 1
        th(  "MLAshtI");	// 1
        th(     "MLBA");	// 1
        th(      "MMA");	// 1
        th(     "MMAN");	// 1
        th(      "MmD");	// 1
        th(      "MME");	// 1
        th(     "MMED");	// 1
        th(      "MMI");	// 1
        th(     "MMYU");	// 1
        th(      "MNA");	// 1
        th(     "MnAL");	// 1
        th(    "MN'Am");	// 1
        th(   "MNANNA");	// 1
        th(     "MndA");	// 1
        th(    "MNdAL");	// 1
        th(    "MNDAL");	// 1
        th(     "MNDO");	// 1
        th(      "MnE");	// 1
        th(  "MNGA'AS");	// 1
        th(   "MNGABS");	// 1
        th(   "MNGAGA");	// 1
        th(    "MNGAt");	// 1
        th(     "MNGL");	// 1
        th(    "MNGOG");	// 1
        th(   "MNGOMS");	// 1
        th(   "^MNGON");	// 1
        th(  "MNGVAGS");	// 1
        th(      "MNI");	// 1
        th(     "MNIL");	// 1
        th(   "M'NN'A");	// 1
        th(    "MNOBS");	// 1
        th(    "MNOGN");	// 1
        th(    "M'NTA");	// 1
        th(    "MNYAD");	// 1
        th(  "MNYADZU");	// 1
        th( "MNYA+DZU");	// 1
        th(   "MNYAMN");	// 1
        th(   "MNYAMs");	// 1
        th(    "MNYAn");	// 1
        th(  "MNYANGS");	// 1
        th(    "MNYAR");	// 1
        th("MNYA-RDZU");	// 1
        th(   "MNYDZA");	// 1
        th(  "MNYE'AM");	// 1
        th(   "MNYE'I");	// 1
        th(   "MNYONG");	// 1
        th(    "MNYUN");	// 1
        th(      "MO:");	// 1
        th(      "MO3");	// 1
        th("MODHARM'I");	// 1
        th(     "MODR");	// 1
        th(     "MODS");	// 1
        th(     "MOGN");	// 1
        th(  "MOKsh'A");	// 1
        th(  "MOKsHRA");	// 1
        th(      "MOM");	// 1
        th(    "MO'MA");	// 1
        th(     "M'ON");	// 1
        th(    "MO'NG");	// 1
        th(  "MONG+GA");	// 1
        th( "MONG+GAL");	// 1
        th(   "MONGNS");	// 1
        th(    "MONGs");	// 1
        th(   "MONGSU");	// 1
        th(     "MONS");	// 1
        th(    "M'O'O");	// 1
        th(   "MOOKTI");	// 1
        th("MOO-NGA-GAL");	// 1
        th("MOONG-GAL");	// 1
        th(  "MOONGGL");	// 1
        th(    "MOTTA");	// 1
        th(    "MO'UM");	// 1
        th(    "MP0OR");	// 1
        th(     "MPAR");	// 1
        th(     "MPAS");	// 1
        th(    "MPHAS");	// 1
        th(  "MPHYOGS");	// 1
        th(      "MPO");	// 1
        th(     "MRA:");	// 1
        th(     "MRA'");	// 1
        th(     "MRAD");	// 1
        th(     "MRDA");	// 1
        th(    "MRDUL");	// 1
        th(   "MRDZAB");	// 1
        th(  "MRDZA'I");	// 1
        th( "MRDZANGS");	// 1
        th(   "MRDZAS");	// 1
        th(    "MRDZU");	// 1
        th(   "MRDZUN");	// 1
        th(     "MRGA");	// 1
        th(    "MRG'A");	// 1
        th(    "MRGAS");	// 1
        th(    "MRGYA");	// 1
        th(     "MRid");	// 1
        th(     "MRID");	// 1
        th(    "MRidA");	// 1
        th(     "MRIG");	// 1
        th(   "MRishA");	// 1
        th(   "MRIshA");	// 1
        th(    "MRJAD");	// 1
        th(    "MRKAD");	// 1
        th(   "MRKANG");	// 1
        th(   "MRNAMS");	// 1
        th(   "MRNGA'");	// 1
        th(     "MROG");	// 1
        th(   "MRONGS");	// 1
        th(    "MRTOG");	// 1
        th(   "MRTOGS");	// 1
        th(    "MRTOL");	// 1
        th(  "MRTZE'I");	// 1
        th(  "MRTZIGS");	// 1
        th(     "MSAD");	// 1
        th(     "MSEL");	// 1
        th(   "MSGRIB");	// 1
        th(   "MSHDZI");	// 1
        th(    "MSKOR");	// 1
        th(    "M'SMA");	// 1
        th(   "MSNGON");	// 1
        th(     "MSON");	// 1
        th(     "MSOS");	// 1
        th(   "MSTENG");	// 1
        th(   "MSUNGS");	// 1
        th(   "MT6HUN");	// 1
        th(    "MTAH'");	// 1
        th(     "MTAL");	// 1
        th(    "MTANG");	// 1
        th(     "MTAR");	// 1
        th(   "MTGHUS");	// 1
        th(   "MTH''A");	// 1
        th(  "MTHA'AM");	// 1
        th( "MTHA'A'O");	// 1
        th(  "MTHA'ES");	// 1
        th("MTHA'GNYIS");	// 1
        th("MTHANGGOS");	// 1
        th(   "MTHE'A");	// 1
        th(   "MTHE'I");	// 1
        th(    "MTHI'");	// 1
        th(    "MTHID");	// 1
        th( "MTHI'I'O");	// 1
        th(    "MTHIR");	// 1
        th(    "MTH'O");	// 1
        th(  "MTHO'AM");	// 1
        th( "MTHO'ANG");	// 1
        th(   "^MTHOL");	// 1
        th(   ":MTHON");	// 1
        th("MTHONGDMIGS");	// 1
        th("MTHONGSRID");	// 1
        th(     "MTHR");	// 1
        th(    "MTHU'");	// 1
        th(    "MTHUE");	// 1
        th(   "MTHUGS");	// 1
        th(   "^MTHUN");	// 1
        th(  "MTHUNGS");	// 1
        th(   "MTHUNI");	// 1
        th(  "MTHUNPA");	// 1
        th("MTHUNPA'I");	// 1
        th(     "MTIS");	// 1
        th(   "MTMSAN");	// 1
        th(    "MTNON");	// 1
        th(   "m:TR'A");	// 1
        th(      "MTS");	// 1
        th(   "MTSAAN");	// 1
        th(   "MTSABS");	// 1
        th(   "MTSAN:");	// 1
        th( "MTSAN-01");	// 1
        th("MTSANNYID");	// 1
        th(   "MTSASM");	// 1
        th(  "MTSENGS");	// 1
        th(    "MTSID");	// 1
        th(   "MTSIGS");	// 1
        th(    "MTSIN");	// 1
        th(   "MTSMAS");	// 1
        th(    "MTSMS");	// 1
        th(    "MTSNA");	// 1
        th(   "MTSNGS");	// 1
        th(    "MTSO'");	// 1
        th(    "MTSO0");	// 1
        th( "MTSO'O'I");	// 1
        th(    "MTSOt");	// 1
        th(    "MTSUD");	// 1
        th(   "MTSUDS");	// 1
        th(    "MTSUG");	// 1
        th(   "MTSUMS");	// 1
        th(    "MTSUS");	// 1
        th(   "MTTHUN");	// 1
        th(    "MTWAN");	// 1
        th(  "MTYHONG");	// 1
        th(    "MTZAD");	// 1
        th(    "MTZAL");	// 1
        th(   "MTZAMS");	// 1
        th(    "MTZER");	// 1
        th(    "MTZON");	// 1
        th(   "MTZONG");	// 1
        th(      "MU4");	// 1
        th(      "MUB");	// 1
        th(     "MUDN");	// 1
        th(    "MUDPA");	// 1
        th(  "MUGDH'A");	// 1
        th(   "MU-KKE");	// 1
        th(    "MUKTA");	// 1
        th(   "MU-KTA");	// 1
        th(    "MUKTE");	// 1
        th(   "MU-KTE");	// 1
        th(   "MU:LHA");	// 1
        th(    "MU:ME");	// 1
        th(    "MUndA");	// 1
        th(    "MUndI");	// 1
        th(  "MUNdI'I");	// 1
        th(   "MUNGGA");	// 1
        th(     "MUnI");	// 1
        th("MUN'INBHA");	// 1
        th(    "MUntA");	// 1
        th(    "MUNtI");	// 1
        th(    "MUNYA");	// 1
        th("MUNYDZA'I");	// 1
        th(  "MUNYDZU");	// 1
        th( "MUNYTZAR");	// 1
        th(  "MU-NYZA");	// 1
        th(    "M'U'O");	// 1
        th(   "MURshA");	// 1
        th(    "MUshA");	// 1
        th(   "MUshdI");	// 1
        th(   "MUshIT");	// 1
        th(  "MUshMIS");	// 1
        th(   "MUshtI");	// 1
        th(      "MUT");	// 1
        th(     "MUTA");	// 1
        th(   "MUT-KA");	// 1
        th(   "MU-TKU");	// 1
        th(    "MUTTA");	// 1
        th("MUTZTSA'AM");	// 1
        th("MUTZTSA'I");	// 1
        th(  "MUTZTSO");	// 1
        th(    "MU:YE");	// 1
        th(      "MWR");	// 1
        th(     "MY'A");	// 1
        th(    "MY'A:");	// 1
        th(  "MYAKSAm");	// 1
        th(    "MY'Am");	// 1
        th(   "MYANTA");	// 1
        th(    "MYE'I");	// 1
        th(    "MYENG");	// 1
        th(      "MYI");	// 1
        th(     "MYID");	// 1
        th(    "MYI'I");	// 1
        th(     "MYIN");	// 1
        th(    "MYIUR");	// 1
        th(     "MYOB");	// 1
        th(    "MYOGS");	// 1
        th(   "MYONMG");	// 1
        th(     "MYOR");	// 1
        th(    "MYUGS");	// 1
        th(    "MYUNR");	// 1
        th(    "'MYUR");	// 1
        th(    "MYUYR");	// 1
        th(    "MZDAD");	// 1
        th(     "MZHI");	// 1
        th(    "MZHIN");	// 1
        th(     "MZHU");	// 1
        th(     "MZIN");	// 1
        th(     "MZOD");	// 1
        th(       "'N");	// 1
        th(       "N3");	// 1
        th(      "NA1");	// 1
        th(    "NA'AI");	// 1
        th(   "NA'ANC");	// 1
        th(   "nA'ANG");	// 1
        th(  "NA'ANGS");	// 1
        th(      "NAd");	// 1
        th(    "NA'DI");	// 1
        th(     "NADS");	// 1
        th(   "NADY'A");	// 1
        th(      "NAE");	// 1
        th(    "N'AGA");	// 1
        th(     "NAGI");	// 1
        th(    "NAGLI");	// 1
        th(     "NAHA");	// 1
        th(    "n'A'I");	// 1
        th(     "NA'i");	// 1
        th(     "N'AI");	// 1
        th( "nA'I'ANG");	// 1
        th(   "NA'JUD");	// 1
        th(      "NAK");	// 1
        th(   "NA'KRI");	// 1
        th(      "nAL");	// 1
        th("N'ALANtARA");	// 1
        th( "NALANt'I");	// 1
        th("N'ALENDRAR");	// 1
        th(     "NA'M");	// 1
        th(    "N'AMA");	// 1
        th(    "NAMED");	// 1
        th(     "NAMG");	// 1
        th("NAM-KHA'I");	// 1
        th(   "N'AMNA");	// 1
        th("NAM:SAMANTA");	// 1
        th("NAM:SARBA");	// 1
        th( "NAMSSVAR");	// 1
        th(   "NAM:SU");	// 1
        th(     "N'AN");	// 1
        th(    "nAn'A");	// 1
        th(    "NANdA");	// 1
        th(   "NAndAL");	// 1
        th(  "NANDESA");	// 1
        th( "NANDHASA");	// 1
        th(   "NANDZA");	// 1
        th(     "nANG");	// 1
        th(    "^NANG");	// 1
        th(   "NA'NGA");	// 1
        th(   "NANGDU");	// 1
        th(   "NANHYO");	// 1
        th( "NANTA'AM");	// 1
        th(   "NANTAR");	// 1
        th(  "NANTI'O");	// 1
        th(    "NANYA");	// 1
        th( "NANYDZ'O");	// 1
        th(    "nA:'O");	// 1
        th(   "NA'O'I");	// 1
        th(   "NA'ONG");	// 1
        th(    "NA'OR");	// 1
        th( "NA'OZHES");	// 1
        th(     "N'AR");	// 1
        th(   "NARBHA");	// 1
        th(     "NARE");	// 1
        th(  "nARnnAm");	// 1
        th(  "NARnnAM");	// 1
        th(   "NARTSA");	// 1
        th(     "NAS:");	// 1
        th(     "NASD");	// 1
        th(  "NASGYUD");	// 1
        th("NASHKARTA");	// 1
        th(   "NAshtE");	// 1
        th(  "NASHTZO");	// 1
        th(    "nAS'I");	// 1
        th(     "NASS");	// 1
        th(    "NASUS");	// 1
        th("NAS+YAS'URLASHR'IBHADROBIHARATISMA:");	// 1
        th(     "NATI");	// 1
        th(    "NAT'I");	// 1
        th(    "nAT'O");	// 1
        th(    "NATRA");	// 1
        th(   "NA-ttE");	// 1
        th( "N'ATZTSA");	// 1
        th(     "NA'U");	// 1
        th(    "nA:YE");	// 1
        th(     "NAYN");	// 1
        th(      "NAZ");	// 1
        th(   "NBIKKA");	// 1
        th(     "NBYA");	// 1
        th(    "NBYAS");	// 1
        th(    "NCHOS");	// 1
        th(       "nd");	// 1
        th(  "nDAKsHE");	// 1
        th(     "ndAL");	// 1
        th(      "NDE");	// 1
        th(    "NDE'I");	// 1
        th(     "NDHE");	// 1
        th(     "NDHO");	// 1
        th(     "NDHU");	// 1
        th(      "NdI");	// 1
        th(    "ndI'I");	// 1
        th(    "NDI'I");	// 1
        th(  "ndiN+YA");	// 1
        th(    "N'DOD");	// 1
        th(    "ND'RA");	// 1
        th(    "NDR'A");	// 1
        th(    "NDRA:");	// 1
        th(     "NDU:");	// 1
        th(      "NE:");	// 1
        th(      "NE2");	// 1
        th(      "NE4");	// 1
        th(   "NE'ANG");	// 1
        th(      "NEB");	// 1
        th(     "nEE:");	// 1
        th(     "NEER");	// 1
        th(     "nE'I");	// 1
        th(    "N'E'I");	// 1
        th(      "nER");	// 1
        th( "NERNYDZA");	// 1
        th(      "NEY");	// 1
        th(    "nE+YA");	// 1
        th(    "N'GA'");	// 1
        th(     "NGA2");	// 1
        th(     "NGA3");	// 1
        th(     "NGA4");	// 1
        th(   "NGA'AI");	// 1
        th(    "NGAG0");	// 1
        th(    "NGAGI");	// 1
        th(     "NGAK");	// 1
        th(    "^NGAL");	// 1
        th(    "'NGAL");	// 1
        th(     "NGAm");	// 1
        th(    "'NGAM");	// 1
        th(    "^NGAN");	// 1
        th("NGAnd'I'I");	// 1
        th( "NGAntI'I");	// 1
        th(   "NG'A'O");	// 1
        th(  "NG'ASTI");	// 1
        th(    "NGCOD");	// 1
        th(     "NG'E");	// 1
        th(    "NGE3S");	// 1
        th(     "NGE4");	// 1
        th(     "NGEB");	// 1
        th(     "NGEG");	// 1
        th(   "'NGEGS");	// 1
        th(    "NGEMS");	// 1
        th(    "'NGEN");	// 1
        th( "NGENG+HA");	// 1
        th(    "'NGES");	// 1
        th(   "NGE-SO");	// 1
        th(   "NGESPA");	// 1
        th(      "NGG");	// 1
        th(  "NG+GANG");	// 1
        th(  "NG+GE'I");	// 1
        th(    "NG+GI");	// 1
        th(     "NGGU");	// 1
        th(    "NGHA'");	// 1
        th(  "NGH'A'I");	// 1
        th(    "NGHAR");	// 1
        th(   "NGHI'O");	// 1
        th(    "NGHIR");	// 1
        th(    "NGHIS");	// 1
        th(    "NGHMA");	// 1
        th(   "NGHOO:");	// 1
        th(    "NGHRA");	// 1
        th(     "NG'I");	// 1
        th(    "NGI'A");	// 1
        th(    "NGI'O");	// 1
        th(     "NGKE");	// 1
        th(     "NGNA");	// 1
        th(    "NGNAN");	// 1
        th(    "NGNAS");	// 1
        th(     "NGNG");	// 1
        th(   "NGNGHA");	// 1
        th(     "NGO:");	// 1
        th(     "NGO3");	// 1
        th(     "NGO4");	// 1
        th(     "NGOB");	// 1
        th(    "NGOBO");	// 1
        th(   "'NGOGS");	// 1
        th(    "NGONG");	// 1
        th(   "'NGONG");	// 1
        th(    "NGONS");	// 1
        th(     "NGOO");	// 1
        th(     "NGSO");	// 1
        th(    "NGSOL");	// 1
        th(    "NGSUM");	// 1
        th(    "NGTIR");	// 1
        th(    "NGT'U");	// 1
        th(     "NGUB");	// 1
        th(     "NGUG");	// 1
        th(    "'NGUL");	// 1
        th(     "NGUM");	// 1
        th(    "NGUNG");	// 1
        th(   "NGU'NG");	// 1
        th(    "NG'UR");	// 1
        th(    "'NGUS");	// 1
        th(     "NGVA");	// 1
        th(      "n'i");	// 1
        th(      "N+I");	// 1
        th(      "NI2");	// 1
        th(     "NI'A");	// 1
        th(    "nI'AM");	// 1
        th(     "NIAS");	// 1
        th(    "NIB'A");	// 1
        th(    "NIBAR");	// 1
        th(  "NI'BRAS");	// 1
        th(  "NI'BREG");	// 1
        th(  "NIBRJED");	// 1
        th(     "nIdA");	// 1
        th(    "NIDGE");	// 1
        th( "NIDHUtRA");	// 1
        th(   "NIDR'A");	// 1
        th(   "NIDRUG");	// 1
        th(     "NIES");	// 1
        th(     "'NIG");	// 1
        th(  "NIGZUNG");	// 1
        th(     "nIHA");	// 1
        th(     "NIID");	// 1
        th(  "NI'JIGS");	// 1
        th(   "NI-KTA");	// 1
        th(    "NINDI");	// 1
        th(   "NINDRA");	// 1
        th(  "N'INDRA");	// 1
        th( "N'INDR'A");	// 1
        th(   "NINGES");	// 1
        th(   "nINI'O");	// 1
        th(    "NIn'O");	// 1
        th(     "nI'O");	// 1
        th(      "NIO");	// 1
        th(     "NIPA");	// 1
        th(    "NIRBA");	// 1
        th(   "NIRB'A");	// 1
        th(   "NIRBHA");	// 1
        th(   "NIRDYA");	// 1
        th(   "NIRGOL");	// 1
        th(   "NIRKAN");	// 1
        th(     "N'IS");	// 1
        th(   "NIsh'A");	// 1
        th(  "NIshADA");	// 1
        th("NIshsh'A'O");	// 1
        th("NIshthA'I");	// 1
        th("NIshthA'O");	// 1
        th(  "NIshV'A");	// 1
        th(    "NISKA");	// 1
        th(    "NISKU");	// 1
        th(    "NITAS");	// 1
        th(    "N'K'A");	// 1
        th(     "NKYI");	// 1
        th(    "NLHAN");	// 1
        th(      "N'm");	// 1
        th(     "N'MA");	// 1
        th(     "NMAS");	// 1
        th(   "NMCHOD");	// 1
        th(      "NME");	// 1
        th(     "NMED");	// 1
        th(      "NMI");	// 1
        th(    "NMYAL");	// 1
        th(      "n'N");	// 1
        th(     "NNAG");	// 1
        th(     "NNAM");	// 1
        th(   "NNDA'I");	// 1
        th(      "NnE");	// 1
        th(    "NNGOS");	// 1
        th(      "nnI");	// 1
        th(     "NNTA");	// 1
        th(   "nNTI'O");	// 1
        th(     "NNYA");	// 1
        th(   "NNYA'I");	// 1
        th(    "NNYAN");	// 1
        th(  "N'NYTZA");	// 1
        th(     "N''O");	// 1
        th(      "NO3");	// 1
        th(      "NOA");	// 1
        th(      "NOL");	// 1
        th(      "nOO");	// 1
        th(    "NOO'O");	// 1
        th(     "N'OR");	// 1
        th(   "NOTA'O");	// 1
        th(   "NPA'AM");	// 1
        th(     "NPAS");	// 1
        th(    "NPHAB");	// 1
        th(   "NPHUNG");	// 1
        th(      "NPO");	// 1
        th(     "NRAM");	// 1
        th(     "NRAR");	// 1
        th(     "NRAS");	// 1
        th(      "NRE");	// 1
        th(      "nRi");	// 1
        th(     "NRIG");	// 1
        th( "NRi-tt'A");	// 1
        th( "NRiTZTSA");	// 1
        th(    "NRTAG");	// 1
        th(    "NRTZA");	// 1
        th(   "NRTZIS");	// 1
        th(    "NRUNG");	// 1
        th(   "NSGRUB");	// 1
        th(    "NsHAm");	// 1
        th(      "NSO");	// 1
        th(     "NSTE");	// 1
        th(      "ntA");	// 1
        th(     "N+TA");	// 1
        th(     "NT'A");	// 1
        th(     "ntAI");	// 1
        th(    "ntA'I");	// 1
        th(     "NTAm");	// 1
        th(     "NTAN");	// 1
        th(     "ntEE");	// 1
        th(     "NTHA");	// 1
        th(   "NTHABS");	// 1
        th(    "NTHUN");	// 1
        th(   "NTI'AM");	// 1
        th(    "NTI'I");	// 1
        th(    "NTISA");	// 1
        th(  "NTKRITA");	// 1
        th(     "NTRA");	// 1
        th(    "NTSAD");	// 1
        th(    "NTSIG");	// 1
        th(    "NTU'O");	// 1
        th(      "N'U");	// 1
        th(      "NU4");	// 1
        th(     "NUDS");	// 1
        th(   "NU'I'O");	// 1
        th(      "nUm");	// 1
        th(    "NUTTA");	// 1
        th(   "NU-TTA");	// 1
        th(    "NUYON");	// 1
        th(      "NVI");	// 1
        th(     "NV'O");	// 1
        th(       "NY");	// 1
        th(    "NY6ES");	// 1
        th(    "NY6ID");	// 1
        th(    "n+Y'A");	// 1
        th(    "NYA''");	// 1
        th(     "NYA2");	// 1
        th(     "NYA3");	// 1
        th(  "NYA'ANG");	// 1
        th(    "NYABS");	// 1
        th(   "NYADZA");	// 1
        th(   "NYAGRO");	// 1
        th(    "NYAJS");	// 1
        th(     "nYAM");	// 1
        th(    "N+YAm");	// 1
        th(   "N+Y'Am");	// 1
        th(    "NY'Am");	// 1
        th(   "N+Y'AM");	// 1
        th(   "N+YANG");	// 1
        th(   "NYANGS");	// 1
        th(    "NYANM");	// 1
        th(  "NYANNTA");	// 1
        th(   "NYANSA");	// 1
        th( "NYANTHOS");	// 1
        th(    "nYA'O");	// 1
        th(    "N+YAR");	// 1
        th(    "N-YAS");	// 1
        th(   "NYATZI");	// 1
        th(    "NYA'U");	// 1
        th(     "NYBI");	// 1
        th(     "NYCA");	// 1
        th(    "NYDAZ");	// 1
        th(   "NYDZ'A");	// 1
        th(   "NYDZHA");	// 1
        th(    "NYDZI");	// 1
        th(     "'NYE");	// 1
        th(     "NYE2");	// 1
        th(     "NYE4");	// 1
        th(    "NYEES");	// 1
        th(    "NYEID");	// 1
        th(    "^NYES");	// 1
        th(  "NYESHAG");	// 1
        th(     "^NYI");	// 1
        th(     "N+YI");	// 1
        th(     "NY'I");	// 1
        th(     "NYI2");	// 1
        th(     "NYI4");	// 1
        th(    "NYI8D");	// 1
        th(     "NYiD");	// 1
        th(    "'NYID");	// 1
        th(  "NYIDANG");	// 1
        th(  "NYIDBDE");	// 1
        th("NYIDBSTAN");	// 1
        th(    "NYIDD");	// 1
        th(  "NYIDMED");	// 1
        th(   "n+YI'I");	// 1
        th(  "NYIshVA");	// 1
        th(     "NYLA");	// 1
        th(     "NYmS");	// 1
        th(    "NYOGS");	// 1
        th(     "NYRA");	// 1
        th(      "NYS");	// 1
        th(   "NYSH'A");	// 1
        th(    "NYSHE");	// 1
        th(     "NYSI");	// 1
        th(   "NYTZAN");	// 1
        th(     "NYU4");	// 1
        th(  "NYUBDZA");	// 1
        th(    "NYUID");	// 1
        th(     "NYUN");	// 1
        th(    "n+YUt");	// 1
        th(    "NYVAR");	// 1
        th(   "NZHING");	// 1
        th(      "'O3");	// 1
        th(       "OA");	// 1
        th(    "OAJTA");	// 1
        th(      "OBA");	// 1
        th(    "'OBAS");	// 1
        th(    "OCHAD");	// 1
        th(     "^'OD");	// 1
        th(     "'OD'");	// 1
        th(    "ODANG");	// 1
        th(    "'ODAS");	// 1
        th(     "'ODD");	// 1
        th(     "ODON");	// 1
        th(    "ODRIL");	// 1
        th(     "'ODS");	// 1
        th(    "'OENG");	// 1
        th(    "OGONG");	// 1
        th(    "'OGRA");	// 1
        th(   "OGZUNG");	// 1
        th(      "O'I");	// 1
        th(      "OLA");	// 1
        th(     "OLAS");	// 1
        th(     "OLTA");	// 1
        th(      "OMA");	// 1
        th(   "OMTHAR");	// 1
        th(   "OMTSAN");	// 1
        th(      "ONE");	// 1
        th(   "'ONGBA");	// 1
        th(     "'ONK");	// 1
        th(     "ONYE");	// 1
        th(    "'O'OS");	// 1
        th(       "OP");	// 1
        th(      "OPO");	// 1
        th(  "ORDZOGS");	// 1
        th(      "O'U");	// 1
        th(     "OUNG");	// 1
        th(       "P'");	// 1
        th(     "P0AS");	// 1
        th(      "P0O");	// 1
        th(      "'PA");	// 1
        th(     "P'A:");	// 1
        th(      "PA0");	// 1
        th(     "PA'0");	// 1
        th(     "PA0R");	// 1
        th(      "PA1");	// 1
        th(      "PA2");	// 1
        th(      "PA4");	// 1
        th(     "PA5R");	// 1
        th(    "PAA'I");	// 1
        th(    "PAARA");	// 1
        th(    "PABHA");	// 1
        th(  "P'ABYAm");	// 1
        th(    "PACAN");	// 1
        th(     "PAD-");	// 1
        th(    "PADAG");	// 1
        th(    "PADAM");	// 1
        th(   "PADANG");	// 1
        th("PAD-DMA'I");	// 1
        th(     "PADE");	// 1
        th(   "PADE'I");	// 1
        th(    "PADHA");	// 1
        th(   "PADH'A");	// 1
        th(     "PAdI");	// 1
        th(     "PADI");	// 1
        th(   "P'AD'I");	// 1
        th(   "PADMA'");	// 1
        th( "PADMA'AM");	// 1
        th("PADMA'ANG");	// 1
        th( "PA-DMA'I");	// 1
        th(   "PADMAL");	// 1
        th(   "PADMAm");	// 1
        th(  "PAD+MAR");	// 1
        th(  "PAD-MED");	// 1
        th(   "PADM'I");	// 1
        th("PADMO'ANG");	// 1
        th( "PAD+MO'I");	// 1
        th(  "PADMO'O");	// 1
        th("PADMOshnI");	// 1
        th(    "PAdnI");	// 1
        th(   "PA'DUS");	// 1
        th(      "PAE");	// 1
        th(     "'PAG");	// 1
        th(    "PAHA'");	// 1
        th(     "PAHN");	// 1
        th(    "'PA'I");	// 1
        th(     "P'AI");	// 1
        th( "PA'I'ANG");	// 1
        th(    "PA'IC");	// 1
        th(    "PA'ID");	// 1
        th(  "PA'IDON");	// 1
        th(  "PA'I'DU");	// 1
        th(    "PA'IG");	// 1
        th(    "PA'IJ");	// 1
        th(    "PA'Im");	// 1
        th(   "PA'IMI");	// 1
        th(  "PA'IMIG");	// 1
        th("PA'I'PHAGS");	// 1
        th( "PA'IRGYA");	// 1
        th( "PA'IRGYU");	// 1
        th("PA'ISBYOR");	// 1
        th( "PA'ISEMS");	// 1
        th( "PA'IZHES");	// 1
        th( "PA'KHRIG");	// 1
        th(    "PA'KI");	// 1
        th(   "PAKshA");	// 1
        th(   "PAKshI");	// 1
        th( "PAKshU'O");	// 1
        th(   "PALA'O");	// 1
        th(     "PALG");	// 1
        th(    "P'ALI");	// 1
        th(   "PALL'A");	// 1
        th(    "PALOG");	// 1
        th(     "P'Am");	// 1
        th(   "PAMAPA");	// 1
        th(    "PAMED");	// 1
        th(     "PAMI");	// 1
        th(     "P'An");	// 1
        th(     "PA-n");	// 1
        th(     "PANA");	// 1
        th(  "PAndA'I");	// 1
        th(    "PAndE");	// 1
        th(  "PAN-DHA");	// 1
        th(   "PANDHE");	// 1
        th(  "PA-n+dI");	// 1
        th(   "PAn+dI");	// 1
        th(    "PANDI");	// 1
        th(   "PAndIT");	// 1
        th("PAndITA'I");	// 1
        th( "PAndITAS");	// 1
        th(   "PAndTI");	// 1
        th(    "PAndU");	// 1
        th(    "'PANG");	// 1
        th(   "PANGGA");	// 1
        th("PANG-KI'I");	// 1
        th(   "'PANGS");	// 1
        th(     "PAnI");	// 1
        th(    "PA'nI");	// 1
        th(    "PAnIt");	// 1
        th(    "PAN'O");	// 1
        th(    "PAntE");	// 1
        th(   "PANthI");	// 1
        th(   "PAn-tI");	// 1
        th(    "PAnTI");	// 1
        th( "PAntITAS");	// 1
        th(   "PANYID");	// 1
        th( "PANYTZ'A");	// 1
        th("PA-NY+TZI");	// 1
        th(    "PA'OD");	// 1
        th(    "PA'OK");	// 1
        th(   "PA'O'O");	// 1
        th(    "PA'PA");	// 1
        th(    "PAPAS");	// 1
        th( "PAPATI'O");	// 1
        th(    "PAPNA");	// 1
        th(     "PAR:");	// 1
        th(     "PARA");	// 1
        th(    "P'ARA");	// 1
        th(  "PARATNA");	// 1
        th(  "PARGYUR");	// 1
        th(  "PARHA'I");	// 1
        th( "PAR'IKsh");	// 1
        th(    "PARMA");	// 1
        th(    "PARNA");	// 1
        th(   "PARNAM");	// 1
        th(  "PARNAMS");	// 1
        th("PARONYAshA'");	// 1
        th(    "PARPA");	// 1
        th(  "PARRGYU");	// 1
        th( "PARSBYOR");	// 1
        th(   "PARSHA");	// 1
        th(  "PAR-SHA");	// 1
        th(  "PAR-SHI");	// 1
        th( "PARTHIB:");	// 1
        th(    "PARTI");	// 1
        th(   "PARY'A");	// 1
        th(   "PARYIN");	// 1
        th(     "P'AS");	// 1
        th("PASBSGRUB");	// 1
        th(   "PASH'A");	// 1
        th(  "PAshTZA");	// 1
        th(     "PASR");	// 1
        th(  "PASSMON");	// 1
        th(  "PASTH'A");	// 1
        th(    "PASVA");	// 1
        th(     "P'At");	// 1
        th(     "PATA");	// 1
        th(    "PA-TA");	// 1
        th(   "PAtASA");	// 1
        th(  "PATHAMS");	// 1
        th(    "PATSA");	// 1
        th(    "PATSE");	// 1
        th(   "PAT+TI");	// 1
        th(   "PATTI:");	// 1
        th(   "PAtVOO");	// 1
        th(   "PATYAm");	// 1
        th(  "PA'TYAm");	// 1
        th(  "PATZTSA");	// 1
        th(   "PATZVA");	// 1
        th(     "PA'U");	// 1
        th(   "PAYANG");	// 1
        th(  "PA'YANG");	// 1
        th(   "PAZHES");	// 1
        th(    "PBA'I");	// 1
        th(    "PBA'O");	// 1
        th(  "PBSGYUR");	// 1
        th(    "PBUNG");	// 1
        th(     "PBYA");	// 1
        th(     "PDAM");	// 1
        th(    "PDGOS");	// 1
        th(   "PDM'AI");	// 1
        th(   "PDMA'I");	// 1
        th(    "PDMAR");	// 1
        th(     "PDMO");	// 1
        th(     "PDOM");	// 1
        th(   "PDOM'I");	// 1
        th(     "PDRA");	// 1
        th(      "PDU");	// 1
        th(      "PE2");	// 1
        th(      "PE4");	// 1
        th(     "PEBS");	// 1
        th(      "PED");	// 1
        th(     "P'EE");	// 1
        th(      "PEL");	// 1
        th(      "PEM");	// 1
        th(    "PENDA");	// 1
        th(     "PENG");	// 1
        th(    "PGZHI");	// 1
        th(      "'PH");	// 1
        th(     "PH'2");	// 1
        th(     "PHA2");	// 1
        th("PHALAGAnANASAMAYABIHARTISMA");	// 1
        th(   "PHALGU");	// 1
        th(    "PHAnI");	// 1
        th(    "PHARG");	// 1
        th(   "PHE'AM");	// 1
        th(    "'PHED");	// 1
        th(    "PHEEB");	// 1
        th(     "PHEm");	// 1
        th(   "'PHENG");	// 1
        th(     "PHES");	// 1
        th(     "PH'I");	// 1
        th(    "'PHIB");	// 1
        th(    "PHIIR");	// 1
        th(    "PH'IN");	// 1
        th(   "'PHING");	// 1
        th(     "PHIS");	// 1
        th(     "PHMA");	// 1
        th(   "PHNYIR");	// 1
        th(     "PHO3");	// 1
        th(  "'PHO'AM");	// 1
        th(    "PHOBs");	// 1
        th(   "'PHOBS");	// 1
        th(   "PHOBSA");	// 1
        th(    "PHODS");	// 1
        th(  "PHO'I'O");	// 1
        th(     "PHOm");	// 1
        th(     "PHOM");	// 1
        th(  "^PHONGS");	// 1
        th(     "PHOO");	// 1
        th(   "PHRABS");	// 1
        th(   "^PHRAN");	// 1
        th(  "PHRANGS");	// 1
        th(   "PHRAWN");	// 1
        th("PHRDZNY'A");	// 1
        th(   "PHREGS");	// 1
        th(  "'PHREGS");	// 1
        th(  "PHREING");	// 1
        th(    "PHREm");	// 1
        th( "PHRE'U'O");	// 1
        th( "PHRI'ANG");	// 1
        th(    "PHRIB");	// 1
        th(   "PHRIGS");	// 1
        th(   "'PHRIM");	// 1
        th(  "'PHRImS");	// 1
        th(  "'PHRIN0");	// 1
        th(    "'PHRL");	// 1
        th(   "'PHROB");	// 1
        th(  "'PHROBS");	// 1
        th(  "'PHROGD");	// 1
        th("PHROGPA'AM");	// 1
        th( "PHROKshA");	// 1
        th(    "PHRON");	// 1
        th(   "'PHRON");	// 1
        th( "'PHRONGS");	// 1
        th(    "PHROR");	// 1
        th(    "PHRUB");	// 1
        th(    "PHRUD");	// 1
        th(  "'PHRU'I");	// 1
        th( "PHRUIPHU");	// 1
        th( "'PHRULDA");	// 1
        th("'PHRUL'DIS");	// 1
        th(   "PHR'Um");	// 1
        th(   "'PHRUM");	// 1
        th(   "PHRUNG");	// 1
        th( "'PHRUNGS");	// 1
        th(    "PHRUS");	// 1
        th(   "'PHRUS");	// 1
        th(    "PHRVA");	// 1
        th(     "PHU:");	// 1
        th(     "PHU4");	// 1
        th("PHU:BRGYAD");	// 1
        th(    "'PHUM");	// 1
        th(    "PHU'O");	// 1
        th(  "PHUR'DI");	// 1
        th( "PHUshtIm");	// 1
        th(   "'PHVOR");	// 1
        th(      "PHY");	// 1
        th(   "PHY9IR");	// 1
        th(   "'PHYA'");	// 1
        th(   "^PHYAG");	// 1
        th(   "PHYAGG");	// 1
        th(   "PHYAMS");	// 1
        th(  "PHYANGS");	// 1
        th(    "^PHYE");	// 1
        th(  "'PHYE'O");	// 1
        th(    "PHYGS");	// 1
        th(   "PHYHIR");	// 1
        th(   "PHYI8R");	// 1
        th(   "PHYI9R");	// 1
        th(    "PHYIB");	// 1
        th(   "'PHYIB");	// 1
        th(   "PHYIBS");	// 1
        th(   "'PHYID");	// 1
        th(    "PHYIE");	// 1
        th(   "PHYI'E");	// 1
        th(  "PHYIHUR");	// 1
        th(   "PHYINI");	// 1
        th(  "PHYINLA");	// 1
        th("PHYIPHEBS");	// 1
        th(    "PhYIR");	// 1
        th(   "PHYIRM");	// 1
        th( "PHYIRYIN");	// 1
        th(    "PHYIt");	// 1
        th(     "PHYK");	// 1
        th("PHYOGSRNAMS");	// 1
        th(   "PHYONG");	// 1
        th(  "PHYONGS");	// 1
        th(  "PHYOOGS");	// 1
        th(   "PHYOSG");	// 1
        th(    "PHYRI");	// 1
        th(   "'PHYUD");	// 1
        th(   "'PHYUL");	// 1
        th(   "'PHYVA");	// 1
        th(     "P'I:");	// 1
        th(      "PIB");	// 1
        th(  "PICKING");	// 1
        th(    "PIDHA");	// 1
        th(   "PI-DYA");	// 1
        th(    "PIGRA");	// 1
        th(    "'PIGS");	// 1
        th(  "PiKRATI");	// 1
        th( "PIKshIRA");	// 1
        th(      "PIL");	// 1
        th(      "PIn");	// 1
        th(   "PIn-dA");	// 1
        th(   "PINGGA");	// 1
        th(  "PI-NGKA");	// 1
        th(  "PINTR'A");	// 1
        th(  "PINYDZA");	// 1
        th(     "P'IR");	// 1
        th(  "PIRYA'A");	// 1
        th(  "PIshthA");	// 1
        th(    "PITSA");	// 1
        th(   "PITYAm");	// 1
        th(    "PITZU");	// 1
        th(   "P'KshA");	// 1
        th(  "PKshU'O");	// 1
        th(     "PLTA");	// 1
        th(      "PmA");	// 1
        th(     "PMED");	// 1
        th(  "PMTHONG");	// 1
        th(      "PnA");	// 1
        th(   "PndITA");	// 1
        th(   "PNGMOS");	// 1
        th(    "PntAI");	// 1
        th(    "PNYON");	// 1
        th(   "PNYTZA");	// 1
        th(      "PO:");	// 1
        th(    "PO'2I");	// 1
        th(      "PO3");	// 1
        th(      "POA");	// 1
        th(    "PO'Am");	// 1
        th(   "PO'AMS");	// 1
        th(      "POE");	// 1
        th(    "'PO'I");	// 1
        th(    "PO'ID");	// 1
        th(    "PO'IG");	// 1
        th(   "PO'I'I");	// 1
        th(    "POINT");	// 1
        th("PO'IPAndI");	// 1
        th(    "PO'IS");	// 1
        th(      "POK");	// 1
        th(   "POLTAR");	// 1
        th(      "POm");	// 1
        th(     "POMA");	// 1
        th(     "PON'");	// 1
        th(     "PONI");	// 1
        th("PONYS'RBA");	// 1
        th(      "POO");	// 1
        th(    "'PO'O");	// 1
        th(   "PO'O'I");	// 1
        th(    "PO'OS");	// 1
        th(    "PORTA");	// 1
        th(     "'POS");	// 1
        th(      "POt");	// 1
        th(     "POtA");	// 1
        th(   "POTSUN");	// 1
        th(    "POYIN");	// 1
        th(   "PPHYIN");	// 1
        th(    "PPO'I");	// 1
        th(      "PPR");	// 1
        th( "PRADKshI");	// 1
        th( "PR'ADSHE");	// 1
        th(   "PRADZA");	// 1
        th("PRA-DZNY'A");	// 1
        th("PRADZ+NY'A");	// 1
        th("PRADZNYA'A");	// 1
        th("PRADZNY'ASA");	// 1
        th("PRADZNYENYTZA");	// 1
        th( "PRAJNY'A");	// 1
        th( "PRAJNY'E");	// 1
        th(    "PRAKA");	// 1
        th(    "'PRAL");	// 1
        th(     "PRAM");	// 1
        th( "PRANYDZA");	// 1
        th("PRANYDZ'A");	// 1
        th(    "PRA'O");	// 1
        th( "PRARnnAm");	// 1
        th(  "PRARshA");	// 1
        th( "PRASHNA:");	// 1
        th( "PRASHYAE");	// 1
        th(   "PRATHA");	// 1
        th(   "PRAT'I");	// 1
        th(   "PRATTI");	// 1
        th(   "PRATYA");	// 1
        th("PRATZTSA'I");	// 1
        th("PRATZTSYA");	// 1
        th("PRDZA-NY'A");	// 1
        th( "PRDZNYA'");	// 1
        th(     "PRED");	// 1
        th(    "PRE'I");	// 1
        th(    "PRENG");	// 1
        th(     "PRES");	// 1
        th(   "PREshO");	// 1
        th(   "PRGYUN");	// 1
        th(     "PRHE");	// 1
        th(     "PRI:");	// 1
        th(    "PRIGS");	// 1
        th("PRiKshA'I");	// 1
        th( "PRiKshAR");	// 1
        th( "PRiKshAS");	// 1
        th(    "'PRIL");	// 1
        th(    "PRING");	// 1
        th(   "'PRING");	// 1
        th(   "PRishA");	// 1
        th("PRish'ASMUDR'AN'UTS'AHA:");	// 1
        th(  "PRiSH'I");	// 1
        th(    "PRiTI");	// 1
        th( "PRiTZTSA");	// 1
        th("PRiTZTSVA");	// 1
        th(    "PRO'I");	// 1
        th(     "PRON");	// 1
        th(     "PROO");	// 1
        th(    "'PROR");	// 1
        th(    "'PROS");	// 1
        th(   "PROshA");	// 1
        th(   "PRTOGS");	// 1
        th("PRTOSHTZA");	// 1
        th(     "PRUG");	// 1
        th(    "PRUGS");	// 1
        th(    "'PRUL");	// 1
        th(    "PRYIN");	// 1
        th(     "PS'A");	// 1
        th(     "PSAR");	// 1
        th(    "PSGRA");	// 1
        th(  "Pshth'A");	// 1
        th(     "PS'I");	// 1
        th(   "PSKABS");	// 1
        th(    "PSKYE");	// 1
        th(   "PSLONG");	// 1
        th(   "PSPYOD");	// 1
        th(      "PtA");	// 1
        th(    "P-T'A");	// 1
        th(     "PTEE");	// 1
        th(     "PthA");	// 1
        th(    "PthYA");	// 1
        th(      "PTO");	// 1
        th(   "PTSASA");	// 1
        th(     "'PUB");	// 1
        th(     "PUGS");	// 1
        th(   "PU-KKA");	// 1
        th(  "PUKKASI");	// 1
        th("PUmshtIttA");	// 1
        th( "PUmSHTZA");	// 1
        th("PUmshTZTSA");	// 1
        th(   "PU-ndA");	// 1
        th(   "PUnd'A");	// 1
        th(    "PUndI");	// 1
        th( "PUnd'R'I");	// 1
        th(    "PUNTA");	// 1
        th(   "PUn-YA");	// 1
        th(   "PUN+YA");	// 1
        th("PUNYDZA'I");	// 1
        th(  "PUn+YEE");	// 1
        th(  "PUnY+'I");	// 1
        th(   "P'URBE");	// 1
        th(    "PURnE");	// 1
        th(  "P'URnnA");	// 1
        th(    "PUshA");	// 1
        th("PUshA'I'O");	// 1
        th(   "PUshBA");	// 1
        th(   "PUshBE");	// 1
        th(   "PUshNA");	// 1
        th( "PUshPEKE");	// 1
        th(   "PUshPI");	// 1
        th(   "PUshtA");	// 1
        th(  "PUshtAm");	// 1
        th(   "PUshYA");	// 1
        th(    "PUSTI");	// 1
        th(     "PU'U");	// 1
        th(     "PVAG");	// 1
        th(     "PYAG");	// 1
        th(     "PYAS");	// 1
        th(     "PYHE");	// 1
        th(     "PYID");	// 1
        th(    "'PYID");	// 1
        th(    "PYING");	// 1
        th(    "'PYOR");	// 1
        th(     "PZHI");	// 1
        th(    "PZHIG");	// 1
        th(    "R2ANG");	// 1
        th(      "R9O");	// 1
        th(      "'RA");	// 1
        th(      "RA2");	// 1
        th(     "RA2B");	// 1
        th(      "RA4");	// 1
        th(      "RAA");	// 1
        th(     "RABB");	// 1
        th(     "RABG");	// 1
        th(    "RABLA");	// 1
        th(  "RABSSUM");	// 1
        th(    "RAdnA");	// 1
        th(   "RADn'A");	// 1
        th(  "RADNA'I");	// 1
        th(     "RADV");	// 1
        th(   "RA-DWA");	// 1
        th(      "RAE");	// 1
        th(   "RAGA'I");	// 1
        th(     "RAGN");	// 1
        th(    "'RA'I");	// 1
        th(    "R'A'I");	// 1
        th(    "RA:'I");	// 1
        th(    "RA'ID");	// 1
        th(   "RAKdhI");	// 1
        th(    "RAK'I");	// 1
        th(    "RAKLA");	// 1
        th(  "RAKsh'A");	// 1
        th( "RAKshA'I");	// 1
        th("RAKshANTU");	// 1
        th(  "RAKshAS");	// 1
        th(  "R'AKshi");	// 1
        th(  "R'AKshI");	// 1
        th( "RA-K+shI");	// 1
        th( "RAKshI'I");	// 1
        th(  "RAKshIT");	// 1
        th("RAKsh'ITA");	// 1
        th(  "RAKshRI");	// 1
        th(  "RAKshTA");	// 1
        th(    "RAKsI");	// 1
        th(   "RAKTAR");	// 1
        th(    "RAKTE");	// 1
        th(    "RAKTI");	// 1
        th(   "RAKTLA");	// 1
        th(     "RALI");	// 1
        th(     "R'Am");	// 1
        th(    "RAMAM");	// 1
        th(   "RAMBHA");	// 1
        th(     "RAMN");	// 1
        th(   "RAnAdA");	// 1
        th(    "RAnAt");	// 1
        th( "RA-ndA'I");	// 1
        th(   "RANDZI");	// 1
        th( "RANDZI'I");	// 1
        th(    "'RANG");	// 1
        th(  "RANG+GA");	// 1
        th(   "RANGNA");	// 1
        th(  "RANGNAS");	// 1
        th(    "RANGV");	// 1
        th(     "RAnI");	// 1
        th(   "RAnn'A");	// 1
        th(    "RANNA");	// 1
        th(     "RANR");	// 1
        th(  "RANTI'I");	// 1
        th(   "RANTRA");	// 1
        th( "RANYJA'I");	// 1
        th( "RANYOTZA");	// 1
        th(  "RANYTZO");	// 1
        th(    "R'A'O");	// 1
        th(    "RA:'O");	// 1
        th(   "RA'ONG");	// 1
        th(      "RAP");	// 1
        th(    "RAPAR");	// 1
        th(    "RA:RA");	// 1
        th(    "RA:Ri");	// 1
        th(     "'RAS");	// 1
        th(    "RASAD");	// 1
        th(   "RAsh'i");	// 1
        th(   "RAsh'I");	// 1
        th(   "RAshKA");	// 1
        th(   "RAshtA");	// 1
        th("RASHTZI'O");	// 1
        th(   "RAshYI");	// 1
        th("RAshYTA'O");	// 1
        th( "RASMEE'I");	// 1
        th( "RASMEE'O");	// 1
        th(  "RASMI'I");	// 1
        th(     "RASU");	// 1
        th( "RAS:ZHES");	// 1
        th(      "RAT");	// 1
        th(     "RAtA");	// 1
        th( "RATANA'I");	// 1
        th(    "RATEN");	// 1
        th(     "RATI");	// 1
        th(     "RATN");	// 1
        th(   "RA-TNA");	// 1
        th(   "RAT-NA");	// 1
        th(   "RATN'E");	// 1
        th("RATNENDRA");	// 1
        th(   "RATNLA");	// 1
        th(    "RATRI");	// 1
        th(     "RATU");	// 1
        th(     "RAVA");	// 1
        th(     "RAWE");	// 1
        th(  "RA+YA'I");	// 1
        th(    "RAYGS");	// 1
        th(    "RA:YI");	// 1
        th(       "RB");	// 1
        th(    "RBABS");	// 1
        th( "RBADZRAS");	// 1
        th(    "RBAHA");	// 1
        th(    "RBA'I");	// 1
        th(   "RBANTU");	// 1
        th( "RBARLABS");	// 1
        th(    "RBB'A");	// 1
        th(      "RBE");	// 1
        th(     "RBER");	// 1
        th(     "RBES");	// 1
        th(    "RBHAS");	// 1
        th(     "RBHI");	// 1
        th(      "RBI");	// 1
        th(     "RBID");	// 1
        th(    "RBI'I");	// 1
        th(    "RBKA'");	// 1
        th(    "RBONG");	// 1
        th(     "RBOS");	// 1
        th(  "RBSHAGS");	// 1
        th(    "RB+TU");	// 1
        th(   "RBTZAN");	// 1
        th(    "RBYAB");	// 1
        th(    "RBYAS");	// 1
        th(    "RBYUN");	// 1
        th(     "RCIS");	// 1
        th(     "RCOD");	// 1
        th(      "RCU");	// 1
        th(     "'RDA");	// 1
        th(     "RD'A");	// 1
        th(    "RDAGS");	// 1
        th(    "'RDAM");	// 1
        th(   "RDBYIG");	// 1
        th(     "RDEN");	// 1
        th(    "'RDEN");	// 1
        th(    "'RDES");	// 1
        th(    "RDH'A");	// 1
        th(   "RDHA'I");	// 1
        th(  "RDHANTU");	// 1
        th(  "RDHARMA");	// 1
        th(     "RDHE");	// 1
        th(     "RDHI");	// 1
        th(     "RDHU");	// 1
        th(    "RDIBS");	// 1
        th(    "RDING");	// 1
        th(   "RDINGS");	// 1
        th(     "RDIS");	// 1
        th(    "RDI'U");	// 1
        th(    "RDJOD");	// 1
        th(     "^RDO");	// 1
        th(  "RDO'I'O");	// 1
        th(     "RDOM");	// 1
        th(     "RDOP");	// 1
        th(   "RDORJE");	// 1
        th(    "RDO'U");	// 1
        th(   "RDOZGS");	// 1
        th(   "'RDUGS");	// 1
        th(    "'RDUL");	// 1
        th(    "RDU'U");	// 1
        th(    "RDUZN");	// 1
        th(     "RDVI");	// 1
        th(  "RDZA'AM");	// 1
        th(   "RDZABS");	// 1
        th("RDZAMBU'I");	// 1
        th(    "RDZAN");	// 1
        th(    "RDZAR");	// 1
        th(    "RDZED");	// 1
        th(    "RDZER");	// 1
        th(   "RDZIGS");	// 1
        th(  "RDZO-dA");	// 1
        th(    "RDZON");	// 1
        th(    "'RDZU");	// 1
        th(    "RDZUM");	// 1
        th(   "RDZUNG");	// 1
        th(   "RDZU'O");	// 1
        th(    "RDZUR");	// 1
        th(    "RDZVA");	// 1
        th(      "RE'");	// 1
        th(      "RE4");	// 1
        th(     "R'ED");	// 1
        th(   "REDshE");	// 1
        th(     "RE'E");	// 1
        th(  "REE'ANG");	// 1
        th(     "REEG");	// 1
        th(    "REE'O");	// 1
        th(     "^REG");	// 1
        th(    "REGAN");	// 1
        th(    "REGPA");	// 1
        th(   "REGYAL");	// 1
        th(   "RE'I'O");	// 1
        th(    "REJ'I");	// 1
        th(  "REN-DRA");	// 1
        th(  "RENDR'A");	// 1
        th(  "RENDRAR");	// 1
        th( "RENYRTRA");	// 1
        th(    "RESHA");	// 1
        th(   "REsh'I");	// 1
        th(   "REshtA");	// 1
        th(     "RG'A");	// 1
        th(     "RGA3");	// 1
        th(    "RGAGS");	// 1
        th(     "RGAm");	// 1
        th(    "'RGAN");	// 1
        th(    "RGAYD");	// 1
        th(     "RGEG");	// 1
        th(    "RGE'U");	// 1
        th(    "RGH'A");	// 1
        th(   "RGHA'I");	// 1
        th(    "RGHAn");	// 1
        th(     "RGHI");	// 1
        th(     "RGIA");	// 1
        th(     "RGIG");	// 1
        th(   "RGNYIS");	// 1
        th(    "RGO9D");	// 1
        th(    "^RGOL");	// 1
        th(     "RGOM");	// 1
        th(    "'RGON");	// 1
        th(   "RGONGS");	// 1
        th(     "RGOR");	// 1
        th(    "RGRAL");	// 1
        th(    "RGRUB");	// 1
        th(   "RGRYAL");	// 1
        th(    "RGTAN");	// 1
        th(   "RGTOGS");	// 1
        th(  "RGU'I'O");	// 1
        th(    "RGY8U");	// 1
        th(    "^RGYA");	// 1
        th(    "RGY'A");	// 1
        th(    "RGYA0");	// 1
        th( "RGYA'ANG");	// 1
        th(  "RGYABYA");	// 1
        th(   "RGYACH");	// 1
        th(   "RGYALO");	// 1
        th(    "RGYAm");	// 1
        th(  "'RGYA'O");	// 1
        th( "RGYA'O'I");	// 1
        th( "RGYAPA'I");	// 1
        th(   "RGYASD");	// 1
        th( "RGYASKYI");	// 1
        th(   "RGYASm");	// 1
        th(   "RGYASO");	// 1
        th(   "RGYASU");	// 1
        th(     "RGYD");	// 1
        th(     "RGYE");	// 1
        th(    "RGYED");	// 1
        th(    "RGYER");	// 1
        th(    "RGYES");	// 1
        th(    "RGYGI");	// 1
        th(    "RGYID");	// 1
        th(    "RGYON");	// 1
        th(  "RGYONGS");	// 1
        th(    "RGYU'");	// 1
        th(  "RGYU'2I");	// 1
        th(   "RGYU8D");	// 1
        th("RGYU'HRTZIS");	// 1
        th(    "RGYUM");	// 1
        th(  "RGYU'NG");	// 1
        th(  "RGYUNGS");	// 1
        th(   "RGYUR5");	// 1
        th( "RGYUS'AM");	// 1
        th("RGYUSBYOR");	// 1
        th( "RGYUSHAD");	// 1
        th(    "RGZHA");	// 1
        th(   "RGZIGS");	// 1
        th(     "RH'A");	// 1
        th(   "RHARMA");	// 1
        th(     "RHES");	// 1
        th(      "RHI");	// 1
        th(     "RHOG");	// 1
        th(    "RHYAS");	// 1
        th(      "^RI");	// 1
        th(    "R''I:");	// 1
        th(      "RI2");	// 1
        th(      "RI4");	// 1
        th(     "RI8G");	// 1
        th(     "RI9G");	// 1
        th(     "Ri'A");	// 1
        th(      "RiB");	// 1
        th(   "RIDH'A");	// 1
        th(     "R'IG");	// 1
        th(     "RIGD");	// 1
        th(    "RIGGS");	// 1
        th(     "RIGI");	// 1
        th(    "RIG'I");	// 1
        th(     "RIGN");	// 1
        th(    "^RIGS");	// 1
        th(    "RIi'I");	// 1
        th(   "RI'I'O");	// 1
        th(      "RIK");	// 1
        th(     "RIKA");	// 1
        th(    "RIK'A");	// 1
        th(   "RIKshA");	// 1
        th(  "R'IKshi");	// 1
        th(    "RINDA");	// 1
        th(   "RIND'A");	// 1
        th(     "RINM");	// 1
        th(     "RINR");	// 1
        th(     "'RIS");	// 1
        th(     "R'IS");	// 1
        th(    "RIshA");	// 1
        th( "RIshtA'O");	// 1
        th(   "RIshYA");	// 1
        th("RIshYANA'O");	// 1
        th(     "RI't");	// 1
        th(     "RiTA");	// 1
        th(     "RItA");	// 1
        th(     "RiTI");	// 1
        th( "RITZETSE");	// 1
        th(  "RITZTSA");	// 1
        th(  "RITZTZA");	// 1
        th(    "RJAGS");	// 1
        th(     "RJAM");	// 1
        th(     "RJDO");	// 1
        th(     "RJEB");	// 1
        th(    "RJEDS");	// 1
        th(    "RJENG");	// 1
        th(   "RJE'NG");	// 1
        th(   "RJES'I");	// 1
        th(    "RJESN");	// 1
        th(    "'RJID");	// 1
        th(    "RJI'U");	// 1
        th(     "RJOG");	// 1
        th(     "RJON");	// 1
        th(     "RJOR");	// 1
        th(    "RJRES");	// 1
        th(     "RJUN");	// 1
        th(    "RKABS");	// 1
        th(     "RKAR");	// 1
        th(  "RKA-shI");	// 1
        th(  "RKAtA'O");	// 1
        th(     "RKEB");	// 1
        th(    "RKEYN");	// 1
        th(   "RKHYED");	// 1
        th(      "RKI");	// 1
        th(   "RKIshA");	// 1
        th(     "RKOB");	// 1
        th(     "RKOL");	// 1
        th(     "RKOM");	// 1
        th(     "RKOt");	// 1
        th(    "RKU8N");	// 1
        th(    "RKUEN");	// 1
        th(     "RKUG");	// 1
        th(     "RKUM");	// 1
        th( "RKUNMA'O");	// 1
        th(   "RKYEEN");	// 1
        th(   "RKYENN");	// 1
        th(   "RKYENS");	// 1
        th(    "RKYOB");	// 1
        th(  "RKYONGS");	// 1
        th(    "RKYUD");	// 1
        th(   "RKYU'I");	// 1
        th(       "RL");	// 1
        th(     "RLAR");	// 1
        th(      "RLE");	// 1
        th(    "RLEBS");	// 1
        th(     "RLEG");	// 1
        th(    "RLEMS");	// 1
        th(     "RLID");	// 1
        th( "RLING+GA");	// 1
        th("RLING+GA'I");	// 1
        th(     "RLIR");	// 1
        th(    "RLLAS");	// 1
        th(     "RLOm");	// 1
        th(     "RLUB");	// 1
        th(     "RLUD");	// 1
        th(    "RLUGN");	// 1
        th(     "RLUN");	// 1
        th(   "RLUNGS");	// 1
        th(       "RM");	// 1
        th(     "RMA'");	// 1
        th(    "RMA'A");	// 1
        th(   "RMA'AM");	// 1
        th(    "RMABS");	// 1
        th(   "RM'A'I");	// 1
        th(     "RMAM");	// 1
        th(    "RMAMS");	// 1
        th("RMANGDHVA");	// 1
        th(    "RME'O");	// 1
        th(     "RMER");	// 1
        th(     "RMES");	// 1
        th(  "RME'U'I");	// 1
        th(   "RMI'AM");	// 1
        th(    "RMMAR");	// 1
        th(    "RMMI:");	// 1
        th(    "RMOGS");	// 1
        th(   "RMOTTA");	// 1
        th(    "RMRAD");	// 1
        th(   "RMRAGS");	// 1
        th(   "RMRA'I");	// 1
        th(    "RMTHA");	// 1
        th(  "RMTHONG");	// 1
        th(     "RMUS");	// 1
        th(   "RMUUGS");	// 1
        th(     "RMVA");	// 1
        th(    "RMYE:");	// 1
        th(     "RMYO");	// 1
        th(       "Rn");	// 1
        th(     "Rn'A");	// 1
        th(  "RNA'ANG");	// 1
        th(    "RNABS");	// 1
        th(    "RNALM");	// 1
        th(    "^RNAM");	// 1
        th(    "'RNAM");	// 1
        th(   "RNAM'A");	// 1
        th(   "RNAMAS");	// 1
        th(    "RNAMJ");	// 1
        th(   "RNAMPA");	// 1
        th( "RNAMPAMI");	// 1
        th(  "RNAMPAR");	// 1
        th(   "RNAMPO");	// 1
        th(   "RNAMPR");	// 1
        th(    "RNAMR");	// 1
        th(  "RNAM-SO");	// 1
        th(   "RNAmSU");	// 1
        th(    "RNANS");	// 1
        th(     "RnAS");	// 1
        th(     "RndA");	// 1
        th(    "Rnd'I");	// 1
        th(    "RNEID");	// 1
        th(     "RNEL");	// 1
        th(     "RNEN");	// 1
        th(      "RNG");	// 1
        th(  "RNGA'AM");	// 1
        th(   "^RNGAN");	// 1
        th(   "RNGE'I");	// 1
        th(   "RNGE'U");	// 1
        th(    "RNGHA");	// 1
        th(    "RNGOL");	// 1
        th(    "RNGUD");	// 1
        th(    "RNGUN");	// 1
        th(      "RNI");	// 1
        th(     "RNIG");	// 1
        th(     "RNIM");	// 1
        th(    "RNING");	// 1
        th(      "RNL");	// 1
        th(      "RNm");	// 1
        th(      "RNM");	// 1
        th(     "RNmS");	// 1
        th(    "Rn+nA");	// 1
        th(      "RnO");	// 1
        th(    "RNOBS");	// 1
        th(     "RNOM");	// 1
        th(    "RNOMS");	// 1
        th(    "RNRYI");	// 1
        th(     "RNUM");	// 1
        th(     "RNUS");	// 1
        th(     "RNVA");	// 1
        th(     "RnYA");	// 1
        th(   "RNYAGS");	// 1
        th(    "RNYAS");	// 1
        th(     "RNYD");	// 1
        th( "RNYDZA'I");	// 1
        th(    "RNY'E");	// 1
        th(   "RNYEGS");	// 1
        th(    "RNYER");	// 1
        th(  "RNYIDZA");	// 1
        th(    "RNYON");	// 1
        th(    "RNYOS");	// 1
        th(      "RO:");	// 1
        th(    "ROd'A");	// 1
        th(    "RODHA");	// 1
        th(   "RO'I'O");	// 1
        th(      "ROK");	// 1
        th(  "ROKsh'A");	// 1
        th("RONYAKsh'");	// 1
        th("RONYAKshA");	// 1
        th("RONYAKshsh'O");	// 1
        th("RONYANDRANA");	// 1
        th("RONY'ANDRANA");	// 1
        th(    "RONYK");	// 1
        th( "RONYKsh'");	// 1
        th("RONYKshA'I");	// 1
        th("RONYKsh'I");	// 1
        th(     "ROOR");	// 1
        th(  "ROshKRI");	// 1
        th(   "ROTOGS");	// 1
        th(     "ROTt");	// 1
        th(      "RPA");	// 1
        th(    "RPA'O");	// 1
        th(     "RPAR");	// 1
        th(     "RPAS");	// 1
        th(   "RPHYIR");	// 1
        th(     "RPRO");	// 1
        th(    "RRDUL");	// 1
        th(  "RRDZOGS");	// 1
        th(  "RRGYAGS");	// 1
        th(     "RRIL");	// 1
        th(    "RRNAM");	// 1
        th(    "RRTZE");	// 1
        th(    "RSANG");	// 1
        th(    "RsH'A");	// 1
        th(     "RSHE");	// 1
        th(    "RSHES");	// 1
        th(     "RshI");	// 1
        th(   "RSKYEL");	// 1
        th(   "RSMISA");	// 1
        th(   "RSNANG");	// 1
        th(    "RSSVA");	// 1
        th(     "RSUM");	// 1
        th(     "^RTA");	// 1
        th(     "RT'A");	// 1
        th(     "RTAD");	// 1
        th(    "^RTAG");	// 1
        th(    "RTANG");	// 1
        th(     "RTEd");	// 1
        th(     "RTEL");	// 1
        th(     "RTEM");	// 1
        th("RTEN'BREL");	// 1
        th(  "RTENGYI");	// 1
        th(     "RTER");	// 1
        th(     "RTES");	// 1
        th(  "RTE'U'I");	// 1
        th(     "RTGA");	// 1
        th(    "RTGAS");	// 1
        th(    "RTGOG");	// 1
        th(     "RTGS");	// 1
        th(   "RTGYAB");	// 1
        th(     "RthA");	// 1
        th(   "RTH'AD");	// 1
        th(   "RTHAMS");	// 1
        th(    "RTHUN");	// 1
        th(     "RT'I");	// 1
        th(     "RTID");	// 1
        th(   "RTI'OO");	// 1
        th(   "RTNA'I");	// 1
        th(     "RTNE");	// 1
        th(     "RT'O");	// 1
        th(    "RTOGN");	// 1
        th(    "RTOGs");	// 1
        th(    "RTOHS");	// 1
        th(     "RTOM");	// 1
        th(     "RTOR");	// 1
        th(     "RTPG");	// 1
        th(     "RTRA");	// 1
        th(     "RTRE");	// 1
        th(     "RTRN");	// 1
        th(     "RTSI");	// 1
        th(    "RTSIM");	// 1
        th(     "RTSO");	// 1
        th(    "RTSOR");	// 1
        th(    "RTTE:");	// 1
        th(    "RTUNG");	// 1
        th(   "RTUNGS");	// 1
        th( "RTZAKshA");	// 1
        th(  "RTZANRA");	// 1
        th(    "RTZBA");	// 1
        th(    "^RTZE");	// 1
        th( "RTZE'ANG");	// 1
        th( "RTZEENdI");	// 1
        th(   "^RTZIG");	// 1
        th(   "RTZIKA");	// 1
        th(  "RTZILLA");	// 1
        th(    "RTZIN");	// 1
        th(  "RTZITSA");	// 1
        th(   "RTZOAM");	// 1
        th(   "RTZODO");	// 1
        th(   "RTZO'I");	// 1
        th(   "'RTZOM");	// 1
        th(    "RTZOR");	// 1
        th(    "RTZOS");	// 1
        th(    "RTZOt");	// 1
        th(   "RTZTTI");	// 1
        th(   "RTZUGS");	// 1
        th(    "RTZUL");	// 1
        th(   "RTZV'A");	// 1
        th(   "RTZVAL");	// 1
        th(  "RTZVA'M");	// 1
        th(    "RTZVE");	// 1
        th(    "RTZVI");	// 1
        th(   "RTZVIG");	// 1
        th( "RTZWA'AM");	// 1
        th(   "RTZWAS");	// 1
        th(      "RU4");	// 1
        th(    "RU8NG");	// 1
        th(  "RUBHYA:");	// 1
        th(     "RUBS");	// 1
        th(    "RUDHA");	// 1
        th(     "RU'E");	// 1
        th(    "RUGNG");	// 1
        th(      "RUH");	// 1
        th(      "RUK");	// 1
        th(     "RUKA");	// 1
        th(    "RUKA:");	// 1
        th(    "RUKKE");	// 1
        th(   "RULKAL");	// 1
        th("RUNDH'A:'O");	// 1
        th(    "^RUNG");	// 1
        th(    "RU:'O");	// 1
        th(   "RUshA:");	// 1
        th(    "RUshI");	// 1
        th(   "RUshKA");	// 1
        th(  "RUshTZA");	// 1
        th(      "RUT");	// 1
        th(    "RUTZA");	// 1
        th(     "RU'U");	// 1
        th(     "^RVA");	// 1
        th(     "RVA:");	// 1
        th(  "RWA'ANG");	// 1
        th(    "RWA'I");	// 1
        th(     "RWAS");	// 1
        th(    "R+Y'A");	// 1
        th(     "RYAM");	// 1
        th(    "RYANG");	// 1
        th("R+YANYTZA");	// 1
        th(     "RYEE");	// 1
        th(    "RYGAL");	// 1
        th(    "RYGUD");	// 1
        th(   "RYGU'I");	// 1
        th(    "R+Y'I");	// 1
        th(     "RYIN");	// 1
        th(     "RYOD");	// 1
        th(     "RYUN");	// 1
        th(     "RYUR");	// 1
        th(     "RYYA");	// 1
        th(     "RZAS");	// 1
        th(     "RZED");	// 1
        th(    "RZONG");	// 1
        th(   "RZTODS");	// 1
        th(     "RZVA");	// 1
        th(   "S2BYAR");	// 1
        th(     "S2TE");	// 1
        th(    "S2TON");	// 1
        th(     "S3ER");	// 1
        th(      "^SA");	// 1
        th(     "S'A:");	// 1
        th(      "SA4");	// 1
        th(     "SA'A");	// 1
        th(  "S'ABH'A");	// 1
        th(     "SABS");	// 1
        th(    "SABTA");	// 1
        th(     "S'AD");	// 1
        th(   "SADDHA");	// 1
        th(  "SADDHI:");	// 1
        th(   "S'ADHA");	// 1
        th("SADHARMMASY'ABIT+S'URYYAN'ASHAY'ANDHASADGHORATNAK'ARA");	// 1
        th(  "SADHU'I");	// 1
        th(     "SADI");	// 1
        th(    "SADMA");	// 1
        th(   "SADYAS");	// 1
        th("SADZDZANA'I");	// 1
        th(    "SAGNS");	// 1
        th(      "SAH");	// 1
        th(     "SAHO");	// 1
        th(     "SAHR");	// 1
        th(   "SAHV'A");	// 1
        th( "S'AHV'AN");	// 1
        th(    "^SA'I");	// 1
        th( "SA'I'ANG");	// 1
        th(   "SAIDHI");	// 1
        th(   "SA'ING");	// 1
        th(   "SA'I'O");	// 1
        th(     "S'AL");	// 1
        th(    "SALK'");	// 1
        th(     "^SAM");	// 1
        th(     "SAmA");	// 1
        th(  "SAMBH'A");	// 1
        th(  "SAMBHIR");	// 1
        th("SAMBUDDHI'O");	// 1
        th("SAMDR'AN'UDS'AHA:");	// 1
        th("SAMKYAKSAm");	// 1
        th(   "SAMNTA");	// 1
        th(     "SAMs");	// 1
        th( "SAMUNDRA");	// 1
        th(      "SAn");	// 1
        th(     "SANA");	// 1
        th("S'ANATA'O");	// 1
        th(   "SANDHU");	// 1
        th(   "SANDZI");	// 1
        th( "SANG+G'A");	// 1
        th("SANG+GAHA");	// 1
        th("SANG+GA'I");	// 1
        th(  "SANGGHA");	// 1
        th(   "SANGHA");	// 1
        th(  "SANGS''");	// 1
        th(  "SANGYAS");	// 1
        th(    "SAn'I");	// 1
        th(   "SANN'A");	// 1
        th(    "SANNG");	// 1
        th(    "SAN'O");	// 1
        th(   "SANT'A");	// 1
        th(   "S'ANTI");	// 1
        th("SANTOshNI");	// 1
        th(  "SANYDZI");	// 1
        th(  "SANYTZI");	// 1
        th(    "S'A'O");	// 1
        th(  "SAPBDZA");	// 1
        th(    "SAPtA");	// 1
        th(   "SAP-TA");	// 1
        th(    "SAPTI");	// 1
        th(     "SAR:");	// 1
        th("SARBADZNY'A");	// 1
        th("SARBAKARMA");	// 1
        th(   "SARBAm");	// 1
        th("SARBAMANG+GA");	// 1
        th("SARB'ARTHA");	// 1
        th(  "SARBBAm");	// 1
        th(    "SARBE");	// 1
        th("SARBEBHYAS");	// 1
        th(   "SARD'A");	// 1
        th( "SAR:DANG");	// 1
        th(  "SARGA'I");	// 1
        th(   "SARGAS");	// 1
        th(   "SARGGA");	// 1
        th(   "SARGHA");	// 1
        th(   "SARK'A");	// 1
        th(    "SARKO");	// 1
        th(    "SARTA");	// 1
        th(   "SARTHA");	// 1
        th(  "SARTI'I");	// 1
        th(  "SARW'A:");	// 1
        th( "SASARB'A");	// 1
        th(    "SASTI");	// 1
        th(     "SATA");	// 1
        th(   "SATA'O");	// 1
        th(   "SATASA");	// 1
        th(  "SA+TH'A");	// 1
        th(   "SATHA'");	// 1
        th(   "SAthVA");	// 1
        th(   "SATI'O");	// 1
        th(   "SATK'A");	// 1
        th(   "S'ATRA");	// 1
        th("SATV'AmSHCA");	// 1
        th( "S'ATZCHA");	// 1
        th("S'ATZTSA'AM");	// 1
        th(     "SAWA");	// 1
        th(    "SA-YA");	// 1
        th(   "SA+YA'");	// 1
        th(   "SA+Y'I");	// 1
        th(     "SB'A");	// 1
        th(     "SBA'");	// 1
        th( "SBADZNYA");	// 1
        th( "SBARADNA");	// 1
        th(  "SBAYNGS");	// 1
        th(    "SBCAD");	// 1
        th(     "SBCI");	// 1
        th(    "SBHAD");	// 1
        th(     "SB'I");	// 1
        th(     "SBIB");	// 1
        th(     "SBIG");	// 1
        th(   "SBINGS");	// 1
        th(    "SBLAB");	// 1
        th(   "SBNYIL");	// 1
        th(   "SBONGS");	// 1
        th(    "SBPYI");	// 1
        th(  "SBPYONG");	// 1
        th(   "SBRANZ");	// 1
        th(    "SBRED");	// 1
        th(     "SBRI");	// 1
        th(    "SBRIL");	// 1
        th(   "SBRJOD");	// 1
        th(  "SBRO'AM");	// 1
        th(   "SBROBS");	// 1
        th(   "SBRO'I");	// 1
        th(   "SBRONG");	// 1
        th(    "SBROR");	// 1
        th(   "SBRUBS");	// 1
        th(  "SBRUMAM");	// 1
        th(    "SBSAD");	// 1
        th(    "SBTAR");	// 1
        th(    "SBYAG");	// 1
        th(   "SBYAGS");	// 1
        th(    "SBYAM");	// 1
        th(   "SBYANS");	// 1
        th(    "SBYEL");	// 1
        th(   "SBYENG");	// 1
        th(   "SBYI8N");	// 1
        th(    "SBYIB");	// 1
        th(   "^SBYIN");	// 1
        th(  "SBYINAm");	// 1
        th(  "SBYINPA");	// 1
        th(   "SBYINR");	// 1
        th(   "SBYI'U");	// 1
        th(   "SBYOGS");	// 1
        th(   "SBYOKR");	// 1
        th(    "SBYOM");	// 1
        th(   "SBYO'O");	// 1
        th(    "SBYOY");	// 1
        th(     "SBYR");	// 1
        th(    "SBYRA");	// 1
        th(   "SBYSNG");	// 1
        th(    "SBYUR");	// 1
        th(   "SBZHIN");	// 1
        th(     "SCAD");	// 1
        th(     "SCAN");	// 1
        th(     "SCAS");	// 1
        th(   "S'CHAR");	// 1
        th(    "SCHEN");	// 1
        th(    "SCHUB");	// 1
        th(     "SCOL");	// 1
        th(     "SD3E");	// 1
        th(  "SDA'ANG");	// 1
        th(     "SDAB");	// 1
        th(    "SDABG");	// 1
        th(    "SDDHI");	// 1
        th(     "SDDO");	// 1
        th(     "SDE'");	// 1
        th(  "SDE'ANG");	// 1
        th(  "SDE'U'O");	// 1
        th(    "SDHRI");	// 1
        th(   "SDHYEE");	// 1
        th(    "^SDIG");	// 1
        th(    "SDI'I");	// 1
        th(     "SDIM");	// 1
        th(     "SDIN");	// 1
        th(    "SDKRI");	// 1
        th(    "SDNAG");	// 1
        th(     "SDNO");	// 1
        th(     "SDOB");	// 1
        th(    "SDODM");	// 1
        th(   "'SDOGS");	// 1
        th(    "SDO'I");	// 1
        th(     "SDOm");	// 1
        th(    "SDR'I");	// 1
        th(    "SDRIG");	// 1
        th(     "SDRO");	// 1
        th(    "SDROS");	// 1
        th(    "SDRUL");	// 1
        th(     "'SDU");	// 1
        th(     "SDUK");	// 1
        th(    "'SDUS");	// 1
        th(    "SDUSD");	// 1
        th(    "SDZAB");	// 1
        th(  "SDZINGS");	// 1
        th(     "SDZO");	// 1
        th(    "SDZUS");	// 1
        th(      "SE2");	// 1
        th(      "SE4");	// 1
        th(   "SEDANG");	// 1
        th(    "SEDGE");	// 1
        th("SEDURYA'I");	// 1
        th(     "SEE:");	// 1
        th(  "SEENGGE");	// 1
        th(     "SEIS");	// 1
        th(     "SEJA");	// 1
        th(      "SEm");	// 1
        th(     "SEMC");	// 1
        th(     "SEMs");	// 1
        th(    ":SEMS");	// 1
        th(   "SEMSAm");	// 1
        th(   "SEMS'O");	// 1
        th(    "SEN'A");	// 1
        th(    "SENGE");	// 1
        th(   "SEN-GE");	// 1
        th(   "SEN+GE");	// 1
        th(   "SENGEG");	// 1
        th("SENGEGA'I");	// 1
        th(  "SENGE'I");	// 1
        th("SENG+GA'I");	// 1
        th("SENGGE'AM");	// 1
        th("SENG+GE'ANG");	// 1
        th(  "SENG+GI");	// 1
        th( "SENGNDHU");	// 1
        th(     "SENm");	// 1
        th(    "SENTI");	// 1
        th(    "SENYA");	// 1
        th(     "^SER");	// 1
        th(     "S'ER");	// 1
        th(     "SESL");	// 1
        th(     "SESS");	// 1
        th(    "SETRE");	// 1
        th(       "SG");	// 1
        th(    "SGAMS");	// 1
        th(     "SGAN");	// 1
        th(   "SGANGS");	// 1
        th(    "SGA'O");	// 1
        th(    "SGEBS");	// 1
        th(     "SGED");	// 1
        th(    "SGE'I");	// 1
        th(    "SGHUS");	// 1
        th(     "SGIG");	// 1
        th(     "SGIL");	// 1
        th(    "SGKAG");	// 1
        th(    "SGNAS");	// 1
        th(    "SGNOM");	// 1
        th(     "SGNS");	// 1
        th(   "SGNYIS");	// 1
        th(     "SGO'");	// 1
        th(     "SGOB");	// 1
        th(    "^SGOG");	// 1
        th(    "SGORL");	// 1
        th(    "SGORN");	// 1
        th(  "SGRA'Am");	// 1
        th(   "'SGRAB");	// 1
        th(  "'SGRAGS");	// 1
        th(   "SGR'AI");	// 1
        th(   "SGRA'M");	// 1
        th( "SGRAS'AM");	// 1
        th(  "SGRASPA");	// 1
        th(   "SGREAS");	// 1
        th(   "SGRE'O");	// 1
        th(  "'SGRIBS");	// 1
        th(    "SGRId");	// 1
        th(    "SGRID");	// 1
        th(   "SGRI'I");	// 1
        th(   "SGROGs");	// 1
        th(   "^SGROL");	// 1
        th(   "SGRO'O");	// 1
        th(    "SGRSA");	// 1
        th("SGRUBDGOS");	// 1
        th(   "SGRUBG");	// 1
        th(  "'SGRUBS");	// 1
        th(   "SGRU'I");	// 1
        th(   "SGRYUR");	// 1
        th(  "SGSUNGS");	// 1
        th(    "SGTZO");	// 1
        th(   "SGUNGS");	// 1
        th(    "SGYAB");	// 1
        th(   "SGYA'I");	// 1
        th(  "SGYANGS");	// 1
        th(    "SGYAS");	// 1
        th(   "SGYEGS");	// 1
        th(   "SGYE'O");	// 1
        th("SGYE'U'AM");	// 1
        th(   "SGYIGS");	// 1
        th(   "SGYI'I");	// 1
        th(   "SGYING");	// 1
        th(     "SGYO");	// 1
        th(    "SGYOB");	// 1
        th(    "SGYOD");	// 1
        th(    "SGYU8");	// 1
        th( "SGYU'ANG");	// 1
        th(    "SGYUN");	// 1
        th(   "SGYUNG");	// 1
        th(   "SGYURD");	// 1
        th(      "SH'");	// 1
        th(   "SH9ING");	// 1
        th(    "sh+'A");	// 1
        th(     "Sh'A");	// 1
        th(     "^SHA");	// 1
        th(    "'SHA'");	// 1
        th(    "SH+'A");	// 1
        th(     "SHA2");	// 1
        th(     "SHA4");	// 1
        th(    "SHA'A");	// 1
        th(  "SHABD'A");	// 1
        th(   "SHABDi");	// 1
        th(    "'SHAD");	// 1
        th(   "shADGA");	// 1
        th(   "shAdHA");	// 1
        th(   "SHADHA");	// 1
        th(    "SHADI");	// 1
        th(  "SHADKTI");	// 1
        th(   "shADLi");	// 1
        th("shADTSY'A");	// 1
        th(  "SHA'I'O");	// 1
        th(     "shAK");	// 1
        th(  "^SH'AKA");	// 1
        th(  "SH'A-KA");	// 1
        th(   "SHA-KA");	// 1
        th(   "SHA'KA");	// 1
        th(   "SHAKTA");	// 1
        th(   "SHAKTI");	// 1
        th( "'SH'AKYA");	// 1
        th(  "SHA+KYA");	// 1
        th("SH'AKYA'O");	// 1
        th( "SHA'KYAR");	// 1
        th("SH'A-KYAS");	// 1
        th("SH'AKYATHUB");	// 1
        th(  "SH'AKYE");	// 1
        th(  "SH'AKYI");	// 1
        th( "SH'A-KYI");	// 1
        th("SH'A-KY'I");	// 1
        th(   "SHALLI");	// 1
        th(    "SHALO");	// 1
        th(    "^SHAM");	// 1
        th(  "SHAMB'A");	// 1
        th("SH'AMBA'IRA");	// 1
        th( "SHAMBHAR");	// 1
        th("SH'AMBH'I");	// 1
        th(   "SHAMBI");	// 1
        th(  "SHAMHAm");	// 1
        th(   "SHA'MI");	// 1
        th("SH'AMIBA'");	// 1
        th("SH'AM'I'I");	// 1
        th( "SHAMLHIR");	// 1
        th(    "SHAmS");	// 1
        th(   "SH'ANG");	// 1
        th( "shANG+GA");	// 1
        th("shANGGA'O");	// 1
        th( "SHANGKHA");	// 1
        th(  "SHANGKU");	// 1
        th(  "SH'ANHI");	// 1
        th("SHANIshKA");	// 1
        th(   "sHAnnA");	// 1
        th(   "SHANNA");	// 1
        th("SH'ANNT'I");	// 1
        th(   "SHANRA");	// 1
        th(   "shANTA");	// 1
        th("SH'A-N+TA");	// 1
        th( "SH'ANT'A");	// 1
        th("SH'ANTASA");	// 1
        th(  "SH'ANTE");	// 1
        th( "SH'A-NTE");	// 1
        th(   "SHAnTI");	// 1
        th( "SH'AN+TI");	// 1
        th( "SHANTI'I");	// 1
        th( "SH'AnTIm");	// 1
        th( "SHA'NTIP");	// 1
        th(  "SHANTIS");	// 1
        th( "SH'ANTPI");	// 1
        th(  "shA:NUS");	// 1
        th("SH'ANYTSSHA");	// 1
        th(     "shAO");	// 1
        th(   "SHA-PA");	// 1
        th(  "sh'APAn");	// 1
        th(     "shAR");	// 1
        th(    "SH'AR");	// 1
        th(  "SHARA'I");	// 1
        th(  "SHARBBA");	// 1
        th(   "shARBU");	// 1
        th(   "SH'ARI");	// 1
        th(  "SHARMMA");	// 1
        th(  "SHARnAm");	// 1
        th("shARshA'I");	// 1
        th(    "shASA");	// 1
        th( "SH'ASANA");	// 1
        th(    "SHAsh");	// 1
        th("SHAshIA'I");	// 1
        th( "shAshthA");	// 1
        th( "shAshthI");	// 1
        th(  "SHA-STA");	// 1
        th(   "SHASTI");	// 1
        th("SH'ASTI'I");	// 1
        th( "SH'ASTRA");	// 1
        th("SH'ASTR'A");	// 1
        th("SH'ASTRAS");	// 1
        th(    "shAtA");	// 1
        th(    "SHATA");	// 1
        th(  "SHATA'I");	// 1
        th(  "SH'ATAm");	// 1
        th(   "shAthA");	// 1
        th(   "shAthI");	// 1
        th( "SHAt'IKA");	// 1
        th(  "SH'ATMA");	// 1
        th( "shAtP'AR");	// 1
        th( "shAtshYA");	// 1
        th("shAT+TRIm");	// 1
        th(    "shAtU");	// 1
        th(   "SHATZA");	// 1
        th(    "shBOO");	// 1
        th(    "SHBTA");	// 1
        th(   "SHCHA'");	// 1
        th(     "SHCO");	// 1
        th(      "SHD");	// 1
        th(     "shdA");	// 1
        th(   "shddHA");	// 1
        th(   "SHDDHO");	// 1
        th("shdGTZTSA");	// 1
        th(    "shdMU");	// 1
        th(    "SHDOO");	// 1
        th(  "shdR'I:");	// 1
        th(     "SHE4");	// 1
        th(     "SHE7");	// 1
        th(     "SHEB");	// 1
        th(     "shEE");	// 1
        th(    "SHEHS");	// 1
        th(    "shE'I");	// 1
        th(     "SHEM");	// 1
        th(    "SHEmS");	// 1
        th(    "SHEMS");	// 1
        th(  "SHENGYA");	// 1
        th("shENYTZA'I");	// 1
        th(    "^SHES");	// 1
        th(  "SHESHA:");	// 1
        th(    "SHESO");	// 1
        th(   "SHESPA");	// 1
        th(    "SHESS");	// 1
        th(  "SHES-SO");	// 1
        th(    "SHESU");	// 1
        th(   "SHE-SU");	// 1
        th(   "SHETZA");	// 1
        th(  "SHE'U'I");	// 1
        th(    "shGRA");	// 1
        th(  "shHA-SE");	// 1
        th(      "SHi");	// 1
        th(     "SH'i");	// 1
        th(    "'SH'I");	// 1
        th(    "SH'I:");	// 1
        th(     "SHI'");	// 1
        th(     "SHI4");	// 1
        th(   "SHI+'A");	// 1
        th(  "SHID'Am");	// 1
        th(  "SHI'I'O");	// 1
        th(     "SHIK");	// 1
        th(    "SHIKA");	// 1
        th(   "SHIK'A");	// 1
        th( "SHIKAshA");	// 1
        th("SHIKshASA");	// 1
        th(    "SH'IL");	// 1
        th( "SH'ILA'I");	// 1
        th(    "SHINN");	// 1
        th(   "shINTA");	// 1
        th( "SH'I-NTA");	// 1
        th(   "SHINTI");	// 1
        th(  "SH'INTI");	// 1
        th( "SH'IRshA");	// 1
        th(   "SH'Ish");	// 1
        th(  "SHIsh'A");	// 1
        th(  "SH'IshI");	// 1
        th("SHIshOSHTZA");	// 1
        th(  "SHIshtA");	// 1
        th( "SHIsht'A");	// 1
        th( "SHIshthA");	// 1
        th("SHIshthA'I");	// 1
        th(  "SHISTHA");	// 1
        th(     "shIt");	// 1
        th(   "SH'ITA");	// 1
        th(   "SHITNA");	// 1
        th(   "SHITZA");	// 1
        th(     "shKA");	// 1
        th(     "shKI");	// 1
        th(    "SHKKU");	// 1
        th(  "SHKNU'I");	// 1
        th(    "shKRi");	// 1
        th(   "SHKshI");	// 1
        th(   "SHKTHI");	// 1
        th(     "shKU");	// 1
        th( "SHLEshMA");	// 1
        th( "SHLEthMA");	// 1
        th(     "shLI");	// 1
        th("SHLishR+YA");	// 1
        th( "SHLKsh'I");	// 1
        th(    "SHLLE");	// 1
        th(    "SHLOG");	// 1
        th(   "SHLOOG");	// 1
        th(     "SHMA");	// 1
        th(   "SHMA'I");	// 1
        th("shMANSAMADZ'A");	// 1
        th(  "SH'MB'I");	// 1
        th("shM'INGNI");	// 1
        th(    "SHMRA");	// 1
        th(     "shnA");	// 1
        th(  "shnANTU");	// 1
        th(  "SH'NATI");	// 1
        th(     "shnI");	// 1
        th(     "SHNI");	// 1
        th("SHNINTRANG");	// 1
        th(    "shnMU");	// 1
        th(     "SHNO");	// 1
        th( "sh'NYTZA");	// 1
        th("SH'NYTZTSA");	// 1
        th(     "sh'O");	// 1
        th(     "S+HO");	// 1
        th(     "SHO3");	// 1
        th(     "shOD");	// 1
        th(   "shODTA");	// 1
        th(     "SHOK");	// 1
        th(  "SHOL-YI");	// 1
        th(    "SH'OO");	// 1
        th(     "shOS");	// 1
        th(   "shOshA");	// 1
        th(    "shOtO");	// 1
        th(     "shPA");	// 1
        th(    "SH+PA");	// 1
        th(    "shR'A");	// 1
        th(    "sHR'A");	// 1
        th(    "SHRA'");	// 1
        th("SHR'ADDHA");	// 1
        th("SHRADDH'A'I");	// 1
        th("SHRADDH'A'O");	// 1
        th("SHRADDH'O");	// 1
        th( "SHRADH'A");	// 1
        th("SHR'A'IDE");	// 1
        th( "SHR'AI'I");	// 1
        th(   "SHR'Am");	// 1
        th(    "SHRAM");	// 1
        th(   "SHRAMA");	// 1
        th( "SHRAMBHU");	// 1
        th( "SHRANIsh");	// 1
        th( "SHRANthI");	// 1
        th( "SHRDDH'A");	// 1
        th(  "SHRDDHE");	// 1
        th(    "SHRDI");	// 1
        th(    "SHR'E");	// 1
        th(     "shRI");	// 1
        th(    "shR'I");	// 1
        th(    "SHR'i");	// 1
        th(   "shRi'A");	// 1
        th( "SHR'IA'I");	// 1
        th( "SHRI'A'I");	// 1
        th( "SHR'I'AM");	// 1
        th( "SHRIDDHA");	// 1
        th( "SHR'IDHA");	// 1
        th(   "shRI'I");	// 1
        th(    "SHRim");	// 1
        th("SHR'IMH'A");	// 1
        th("shRINYTZA");	// 1
        th("SHRIRETHU'I");	// 1
        th(   "SHRI'S");	// 1
        th("SHRIY'ADIKSRBABHYA:");	// 1
        th("SHR'I:ZHES");	// 1
        th(   "SHRNIS");	// 1
        th(  "SHRnnAM");	// 1
        th(  "SHR'NTI");	// 1
        th(     "SHRO");	// 1
        th(   "shRtI:");	// 1
        th(   "SHRU'I");	// 1
        th(   "shRYAm");	// 1
        th(   "SH'shI");	// 1
        th(    "SHSHI");	// 1
        th( "shshth'I");	// 1
        th(    "SHSHU");	// 1
        th(      "sht");	// 1
        th(     "SH'T");	// 1
        th(    "SHt'A");	// 1
        th(    "SHTAR");	// 1
        th(     "shtE");	// 1
        th(    "shth'");	// 1
        th(    "shTHA");	// 1
        th(    "SHTHA");	// 1
        th( "shthANTU");	// 1
        th( "shth'A'O");	// 1
        th(   "sht'i:");	// 1
        th(   "shtI'I");	// 1
        th(    "shtOm");	// 1
        th(  "SHTS'AD");	// 1
        th(   "SHTSYA");	// 1
        th(   "SHTU'O");	// 1
        th(    "shTZA");	// 1
        th("SHTZARYAm");	// 1
        th(    "shTZI");	// 1
        th(  "SHTZI'I");	// 1
        th(    "SHTZO");	// 1
        th(   "SHTZYA");	// 1
        th(     "shU:");	// 1
        th(   "shU'AN");	// 1
        th(  "SHUBHAm");	// 1
        th("SHU-DADHI");	// 1
        th("SHUDDH'A:SARBA");	// 1
        th("SHUDDHONYAHAm:");	// 1
        th("SHUDDH'O:SARBA");	// 1
        th( "SHUDH'A:");	// 1
        th(   "SHUDHI");	// 1
        th("SHUDHONYA");	// 1
        th("SHUDHONYAHAm");	// 1
        th(  "SHUDHYA");	// 1
        th(   "SHUKLA");	// 1
        th(   "SHUM'A");	// 1
        th(   "SHUndI");	// 1
        th(    "shUNG");	// 1
        th(   "SHUNGS");	// 1
        th(  "SHUNTZI");	// 1
        th(   "shUNYA");	// 1
        th(  "SH'UNYA");	// 1
        th( "SHUNYTZA");	// 1
        th(    "SH'UR");	// 1
        th(  "SHURTZA");	// 1
        th(  "SHUshKA");	// 1
        th(    "sh'VA");	// 1
        th(   "SHV'A:");	// 1
        th(    "SHVAG");	// 1
        th( "SHV'AH'A");	// 1
        th("SHVAKYA'I");	// 1
        th(    "SHVAL");	// 1
        th(   "SHV'AN");	// 1
        th(   "SHVANA");	// 1
        th(   "SHVANG");	// 1
        th(   "SHVAR:");	// 1
        th("SHVAR:DPAL");	// 1
        th( "SHVASMEE");	// 1
        th("SHVASTAN+YA:");	// 1
        th(  "SHVATRI");	// 1
        th( "SHVATTHA");	// 1
        th(   "shV'I:");	// 1
        th(   "SHVIDI");	// 1
        th(   "SHVI'I");	// 1
        th(    "SHVOD");	// 1
        th(   "SHVSTA");	// 1
        th(    "SHVTA");	// 1
        th("SHWANG-THA'I");	// 1
        th("SHY'A'I'O");	// 1
        th(  "shYANTA");	// 1
        th(  "shYANTI");	// 1
        th( "shYA-NYA");	// 1
        th(   "SHYA'O");	// 1
        th(     "shYE");	// 1
        th(   "SHYE'O");	// 1
        th(     "shYI");	// 1
        th(   "SHYING");	// 1
        th(      "SI2");	// 1
        th(      "SI4");	// 1
        th(    "SI'AI");	// 1
        th(      "SIC");	// 1
        th(     "'SID");	// 1
        th(  "SIDDHA:");	// 1
        th(   "SIDDHm");	// 1
        th(   "SIDDHO");	// 1
        th(   "SIDDHU");	// 1
        th( "SIDDHYAm");	// 1
        th("SIDDHYANTA");	// 1
        th(     "SIDE");	// 1
        th(    "SIDHI");	// 1
        th(   "SIDH'I");	// 1
        th(  "SIDHI'I");	// 1
        th(   "SIDHYA");	// 1
        th( "SI-DIdhI");	// 1
        th(     "SIHA");	// 1
        th(    "SIKLI");	// 1
        th(  "SINDD'U");	// 1
        th( "SINDJU'I");	// 1
        th(   "SINDRA");	// 1
        th(    "SINDU");	// 1
        th(    "SINGA");	// 1
        th( "SINGDDHI");	// 1
        th("SINGDDHY'A");	// 1
        th(  "SINGDHI");	// 1
        th(  "SING-GA");	// 1
        th(  "SING-GE");	// 1
        th( "SING+GHA");	// 1
        th(  "SING-HA");	// 1
        th( "SINGHA'I");	// 1
        th( "SINGHI'I");	// 1
        th(   "SINGHU");	// 1
        th("SING+NGHA");	// 1
        th(    "SINHU");	// 1
        th(    "S'I'O");	// 1
        th(     "SIRD");	// 1
        th(   "SIRDHU");	// 1
        th("S'Ishsh'AS");	// 1
        th("S'IshthAS");	// 1
        th(   "SITHAR");	// 1
        th(    "SITZA");	// 1
        th(     "SJOm");	// 1
        th(    "S'JUG");	// 1
        th(     "SKA'");	// 1
        th(   "^SKABS");	// 1
        th("SKADKYANG");	// 1
        th(    "SKADU");	// 1
        th("SKANDA'MNDO");	// 1
        th("SKANDHASA");	// 1
        th(   "SKANDI");	// 1
        th(   "SKAR'A");	// 1
        th(   "SKARBS");	// 1
        th(   "SKAYES");	// 1
        th(     "SKD0");	// 1
        th(   "SKE'AM");	// 1
        th(     "SKEN");	// 1
        th(   "SKENGS");	// 1
        th(    "SKEYD");	// 1
        th(     "SKHA");	// 1
        th(    "SKHES");	// 1
        th(      "SKI");	// 1
        th(    "SKIBS");	// 1
        th(    "SKLOB");	// 1
        th(     "SKLU");	// 1
        th(     "SKOB");	// 1
        th(    "SKO'O");	// 1
        th( "SKRA'ANG");	// 1
        th(    "SKRAm");	// 1
        th(    "SKRAM");	// 1
        th(    "SKRED");	// 1
        th(   "SKREGS");	// 1
        th(   "SKRE'I");	// 1
        th(    "SKREM");	// 1
        th(    "SKREN");	// 1
        th(    "SKRIS");	// 1
        th(    "SKROR");	// 1
        th(    "SKROS");	// 1
        th(    "SKRUG");	// 1
        th(    "SKRUL");	// 1
        th(   "'SKRUN");	// 1
        th(     "SKUB");	// 1
        th(  "SKU'I'O");	// 1
        th(     "SKUJ");	// 1
        th(     "SKUm");	// 1
        th(    "SKUMS");	// 1
        th(  "SKU'O'I");	// 1
        th(  "SKYABGS");	// 1
        th(  "'SKYABS");	// 1
        th(  "SKYABSU");	// 1
        th(    "SKYAM");	// 1
        th(    "^SKYE");	// 1
        th(   "SKYEBA");	// 1
        th("SKYEBSTAN");	// 1
        th(   "^SKYED");	// 1
        th(   "SKYES2");	// 1
        th(   "SKYESK");	// 1
        th(  "SKYES'O");	// 1
        th( "SKYE'U'O");	// 1
        th(    "SKY'I");	// 1
        th(   "SKYI8L");	// 1
        th(   "SKYI'I");	// 1
        th(    "SKYIM");	// 1
        th(  "SKYINGS");	// 1
        th(   "SKYI'O");	// 1
        th(    "SKYO'");	// 1
        th(  "SKYOANG");	// 1
        th(   "SKYOGN");	// 1
        th(  "SKYONG:");	// 1
        th(   "SKYONS");	// 1
        th(     "SKYS");	// 1
        th(   "SKYUBS");	// 1
        th(   "SKYU'I");	// 1
        th(    "SKYUL");	// 1
        th(       "SL");	// 1
        th(     "SLA'");	// 1
        th(   "SLA-'O");	// 1
        th(      "SLB");	// 1
        th(    "SLDAD");	// 1
        th(    "SLDAN");	// 1
        th(    "SLDBS");	// 1
        th(     "SLEG");	// 1
        th(     "SLEM");	// 1
        th(    "SLE'O");	// 1
        th(     "SLER");	// 1
        th(  "SLETRES");	// 1
        th(  "SLE'U'I");	// 1
        th(      "SLi");	// 1
        th(     "SLIB");	// 1
        th(    "SLi'I");	// 1
        th(     "SLIN");	// 1
        th(   "SLINGS");	// 1
        th("SLOBA'LNGA");	// 1
        th(    "SLOMG");	// 1
        th(    "SLOMS");	// 1
        th( "SLORNAMS");	// 1
        th(    "SLO'U");	// 1
        th(     "SLRA");	// 1
        th(   "SLUNGS");	// 1
        th(   "SLURYA");	// 1
        th(       "SM");	// 1
        th(    "^SMAN");	// 1
        th(    "SMAtA");	// 1
        th(   "SM'ATA");	// 1
        th( "SM'ATA'O");	// 1
        th(   "SMATRA");	// 1
        th( "SMBANDHA");	// 1
        th(  "SMBHOtA");	// 1
        th(   "SMBHUt");	// 1
        th(   "SMCHOG");	// 1
        th(   "SMEE'O");	// 1
        th(   "SMEERU");	// 1
        th(    "SMEES");	// 1
        th(    "SME'I");	// 1
        th(    "SME'U");	// 1
        th(     "SMiG");	// 1
        th(    "'SMIN");	// 1
        th(    "SMINE");	// 1
        th(     "SMIR");	// 1
        th(   "SMNGON");	// 1
        th(   "SMONGS");	// 1
        th(      "SMR");	// 1
        th(    "'SMRA");	// 1
        th("SMRA'BA'I");	// 1
        th(   "SMRABS");	// 1
        th(    "SMRAG");	// 1
        th(    "SMRER");	// 1
        th(    "SMROD");	// 1
        th(     "SMRS");	// 1
        th(   "SM'TRA");	// 1
        th( "SM'TRA'O");	// 1
        th(     "SMUD");	// 1
        th(    "SMU'I");	// 1
        th(     "SMUN");	// 1
        th( "SMUNGP'A");	// 1
        th(    "SMYAK");	// 1
        th(   "SMYIGS");	// 1
        th(    "SMYIL");	// 1
        th(    "SMYOR");	// 1
        th(    "SMYUN");	// 1
        th(      "SnA");	// 1
        th(     "SNAm");	// 1
        th(    "SNAMG");	// 1
        th(  "SNA+TYA");	// 1
        th(    "SNA'U");	// 1
        th(    "SNDOM");	// 1
        th(   "SNE'AM");	// 1
        th(    "SNEMS");	// 1
        th(     "SNEN");	// 1
        th(    "SNENG");	// 1
        th(  "SNE'U'I");	// 1
        th(    "SNEYD");	// 1
        th(  "SNGA'AM");	// 1
        th(    "SNGAB");	// 1
        th( "SNGA-GHA");	// 1
        th(   "SNGARS");	// 1
        th(   "SNGASG");	// 1
        th(   "SNGASO");	// 1
        th(   "SNGDAR");	// 1
        th(    "SNGEB");	// 1
        th(    "SNGEM");	// 1
        th(    "SNGEN");	// 1
        th(   "SNGENG");	// 1
        th( "SNGE'U'I");	// 1
        th(     "SNGI");	// 1
        th(    "SNGIN");	// 1
        th(    "SNGOB");	// 1
        th(    "SNGOL");	// 1
        th(   "^SNGON");	// 1
        th(   "SNGO'O");	// 1
        th(   "SNGUBS");	// 1
        th(    "SNGUR");	// 1
        th(     "SNIG");	// 1
        th(    "SNI'I");	// 1
        th(     "SNIL");	// 1
        th(   "SNMA'O");	// 1
        th(    "SNNAG");	// 1
        th(     "SNNI");	// 1
        th(     "Sn'O");	// 1
        th(    "SNOGN");	// 1
        th(     "SNRA");	// 1
        th(    "SNRAL");	// 1
        th(    "SNRAS");	// 1
        th(   "SNTI'O");	// 1
        th(  "SNUKA'O");	// 1
        th(     "SNUL");	// 1
        th(     "SNUm");	// 1
        th(    "SNUMS");	// 1
        th(    "SNU'O");	// 1
        th(     "SNUS");	// 1
        th(   "SNYABS");	// 1
        th(   "SNYAMD");	// 1
        th(   "SNYAMO");	// 1
        th(   "SNYASM");	// 1
        th(     "SNYD");	// 1
        th(  "SNYENGS");	// 1
        th(    "SNYId");	// 1
        th( "SNYINGLA");	// 1
        th(    "SNYOm");	// 1
        th(   "SNYOMD");	// 1
        th(  "SNYOMGS");	// 1
        th(   "SNYONS");	// 1
        th(    "SNYOR");	// 1
        th(  "SNYOTZA");	// 1
        th(   "SNYTZA");	// 1
        th(    "SNYUB");	// 1
        th(    "SNYUR");	// 1
        th(      "'SO");	// 1
        th(      "SO:");	// 1
        th(      "SO'");	// 1
        th(      "SO3");	// 1
        th(    "SO9GS");	// 1
        th(     "SODM");	// 1
        th(     "SOGD");	// 1
        th(    "SOGGS");	// 1
        th(     "SOGN");	// 1
        th(    "SOGSC");	// 1
        th(   "SOGSED");	// 1
        th( "SOGSPA'I");	// 1
        th( "SOGSPA'O");	// 1
        th(     "SOHA");	// 1
        th(    "SO-HO");	// 1
        th(   "SO'I'O");	// 1
        th(     "SOKS");	// 1
        th(   "SOKY'A");	// 1
        th(    "SONAM");	// 1
        th(    "SOOGS");	// 1
        th(    "SOO'I");	// 1
        th(     "SOOS");	// 1
        th(    "SORNG");	// 1
        th(     "SORR");	// 1
        th(      "SOs");	// 1
        th(     "SOSL");	// 1
        th(   "SOSPAS");	// 1
        th(     "SOtA");	// 1
        th(    "SOTNG");	// 1
        th(     "SO'U");	// 1
        th(       "SP");	// 1
        th(     "SPA'");	// 1
        th(  "SPA'ANG");	// 1
        th(     "SPAB");	// 1
        th(    "SP'AI");	// 1
        th(    "SPANS");	// 1
        th(  "SPA'O'I");	// 1
        th("SPARASHA'I");	// 1
        th(   "SPARNG");	// 1
        th("SPARSHA'I");	// 1
        th(    "SPBAL");	// 1
        th(     "SPEM");	// 1
        th(     "SPER");	// 1
        th(   "SPE'UR");	// 1
        th(    "SPH'I");	// 1
        th(  "SPH'I'O");	// 1
        th(    "SPHOM");	// 1
        th(   "SPHONG");	// 1
        th(  "SPHRANG");	// 1
        th(   "SPHRIN");	// 1
        th(    "SPHRO");	// 1
        th(  "SPHUNGS");	// 1
        th(  "SPHYI'I");	// 1
        th(   "SPHYIN");	// 1
        th(   "SPHYIR");	// 1
        th(   "SPHYOD");	// 1
        th(     "SP'I");	// 1
        th(     "SPIG");	// 1
        th(   "SPINGS");	// 1
        th(     "SPIR");	// 1
        th(   "SPIRNG");	// 1
        th(   "SPLYOD");	// 1
        th(     "^SPO");	// 1
        th(     "SPOm");	// 1
        th(    "SPOMN");	// 1
        th(   "'SPONG");	// 1
        th(     "SPOT");	// 1
        th(      "SPR");	// 1
        th(   "SPRA'O");	// 1
        th(   "SPRA'U");	// 1
        th(    "SPRED");	// 1
        th(  "SPREE'U");	// 1
        th(   "SPRE'I");	// 1
        th(    "SPRIG");	// 1
        th(   "SPRI'U");	// 1
        th(   "SPROBS");	// 1
        th(   "SPRULS");	// 1
        th(  "SPRUNGS");	// 1
        th(     "SPTA");	// 1
        th(     "SP'U");	// 1
        th(     "SPU'");	// 1
        th(   "SPU'AM");	// 1
        th(     "SPUm");	// 1
        th(      "SPY");	// 1
        th(   "SPYAGS");	// 1
        th(   "SPY'AI");	// 1
        th(    "SPYAL");	// 1
        th(    "SPYAS");	// 1
        th(   "SPYE'I");	// 1
        th(    "SPY'I");	// 1
        th(    "SPYIG");	// 1
        th(   "SPYIGS");	// 1
        th(   "^SPYIR");	// 1
        th(     "SPYU");	// 1
        th(    "SPYUR");	// 1
        th(       "SR");	// 1
        th(     "^SRA");	// 1
        th(    "'S'RA");	// 1
        th(     "SRAK");	// 1
        th(   "SRANSU");	// 1
        th(    "SRA'O");	// 1
        th(    "'SRAR");	// 1
        th(    "S'RBA");	// 1
        th(     "SRDI");	// 1
        th(   "'SREGS");	// 1
        th(    "SREGT");	// 1
        th(    "SRE'I");	// 1
        th(    "'SREL");	// 1
        th(    "SREMS");	// 1
        th(    "SRE'O");	// 1
        th(     "SRER");	// 1
        th(    "SRGOD");	// 1
        th(    "SRGUD");	// 1
        th(    "SRGYA");	// 1
        th(    "SRGYU");	// 1
        th(   "SRGYUR");	// 1
        th("SRiDZIDRi");	// 1
        th(     "SRiL");	// 1
        th(     "SRIM");	// 1
        th(     "SRIR");	// 1
        th(     "SRiS");	// 1
        th(    "SRI'T");	// 1
        th(     "SRKD");	// 1
        th( "SR'KIshA");	// 1
        th(   "SRLUNG");	// 1
        th(   "SRMA'O");	// 1
        th(    "SROBS");	// 1
        th(   "^SRONG");	// 1
        th(     "SROR");	// 1
        th(  "SRPUNGS");	// 1
        th(     "SRSD");	// 1
        th(     "SRTG");	// 1
        th(     "SRTI");	// 1
        th(    "SRUGS");	// 1
        th(   "'SRUNG");	// 1
        th(  "'SRUNGS");	// 1
        th(     "SRVA");	// 1
        th("SRWANGH'A");	// 1
        th(    "SRY'A");	// 1
        th(      "SSA");	// 1
        th(     "SSAM");	// 1
        th(   "SSARBO");	// 1
        th(    "SSEMS");	// 1
        th(    "SSGRA");	// 1
        th(   "SSGRUB");	// 1
        th( "SSH'AKYA");	// 1
        th(    "SSHES");	// 1
        th(      "SSI");	// 1
        th(   "SSKYES");	// 1
        th(   "SSNANG");	// 1
        th(   "SSNGON");	// 1
        th(    "SSOGS");	// 1
        th(   "SSPYOD");	// 1
        th(  "SS'R'BA");	// 1
        th(    "SSROG");	// 1
        th(    "SSVAM");	// 1
        th(     "STA:");	// 1
        th("STABIDYAMUKTI:BODHYANTIK'ARUntRISHSHAR'AnAP'ATUm");	// 1
        th(    "ST'AM");	// 1
        th(  "STAMBHA");	// 1
        th(    "^STAN");	// 1
        th(    "STANS");	// 1
        th(    "ST'AR");	// 1
        th( "STARTING");	// 1
        th(     "STE:");	// 1
        th(     "STE'");	// 1
        th(     "STEL");	// 1
        th(     "STEM");	// 1
        th(   "^STENG");	// 1
        th(      "STG");	// 1
        th(     "SthA");	// 1
        th(    "STHA'");	// 1
        th(  "STH'A'I");	// 1
        th(   "STH'AM");	// 1
        th(   "STHAMS");	// 1
        th("STH'ANTU:");	// 1
        th(    "STHAR");	// 1
        th(   "STH'AS");	// 1
        th(     "STHE");	// 1
        th(    "STH'E");	// 1
        th(     "SthI");	// 1
        th(    "STHOG");	// 1
        th(    "STHON");	// 1
        th(    "STHUM");	// 1
        th(    "STIBS");	// 1
        th(    "STIMS");	// 1
        th(   "STINGS");	// 1
        th(    "STNEG");	// 1
        th(     "'STO");	// 1
        th(    "STO9N");	// 1
        th(    "STOAS");	// 1
        th(     "STOL");	// 1
        th(   "STONGG");	// 1
        th(     "STOO");	// 1
        th(   "STRA'I");	// 1
        th(    "STRAm");	// 1
        th(   "STRANG");	// 1
        th(    "STRIN");	// 1
        th(    "STSON");	// 1
        th(     "STUD");	// 1
        th(     "STUM");	// 1
        th(    "STV'A");	// 1
        th(    "STVAS");	// 1
        th(     "STVI");	// 1
        th(     "STYA");	// 1
        th(   "STYA'I");	// 1
        th(   "STZAGS");	// 1
        th(  "STZALDA");	// 1
        th(    "STZAM");	// 1
        th(    "STZAS");	// 1
        th(    "STZEL");	// 1
        th(   "STZOLD");	// 1
        th(    "STZOM");	// 1
        th(   "STZONG");	// 1
        th(    "STZUR");	// 1
        th(      "'SU");	// 1
        th(   "SUBCAD");	// 1
        th(  "SU-BH'A");	// 1
        th(   "SUC'UN");	// 1
        th(     "S'UD");	// 1
        th( "SUDHARME");	// 1
        th(   "SUDH'I");	// 1
        th(  "SUDH'I:");	// 1
        th( "SUDH'I'O");	// 1
        th(    "SUGTA");	// 1
        th(    "S'U'I");	// 1
        th(  "SUKshMA");	// 1
        th(     "S'Um");	// 1
        th(    "SUMDU");	// 1
        th(    "SUNYA");	// 1
        th(    "SU'OS");	// 1
        th(      "SUP");	// 1
        th(    "SUPRA");	// 1
        th(     "S'UR");	// 1
        th( "SURENDRA");	// 1
        th(  "S'UR+YA");	// 1
        th(   "SUR+YA");	// 1
        th(  "SURYA'I");	// 1
        th(      "SUt");	// 1
        th(   "S'UTRA");	// 1
        th(      "SUY");	// 1
        th(     "SVA:");	// 1
        th(  "SVA'AHA");	// 1
        th( "SV'ABH'A");	// 1
        th(  "SVA'H'A");	// 1
        th("SV'A-HA'A");	// 1
        th(  "SVAHA'A");	// 1
        th("SV'AH'A'I");	// 1
        th(   "SVAH'I");	// 1
        th(  "SV'AH'U");	// 1
        th(    "SVA'I");	// 1
        th(  "SVAMBHA");	// 1
        th(    "SVANA");	// 1
        th(  "SVANDZI");	// 1
        th(    "SVANI");	// 1
        th( "SVANYDZI");	// 1
        th(     "SVAP");	// 1
        th(   "SVARBA");	// 1
        th(   "SVARTI");	// 1
        th(     "SVAS");	// 1
        th(  "SVAshTA");	// 1
        th(   "SVASTE");	// 1
        th(  "S'VASTI");	// 1
        th( "SVASTI'I");	// 1
        th(  "SVASTVI");	// 1
        th(      "SVE");	// 1
        th(   "SV'H'A");	// 1
        th(  "SVIDHYA");	// 1
        th(     "SVOL");	// 1
        th(     "SVOO");	// 1
        th(    "SVSTI");	// 1
        th(    "S+YAm");	// 1
        th(    "SYANT");	// 1
        th(    "SYATI");	// 1
        th(  "SYBANGS");	// 1
        th(     "SY'E");	// 1
        th(     "SYED");	// 1
        th(   "S+YE'I");	// 1
        th(    "SYE'O");	// 1
        th(     "SYER");	// 1
        th(      "SYi");	// 1
        th(     "SYIN");	// 1
        th(    "SYOMS");	// 1
        th(    "SYONG");	// 1
        th(     "SYOO");	// 1
        th(     "SYOR");	// 1
        th(   "SYSEMS");	// 1
        th(   "SY'T'O");	// 1
        th(     "SZEL");	// 1
        th(    "T2ZAM");	// 1
        th(      "tA:");	// 1
        th(     "T'A:");	// 1
        th(      "TA4");	// 1
        th(    "tA'AM");	// 1
        th(   "tA'ANG");	// 1
        th(     "TAAS");	// 1
        th(    "TABHA");	// 1
        th(     "'TAD");	// 1
        th(     "TAdA");	// 1
        th(   "TADDHI");	// 1
        th(    "TADHI");	// 1
        th(    "TAD'I");	// 1
        th(   "TAD+YA");	// 1
        th(  "TADZDZA");	// 1
        th( "TADZDZHA");	// 1
        th(      "TAE");	// 1
        th(      "TAH");	// 1
        th(     "TAHL");	// 1
        th(     "TAHR");	// 1
        th(    "t'A'I");	// 1
        th( "TA'I'ANG");	// 1
        th(   "TA'I'I");	// 1
        th(  "tA'IKKA");	// 1
        th(      "TAK");	// 1
        th(    "tA'KA");	// 1
        th(    "t'AKI");	// 1
        th(    "TAKKA");	// 1
        th(    "TAKKE");	// 1
        th(   "TAKshA");	// 1
        th(     "T'AL");	// 1
        th(    "TALLU");	// 1
        th(     "tA'm");	// 1
        th(     "TAM:");	// 1
        th(    "T'AMA");	// 1
        th(   "TAMA'O");	// 1
        th(    "TAM'O");	// 1
        th( "T'AMRA'O");	// 1
        th(     "tANG");	// 1
        th(    "'TANG");	// 1
        th(   "TANGGI");	// 1
        th(    "TANNA");	// 1
        th(   "TANTAS");	// 1
        th(   "T'ANTE");	// 1
        th(    "TA-NU");	// 1
        th(    "TANU:");	// 1
        th(    "t'A'O");	// 1
        th(    "TA'OS");	// 1
        th(      "TAP");	// 1
        th(     "TAPA");	// 1
        th(    "TAPAL");	// 1
        th(    "TA'RI");	// 1
        th(    "TARKA");	// 1
        th(    "TARKI");	// 1
        th(   "TARTHA");	// 1
        th(    "TASBS");	// 1
        th(   "T'ASMA");	// 1
        th(  "T'ASMAS");	// 1
        th(    "TASMI");	// 1
        th(    "TASRU");	// 1
        th(  "TA+STHA");	// 1
        th(  "T'ASWAS");	// 1
        th(   "tATA'O");	// 1
        th(   "TA-TAR");	// 1
        th(    "TATHA");	// 1
        th(   "TATKRI");	// 1
        th(  "TATshVE");	// 1
        th( "TATSSHVA");	// 1
        th("T'ATT'A'O");	// 1
        th( "TATTRASA");	// 1
        th(   "TATTVA");	// 1
        th(    "TATVA");	// 1
        th(    "TATZA");	// 1
        th("TATZETSVA");	// 1
        th( "TATZSH'U");	// 1
        th( "TATZshVI");	// 1
        th(  "TATZTSA");	// 1
        th( "TATZTSA'");	// 1
        th(  "TATZTSI");	// 1
        th( "TATZTSVI");	// 1
        th(     "TA'U");	// 1
        th(    "tA'UR");	// 1
        th(  "TA:ZHES");	// 1
        th(     "TBAR");	// 1
        th(   "TBCA'I");	// 1
        th(     "TCAN");	// 1
        th(    "TCHED");	// 1
        th(     "TDAD");	// 1
        th(    "TddHO");	// 1
        th(     "TddI");	// 1
        th(      "tDE");	// 1
        th(     "TDGE");	// 1
        th(    "TDPA'");	// 1
        th(     "TDRA");	// 1
        th(      "TdU");	// 1
        th(      "TDU");	// 1
        th(    "TDUGS");	// 1
        th(    "TDZAS");	// 1
        th(   "TDZDZA");	// 1
        th(   "T'DZIN");	// 1
        th(      "'TE");	// 1
        th(      "TE2");	// 1
        th(    "TEBOD");	// 1
        th(     "TE'E");	// 1
        th(     "TEKA");	// 1
        th(      "tEm");	// 1
        th(     "T'EN");	// 1
        th(    "TENGS");	// 1
        th( "TENYATRA");	// 1
        th(  "TENYTRA");	// 1
        th(  "TE'ONGS");	// 1
        th(    "TEPRA");	// 1
        th(     "TER:");	// 1
        th("TEsh'ANDA'O");	// 1
        th("TEsh'NYTZA");	// 1
        th(  "TEshTZA");	// 1
        th(  "TESHTZE");	// 1
        th(    "TETNA");	// 1
        th(  "TE'U'AM");	// 1
        th(    "TE'UR");	// 1
        th(    "TE'US");	// 1
        th(    "TGNOD");	// 1
        th(      "TGO");	// 1
        th(      "TGS");	// 1
        th(     "TGYI");	// 1
        th(   "TGZHAN");	// 1
        th(   "TGZUGS");	// 1
        th(      "'TH");	// 1
        th(    "TH2OB");	// 1
        th(      "TH6");	// 1
        th(     "th'A");	// 1
        th(      "tHA");	// 1
        th(     "THA:");	// 1
        th(     "THA2");	// 1
        th(     "THA4");	// 1
        th(   "THA'AM");	// 1
        th(   "^'THAB");	// 1
        th(    "THABs");	// 1
        th(  "THABSLA");	// 1
        th("THABSSHES");	// 1
        th(  "THADZAM");	// 1
        th(    "THAGN");	// 1
        th( "THAGZHUS");	// 1
        th(     "THAK");	// 1
        th( "THA:KHAm");	// 1
        th(     "thAL");	// 1
        th("THALKHYOD");	// 1
        th(    "THALR");	// 1
        th(    "TH'Am");	// 1
        th(    "THAMD");	// 1
        th(    "THAMG");	// 1
        th(   "THAM'O");	// 1
        th(   "THAmSD");	// 1
        th(  "'THANGS");	// 1
        th(    "thA'O");	// 1
        th(   "TH'A'O");	// 1
        th(   "THA:'O");	// 1
        th(   "THARDA");	// 1
        th( "THARPA'I");	// 1
        th(   "thAT'O");	// 1
        th(      "THB");	// 1
        th(   "THDANG");	// 1
        th(      "thE");	// 1
        th(     "THE:");	// 1
        th(     "THE2");	// 1
        th(     "THE4");	// 1
        th(   "THE'AM");	// 1
        th(    "'THEG");	// 1
        th(   "'THEGS");	// 1
        th(    "THE'O");	// 1
        th(    "'THES");	// 1
        th(     "'THG");	// 1
        th(    "THHOS");	// 1
        th(      "thi");	// 1
        th(      "thI");	// 1
        th(     "THI2");	// 1
        th(     "THID");	// 1
        th(    "'THIG");	// 1
        th(   "'THI'I");	// 1
        th(  "THILA'I");	// 1
        th(    "'THIM");	// 1
        th(    "'THIN");	// 1
        th(    "THI'U");	// 1
        th(    "THKHA");	// 1
        th(     "THKU");	// 1
        th(     "THLA");	// 1
        th(    "THLIm");	// 1
        th(   "THLimI");	// 1
        th(     "THMS");	// 1
        th(     "th'O");	// 1
        th(     "THO3");	// 1
        th(   "THO'AM");	// 1
        th(    "THOBB");	// 1
        th("THOBBSTAN");	// 1
        th(    "THOBK");	// 1
        th(    "THOmS");	// 1
        th(    "THOMS");	// 1
        th(   "'THONB");	// 1
        th(    "THO'O");	// 1
        th(  "THO'O'I");	// 1
        th( "THOSPA'I");	// 1
        th("THO'TSAMS");	// 1
        th(     "THRA");	// 1
        th(     "THRI");	// 1
        th(  "THRINYA");	// 1
        th(    "THSES");	// 1
        th(     "THU4");	// 1
        th(    "THUAN");	// 1
        th(   "'THUBS");	// 1
        th(    "THUGI");	// 1
        th(   "'THUGN");	// 1
        th(   "^THUGS");	// 1
        th(    "THUIB");	// 1
        th(    "THUMS");	// 1
        th(   "THUNG'");	// 1
        th(     "THVA");	// 1
        th(   "THVAm:");	// 1
        th(    "THYUS");	// 1
        th(      "tI'");	// 1
        th(     "T''I");	// 1
        th(    "tI+'A");	// 1
        th(    "tIAKA");	// 1
        th(    "tIddA");	// 1
        th( "TIDZNY'A");	// 1
        th(     "tIGA");	// 1
        th(     "ti'I");	// 1
        th(    "t'I'I");	// 1
        th(    "'t'IK");	// 1
        th(     "tI'K");	// 1
        th(    "ti-KA");	// 1
        th(    "tI'KA");	// 1
        th(    "TIK'A");	// 1
        th(    "tIKAK");	// 1
        th(   "tIKAKA");	// 1
        th(    "t'IKK");	// 1
        th(  "t'IK-KA");	// 1
        th(  "T'IKK'A");	// 1
        th(   "tIKKAm");	// 1
        th(  "T'IKKAR");	// 1
        th(  "tiKKARA");	// 1
        th(  "t'IKKAS");	// 1
        th( "t'IKKI'I");	// 1
        th(   "t'IKTU");	// 1
        th(      "tIm");	// 1
        th(      "TIm");	// 1
        th(   "TIND'A");	// 1
        th(    "TINDU");	// 1
        th(   "TIN-DU");	// 1
        th(  "TINGA'I");	// 1
        th(   "TI-NGE");	// 1
        th(  "TING-GA");	// 1
        th(  "TING+GA");	// 1
        th( "T'IN+YAm");	// 1
        th(  "T'IRTHA");	// 1
        th(   "TIRTHI");	// 1
        th(    "T'ISA");	// 1
        th(    "TIS'A");	// 1
        th(    "TISGS");	// 1
        th(    "tIshA");	// 1
        th(  "TIsh'RI");	// 1
        th(  "TIshshA");	// 1
        th("TIshshA'O");	// 1
        th(   "TIshth");	// 1
        th( "TIshthA:");	// 1
        th(  "TIshtHA");	// 1
        th("TIshthA'O");	// 1
        th(   "TISH'U");	// 1
        th(   "TIsthA");	// 1
        th(  "TISTH'I");	// 1
        th(   "tKHYAD");	// 1
        th(   "t'KIRA");	// 1
        th(   "t'KK'I");	// 1
        th(  "t'KKI'S");	// 1
        th(     "TLAR");	// 1
        th(     "tLAS");	// 1
        th(      "TLI");	// 1
        th(    "TLTAR");	// 1
        th(    "TLTOS");	// 1
        th(     "TLUN");	// 1
        th(    "TLUNG");	// 1
        th(      "TmA");	// 1
        th(     "T+MA");	// 1
        th(     "TM'A");	// 1
        th(    "TMAHR");	// 1
        th(    "TMHA'");	// 1
        th(   "tMNGON");	// 1
        th(   "TMSAMS");	// 1
        th(    "TMSAN");	// 1
        th(  "TMTSAMS");	// 1
        th(       "TN");	// 1
    }

    /** Tests some of the tsheg bars that appear only once (likely
     *  these are typos or non-native). */
    public void test1to1c() {
        th(      "TnA");	// 1
        th(     "TNA:");	// 1
        th(    "TNA'I");	// 1
        th(     "TNAS");	// 1
        th(    "TNGNA");	// 1
        th(     "TN'I");	// 1
        th(     "TnnA");	// 1
        th(   "TNYNYA");	// 1
        th(      "t'O");	// 1
        th(     "T''O");	// 1
        th(      "TO9");	// 1
        th(      "tOG");	// 1
        th(     "'TOG");	// 1
        th(    "T'O'I");	// 1
        th(      "tOm");	// 1
        th(      "tON");	// 1
        th(  "TONGBHA");	// 1
        th(    "TONTO");	// 1
        th(    "TONYA");	// 1
        th("TONYANR'iTZTSA:");	// 1
        th(   "TONZAD");	// 1
        th(      "tOO");	// 1
        th(    "TOO'I");	// 1
        th(     "tORA");	// 1
        th(    "TORGE");	// 1
        th(    "TOR'I");	// 1
        th(  "TORTH'A");	// 1
        th(    "TOSGS");	// 1
        th(  "TOSHTZA");	// 1
        th(     "TO'U");	// 1
        th(    "TPA'I");	// 1
        th(    "TPALA");	// 1
        th(     "tPAR");	// 1
        th(      "TPO");	// 1
        th(      "tRA");	// 1
        th(     "T'RA");	// 1
        th(   "TRA'Am");	// 1
        th(   "TRA'AM");	// 1
        th(     "TRAD");	// 1
        th(     "TRAK");	// 1
        th( "TRAKshAD");	// 1
        th("TRAKSHADA");	// 1
        th( "TRAKshAR");	// 1
        th(     "TRAL");	// 1
        th(    "tR'Am");	// 1
        th(    "T'RAm");	// 1
        th(    "TRA'm");	// 1
        th(     "TRAM");	// 1
        th(    "TR'AM");	// 1
        th(    "TRAMS");	// 1
        th("TR'ARTHA:");	// 1
        th(    "TRAtA");	// 1
        th(     "TRE:");	// 1
        th(     "TRE+");	// 1
        th(     "TREL");	// 1
        th(    "TRENG");	// 1
        th(     "tR'I");	// 1
        th(    "TR'I:");	// 1
        th(     "tRIG");	// 1
        th(    "TRIGS");	// 1
        th(    "tR'IK");	// 1
        th(   "TRishI");	// 1
        th("TRIshNUPMI");	// 1
        th("TRIshtUMMI");	// 1
        th("TRishtUPCHRU");	// 1
        th("TRishtUPSHRU");	// 1
        th( "TRITZTSA");	// 1
        th(  "TRI'U'I");	// 1
        th(     "TR'm");	// 1
        th(    "TR'mA");	// 1
        th(      "TRO");	// 1
        th(    "TROGS");	// 1
        th("TROOshthAm");	// 1
        th(    "TRTOG");	// 1
        th(     "TRU:");	// 1
        th(    "TRU'I");	// 1
        th(    "TR'Um");	// 1
        th(     "TRUN");	// 1
        th(     "TRYE");	// 1
        th(    "TRYEE");	// 1
        th(     "TRZE");	// 1
        th(       "ts");	// 1
        th(     "T+SA");	// 1
        th(     "TSA2");	// 1
        th(   "TSACHA");	// 1
        th("TSAD'BRAS");	// 1
        th( "TSADMA'I");	// 1
        th(    "TS'AN");	// 1
        th(  "TSANDRA");	// 1
        th(  "TSANGGI");	// 1
        th(  "TSANGGS");	// 1
        th(    "TSANS");	// 1
        th(   "'TSASL");	// 1
        th(   "TSATSA");	// 1
        th(   "TSA'UB");	// 1
        th(     "^TSE");	// 1
        th(     "TSE2");	// 1
        th(     "TSE4");	// 1
        th(     "TSE7");	// 1
        th(   "TSE'Am");	// 1
        th(     "TSEB");	// 1
        th(   "'TSEBS");	// 1
        th(   "'TSEGS");	// 1
        th(   "TSEMBU");	// 1
        th(    "'TSEN");	// 1
        th("TSENDNTI'O");	// 1
        th(   "TSENGS");	// 1
        th(    "TSERC");	// 1
        th(    "TSES1");	// 1
        th(   "TSES11");	// 1
        th(   "TSES26");	// 1
        th(    "TSES7");	// 1
        th(    "TSES8");	// 1
        th(     "TSET");	// 1
        th(      "TSG");	// 1
        th(     "TSGI");	// 1
        th(    "TSGIS");	// 1
        th(    "TSGSO");	// 1
        th(    "tsHAR");	// 1
        th(     "T+SI");	// 1
        th(     "TSI2");	// 1
        th(     "TSI4");	// 1
        th(    "TSIDI");	// 1
        th(   "TSIGGI");	// 1
        th(   "TSIGLA");	// 1
        th(   "^TSIGS");	// 1
        th(    "TSI'I");	// 1
        th(    "'TSIN");	// 1
        th(     "TSO'");	// 1
        th(     "TSO3");	// 1
        th(   "TSO'AM");	// 1
        th("TSOGS'ONG");	// 1
        th(  "TSOGSPA");	// 1
        th(    "TS'ON");	// 1
        th(    "^TSOR");	// 1
        th(     "TSRA");	// 1
        th(    "TSSAD");	// 1
        th(   "TSSOGS");	// 1
        th(     "TSU4");	// 1
        th(   "'TSUBB");	// 1
        th(     "TSUd");	// 1
        th(    "TSU'I");	// 1
        th(   "TSULGS");	// 1
        th(    "TSULR");	// 1
        th(    "'TSUM");	// 1
        th(   "'TSUMS");	// 1
        th(  "TSUNDHA");	// 1
        th(    "TSU'O");	// 1
        th(    "'TSUS");	// 1
        th(    "TSUUL");	// 1
        th(    "TSVAB");	// 1
        th(    "TSWUL");	// 1
        th(     "TT'A");	// 1
        th(    "TTAGS");	// 1
        th(     "TTAm");	// 1
        th(     "ttAR");	// 1
        th(     "TTAR");	// 1
        th("TTARSYABODHOOYOGEBHYONYDHIssHANAmKURBANTU");	// 1
        th(      "ttE");	// 1
        th(    "TTE'O");	// 1
        th(     "tthA");	// 1
        th(   "T'THAD");	// 1
        th(   "TththA");	// 1
        th(      "ttI");	// 1
        th(    "TTI'I");	// 1
        th(   "TTIN'A");	// 1
        th(    "TTI'O");	// 1
        th(     "TTPA");	// 1
        th(     "TTRA");	// 1
        th(    "tTSUN");	// 1
        th(    "Ttt'I");	// 1
        th(      "TTU");	// 1
        th(     "TTYA");	// 1
        th(     "T'U:");	// 1
        th(      "TU'");	// 1
        th(      "TU2");	// 1
        th(   "tU'ANG");	// 1
        th(     "TUBS");	// 1
        th(     "tU'I");	// 1
        th(      "TUI");	// 1
        th(    "TU''I");	// 1
        th(   "TU'JUG");	// 1
        th(    "tUKRi");	// 1
        th(      "tUm");	// 1
        th(     "t'Um");	// 1
        th(   "TUMA'I");	// 1
        th(    "TUMHA");	// 1
        th(    "TUndE");	// 1
        th(    "TUNTA");	// 1
        th(    "tUNYA");	// 1
        th(    "tU:'O");	// 1
        th(    "TU''O");	// 1
        th(      "tUR");	// 1
        th(    "TURMU");	// 1
        th(   "TU:SHA");	// 1
        th(  "T'UshnI");	// 1
        th(   "TUTT'A");	// 1
        th(     "TU'U");	// 1
        th(     "TVAD");	// 1
        th(    "TVADA");	// 1
        th(    "TVAKS");	// 1
        th(     "TVAM");	// 1
        th(    "TVAMA");	// 1
        th(    "TV'AN");	// 1
        th(    "TVARA");	// 1
        th("TVARTHTHE");	// 1
        th(    "TVDKA");	// 1
        th(     "tV'I");	// 1
        th(    "TVI'I");	// 1
        th(   "tVIKAS");	// 1
        th(  "tVIKKAR");	// 1
        th(     "TVO:");	// 1
        th(      "TWA");	// 1
        th(     "TYAD");	// 1
        th(  "TYADA'I");	// 1
        th(   "T+YA'I");	// 1
        th(  "TYA-''I");	// 1
        th(  "TYAKshA");	// 1
        th( "TYANGG'I");	// 1
        th(  "TYA-NTA");	// 1
        th(    "TYA'O");	// 1
        th(     "TYAS");	// 1
        th(    "TYDRA");	// 1
        th(      "TYI");	// 1
        th(     "TY'I");	// 1
        th(    "TY''I");	// 1
        th(      "tYO");	// 1
        th(  "TYSHTZA");	// 1
        th(     "TYVA");	// 1
        th(       "TZ");	// 1
        th(     "TZA2");	// 1
        th(     "TZA4");	// 1
        th(    "TZA'A");	// 1
        th(    "TZ'AB");	// 1
        th(    "TZABS");	// 1
        th(  "TZADKRA");	// 1
        th(   "TZADNA");	// 1
        th( "TZADRA'I");	// 1
        th(    "TZAGS");	// 1
        th(    "TZ'AI");	// 1
        th(     "TZAK");	// 1
        th(  "TZAKA'I");	// 1
        th(  "TZAK'Am");	// 1
        th( "TZAKKE'O");	// 1
        th(   "TZAKKO");	// 1
        th("TZAKKRE'O");	// 1
        th( "TZAKRA'O");	// 1
        th("TZAKshING");	// 1
        th(  "TZAMB'U");	// 1
        th(    "TZAMP");	// 1
        th(  "TZANA'I");	// 1
        th( "TZANANDA");	// 1
        th(  "TZA-ndA");	// 1
        th(  "TZANd'A");	// 1
        th( "TZA-N+DA");	// 1
        th( "TZAndA'I");	// 1
        th("TZA-N+DA-NA");	// 1
        th(   "TZANDN");	// 1
        th( "TZANDR'A");	// 1
        th( "TZANDRA:");	// 1
        th("TZANDRA'ANG");	// 1
        th(  "TZANGGA");	// 1
        th(   "TZANGN");	// 1
        th(   "TZANGS");	// 1
        th(   "TZAN'I");	// 1
        th(   "TZANNA");	// 1
        th(   "TZANPI");	// 1
        th(   "TZANTI");	// 1
        th(   "TZA'OS");	// 1
        th(   "TZARMA");	// 1
        th(   "TZARPI");	// 1
        th(  "TZA'RYA");	// 1
        th( "TZARY'AM");	// 1
        th( "TZ'ARYAR");	// 1
        th("TZAR+YASA");	// 1
        th("TZAshISATH");	// 1
        th(  "TZATA'O");	// 1
        th(   "TZAT'O");	// 1
        th(    "TZATS");	// 1
        th(  "TZATSAB");	// 1
        th( "TZATSA'O");	// 1
        th(   "TZ'AtU");	// 1
        th(    "TZ'AU");	// 1
        th(   "TZA+YA");	// 1
        th(  "TZA+YA'");	// 1
        th(   "TZCHAS");	// 1
        th(     "TZE2");	// 1
        th(    "TZEGS");	// 1
        th(  "TZE'L'A");	// 1
        th(    "'TZEM");	// 1
        th(    "TZEmS");	// 1
        th(    "TZENG");	// 1
        th(    "TZE'O");	// 1
        th( "TZE-TATE");	// 1
        th(  "TZE-TSA");	// 1
        th(    "TZHIG");	// 1
        th(      "TZi");	// 1
        th(     "TZI4");	// 1
        th(   "TZI'AM");	// 1
        th(    "TZIKA");	// 1
        th(   "TZIK'A");	// 1
        th(   "TZINdA");	// 1
        th(   "TZINDA");	// 1
        th(   "TZIN'I");	// 1
        th( "TZINTASA");	// 1
        th(   "TZINTI");	// 1
        th(  "TZINTIS");	// 1
        th(   "TZINYA");	// 1
        th( "TZIRRA'I");	// 1
        th(     "TZIT");	// 1
        th(   "TZITA:");	// 1
        th(    "TZITT");	// 1
        th(  "TZITTAL");	// 1
        th(  "TZITTHA");	// 1
        th(  "TZI-ttI");	// 1
        th(  "TZITTRA");	// 1
        th(   "TZITYA");	// 1
        th(  "TZITZAM");	// 1
        th(   "TZKshU");	// 1
        th(     "TZNA");	// 1
        th(    "TZNDA");	// 1
        th(    "TZndI");	// 1
        th(   "TZNDRA");	// 1
        th(    "TZNNA");	// 1
        th(     "'TZO");	// 1
        th(     "TZO3");	// 1
        th(    "TZOGS");	// 1
        th(   "TZONTA");	// 1
        th(  "TZRYASA");	// 1
        th(  "TZshA'O");	// 1
        th(   "TZ-TSA");	// 1
        th("TZTSANDHO");	// 1
        th( "TZTSANTU");	// 1
        th(   "TZTSAR");	// 1
        th(   "TZTSED");	// 1
        th(   "TZTSID");	// 1
        th("TZTSIshtA");	// 1
        th( "TZTSUNDA");	// 1
        th("TZTSUNDHA");	// 1
        th("TZTSUshMA");	// 1
        th( "TZTSUSMA");	// 1
        th(    "TZTTI");	// 1
        th(   "TZTZEE");	// 1
        th(     "TZU4");	// 1
        th(    "TZ'UN");	// 1
        th(   "TZUndA");	// 1
        th(  "TZUND'A");	// 1
        th(   "TZUNDE");	// 1
        th(   "TZURYA");	// 1
        th(   "TZUSMA");	// 1
        th(     "TZVE");	// 1
        th(   "TZ+YAm");	// 1
        th(     "TZYE");	// 1
        th(     "TZYI");	// 1
        th(    "UBA'I");	// 1
        th(       "UD");	// 1
        th(     "'UDN");	// 1
        th(  "U'DRA'O");	// 1
        th(      "UJG");	// 1
        th(      "ULL");	// 1
        th(      "UMA");	// 1
        th(  "UNCLEAR");	// 1
        th(      "UNG");	// 1
        th(       "UP");	// 1
        th(   "URGYAL");	// 1
        th(      "USM");	// 1
        th(    "USONG");	// 1
        th(     "USTE");	// 1
        th("UZHALGDAMS");	// 1
        th(    "UZHES");	// 1
        th(   "UZHING");	// 1
        th(      "'VA");	// 1
        th(     "VA'I");	// 1
        th(   "V'BREL");	// 1
        th(      "VDU");	// 1
        th(    "VMA'I");	// 1
        th(     "VPAS");	// 1
        th(     "VSUL");	// 1
        th(        "W");	// 1
        th(      "WA'");	// 1
        th(      "WA2");	// 1
        th(      "WA4");	// 1
        th(     "WA'A");	// 1
        th(   "WA'ANG");	// 1
        th(    "WAd'A");	// 1
        th(  "WADDHAR");	// 1
        th(    "WADHU");	// 1
        th(   "W'ADMA");	// 1
        th(   "WA'DMA");	// 1
        th( "WAENDRA:");	// 1
        th(  "W'AGH'A");	// 1
        th(   "WA'GMA");	// 1
        th( "W'AKKH'A");	// 1
        th("W'AKLUDHI:");	// 1
        th(   "WA'KRA");	// 1
        th( "W'AKshLA");	// 1
        th( "W'AKSH'U");	// 1
        th(  "W'AKSTU");	// 1
        th("W'AKSUDH'A");	// 1
        th(    "WALLE");	// 1
        th(   "W'ALLU");	// 1
        th(   "WAMRMA");	// 1
        th(      "WAn");	// 1
        th(   "WANDHE");	// 1
        th(  "WA'ndHO");	// 1
        th(   "WA'ndI");	// 1
        th(    "WANGS");	// 1
        th(   "WANT'I");	// 1
        th(  "WANTI'O");	// 1
        th(    "WAN'U");	// 1
        th( "WA'NYCHA");	// 1
        th( "W'ANYDZA");	// 1
        th( "W'ANYNYA");	// 1
        th( "WA'NYSHA");	// 1
        th(   "WARKAS");	// 1
        th(   "WAR+MA");	// 1
        th(   "WARM'A");	// 1
        th(  "WARMMAS");	// 1
        th(   "WARMRA");	// 1
        th(    "WARNA");	// 1
        th(  "WARnA'O");	// 1
        th(   "WARnAR");	// 1
        th(  "WARNGHA");	// 1
        th(   "WARnnA");	// 1
        th( "WA-Rn+nA");	// 1
        th(   "WART'A");	// 1
        th(     "WATA");	// 1
        th(   "WATA'O");	// 1
        th(    "WATRA");	// 1
        th(    "WATZA");	// 1
        th( "WA'TZA'O");	// 1
        th(      "WBU");	// 1
        th(      "WE3");	// 1
        th(      "WE4");	// 1
        th(     "WE'E");	// 1
        th(     "WEEL");	// 1
        th(      "WEG");	// 1
        th(    "WEN'A");	// 1
        th(   "WENTRA");	// 1
        th(      "WI4");	// 1
        th(  "W'IDY'A");	// 1
        th(   "WIDY'A");	// 1
        th(     "WI'I");	// 1
        th(      "WIN");	// 1
        th(     "WING");	// 1
        th(    "WIPED");	// 1
        th(      "WIS");	// 1
        th(    "WLAGS");	// 1
        th(      "WON");	// 1
        th(      "WOR");	// 1
        th(  "WSHAshA");	// 1
        th(      "WU4");	// 1
        th(     "WU'U");	// 1
        th(    "WVANG");	// 1
        th(      "WYA");	// 1
        th(     "Y'A:");	// 1
        th(     "YA':");	// 1
        th(      "YA2");	// 1
        th(      "YAA");	// 1
        th(    "YA'AM");	// 1
        th(     "^YAB");	// 1
        th(     "Y'AB");	// 1
        th(     "YABS");	// 1
        th(    "YADRA");	// 1
        th(     "YADU");	// 1
        th(    "YADZI");	// 1
        th(    "YA:GU");	// 1
        th(   "YAKshA");	// 1
        th(     "Y'AL");	// 1
        th(     "YAMG");	// 1
        th(   "YAMM'A");	// 1
        th(   "Y'AM'O");	// 1
        th(   "YAM''O");	// 1
        th(     "YAnA");	// 1
        th(    "Y'ANA");	// 1
        th(  "YANCHAD");	// 1
        th(    "YANGH");	// 1
        th(  "YANGZHE");	// 1
        th(    "YANHG");	// 1
        th(     "YANm");	// 1
        th(    "YAn'O");	// 1
        th(    "YAN'O");	// 1
        th(    "YANYG");	// 1
        th(    "YA:'O");	// 1
        th(    "YA:PA");	// 1
        th(  "YARGHAm");	// 1
        th(  "YARTI'I");	// 1
        th(   "Y'ARYA");	// 1
        th(     "-YAS");	// 1
        th(   "Y'ASAM");	// 1
        th(   "Y'ASBA");	// 1
        th(     "YASH");	// 1
        th(   "YAshIt");	// 1
        th(    "YASKA");	// 1
        th(   "Y'ASMA");	// 1
        th(  "Y'ASTAM");	// 1
        th( "Y'AST'AM");	// 1
        th(    "YASYA");	// 1
        th(     "YATA");	// 1
        th(  "Y'ATA'O");	// 1
        th(     "YATI");	// 1
        th(   "YATNA:");	// 1
        th(    "YAT'O");	// 1
        th( "Y'ATRA'O");	// 1
        th("YATZTSA'O");	// 1
        th("YATZTSETA'O");	// 1
        th(    "YA+YA");	// 1
        th("YA+YANTATZCA");	// 1
        th(    "YBU'O");	// 1
        th(     "YBYA");	// 1
        th(    "YDANG");	// 1
        th(      "YDi");	// 1
        th(    "YDi'I");	// 1
        th(   "Y'DI'I");	// 1
        th(      "YDO");	// 1
        th(     "YDZA");	// 1
        th(      "'YE");	// 1
        th(      "Y'E");	// 1
        th(      "YE2");	// 1
        th(     "YEBB");	// 1
        th(     "YEBS");	// 1
        th(    "YEDHA");	// 1
        th(     "YEMS");	// 1
        th(      "YEn");	// 1
        th(   "YENT'A");	// 1
        th(  "YENYANU");	// 1
        th(     "YE'O");	// 1
        th(   "YE'U'I");	// 1
        th(    "YGCIG");	// 1
        th(   "YGNYIS");	// 1
        th(    "YGONG");	// 1
        th(     "YGYI");	// 1
        th(    "YHANG");	// 1
        th(     "YHIG");	// 1
        th(       "Yi");	// 1
        th(      "YI2");	// 1
        th(    "YI'AM");	// 1
        th(    "YIDLA");	// 1
        th(      "YIE");	// 1
        th(      "YIH");	// 1
        th(     "YIHN");	// 1
        th(     "YIIN");	// 1
        th(      "YIM");	// 1
        th(     "Y'IN");	// 1
        th(     "YIN:");	// 1
        th(     "YINA");	// 1
        th(     "YIND");	// 1
        th(   "YINDRI");	// 1
        th(     "YINN");	// 1
        th(    "YINNA");	// 1
        th(    "YIN'O");	// 1
        th(    "YINPA");	// 1
        th(    "YINTE");	// 1
        th( "YINZHING");	// 1
        th(     "YISS");	// 1
        th(     "YKRA");	// 1
        th(    "YKshA");	// 1
        th(    "YKYIS");	// 1
        th(    "YLDOG");	// 1
        th(    "YLTAR");	// 1
        th(       "Ym");	// 1
        th(   "YMTSON");	// 1
        th(      "YNG");	// 1
        th(   "YNGANG");	// 1
        th(      "YNI");	// 1
        th(     "YNID");	// 1
        th(      "YNO");	// 1
        th(    "YNYID");	// 1
        th(      "^YO");	// 1
        th(      "Y'O");	// 1
        th(      "YO3");	// 1
        th(     "YO9D");	// 1
        th(    "YO'AM");	// 1
        th(     "'YOD");	// 1
        th(     "YODR");	// 1
        th(     "+YON");	// 1
        th(     "Y'ON");	// 1
        th(    "Y'ONG");	// 1
        th(  "YONGSSU");	// 1
        th( "YONTU-NA");	// 1
        th(      "YOO");	// 1
        th(    "YOO'O");	// 1
        th(   "YORDHA");	// 1
        th(   "YORGYU");	// 1
        th(     "YO'U");	// 1
        th(  "YO:ZHES");	// 1
        th(      "YPA");	// 1
        th(    "YPA'I");	// 1
        th(   "YRNAMS");	// 1
        th(    "YSH'A");	// 1
        th(    "YSHES");	// 1
        th(  "YshTZYU");	// 1
        th(      "YtA");	// 1
        th( "YTONYTON");	// 1
        th(      "YU4");	// 1
        th(     "YU7L");	// 1
        th(    "YUDHA");	// 1
        th(    "YUDZA");	// 1
        th(    "YUGTA");	// 1
        th(    "YUGTI");	// 1
        th(   "YUKT'A");	// 1
        th(  "YU-LNGA");	// 1
        th(   "YURSAm");	// 1
        th(     "Y'US");	// 1
        th( "YUshMADA");	// 1
        th(  "YUshMDA");	// 1
        th(  "YUshMTA");	// 1
        th("YUSMANGRA");	// 1
        th(     "YUtA");	// 1
        th( "YUtA'ANG");	// 1
        th(    "YUWAM");	// 1
        th(     "YUYE");	// 1
        th(     "YVAN");	// 1
        th(      "YVE");	// 1
        th(    "YVE'I");	// 1
        th(     "YYID");	// 1
        th(    "YZHUS");	// 1
        th(       "Z'");	// 1
        th(      "ZA2");	// 1
        th(      "ZA4");	// 1
        th(      "ZAC");	// 1
        th(     "ZA'E");	// 1
        th(     "'ZAS");	// 1
        th(     "ZASG");	// 1
        th(     "ZAZS");	// 1
        th(    "ZBHIN");	// 1
        th(  "ZBSKYED");	// 1
        th(    "ZCING");	// 1
        th(    "'ZDIN");	// 1
        th(      "ZE2");	// 1
        th(      "ZE4");	// 1
        th(      "ZEB");	// 1
        th(     "ZEER");	// 1
        th(   "'ZENGS");	// 1
        th(     "ZERO");	// 1
        th(     "ZERR");	// 1
        th(      "ZET");	// 1
        th(   "ZE'U'I");	// 1
        th(   "ZGE'AM");	// 1
        th(     "ZGYI");	// 1
        th(   "ZH8ING");	// 1
        th(     "ZHA2");	// 1
        th(     "ZHA4");	// 1
        th(    "'ZHAG");	// 1
        th(    "ZHAMS");	// 1
        th(   "'ZHANG");	// 1
        th(    "ZHANH");	// 1
        th(  "ZH'ANTA");	// 1
        th(     "ZHAt");	// 1
        th(     "ZHBS");	// 1
        th(     "ZHE2");	// 1
        th(    "ZHE3S");	// 1
        th(     "ZHE4");	// 1
        th(  "^ZHE'AM");	// 1
        th(    "ZHEBS");	// 1
        th(     "ZHEC");	// 1
        th(    "ZHECS");	// 1
        th("ZHEGSHEGS");	// 1
        th(     "ZHEI");	// 1
        th(    "'ZHEN");	// 1
        th(    "ZHENO");	// 1
        th(   "ZHESBA");	// 1
        th(    "ZHESE");	// 1
        th(   "ZHE-SU");	// 1
        th(     "ZHGA");	// 1
        th(     "ZHHE");	// 1
        th(    "ZHHES");	// 1
        th(     "ZHI2");	// 1
        th(     "ZHI4");	// 1
        th(   "ZHI'AM");	// 1
        th(    "ZHIES");	// 1
        th(   "ZH'I'I");	// 1
        th(   "ZHING:");	// 1
        th(     "ZHNG");	// 1
        th(     "ZHO3");	// 1
        th(     "ZHOB");	// 1
        th(     "ZHPA");	// 1
        th(      "ZHR");	// 1
        th(   "ZHRAGS");	// 1
        th(     "^ZHU");	// 1
        th(     "ZH'U");	// 1
        th(     "ZHU:");	// 1
        th(     "ZHU4");	// 1
        th(    "'ZHUG");	// 1
        th(    "ZHUGI");	// 1
        th(    "ZHUGO");	// 1
        th(   "'ZHUGS");	// 1
        th(     "ZHUO");	// 1
        th(  "ZHVA'AM");	// 1
        th(   "ZHVABS");	// 1
        th(    "ZHVAN");	// 1
        th(     "ZHVI");	// 1
        th(    "ZHYIG");	// 1
        th(      "'ZI");	// 1
        th(      "ZI2");	// 1
        th(      "ZI4");	// 1
        th(     "ZIND");	// 1
        th(    "ZINTE");	// 1
        th(    "ZINTO");	// 1
        th(  "ZKHYE'I");	// 1
        th(    "ZLA10");	// 1
        th(     "ZLA6");	// 1
        th(    "ZLABS");	// 1
        th(    "ZLAGS");	// 1
        th(    "ZLANG");	// 1
        th(    "'ZLAS");	// 1
        th(    "ZLA'U");	// 1
        th(      "ZLI");	// 1
        th(    "ZLI'I");	// 1
        th(     "ZLOB");	// 1
        th(     "ZLOD");	// 1
        th(    "ZLO'I");	// 1
        th(     "ZLOM");	// 1
        th(     "ZLSA");	// 1
        th(    "ZLUGS");	// 1
        th(   "ZMTHUN");	// 1
        th(    "ZNAGS");	// 1
        th(      "ZO3");	// 1
        th(     "'ZOD");	// 1
        th(     "'ZOG");	// 1
        th(     "ZOGS");	// 1
        th(     "ZO'I");	// 1
        th(     "'ZOM");	// 1
        th(    "'ZOMS");	// 1
        th(     "ZSER");	// 1
        th(      "ZSO");	// 1
        th(      "'ZU");	// 1
        th(      "ZU4");	// 1
        th(     "'ZUG");	// 1
        th(    "'ZUGS");	// 1
        th(      "ZUm");	// 1
        th(   "'ZUNGS");	// 1
        th(     "^ZUR");	// 1
        th(     "'ZUR");	// 1
        th(     "ZVAN");	// 1
        th(      "ZYA");	// 1
    }

    /** Tests the tsheg bars that appear [2, 9] times. */
    public void test2to9() {
        th(      "002");	// 2
        th(      "003");	// 2
        th(      "01A");	// 2
        th(      "065");	// 2
        th(       "0A");	// 2
        th(    "104NA");	// 2
        th(    "105B2");	// 2
        th(    "110NA");	// 2
        th(    "112B1");	// 2
        th(      "115");	// 2
        th(     "11BA");	// 2
        th(    "11NA5");	// 2
        th(      "120");	// 2
        th(      "124");	// 2
        th(    "129NA");	// 2
        th(     "12B3");	// 2
        th(    "132NA");	// 2
        th(    "138NA");	// 2
        th(    "139B3");	// 2
        th(     "13B7");	// 2
        th(      "141");	// 2
        th(    "142NA");	// 2
        th(      "144");	// 2
        th(    "144NA");	// 2
        th(      "147");	// 2
        th(      "149");	// 2
        th(      "154");	// 2
        th(      "156");	// 2
        th(     "15NA");	// 2
        th(      "162");	// 2
        th(      "163");	// 2
        th(      "164");	// 2
        th(      "165");	// 2
        th( "16596321");	// 2
        th(    "165B5");	// 2
        th(      "169");	// 2
        th(   "169BA7");	// 2
        th(     "16BA");	// 2
        th(    "172N1");	// 2
        th(      "174");	// 2
        th(      "175");	// 2
        th(      "179");	// 2
        th(    "179B3");	// 2
        th(      "181");	// 2
        th(      "184");	// 2
        th(      "192");	// 2
        th(      "195");	// 2
        th(     "1959");	// 2
        th(     "1970");	// 2
        th(     "1973");	// 2
        th(     "1978");	// 2
        th(      "198");	// 2
        th(     "1987");	// 2
        th(     "1995");	// 2
        th(     "19N3");	// 2
        th(     "1NA1");	// 2
        th(     "2000");	// 2
        th(   "207BA1");	// 2
        th(    "20BA5");	// 2
        th(      "210");	// 2
        th(     "2100");	// 2
        th(     "2119");	// 2
        th(      "212");	// 2
        th(     "2120");	// 2
        th(      "214");	// 2
        th(      "216");	// 2
        th(      "218");	// 2
        th(    "21NA2");	// 2
        th(      "220");	// 2
        th(      "225");	// 2
        th(      "227");	// 2
        th(      "228");	// 2
        th(   "231BA5");	// 2
        th(    "232N7");	// 2
        th(   "234BA7");	// 2
        th(      "238");	// 2
        th(    "23BA3");	// 2
        th(      "240");	// 2
        th(    "240N3");	// 2
        th(      "242");	// 2
        th(      "247");	// 2
        th(    "247N3");	// 2
        th(    "250N1");	// 2
        th(    "252N6");	// 2
        th(   "254NA4");	// 2
        th(      "255");	// 2
        th(      "256");	// 2
        th(      "257");	// 2
        th(    "258N7");	// 2
        th(      "259");	// 2
        th(   "261NA1");	// 2
        th(      "263");	// 2
        th(      "266");	// 2
        th(      "267");	// 2
        th(      "272");	// 2
        th(      "273");	// 2
        th(      "278");	// 2
        th(      "283");	// 2
        th(      "286");	// 2
        th(     "28B7");	// 2
        th(     "28NA");	// 2
        th(      "290");	// 2
        th(   "291NA1");	// 2
        th(      "292");	// 2
        th(      "296");	// 2
        th(    "297B3");	// 2
        th(     "29N7");	// 2
        th(      "302");	// 2
        th(      "306");	// 2
        th(      "308");	// 2
        th(      "309");	// 2
        th(     "30B5");	// 2
        th(      "311");	// 2
        th(      "312");	// 2
        th(      "316");	// 2
        th(      "318");	// 2
        th(      "321");	// 2
        th(      "322");	// 2
        th(      "330");	// 2
        th(      "332");	// 2
        th(      "333");	// 2
        th(  "3355443");	// 2
        th(      "339");	// 2
        th(      "341");	// 2
        th(      "345");	// 2
        th(      "348");	// 2
        th(   "349NA4");	// 2
        th(      "350");	// 2
        th(      "351");	// 2
        th(      "355");	// 2
        th(      "365");	// 2
        th(      "371");	// 2
        th(      "372");	// 2
        th(      "375");	// 2
        th(      "377");	// 2
        th(    "378B3");	// 2
        th(      "382");	// 2
        th(      "388");	// 2
        th(      "391");	// 2
        th(      "392");	// 2
        th(      "394");	// 2
        th(      "396");	// 2
        th(    "39NA3");	// 2
        th(    "404N1");	// 2
        th(     "40NA");	// 2
        th(      "410");	// 2
        th(      "416");	// 2
        th(      "417");	// 2
        th(      "419");	// 2
        th(     "41BA");	// 2
        th(     "41NA");	// 2
        th(      "424");	// 2
        th(      "425");	// 2
        th(    "429N6");	// 2
        th(      "431");	// 2
        th(      "439");	// 2
        th(      "450");	// 2
        th(      "457");	// 2
        th(      "459");	// 2
        th(      "465");	// 2
        th(      "466");	// 2
        th(      "471");	// 2
        th(      "474");	// 2
        th(      "477");	// 2
        th(      "481");	// 2
        th(    "487B2");	// 2
        th(      "489");	// 2
        th(    "48NA2");	// 2
        th(      "494");	// 2
        th(      "495");	// 2
        th(    "50000");	// 2
        th(      "504");	// 2
        th(     "50B7");	// 2
        th(      "517");	// 2
        th(      "518");	// 2
        th(      "519");	// 2
        th(      "520");	// 2
        th(      "528");	// 2
        th(      "531");	// 2
        th(      "532");	// 2
        th(      "543");	// 2
        th(      "544");	// 2
        th(     "57N3");	// 2
        th(      "5NA");	// 2
        th(     "60NA");	// 2
        th(     "62BA");	// 2
        th(     "64B2");	// 2
        th(    "64BA2");	// 2
        th(     "67BA");	// 2
        th(     "67NA");	// 2
        th(      "6NA");	// 2
        th(      "7BA");	// 2
        th(     "7BDE");	// 2
        th(    "7BDUN");	// 2
        th(    "7BSAM");	// 2
        th(   "7BSDUS");	// 2
        th(     "7BYE");	// 2
        th(     "7CHA");	// 2
        th(    "7DBEN");	// 2
        th(    "7DGA'");	// 2
        th(  "7DHARMA");	// 2
        th(    "7'DIR");	// 2
        th(   "7DMIGS");	// 2
        th(     "7DPE");	// 2
        th(     "7'DU");	// 2
        th(  "7GRANGS");	// 2
        th(    "7GRUB");	// 2
        th(   "7GSUNG");	// 2
        th(    "7KHRI");	// 2
        th(      "7MA");	// 2
        th(   "7MDZES");	// 2
        th( "7MTSUNGS");	// 2
        th(    "7NYER");	// 2
        th(     "7PAn");	// 2
        th(    "7PHAL");	// 2
        th(    "7RANG");	// 2
        th(     "7RDO");	// 2
        th(    "7RNAL");	// 2
        th(    "7RNAM");	// 2
        th(   "7SGROL");	// 2
        th(   "7SGRUB");	// 2
        th(    "7SLOB");	// 2
        th(    "7SNAR");	// 2
        th(    "7STAG");	// 2
        th(     "8777");	// 2
        th(      "934");	// 2
        th(    "98NA3");	// 2
        th(     "99NA");	// 2
        th(      "9BA");	// 2
        th(       "A:");	// 2
        th(      "'A4");	// 2
        th(     "AA'A");	// 2
        th(     "A'AD");	// 2
        th(    "AADHI");	// 2
        th(  "A'A:DUS");	// 2
        th(   "A'ADYA");	// 2
        th("A'ADZNY'A");	// 2
        th(    "AAGNE");	// 2
        th(  "AAGNI'O");	// 2
        th("A'A:HR'I:");	// 2
        th("A'A:-H'Um");	// 2
        th( "AA:-H'Um");	// 2
        th(     "AAIn");	// 2
        th(     "AAIS");	// 2
        th(     "AAIt");	// 2
        th(    "AAKKA");	// 2
        th(  "AA-KshA");	// 2
        th(  "AAKshAR");	// 2
        th(   "A'A:MA");	// 2
        th(     "AANA");	// 2
        th(    "AA+NA");	// 2
        th(    "AAndA");	// 2
        th(   "AANGGA");	// 2
        th(    "AAn'O");	// 2
        th(    "AANTU");	// 2
        th(    "AANYA");	// 2
        th(      "AAO");	// 2
        th(    "AA'OS");	// 2
        th(      "AAP");	// 2
        th(  "AARGA'I");	// 2
        th(  "A'ARGYA");	// 2
        th(    "AARKA");	// 2
        th(     "AARU");	// 2
        th(  "A'A-RYA");	// 2
        th(   "AAR+YA");	// 2
        th(    "A'ASA");	// 2
        th(  "AAshM'A");	// 2
        th(   "AASMAD");	// 2
        th(  "AASMRAD");	// 2
        th(    "AASTI");	// 2
        th(     "AATA");	// 2
        th(   "A'ATAM");	// 2
        th(   "AATA'O");	// 2
        th(  "A'ATM'A");	// 2
        th(  "AATZA'I");	// 2
        th(     "AAUG");	// 2
        th(   "A'A:YA");	// 2
        th(      "ABA");	// 2
        th(     "ADHI");	// 2
        th(      "ADO");	// 2
        th(      "A'E");	// 2
        th(      "AEB");	// 2
        th(    "AEE'O");	// 2
        th(      "AEt");	// 2
        th(      "AET");	// 2
        th(     "AETA");	// 2
        th(      "'AG");	// 2
        th(      "AId");	// 2
        th(    "AIDAm");	// 2
        th(      "AIM");	// 2
        th(    "AiMRA");	// 2
        th(     "AINA");	// 2
        th(   "AINA'I");	// 2
        th(  "AIN-DRA");	// 2
        th(    "AINTA");	// 2
        th(   "AINtRA");	// 2
        th(     "A'IR");	// 2
        th(    "AIshI");	// 2
        th(   "AIshnU");	// 2
        th(      "AIT");	// 2
        th(     "AITA");	// 2
        th(   "AItA'O");	// 2
        th(    "AiTHI");	// 2
        th(    "AITHI");	// 2
        th(    "AIT'O");	// 2
        th(  "AITZTSA");	// 2
        th(      "ALA");	// 2
        th(  "AMB'A'I");	// 2
        th(   "ANTE'I");	// 2
        th(    "ANYRA");	// 2
        th(      "AOB");	// 2
        th(     "A'OD");	// 2
        th(     "A'Om");	// 2
        th( "AOmSARBA");	// 2
        th(   "AOOddI");	// 2
        th( "AOOdYAnA");	// 2
        th(    "AOSRA");	// 2
        th(     "AOTA");	// 2
        th(     "ASTI");	// 2
        th(   "ATHI'I");	// 2
        th(  "ATHiI'I");	// 2
        th(   "ATSIGS");	// 2
        th(    "AUDAP");	// 2
        th(    "AUDPA");	// 2
        th(   "AUPDZA");	// 2
        th(   "A'URGA");	// 2
        th(   "AURYAN");	// 2
        th(  "AUshn'I");	// 2
        th(     "A'UT");	// 2
        th(    "A'UTA");	// 2
        th(   "AU-TAP");	// 2
        th("AUTAPLA'I");	// 2
        th(  "AUT+PAL");	// 2
        th( "A'UTPALA");	// 2
        th("AUTTIshthA");	// 2
        th(  "AUTZTSA");	// 2
        th(    "AYOnA");	// 2
        th(    "AZHES");	// 2
        th(     "BA'A");	// 2
        th(     "BAAR");	// 2
        th(  "BADZR'A");	// 2
        th(  "BADZR'I");	// 2
        th(   "BAKSHI");	// 2
        th(   "BALANG");	// 2
        th(     "B'Am");	// 2
        th(     "BA'M");	// 2
        th(     "B'AN");	// 2
        th(   "BAN-DE");	// 2
        th(    "BANDU");	// 2
        th(   "BANHHA");	// 2
        th(    "BAN'O");	// 2
        th(    "BANTA");	// 2
        th(  "BANTA'I");	// 2
        th(    "BANTI");	// 2
        th(    "BA:'O");	// 2
        th(  "BAR-SHI");	// 2
        th(    "BARTI");	// 2
        th(    "BARWI");	// 2
        th(     "BASD");	// 2
        th(    "BASDE");	// 2
        th(    "BASHD");	// 2
        th(   "BASTRA");	// 2
        th(      "BAT");	// 2
        th(    "BATAD");	// 2
        th(   "BATT'A");	// 2
        th(   "BCA'AM");	// 2
        th(    "^BCAS");	// 2
        th(    "BCHAB");	// 2
        th(   "BCHING");	// 2
        th(  "BCI'ANG");	// 2
        th(     "BCID");	// 2
        th(     "BCIM");	// 2
        th(     "BCIS");	// 2
        th(    "BCONG");	// 2
        th(     "BCUB");	// 2
        th(    "BCUIG");	// 2
        th(     "^BDE");	// 2
        th(     "'BDE");	// 2
        th(    "BDESN");	// 2
        th(     "BDGA");	// 2
        th(    "BDING");	// 2
        th(      "BDN");	// 2
        th(    "BDOMS");	// 2
        th(    "BDRAL");	// 2
        th(    "BDRAN");	// 2
        th(  "BDRANGS");	// 2
        th(    "BDROL");	// 2
        th(     "BDSU");	// 2
        th(    "'BDUN");	// 2
        th(     "BDUR");	// 2
        th(    "BDZES");	// 2
        th(    "BDZIN");	// 2
        th(      "BEB");	// 2
        th(     "B'EE");	// 2
        th( "BEEd'URA");	// 2
        th("BEE-d'U-RYA");	// 2
        th(     "BEEN");	// 2
        th(     "BEER");	// 2
        th(  "BEEt'UR");	// 2
        th(    "'BEGS");	// 2
        th(   "BEGTZE");	// 2
        th(    "'BE'I");	// 2
        th(   "BEKTZE");	// 2
        th(    "BENYA");	// 2
        th(      "BEt");	// 2
        th(      "BGA");	// 2
        th(    "BGANG");	// 2
        th(   "BGENGS");	// 2
        th(     "BGOG");	// 2
        th(    "BGRAM");	// 2
        th(   "BGREGS");	// 2
        th(    "BGREN");	// 2
        th(   "BGRIBS");	// 2
        th(   "BGRIGS");	// 2
        th(   "BGROGS");	// 2
        th(   "BGYA'O");	// 2
        th(    "BGYAS");	// 2
        th(    "BGYEN");	// 2
        th(    "BGYER");	// 2
        th( "BGYI'ANG");	// 2
        th(  "BGYINGS");	// 2
        th(   "BHADRA");	// 2
        th(     "BHAM");	// 2
        th(   "BHARMA");	// 2
        th(   "BH'ASA");	// 2
        th( "BHATZTSA");	// 2
        th(    "BHA'U");	// 2
        th(   "BHA+YA");	// 2
        th(    "BHEE:");	// 2
        th(    "BHEEm");	// 2
        th(     "BHEN");	// 2
        th(     "BHER");	// 2
        th(     "BHES");	// 2
        th(     "BHID");	// 2
        th(    "BHING");	// 2
        th(     "BHmA");	// 2
        th(     "BHOG");	// 2
        th(    "BHR'A");	// 2
        th(    "BHR'i");	// 2
        th(    "BHRUM");	// 2
        th("BHRUm:HRI:YA");	// 2
        th(     "BHSA");	// 2
        th( "BHUGMA'I");	// 2
        th(    "BHU'I");	// 2
        th(    "BHUtA");	// 2
        th(    "BHYAm");	// 2
        th(   "BHY'AM");	// 2
        th( "BHYA-NTA");	// 2
        th("BHYASHTZA");	// 2
        th(  "BHYASTA");	// 2
        th( "BHYAST'I");	// 2
        th(    "BHYIS");	// 2
        th(   "BIDHYA");	// 2
        th(    "BINDA");	// 2
        th(   "BIRY'A");	// 2
        th(   "BISHVA");	// 2
        th(  "BITZTSA");	// 2
        th(     "BJIG");	// 2
        th(     "BJOG");	// 2
        th(    "'BKA'");	// 2
        th(    "BKALG");	// 2
        th(   "BKHRUL");	// 2
        th(   "BKHYOD");	// 2
        th(      "BKI");	// 2
        th(  "BKRANGS");	// 2
        th(   "BKRI'I");	// 2
        th(  "BKRUNGS");	// 2
        th(    "BKSAL");	// 2
        th(   "BKSYED");	// 2
        th(     "BKUL");	// 2
        th(     "BKVA");	// 2
        th(    "BKVA'");	// 2
        th(   "BKYANG");	// 2
        th(    "BKYIL");	// 2
        th(   "BKYOGS");	// 2
        th(    "BLAMS");	// 2
        th(     "BLDA");	// 2
        th(    "BLHAG");	// 2
        th(      "BLI");	// 2
        th(    "BLKOG");	// 2
        th(   "BLNGAD");	// 2
        th(     "BLOM");	// 2
        th(    "BLTES");	// 2
        th(   "BLTO'O");	// 2
        th(     "BLUM");	// 2
        th(     "BLUR");	// 2
        th(     "BMAS");	// 2
        th(      "BNA");	// 2
        th(    "BNABS");	// 2
        th(     "BNAN");	// 2
        th(     "BNGO");	// 2
        th(     "BNON");	// 2
        th(     "BNYE");	// 2
        th(    "BNYED");	// 2
        th(     "BNYO");	// 2
        th(   "BNYTZI");	// 2
        th(   "BNYUNG");	// 2
        th(      "BOB");	// 2
        th(     "BOCM");	// 2
        th(   "BO'I'O");	// 2
        th(      "BOJ");	// 2
        th(     "'BON");	// 2
        th(   "'BONGS");	// 2
        th(    "BOO'O");	// 2
        th(      "BOT");	// 2
        th(     "BPAR");	// 2
        th(     "BR'A");	// 2
        th(    "BRA'I");	// 2
        th(     "BRAm");	// 2
        th( "BRASHTZA");	// 2
        th(    "BRDAD");	// 2
        th(    "BRDEB");	// 2
        th(    "BRDIG");	// 2
        th(    "BRDOR");	// 2
        th(    "BRDUN");	// 2
        th(     "BREM");	// 2
        th(     "BREN");	// 2
        th(   "'BRE'O");	// 2
        th(    "BRE'O");	// 2
        th(   "'BRE'U");	// 2
        th(    "BRGAS");	// 2
        th(     "BRGO");	// 2
        th(    "BRGOS");	// 2
        th(   "BRGYUd");	// 2
        th(  "BRGYUNG");	// 2
        th(  "'BRI'AM");	// 2
        th(     "BRIB");	// 2
        th(    "'BRIB");	// 2
        th(   "BRiDHA");	// 2
        th(    "'BRIG");	// 2
        th( "BRiKsh'A");	// 2
        th("BRiKsh'A:");	// 2
        th( "BRiKshAS");	// 2
        th(  "BRiKshE");	// 2
        th(    "BRiNG");	// 2
        th(  "BRI'O'I");	// 2
        th(    "BRJEG");	// 2
        th(   "BRJHOD");	// 2
        th(   "BRJIBS");	// 2
        th(   "BRJING");	// 2
        th(   "BRLA'O");	// 2
        th(    "BRLAS");	// 2
        th(    "BRLID");	// 2
        th(   "BRLUBS");	// 2
        th(   "BRLUNG");	// 2
        th(    "BRNAM");	// 2
        th(   "BRNANG");	// 2
        th(   "BRNGA'");	// 2
        th(  "BRNGABS");	// 2
        th(  "BRNGAMS");	// 2
        th(  "BRNGO'O");	// 2
        th(   "BRNGUB");	// 2
        th(   "BRNYA'");	// 2
        th(  "BRNYA'A");	// 2
        th(    "BRNYE");	// 2
        th(   "BRNYEL");	// 2
        th( "BRNYENGS");	// 2
        th(  "BRNYING");	// 2
        th(   "BRNYIS");	// 2
        th(    "BROGS");	// 2
        th(     "BROR");	// 2
        th(  "BRTENPA");	// 2
        th(     "BRTI");	// 2
        th(    "BRTIN");	// 2
        th(   "BRTING");	// 2
        th(   "BRTUGS");	// 2
        th(   "BRTZAR");	// 2
        th( "BRTZE'AM");	// 2
        th( "BRTZI'AM");	// 2
        th(  "BRTZIBS");	// 2
        th( "BRTZINGS");	// 2
        th(  "BRTZONG");	// 2
        th(   "BRTZUL");	// 2
        th(     "BR'U");	// 2
        th(  "'BRU'AM");	// 2
        th(    "'BRUD");	// 2
        th(   "'BRUGS");	// 2
        th(    "BRU'I");	// 2
        th(    "'BRUN");	// 2
        th(   "BSAHGS");	// 2
        th(    "^BSAM");	// 2
        th(    "BSAMO");	// 2
        th(   "BSBUBS");	// 2
        th(  "BSBYANG");	// 2
        th(   "BSBYIN");	// 2
        th(    "BSDAR");	// 2
        th(    "BSDEM");	// 2
        th(    "BSDES");	// 2
        th(    "BSDON");	// 2
        th(   "BSDUBS");	// 2
        th(  "BSDUSPA");	// 2
        th(     "BSEG");	// 2
        th(     "BSEM");	// 2
        th(    "BSERU");	// 2
        th(    "BSGAG");	// 2
        th(   "BSGAMS");	// 2
        th(    "BSGEG");	// 2
        th( "BSGO'ANG");	// 2
        th(   "BSGOmS");	// 2
        th(  "BSGRA'I");	// 2
        th(  "BSGREBS");	// 2
        th(  "BSGREGS");	// 2
        th(   "BSGREL");	// 2
        th( "BSGRINGS");	// 2
        th(   "BSGROG");	// 2
        th(  "BSGRUNG");	// 2
        th(   "BSGRUS");	// 2
        th(     "BSGS");	// 2
        th(   "BSGUBS");	// 2
        th(   "BSGYED");	// 2
        th(   "BSGYOD");	// 2
        th(  "BSGYUBS");	// 2
        th(   "BSHA'A");	// 2
        th(  "BSHADDE");	// 2
        th(   "BSHAES");	// 2
        th(     "BSHE");	// 2
        th(    "BSHEN");	// 2
        th(    "BSHEt");	// 2
        th(    "BSHIS");	// 2
        th(   "BSHU'I");	// 2
        th(     "BSIG");	// 2
        th( "BSIYANGS");	// 2
        th(    "BSKES");	// 2
        th(   "BSKRAB");	// 2
        th(   "BSKRED");	// 2
        th(  "BSKRIBS");	// 2
        th(   "BSKRIN");	// 2
        th(    "BSKRO");	// 2
        th(   "BSKROR");	// 2
        th(    "BSKRU");	// 2
        th(   "BSKUBS");	// 2
        th(   "BSKURN");	// 2
        th(   "BSLA'I");	// 2
        th(  "BSLENGS");	// 2
        th(     "BSLO");	// 2
        th(    "BSLOG");	// 2
        th(    "BSLUN");	// 2
        th(    "BSMOS");	// 2
        th(    "BSNAG");	// 2
        th(   "BSNAGS");	// 2
        th(   "BSNGAd");	// 2
        th(   "BSNGOB");	// 2
        th("BSNGO'O'I");	// 2
        th(   "BSNOGS");	// 2
        th(   "BSNUMS");	// 2
        th(  "BSNYANG");	// 2
        th(   "BSNYEM");	// 2
        th(  "BSNYENG");	// 2
        th(    "BSNYI");	// 2
        th(  "BSNYING");	// 2
        th(   "BSNYOL");	// 2
        th(   "BSO'AM");	// 2
        th(   "BSPANG");	// 2
        th(  "BSPANGS");	// 2
        th(   "BSPO'I");	// 2
        th(   "BSPU'I");	// 2
        th(    "BSPUN");	// 2
        th(   "BSPYAD");	// 2
        th(  "BSRENGS");	// 2
        th(    "BSREt");	// 2
        th(   "BSRIGS");	// 2
        th(   "BSRO'I");	// 2
        th(    "BSROR");	// 2
        th(   "BSTI'I");	// 2
        th(    "BSTIL");	// 2
        th(     "BSTN");	// 2
        th(    "BSTNA");	// 2
        th(     "BSTO");	// 2
        th(    "BSTUG");	// 2
        th(     "BSUL");	// 2
        th(    "BTA'I");	// 2
        th(     "BTAL");	// 2
        th(     "BTAM");	// 2
        th(    "BTA'O");	// 2
        th(     "BTEL");	// 2
        th(    "BTENG");	// 2
        th(     "BTIB");	// 2
        th(     "BTLA");	// 2
        th(     "BTOR");	// 2
        th(     "BTOS");	// 2
        th(    "BTREN");	// 2
        th(     "BTSA");	// 2
        th(    "BTSAL");	// 2
        th(     "BTSO");	// 2
        th(   "BTSO'I");	// 2
        th(    "BTSON");	// 2
        th(    "BTU'O");	// 2
        th(    "BTZER");	// 2
        th(    "BTZOL");	// 2
        th(    "BTZUd");	// 2
        th(   "^BTZUN");	// 2
        th(    "BTZVA");	// 2
        th(      "BU4");	// 2
        th(    "BU'Am");	// 2
        th(      "BUC");	// 2
        th(   "BUDHYA");	// 2
        th(    "'BU'I");	// 2
        th(      "BUn");	// 2
        th(    "BUNDE");	// 2
        th(    "BU'NG");	// 2
        th(    "'BU'O");	// 2
        th(   "BYA'Am");	// 2
        th(  "BYA'I'O");	// 2
        th(     "BYAm");	// 2
        th(    "BYAMG");	// 2
        th(   "BYE'AM");	// 2
        th(     "BYEB");	// 2
        th(     "BYEE");	// 2
        th(     "BYEL");	// 2
        th(    "BYEMS");	// 2
        th(   "'BYENG");	// 2
        th(  "BYE'U'I");	// 2
        th(     "BY'I");	// 2
        th(    "'BYIG");	// 2
        th(     "BYiN");	// 2
        th(    "^BYIN");	// 2
        th(  "'BYINGS");	// 2
        th(      "BYN");	// 2
        th(  "'BYORDA");	// 2
        th(      "BYR");	// 2
        th(    "BYUGN");	// 2
        th(   "'BYUGN");	// 2
        th(   "'BYUGS");	// 2
        th(     "BYVA");	// 2
        th(     "BZ'A");	// 2
        th(   "BZANG0");	// 2
        th(   "BZENGS");	// 2
        th(      "BZH");	// 2
        th(  "BZHANGS");	// 2
        th(   "BZHEGS");	// 2
        th(   "BZHE'O");	// 2
        th(     "BZHG");	// 2
        th(   "BZHINN");	// 2
        th(  "BZHUGSO");	// 2
        th(  "BZHUNGS");	// 2
        th(     "BZID");	// 2
        th(   "BZINGS");	// 2
        th(     "BZIR");	// 2
        th(     "BZIS");	// 2
        th(    "BZLUG");	// 2
        th(      "BZU");	// 2
        th(     "BZUG");	// 2
        th(      "CA4");	// 2
        th(    "'CAHD");	// 2
        th(     "CAMS");	// 2
        th(    "CANGS");	// 2
        th(     "CANM");	// 2
        th(      "CE4");	// 2
        th(  "'CHA'AM");	// 2
        th(   "^'CHAB");	// 2
        th(    "'CHAd");	// 2
        th(    "CHAMS");	// 2
        th(    "CH'AN");	// 2
        th(     "CHAt");	// 2
        th(    "CHE'A");	// 2
        th(    "'CHEB");	// 2
        th(     "CHEE");	// 2
        th(   "'CHEMS");	// 2
        th(     "CH'I");	// 2
        th(   "'CHOGS");	// 2
        th(   "'CHO'I");	// 2
        th(    "'CHUM");	// 2
        th(    "'CHUR");	// 2
        th(   "CHVA'I");	// 2
        th(      "CIB");	// 2
        th(     "CIGN");	// 2
        th(     "CIGS");	// 2
        th(      "CNA");	// 2
        th(     "CNEN");	// 2
        th(      "CUM");	// 2
        th(      "CVO");	// 2
        th(       "'D");	// 2
        th(      "dA'");	// 2
        th(     "D''A");	// 2
        th(    "DAANG");	// 2
        th(  "'DA'ANG");	// 2
        th(     "DABG");	// 2
        th(     "DAGI");	// 2
        th(   "d'AKKI");	// 2
        th(   "DAKshE");	// 2
        th(     "D'Am");	// 2
        th(     "D'AN");	// 2
        th(  "DAndI'I");	// 2
        th(   "DAndIS");	// 2
        th(    "D'ANG");	// 2
        th(   "DANGPO");	// 2
        th(     "DANS");	// 2
        th(    "DANTI");	// 2
        th(     "d'AR");	// 2
        th(  "D'ARBAN");	// 2
        th(     "dATI");	// 2
        th(   "DATTAS");	// 2
        th(  "DATTNYA");	// 2
        th(     "dA'U");	// 2
        th(     "D'AU");	// 2
        th(  "dA'URYA");	// 2
        th(      "DAV");	// 2
        th(    "DBAGN");	// 2
        th(  "DBAYNGS");	// 2
        th(     "DBEB");	// 2
        th(     "DBEL");	// 2
        th(    "DBNAG");	// 2
        th(     "DBOS");	// 2
        th(    "DBRAS");	// 2
        th(     "DBUd");	// 2
        th(     "DBUN");	// 2
        th(   "DBUNGS");	// 2
        th(   "DBYABS");	// 2
        th(  "DBYAGNS");	// 2
        th(    "DBYAL");	// 2
        th(   "DBYANS");	// 2
        th(    "'DBYE");	// 2
        th(    "DBYEB");	// 2
        th(    "DBYEG");	// 2
        th(   "DBYENG");	// 2
        th(     "DDAG");	// 2
        th(   "DDANTA");	// 2
        th(    "D-DHA");	// 2
        th(    "DDH'A");	// 2
        th(   "DDH'A:");	// 2
        th(   "DDNGOS");	// 2
        th(      "^DE");	// 2
        th(      "DE2");	// 2
        th(      "DE4");	// 2
        th(     "DEE:");	// 2
        th(      "dEm");	// 2
        th(      "DEs");	// 2
        th(    "'DE'U");	// 2
        th(    "DGA'A");	// 2
        th(   "DGA'AM");	// 2
        th( "DGA'-'AM");	// 2
        th(  "DGA'ANG");	// 2
        th(   "DGE'AM");	// 2
        th(     "DGEG");	// 2
        th( "DGESLONG");	// 2
        th(    "DGE'U");	// 2
        th(     "DGHA");	// 2
        th(    "DGRAN");	// 2
        th(     "DGRE");	// 2
        th(   "DGROGS");	// 2
        th(     "DGU'");	// 2
        th(     "DGUB");	// 2
        th(   "DGUNGS");	// 2
        th(  "DGYANGS");	// 2
        th(    "DGYID");	// 2
        th(    "DGYUS");	// 2
        th(     "DHA:");	// 2
        th(    "DHA'A");	// 2
        th(  "DHA'A'I");	// 2
        th(   "DH'AI:");	// 2
        th( "DH'A'I'O");	// 2
        th(     "DHAL");	// 2
        th(     "DHAN");	// 2
        th(    "dHA'O");	// 2
        th(   "DH'A'O");	// 2
        th(    "DHA'O");	// 2
        th(  "DHA-RMA");	// 2
        th(   "DHARMI");	// 2
        th(  "DHARMIS");	// 2
        th(    "DHA'U");	// 2
        th(    "DHISA");	// 2
        th( "DHIshthI");	// 2
        th("DHIshthi'I");	// 2
        th(    "DHM'A");	// 2
        th(   "DHONYA");	// 2
        th(     "DHRA");	// 2
        th(  "DHRAMMA");	// 2
        th(     "DHRI");	// 2
        th(  "DHRishI");	// 2
        th(   "DHRM'A");	// 2
        th(     "DHU:");	// 2
        th(     "DHUR");	// 2
        th(    "DHYA'");	// 2
        th(    "DHYEE");	// 2
        th(     "DHYI");	// 2
        th(     "^'DI");	// 2
        th(     "DI'A");	// 2
        th(    "'DIGS");	// 2
        th(   "'DI''I");	// 2
        th(    "D'I'I");	// 2
        th(     "d'IK");	// 2
        th(   "d'IKAR");	// 2
        th( "d'IKKA'I");	// 2
        th(      "DIM");	// 2
        th(    "D'I'O");	// 2
        th(     "DI'O");	// 2
        th(  "D'IRGHA");	// 2
        th( "D''IRGHA");	// 2
        th("D'IRGHA'O");	// 2
        th(     "DIYA");	// 2
        th(      "DKA");	// 2
        th(     "DK'A");	// 2
        th(    "DKABS");	// 2
        th(   "DKHYED");	// 2
        th(    "^DKON");	// 2
        th(    "DKRAL");	// 2
        th(    "DKRAM");	// 2
        th(    "DKRES");	// 2
        th(    "DKRIG");	// 2
        th(     "DKUS");	// 2
        th(   "DKYABS");	// 2
        th(  "DKYILOR");	// 2
        th(    "DLANG");	// 2
        th(     "DLAS");	// 2
        th(    "DLTAR");	// 2
        th(     "DMAM");	// 2
        th(     "DMIN");	// 2
        th(    "DMONG");	// 2
        th(     "DMUG");	// 2
        th(     "DMYA");	// 2
        th(  "DMYANGS");	// 2
        th(    "DMYAR");	// 2
        th(     "DNGE");	// 2
        th(    "DNGOG");	// 2
        th(   "DNGOMS");	// 2
        th(   "DNGONS");	// 2
        th(     "DNGS");	// 2
        th(   "DNGUNG");	// 2
        th(      "DNI");	// 2
        th(    "DNOGS");	// 2
        th(    "D'NYA");	// 2
        th(    "DNYID");	// 2
        th(    "DNYIS");	// 2
        th(     "'D'O");	// 2
        th(    "'DOBS");	// 2
        th(      "DOm");	// 2
        th(      "DOt");	// 2
        th(     "DPAB");	// 2
        th(  "DPA'I'O");	// 2
        th("DPA'RNAMS");	// 2
        th(     "DPE'");	// 2
        th(    "DPHYE");	// 2
        th(   "DPHYIR");	// 2
        th(     "DPRE");	// 2
        th(    "DPYER");	// 2
        th(    "DPYIR");	// 2
        th(     "DRA'");	// 2
        th(    "DRABS");	// 2
        th(   "DR'A'I");	// 2
        th(    "DR'AS");	// 2
        th(   "'DRENG");	// 2
        th(      "dRI");	// 2
        th(     "DR'I");	// 2
        th(    "DRNAG");	// 2
        th(  "DRO'ANG");	// 2
        th(    "DRU'I");	// 2
        th(     "DRUm");	// 2
        th(   "DRU:MA");	// 2
        th(   "'DRUNG");	// 2
        th(   "'DRU'O");	// 2
        th(    "DRU'O");	// 2
        th(  "DRVANGS");	// 2
        th(     "DRVI");	// 2
        th(     "DSAN");	// 2
        th(     "DSAR");	// 2
        th(      "DSE");	// 2
        th(    "DSOGS");	// 2
        th(     "DSRA");	// 2
        th(      "DTA");	// 2
        th(     "DT'A");	// 2
        th(      "^DU");	// 2
        th(     "DUBS");	// 2
        th(    "d'U'I");	// 2
        th(    "'DUL0");	// 2
        th(    "DUNDU");	// 2
        th(  "DU'ONGS");	// 2
        th(     "DURG");	// 2
        th("d'URYA'AM");	// 2
        th("D'URYA'AM");	// 2
        th( "d'U-RYAS");	// 2
        th( "d'UR+YAS");	// 2
        th(  "DUsht'A");	// 2
        th(       "DV");	// 2
        th(     "DV'A");	// 2
        th(    "DVABS");	// 2
        th(   "DVIshA");	// 2
        th(     "DVYA");	// 2
        th(     "dY'A");	// 2
        th(     "DYAN");	// 2
        th(      "DYE");	// 2
        th(     "DYEE");	// 2
        th(      "'DZ");	// 2
        th(     "'DZA");	// 2
        th(     "DZA4");	// 2
        th(     "DZAG");	// 2
        th(   "DZ'A'I");	// 2
        th(   "DZAM'A");	// 2
        th(   "DZAMLA");	// 2
        th(  "DZAMPHA");	// 2
        th(   "'DZAMS");	// 2
        th(   "'DZANG");	// 2
        th( "DZA-NY'A");	// 2
        th(    "DZDZA");	// 2
        th(     "DZ'E");	// 2
        th(   "'DZEBS");	// 2
        th("DZENTAKAS");	// 2
        th(    "DZHEE");	// 2
        th(    "DZ'Im");	// 2
        th(   "''DZIN");	// 2
        th(      "DZn");	// 2
        th( "DZNY'A'I");	// 2
        th(    "DZNYE");	// 2
        th(    "DZNYI");	// 2
        th(     "DZOB");	// 2
        th(     "DZOD");	// 2
        th(    "DZONG");	// 2
        th(    "'DZOR");	// 2
        th(     "DZRA");	// 2
        th(    "'DZUd");	// 2
        th(     "DZUL");	// 2
        th(    "DZUNG");	// 2
        th(   "'DZUNG");	// 2
        th(     "DZUR");	// 2
        th(     "DZWA");	// 2
        th(      "EMD");	// 2
        th(     "'ENG");	// 2
        th(      "ENI");	// 2
        th(    "ENYID");	// 2
        th(       "G-");	// 2
        th(    "'GA'A");	// 2
        th(   "'GA'AM");	// 2
        th(     "GAGN");	// 2
        th(     "GAHO");	// 2
        th(   "GA'I'O");	// 2
        th(    "'GAMS");	// 2
        th(   "GA-NAS");	// 2
        th( "GAnd'A'I");	// 2
        th(  "GAndH'I");	// 2
        th(   "GANDHI");	// 2
        th(  "GANdI'I");	// 2
        th(    "G'ANG");	// 2
        th(  "GANGA'I");	// 2
        th(  "GANGGAM");	// 2
        th( "GANG-G'I");	// 2
        th( "GANG+GLA");	// 2
        th(   "GANG'O");	// 2
        th(  "GANYJIR");	// 2
        th(    "G'A'O");	// 2
        th(     "GARB");	// 2
        th(  "GARBH'A");	// 2
        th( "GARBHASA");	// 2
        th(   "GARDZU");	// 2
        th( "GAR-TH'A");	// 2
        th(   "GA-SHE");	// 2
        th(    "GATTA");	// 2
        th(    "GA'UM");	// 2
        th(   "GA-Y'A");	// 2
        th(    "GA-YE");	// 2
        th(  "GA-YING");	// 2
        th(  "GA-YONG");	// 2
        th( "'GA'ZHIG");	// 2
        th(      "GBO");	// 2
        th(    "'GBOR");	// 2
        th(    "GBRAL");	// 2
        th(    "GBUGS");	// 2
        th(   "GBYA'O");	// 2
        th(    "GBYED");	// 2
        th(  "GCE'U'O");	// 2
        th(    "GCHOS");	// 2
        th(     "GCIB");	// 2
        th(     "GCIL");	// 2
        th(     "GCOL");	// 2
        th(     "GCON");	// 2
        th(     "GCUD");	// 2
        th(   "GCUGAS");	// 2
        th(     "GDAD");	// 2
        th(    "GDA'I");	// 2
        th(     "GDEB");	// 2
        th(     "GDED");	// 2
        th(    "GDEGS");	// 2
        th(     "GDIG");	// 2
        th(      "GDO");	// 2
        th(     "GDOB");	// 2
        th(     "GDOR");	// 2
        th(      "GEG");	// 2
        th(   "GE'I'O");	// 2
        th(     "GE'U");	// 2
        th(     "GGIS");	// 2
        th(     "GGOS");	// 2
        th(     "GHAG");	// 2
        th(   "GHAndI");	// 2
        th(    "GHANG");	// 2
        th(  "GHARBHA");	// 2
        th(  "GHARGAS");	// 2
        th(     "GHNA");	// 2
        th(    "GHNAN");	// 2
        th(    "GHOsh");	// 2
        th(   "GHUshA");	// 2
        th(      "GIL");	// 2
        th(      "GIM");	// 2
        th(      "GIs");	// 2
        th(     "GKRA");	// 2
        th(   "GKYANG");	// 2
        th(    "GLA'O");	// 2
        th(    "GLDAN");	// 2
        th(     "GLEt");	// 2
        th(    "GLE'U");	// 2
        th(     "GLID");	// 2
        th(     "GLIG");	// 2
        th(   "GLO'AM");	// 2
        th(    "GLO'I");	// 2
        th(     "GLOO");	// 2
        th(      "GMA");	// 2
        th(    "GNABS");	// 2
        th(    "GNAMS");	// 2
        th(    "^GNAS");	// 2
        th(    "GNASU");	// 2
        th(     "GNED");	// 2
        th(     "GNGA");	// 2
        th(    "GNGOL");	// 2
        th(    "GNGOS");	// 2
        th(    "GNGUL");	// 2
        th(     "GNOG");	// 2
        th(   "GNYA'I");	// 2
        th(  "GNYDZIR");	// 2
        th(    "GNYEL");	// 2
        th(   "GNYE'U");	// 2
        th(   "GNYIJA");	// 2
        th(  "'GO'ANG");	// 2
        th(    "GO+HA");	// 2
        th(     "GOSM");	// 2
        th(     "GPAR");	// 2
        th(    "GPHAN");	// 2
        th(    "GPO'I");	// 2
        th(   "GRA'AM");	// 2
        th(     "GRAH");	// 2
        th(    "GRANS");	// 2
        th(    "'GRAR");	// 2
        th(   "'GREBS");	// 2
        th(     "GRED");	// 2
        th(    "'GREG");	// 2
        th(    "GRE'I");	// 2
        th(     "GREN");	// 2
        th(    "GRGYA");	// 2
        th(    "GRIGS");	// 2
        th(  "GRI'U'I");	// 2
        th(    "'GRLE");	// 2
        th(  "GRO'ANG");	// 2
        th(     "GROB");	// 2
        th(   "'GROBS");	// 2
        th(     "GROd");	// 2
        th(    "GRO'U");	// 2
        th(   "GRTZUG");	// 2
        th(    "'GRUD");	// 2
        th(    "'GRUG");	// 2
        th(     "GRUN");	// 2
        th(    "GRVAB");	// 2
        th(   "GRVANG");	// 2
        th(    "'GRVO");	// 2
        th(  "GSHA'AM");	// 2
        th(    "GSHAS");	// 2
        th(    "GSHEL");	// 2
        th(    "GSHID");	// 2
        th(     "GSHO");	// 2
        th(     "GSHU");	// 2
        th(   "GSHUBS");	// 2
        th(   "GSHUGS");	// 2
        th(    "GSIGS");	// 2
        th(   "GSNYEN");	// 2
        th(    "GSRID");	// 2
        th(   "GSUNG0");	// 2
        th(   "GTHEGS");	// 2
        th(   "GTHUGS");	// 2
        th(    "GTSUL");	// 2
        th(     "GTUd");	// 2
        th(    "GTUNG");	// 2
        th(   "GTUNGS");	// 2
        th(     "GTUS");	// 2
        th(     "GTUt");	// 2
        th(      "GTZ");	// 2
        th(    "GTZA'");	// 2
        th(    "GTZAB");	// 2
        th(    "GTZAL");	// 2
        th(    "GTZED");	// 2
        th(   "GTZOGS");	// 2
        th(    "GTZOL");	// 2
        th(    "GTZUR");	// 2
        th(    "'GU'I");	// 2
        th(    "GUPSA");	// 2
        th(     "'GUS");	// 2
        th(     "GVAN");	// 2
        th(      "GVO");	// 2
        th(     "'GYA");	// 2
        th(     "GYA'");	// 2
        th(   "G-YA'I");	// 2
        th(   "G+YANG");	// 2
        th(   "G-YEGS");	// 2
        th(   "G-YE'I");	// 2
        th(  "'GYENGS");	// 2
        th(    "G-YES");	// 2
        th(     "GYI9");	// 2
        th(     "GYIB");	// 2
        th(    "'GYIN");	// 2
        th(    "G-YIN");	// 2
        th(   "GYINGS");	// 2
        th(    "G-YIS");	// 2
        th(    "GYI'U");	// 2
        th(     "GYOB");	// 2
        th(    "GYO'I");	// 2
        th(     "GYOR");	// 2
        th(    "GYO'U");	// 2
        th(     "GYUG");	// 2
        th(    "GYU'O");	// 2
        th(    "G-YUS");	// 2
        th(       "GZ");	// 2
        th(     "GZAM");	// 2
        th(     "GZEM");	// 2
        th(      "GZH");	// 2
        th(    "GZHEM");	// 2
        th(     "GZHG");	// 2
        th(    "GZH'I");	// 2
        th( "GZHI'I'O");	// 2
        th(     "GZHN");	// 2
        th(    "GZHUB");	// 2
        th(   "GZHUGN");	// 2
        th(    "GZIGs");	// 2
        th(     "GZIL");	// 2
        th(     "GZIt");	// 2
        th(     "GZOG");	// 2
        th(    "GZOGS");	// 2
        th(     "GZOL");	// 2
        th(    "GZUBS");	// 2
        th(     "H'A:");	// 2
        th(      "HA2");	// 2
        th(     "HAGS");	// 2
        th(    "H'AGS");	// 2
        th(     "HAMS");	// 2
        th(      "HAn");	// 2
        th(    "H'ANG");	// 2
        th(    "HASTA");	// 2
        th(     "HA'U");	// 2
        th(     "HENG");	// 2
        th(     "HERU");	// 2
        th(      "HIm");	// 2
        th(    "HiNYA");	// 2
        th(     "H'KA");	// 2
        th(     "HM'A");	// 2
        th(      "HME");	// 2
        th(   "HMRYUm");	// 2
        th(     "HO:-");	// 2
        th(   "HO:-SA");	// 2
        th(  "HO:TING");	// 2
        th(     "HRAM");	// 2
        th(     "HRAN");	// 2
        th(   "HRANGS");	// 2
        th(     "HR'i");	// 2
        th("HR'I:shIt':");	// 2
        th("HR'I:sht'I:");	// 2
        th(     "HSES");	// 2
        th(     "HTEG");	// 2
        th(   "HTSOGS");	// 2
        th(    "H'USA");	// 2
        th(     "HVAm");	// 2
        th(     "HVAR");	// 2
        th(      "HYE");	// 2
        th(       "'i");	// 2
        th(       "ID");	// 2
        th(       "IG");	// 2
        th(      "IGS");	// 2
        th(     "III-");	// 2
        th(   "III-02");	// 2
        th(   "III-05");	// 2
        th(   "III-07");	// 2
        th(  "III-102");	// 2
        th(   "III-11");	// 2
        th(  "III-110");	// 2
        th(  "III-112");	// 2
        th(  "III-117");	// 2
        th(  "III-121");	// 2
        th(  "III-129");	// 2
        th(  "III-132");	// 2
        th(   "III-14");	// 2
        th(  "III-145");	// 2
        th(  "III-151");	// 2
        th(  "III-158");	// 2
        th(  "III-162");	// 2
        th(  "III-167");	// 2
        th(  "III-180");	// 2
        th(  "III-185");	// 2
        th(   "III-20");	// 2
        th(  "III-204");	// 2
        th(  "III-211");	// 2
        th(  "III-216");	// 2
        th(  "III-224");	// 2
        th(  "III-232");	// 2
        th(  "III-239");	// 2
        th(  "III-242");	// 2
        th(  "III-244");	// 2
        th(   "III-27");	// 2
        th(   "III-32");	// 2
        th(   "III-43");	// 2
        th(   "III-58");	// 2
        th(   "III-65");	// 2
        th(   "III-66");	// 2
        th(   "III-69");	// 2
        th(   "III-73");	// 2
        th(   "III-77");	// 2
        th(   "III-80");	// 2
        th(   "III-82");	// 2
        th(   "III-86");	// 2
        th(   "III-92");	// 2
        th(   "III-93");	// 2
        th(   "III-95");	// 2
        th(      "'IN");	// 2
        th(      "'IS");	// 2
        th(    "IV-07");	// 2
        th(    "IV-10");	// 2
        th(   "IV-105");	// 2
        th(    "IV-11");	// 2
        th(   "IV-118");	// 2
        th(   "IV-119");	// 2
        th(   "IV-121");	// 2
        th(   "IV-124");	// 2
        th(   "IV-127");	// 2
        th(   "IV-134");	// 2
        th(   "IV-136");	// 2
        th(   "IV-138");	// 2
        th(   "IV-154");	// 2
        th(   "IV-156");	// 2
        th(   "IV-169");	// 2
        th(    "IV-17");	// 2
        th(   "IV-171");	// 2
        th(    "IV-19");	// 2
        th(   "IV-206");	// 2
        th(    "IV-22");	// 2
        th(    "IV-27");	// 2
        th(    "IV-29");	// 2
        th(    "IV-33");	// 2
        th(    "IV-34");	// 2
        th(    "IV-50");	// 2
        th(    "IV-57");	// 2
        th(    "IV-58");	// 2
        th(    "IV-61");	// 2
        th(    "IV-66");	// 2
        th(    "IV-67");	// 2
        th(    "IV-81");	// 2
        th(    "IV-90");	// 2
        th(     "IYIS");	// 2
        th(    "IZHES");	// 2
        th(    "'JA'I");	// 2
        th(      "'JE");	// 2
        th(     "'JEB");	// 2
        th(     "JENG");	// 2
        th(      "JI2");	// 2
        th(   "JI'ANG");	// 2
        th(      "JIM");	// 2
        th(    "'JIMS");	// 2
        th(     "'JIR");	// 2
        th(     "JOHN");	// 2
        th(     "'JUL");	// 2
        th(     "JU'U");	// 2
        th(      "JYI");	// 2
        th(     "'JZO");	// 2
        th(     "'KAG");	// 2
        th(     "KAGS");	// 2
        th(     "KAHA");	// 2
        th(    "KA:'I");	// 2
        th(   "KA'I'O");	// 2
        th(    "KAKnI");	// 2
        th(     "K'AL");	// 2
        th(    "KAL'A");	// 2
        th(   "KALY'A");	// 2
        th(     "K'AM");	// 2
        th(   "KAMAPA");	// 2
        th(  "KANGnnA");	// 2
        th(    "KAntA");	// 2
        th( "KAnthA'I");	// 2
        th(  "KANYDZI");	// 2
        th(  "KANYTZA");	// 2
        th("KANYTZI'I");	// 2
        th(    "K'A'O");	// 2
        th(     "KARA");	// 2
        th(   "KA'RA'");	// 2
        th(   "KARMMA");	// 2
        th(   "KARNAS");	// 2
        th(  "K'ARshA");	// 2
        th(   "KARshI");	// 2
        th(   "KARTRi");	// 2
        th(    "KAshE");	// 2
        th(    "KAshU");	// 2
        th(     "KAtA");	// 2
        th(   "KAtA'I");	// 2
        th(     "KEGS");	// 2
        th(    "'KHAB");	// 2
        th(   "KHAdGA");	// 2
        th(   "KHADGA");	// 2
        th(   "KHADRA");	// 2
        th(   "KHAMS:");	// 2
        th( "KHAndA'I");	// 2
        th(  "'KHANGS");	// 2
        th(    "'KHEN");	// 2
        th(  "'KHENGS");	// 2
        th(     "KHIR");	// 2
        th(     "KHIS");	// 2
        th(   "'KHO'I");	// 2
        th(     "KHOO");	// 2
        th(      "KHR");	// 2
        th(  "'KHRABS");	// 2
        th(   "'KHRAD");	// 2
        th( "KHRA'ING");	// 2
        th(   "'KHRAN");	// 2
        th(   "KHRA'U");	// 2
        th(   "KHREMS");	// 2
        th(    "KHREN");	// 2
        th(   "KHRENG");	// 2
        th(   "'KHRES");	// 2
        th( "KHRE'U'I");	// 2
        th(   "KHRI'A");	// 2
        th(   "'KHRIN");	// 2
        th( "KHRI'U'O");	// 2
        th(   "'KHROR");	// 2
        th(   "KHRUGS");	// 2
        th(  "'KHRUNS");	// 2
        th(    "KHRVI");	// 2
        th(  "'KHU'AM");	// 2
        th(    "'KHUG");	// 2
        th(    "KHVAS");	// 2
        th(     "KHVI");	// 2
        th(     "KHWA");	// 2
        th(   "^KHYAB");	// 2
        th(     "KHYD");	// 2
        th(   "KHYE'I");	// 2
        th(    "KHYES");	// 2
        th(    "KHYIm");	// 2
        th( "KHYU'ANG");	// 2
        th(  "'KHYUNG");	// 2
        th(  "KHYUNGS");	// 2
        th(      "KIm");	// 2
        th(   "KIM-PA");	// 2
        th(   "K'IRTA");	// 2
        th(   "KIRTIR");	// 2
        th(   "KIRTIS");	// 2
        th(     "KI'U");	// 2
        th(    "KLA'I");	// 2
        th(   "KLANGS");	// 2
        th(     "KLAR");	// 2
        th(    "KLENG");	// 2
        th(     "KL'i");	// 2
        th(   "'KLUBS");	// 2
        th(     "KLUM");	// 2
        th(      "KNU");	// 2
        th(      "K'O");	// 2
        th(     "'KOB");	// 2
        th(     "KOGS");	// 2
        th(   "KOshAR");	// 2
        th(     "KRAB");	// 2
        th(     "KRAM");	// 2
        th(  "KR'ANTI");	// 2
        th(    "KRI'A");	// 2
        th(   "KR'I'I");	// 2
        th(     "KRIN");	// 2
        th("KR'INTE'O");	// 2
        th(     "KRiS");	// 2
        th(     "KROD");	// 2
        th(     "KRON");	// 2
        th(    "KRU'U");	// 2
        th(    "KshA:");	// 2
        th(    "KshAm");	// 2
        th(   "KshI'I");	// 2
        th(  "KshIPTA");	// 2
        th(   "KshITA");	// 2
        th( "KshMRYUm");	// 2
        th(    "KTI'O");	// 2
        th(   "KUBDZA");	// 2
        th(  "KUBDZ'A");	// 2
        th(     "KUKN");	// 2
        th(     "KUN0");	// 2
        th(    "KUNAD");	// 2
        th(   "KUNDHA");	// 2
        th(    "KUndI");	// 2
        th( "KUNNGA'I");	// 2
        th(     "KU'O");	// 2
        th(     "KU'U");	// 2
        th(    "KY'AN");	// 2
        th(    "KY'AS");	// 2
        th(     "KYEM");	// 2
        th(     "KYER");	// 2
        th(    "KYI'I");	// 2
        th(   "KYI'NG");	// 2
        th(    "KYI'U");	// 2
        th(      "KYS");	// 2
        th(     "KYUN");	// 2
        th(    "'KYUR");	// 2
        th(      "LA4");	// 2
        th(     "LA'A");	// 2
        th(   "LABDZA");	// 2
        th(     "L'AD");	// 2
        th(  "LADMIGS");	// 2
        th(      "LAH");	// 2
        th(  "LAKsh'I");	// 2
        th(  "LAKshIm");	// 2
        th(  "LAMSGOM");	// 2
        th(  "LAndA'I");	// 2
        th(  "LAndARA");	// 2
        th(  "LANdARA");	// 2
        th(  "LANGKRA");	// 2
        th(    "LANTA");	// 2
        th(  "LANYTZA");	// 2
        th(   "LA'O'I");	// 2
        th(    "LA'OS");	// 2
        th(     "LASO");	// 2
        th(      "LAt");	// 2
        th(     "L'AU");	// 2
        th(     "LBAS");	// 2
        th(     "LBUG");	// 2
        th(     "LCAB");	// 2
        th(     "LCAN");	// 2
        th(   "LCANGS");	// 2
        th(    "LCEGS");	// 2
        th(     "LCUM");	// 2
        th(    "LCUNG");	// 2
        th(     "LD'A");	// 2
        th(    "'LDAN");	// 2
        th(    "LDANS");	// 2
        th(    "LDA'O");	// 2
        th(     "LDEN");	// 2
        th(  "LDE'U'I");	// 2
        th(     "LDID");	// 2
        th(     "LDNA");	// 2
        th(    "LDOBS");	// 2
        th(     "LDUS");	// 2
        th(      "L'E");	// 2
        th(      "LEM");	// 2
        th( "LENTRA'I");	// 2
        th(     "LESG");	// 2
        th(    "LGA'I");	// 2
        th(    "LGENG");	// 2
        th(     "LGOG");	// 2
        th(     "^LHA");	// 2
        th(     "LHAL");	// 2
        th(     "LHAm");	// 2
        th(     "LHAT");	// 2
        th(    "LHE'U");	// 2
        th(    "LHING");	// 2
        th(   "^LHUNG");	// 2
        th(    "LIDTA");	// 2
        th(  "LINGGAR");	// 2
        th(   "LINGKA");	// 2
        th(    "LINTI");	// 2
        th(     "LJAD");	// 2
        th(    "LJA'I");	// 2
        th(     "LJAS");	// 2
        th(    "LJOGS");	// 2
        th(      "LKA");	// 2
        th(      "LLI");	// 2
        th(    "LNGA'");	// 2
        th(    "LNGAB");	// 2
        th(    "LNGAL");	// 2
        th(   "LNG'AM");	// 2
        th(   "LNGE'I");	// 2
        th(    "LNGEM");	// 2
        th(    "LNGER");	// 2
        th(    "LNGKA");	// 2
        th(      "LNU");	// 2
        th(    "LOKYA");	// 2
        th( "LOTZTSAS");	// 2
        th(     "LPAR");	// 2
        th(      "LRA");	// 2
        th(    "LSTAR");	// 2
        th(     "^LTA");	// 2
        th(     "LTAL");	// 2
        th(    "LTARO");	// 2
        th(     "LTED");	// 2
        th(    "LTEGS");	// 2
        th(     "LTRA");	// 2
        th(     "LTUL");	// 2
        th(    "LTZAL");	// 2
        th(    "LU'AM");	// 2
        th(      "LUB");	// 2
        th( "LUNDHA'I");	// 2
        th(     "LUS0");	// 2
        th(      "LUt");	// 2
        th(     "LVAR");	// 2
        th(      "LVE");	// 2
        th(     "L+YA");	// 2
        th(    "L+YAB");	// 2
        th(     "LYAm");	// 2
        th(      "MAA");	// 2
        th(  "MADHY'A");	// 2
        th(     "MAGS");	// 2
        th(   "MA-H'A");	// 2
        th(    "MA'IS");	// 2
        th(   "MANdAL");	// 2
        th(    "MAnER");	// 2
        th(  "MANGG'A");	// 2
        th( "MANG+GAL");	// 2
        th("MANG+GALAm");	// 2
        th(  "MAN-NGA");	// 2
        th(   "MANT'A");	// 2
        th(   "MANTHA");	// 2
        th(    "MAntO");	// 2
        th( "MANYADZU");	// 2
        th(  "MANYDZA");	// 2
        th(   "MANYJI");	// 2
        th(    "MA:'O");	// 2
        th(   "MA'ONG");	// 2
        th(    "MA'OR");	// 2
        th(   "MA:PRA");	// 2
        th(     "M'AR");	// 2
        th(     "MAR:");	// 2
        th(  "M'ARTHA");	// 2
        th( "MA:SARBA");	// 2
        th(    "MAZAD");	// 2
        th(      "MBA");	// 2
        th(     "MBHU");	// 2
        th(    "M+B'I");	// 2
        th(    "MB'IR");	// 2
        th(    "MBYED");	// 2
        th(    "MCHA'");	// 2
        th(   "MCHA'I");	// 2
        th(    "MCHAS");	// 2
        th(     "MCHD");	// 2
        th(    "MCHEG");	// 2
        th(   "MCHEMS");	// 2
        th(    "MCHGO");	// 2
        th(   "MCHI'I");	// 2
        th(    "MCHOL");	// 2
        th(   "MCHOMS");	// 2
        th( "MCHU'ANG");	// 2
        th(     "MCIS");	// 2
        th(    "MDE'I");	// 2
        th(     "MDER");	// 2
        th(    "MDHAD");	// 2
        th(     "MDRA");	// 2
        th(    "MDRIS");	// 2
        th(     "MDUM");	// 2
        th(     "MDUS");	// 2
        th( "MDZA-DZA");	// 2
        th(    "MDZAG");	// 2
        th(    "MDZAL");	// 2
        th(  "MDZE'AM");	// 2
        th(   "MDZES0");	// 2
        th(   "MDZOGS");	// 2
        th(   "MDZONG");	// 2
        th(    "MDZUM");	// 2
        th(     "ME3D");	// 2
        th(     "MEEm");	// 2
        th(      "MEm");	// 2
        th(     "MEMD");	// 2
        th(    "MGA'I");	// 2
        th(     "MGAN");	// 2
        th(     "MGIN");	// 2
        th(  "MGO'ANG");	// 2
        th(   "MGRONG");	// 2
        th(     "MGRU");	// 2
        th(     "MGUB");	// 2
        th(     "MGUG");	// 2
        th(    "MGUNG");	// 2
        th(    "MGYOd");	// 2
        th(    "MGYOt");	// 2
        th(    "MHA'A");	// 2
        th(     "MHVA");	// 2
        th(       "mi");	// 2
        th(     "MI'A");	// 2
        th(    "MILLS");	// 2
        th(    "MINYA");	// 2
        th(    "M'I'O");	// 2
        th(   "MIRBA'");	// 2
        th(    "M'ISA");	// 2
        th(   "MJANGS");	// 2
        th(     "MJID");	// 2
        th(   "MJINGS");	// 2
        th(    "MJOMS");	// 2
        th(     "MKA'");	// 2
        th(    "MKAH'");	// 2
        th(    "MKAHS");	// 2
        th(    "MKHAM");	// 2
        th(   "MKHAS0");	// 2
        th(   "MKHREL");	// 2
        th(  "MKHRIGS");	// 2
        th(     "MKHU");	// 2
        th(   "MKHUGS");	// 2
        th(   "MKHYUN");	// 2
        th(      "MnA");	// 2
        th(     "MNAD");	// 2
        th(    "MNA'I");	// 2
        th(    "MND'A");	// 2
        th(     "MnER");	// 2
        th(    "MNGAD");	// 2
        th(    "MNGUN");	// 2
        th(    "MNING");	// 2
        th(    "MNYA'");	// 2
        th( "MNYA-DZU");	// 2
        th(   "MNYAmS");	// 2
        th(     "MNYI");	// 2
        th(     "MNYm");	// 2
        th(    "MNYON");	// 2
        th("MOONG+GAL");	// 2
        th(     "MRDE");	// 2
        th(     "MRDO");	// 2
        th(   "MRiDZA");	// 2
        th(     "MRiN");	// 2
        th(    "MRJES");	// 2
        th(     "MRKA");	// 2
        th(       "MS");	// 2
        th(    "MSDUG");	// 2
        th(   "MSTAMS");	// 2
        th(      "MTA");	// 2
        th(     "MTA'");	// 2
        th(    "MTHIB");	// 2
        th(   "MTHI'I");	// 2
        th(    "MTHOM");	// 2
        th(    "MTHUM");	// 2
        th(     "MTOS");	// 2
        th(    "MTSEG");	// 2
        th(     "MTSI");	// 2
        th(    "MTSOB");	// 2
        th(   "MTSOMS");	// 2
        th(     "MTUN");	// 2
        th(   "MU'ANG");	// 2
        th(    "MUDRA");	// 2
        th(   "MUK-TA");	// 2
        th(      "MUm");	// 2
        th(    "MUntI");	// 2
        th(  "MUNYTZA");	// 2
        th(    "MURTI");	// 2
        th( "MUTZTS'A");	// 2
        th(     "MYAG");	// 2
        th(    "MYAKA");	// 2
        th(     "MZA'");	// 2
        th(    "MZHES");	// 2
        th(    "MZHUM");	// 2
        th(     "MZUN");	// 2
        th(     "NAGN");	// 2
        th(   "nA'I'O");	// 2
        th(     "NAM:");	// 2
        th( "NAMKHA'I");	// 2
        th(     "NAMO");	// 2
        th(   "NAN+TA");	// 2
        th(   "NANT'A");	// 2
        th(  "NANTA'O");	// 2
        th(  "NANTASA");	// 2
        th(    "N'A'O");	// 2
        th(     "NATA");	// 2
        th(    "NAttE");	// 2
        th(     "n+AY");	// 2
        th(   "NA-Y'A");	// 2
        th(      "NBA");	// 2
        th(    "NBA'I");	// 2
        th(     "NCAN");	// 2
        th(    "NDANG");	// 2
        th(      "NDI");	// 2
        th(    "NDRAS");	// 2
        th(      "NEL");	// 2
        th(      "NEN");	// 2
        th(     "NE'O");	// 2
        th(    "'NGAB");	// 2
        th(    "NGASI");	// 2
        th(     "NGEL");	// 2
        th(    "NGENG");	// 2
        th(    "NG+GA");	// 2
        th(     "NGID");	// 2
        th(     "NGIN");	// 2
        th(     "NG'O");	// 2
        th(     "NGO'");	// 2
        th(   "NGO'AM");	// 2
        th(    "'NGOG");	// 2
        th(    "'NGON");	// 2
        th(      "NGS");	// 2
        th(    "NGYIS");	// 2
        th(     "nHRI");	// 2
        th(    "NI'AM");	// 2
        th(    "NI'DI");	// 2
        th(     "ni'I");	// 2
        th(     "nIKA");	// 2
        th(      "NIP");	// 2
        th(      "nIR");	// 2
        th(    "NIRYA");	// 2
        th(  "NIR+Y'A");	// 2
        th( "NIshsh'A");	// 2
        th( "NIshth'A");	// 2
        th(     "NITA");	// 2
        th(   "NIZHIG");	// 2
        th(      "NLA");	// 2
        th(     "NLTA");	// 2
        th(      "NNI");	// 2
        th(    "NNYID");	// 2
        th(      "n'O");	// 2
        th(     "'NOD");	// 2
        th(     "NO'U");	// 2
        th(      "NPA");	// 2
        th(   "NPHYIR");	// 2
        th(    "NRAMS");	// 2
        th(     "NR'I");	// 2
        th(    "NRNAM");	// 2
        th(       "NS");	// 2
        th(      "ntI");	// 2
        th(     "nt'I");	// 2
        th(     "NT'I");	// 2
        th(      "NUN");	// 2
        th(     "NUNG");	// 2
        th(     "NU'U");	// 2
        th(     "NYA4");	// 2
        th(     "NYAB");	// 2
        th(    "NYAGS");	// 2
        th(   "NYAHAm");	// 2
        th(   "NYAMSU");	// 2
        th(   "NYATZA");	// 2
        th(      "NYD");	// 2
        th(     "^NYE");	// 2
        th(   "NYE'AM");	// 2
        th(  "NYE'ANG");	// 2
        th(     "NYEB");	// 2
        th(    "N+YEE");	// 2
        th(    "NYEMS");	// 2
        th(    "NYIDU");	// 2
        th(     "NYIm");	// 2
        th(   "NYINGS");	// 2
        th( "NY+RAHAm");	// 2
        th(  "NYTZA'I");	// 2
        th(    "NYTZI");	// 2
        th(     "NYUR");	// 2
        th(    "NYU'U");	// 2
        th(      "^'O");	// 2
        th(       "OG");	// 2
        th(       "OK");	// 2
        th(       "OL");	// 2
        th(     "ONGS");	// 2
        th(     "'ONS");	// 2
        th(      "P9O");	// 2
        th(     "'PA'");	// 2
        th(     "P''A");	// 2
        th(      "PAA");	// 2
        th(    "PA'AN");	// 2
        th(     "'PAD");	// 2
        th(    "PA'DI");	// 2
        th(   "PA-DMA");	// 2
        th( "PADMANTA");	// 2
        th(  "PADMASA");	// 2
        th(   "PAD+MO");	// 2
        th(     "PADR");	// 2
        th(    "PAGES");	// 2
        th(    "P'A'I");	// 2
        th(     "PAI'");	// 2
        th(    "PA'I0");	// 2
        th(      "PAK");	// 2
        th(     "PAKR");	// 2
        th(   "PAKshE");	// 2
        th( "PAKSHI'I");	// 2
        th(  "PAKSHIS");	// 2
        th( "PAKshU'I");	// 2
        th(    "PAKTA");	// 2
        th(    "PALAS");	// 2
        th(     "PALS");	// 2
        th(     "PA'm");	// 2
        th("PAndIRTHA");	// 2
        th(    "PAnI'");	// 2
        th(  "P'AnI'I");	// 2
        th(    "PANNA");	// 2
        th(    "PAntA");	// 2
        th(   "PANYCA");	// 2
        th("PANYTZA'I");	// 2
        th(     "'PAR");	// 2
        th(     "P'AR");	// 2
        th(    "PARBA");	// 2
        th( "PARDZNYA");	// 2
        th( "PAR'GYUR");	// 2
        th(    "PAR'I");	// 2
        th( "PARIshtA");	// 2
        th(    "PARnA");	// 2
        th(  "PARnA'I");	// 2
        th(   "PARshA");	// 2
        th(     "PAS'");	// 2
        th(    "PAshA");	// 2
        th(  "PASth'A");	// 2
        th(    "P'ATA");	// 2
        th(   "PATA'O");	// 2
        th(    "PATTI");	// 2
        th(   "PATZYA");	// 2
        th(    "PAZER");	// 2
        th(     "PBAR");	// 2
        th(    "PDA'I");	// 2
        th(    "'PHA'");	// 2
        th(     "PHAn");	// 2
        th(    "PHAt:");	// 2
        th(     "PHE2");	// 2
        th(    "'PHEB");	// 2
        th(   "'PHEGS");	// 2
        th(     "PHEM");	// 2
        th(    "PHI'I");	// 2
        th(    "'PHIN");	// 2
        th(    "'PHIS");	// 2
        th(     "PHOt");	// 2
        th(      "PHR");	// 2
        th(  "'PHRAGS");	// 2
        th( "'PHRANGS");	// 2
        th(   "'PHRAR");	// 2
        th(  "PHRENGS");	// 2
        th(    "PHRER");	// 2
        th(  "PHRE'US");	// 2
        th(   "'PHRIL");	// 2
        th(    "PHRIS");	// 2
        th(   "'PHROd");	// 2
        th(   "'PHROM");	// 2
        th(   "PHRONG");	// 2
        th(   "'PHRUB");	// 2
        th(  "'PHRUGS");	// 2
        th(   "'PHRUN");	// 2
        th(     "'PHU");	// 2
        th(     "PH'U");	// 2
        th(    "'PHUB");	// 2
        th(   "PHULLA");	// 2
        th(     "PHUm");	// 2
        th(    "^PHUN");	// 2
        th(   "'PHYAB");	// 2
        th(    "PHYAd");	// 2
        th(   "PHYA'I");	// 2
        th(  "'PHYA'O");	// 2
        th(   "PHYA'O");	// 2
        th(    "PHYEB");	// 2
        th(   "'PHYEN");	// 2
        th( "PHYI'ANG");	// 2
        th(   "PHYIMS");	// 2
        th(  "PHYINGS");	// 2
        th( "'PHYINGS");	// 2
        th(   "^PHYIR");	// 2
        th(   "PHYIRK");	// 2
        th(   "PHYIRN");	// 2
        th( "PHYIYI'I");	// 2
        th( "'PHYO'AM");	// 2
        th(   "'PHYOG");	// 2
        th(  "'PHYO'O");	// 2
        th(       "Pi");	// 2
        th(     "P''I");	// 2
        th(      "PI'");	// 2
        th(   "PInd'A");	// 2
        th( "PINGGA'I");	// 2
        th(    "P'I'O");	// 2
        th(      "PIS");	// 2
        th(   "PIshtI");	// 2
        th(    "PKshA");	// 2
        th(      "PMA");	// 2
        th(  "PNGMA'I");	// 2
        th(   "PNGM'I");	// 2
        th(     "PnId");	// 2
        th(     "PntI");	// 2
        th(     "PO'A");	// 2
        th(     "'POG");	// 2
        th(    "'POGS");	// 2
        th(    "P'O'I");	// 2
        th(      "POL");	// 2
        th(      "POM");	// 2
        th(    "PPA'I");	// 2
        th(  "PRABH'A");	// 2
        th(    "'PRAD");	// 2
        th(     "PRAN");	// 2
        th(  "PRAST'A");	// 2
        th(     "PR'I");	// 2
        th("PRIKshA'I");	// 2
        th(     "PRIm");	// 2
        th(  "PRINTED");	// 2
        th(    "PRiTA");	// 2
        th(   "PRnA'I");	// 2
        th(   "PRNAMS");	// 2
        th(     "PROD");	// 2
        th(   "PRTI'I");	// 2
        th(  "PRTZTSA");	// 2
        th(      "PRU");	// 2
        th(     "'PRU");	// 2
        th( "PUDZNY'A");	// 2
        th(   "PUKshU");	// 2
        th(    "PUNDA");	// 2
        th(    "PUNGS");	// 2
        th(   "PUnY'A");	// 2
        th(     "P'UR");	// 2
        th(     "PY'A");	// 2
        th(    "'PYED");	// 2
        th(    "PYI'I");	// 2
        th(      "PYO");	// 2
        th(    "'PYOD");	// 2
        th(     "RAB0");	// 2
        th( "RADZNY'A");	// 2
        th(     "RAGB");	// 2
        th(     "R'AI");	// 2
        th(    "RAKMO");	// 2
        th(  "RAKshA:");	// 2
        th(  "RAKsh'I");	// 2
        th( "RAKshITA");	// 2
        th(    "RAKTO");	// 2
        th(      "RAn");	// 2
        th(     "R'AN");	// 2
        th(    "RAN'A");	// 2
        th(  "RAndA'I");	// 2
        th(  "RANDANA");	// 2
        th(    "RARMA");	// 2
        th(   "RATA'O");	// 2
        th(    "RATnA");	// 2
        th( "R'A:ZHES");	// 2
        th(    "RBANG");	// 2
        th(     "RBEG");	// 2
        th(    "RBH'A");	// 2
        th(     "RBHU");	// 2
        th(     "RBOG");	// 2
        th(     "RDA'");	// 2
        th(    "RDABS");	// 2
        th(    "RDANG");	// 2
        th(     "RDES");	// 2
        th(  "RDE'U'I");	// 2
        th(     "RDHA");	// 2
        th(   "RDHOGS");	// 2
        th(   "RDHRMA");	// 2
        th(    "RDOBS");	// 2
        th(    "'RDUG");	// 2
        th(    "RDZAM");	// 2
        th(   "RDZANG");	// 2
        th(    "RDZEL");	// 2
        th(    "RDZEN");	// 2
        th( "RDZE'U'I");	// 2
        th(    "RDZIB");	// 2
        th(    "RDZIL");	// 2
        th(    "RDZIN");	// 2
        th("RDZ+NY'A+NA");	// 2
        th(   "RDZOBS");	// 2
        th(   "RDZO'I");	// 2
        th(    "RDZOM");	// 2
        th(     "RDZS");	// 2
        th(   "^RDZUN");	// 2
        th(     "REBS");	// 2
        th(      "REJ");	// 2
        th( "RENDRA'I");	// 2
        th(  "RENGshE");	// 2
        th(  "REVERSE");	// 2
        th(       "RG");	// 2
        th(     "RGED");	// 2
        th(     "RGHO");	// 2
        th(     "RGIS");	// 2
        th(     "RGOG");	// 2
        th(      "RGS");	// 2
        th(    "'RGYA");	// 2
        th(   "RGYABS");	// 2
        th(   "'RGYAL");	// 2
        th(   "RGYAL0");	// 2
        th(  "RGYA-SO");	// 2
        th(    "RGYEL");	// 2
        th(   "RGYI'U");	// 2
        th(     "RGYO");	// 2
        th(   "RGYOBS");	// 2
        th(    "RGYOD");	// 2
        th(    "RGYOS");	// 2
        th(    "'RGYU");	// 2
        th(   "'RGYUN");	// 2
        th(   "R'GYUR");	// 2
        th(     "RIGs");	// 2
        th(    "R'I'O");	// 2
        th(    "R'iTA");	// 2
        th(    "RJA'I");	// 2
        th(     "RJE'");	// 2
        th(    "'RJES");	// 2
        th(    "RJES0");	// 2
        th(    "RJONG");	// 2
        th(   "RKANGS");	// 2
        th(     "RKAt");	// 2
        th(     "RKEN");	// 2
        th(    "RKHAS");	// 2
        th(     "RKOD");	// 2
        th(     "RKON");	// 2
        th(    "RKshI");	// 2
        th(     "RKUL");	// 2
        th(   "RKYAGS");	// 2
        th(    "RKYAR");	// 2
        th(   "RKYENG");	// 2
        th(     "RKYI");	// 2
        th(    "RKYON");	// 2
        th(    "RKYUN");	// 2
        th(    "RKYUS");	// 2
        th(     "RLEN");	// 2
        th(      "RLI");	// 2
        th(     "RLIS");	// 2
        th(   "RLONGS");	// 2
        th(     "RLUG");	// 2
        th(     "RLUS");	// 2
        th(    "RMA'O");	// 2
        th( "RMARKshI");	// 2
        th(    "RMENG");	// 2
        th(   "RMENGS");	// 2
        th(    "RMI'I");	// 2
        th(     "RMUN");	// 2
        th(    "RNAMM");	// 2
        th(    "RNAMs");	// 2
        th(   "RNGAmS");	// 2
        th(   "RNGA'O");	// 2
        th(     "RNGE");	// 2
        th(   "RNGO'I");	// 2
        th(    "RNGOM");	// 2
        th(     "RNMA");	// 2
        th(     "RNMS");	// 2
        th(     "RNOL");	// 2
        th(     "RNTA");	// 2
        th(    "RNYAG");	// 2
        th(    "RNYIR");	// 2
        th(    "RNYOB");	// 2
        th(      "R'O");	// 2
        th("RONYKAsh'A");	// 2
        th("RONYKsh''I");	// 2
        th(     "RPOD");	// 2
        th(     "RREN");	// 2
        th(       "RT");	// 2
        th(  "RTA'ANG");	// 2
        th(    "RTH'A");	// 2
        th(    "RTHA:");	// 2
        th(    "RTHAm");	// 2
        th(    "RTOSG");	// 2
        th(    "RTSZA");	// 2
        th(    "RTUGS");	// 2
        th(     "RTZD");	// 2
        th(   "RTZEBS");	// 2
        th(    "RTZIM");	// 2
        th(  "RTZITTA");	// 2
        th(    "RTZOB");	// 2
        th(    "RTZUN");	// 2
        th(   "RTZUNG");	// 2
        th(  "RTZVA'O");	// 2
        th(   "RTZV'I");	// 2
        th(  "RTZWA'I");	// 2
        th(    "RUANG");	// 2
        th(     "RUGS");	// 2
        th(     "RUMS");	// 2
        th(     "RV'A");	// 2
        th(    "RV'AI");	// 2
        th(     "RVAm");	// 2
        th(     "RV'I");	// 2
        th(    "RVI'I");	// 2
        th(     "R-YA");	// 2
        th(     "RYGU");	// 2
        th(      "RZA");	// 2
        th(     "RZAM");	// 2
        th(   "RZDOGS");	// 2
        th(     "RZHI");	// 2
        th(     "RZOM");	// 2
        th("SADDHARMA");	// 2
        th(    "SADHI");	// 2
        th(   "SADKRI");	// 2
        th(    "SADYA");	// 2
        th(  "S'A-H'A");	// 2
        th(    "SAM'A");	// 2
        th(   "SAmBHA");	// 2
        th( "SAMBHOtA");	// 2
        th(  "SAmSKRi");	// 2
        th( "SAMUDR'A");	// 2
        th(   "SANDHA");	// 2
        th(  "SANG+GA");	// 2
        th( "SANG-GHA");	// 2
        th(    "SANTI");	// 2
        th(  "SANYDZA");	// 2
        th(     "SARB");	// 2
        th(   "SARB'A");	// 2
        th(  "SARBA'I");	// 2
        th(    "SARWE");	// 2
        th(  "SATVA'I");	// 2
        th(    "SA+YA");	// 2
        th(     "SBAN");	// 2
        th(    "SBCOM");	// 2
        th(    "SBEBS");	// 2
        th(   "SBGYUR");	// 2
        th(     "SBOL");	// 2
        th(    "SBRAB");	// 2
        th(  "SBRANGS");	// 2
        th(   "SBRA'O");	// 2
        th(    "SBRIB");	// 2
        th(    "SBRIS");	// 2
        th(   "SBROGS");	// 2
        th(    "SBROL");	// 2
        th(    "SBROM");	// 2
        th(   "SBRO'O");	// 2
        th(   "SBRUNG");	// 2
        th(  "SBSRUNG");	// 2
        th(     "SBVA");	// 2
        th(   "SBYA'I");	// 2
        th(   "SBYAMS");	// 2
        th(    "SBYAS");	// 2
        th(  "SBYI'AM");	// 2
        th(    "SBYIG");	// 2
        th(    "SBYOB");	// 2
        th(   "SBYONR");	// 2
        th(   "^SBYOR");	// 2
        th(    "SDA'I");	// 2
        th(     "SDID");	// 2
        th(    "SDUBS");	// 2
        th(    "SDU'I");	// 2
        th(   "SDUNGS");	// 2
        th(    "SDU'O");	// 2
        th(      "S'E");	// 2
        th(    "SE'AM");	// 2
        th(     "^SEL");	// 2
        th(     "SELS");	// 2
        th(     "SEmN");	// 2
        th(    "SENG-");	// 2
        th("SENG-DGE'I");	// 2
        th( "SENGEG'I");	// 2
        th(   "SENGGA");	// 2
        th( "SENGGA'I");	// 2
        th("SE-NGGE'I");	// 2
        th(    "SERIN");	// 2
        th(   "SE'U'I");	// 2
        th(     "SGAG");	// 2
        th(    "SGEGS");	// 2
        th(     "SGEN");	// 2
        th(     "SGIB");	// 2
        th(    "SGIRB");	// 2
        th(     "SGIS");	// 2
        th(    "SGNON");	// 2
        th(    "SGOBS");	// 2
        th(    "SGO'U");	// 2
        th(   "SGRABS");	// 2
        th(    "SGRAD");	// 2
        th(   "SGRE'U");	// 2
        th(   "SGRONG");	// 2
        th(  "SGRUNGS");	// 2
        th(    "SGRUS");	// 2
        th(    "SGSRA");	// 2
        th(    "SGUBS");	// 2
        th(     "SGUS");	// 2
        th(   "SGYABS");	// 2
        th(    "SGYAN");	// 2
        th(    "SGYEN");	// 2
        th(  "SGYU'AM");	// 2
        th(    "SGYUB");	// 2
        th(   "'SGYUR");	// 2
        th(     "shA'");	// 2
        th(     "sH'A");	// 2
        th(   "SH'A'I");	// 2
        th(  "SHAKY'A");	// 2
        th("SH'AKYI'I");	// 2
        th( "SH'AMBHA");	// 2
        th( "SHAMBHU:");	// 2
        th(  "SHAMBIR");	// 2
        th("SH'AMB'IR");	// 2
        th("SH'AMB'IRA");	// 2
        th(     "shAN");	// 2
        th(     "SHAn");	// 2
        th(   "SH'ANA");	// 2
        th(   "SHANSI");	// 2
        th( "SH'A-NTI");	// 2
        th("SH'A-N+TI");	// 2
        th(  "SHANTIm");	// 2
        th("SH'ANTISA");	// 2
        th(   "SHAPDA");	// 2
        th(    "SH'AS");	// 2
        th(   "SHAshI");	// 2
        th(  "SH'ASTA");	// 2
        th(   "SH'ASU");	// 2
        th(  "SH'ATAM");	// 2
        th(  "shATA'O");	// 2
        th(    "SHDZI");	// 2
        th(    "SHESL");	// 2
        th(     "shHO");	// 2
        th(      "sHI");	// 2
        th(    "SHIGN");	// 2
        th(     "SHIm");	// 2
        th(  "SHINTAm");	// 2
        th(   "SHINTU");	// 2
        th("shINYTZA'I");	// 2
        th( "SHIRshAD");	// 2
        th(    "SHIsh");	// 2
        th(   "SHIshA");	// 2
        th(  "SHI'shA");	// 2
        th( "SHIshA'I");	// 2
        th(    "SHItA");	// 2
        th(  "SH+'KYA");	// 2
        th("SHLOOKA'I");	// 2
        th(   "SHODHA");	// 2
        th(  "SHODH'A");	// 2
        th(  "SHOLOKA");	// 2
        th(     "shPE");	// 2
        th(    "SHPTA");	// 2
        th(     "shRA");	// 2
        th("SHRA'DDHA");	// 2
        th("SHRADDHA'I");	// 2
        th( "SHRADDHE");	// 2
        th( "SHRANTHI");	// 2
        th(   "SHR'I:");	// 2
        th(    "shRYA");	// 2
        th(     "shsh");	// 2
        th(    "SHTHU");	// 2
        th(    "shtRA");	// 2
        th(   "SHTZ'A");	// 2
        th(    "SHTZI");	// 2
        th(    "SHTZU");	// 2
        th(     "SHUd");	// 2
        th(  "SHUDDHI");	// 2
        th("SHUDDHONYAHAm");	// 2
        th(   "SHUDHA");	// 2
        th(    "SHVA'");	// 2
        th( "SHVADTHA");	// 2
        th(   "SHVA'I");	// 2
        th(    "SHVAN");	// 2
        th(    "SHV'I");	// 2
        th(    "SH+YA");	// 2
        th(    "SHY'A");	// 2
        th(    "shYAm");	// 2
        th(     "SHYE");	// 2
        th(      "'SI");	// 2
        th(  "SIDDH'I");	// 2
        th(  "SIDDHIS");	// 2
        th(  "SIDDHYA");	// 2
        th(     "SIGS");	// 2
        th(     "S'IN");	// 2
        th(   "SINDHA");	// 2
        th(  "SINGGHA");	// 2
        th("SING+GHYA");	// 2
        th(    "SINGS");	// 2
        th(  "S'IshtA");	// 2
        th(     "SITA");	// 2
        th(    "SITVA");	// 2
        th(     "SK'A");	// 2
        th(  "SKANDHA");	// 2
        th( "SKANDHAS");	// 2
        th(    "SKE'I");	// 2
        th(    "SKO'I");	// 2
        th(   "SKRABS");	// 2
        th(     "SKRE");	// 2
        th(    "SKRIB");	// 2
        th(    "SKROL");	// 2
        th(    "SKROM");	// 2
        th(    "SKRUR");	// 2
        th(    "SKRUS");	// 2
        th(  "SKYA'AM");	// 2
        th(    "SKYEG");	// 2
        th(    "SKYIG");	// 2
        th(    "SKYIR");	// 2
        th(  "SKYO'AM");	// 2
        th(   "SKYOMS");	// 2
        th(     "SLEN");	// 2
        th(    "SLING");	// 2
        th(    "SLi'O");	// 2
        th(    "SLOON");	// 2
        th(    "SLUBS");	// 2
        th(     "SLUN");	// 2
        th(     "SMA:");	// 2
        th(    "SMARS");	// 2
        th(    "SMBHU");	// 2
        th(      "SMi");	// 2
        th(   "SMIN'O");	// 2
        th(   "SMINRA");	// 2
        th(     "SMIS");	// 2
        th(     "SMOG");	// 2
        th(    "SMRA'");	// 2
        th( "SMRA'ANG");	// 2
        th(    "SMRAL");	// 2
        th(    "SMRAM");	// 2
        th(    "SMREG");	// 2
        th(   "SMRYUm");	// 2
        th(    "SMUGS");	// 2
        th(     "SMUS");	// 2
        th(   "SMYAGS");	// 2
        th(    "SMYAM");	// 2
        th(     "SMYE");	// 2
        th(     "SMYI");	// 2
        th(    "SMYIN");	// 2
        th(   "SMYOMS");	// 2
        th(       "SN");	// 2
        th(  "SNA'ANG");	// 2
        th(    "SNAGN");	// 2
        th(    "SNAYD");	// 2
        th(    "SNAYM");	// 2
        th(   "SNGA'O");	// 2
        th(   "^SNGAR");	// 2
        th(   "SNGE'U");	// 2
        th(    "SNGIG");	// 2
        th(    "SNGUD");	// 2
        th(     "SNIN");	// 2
        th(    "SNING");	// 2
        th(     "SNOR");	// 2
        th(    "SNUNG");	// 2
        th(   "SNUNGS");	// 2
        th( "SNYAM'AM");	// 2
        th(   "SNYAN0");	// 2
        th(   "SNYIGN");	// 2
        th(   "SNYIMS");	// 2
        th(  "SNYINGS");	// 2
        th(    "SNYIt");	// 2
        th(  "SNYUNGS");	// 2
        th(     "S'OB");	// 2
        th(    "SO-dA");	// 2
        th(     "SOGs");	// 2
        th(      "SOm");	// 2
        th(     "'SOS");	// 2
        th(   "^SPANG");	// 2
        th(   "SPARGA");	// 2
        th(  "SPARSHA");	// 2
        th(    "SPEBS");	// 2
        th(   "SPENGS");	// 2
        th(     "SPES");	// 2
        th(    "SPHAR");	// 2
        th(     "SPIL");	// 2
        th(     "SPNG");	// 2
        th(    "SPO'I");	// 2
        th(    "SPOMS");	// 2
        th(   "SPRA'I");	// 2
        th(   "SPREBS");	// 2
        th(    "SPRES");	// 2
        th(    "SPRUD");	// 2
        th(   "SPRU'I");	// 2
        th(    "SPRUR");	// 2
        th(  "SPU'ANG");	// 2
        th(    "SPURL");	// 2
        th(    "SPYES");	// 2
        th( "SPYI'I'O");	// 2
        th(    "SPYOM");	// 2
        th(   "SPYO'O");	// 2
        th(     "SRAL");	// 2
        th(     "SRAR");	// 2
        th(    "SRE'U");	// 2
        th(     "SRGA");	// 2
        th(   "SRGYAS");	// 2
        th(    "^SRID");	// 2
        th(    "SRi'I");	// 2
        th(    "SRO'I");	// 2
        th(     "SRUR");	// 2
        th(   "SSARBA");	// 2
        th(      "SSO");	// 2
        th(     "STAL");	// 2
        th(   "STAMBA");	// 2
        th(    "STA'O");	// 2
        th(   "STE'AM");	// 2
        th(     "STEE");	// 2
        th(    "STEMS");	// 2
        th(     "STEt");	// 2
        th(    "STHAL");	// 2
        th(    "STHAS");	// 2
        th(     "STI:");	// 2
        th(    "STON0");	// 2
        th(     "STUL");	// 2
        th(     "STZE");	// 2
        th(      "SU'");	// 2
        th(     "SUGS");	// 2
        th( "S'UKshMA");	// 2
        th(     "SUMS");	// 2
        th(     "SU'U");	// 2
        th(     "SVA'");	// 2
        th(   "SVABHA");	// 2
        th(  "SV'ABHA");	// 2
        th(     "SVAm");	// 2
        th(     "SVAR");	// 2
        th(     "SVEE");	// 2
        th(     "SW'A");	// 2
        th(     "SWA'");	// 2
        th( "SW'A-H'A");	// 2
        th(     "SYA'");	// 2
        th(    "SY'AB");	// 2
        th(    "SY'AM");	// 2
        th(    "SYAND");	// 2
        th(   "SYANTA");	// 2
        th(   "SYANTE");	// 2
        th(   "SYANTI");	// 2
        th(    "SYA'O");	// 2
        th(    "SYING");	// 2
        th(      "SYU");	// 2
        th(    "TAdYA");	// 2
        th(    "'TAHD");	// 2
        th(     "t'AI");	// 2
        th(  "T'AKshA");	// 2
        th(     "t'Am");	// 2
        th(     "TAMS");	// 2
        th(      "TAn");	// 2
        th(    "TANTI");	// 2
        th(    "tA:'O");	// 2
        th(   "TA-PAL");	// 2
        th(   "TARKAL");	// 2
        th(   "TASM'A");	// 2
        th(    "TASRA");	// 2
        th(   "TATH'A");	// 2
        th(     "TATI");	// 2
        th( "tAttANDA");	// 2
        th(   "TATV'A");	// 2
        th(    "TATYA");	// 2
        th( "TATZSHMA");	// 2
        th(     "TBRA");	// 2
        th(      "tdA");	// 2
        th(     "TDYA");	// 2
        th(      "TE4");	// 2
        th(     "TEGS");	// 2
        th(     "tE'I");	// 2
        th(       "TH");	// 2
        th(     "^THA");	// 2
        th(   "^THAGS");	// 2
        th(   "thA-KA");	// 2
        th(    "THAMs");	// 2
        th(     "thAN");	// 2
        th(     "TH'E");	// 2
        th(     "THEE");	// 2
        th(  "THE'U'I");	// 2
        th(   "THIGAm");	// 2
        th(   "'THIGS");	// 2
        th(     "THKA");	// 2
        th(      "THL");	// 2
        th(    "'THOS");	// 2
        th(     "TH'U");	// 2
        th(   "THU'AM");	// 2
        th(       "Ti");	// 2
        th(      "TI'");	// 2
        th(   "TI'ANG");	// 2
        th(      "TIB");	// 2
        th(      "TIK");	// 2
        th(     "T'IK");	// 2
        th(   "t'IKAK");	// 2
        th(   "t'IK'I");	// 2
        th(     "tIKK");	// 2
        th(   "TINGKA");	// 2
        th(    "T'I'O");	// 2
        th(    "TIRYA");	// 2
        th(      "tIS");	// 2
        th(  "TIsht'A");	// 2
        th(     "tI'U");	// 2
        th(      "TKA");	// 2
        th(      "tKI");	// 2
        th(     "t'KI");	// 2
        th(    "t'KKA");	// 2
        th(      "TLi");	// 2
        th(      "TMA");	// 2
        th(     "TN'A");	// 2
        th(    "TNAMS");	// 2
        th(     "TOBS");	// 2
        th(      "tOM");	// 2
        th(   "tOMBHI");	// 2
        th(     "TOSM");	// 2
        th(     "TP'A");	// 2
        th(     "TP'U");	// 2
        th(       "TR");	// 2
        th(     "TRAN");	// 2
        th(    "TRE'O");	// 2
        th(    "TRE'U");	// 2
        th(      "tRI");	// 2
        th(     "TRIL");	// 2
        th(     "TRIM");	// 2
        th(    "TRING");	// 2
        th(     "TRUm");	// 2
        th(     "TSA'");	// 2
        th(     "TSA4");	// 2
        th(  "TSA'ANG");	// 2
        th(   "'TSALO");	// 2
        th(   "TSES21");	// 2
        th(    "TSE'U");	// 2
        th(    "TSGOS");	// 2
        th(     "TS'I");	// 2
        th(      "TSL");	// 2
        th( "'TSO'ANG");	// 2
        th(     "TSOm");	// 2
        th(    "TSTSA");	// 2
        th(     "'TSU");	// 2
        th(    "'TSUG");	// 2
        th(    "TSULO");	// 2
        th(  "TSUshMA");	// 2
        th(    "TSV'I");	// 2
        th(    "TTA'I");	// 2
        th(      "TTE");	// 2
        th(    "'TUNG");	// 2
        th(    "TUNTE");	// 2
        th(    "TVA'I");	// 2
        th(     "tVAm");	// 2
        th(    "TV'AR");	// 2
        th(      "TVE");	// 2
        th(     "TV'I");	// 2
        th(      "tYA");	// 2
        th(     "TYAm");	// 2
        th(     "TYEE");	// 2
        th(      "TYU");	// 2
        th(   "TZA'AM");	// 2
        th(   "TZ'ABA");	// 2
        th( "TZAKRA'I");	// 2
        th(    "'TZAM");	// 2
        th( "TZAN+DAN");	// 2
        th(  "TZANDHA");	// 2
        th("TZA-N+DRA");	// 2
        th("TZANDRA'AM");	// 2
        th(   "TZANtA");	// 2
        th( "TZANTI'O");	// 2
        th(    "TZ'AR");	// 2
        th(  "TZARTZI");	// 2
        th(  "TZAR-YA");	// 2
        th( "TZAR+YAS");	// 2
        th(    "TZDAN");	// 2
        th(     "TZEM");	// 2
        th(   "TZENGS");	// 2
        th(    "TZ''I");	// 2
        th(     "TZIL");	// 2
        th(    "TZING");	// 2
        th(  "TZINT'A");	// 2
        th(    "TZI'O");	// 2
        th(     "TZ'O");	// 2
        th(    "TZO'O");	// 2
        th(   "TZU'UN");	// 2
        th(     "TZYU");	// 2
        th(    "'UBAS");	// 2
        th(       "UM");	// 2
        th(     "UNGS");	// 2
        th(       "US");	// 2
        th(       "VA");	// 2
        th(    "WA'AM");	// 2
        th(     "W'AR");	// 2
        th(   "WARDHA");	// 2
        th(   "WARMMA");	// 2
        th(    "WARTU");	// 2
        th(      "WAt");	// 2
        th(     "WB'A");	// 2
        th( "WENDRA'I");	// 2
        th(      "WER");	// 2
        th(      "WOO");	// 2
        th(     "WU'I");	// 2
        th(      "WUN");	// 2
        th(      "WVA");	// 2
        th(     "Y8IN");	// 2
        th(      "YA4");	// 2
        th(   "YA'ANG");	// 2
        th(    "YADZA");	// 2
        th(   "YAKshI");	// 2
        th(    "YAMRA");	// 2
        th(    "'YANG");	// 2
        th(    "Y'ANG");	// 2
        th(  "YANTU'I");	// 2
        th(     "Y'AR");	// 2
        th(   "YASMEE");	// 2
        th(   "Y'ASTA");	// 2
        th(  "YASTH'A");	// 2
        th(      "YAt");	// 2
        th(      "YAT");	// 2
        th(   "Y'ATRA");	// 2
        th(       "YD");	// 2
        th(      "YDE");	// 2
        th(      "YE4");	// 2
        th(      "YEB");	// 2
        th(    "YEE'O");	// 2
        th(      "^YI");	// 2
        th(     "YI'A");	// 2
        th(     "YIBN");	// 2
        th(     "^YID");	// 2
        th(  "YINPA'I");	// 2
        th(   "YINPAR");	// 2
        th(    "YIshA");	// 2
        th(   "YMRYUm");	// 2
        th(     "YODE");	// 2
        th(     "YODO");	// 2
        th(     "YO'O");	// 2
        th(      "YUA");	// 2
        th(   "YUKTAS");	// 2
        th(  "YUKTASA");	// 2
        th(    "YUKTI");	// 2
        th(     "YYIN");	// 2
        th(     "ZAMS");	// 2
        th(   "ZE'ANG");	// 2
        th(   "ZHA'AM");	// 2
        th(   "ZHANGS");	// 2
        th(    "ZHASG");	// 2
        th(    "ZHE4S");	// 2
        th(     "ZHEB");	// 2
        th(     "ZHEE");	// 2
        th(    "^ZHES");	// 2
        th(  "ZHESBYA");	// 2
        th( "ZHESDANG");	// 2
        th( "ZHE-SOGS");	// 2
        th(     "ZHI0");	// 2
        th(    "'ZHIG");	// 2
        th(    "'ZHIN");	// 2
        th(   "ZHING0");	// 2
        th(     "ZHLA");	// 2
        th(    "ZHOMS");	// 2
        th(      "ZHS");	// 2
        th(   "ZHVANG");	// 2
        th(      "ZIB");	// 2
        th(      "ZID");	// 2
        th(     "ZLA'");	// 2
        th(    "ZLA11");	// 2
        th(   "ZLANGS");	// 2
        th(     "ZLUB");	// 2
        th(    "ZONGS");	// 2
        th(     "1000");	// 3
        th(      "106");	// 3
        th(      "107");	// 3
        th(      "109");	// 3
        th(      "118");	// 3
        th(      "123");	// 3
        th(      "125");	// 3
        th(      "126");	// 3
        th(     "12NA");	// 3
        th(   "146BA3");	// 3
        th(      "148");	// 3
        th(      "150");	// 3
        th(      "152");	// 3
        th(      "153");	// 3
        th(      "157");	// 3
        th(      "167");	// 3
        th(      "168");	// 3
        th(      "170");	// 3
        th(      "176");	// 3
        th(      "177");	// 3
        th(      "180");	// 3
        th(      "186");	// 3
        th(      "187");	// 3
        th(      "188");	// 3
        th(      "191");	// 3
        th(      "196");	// 3
        th(     "1992");	// 3
        th(     "1993");	// 3
        th(    "19NA2");	// 3
        th(      "205");	// 3
        th(      "206");	// 3
        th(      "209");	// 3
        th(     "2115");	// 3
        th(      "223");	// 3
        th(      "224");	// 3
        th(      "226");	// 3
        th(      "236");	// 3
        th(      "237");	// 3
        th(    "240N7");	// 3
        th(      "252");	// 3
        th(      "261");	// 3
        th(      "264");	// 3
        th(      "265");	// 3
        th(      "268");	// 3
        th(      "269");	// 3
        th(    "26NA6");	// 3
        th(      "270");	// 3
        th(      "275");	// 3
        th(      "277");	// 3
        th(      "299");	// 3
        th(      "303");	// 3
        th(      "305");	// 3
        th(      "323");	// 3
        th(      "326");	// 3
        th(      "329");	// 3
        th(      "331");	// 3
        th(      "334");	// 3
        th(      "340");	// 3
        th(      "342");	// 3
        th(      "347");	// 3
        th(      "349");	// 3
        th(      "352");	// 3
        th(      "353");	// 3
        th(      "356");	// 3
        th(      "362");	// 3
        th(      "363");	// 3
        th(      "373");	// 3
        th(      "376");	// 3
        th(      "383");	// 3
        th(      "386");	// 3
        th(      "387");	// 3
        th(      "3BA");	// 3
        th(      "403");	// 3
        th(      "408");	// 3
        th(      "409");	// 3
        th(      "411");	// 3
        th(      "412");	// 3
        th(      "413");	// 3
        th(      "418");	// 3
        th(      "422");	// 3
        th(      "432");	// 3
        th(      "436");	// 3
        th(      "437");	// 3
        th(      "438");	// 3
        th(      "440");	// 3
        th(      "441");	// 3
        th(      "442");	// 3
        th(      "443");	// 3
        th(      "445");	// 3
        th(      "446");	// 3
        th(      "447");	// 3
        th(      "452");	// 3
        th(      "458");	// 3
        th(      "463");	// 3
        th(      "467");	// 3
        th(      "469");	// 3
        th(    "495B1");	// 3
        th(      "500");	// 3
        th(      "523");	// 3
        th(      "547");	// 3
        th(     "70N3");	// 3
        th(     "7BCO");	// 3
        th(  "7BSKRAD");	// 3
        th(     "7DGE");	// 3
        th(     "7'DI");	// 3
        th(  "7G-YUNG");	// 3
        th(      "7MI");	// 3
        th(     "7NYE");	// 3
        th(      "7'O");	// 3
        th(       "99");	// 3
        th(   "AABHYA");	// 3
        th(    "AAGNA");	// 3
        th(  "AAGNI'I");	// 3
        th(    "A'A'I");	// 3
        th(     "AAIN");	// 3
        th(   "AAKshO");	// 3
        th(   "AAKY'A");	// 3
        th(     "AAMA");	// 3
        th( "AAMBASTA");	// 3
        th(    "AAMRA");	// 3
        th(    "AAMRi");	// 3
        th(     "A'AN");	// 3
        th(  "AANG+GA");	// 3
        th(    "AANTE");	// 3
        th(     "AANU");	// 3
        th(    "AARGA");	// 3
        th(   "AARGAm");	// 3
        th( "AARTHA'I");	// 3
        th(  "A'AR-YA");	// 3
        th("A'A:THUGS");	// 3
        th(     "AATI");	// 3
        th(     "AAWA");	// 3
        th(     "AE'E");	// 3
        th(   "AERKHE");	// 3
        th(    "AEWAm");	// 3
        th(     "AGNI");	// 3
        th(      "'AI");	// 3
        th(      "AIm");	// 3
        th(    "AINRA");	// 3
        th(     "AItA");	// 3
        th(    "A'ITA");	// 3
        th(   "AITA'O");	// 3
        th(      "AMA");	// 3
        th(   "AMBHAN");	// 3
        th(      "AMS");	// 3
        th(      "AON");	// 3
        th(    "A'ONG");	// 3
        th(    "AOO'I");	// 3
        th(     "APAR");	// 3
        th(   "ARGA'I");	// 3
        th(   "ARKA'I");	// 3
        th(    "AshtA");	// 3
        th(       "AT");	// 3
        th(  "AUDPALA");	// 3
        th(      "AUM");	// 3
        th(      "AUn");	// 3
        th(  "AUNG+GA");	// 3
        th(   "AUshnI");	// 3
        th(   "AUT+PA");	// 3
        th(   "AUTP'A");	// 3
        th(    "AUTZA");	// 3
        th(     "AU'U");	// 3
        th(      "AVA");	// 3
        th(   "BADZRO");	// 3
        th(   "BAKshU");	// 3
        th(    "BA'NG");	// 3
        th(     "BANI");	// 3
        th(      "BAO");	// 3
        th(   "BA'ONG");	// 3
        th(     "BASM");	// 3
        th(      "BBA");	// 3
        th(     "BCUL");	// 3
        th(    "BCUMS");	// 3
        th(    "^BDAG");	// 3
        th(     "BDED");	// 3
        th(     "BDEG");	// 3
        th(      "BDG");	// 3
        th(     "BDIN");	// 3
        th(     "BDIR");	// 3
        th(     "BDOM");	// 3
        th(    "BDREN");	// 3
        th( "BEd'URYA");	// 3
        th(   "BEED'U");	// 3
        th("BEED'URYA");	// 3
        th(     "BEGS");	// 3
        th(   "'BENGS");	// 3
        th(   "BGLANG");	// 3
        th(     "BGOL");	// 3
        th(     "BGRU");	// 3
        th(    "BGYES");	// 3
        th(    "BGYIB");	// 3
        th(    "BGYOD");	// 3
        th(    "BGYUD");	// 3
        th(   "BHAttA");	// 3
        th(     "BHEL");	// 3
        th(     "BHIL");	// 3
        th(    "BHO'I");	// 3
        th(    "BHRAm");	// 3
        th(    "BHRAM");	// 3
        th(   "BH'U'I");	// 3
        th(     "BHUS");	// 3
        th(      "BIG");	// 3
        th(  "BIGHN'A");	// 3
        th(    "BIHAR");	// 3
        th(   "BIKshA");	// 3
        th(   "BIM-PA");	// 3
        th(    "BIN'A");	// 3
        th(     "'BIR");	// 3
        th(   "BIR+YA");	// 3
        th(   "BIshtI");	// 3
        th("BIshYANTI");	// 3
        th(     "BK'A");	// 3
        th(    "BKLAS");	// 3
        th(   "BKLOGS");	// 3
        th(   "BKONGS");	// 3
        th(    "BKRAB");	// 3
        th(    "BKRAD");	// 3
        th(   "BKRANG");	// 3
        th(     "BKRE");	// 3
        th(   "BKRIGS");	// 3
        th(    "BKROD");	// 3
        th(    "BKRUL");	// 3
        th(     "BKYI");	// 3
        th(   "BKYIGS");	// 3
        th(    "BKYIS");	// 3
        th(     "BLAB");	// 3
        th(     "BLCI");	// 3
        th(    "BLDAB");	// 3
        th(    "BLDAD");	// 3
        th(   "BLUNGS");	// 3
        th(       "Bm");	// 3
        th(    "BNGOS");	// 3
        th(    "BNYIS");	// 3
        th(     "B''O");	// 3
        th(    "BPA'I");	// 3
        th(    "'BRAN");	// 3
        th(     "BRAR");	// 3
        th(   "BRATTI");	// 3
        th(    "BRDAM");	// 3
        th(     "BRDE");	// 3
        th(   "BRDEBS");	// 3
        th(    "BRDIB");	// 3
        th(   "BRDIBS");	// 3
        th(   "BRDUGS");	// 3
        th(   "BRDZAD");	// 3
        th(   "BRDZED");	// 3
        th(  "BRDZU'I");	// 3
        th(  "BRDZUNG");	// 3
        th(    "'BREM");	// 3
        th(    "'BREN");	// 3
        th(    "BRGOL");	// 3
        th(  "BRGYU'O");	// 3
        th(     "BRIG");	// 3
        th(    "BRJEB");	// 3
        th(   "BRJE'O");	// 3
        th(    "BRJIS");	// 3
        th(     "BRJU");	// 3
        th(    "BRKAG");	// 3
        th(    "BRKAL");	// 3
        th(  "BRLANGS");	// 3
        th(    "BRLOG");	// 3
        th(  "BRNGA'I");	// 3
        th(   "BRNGAL");	// 3
        th(   "BRNGAM");	// 3
        th(    "'BROD");	// 3
        th(     "BROM");	// 3
        th(    "BRO'O");	// 3
        th(     "'BRS");	// 3
        th(    "BRTAd");	// 3
        th(   "BRTAMS");	// 3
        th(   "BRTEGS");	// 3
        th(  "BRTZANG");	// 3
        th(   "BRTZEL");	// 3
        th(    "BRTZO");	// 3
        th(   "BRTZOG");	// 3
        th( "BRTZONGS");	// 3
        th(  "BRTZUGS");	// 3
        th(    "'BRUm");	// 3
        th(    "BSALB");	// 3
        th(    "BSDId");	// 3
        th(   "BSDONG");	// 3
        th(   "BSDUNG");	// 3
        th(     "BSEB");	// 3
        th(     "BSEN");	// 3
        th(   "BSENGS");	// 3
        th(   "BSGANG");	// 3
        th(   "BSGEGS");	// 3
        th(   "BSGOGS");	// 3
        th(   "BSGONG");	// 3
        th(   "BSGRAd");	// 3
        th( "BSGRANGS");	// 3
        th(  "BSGRING");	// 3
        th(  "BSGRUGS");	// 3
        th(    "BSGUB");	// 3
        th(   "BSHADO");	// 3
        th(   "BSHA'O");	// 3
        th(  "BSHONGS");	// 3
        th(    "BSHUR");	// 3
        th(    "BSKOB");	// 3
        th(   "BSKRAG");	// 3
        th(    "BSKUB");	// 3
        th(   "BSKU'I");	// 3
        th(   "BSKYOS");	// 3
        th(  "BSKYUGS");	// 3
        th(    "BSMRA");	// 3
        th(     "BSNA");	// 3
        th(    "BSNAR");	// 3
        th(   "BSNGAG");	// 3
        th(   "BSNGOG");	// 3
        th(    "BSNYE");	// 3
        th("BSNYENPAR");	// 3
        th( "BSNYINGS");	// 3
        th(    "BSNYO");	// 3
        th(   "BSNYUG");	// 3
        th(    "BSOMS");	// 3
        th(     "BSON");	// 3
        th(    "BSRAL");	// 3
        th(   "BSRIBS");	// 3
        th(     "BSRU");	// 3
        th(    "BSRUD");	// 3
        th(    "BSRUS");	// 3
        th(   "BSTAMS");	// 3
        th(   "BSTAN0");	// 3
        th(   "BSTENG");	// 3
        th(  "BSTINGS");	// 3
        th(    "BSTOR");	// 3
        th(    "BSTUS");	// 3
        th(  "BSTZOGS");	// 3
        th(     "BTA'");	// 3
        th(      "BTE");	// 3
        th(    "BTHAD");	// 3
        th(    "BTHUN");	// 3
        th(      "BTI");	// 3
        th(    "BTIBS");	// 3
        th(   "BTSAMS");	// 3
        th(     "BTUG");	// 3
        th(   "BTZA'O");	// 3
        th(    "BTZAR");	// 3
        th(   "BTZUNG");	// 3
        th(   "BUDDHI");	// 3
        th(   "BUDH'A");	// 3
        th(     "BUGS");	// 3
        th(     "'BUm");	// 3
        th(      "BVA");	// 3
        th(    "'BYEN");	// 3
        th(    "BYENG");	// 3
        th(   "'BYE'O");	// 3
        th(    "BYE'O");	// 3
        th(    "BYIBS");	// 3
        th(    "BYOGS");	// 3
        th(   "BYONGS");	// 3
        th(     "BYUN");	// 3
        th(  "'BYUNG:");	// 3
        th(    "BZALS");	// 3
        th(    "BZE'I");	// 3
        th(    "BZHAM");	// 3
        th(   "BZHE'I");	// 3
        th(    "BZHOL");	// 3
        th(    "BZLAG");	// 3
        th(    "BZLOB");	// 3
        th(     "BZOL");	// 3
        th(    "BZO'O");	// 3
        th(     "'CA'");	// 3
        th(    "CA'AM");	// 3
        th(    "'CANG");	// 3
        th(     "CA'O");	// 3
        th(     "CENG");	// 3
        th(     "CHA'");	// 3
        th(     "'CHD");	// 3
        th(   "CHENGS");	// 3
        th(    "'CHOB");	// 3
        th(    "CHUBS");	// 3
        th(    "CHUGN");	// 3
        th(     "CHVA");	// 3
        th(    "CI'AM");	// 3
        th(    "'CING");	// 3
        th(  "CORRUPT");	// 3
        th(      "CSE");	// 3
        th(     "D'AG");	// 3
        th(      "dAI");	// 3
        th(     "'DAm");	// 3
        th(     "DAM'");	// 3
        th("DAmshtR'A");	// 3
        th(   "DAnd'A");	// 3
        th(    "DANTE");	// 3
        th(     "DAR:");	// 3
        th(    "DA+YA");	// 3
        th(    "DBABS");	// 3
        th(    "DBAYR");	// 3
        th(    "DBENG");	// 3
        th(     "DBHA");	// 3
        th(    "DBONG");	// 3
        th(    "DBO'O");	// 3
        th(     "DBUt");	// 3
        th(   "DBYONG");	// 3
        th(    "DBYOR");	// 3
        th(    "DBYUR");	// 3
        th(     "D'EI");	// 3
        th(     "D'EN");	// 3
        th(       "DG");	// 3
        th(     "DGAD");	// 3
        th(    "DGA'S");	// 3
        th(  "DGE'ANG");	// 3
        th(  "DGE'DUN");	// 3
        th(     "DGIS");	// 3
        th(    "DGONS");	// 3
        th(      "DGR");	// 3
        th(    "DGRUB");	// 3
        th(    "DGYED");	// 3
        th(     "DGYI");	// 3
        th(    "DGYOR");	// 3
        th(      "DH'");	// 3
        th(     "dHAm");	// 3
        th( "DHA-RM'A");	// 3
        th(  "DHARMAS");	// 3
        th("DHIshth'A");	// 3
        th(  "DHIshtI");	// 3
        th(    "DHRiK");	// 3
        th(     "DHWA");	// 3
        th(  "d'IKKAR");	// 3
        th(      "DIN");	// 3
        th(     "'DIO");	// 3
        th(     "dITA");	// 3
        th(     "DKRA");	// 3
        th(    "DKRAS");	// 3
        th(     "DLAN");	// 3
        th(      "DMO");	// 3
        th(     "DMOS");	// 3
        th(     "DMUN");	// 3
        th(    "DNGAL");	// 3
        th(    "DNGOR");	// 3
        th(    "'DO'I");	// 3
        th(     "'DOt");	// 3
        th(    "DOVGS");	// 3
        th(  "DPA'ANG");	// 3
        th(   "DPHUNG");	// 3
        th(   "DPHYOD");	// 3
        th(     "DPUD");	// 3
        th(    "DPUGS");	// 3
        th(       "DR");	// 3
        th(    "'DRAG");	// 3
        th(    "'DRAM");	// 3
        th(   "'DREGS");	// 3
        th(    "DRE'I");	// 3
        th(    "DRIBS");	// 3
        th(     "DRIG");	// 3
        th(    "'DRIG");	// 3
        th(    "'DROL");	// 3
        th(    "'DRVA");	// 3
        th(  "'DU'ANG");	// 3
        th(  "DU:KHAS");	// 3
        th(    "DU'NG");	// 3
        th(    "D'URA");	// 3
        th(    "DURGA");	// 3
        th(  "d'URYAR");	// 3
        th(  "d'URY'I");	// 3
        th(      "DYI");	// 3
        th(    "'DZAB");	// 3
        th(    "DZA'O");	// 3
        th(   "DZASRA");	// 3
        th(     "DZIm");	// 3
        th(   "'DZIMS");	// 3
        th(  "DZ+NY'A");	// 3
        th(     "DZ'O");	// 3
        th(   "'DZOGS");	// 3
        th(    "DZRIm");	// 3
        th(    "DZV'A");	// 3
        th(      "GA-");	// 3
        th(   "GADH'A");	// 3
        th( "GADH'A'I");	// 3
        th(    "GANAG");	// 3
        th(    "GAndE");	// 3
        th( "GAndIA'I");	// 3
        th( "GAnd'I'O");	// 3
        th(  "GAndI'O");	// 3
        th(    "^GANG");	// 3
        th("GANG-GA'A'I");	// 3
        th( "GANGG'AR");	// 3
        th( "GANGGE'I");	// 3
        th(  "GANGG'I");	// 3
        th(     "G'AR");	// 3
        th(    "GARBA");	// 3
        th(    "GARGA");	// 3
        th(   "GA'U'I");	// 3
        th(   "GA-YOD");	// 3
        th(    "GAYOG");	// 3
        th(   "GA-YOL");	// 3
        th(   "GCE'US");	// 3
        th(     "GCHA");	// 3
        th(     "GDAd");	// 3
        th(    "GDAmS");	// 3
        th(     "GDES");	// 3
        th(     "GDID");	// 3
        th(     "GDIR");	// 3
        th(    "GDOMS");	// 3
        th(      "'GE");	// 3
        th(      "GEE");	// 3
        th(      "GEM");	// 3
        th(     "GEMS");	// 3
        th(     "GHAL");	// 3
        th(    "GHA'O");	// 3
        th(    "GHI'I");	// 3
        th(     "GH'O");	// 3
        th(     "GHOR");	// 3
        th(    "GHR'A");	// 3
        th(     "GHZI");	// 3
        th(       "Gi");	// 3
        th(     "'GIS");	// 3
        th(      "GLI");	// 3
        th(    "GLIGS");	// 3
        th(     "GLOB");	// 3
        th(    "GNA'I");	// 3
        th(    "GNAS:");	// 3
        th(     "GNUM");	// 3
        th(    "GNYAD");	// 3
        th(    "GNYAM");	// 3
        th(   "GNYA'O");	// 3
        th(    "GNYOM");	// 3
        th(    "GNYSI");	// 3
        th(   "GO'ANG");	// 3
        th(      "GPA");	// 3
        th(    "GRAMS");	// 3
        th(   "'GRAMS");	// 3
        th(   "'GRA'O");	// 3
        th(    "GRA'O");	// 3
        th(     "GRAW");	// 3
        th(    "GRAWA");	// 3
        th(   "'GRE'I");	// 3
        th(     "GR'i");	// 3
        th(    "GRI'O");	// 3
        th(    "'GRIS");	// 3
        th( "'GRO'ONG");	// 3
        th(    "GRTZO");	// 3
        th(     "GRUD");	// 3
        th(    "'GRUN");	// 3
        th(   "GRVA'O");	// 3
        th(     "GSA'");	// 3
        th(   "GSHENG");	// 3
        th(    "GSHON");	// 3
        th(      "GSI");	// 3
        th(     "GSIR");	// 3
        th(  "GSRUNGS");	// 3
        th(    "GSUBS");	// 3
        th(   "GSUNGs");	// 3
        th(     "GTAm");	// 3
        th(     "GTEG");	// 3
        th(    "GTIGS");	// 3
        th(   "GTSANG");	// 3
        th(     "GTSO");	// 3
        th(    "GTZAG");	// 3
        th(  "GTZANGS");	// 3
        th(   "GTZEGS");	// 3
        th(   "GTZOOR");	// 3
        th(    "GTZUN");	// 3
        th(   "GTZUNG");	// 3
        th(      "G'U");	// 3
        th(    "GUHYA");	// 3
        th(      "GUm");	// 3
        th(   "GUSNGS");	// 3
        th(    "'GYIL");	// 3
        th(  "G-YO'AM");	// 3
        th(  "G-YONGS");	// 3
        th(     "GYUB");	// 3
        th(   "'GYU'I");	// 3
        th(   "'GYUR:");	// 3
        th(    "GZHA'");	// 3
        th(   "GZHEGS");	// 3
        th(   "GZHE'I");	// 3
        th(    "GZHID");	// 3
        th(    "GZHUN");	// 3
        th(     "GZLA");	// 3
        th(    "GZO'O");	// 3
        th(     "GZOR");	// 3
        th(    "GZUGs");	// 3
        th(    "GZU'I");	// 3
        th(      "HA:");	// 3
        th(    "HANTI");	// 3
        th(      "HEL");	// 3
        th(     "HING");	// 3
        th(    "HRANG");	// 3
        th(     "HROB");	// 3
        th(    "H'U'I");	// 3
        th(    "H'Um:");	// 3
        th(     "H'UM");	// 3
        th(     "HU'U");	// 3
        th(     "HY'A");	// 3
        th(       "id");	// 3
        th(       "in");	// 3
        th(     "'ING");	// 3
        th(    "IV-03");	// 3
        th(      "JAN");	// 3
        th(     "'JID");	// 3
        th(     "KABS");	// 3
        th(      "KAH");	// 3
        th(      "KAK");	// 3
        th(     "'KAL");	// 3
        th(      "KAn");	// 3
        th(     "K'AN");	// 3
        th(   "KANGKA");	// 3
        th(     "KAsh");	// 3
        th(    "KAYNG");	// 3
        th(     "KE'E");	// 3
        th(    "'KHAM");	// 3
        th(   "^KHANG");	// 3
        th(    "KHA'O");	// 3
        th(    "KHI'I");	// 3
        th(   "KHRAGS");	// 3
        th(   "KHRA'O");	// 3
        th(    "KHRAR");	// 3
        th(    "KHRED");	// 3
        th(   "'KHREG");	// 3
        th(   "'KHRIB");	// 3
        th(   "'KHRIR");	// 3
        th( "'KHRU'AM");	// 3
        th(     "KH'U");	// 3
        th(   "'KHUGS");	// 3
        th(   "'KHU'I");	// 3
        th(   "'KHYAD");	// 3
        th(   "'KHYED");	// 3
        th(   "KHYEGS");	// 3
        th(  "KHYI'US");	// 3
        th(   "'KHYUR");	// 3
        th(    "KI'AM");	// 3
        th(  "K'IRT'I");	// 3
        th(  "KIRTI'I");	// 3
        th(  "KIRTISA");	// 3
        th(    "K'ISA");	// 3
        th(     "KKAR");	// 3
        th(     "KKYI");	// 3
        th(     "KLEG");	// 3
        th(     "KLES");	// 3
        th(    "KLING");	// 3
        th(      "KOL");	// 3
        th(    "KOndI");	// 3
        th(     "KRID");	// 3
        th(    "KR'IN");	// 3
        th("KRishnA'I");	// 3
        th(     "KRIT");	// 3
        th(     "KRUB");	// 3
        th(     "KRUG");	// 3
        th(    "KRUGS");	// 3
        th(     "KRUN");	// 3
        th(     "KsHA");	// 3
        th(   "KsHA'I");	// 3
        th( "Ksh'ANTI");	// 3
        th(     "KSHI");	// 3
        th(     "KTAm");	// 3
        th(      "KtI");	// 3
        th(    "KUNdA");	// 3
        th(  "KUNYDZI");	// 3
        th(     "KYI'");	// 3
        th(    "KYOGS");	// 3
        th(     "KYOS");	// 3
        th(    "KYO'U");	// 3
        th(      "LAd");	// 3
        th(      "LAK");	// 3
        th(  "LAKshMI");	// 3
        th(    "LAshA");	// 3
        th(   "LBANGS");	// 3
        th(     "LCAR");	// 3
        th(     "LDOB");	// 3
        th(    "LDUMS");	// 3
        th(     "LDUR");	// 3
        th(     "LE'O");	// 3
        th(       "LG");	// 3
        th(      "LGA");	// 3
        th(      "LGO");	// 3
        th(      "LHI");	// 3
        th(   "LHINGS");	// 3
        th(     "LHON");	// 3
        th(      "LID");	// 3
        th(     "LINE");	// 3
        th(    "L'ING");	// 3
        th(     "LI'U");	// 3
        th(    "LJING");	// 3
        th(     "LJOG");	// 3
        th(       "Lm");	// 3
        th(    "LNGAD");	// 3
        th(     "LNGO");	// 3
        th( "LO-TZTSA");	// 3
        th(      "LPU");	// 3
        th(      "LSA");	// 3
        th(      "LTR");	// 3
        th(     "LTUB");	// 3
        th(     "LTUN");	// 3
        th(  "LUMBA'I");	// 3
        th(     "LY'A");	// 3
        th(     "LYAN");	// 3
        th(    "MA'Am");	// 3
        th(      "MAB");	// 3
        th(    "M'A'I");	// 3
        th(     "MAnd");	// 3
        th(    "MANdA");	// 3
        th(  "MAN-DA'");	// 3
        th(   "MANDRA");	// 3
        th( "MANTRA'I");	// 3
        th(    "MARDA");	// 3
        th(   "MARTHA");	// 3
        th("MA:-SARBA");	// 3
        th(     "MA'U");	// 3
        th(     "MBHA");	// 3
        th(     "MBHI");	// 3
        th(      "MBI");	// 3
        th(     "MB'I");	// 3
        th(     "MCAN");	// 3
        th(    "MCHAR");	// 3
        th(   "MCHIBS");	// 3
        th(  "MCH'I'O");	// 3
        th(   "MCHO'I");	// 3
        th(   "MCHUGS");	// 3
        th(     "MDAD");	// 3
        th(     "MDAN");	// 3
        th(     "MDEN");	// 3
        th(      "MDI");	// 3
        th(    "MDOGS");	// 3
        th(    "'MDOR");	// 3
        th(    "MDROS");	// 3
        th(   "MDZANG");	// 3
        th(    "MDZIN");	// 3
        th(    "MENGS");	// 3
        th(      "MGE");	// 3
        th(    "MGYOG");	// 3
        th(     "MHA'");	// 3
        th(      "MIO");	// 3
        th(    "MI'OS");	// 3
        th(      "MJE");	// 3
        th(      "MJO");	// 3
        th(     "MKAN");	// 3
        th(   "MKHA'A");	// 3
        th(   "MKHRAD");	// 3
        th(    "MKHYE");	// 3
        th(   "MKHYIM");	// 3
        th(   "MKHYOD");	// 3
        th(    "MNGGA");	// 3
        th(   "MNGO'I");	// 3
        th(   "MNGONG");	// 3
        th(   "MNGONS");	// 3
        th(    "MNO'I");	// 3
        th(    "MNYAG");	// 3
        th(  "MNY-DZU");	// 3
        th(  "MNY+DZU");	// 3
        th(    "MNYIS");	// 3
        th(    "MNYUL");	// 3
        th(   "MRDZES");	// 3
        th(    "MRGAN");	// 3
        th(     "MSAM");	// 3
        th(     "MSAN");	// 3
        th(    "MSHAR");	// 3
        th(   "MSKYES");	// 3
        th(    "MTAMS");	// 3
        th(    "MTHAB");	// 3
        th( "MTHA'I'O");	// 3
        th(   "MTHAMS");	// 3
        th(    "MTHEN");	// 3
        th(    "MTHNG");	// 3
        th(   "MTHOGN");	// 3
        th( "MTHU'ANG");	// 3
        th(     "MTOG");	// 3
        th(   "MTSA'I");	// 3
        th(   "MTSAmS");	// 3
        th(   "^MTSAN");	// 3
        th(   "MTSAN0");	// 3
        th(    "MTSAS");	// 3
        th(   "MTSE'I");	// 3
        th(   "MTSEMS");	// 3
        th( "MTSE'U'I");	// 3
        th(  "MTSE'US");	// 3
        th("MTSOt'IKA");	// 3
        th(     "MTSU");	// 3
        th(    "MTZAM");	// 3
        th(    "MTZAN");	// 3
        th(    "MTZUN");	// 3
        th(  "MUNYDZA");	// 3
        th(     "MU'O");	// 3
        th(     "MYAM");	// 3
        th(     "MYAR");	// 3
        th(     "MYED");	// 3
        th(      "nA'");	// 3
        th(      "^NA");	// 3
        th(      "NA3");	// 3
        th(     "NA'A");	// 3
        th(     "N'AM");	// 3
        th(     "NAND");	// 3
        th(  "NANDASA");	// 3
        th(    "N'ANG");	// 3
        th(      "NAt");	// 3
        th(    "NDA'I");	// 3
        th(     "NDAN");	// 3
        th(     "NDRA");	// 3
        th(     "NENG");	// 3
        th(      "'NG");	// 3
        th(     "'NGA");	// 3
        th(     "NGEN");	// 3
        th(     "NGHI");	// 3
        th(     "NGUN");	// 3
        th(      "'NI");	// 3
        th(     "N''I");	// 3
        th(      "NI:");	// 3
        th(      "NIL");	// 3
        th(    "N'I'O");	// 3
        th(   "NIRY'A");	// 3
        th(     "NO'O");	// 3
        th(     "NOOR");	// 3
        th(      "NOT");	// 3
        th(      "NRI");	// 3
        th(       "nU");	// 3
        th(  "NYA'I'O");	// 3
        th(     "n+YE");	// 3
        th(    "n+YEE");	// 3
        th(     "NYEG");	// 3
        th(     "NYEL");	// 3
        th(     "NYUS");	// 3
        th(      "OGS");	// 3
        th(     "OLIO");	// 3
        th(      "'OO");	// 3
        th(    "OPA'I");	// 3
        th("PA'IPHYIR");	// 3
        th(      "PAJ");	// 3
        th(     "PALA");	// 3
        th(     "PA'M");	// 3
        th(    "PANDE");	// 3
        th(  "PAndI'I");	// 3
        th(     "PANI");	// 3
        th(    "PA'O:");	// 3
        th(      "PAP");	// 3
        th(     "^PAS");	// 3
        th(     "PAS:");	// 3
        th(      "PAt");	// 3
        th(      "PBA");	// 3
        th(      "PDE");	// 3
        th(      "PES");	// 3
        th(     "^PHA");	// 3
        th(     "PHA'");	// 3
        th(    "'PHAS");	// 3
        th(    "PHEEm");	// 3
        th(    "PHEGS");	// 3
        th(    "PHE'I");	// 3
        th(    "'PHER");	// 3
        th(    "PHING");	// 3
        th( "'PHO'ANG");	// 3
        th(    "'PHOL");	// 3
        th(   "'PHREG");	// 3
        th( "'PHRENGS");	// 3
        th(   "'PHRER");	// 3
        th(    "PHRID");	// 3
        th(   "PHRIMS");	// 3
        th(  "'PHRO'O");	// 3
        th(    "PHRUm");	// 3
        th(  "PHYE'AM");	// 3
        th(   "PHYIGS");	// 3
        th(     "PHYO");	// 3
        th(      "PLA");	// 3
        th(     "PndI");	// 3
        th(    "PNGAM");	// 3
        th(    "PNGMO");	// 3
        th(      "PO'");	// 3
        th(    "PONGS");	// 3
        th(   "PPHYIR");	// 3
        th("PRADZNYA'I");	// 3
        th( "PRDZNY'A");	// 3
        th(    "'PREL");	// 3
        th(     "PR'i");	// 3
        th(   "PRiTTA");	// 3
        th(      "PTE");	// 3
        th(      "PTI");	// 3
        th(      "PUM");	// 3
        th(      "PUN");	// 3
        th(    "PURNA");	// 3
        th(    "PYUNG");	// 3
        th(      "RAd");	// 3
        th(     "R'AD");	// 3
        th(     "RAKA");	// 3
        th( "RAKSHA'I");	// 3
        th(  "RAKTA'I");	// 3
        th(   "RAND'A");	// 3
        th(    "RANMS");	// 3
        th(   "RATNAS");	// 3
        th(     "RBAL");	// 3
        th(     "RBAN");	// 3
        th(     "RDIG");	// 3
        th(    "RDZA'");	// 3
        th(   "RDZEGS");	// 3
        th(   "RDZENG");	// 3
        th(    "'RDZI");	// 3
        th(   "'RDZIN");	// 3
        th(   "RDZUBS");	// 3
        th(    "RDZUD");	// 3
        th(      "R'E");	// 3
        th(     "RGAM");	// 3
        th(      "RGI");	// 3
        th(   "RGO'AM");	// 3
        th(     "RGUB");	// 3
        th(   "RGYONG");	// 3
        th(      "RI'");	// 3
        th(     "RiGS");	// 3
        th(     "Ri'I");	// 3
        th(     "^RIN");	// 3
        th(     "RJAB");	// 3
        th(     "RJAD");	// 3
        th(    "'RJOD");	// 3
        th(   "RJONGS");	// 3
        th(     "RKER");	// 3
        th(    "RKUNG");	// 3
        th(    "RKYAS");	// 3
        th(    "RKYIN");	// 3
        th(    "RKYOD");	// 3
        th(     "RLOS");	// 3
        th(  "RMA'ANG");	// 3
        th(     "RMID");	// 3
        th(   "RMILAM");	// 3
        th(    "RMO'O");	// 3
        th(   "RMYANG");	// 3
        th(  "RMYANGS");	// 3
        th(  "RNAM-SU");	// 3
        th(    "RNGEN");	// 3
        th(     "RNOD");	// 3
        th(    "RNO'I");	// 3
        th(     "RNYA");	// 3
        th(    "RO'AM");	// 3
        th( "RONYKshA");	// 3
        th(    "RTA'U");	// 3
        th(    "RTE'I");	// 3
        th(    "RTGOS");	// 3
        th(    "RTI'O");	// 3
        th(     "RTNA");	// 3
        th(    "RTONG");	// 3
        th(    "RTZAm");	// 3
        th(   "RTZA'O");	// 3
        th(  "RTZE'AM");	// 3
        th(    "RTZEB");	// 3
        th(    "RTZ'I");	// 3
        th(  "RTZI'AM");	// 3
        th(    "RTZUG");	// 3
        th(    "RVAGS");	// 3
        th(     "SABA");	// 3
        th(  "SADZDZA");	// 3
        th(   "S'AH'A");	// 3
        th(     "SAKD");	// 3
        th(     "S'AM");	// 3
        th(    "SAMYA");	// 3
        th("SAMYAKsAm");	// 3
        th(    "SANTA");	// 3
        th(     "S'AR");	// 3
        th(   "SA-RBA");	// 3
        th(   "SAR:RA");	// 3
        th(    "SARVA");	// 3
        th(      "SAT");	// 3
        th(     "SBAB");	// 3
        th(    "SBABS");	// 3
        th(    "SBA'O");	// 3
        th(    "SBAYR");	// 3
        th(     "SBEM");	// 3
        th(   "SBENGS");	// 3
        th(     "SBOB");	// 3
        th(     "SBOG");	// 3
        th(     "SBON");	// 3
        th(   "SBRE'U");	// 3
        th(     "SBUM");	// 3
        th(   "SBYIBS");	// 3
        th(   "SBYI'O");	// 3
        th(    "SBYNG");	// 3
        th(     "SBYU");	// 3
        th(   "SBYUNG");	// 3
        th(     "SDZA");	// 3
        th(     "SEMD");	// 3
        th(     "SGEM");	// 3
        th(    "SGENG");	// 3
        th(     "SGES");	// 3
        th(  "SGRANGS");	// 3
        th(    "SGR'O");	// 3
        th(   "'SGRUB");	// 3
        th(    "SGRUD");	// 3
        th(    "SGRUM");	// 3
        th(    "SGRUR");	// 3
        th(     "SGTA");	// 3
        th(    "SGURB");	// 3
        th(     "SGYA");	// 3
        th(  "SGYINGS");	// 3
        th(    "SGYIS");	// 3
        th(   "SGZHAN");	// 3
        th(    "SH'AG");	// 3
        th( "SHA'-KYA");	// 3
        th("^SH'AKYA'I");	// 3
        th("SH'ANTI'I");	// 3
        th("sh'ANYTZA");	// 3
        th(  "SHARMAS");	// 3
        th(   "SHAshA");	// 3
        th(  "SHASTRA");	// 3
        th(    "SHA'U");	// 3
        th(    "shI'I");	// 3
        th(  "SHIRshA");	// 3
        th(    "SHKNU");	// 3
        th(  "SH'KYAS");	// 3
        th(    "SH+LO");	// 3
        th(   "SHLOKA");	// 3
        th(    "shn'I");	// 3
        th(  "SHONGBA");	// 3
        th(  "SHOO+LO");	// 3
        th(   "shOTTA");	// 3
        th(  "SHPDA'I");	// 3
        th(    "SHRI'");	// 3
        th(  "shthA'I");	// 3
        th(    "shthI");	// 3
        th(    "sht'I");	// 3
        th(    "shtIm");	// 3
        th("SHUDDH'A:");	// 3
        th( "SH'UN+YA");	// 3
        th(    "SHU'U");	// 3
        th("SHVANGTHA");	// 3
        th("SHVATTHA'I");	// 3
        th(     "SHWA");	// 3
        th(     "sHYA");	// 3
        th(    "SHYAm");	// 3
        th(  "SIDHA'I");	// 3
        th(   "SIDHAm");	// 3
        th(     "SIMS");	// 3
        th(    "S'INA");	// 3
        th( "SINDHU'I");	// 3
        th(   "SKABSU");	// 3
        th(    "SKAGS");	// 3
        th(    "SKOGS");	// 3
        th(    "SKRAL");	// 3
        th(   "SKROGS");	// 3
        th(    "SKRON");	// 3
        th(     "SKRU");	// 3
        th(   "SKUNGS");	// 3
        th(   "SKYAGS");	// 3
        th(    "SKYIB");	// 3
        th(    "SKYUD");	// 3
        th(    "SLAGS");	// 3
        th(  "SLAKshA");	// 3
        th(     "SLAL");	// 3
        th(    "SLOGS");	// 3
        th(    "SLU'O");	// 3
        th(     "SMAL");	// 3
        th(    "SMA'O");	// 3
        th(   "SMEE'I");	// 3
        th(    "SMING");	// 3
        th(    "SMRAB");	// 3
        th(    "SMREL");	// 3
        th(    "SMRIN");	// 3
        th(    "SMRON");	// 3
        th(      "SMU");	// 3
        th(  "SMYANGS");	// 3
        th(    "SMYAS");	// 3
        th(  "SMYO'AM");	// 3
        th(   "SMYONG");	// 3
        th(    "SMYUD");	// 3
        th(      "SNG");	// 3
        th(    "SNGEL");	// 3
        th(   "SNGOBS");	// 3
        th(    "SNGOG");	// 3
        th(     "SNOG");	// 3
        th(    "SNYAL");	// 3
        th(    "SNYAS");	// 3
        th(    "SNYIG");	// 3
        th(    "SNYIR");	// 3
        th(     "SNYO");	// 3
        th(    "SNYOB");	// 3
        th(   "SNYOSM");	// 3
        th(      "S'O");	// 3
        th(     "SOSG");	// 3
        th(    "SPA'O");	// 3
        th(     "SPHO");	// 3
        th(   "SPNAGS");	// 3
        th(    "SPO'O");	// 3
        th(   "SPRUNG");	// 3
        th(     "SPUM");	// 3
        th(    "SPU'O");	// 3
        th(     "SPYE");	// 3
        th(     "SR'A");	// 3
        th(     "SRAm");	// 3
        th(    "SRIGS");	// 3
        th(    "^SROG");	// 3
        th(     "SRUD");	// 3
        th(  "SSARBBA");	// 3
        th(     "STAm");	// 3
        th(  "STE'U'I");	// 3
        th(    "S+THI");	// 3
        th(     "STRi");	// 3
        th(    "STUMS");	// 3
        th(    "STZUS");	// 3
        th(   "SUNDHA");	// 3
        th(  "SVABH'A");	// 3
        th(     "SVAN");	// 3
        th(    "SV'OO");	// 3
        th(     "SYAD");	// 3
        th(     "SYAM");	// 3
        th(     "SYEE");	// 3
        th(    "TA'AM");	// 3
        th(   "TA'ANG");	// 3
        th(   "t'AKKI");	// 3
        th(    "TAMBU");	// 3
        th(   "TANTRA");	// 3
        th(      "^TE");	// 3
        th(      "TE:");	// 3
        th(      "TE'");	// 3
        th(      "TED");	// 3
        th(     "tEKA");	// 3
        th(    "TE''O");	// 3
        th(    "THAG:");	// 3
        th(    "'THAN");	// 3
        th(    "THASM");	// 3
        th(    "THBAS");	// 3
        th(     "THBS");	// 3
        th(    "'THIL");	// 3
        th(     "THIN");	// 3
        th(     "THIR");	// 3
        th(     "THYA");	// 3
        th(    "t'IGA");	// 3
        th(     "tiKA");	// 3
        th(     "TIKA");	// 3
        th(      "tIR");	// 3
        th("TIshthANTU");	// 3
        th(    "TISMA");	// 3
        th(     "t'KA");	// 3
        th(      "TOD");	// 3
        th(      "TOM");	// 3
        th(     "TO'O");	// 3
        th(    "TRAKA");	// 3
        th(     "TRIm");	// 3
        th(     "TRIN");	// 3
        th(    "TRUNG");	// 3
        th(    "'TSA'");	// 3
        th(   "TSA'AM");	// 3
        th(    "'TSAN");	// 3
        th( "TSATTRAm");	// 3
        th( "'TSE'ANG");	// 3
        th(    "'TSEB");	// 3
        th(    "TSIBS");	// 3
        th(    "'TSIL");	// 3
        th(   "'TSING");	// 3
        th(   "TSINGS");	// 3
        th(    "TSOBS");	// 3
        th(    "TSO'O");	// 3
        th(    "TSUBS");	// 3
        th(   "'TSUGS");	// 3
        th(     "TVAS");	// 3
        th(      "tVI");	// 3
        th(     "TYAT");	// 3
        th(  "TZAKR'A");	// 3
        th( "TZAKRE'O");	// 3
        th(    "'TZAL");	// 3
        th( "TZA-NDAN");	// 3
        th( "TZANDANA");	// 3
        th(   "TZANTA");	// 3
        th( "TZ'AR+YA");	// 3
        th("TZAR+YA'I");	// 3
        th( "TZ'ARYAS");	// 3
        th( "TZARYASA");	// 3
        th(  "TZA-TSA");	// 3
        th(    "TZITA");	// 3
        th(   "TZITSA");	// 3
        th( "TZITTA'I");	// 3
        th(   "TZNADN");	// 3
        th(   "TZO'AM");	// 3
        th(   "TZ+TSA");	// 3
        th(   "TZTS'A");	// 3
        th(     "WA'm");	// 3
        th(   "WASTRA");	// 3
        th(   "YASTHA");	// 3
        th(     "YIBS");	// 3
        th(     "YI'I");	// 3
        th(    "YINGS");	// 3
        th(    "YINNO");	// 3
        th(      "YO:");	// 3
        th(     "YOND");	// 3
        th( "YONGS-SU");	// 3
        th(  "YONG-SU");	// 3
        th(   "Y'TA'O");	// 3
        th(      "YU:");	// 3
        th(    "YURDA");	// 3
        th(   "YUshMA");	// 3
        th(        "Z");	// 3
        th(      "Z'A");	// 3
        th(      "ZA'");	// 3
        th(    "ZHA'O");	// 3
        th(     "ZHEM");	// 3
        th(   "ZHENGS");	// 3
        th(     "ZH'I");	// 3
        th(   "ZHO'AM");	// 3
        th(     "ZHUD");	// 3
        th(     "ZHUL");	// 3
        th(     "ZHWA");	// 3
        th(     "ZHWS");	// 3
        th(   "ZI'ANG");	// 3
        th(     "ZI'U");	// 3
        th(  "ZLA'ANG");	// 3
        th(     "ZU'I");	// 3
        th(     "ZUMS");	// 3
        th("100000000");	// 4
        th(      "103");	// 4
        th(      "104");	// 4
        th(      "108");	// 4
        th(      "121");	// 4
        th(      "122");	// 4
        th(      "131");	// 4
        th(      "136");	// 4
        th(      "137");	// 4
        th(      "139");	// 4
        th(     "13NA");	// 4
        th(      "143");	// 4
        th(      "145");	// 4
        th(      "151");	// 4
        th(      "159");	// 4
        th(      "173");	// 4
        th(      "189");	// 4
        th(      "194");	// 4
        th(     "1994");	// 4
        th(      "207");	// 4
        th(      "208");	// 4
        th(     "2096");	// 4
        th(      "211");	// 4
        th(      "213");	// 4
        th(      "215");	// 4
        th(      "234");	// 4
        th(      "235");	// 4
        th(      "246");	// 4
        th(      "248");	// 4
        th(      "253");	// 4
        th(      "260");	// 4
        th(      "262");	// 4
        th(    "262B1");	// 4
        th(      "276");	// 4
        th(      "285");	// 4
        th(      "289");	// 4
        th(      "297");	// 4
        th(     "29N2");	// 4
        th(      "307");	// 4
        th(      "313");	// 4
        th(      "320");	// 4
        th(      "335");	// 4
        th(      "354");	// 4
        th(      "358");	// 4
        th(      "361");	// 4
        th(      "364");	// 4
        th(      "380");	// 4
        th(      "384");	// 4
        th(      "389");	// 4
        th(      "397");	// 4
        th(      "402");	// 4
        th(      "405");	// 4
        th(      "423");	// 4
        th(      "428");	// 4
        th(      "430");	// 4
        th(      "433");	// 4
        th(      "448");	// 4
        th(      "454");	// 4
        th(      "462");	// 4
        th(      "470");	// 4
        th(      "476");	// 4
        th(      "478");	// 4
        th(      "482");	// 4
        th(      "490");	// 4
        th(      "503");	// 4
        th(    "7BKA'");	// 4
        th(     "7BLA");	// 4
        th(    "7BRAM");	// 4
        th(    "7GSUM");	// 4
        th(     "7MIG");	// 4
        th(     "7POD");	// 4
        th("A'A:HR'I:PRA");	// 4
        th(      "AAI");	// 4
        th(   "AANGKI");	// 4
        th(   "AARDZA");	// 4
        th(     "AA'U");	// 4
        th(     "ADYA");	// 4
        th(   "AEMCHI");	// 4
        th(     "'AGS");	// 4
        th(     "AI'A");	// 4
        th(  "A'IKshA");	// 4
        th( "AINDRA'I");	// 4
        th(      "AOD");	// 4
        th(      "AOG");	// 4
        th(   "'AONGS");	// 4
        th(    "ARGAm");	// 4
        th(    "ARTHA");	// 4
        th(     "AUNG");	// 4
        th(     "AYIN");	// 4
        th(  "BA-DZRA");	// 4
        th(    "BANDI");	// 4
        th(    "B'ANG");	// 4
        th(    "BANTU");	// 4
        th(    "BARMA");	// 4
        th(     "BAYS");	// 4
        th(     "BCED");	// 4
        th(     "BCHU");	// 4
        th(     "BCOG");	// 4
        th(   "BCONGS");	// 4
        th(  "BCU'ANG");	// 4
        th(     "BDEM");	// 4
        th(    "BDRUG");	// 4
        th(    "BDZUN");	// 4
        th( "BEEd'U'I");	// 4
        th(      "BGE");	// 4
        th(     "BGRA");	// 4
        th(     "BGRE");	// 4
        th(      "BGU");	// 4
        th(     "BGUR");	// 4
        th(    "BGYIN");	// 4
        th(     "BHA:");	// 4
        th(  "BHAKshA");	// 4
        th( "BHANYDZA");	// 4
        th(   "BHAtdA");	// 4
        th(    "BHENG");	// 4
        th(     "BHLA");	// 4
        th(     "BHON");	// 4
        th(     "BHOR");	// 4
        th(  "BHRiNYA");	// 4
        th(    "BHYA'");	// 4
        th(   "BHY'AS");	// 4
        th(      "BIm");	// 4
        th(    "BIMPA");	// 4
        th(    "BIRVA");	// 4
        th(     "BI'U");	// 4
        th(      "BKA");	// 4
        th(     "BKOS");	// 4
        th(    "BKYER");	// 4
        th(     "BLAM");	// 4
        th(    "BLDOG");	// 4
        th(    "BLING");	// 4
        th(    "BLNGA");	// 4
        th(   "BLNGAG");	// 4
        th( "BLTA'ANG");	// 4
        th(   "BLTUNG");	// 4
        th(    "BLU'I");	// 4
        th(     "BNAD");	// 4
        th(      "BPA");	// 4
        th(    "BRDAN");	// 4
        th(    "BRDOG");	// 4
        th(    "BRDUB");	// 4
        th(    "BRDZA");	// 4
        th(   "BRDZID");	// 4
        th(   "BRDZON");	// 4
        th(  "BRDZONG");	// 4
        th( "BRDZONGS");	// 4
        th(     "BREd");	// 4
        th(   "'BREGS");	// 4
        th(    "BRGOD");	// 4
        th(  "BRGYU'I");	// 4
        th(    "'BRIL");	// 4
        th(     "BRJI");	// 4
        th(     "BRJO");	// 4
        th(   "BRJONG");	// 4
        th(    "BRJOS");	// 4
        th(   "BRKANG");	// 4
        th(  "BRKU'AM");	// 4
        th(    "BRKUN");	// 4
        th(   "BRKYEN");	// 4
        th(    "BRLOM");	// 4
        th(    "BRNGO");	// 4
        th(   "BRNYIL");	// 4
        th(   "'BRO'I");	// 4
        th(     "BRON");	// 4
        th(    "BRTAB");	// 4
        th(   "BRTZED");	// 4
        th(   "BRTZOS");	// 4
        th(    "BSAmS");	// 4
        th(     "BSBU");	// 4
        th(     "BSDA");	// 4
        th(    "BSDAN");	// 4
        th(   "BSDEGS");	// 4
        th(   "BSDO'I");	// 4
        th(     "BSGA");	// 4
        th(    "BSGOB");	// 4
        th(   "BSGRAB");	// 4
        th(   "BSGRAS");	// 4
        th(    "BSGRI");	// 4
        th(  "BSGROGS");	// 4
        th(  "BSGROMS");	// 4
        th(   "BSGROS");	// 4
        th(   "BSGRUR");	// 4
        th(  "BSGYUNG");	// 4
        th(    "BSHEG");	// 4
        th(    "BSHOM");	// 4
        th(    "BSHUD");	// 4
        th(  "BSHUNGS");	// 4
        th(    "BSING");	// 4
        th(    "BSKA'");	// 4
        th(   "BSKO'I");	// 4
        th(  "BSKONGS");	// 4
        th(  "BSKRAGS");	// 4
        th(   "BSKRAL");	// 4
        th(   "BSKRUL");	// 4
        th(   "BSKRUS");	// 4
        th(   "BSKYOB");	// 4
        th(   "BSKYOL");	// 4
        th(  "BSLONGS");	// 4
        th(   "BSLU'O");	// 4
        th(    "BSMUN");	// 4
        th(   "BSNABS");	// 4
        th(   "BSNGAN");	// 4
        th(   "BSNYUR");	// 4
        th(     "BSOL");	// 4
        th(    "BSREB");	// 4
        th(    "BSRED");	// 4
        th(    "BSRIL");	// 4
        th(    "BSROD");	// 4
        th(    "BSTAM");	// 4
        th(    "BSTOB");	// 4
        th(    "BSU'O");	// 4
        th(    "BTHOB");	// 4
        th(     "BTID");	// 4
        th(    "BTSUN");	// 4
        th(  "BTZANGS");	// 4
        th(   "BTZEGS");	// 4
        th(     "BTZI");	// 4
        th(   "BTZOGS");	// 4
        th(      "B'U");	// 4
        th(     "BU'A");	// 4
        th( "BUDDHA'I");	// 4
        th(    "BYABA");	// 4
        th(    "BYAmS");	// 4
        th(    "'BYAS");	// 4
        th(      "BYD");	// 4
        th(    "'BYEL");	// 4
        th(     "BY'O");	// 4
        th(       "BZ");	// 4
        th(      "BZA");	// 4
        th(     "BZAB");	// 4
        th(     "BZAN");	// 4
        th(    "BZHAB");	// 4
        th(    "BZH'I");	// 4
        th(    "BZHOR");	// 4
        th(    "BZHUR");	// 4
        th(    "BZLAB");	// 4
        th(     "BZON");	// 4
        th(      "C'A");	// 4
        th(     "CAHD");	// 4
        th(       "CH");	// 4
        th(  "'CHANGS");	// 4
        th( "'CHE'ANG");	// 4
        th(   "'CHE'O");	// 4
        th(    "'CHER");	// 4
        th(    "CHI'O");	// 4
        th(     "CHOB");	// 4
        th(     "CHSO");	// 4
        th(   "'CHUNG");	// 4
        th(      "'CI");	// 4
        th(      "CUL");	// 4
        th(     "D'AB");	// 4
        th( "DAnd'I'I");	// 4
        th(     "D'AR");	// 4
        th(   "DARSHA");	// 4
        th(     "DBOR");	// 4
        th(    "DBRES");	// 4
        th(      "DDE");	// 4
        th(     "'DEL");	// 4
        th(   "DGA'BA");	// 4
        th(      "DGI");	// 4
        th(     "DGOL");	// 4
        th(    "'DGOS");	// 4
        th(     "DGUD");	// 4
        th(     "DHAm");	// 4
        th(    "DHARA");	// 4
        th(  "DH'ARMA");	// 4
        th(  "DH'I:'I");	// 4
        th(    "DHI'O");	// 4
        th(      "dHO");	// 4
        th(      "dhU");	// 4
        th(     "DHVI");	// 4
        th(     "DIGS");	// 4
        th(      "DIK");	// 4
        th(    "D'ING");	// 4
        th(  "DKA'ANG");	// 4
        th(    "DKRUd");	// 4
        th(    "DKRUS");	// 4
        th(     "DKYE");	// 4
        th(  "DKYUNGS");	// 4
        th(     "DMAL");	// 4
        th(    "DMIGs");	// 4
        th(     "DMUL");	// 4
        th(     "'DOd");	// 4
        th(     "'DOL");	// 4
        th(   "'DONGS");	// 4
        th(      "DOO");	// 4
        th( "DPA'SEMS");	// 4
        th(    "DPO'I");	// 4
        th(     "DPRI");	// 4
        th(     "DPUL");	// 4
        th(     "DPUR");	// 4
        th(   "DPYI'I");	// 4
        th(   "'DRABS");	// 4
        th(  "DRE'U'I");	// 4
        th(   "DRI'AM");	// 4
        th(   "DRO'AM");	// 4
        th(    "'DROD");	// 4
        th(   "'DROGS");	// 4
        th(    "'DROS");	// 4
        th(     "DRUL");	// 4
        th(    "'DRUM");	// 4
        th(    "d'URA");	// 4
        th("d'U-RYA'I");	// 4
        th(     "DYAm");	// 4
        th(    "DZA:-");	// 4
        th(   "'DZA'I");	// 4
        th(    "'DZAN");	// 4
        th(    "'DZAS");	// 4
        th(    "'DZEL");	// 4
        th(    "'DZEN");	// 4
        th(    "'DZON");	// 4
        th(     "DZYO");	// 4
        th(      "GA:");	// 4
        th(    "GANDA");	// 4
        th( "GANG-G'A");	// 4
        th("GANG-GA''I");	// 4
        th( "GANG+G'I");	// 4
        th( "GANGHA'I");	// 4
        th(    "'GA'O");	// 4
        th(  "GA-YO'O");	// 4
        th(    "GAZHN");	// 4
        th(     "GDUM");	// 4
        th(     "GDUN");	// 4
        th(      "GED");	// 4
        th(   "GHANYA");	// 4
        th(     "GHAR");	// 4
        th(     "GHEN");	// 4
        th(     "GHOL");	// 4
        th(   "GHRA'I");	// 4
        th(    "GHZAN");	// 4
        th(     "GICG");	// 4
        th(   "GINDRA");	// 4
        th(     "'GLA");	// 4
        th(     "GLAM");	// 4
        th(     "GLEB");	// 4
        th(    "GLE'O");	// 4
        th(      "GMI");	// 4
        th(     "GNAL");	// 4
        th(    "Gnd'I");	// 4
        th(      "GNE");	// 4
        th(     "GNUB");	// 4
        th(     "GNUS");	// 4
        th(   "GNYAMS");	// 4
        th(    "GNYOD");	// 4
        th(     "GNYS");	// 4
        th(    "GOO'U");	// 4
        th(     "^GOS");	// 4
        th(      "GPO");	// 4
        th(    "'GRAG");	// 4
        th(     "GRAN");	// 4
        th(     "GRAt");	// 4
        th(    "'GRER");	// 4
        th(    "'GRO'");	// 4
        th(    "'GROB");	// 4
        th(     "GROt");	// 4
        th(    "'GROt");	// 4
        th(    "GRTAN");	// 4
        th(   "'GRUBS");	// 4
        th(    "'GRUR");	// 4
        th(    "GRWAR");	// 4
        th(       "GS");	// 4
        th(      "GSA");	// 4
        th(     "GSEN");	// 4
        th( "GSHE'ANG");	// 4
        th(    "GSHEB");	// 4
        th(    "GSHOD");	// 4
        th(    "GSHOS");	// 4
        th(     "GSOM");	// 4
        th(    "GSUMS");	// 4
        th(     "GTIS");	// 4
        th(     "GTOd");	// 4
        th(     "GTUL");	// 4
        th(     "GTZA");	// 4
        th(    "GTZAR");	// 4
        th(     "GTZI");	// 4
        th(   "GU'ANG");	// 4
        th(    "GUBTA");	// 4
        th(     "GUGS");	// 4
        th(     "GUMS");	// 4
        th(    "GUNGS");	// 4
        th(    "'GYA'");	// 4
        th(     "G-YA");	// 4
        th(   "GYANGS");	// 4
        th(    "GYA'O");	// 4
        th(    "'GYAR");	// 4
        th(    "G-YED");	// 4
        th(   "'GYENG");	// 4
        th(     "'GYI");	// 4
        th(    "'GYIS");	// 4
        th(    "'GYUD");	// 4
        th(   "G-YUGS");	// 4
        th(     "GYUN");	// 4
        th(   "'GYURO");	// 4
        th(   "GZHA'I");	// 4
        th(  "GZHONGS");	// 4
        th(     "GZIN");	// 4
        th(      "HAG");	// 4
        th(    "H'A'I");	// 4
        th(      "HEN");	// 4
        th(      "HIG");	// 4
        th(      "HNA");	// 4
        th(    "HPA'I");	// 4
        th(      "HRA");	// 4
        th("HR'I:A'A:");	// 4
        th("HR'I:A'A:SOGS");	// 4
        th("HR'I:shtR'I:WI");	// 4
        th(      "HU:");	// 4
        th(     "HYIR");	// 4
        th(        "i");	// 4
        th(      "JAM");	// 4
        th(     "'JEM");	// 4
        th(      "JES");	// 4
        th(      "JIR");	// 4
        th(    "'JOBS");	// 4
        th(      "JOO");	// 4
        th(     "JO'U");	// 4
        th(     "'JUB");	// 4
        th(    "'JUMS");	// 4
        th(    "'JUNG");	// 4
        th(     "KAHS");	// 4
        th(    "KAMBA");	// 4
        th(   "KARKAt");	// 4
        th(   "KARM'A");	// 4
        th(    "KARNA");	// 4
        th(    "KARTA");	// 4
        th(  "K'ATY'A");	// 4
        th(    "'KHAS");	// 4
        th(     "KHED");	// 4
        th(     "KHIM");	// 4
        th(    "KHndA");	// 4
        th(    "KHREG");	// 4
        th(    "KHRIB");	// 4
        th(  "'KHRING");	// 4
        th(    "KHROB");	// 4
        th(   "'KHRUd");	// 4
        th(   "KHRU'O");	// 4
        th(  "'KHUNGS");	// 4
        th(   "KHYAGS");	// 4
        th(    "KHYAN");	// 4
        th(    "KHYEM");	// 4
        th("KHYE'U'AM");	// 4
        th(  "'KHYIMS");	// 4
        th(    "KHYIN");	// 4
        th(   "KHYO'I");	// 4
        th(    "KHYOL");	// 4
        th(  "'KHYOMS");	// 4
        th(       "Ki");	// 4
        th(    "KLEGS");	// 4
        th(     "KLON");	// 4
        th(     "KLOS");	// 4
        th(     "KOO'");	// 4
        th(     "KRAN");	// 4
        th(   "KRiTYA");	// 4
        th(     "KRUM");	// 4
        th(     "KSAm");	// 4
        th(      "Ksh");	// 4
        th(    "KshA'");	// 4
        th(   "KshA'I");	// 4
        th(    "Ksh'I");	// 4
        th(     "KsHI");	// 4
        th(     "KT'A");	// 4
        th(    "KTA'I");	// 4
        th(      "KTI");	// 4
        th(      "KUG");	// 4
        th(      "KUm");	// 4
        th(    "KURMA");	// 4
        th(      "KVI");	// 4
        th(      "LA:");	// 4
        th(     "LAHG");	// 4
        th(  "LANdA'I");	// 4
        th(   "LANGGA");	// 4
        th(  "LANGK'A");	// 4
        th(  "LANYDZA");	// 4
        th(     "LCAS");	// 4
        th(     "LCIG");	// 4
        th(      "LCU");	// 4
        th(    "LCUGS");	// 4
        th(   "LDE'AM");	// 4
        th(     "LDIG");	// 4
        th(   "LDINGS");	// 4
        th(     "LDOS");	// 4
        th(    "LDUNG");	// 4
        th(    "LE'AM");	// 4
        th(      "LED");	// 4
        th(    "LENdA");	// 4
        th(  "LENdA'I");	// 4
        th(      "LEt");	// 4
        th(    "LHAMS");	// 4
        th(     "LHED");	// 4
        th(     "LHOS");	// 4
        th(     "LHUD");	// 4
        th(     "LHUS");	// 4
        th(     "Li'I");	// 4
        th("LING+GA'I");	// 4
        th(    "LINTA");	// 4
        th(    "LJIBS");	// 4
        th(     "LKAG");	// 4
        th(    "LTAMS");	// 4
        th(   "LTANGS");	// 4
        th(    "LTOBS");	// 4
        th(      "LYE");	// 4
        th(     "L+YU");	// 4
        th(     "M'A:");	// 4
        th(    "M'ANG");	// 4
        th(    "MANGA");	// 4
        th(  "MANYDZI");	// 4
        th(   "M'ARGA");	// 4
        th(   "MARGAN");	// 4
        th(     "M'AS");	// 4
        th(    "MAYIN");	// 4
        th(      "MBU");	// 4
        th(     "MDZA");	// 4
        th(     "ME'U");	// 4
        th(      "MGI");	// 4
        th(    "MGU'O");	// 4
        th(       "Mi");	// 4
        th(      "MIM");	// 4
        th(    "MITRA");	// 4
        th(    "MITTE");	// 4
        th(     "MJON");	// 4
        th(     "MJUB");	// 4
        th(    "MKHOL");	// 4
        th(    "MKHRI");	// 4
        th(    "MKHUN");	// 4
        th(   "MKHYER");	// 4
        th(    "MKYEN");	// 4
        th(     "MN'A");	// 4
        th(    "MndAL");	// 4
        th(    "MNGEN");	// 4
        th(    "MNGOD");	// 4
        th(     "MNOM");	// 4
        th(     "MNYA");	// 4
        th(    "MNYEM");	// 4
        th(   "MO'I'O");	// 4
        th(      "MPA");	// 4
        th(   "MRDZOD");	// 4
        th(      "MRI");	// 4
        th(    "MSTAN");	// 4
        th(  "MTHANGS");	// 4
        th(     "MTHI");	// 4
        th(    "MTH'I");	// 4
        th(    "MTHO'");	// 4
        th(    "MTONG");	// 4
        th(    "MTSAB");	// 4
        th(    "MTSIG");	// 4
        th(    "MTSIS");	// 4
        th(   "MTSO'U");	// 4
        th(     "MZOS");	// 4
        th(    "NANDI");	// 4
        th(   "NANTAS");	// 4
        th(    "N'ASA");	// 4
        th(      "ndA");	// 4
        th(      "NDA");	// 4
        th(     "NDHA");	// 4
        th(    "'NGAS");	// 4
        th(     "NGGE");	// 4
        th(     "NGHU");	// 4
        th(     "NGOG");	// 4
        th(    "NGO'O");	// 4
        th(       "Ni");	// 4
        th(   "NI'ANG");	// 4
        th(   "NIshKA");	// 4
        th(      "NNA");	// 4
        th(      "NOO");	// 4
        th(      "NTI");	// 4
        th(    "NTI'O");	// 4
        th(   "NU'ANG");	// 4
        th(  "NYAM-SU");	// 4
        th(    "NYDZA");	// 4
        th(    "NYENG");	// 4
        th(       "OD");	// 4
        th(      "ONG");	// 4
        th(       "OR");	// 4
        th(      "PAH");	// 4
        th(     "PA'i");	// 4
        th(    "PA'IR");	// 4
        th(    "PA'IS");	// 4
        th(    "PAndA");	// 4
        th(    "P'AnI");	// 4
        th(  "PANYDZI");	// 4
        th(  "PANYTZI");	// 4
        th(    "PA'OG");	// 4
        th(     "PARI");	// 4
        th(    "PAYIN");	// 4
        th(       "PD");	// 4
        th(    "PHAYG");	// 4
        th(     "PH'E");	// 4
        th(    "'PHES");	// 4
        th(    "'PHON");	// 4
        th( "PHRA'ANG");	// 4
        th(     "PHRE");	// 4
        th(    "PHREL");	// 4
        th(  "PHRI'AM");	// 4
        th(   "'PHRIS");	// 4
        th(   "PHUttA");	// 4
        th(  "'PHYAGS");	// 4
        th( "PHYE'ANG");	// 4
        th(   "PHYEBS");	// 4
        th(    "PHYEL");	// 4
        th( "'PHYI'AM");	// 4
        th("PHYI'I'AM");	// 4
        th(  "'PHYI'O");	// 4
        th(  "'PHYOGS");	// 4
        th(      "PIN");	// 4
        th(    "PIndA");	// 4
        th(   "PINGKA");	// 4
        th(      "PLU");	// 4
        th(      "'PO");	// 4
        th(     "PRAD");	// 4
        th(     "PRAm");	// 4
        th(     "PRAS");	// 4
        th(    "'PRAS");	// 4
        th(   "PRiTTI");	// 4
        th(      "PYE");	// 4
        th(      "RA'");	// 4
        th(   "RA'ANG");	// 4
        th(     "R'AG");	// 4
        th(    "RAKTA");	// 4
        th(    "RANDA");	// 4
        th(    "RASMI");	// 4
        th(     "RBBA");	// 4
        th(     "RDAB");	// 4
        th(     "RDAD");	// 4
        th(     "RDAR");	// 4
        th(    "RDEGS");	// 4
        th(    "RDENG");	// 4
        th(     "RDER");	// 4
        th(     "RDUR");	// 4
        th(    "RDZIR");	// 4
        th(     "RGAB");	// 4
        th(    "RGAYS");	// 4
        th(     "RGEL");	// 4
        th(     "RGHA");	// 4
        th(    "RGHAm");	// 4
        th(     "RGON");	// 4
        th(     "RGUL");	// 4
        th(     "RGYI");	// 4
        th( "RGYU'I'O");	// 4
        th(      "RHA");	// 4
        th(   "RIG-SU");	// 4
        th(    "R'I'I");	// 4
        th(      "RIm");	// 4
        th(     "RI'U");	// 4
        th(    "RJESU");	// 4
        th(   "RJE-SU");	// 4
        th(    "RJING");	// 4
        th(      "RJU");	// 4
        th(   "RKO'AM");	// 4
        th(    "RKO'I");	// 4
        th(     "RKUD");	// 4
        th(    "RKYED");	// 4
        th(     "RKYU");	// 4
        th(    "RLAGS");	// 4
        th(    "RLAMS");	// 4
        th(     "RLEM");	// 4
        th(     "RMAG");	// 4
        th(     "RMIL");	// 4
        th(     "RMOL");	// 4
        th(    "RnA'I");	// 4
        th(   "RNGOGS");	// 4
        th(   "RNGUGS");	// 4
        th(     "RNOR");	// 4
        th(    "RNYEG");	// 4
        th(   "RO'ANG");	// 4
        th(    "RONYA");	// 4
        th(     "RSHA");	// 4
        th(    "RTENG");	// 4
        th(     "RTIG");	// 4
        th(     "RTIN");	// 4
        th(    "RTOBS");	// 4
        th(   "RTZVAR");	// 4
        th(      "RU:");	// 4
        th(   "RVA'AM");	// 4
        th(    "RYGAS");	// 4
        th(     "SAHA");	// 4
        th(     "S'Am");	// 4
        th(  "SAMBHAR");	// 4
        th(  "SANYTZA");	// 4
        th(   "SAR-BA");	// 4
        th(   "SARDZA");	// 4
        th(    "SARTI");	// 4
        th(    "SATNA");	// 4
        th(   "SATVAS");	// 4
        th(    "SBAMS");	// 4
        th(      "SBE");	// 4
        th(     "SBRE");	// 4
        th(    "SBYEN");	// 4
        th(    "SBYES");	// 4
        th(    "SBYIL");	// 4
        th(     "SCAL");	// 4
        th(    "SCOGS");	// 4
        th(    "SDABS");	// 4
        th(    "SDEGS");	// 4
        th(  "SDE'U'I");	// 4
        th(    "SDIBS");	// 4
        th(     "SDIR");	// 4
        th(    "SDOBS");	// 4
        th(    "SDZUN");	// 4
        th(     "SEEN");	// 4
        th(  "SENGGER");	// 4
        th(    "SGREB");	// 4
        th(    "SGRED");	// 4
        th(    "SGROR");	// 4
        th(    "SGRVA");	// 4
        th(     "SGUB");	// 4
        th(    "SGU'I");	// 4
        th(   "SGYANG");	// 4
        th(    "SGYER");	// 4
        th(   "SGYUGS");	// 4
        th(     "shA:");	// 4
        th( "SHAKYA'I");	// 4
        th( "SH'AKY'I");	// 4
        th("SHAMBHI'I");	// 4
        th(    "sh'AN");	// 4
        th(  "SHANGKA");	// 4
        th(    "shA'O");	// 4
        th(  "SHA'O'I");	// 4
        th(  "SHIKshA");	// 4
        th("SHiNTRANG");	// 4
        th( "shINYTZA");	// 4
        th(   "SH'NTI");	// 4
        th(  "SHRDDHA");	// 4
        th(   "shth'A");	// 4
        th(     "shtI");	// 4
        th(    "shtI:");	// 4
        th(    "SHTZE");	// 4
        th(   "SHTZYU");	// 4
        th(    "sh+YA");	// 4
        th(     "shYO");	// 4
        th(   "SIDDHA");	// 4
        th( "SIDDHI'I");	// 4
        th(  "SINDH'U");	// 4
        th(  "SINDHUR");	// 4
        th(  "SINGHAS");	// 4
        th(   "SKANDA");	// 4
        th(     "SKES");	// 4
        th(    "SKEYS");	// 4
        th(     "SKIS");	// 4
        th(   "SKRA'O");	// 4
        th(    "SKRIN");	// 4
        th(    "SKROG");	// 4
        th(    "SKYEM");	// 4
        th(   "SKYE'U");	// 4
        th(    "SKYUS");	// 4
        th(     "SLAG");	// 4
        th(    "SLUNG");	// 4
        th(     "SM'A");	// 4
        th(  "SM'ATRA");	// 4
        th(     "SMOM");	// 4
        th(   "SMRANG");	// 4
        th(   "SMRENG");	// 4
        th(    "SMRES");	// 4
        th(    "SMYUR");	// 4
        th(   "SNYOBS");	// 4
        th(   "SNYOmS");	// 4
        th(    "SNYOS");	// 4
        th(   "SO'ANG");	// 4
        th(    "SPE'I");	// 4
        th(    "SPH'A");	// 4
        th(    "SPHYI");	// 4
        th(     "SPID");	// 4
        th(    "SPRAL");	// 4
        th(    "SPRAR");	// 4
        th(    "SPRUB");	// 4
        th(    "SPRUM");	// 4
        th(    "SPRUN");	// 4
        th(   "SPYA'I");	// 4
        th(  "SPYINGS");	// 4
        th(   "SPYO'I");	// 4
        th(    "SRIBS");	// 4
        th(     "SROM");	// 4
        th(     "SRUG");	// 4
        th(      "SSU");	// 4
        th(     "STEB");	// 4
        th(     "STHU");	// 4
        th(    "STH'U");	// 4
        th(     "STIL");	// 4
        th(     "STIM");	// 4
        th(    "STI'O");	// 4
        th(     "STOG");	// 4
        th(    "STREE");	// 4
        th(    "STR'I");	// 4
        th(   "SVAH'A");	// 4
        th(     "SYAN");	// 4
        th(     "SY'I");	// 4
        th(      "SYO");	// 4
        th(     "TABS");	// 4
        th(  "TANG+GI");	// 4
        th(    "TA:'O");	// 4
        th(      "TEL");	// 4
        th(       "th");	// 4
        th(    "THADm");	// 4
        th(   "THA-LO");	// 4
        th(   "'THEBS");	// 4
        th(      "thU");	// 4
        th(   "'THUGS");	// 4
        th(    "'THUM");	// 4
        th(   "tIKKAS");	// 4
        th(     "T'IS");	// 4
        th(    "t'KKI");	// 4
        th(      "TLA");	// 4
        th(      "TNE");	// 4
        th(     "TRA:");	// 4
        th(    "TRA'O");	// 4
        th(     "TRAR");	// 4
        th(    "TRE'I");	// 4
        th(   "tR'IKA");	// 4
        th(     "TRIS");	// 4
        th(     "TRYA");	// 4
        th(   "'TSAGS");	// 4
        th(     "TS'E");	// 4
        th(   "'TSIGS");	// 4
        th(    "'TSUN");	// 4
        th(   "TSVA'O");	// 4
        th(   "TSWA'I");	// 4
        th(     "TTHA");	// 4
        th(      "t'U");	// 4
        th(      "T'U");	// 4
        th(     "TUGS");	// 4
        th(      "TVI");	// 4
        th(     "T+YA");	// 4
        th(      "TYE");	// 4
        th(     "TZAD");	// 4
        th(  "TZANTRA");	// 4
        th(   "TZARBA");	// 4
        th(     "TZEG");	// 4
        th(     "TZMA");	// 4
        th(    "TZTZA");	// 4
        th(     "TZUR");	// 4
        th(    "TZYEE");	// 4
        th(      "'UM");	// 4
        th(      "'UN");	// 4
        th(      "WA:");	// 4
        th(     "W'Am");	// 4
        th(   "WANDRA");	// 4
        th(      "YA'");	// 4
        th(      "YBA");	// 4
        th(    "YUKTA");	// 4
        th(    "YU:'O");	// 4
        th(      "YYA");	// 4
        th(    "ZH'AN");	// 4
        th(    "ZHEGS");	// 4
        th(    "ZHE'I");	// 4
        th(   "ZHUNGS");	// 4
        th(    "ZHVAR");	// 4
        th(    "ZLUMS");	// 4
        th(    "'ZUNG");	// 4
        th(      "ZVA");	// 4
        th(      "111");	// 5
        th(      "112");	// 5
        th(      "114");	// 5
        th(      "128");	// 5
        th(      "134");	// 5
        th(      "160");	// 5
        th(      "197");	// 5
        th(     "1979");	// 5
        th(      "202");	// 5
        th(      "222");	// 5
        th(      "241");	// 5
        th(      "244");	// 5
        th(      "284");	// 5
        th(      "288");	// 5
        th(      "319");	// 5
        th(      "327");	// 5
        th(      "328");	// 5
        th(      "367");	// 5
        th(      "420");	// 5
        th(      "444");	// 5
        th(      "600");	// 5
        th(   "7GONGS");	// 5
        th(     "7KUN");	// 5
        th(    "7LNGA");	// 5
        th(      "7SA");	// 5
        th(   "7SPRUL");	// 5
        th(       "98");	// 5
        th("A'A:BIGHN'A");	// 5
        th("A'A:BIGHN'NTA");	// 5
        th(    "AADYA");	// 5
        th( "A'A:KHAm");	// 5
        th("AANG+GA'I");	// 5
        th(    "A'A'O");	// 5
        th(     "AA'O");	// 5
        th(    "AATMA");	// 5
        th(      "'AD");	// 5
        th(      "AID");	// 5
        th(      "AIL");	// 5
        th(       "AL");	// 5
        th(      "'AL");	// 5
        th(  "AOdYANA");	// 5
        th(      "AOT");	// 5
        th(      "APR");	// 5
        th(     "ARHA");	// 5
        th(     "AU'O");	// 5
        th(      "AUt");	// 5
        th(  "AUT-PAL");	// 5
        th(   "AUTPLA");	// 5
        th(       "'B");	// 5
        th(      "BA4");	// 5
        th(      "BAA");	// 5
        th( "BADZRASA");	// 5
        th(    "'BA'I");	// 5
        th(   "BA'O'I");	// 5
        th(     "BCHI");	// 5
        th(     "BDRI");	// 5
        th(     "BDUM");	// 5
        th(    "BDZAD");	// 5
        th(     "'BED");	// 5
        th("BEEd'UR+YA");	// 5
        th(      "BES");	// 5
        th(     "BGA'");	// 5
        th(     "BGES");	// 5
        th(     "BGOR");	// 5
        th(   "BGRAGS");	// 5
        th(    "BGRED");	// 5
        th(     "BGYA");	// 5
        th(     "BGYE");	// 5
        th(    "BHZIN");	// 5
        th(    "B'I'I");	// 5
        th(    "BKABS");	// 5
        th(     "BKED");	// 5
        th(   "BKRUGS");	// 5
        th(    "BKUMS");	// 5
        th(  "BLO'I'O");	// 5
        th(     "BLOL");	// 5
        th(    "BNGAL");	// 5
        th(    "BNGAS");	// 5
        th(    "BRABS");	// 5
        th(   "BRDANG");	// 5
        th(     "BRDO");	// 5
        th(  "BRDZANG");	// 5
        th(  "BRiKshA");	// 5
        th(    "'BRIS");	// 5
        th(     "BRNA");	// 5
        th(   "BRNYAD");	// 5
        th( "BRNYINGS");	// 5
        th(    "BRO'I");	// 5
        th(    "BRTAt");	// 5
        th(  "BRTZAGS");	// 5
        th(   "BRTZAS");	// 5
        th(   "BRTZUB");	// 5
        th(    "BSBUS");	// 5
        th(    "BSDAS");	// 5
        th(  "BSGYANG");	// 5
        th(   "BSGYUD");	// 5
        th(   "BSHA'I");	// 5
        th(    "BSHOD");	// 5
        th(   "BSHONG");	// 5
        th(      "BSI");	// 5
        th(     "BSIR");	// 5
        th(    "BSKAB");	// 5
        th(    "BSKOG");	// 5
        th(  "BSKRANG");	// 5
        th(   "BSKRUB");	// 5
        th(  "BSKUNGS");	// 5
        th(    "BSKYA");	// 5
        th(   "BSKYEN");	// 5
        th(   "BSKYIN");	// 5
        th(    "BSMAS");	// 5
        th(  "BSNYOGS");	// 5
        th(    "BSPUS");	// 5
        th(    "BSRIG");	// 5
        th(    "BSRIS");	// 5
        th(    "BSROG");	// 5
        th(    "BSTIR");	// 5
        th(     "BSTU");	// 5
        th(    "BSTUR");	// 5
        th(     "BSUG");	// 5
        th(    "BTZES");	// 5
        th(      "BU'");	// 5
        th(     "'BYA");	// 5
        th(    "'BYIL");	// 5
        th(   "BYI'US");	// 5
        th(  "'BYUNGS");	// 5
        th(     "BZAG");	// 5
        th(     "BZER");	// 5
        th(     "BZHA");	// 5
        th(    "BZHUN");	// 5
        th(    "'CHIN");	// 5
        th(  "CHU'ANG");	// 5
        th(     "CI'O");	// 5
        th(    "DA'AM");	// 5
        th(   "DAKshI");	// 5
        th(     "DAND");	// 5
        th(     "dA'O");	// 5
        th(    "DBYAN");	// 5
        th(      "DDU");	// 5
        th(      "D'E");	// 5
        th(      "DE:");	// 5
        th(    "'DE'O");	// 5
        th(   "DGENGS");	// 5
        th(    "DGRAL");	// 5
        th(     "DGRO");	// 5
        th(     "DGUL");	// 5
        th(   "DGYE'O");	// 5
        th(     "DHI:");	// 5
        th(   "DH'I'I");	// 5
        th(   "DHRiKA");	// 5
        th(    "DHUtA");	// 5
        th(    "'DIBS");	// 5
        th(  "'DI'I'O");	// 5
        th(    "DKA'A");	// 5
        th(   "DKRI'O");	// 5
        th(    "DKRUM");	// 5
        th(    "DMA'I");	// 5
        th(    "DMA'O");	// 5
        th(    "DNGAG");	// 5
        th(    "DNGOM");	// 5
        th(   "DPA'AM");	// 5
        th(    "DPYIG");	// 5
        th(  "'DRANGS");	// 5
        th(    "DRENG");	// 5
        th(  "DRI'ANG");	// 5
        th(     "DRIB");	// 5
        th(    "DRIGS");	// 5
        th(    "DRIMS");	// 5
        th(    "DRUGS");	// 5
        th(     "DSOL");	// 5
        th(   "DSPYAD");	// 5
        th(    "DUMBA");	// 5
        th(     "DUMS");	// 5
        th(     "DU'U");	// 5
        th(     "DVAd");	// 5
        th(     "DVAG");	// 5
        th(    "DVA'I");	// 5
        th(     "dYAN");	// 5
        th(  "'DZAMBU");	// 5
        th(     "'DZE");	// 5
        th(    "'DZES");	// 5
        th(     "DZHI");	// 5
        th(   "'DZIGS");	// 5
        th(    "'DZIL");	// 5
        th(   "DZNYA'");	// 5
        th(    "'DZUN");	// 5
        th(       "'E");	// 5
        th(   "GA'A'I");	// 5
        th(     "'GAd");	// 5
        th(     "'GAD");	// 5
        th( "GANYDZIR");	// 5
        th(   "GANYJI");	// 5
        th(   "GA-YOR");	// 5
        th(      "GCE");	// 5
        th(     "GCEG");	// 5
        th(     "GCID");	// 5
        th(     "GCOR");	// 5
        th(     "GDAS");	// 5
        th(      "GDE");	// 5
        th(     "GHAn");	// 5
        th(   "GHAndA");	// 5
        th(     "GHRA");	// 5
        th(      "GIG");	// 5
        th(     "GLIN");	// 5
        th(   "GLONGS");	// 5
        th(    "GNAGS");	// 5
        th(      "GNI");	// 5
        th(   "GNYING");	// 5
        th(  "GNYI-SU");	// 5
        th(     "GRID");	// 5
        th(    "GRING");	// 5
        th(     "GROM");	// 5
        th(    "GRO'O");	// 5
        th(    "GRUBS");	// 5
        th(   "'GRUNG");	// 5
        th(   "GSHAMS");	// 5
        th(  "GSHE'AM");	// 5
        th(   "GSHOGS");	// 5
        th(    "GSO'O");	// 5
        th(    "GTA'I");	// 5
        th(    "GTZIG");	// 5
        th(  "GUPTA'I");	// 5
        th(     "GYEM");	// 5
        th(     "GYIL");	// 5
        th(    "GYING");	// 5
        th(   "G-YONG");	// 5
        th(   "GZHAGS");	// 5
        th(    "GZHEL");	// 5
        th(    "GZHIM");	// 5
        th(    "GZHNA");	// 5
        th(     "HA'O");	// 5
        th(      "HED");	// 5
        th(      "HEE");	// 5
        th(      "HOB");	// 5
        th(    "HVANG");	// 5
        th(    "HYA'I");	// 5
        th(     "HYAN");	// 5
        th(      "'ID");	// 5
        th(      "'IR");	// 5
        th(       "IS");	// 5
        th(        "J");	// 5
        th(     "JE'I");	// 5
        th(     "'JId");	// 5
        th(     "JIGS");	// 5
        th(      "JOG");	// 5
        th(      "JUL");	// 5
        th(  "KA:DPAL");	// 5
        th(    "K'A'I");	// 5
        th(    "KANTA");	// 5
        th(    "KARTI");	// 5
        th(     "K'AS");	// 5
        th(      "KEG");	// 5
        th(       "KH");	// 5
        th(   "'KHAMS");	// 5
        th(    "KHASM");	// 5
        th(    "KHAYD");	// 5
        th(     "KHEd");	// 5
        th(     "KHEM");	// 5
        th(    "KHRAD");	// 5
        th(   "KHRAMS");	// 5
        th(   "KHRE'I");	// 5
        th(   "'KHREL");	// 5
        th( "KHRI'ANG");	// 5
        th(   "'KHROD");	// 5
        th(   "'KHROG");	// 5
        th(  "KHRU'AM");	// 5
        th(   "'KHRUB");	// 5
        th(  "KHRUNGS");	// 5
        th(   "KHYABS");	// 5
        th(    "KHYEB");	// 5
        th(   "'KHYEN");	// 5
        th( "KHYE'U'O");	// 5
        th(   "'KHYIM");	// 5
        th(   "KHYUGS");	// 5
        th( "K'IRTISA");	// 5
        th(      "KLi");	// 5
        th(      "KOM");	// 5
        th(   "KONGGA");	// 5
        th(   "KONGKA");	// 5
        th(    "KOO'U");	// 5
        th("KO'UdIN+YA");	// 5
        th(     "KR'A");	// 5
        th(     "KRIR");	// 5
        th(    "'KRUL");	// 5
        th(      "KUD");	// 5
        th(     "KU'I");	// 5
        th(      "KUL");	// 5
        th(    "KYAGN");	// 5
        th(    "KYU'O");	// 5
        th(     "L'AN");	// 5
        th(  "LANG-KA");	// 5
        th(    "LBANG");	// 5
        th(     "LCAD");	// 5
        th(   "LHANGS");	// 5
        th(     "LIGS");	// 5
        th( "LIN-DA'I");	// 5
        th( "LINGGA'I");	// 5
        th( "LINGKA'I");	// 5
        th(     "LJIB");	// 5
        th(     "LTER");	// 5
        th(      "LTI");	// 5
        th(  "LTO'ANG");	// 5
        th(   "LUM-BI");	// 5
        th(    "^LUNG");	// 5
        th(    "L'U'U");	// 5
        th(     "M'AN");	// 5
        th(    "MAndO");	// 5
        th(  "MANG-GA");	// 5
        th("MANYDZI'I");	// 5
        th(   "MCHANG");	// 5
        th(   "MCHOGS");	// 5
        th(     "MCOM");	// 5
        th(     "MDUG");	// 5
        th(     "MDUR");	// 5
        th(    "MDZEM");	// 5
        th(    "MDZER");	// 5
        th(   "ME'ANG");	// 5
        th(     "MGOL");	// 5
        th(    "MGROL");	// 5
        th(     "MGUN");	// 5
        th(    "MINGS");	// 5
        th(   "MKHAMS");	// 5
        th(   "MKHO'O");	// 5
        th(   "MKHYAD");	// 5
        th(   "MNAD'A");	// 5
        th(     "MNGA");	// 5
        th(    "MNYAS");	// 5
        th(   "MNYE'O");	// 5
        th(    "MNYID");	// 5
        th(    "MOONG");	// 5
        th(   "MRA'AM");	// 5
        th(   "MTHOGS");	// 5
        th(    "MTHUL");	// 5
        th(    "MTSED");	// 5
        th(    "MTSEM");	// 5
        th(      "MYE");	// 5
        th(     "MYES");	// 5
        th(      "NA:");	// 5
        th(      "NA5");	// 5
        th(    "nA'AM");	// 5
        th(  "NANDA'I");	// 5
        th(  "NANTA'I");	// 5
        th(      "NED");	// 5
        th(     "NEMS");	// 5
        th(     "NGA'");	// 5
        th(  "NGA'I'O");	// 5
        th(     "NGOD");	// 5
        th(      "NIN");	// 5
        th(     "NOMS");	// 5
        th(      "NRA");	// 5
        th(      "NTU");	// 5
        th(     "N-YA");	// 5
        th(   "NYA'AM");	// 5
        th(    "NYIMS");	// 5
        th(    "NYI'O");	// 5
        th(    "NYOMS");	// 5
        th(   "NYONGS");	// 5
        th(    "NYO'O");	// 5
        th(      "P0A");	// 5
        th(      "PA:");	// 5
        th(     "PA'A");	// 5
        th(    "PA'AI");	// 5
        th(     "P'AM");	// 5
        th(  "PAntITA");	// 5
        th(   "PA'ONG");	// 5
        th(     "PDMA");	// 5
        th(    "PHENG");	// 5
        th(     "PHIG");	// 5
        th(    "'PHIR");	// 5
        th(    "PHI'U");	// 5
        th(     "PH'O");	// 5
        th(    "PHO'O");	// 5
        th(    "PHREN");	// 5
        th(    "PHRIG");	// 5
        th(    "PHROM");	// 5
        th(     "PHUM");	// 5
        th(   "'PHYAM");	// 5
        th(   "'PHYER");	// 5
        th(     "PI'I");	// 5
        th("PING-KA'I");	// 5
        th(     "PO'U");	// 5
        th(  "PROKshA");	// 5
        th(      "PUm");	// 5
        th(    "P'UNG");	// 5
        th(   "PUn+YA");	// 5
        th(    "PUnYE");	// 5
        th(  "P'URB'A");	// 5
        th(     "PYAD");	// 5
        th(     "PYIS");	// 5
        th(     "PYOD");	// 5
        th(   "RANGGA");	// 5
        th(   "RASMEE");	// 5
        th(    "RASTU");	// 5
        th(     "RB'A");	// 5
        th(    "RDU'O");	// 5
        th(    "RDZ'A");	// 5
        th(   "RDZAGS");	// 5
        th(  "RDZI'AM");	// 5
        th(   "RDZI'O");	// 5
        th(     "RGEN");	// 5
        th(   "'RGYUD");	// 5
        th(      "RiS");	// 5
        th(   "RIshtA");	// 5
        th(     "RJID");	// 5
        th(    "RJI'I");	// 5
        th(     "RJOS");	// 5
        th(    "RKU'I");	// 5
        th(    "RLONG");	// 5
        th(     "RMIN");	// 5
        th(     "RMMA");	// 5
        th(     "RnnA");	// 5
        th(    "RONGS");	// 5
        th(      "ROO");	// 5
        th(     "RTIB");	// 5
        th(      "RTU");	// 5
        th(   "RTZAGS");	// 5
        th(    "RTZUM");	// 5
        th(    "RVA'O");	// 5
        th(     "RYAS");	// 5
        th(   "SAMBHO");	// 5
        th( "SANG+GHA");	// 5
        th(    "SATYA");	// 5
        th(  "SATZTSA");	// 5
        th(    "SBA'I");	// 5
        th(    "SBUNG");	// 5
        th(     "SBYE");	// 5
        th(    "SBYID");	// 5
        th(     "SDEG");	// 5
        th(     "SDEL");	// 5
        th(    "SDEMS");	// 5
        th(     "SDEN");	// 5
        th(      "SDI");	// 5
        th(  "SGO'ANG");	// 5
        th(    "SGOGS");	// 5
        th( "SGRA'I'O");	// 5
        th(  "SGRINGS");	// 5
        th(     "SGYI");	// 5
        th(    "SGYUD");	// 5
        th(    "SGYUM");	// 5
        th(   "SHA'AM");	// 5
        th(   "SHAKYA");	// 5
        th(  "sh'ANTA");	// 5
        th(  "SHI'ANG");	// 5
        th(    "SHIBS");	// 5
        th(   "SH'I'I");	// 5
        th(     "SHIt");	// 5
        th(  "SH'KYAR");	// 5
        th(   "SHR'AI");	// 5
        th(   "SHRI'A");	// 5
        th(    "SHshA");	// 5
        th(  "SHUDDHO");	// 5
        th(    "SHUNG");	// 5
        th(    "SHV'A");	// 5
        th(       "Si");	// 5
        th(    "SKA'I");	// 5
        th(     "SKER");	// 5
        th(   "SKRONG");	// 5
        th(   "SKYEBS");	// 5
        th(   "SKYO'I");	// 5
        th(   "SKYO'O");	// 5
        th(     "SLED");	// 5
        th(    "SLEGS");	// 5
        th(    "SMYAD");	// 5
        th(   "SNA'AM");	// 5
        th(     "SNAB");	// 5
        th(   "SNYE'I");	// 5
        th(     "SNYU");	// 5
        th(     "SOBS");	// 5
        th(   "SPHRUL");	// 5
        th(  "SPRE'UR");	// 5
        th(     "SPRi");	// 5
        th(    "SPROB");	// 5
        th(   "SPRONG");	// 5
        th(  "SPYONGS");	// 5
        th(    "SRA'I");	// 5
        th(    "SRMAS");	// 5
        th(    "SRU'I");	// 5
        th(     "SRUS");	// 5
        th(     "SRWA");	// 5
        th(   "STI'AM");	// 5
        th(    "STOMS");	// 5
        th(    "STUGS");	// 5
        th(     "STZA");	// 5
        th(      "SVO");	// 5
        th(      "SWA");	// 5
        th(     "SYAS");	// 5
        th(      "SYI");	// 5
        th(   "tA'IKA");	// 5
        th(    "T'A'O");	// 5
        th(    "THALO");	// 5
        th(    "TH'AM");	// 5
        th(    "THA'O");	// 5
        th(    "TH'AS");	// 5
        th(     "THIL");	// 5
        th(     "THOd");	// 5
        th(    "THUBS");	// 5
        th(      "TID");	// 5
        th(     "t'IG");	// 5
        th(  "t'IKA'I");	// 5
        th(   "tIKA'I");	// 5
        th(   "t'IKAS");	// 5
        th(  "t'IKK'A");	// 5
        th(   "T'IKKA");	// 5
        th( "t'IKKARA");	// 5
        th(     "TPAL");	// 5
        th(     "TRAG");	// 5
        th(     "TR'i");	// 5
        th(     "'TSA");	// 5
        th(   "'TSABS");	// 5
        th(   "'TSEMS");	// 5
        th(    "'TSON");	// 5
        th(  "'TSONGS");	// 5
        th(    "TSUMS");	// 5
        th(  "'TSUNGS");	// 5
        th(      "TUL");	// 5
        th(     "TYA'");	// 5
        th(   "TZAndI");	// 5
        th( "TZARYA'I");	// 5
        th(     "TZ'I");	// 5
        th(     "TZID");	// 5
        th(     "TZIR");	// 5
        th(    "TZTSE");	// 5
        th(    "TZTSI");	// 5
        th(     "TZYA");	// 5
        th(    "WANTU");	// 5
        th(      "WUR");	// 5
        th(     "Y'AD");	// 5
        th(    "^YANG");	// 5
        th(  "YATZTSA");	// 5
        th(     "YEES");	// 5
        th(     "YEGS");	// 5
        th(      "YEL");	// 5
        th(      "YEM");	// 5
        th(     "YU'I");	// 5
        th(     "YU'O");	// 5
        th(     "'ZAD");	// 5
        th(     "ZHEL");	// 5
        th(    "ZHESO");	// 5
        th(    "ZHIBS");	// 5
        th(     "ZHUd");	// 5
        th(    "ZINGS");	// 5
        th(   "ZLA'AM");	// 5
        th(     "ZLAB");	// 5
        th(      "ZUD");	// 5
        th(       "-:");	// 6
        th(      "001");	// 6
        th(      "101");	// 6
        th(      "105");	// 6
        th(      "129");	// 6
        th(      "130");	// 6
        th(     "1969");	// 6
        th(     "1981");	// 6
        th(      "287");	// 6
        th(      "300");	// 6
        th(      "314");	// 6
        th(      "426");	// 6
        th(      "468");	// 6
        th(      "473");	// 6
        th(      "483");	// 6
        th(    "7BZHI");	// 6
        th(    "7NYIN");	// 6
        th(      "AA'");	// 6
        th("A'A:BIGHN'ANTA");	// 6
        th( "A'A:DMAR");	// 6
        th(     "A'Am");	// 6
        th(  "AANYTZA");	// 6
        th(    "AARTI");	// 6
        th(     "A'AT");	// 6
        th(  "A'ATZ'A");	// 6
        th(     "AE'I");	// 6
        th(     "AE'O");	// 6
        th(      "AES");	// 6
        th(     "'A'I");	// 6
        th(     "A'IT");	// 6
        th(      "'Am");	// 6
        th(      "'AN");	// 6
        th(     "ANTI");	// 6
        th(       "AP");	// 6
        th(       "At");	// 6
        th(   "AUDPAL");	// 6
        th(      "AUL");	// 6
        th(   "AUTAPA");	// 6
        th(      "BA:");	// 6
        th(     "BADG");	// 6
        th(   "BADZRI");	// 6
        th(      "BAI");	// 6
        th(    "BARnA");	// 6
        th(  "BDE'ANG");	// 6
        th(     "BDOR");	// 6
        th(     "BDRA");	// 6
        th(    "BGOMS");	// 6
        th(   "BGRENG");	// 6
        th(    "BGYAD");	// 6
        th(  "BGYANGS");	// 6
        th(    "BHANG");	// 6
        th(     "BHIS");	// 6
        th(    "BH'Um");	// 6
        th("BIGHN'ANTA");	// 6
        th(    "BKLOG");	// 6
        th(      "BKO");	// 6
        th(     "BKOR");	// 6
        th(   "BKRI'O");	// 6
        th(    "BKRUM");	// 6
        th(    "BKRUN");	// 6
        th(    "BKYES");	// 6
        th(    "BLABS");	// 6
        th(     "BLOD");	// 6
        th(    "BNYEN");	// 6
        th(    "'BRAB");	// 6
        th(    "'BRAG");	// 6
        th(    "BRA'O");	// 6
        th(    "BRDOD");	// 6
        th(    "BRGAD");	// 6
        th(   "BRNABS");	// 6
        th(  "'BRONGS");	// 6
        th(   "BRTZIN");	// 6
        th(   "'BRU'O");	// 6
        th(       "BS");	// 6
        th(     "BSES");	// 6
        th(    "BSGOm");	// 6
        th(    "BSGON");	// 6
        th(   "BSGRAD");	// 6
        th(    "BSGYU");	// 6
        th(   "BSKOMS");	// 6
        th(  "BSKYAMS");	// 6
        th(  "BSKYENG");	// 6
        th(    "BSNAL");	// 6
        th(   "BSNGUS");	// 6
        th(   "BSNYAS");	// 6
        th(   "BSNYIN");	// 6
        th(     "BSOM");	// 6
        th(     "BSPU");	// 6
        th(    "BSRAG");	// 6
        th(   "BSRO'O");	// 6
        th(    "BSRUL");	// 6
        th(    "BSTER");	// 6
        th(    "BSTOS");	// 6
        th(     "BSUD");	// 6
        th(      "BTA");	// 6
        th(     "BTZA");	// 6
        th(    "BTZOR");	// 6
        th(    "BUDHA");	// 6
        th(    "'BYUN");	// 6
        th(     "BZAL");	// 6
        th(    "BZHAL");	// 6
        th(   "BZHOGS");	// 6
        th(    "BZHUd");	// 6
        th(      "CEM");	// 6
        th(   "'CHA'I");	// 6
        th(    "'CHAN");	// 6
        th(    "CHIMS");	// 6
        th(    "CHO'I");	// 6
        th(    "^CHOS");	// 6
        th(     "CHUM");	// 6
        th(    "'CHUS");	// 6
        th(      "DA:");	// 6
        th(   "DA'ANG");	// 6
        th(      "dAM");	// 6
        th(    "DAndA");	// 6
        th(     "D'AS");	// 6
        th(     "DATA");	// 6
        th(    "DBA'I");	// 6
        th(    "DBOGS");	// 6
        th(    "DBRAD");	// 6
        th(     "DBUD");	// 6
        th(  "DBYE'AM");	// 6
        th( "DBYE'ANG");	// 6
        th(    "DBYIB");	// 6
        th(    "DDANG");	// 6
        th(    "DGREL");	// 6
        th(    "DGYAS");	// 6
        th(    "DHVAM");	// 6
        th(     "dI'I");	// 6
        th(   "d'IKKA");	// 6
        th(    "DKANG");	// 6
        th(    "DKROG");	// 6
        th(     "DKUN");	// 6
        th(    "DKYES");	// 6
        th(   "DKYOGS");	// 6
        th(     "DMIS");	// 6
        th(      "DNA");	// 6
        th(    "DO'AM");	// 6
        th(  "DPE'ANG");	// 6
        th(    "DPE'U");	// 6
        th(   "DPHYID");	// 6
        th(    "'DRAB");	// 6
        th(     "DRAM");	// 6
        th(   "'DRING");	// 6
        th(    "'DROB");	// 6
        th(    "'DRUG");	// 6
        th(     "'DUB");	// 6
        th(   "DUMW'A");	// 6
        th(     "DZAB");	// 6
        th(    "DZANG");	// 6
        th(   "DZKshA");	// 6
        th(  "DZNY'AN");	// 6
        th(    "DZY'A");	// 6
        th(      "GA'");	// 6
        th(  "GAndA'I");	// 6
        th("GANG-GA'I");	// 6
        th(  "GANGGAR");	// 6
        th(  "GARBHAS");	// 6
        th(    "GCENG");	// 6
        th(   "GCINGS");	// 6
        th(     "GCUR");	// 6
        th(     "GDAR");	// 6
        th(     "'GEM");	// 6
        th(     "GHON");	// 6
        th(     "GLED");	// 6
        th(     "GLEG");	// 6
        th(     "GNAR");	// 6
        th(    "GNYIR");	// 6
        th(     "'GON");	// 6
        th(     "GRAD");	// 6
        th(   "'GRA'I");	// 6
        th(  "GRE'U'I");	// 6
        th(   "GRTAGS");	// 6
        th(    "'GRUM");	// 6
        th(   "GRUNGS");	// 6
        th(     "GSAS");	// 6
        th(    "GSHAN");	// 6
        th(    "GSHEd");	// 6
        th(   "GSRUNG");	// 6
        th(      "GTA");	// 6
        th(    "GTENG");	// 6
        th(   "GTONGS");	// 6
        th(   "GTZTSA");	// 6
        th(       "GY");	// 6
        th(     "GYAB");	// 6
        th(    "G-YAL");	// 6
        th(    "GYI'I");	// 6
        th(     "GYIM");	// 6
        th(     "GZHO");	// 6
        th(    "GZHOD");	// 6
        th(        "H");	// 6
        th(      "HER");	// 6
        th(      "HMA");	// 6
        th(  "HR'I:HA");	// 6
        th("HR'I:shtR'I:");	// 6
        th(     "HRUD");	// 6
        th(    "'JABS");	// 6
        th(      "JAS");	// 6
        th(      "JID");	// 6
        th(     "'JIt");	// 6
        th(     "'JOD");	// 6
        th(      "JOS");	// 6
        th(    "'JUGO");	// 6
        th(     "JUNG");	// 6
        th(    "'JU'O");	// 6
        th(  "KARnA'I");	// 6
        th(     "KHAn");	// 6
        th(    "KHE'U");	// 6
        th(    "'KHRA");	// 6
        th(   "'KHRAL");	// 6
        th(    "KHRAN");	// 6
        th(   "KHREGS");	// 6
        th(    "KHRIN");	// 6
        th(   "KHROMS");	// 6
        th(   "KHU'AM");	// 6
        th(   "'KHYAB");	// 6
        th(    "KHYAL");	// 6
        th(  "KHYONGS");	// 6
        th(  "'KHYUGS");	// 6
        th(      "KLI");	// 6
        th(     "KLUB");	// 6
        th(     "KLUD");	// 6
        th( "KONYAHAm");	// 6
        th(   "KOOndA");	// 6
        th(     "'KOR");	// 6
        th(     "KRAL");	// 6
        th(     "KRIG");	// 6
        th(    "KTA'O");	// 6
        th(     "KYA'");	// 6
        th(     "KYEN");	// 6
        th(      "LA'");	// 6
        th(     "LAMS");	// 6
        th(    "LANDA");	// 6
        th(    "LCI'O");	// 6
        th(     "LEBS");	// 6
        th(     "LHEM");	// 6
        th(    "LPANG");	// 6
        th(       "LT");	// 6
        th(   "LTONGS");	// 6
        th(     "LU'O");	// 6
        th(      "LWA");	// 6
        th(        "m");	// 6
        th(   "MANDHA");	// 6
        th(    "MASTU");	// 6
        th(    "MCHIR");	// 6
        th(    "MDZAR");	// 6
        th(      "^MI");	// 6
        th(     "MJES");	// 6
        th(     "MJOS");	// 6
        th(    "MKHAD");	// 6
        th(     "MKHE");	// 6
        th(    "MKHON");	// 6
        th(   "MKHYID");	// 6
        th(   "MKHYIN");	// 6
        th(      "M'O");	// 6
        th(   "MRNYED");	// 6
        th(      "MSO");	// 6
        th(    "MTAHR");	// 6
        th(   "MTHA'A");	// 6
        th(    "MTSOM");	// 6
        th(   "MTSONG");	// 6
        th(  "MTSONGS");	// 6
        th(     "MUNG");	// 6
        th(     "MYAD");	// 6
        th(   "MZHUGS");	// 6
        th(      "NA2");	// 6
        th(   "NA'I'O");	// 6
        th(      "nAS");	// 6
        th(     "N'AS");	// 6
        th(      "N'E");	// 6
        th(     "NGUD");	// 6
        th(       "ni");	// 6
        th(      "^NI");	// 6
        th(    "N'I'I");	// 6
        th(      "NSU");	// 6
        th(      "NVA");	// 6
        th(    "NYAmS");	// 6
        th(    "N+YAS");	// 6
        th(   "N+YASA");	// 6
        th(      "nYE");	// 6
        th(     "NYOB");	// 6
        th(      "PAB");	// 6
        th(  "P'ADYAm");	// 6
        th(    "'PAGS");	// 6
        th(   "PAn-dI");	// 6
        th(    "PANdI");	// 6
        th(    "P'ANG");	// 6
        th(       "PH");	// 6
        th(     "'PHA");	// 6
        th(   "PHO'AM");	// 6
        th(    "PHRAB");	// 6
        th(   "PHUNGS");	// 6
        th(    "PHU'U");	// 6
        th( "'PHYA'AM");	// 6
        th(    "PHYAB");	// 6
        th(   "PHYIRO");	// 6
        th(    "PNGOM");	// 6
        th(     "P''O");	// 6
        th(      "PPA");	// 6
        th(     "PRAG");	// 6
        th( "PRATZTSA");	// 6
        th(     "PROS");	// 6
        th(      "PTA");	// 6
        th(    "PU'AM");	// 6
        th(     "R'A:");	// 6
        th(      "RAK");	// 6
        th(    "RANTA");	// 6
        th(    "RATNE");	// 6
        th(     "RBHA");	// 6
        th(      "RBO");	// 6
        th(     "RDAG");	// 6
        th(     "RDAM");	// 6
        th(     "RDAS");	// 6
        th(   "RDO'AM");	// 6
        th(     "RDUN");	// 6
        th(    "RDZUB");	// 6
        th(   "RDZU'I");	// 6
        th(      "REE");	// 6
        th(     "RGOS");	// 6
        th(      "RGY");	// 6
        th(   "'RGYUR");	// 6
        th(     "RIBS");	// 6
        th(     "RJUD");	// 6
        th(     "RKAL");	// 6
        th(     "RKAM");	// 6
        th(    "RKYAD");	// 6
        th(      "RLU");	// 6
        th(    "RMIGS");	// 6
        th(     "RNAN");	// 6
        th(     "RNOB");	// 6
        th(  "RNY-DZA");	// 6
        th(     "RTAL");	// 6
        th(     "RTED");	// 6
        th(     "RTOS");	// 6
        th( "RTZI'ANG");	// 6
        th(   "SANGGA");	// 6
        th(   "SATV'A");	// 6
        th("S'ATZTSA'I");	// 6
        th(     "SBRU");	// 6
        th(   "SBYING");	// 6
        th(   "SBYOMS");	// 6
        th(    "SBYUG");	// 6
        th(     "SDIB");	// 6
        th(      "SEB");	// 6
        th( "SENG-GES");	// 6
        th( "SENG+GES");	// 6
        th(      "SGI");	// 6
        th(      "SGR");	// 6
        th(    "SGRAB");	// 6
        th(   "SGRUGS");	// 6
        th(    "SGYES");	// 6
        th(    "SGYIN");	// 6
        th(   "SGYU'O");	// 6
        th(       "sh");	// 6
        th(      "sHA");	// 6
        th(     "SHA:");	// 6
        th(  "SHA'ANG");	// 6
        th(    "SHABS");	// 6
        th(     "shAm");	// 6
        th(    "SH'AM");	// 6
        th(   "SHANTA");	// 6
        th(     "SHAt");	// 6
        th(     "sh'I");	// 6
        th(   "SH'ING");	// 6
        th(   "SHINGS");	// 6
        th(    "SHO'U");	// 6
        th("SHR'ADDH'A");	// 6
        th(     "SHRE");	// 6
        th(      "SHS");	// 6
        th(  "SHVASTA");	// 6
        th(     "SHVE");	// 6
        th(     "SI'O");	// 6
        th(     "SKEL");	// 6
        th(     "SKUG");	// 6
        th(     "SLOM");	// 6
        th(     "SLOR");	// 6
        th(     "SMED");	// 6
        th(     "SMEN");	// 6
        th(      "SMO");	// 6
        th(     "SMYA");	// 6
        th(    "SMYAG");	// 6
        th(    "SNGAd");	// 6
        th(    "SNGAD");	// 6
        th(    "SNGUG");	// 6
        th(      "SNO");	// 6
        th(     "SNUB");	// 6
        th(    "SNYAm");	// 6
        th(     "SOOR");	// 6
        th(    "SPE'U");	// 6
        th(    "SPRAG");	// 6
        th(  "SPRANGS");	// 6
        th(    "SPRUS");	// 6
        th(    "SPUGS");	// 6
        th(   "SPYING");	// 6
        th(     "SREN");	// 6
        th(    "SRI'I");	// 6
        th(     "SROB");	// 6
        th(   "SRONGS");	// 6
        th(     "STAS");	// 6
        th(     "STIG");	// 6
        th(    "STIGS");	// 6
        th(   "STZANG");	// 6
        th(     "SUBS");	// 6
        th(    "S+Y'A");	// 6
        th(     "SYAT");	// 6
        th(   "TA-DYA");	// 6
        th(    "tAKKI");	// 6
        th(      "TAL");	// 6
        th(     "T'Am");	// 6
        th(     "T'AR");	// 6
        th(      "TAT");	// 6
        th(    "'THED");	// 6
        th(     "THOt");	// 6
        th(    "'THUB");	// 6
        th( "T'ITZTSA");	// 6
        th(      "TOS");	// 6
        th(       "TS");	// 6
        th(    "TSVAR");	// 6
        th(    "TZAKA");	// 6
        th(   "TZAMPA");	// 6
        th(   "TZANdA");	// 6
        th(  "TZARYAS");	// 6
        th(    "TZ'AS");	// 6
        th(  "TZTSA'I");	// 6
        th(    "TZTSU");	// 6
        th(     "TZUL");	// 6
        th(       "WO");	// 6
        th(     "YAGN");	// 6
        th(     "Y'Am");	// 6
        th(     "Y'AM");	// 6
        th(     "YINO");	// 6
        th(       "YN");	// 6
        th(     "YO'I");	// 6
        th(    "Y'T'O");	// 6
        th(      "YUm");	// 6
        th(     "ZH'A");	// 6
        th(    "ZHIGN");	// 6
        th(    "ZHO'O");	// 6
        th(   "ZHU'AM");	// 6
        th(      "132");	// 7
        th(      "193");	// 7
        th(     "1988");	// 7
        th(      "325");	// 7
        th(      "385");	// 7
        th(      "472");	// 7
        th(       "69");	// 7
        th(       "71");	// 7
        th(      "7DE");	// 7
        th(       "84");	// 7
        th(       "94");	// 7
        th("A'A:BADZRA");	// 7
        th(    "AAMBA");	// 7
        th( "A'ANYTZA");	// 7
        th(  "AARKA'I");	// 7
        th(   "AARTHA");	// 7
        th(     "A'AS");	// 7
        th(     "AAtA");	// 7
        th(     "AENG");	// 7
        th(     "ANDA");	// 7
        th(     "ANGS");	// 7
        th(    "AOO'O");	// 7
        th(      "APA");	// 7
        th(      "'AR");	// 7
        th(      "A'R");	// 7
        th(   "A'R-YA");	// 7
        th(      "'BA");	// 7
        th(    "BA'Am");	// 7
        th(   "BANDHU");	// 7
        th(    "BAndI");	// 7
        th(  "BANYTZA");	// 7
        th(     "B'AR");	// 7
        th(       "BC");	// 7
        th(    "BCHOD");	// 7
        th(     "BDAD");	// 7
        th(     "BDEB");	// 7
        th(     "BDOD");	// 7
        th(    "BDOGS");	// 7
        th(     "BDUL");	// 7
        th("BEEd'UR+YA'I");	// 7
        th(      "BEN");	// 7
        th(    "BGREL");	// 7
        th(     "BHAG");	// 7
        th(     "BHIN");	// 7
        th(   "BHYA'I");	// 7
        th("BHYA:SV'AH'A");	// 7
        th(    "BINDU");	// 7
        th(    "BIR'U");	// 7
        th(    "BKRUR");	// 7
        th(   "BKYE'O");	// 7
        th(    "BLTAD");	// 7
        th(    "BLUNG");	// 7
        th(     "BNAS");	// 7
        th(     "BO'U");	// 7
        th(     "BRIL");	// 7
        th(   "BRiTTA");	// 7
        th(    "BRKOD");	// 7
        th(   "BRTANG");	// 7
        th(  "BRTZEMS");	// 7
        th(  "BRTZI'I");	// 7
        th(    "BRUNG");	// 7
        th(   "BRUNGS");	// 7
        th(    "BSDUL");	// 7
        th(    "BSEMS");	// 7
        th(    "BSGAL");	// 7
        th(    "BSGEM");	// 7
        th(  "BSGO'AM");	// 7
        th(   "BSGROL");	// 7
        th( "BSGRUNGS");	// 7
        th( "BSGYANGS");	// 7
        th(   "BSGYAR");	// 7
        th(    "BSHUG");	// 7
        th(  "BSKROGS");	// 7
        th(   "BSNYAM");	// 7
        th(  "BSNYUGS");	// 7
        th(    "BSTAS");	// 7
        th(    "BSTED");	// 7
        th(   "BSTUMS");	// 7
        th(   "BSTZOL");	// 7
        th(    "BSUDS");	// 7
        th(     "BTAd");	// 7
        th(     "BTUN");	// 7
        th(     "'BUG");	// 7
        th(    "BYI'O");	// 7
        th(   "BYUNGS");	// 7
        th(    "BZLAR");	// 7
        th(      "CAB");	// 7
        th(     "'CAD");	// 7
        th(    "'CHAS");	// 7
        th(     "CHEG");	// 7
        th(    "'CHIB");	// 7
        th(   "'CHIBS");	// 7
        th(    "'CHIG");	// 7
        th(    "CHUMS");	// 7
        th(      "CUB");	// 7
        th(     "DAG'");	// 7
        th(   "DAKshA");	// 7
        th(   "'DANGS");	// 7
        th(    "DANTA");	// 7
        th(   "DARPAN");	// 7
        th(     "DBAD");	// 7
        th(    "DBO'I");	// 7
        th(     "DDHI");	// 7
        th(     "DGEN");	// 7
        th(   "DGRA'O");	// 7
        th(   "DGRONG");	// 7
        th(  "DGRONGS");	// 7
        th(   "DHARME");	// 7
        th(    "DHE'I");	// 7
        th(    "DKRIR");	// 7
        th(    "DKYIS");	// 7
        th(     "DKYU");	// 7
        th(    "DMGIS");	// 7
        th(     "DMIt");	// 7
        th(    "DMUGS");	// 7
        th(    "DNGUS");	// 7
        th(     "DPEL");	// 7
        th(    "DPYED");	// 7
        th(    "DRI'U");	// 7
        th(    "DROGS");	// 7
        th(     "DSAL");	// 7
        th(    "DUNGS");	// 7
        th("d'UR+YA'I");	// 7
        th(     "DZAD");	// 7
        th(     "DZES");	// 7
        th(    "'DZOD");	// 7
        th(    "'DZUR");	// 7
        th(   "GA'O'I");	// 7
        th(     "G'AS");	// 7
        th( "GA-SHEGS");	// 7
        th(   "GA-YAM");	// 7
        th(     "GCAS");	// 7
        th(    "GCIGS");	// 7
        th(      "GGI");	// 7
        th(  "GHOsh'A");	// 7
        th(     "GHRI");	// 7
        th(      "'GI");	// 7
        th(     "GLES");	// 7
        th(    "GLOGS");	// 7
        th(    "GMIGS");	// 7
        th(   "GNONGS");	// 7
        th(     "GR'A");	// 7
        th(   "'GRAGS");	// 7
        th(    "'GREN");	// 7
        th(    "GRI'U");	// 7
        th(   "'GRUMS");	// 7
        th(     "GRWA");	// 7
        th(    "GSEGS");	// 7
        th(     "GSHI");	// 7
        th(   "GSHIGS");	// 7
        th(     "GTEN");	// 7
        th(     "GTIB");	// 7
        th(     "GUSM");	// 7
        th(     "GYEL");	// 7
        th(     "G+YO");	// 7
        th(    "'GYOG");	// 7
        th(     "GZAD");	// 7
        th(     "GZED");	// 7
        th(  "HANTI'I");	// 7
        th(      "HOL");	// 7
        th(      "HOM");	// 7
        th(     "HV'A");	// 7
        th(    "HVONG");	// 7
        th(       "IN");	// 7
        th(    "'JEMS");	// 7
        th(    "'JIBS");	// 7
        th(      "KAB");	// 7
        th(    "KARtA");	// 7
        th(    "KAshA");	// 7
        th("KHRI'U'AM");	// 7
        th(    "KHROR");	// 7
        th(   "'KHRUN");	// 7
        th(    "KHYAM");	// 7
        th(    "KHYOM");	// 7
        th(   "KHYONG");	// 7
        th(   "KIRTTI");	// 7
        th(     "KO'O");	// 7
        th(     "KRAS");	// 7
        th(    "KSYES");	// 7
        th(    "LDAGS");	// 7
        th(    "LDEMS");	// 7
        th(     "LDIS");	// 7
        th(    "LENGS");	// 7
        th(    "LGANG");	// 7
        th(     "LHUM");	// 7
        th(     "LJAB");	// 7
        th(      "LJI");	// 7
        th(    "LKOGS");	// 7
        th(      "LNG");	// 7
        th(      "LUL");	// 7
        th(  "LUMBI'I");	// 7
        th(     "M'Am");	// 7
        th(  "MDO'ANG");	// 7
        th(   "MKHANG");	// 7
        th(   "MKHYON");	// 7
        th(     "MNAB");	// 7
        th(    "MNONG");	// 7
        th(     "MNUS");	// 7
        th(     "MOGS");	// 7
        th("MOONGGALA");	// 7
        th(  "MRDZOGS");	// 7
        th(    "MTHAN");	// 7
        th(    "MTHUB");	// 7
        th(   "MTSIMS");	// 7
        th(      "MU:");	// 7
        th(      "MUL");	// 7
        th(      "nAM");	// 7
        th(  "NAMKHA'");	// 7
        th(    "'NGA'");	// 7
        th(     "NGAB");	// 7
        th(     "NGOL");	// 7
        th(    "N+Y'A");	// 7
        th(     "NYDI");	// 7
        th(    "NYDZU");	// 7
        th(      "'OB");	// 7
        th(       "ON");	// 7
        th(  "PAD-MAS");	// 7
        th( "PAD-MO'I");	// 7
        th(  "PHA'ANG");	// 7
        th(    "'PHAB");	// 7
        th(     "PHAm");	// 7
        th(   "PHENGS");	// 7
        th(   "PHRA'U");	// 7
        th(    "'PHRE");	// 7
        th(    "'PHRU");	// 7
        th(    "PHYAN");	// 7
        th(    "PHYEM");	// 7
        th(    "PHYOL");	// 7
        th(    "PHYON");	// 7
        th(   "'PHYOS");	// 7
        th(  "PING-KA");	// 7
        th(    "PRA'I");	// 7
        th(    "PRANG");	// 7
        th(      "PUG");	// 7
        th(    "PUnYA");	// 7
        th(   "P'URnA");	// 7
        th(  "PUshtIm");	// 7
        th(    "RAndA");	// 7
        th(   "RATN'A");	// 7
        th(      "RBU");	// 7
        th(     "RDON");	// 7
        th(    "RDONG");	// 7
        th(     "RJIS");	// 7
        th(     "RJOB");	// 7
        th(    "RLOGS");	// 7
        th(    "RM'A:");	// 7
        th(    "RNGAR");	// 7
        th(    "RNGUM");	// 7
        th(   "RNGU'O");	// 7
        th(     "RNOS");	// 7
        th(     "RTAM");	// 7
        th(   "RTZVAS");	// 7
        th(    "RU'AM");	// 7
        th(    "SAMBA");	// 7
        th(     "SANS");	// 7
        th(    "SBENG");	// 7
        th(      "SBI");	// 7
        th(   "SBREBS");	// 7
        th(   "SGONGS");	// 7
        th(    "SGREG");	// 7
        th(    "SGROB");	// 7
        th(    "SGYAR");	// 7
        th(    "SHAKA");	// 7
        th(    "sh'Am");	// 7
        th(  "SHAMBHI");	// 7
        th( "SH'ANTIm");	// 7
        th(   "SHARMA");	// 7
        th(    "SH'KA");	// 7
        th(    "SHR'A");	// 7
        th(    "SHRAD");	// 7
        th(  "SHUDDHE");	// 7
        th(     "SHUM");	// 7
        th(    "SHVAR");	// 7
        th(  "SIDHASA");	// 7
        th(      "SIm");	// 7
        th(    "SKBAS");	// 7
        th(     "^SKU");	// 7
        th(   "SKYENG");	// 7
        th(   "SKYING");	// 7
        th(   "SLA'AM");	// 7
        th(    "SLENG");	// 7
        th(     "SLOS");	// 7
        th(     "SLUR");	// 7
        th(    "SMIGS");	// 7
        th(   "SNGAMS");	// 7
        th(    "SNGES");	// 7
        th(   "SNGO'I");	// 7
        th(   "SNYUGS");	// 7
        th(      "SOB");	// 7
        th(     "SPHU");	// 7
        th(     "SPIN");	// 7
        th(     "SRAM");	// 7
        th(     "STAD");	// 7
        th(  "STE'ANG");	// 7
        th(     "STOM");	// 7
        th(     "STVA");	// 7
        th(   "S'URYA");	// 7
        th(      "TA'");	// 7
        th(      "T'E");	// 7
        th(   "'THAGS");	// 7
        th(   "'THING");	// 7
        th(     "'THO");	// 7
        th(    "THU'O");	// 7
        th(    "TSENG");	// 7
        th(    "'TSIM");	// 7
        th(     "TSOB");	// 7
        th(     "TSOt");	// 7
        th(      "TUB");	// 7
        th(     "TUSL");	// 7
        th(   "TZATSA");	// 7
        th(     "TZEN");	// 7
        th(    "TZI'U");	// 7
        th(    "TZRYA");	// 7
        th(    "TZUGS");	// 7
        th(  "WARMASA");	// 7
        th(     "Y'AT");	// 7
        th(      "Y'I");	// 7
        th(     "YU'U");	// 7
        th(      "ZEG");	// 7
        th(       "ZH");	// 7
        th(     "ZHIL");	// 7
        th(    "ZI'AM");	// 7
        th(     "ZIGS");	// 7
        th(      "ZLU");	// 7
        th(       ":-");	// 8
        th(      "102");	// 8
        th(      "110");	// 8
        th(      "127");	// 8
        th(      "133");	// 8
        th(      "200");	// 8
        th(       "54");	// 8
        th(       "58");	// 8
        th(       "61");	// 8
        th(       "67");	// 8
        th(       "77");	// 8
        th(    "7'JAM");	// 8
        th(       "80");	// 8
        th(       "81");	// 8
        th(       "82");	// 8
        th(       "88");	// 8
        th(       "96");	// 8
        th(      "AAT");	// 8
        th(    "A'ATA");	// 8
        th(       "Ai");	// 8
        th(     "AI'I");	// 8
        th(      "AIn");	// 8
        th(   "AINTRA");	// 8
        th(     "AI'O");	// 8
        th(     "AMBA");	// 8
        th(    "'ANGS");	// 8
        th(     "ANTA");	// 8
        th(      "AOL");	// 8
        th(    "AUTRA");	// 8
        th(    "BAKKU");	// 8
        th(    "BCHOS");	// 8
        th(     "BCIG");	// 8
        th(     "BCWA");	// 8
        th(      "BDI");	// 8
        th(     "BDOS");	// 8
        th(  "BDZRA'I");	// 8
        th(    "BDZRE");	// 8
        th(     "BENG");	// 8
        th(    "BE'US");	// 8
        th(     "BGON");	// 8
        th(    "BGRUS");	// 8
        th(    "BGYED");	// 8
        th(    "BHA'I");	// 8
        th(  "BHIKshU");	// 8
        th(     "BHRI");	// 8
        th(   "BHY'Am");	// 8
        th(     "BJOD");	// 8
        th(    "BKRAN");	// 8
        th(    "BKRUB");	// 8
        th(   "BLDABS");	// 8
        th(    "'BRIN");	// 8
        th(     "BRnA");	// 8
        th(    "BRNAL");	// 8
        th(     "BROG");	// 8
        th(    "BRTEG");	// 8
        th(   "BRTZEG");	// 8
        th(    "BRUBS");	// 8
        th(      "BSA");	// 8
        th(     "BSAd");	// 8
        th(   "BSANGL");	// 8
        th(   "BSDO'O");	// 8
        th(    "BSGRA");	// 8
        th(    "BSKUN");	// 8
        th(   "BSLEBS");	// 8
        th(  "BSNYOMS");	// 8
        th(   "BSROGS");	// 8
        th(    "BSROS");	// 8
        th(   "BSTUNG");	// 8
        th(     "BTAR");	// 8
        th(     "BTEN");	// 8
        th(    "BTUGS");	// 8
        th(     "BTZU");	// 8
        th(     "'BYI");	// 8
        th(     "BYIM");	// 8
        th(    "BYONG");	// 8
        th(    "'BYUD");	// 8
        th(   "BZHABS");	// 8
        th(  "BZHINGS");	// 8
        th(    "BZHOD");	// 8
        th(   "BZLUGS");	// 8
        th(  "'CHAGSO");	// 8
        th(  "'CHE'AM");	// 8
        th(    "'CHEN");	// 8
        th(  "'CHI'AM");	// 8
        th(    "CHIGS");	// 8
        th(   "'CHIMS");	// 8
        th(  "'CHONGS");	// 8
        th(  "DARPANA");	// 8
        th(     "DBRA");	// 8
        th(   "DBRI'O");	// 8
        th(   "DBYEBS");	// 8
        th(      "DEE");	// 8
        th(     "'DIG");	// 8
        th(   "DKRONG");	// 8
        th(  "DKRONGS");	// 8
        th(   "DKRUNG");	// 8
        th(      "DMI");	// 8
        th(       "dO");	// 8
        th(     "DPAD");	// 8
        th(     "DPAN");	// 8
        th(    "DPROG");	// 8
        th(    "'DRA'");	// 8
        th(   "DSPYOD");	// 8
        th(    "'DUGS");	// 8
        th(   "DU'ONG");	// 8
        th(  "d'URYAS");	// 8
        th(    "'DZOG");	// 8
        th(   "GARBHE");	// 8
        th(  "G'ARDZU");	// 8
        th(     "GDEN");	// 8
        th(    "GHA'I");	// 8
        th(    "GNGAS");	// 8
        th(      "GNO");	// 8
        th(    "GNYAG");	// 8
        th(       "GR");	// 8
        th(     "GREE");	// 8
        th(    "GREGS");	// 8
        th(    "GSHUL");	// 8
        th(     "GSUD");	// 8
        th(     "GSUL");	// 8
        th(     "GTAB");	// 8
        th(     "G-YI");	// 8
        th(    "G-YOL");	// 8
        th(     "GZId");	// 8
        th(        "h");	// 8
        th(      "HON");	// 8
        th(     "HO'U");	// 8
        th(     "HRI:");	// 8
        th(     "HRIB");	// 8
        th(      "H'U");	// 8
        th(      "HUL");	// 8
        th(      "HVE");	// 8
        th(      "'JA");	// 8
        th(    "'JEGS");	// 8
        th(    "'JI'I");	// 8
        th(     "'JON");	// 8
        th(      "KEN");	// 8
        th(   "'KHRAS");	// 8
        th(   "KHRING");	// 8
        th(  "'KHRUMS");	// 8
        th(   "KHVA'I");	// 8
        th(   "'KHYOD");	// 8
        th(  "'KHYONG");	// 8
        th(      "KRU");	// 8
        th(      "KUB");	// 8
        th(   "KYANGS");	// 8
        th(   "LAKshA");	// 8
        th(      "LAL");	// 8
        th(     "LBUR");	// 8
        th(     "LCEB");	// 8
        th(    "LCE'O");	// 8
        th(     "LCIB");	// 8
        th(    "LCIGS");	// 8
        th(    "LDE'I");	// 8
        th(     "LGAS");	// 8
        th(    "LHVAM");	// 8
        th(    "LNGAG");	// 8
        th(      "LPA");	// 8
        th(    "LSONG");	// 8
        th(    "LUMBI");	// 8
        th(     "MAnI");	// 8
        th(     "MDEL");	// 8
        th(      "MDU");	// 8
        th(   "MERGEN");	// 8
        th(     "MNAG");	// 8
        th(     "MNYU");	// 8
        th(    "MRJOD");	// 8
        th(  "MTHU'AM");	// 8
        th(    "MTSA'");	// 8
        th(    "MTSIM");	// 8
        th( "MTSO'ANG");	// 8
        th(  "MUTZTSA");	// 8
        th(    "N'A'I");	// 8
        th(     "NG'A");	// 8
        th(       "nO");	// 8
        th(    "NYE'U");	// 8
        th(   "NYI'AM");	// 8
        th(     "NYIG");	// 8
        th( "PAD+MA'I");	// 8
        th(  "PADMA'O");	// 8
        th(     "PAGE");	// 8
        th(    "PANGS");	// 8
        th(      "PAT");	// 8
        th(     "PH'A");	// 8
        th(    "'PHAt");	// 8
        th(     "'PHE");	// 8
        th(    "'PHOD");	// 8
        th(    "'PHOR");	// 8
        th(   "PHRA'O");	// 8
        th(   "'PHREN");	// 8
        th(   "'PHRES");	// 8
        th(    "PHROD");	// 8
        th(  "'PHRO'I");	// 8
        th(  "'PHRONG");	// 8
        th(    "PHROS");	// 8
        th(    "'PHUG");	// 8
        th(   "'PHYAD");	// 8
        th(    "PHYIL");	// 8
        th(  "'PHYING");	// 8
        th(    "PHYOS");	// 8
        th( "PRADZNYA");	// 8
        th(     "PRIS");	// 8
        th(   "PUshPA");	// 8
        th(    "PUSTA");	// 8
        th(      "PYI");	// 8
        th(   "RAKSHA");	// 8
        th(   "RANDHA");	// 8
        th(    "RDOGS");	// 8
        th(    "RDZIM");	// 8
        th(    "RGANG");	// 8
        th(    "RGA'O");	// 8
        th(      "RiT");	// 8
        th(     "^RJE");	// 8
        th(  "RJE'U'I");	// 8
        th(     "RMAL");	// 8
        th(     "RMAN");	// 8
        th(    "RNGOS");	// 8
        th(    "RNGUR");	// 8
        th(   "RNYDZA");	// 8
        th(    "RNYES");	// 8
        th(    "RNYOD");	// 8
        th(      "ROB");	// 8
        th(   "RTZE'O");	// 8
        th("SAMYAKSAm");	// 8
        th(   "SARBBA");	// 8
        th(    "SBRAL");	// 8
        th(    "SBYER");	// 8
        th(   "SDE'AM");	// 8
        th(     "SDED");	// 8
        th(    "SGABS");	// 8
        th(     "SGAD");	// 8
        th(     "SGOL");	// 8
        th(    "SGRAM");	// 8
        th(     "SGRI");	// 8
        th(   "SGYE'U");	// 8
        th(    "SHAMS");	// 8
        th(   "SHINTA");	// 8
        th(   "SHONGS");	// 8
        th(   "SHRA'I");	// 8
        th(     "SI'U");	// 8
        th(     "SKRO");	// 8
        th(    "SKRUB");	// 8
        th(   "SKU'AM");	// 8
        th(    "SKYOM");	// 8
        th(     "SLUB");	// 8
        th(    "SMA'I");	// 8
        th(    "SNYAG");	// 8
        th(    "SPENG");	// 8
        th(  "SPRENGS");	// 8
        th(  "SPRE'US");	// 8
        th(     "SPRI");	// 8
        th(    "SPROL");	// 8
        th(     "SPYA");	// 8
        th(    "SRAGS");	// 8
        th(     "^STE");	// 8
        th(   "STE'US");	// 8
        th(    "'THAR");	// 8
        th(     "TH'O");	// 8
        th(    "'THOD");	// 8
        th(   "'THUMS");	// 8
        th(      "TI:");	// 8
        th(  "tIKKA'I");	// 8
        th(       "tO");	// 8
        th(     "TS'A");	// 8
        th(     "'TSI");	// 8
        th(     "TSID");	// 8
        th(     "TSIN");	// 8
        th(    "'TSUR");	// 8
        th(      "ttA");	// 8
        th(      "TU:");	// 8
        th(      "TUD");	// 8
        th(  "TZAKshU");	// 8
        th(     "TZIN");	// 8
        th(     "TZOS");	// 8
        th(     "TZUM");	// 8
        th(    "TZUNG");	// 8
        th(      "WAM");	// 8
        th(        "Y");	// 8
        th(    "Y'A'I");	// 8
        th(     "Y'TA");	// 8
        th(      "Y'U");	// 8
        th(     "ZEHS");	// 8
        th(      "ZEL");	// 8
        th(    "ZHUMS");	// 8
        th(      "ZIS");	// 8
        th(     "ZLAG");	// 8
        th(     "ZLAM");	// 8
        th(     "1980");	// 9
        th(       "56");	// 9
        th(       "63");	// 9
        th(       "65");	// 9
        th(       "68");	// 9
        th(       "74");	// 9
        th(       "78");	// 9
        th(     "7PHA");	// 9
        th(       "85");	// 9
        th(       "86");	// 9
        th(       "91");	// 9
        th(       "93");	// 9
        th(       "95");	// 9
        th(       "97");	// 9
        th(    "AASMA");	// 9
        th(      "AIR");	// 9
        th(  "AiTHI'I");	// 9
        th(       "Am");	// 9
        th(      "BA3");	// 9
        th(      "B'E");	// 9
        th(     "'BEM");	// 9
        th(     "BGAS");	// 9
        th(    "BGONG");	// 9
        th(   "BGYI'I");	// 9
        th(     "BKOG");	// 9
        th(    "BKRIN");	// 9
        th(    "BKUGS");	// 9
        th(     "BLAD");	// 9
        th(   "BLONGS");	// 9
        th(  "BLTA'AM");	// 9
        th(     "'BOB");	// 9
        th(    "'BRAM");	// 9
        th(  "'BRENGS");	// 9
        th("BRGYA'ANG");	// 9
        th(   "'BRIMS");	// 9
        th(     "BRKA");	// 9
        th(   "BRNGOS");	// 9
        th(    "'BROL");	// 9
        th(    "BSDUB");	// 9
        th(    "BSEGS");	// 9
        th(   "BSGAGS");	// 9
        th(    "BSLUB");	// 9
        th(   "BSNANG");	// 9
        th(   "BSNGOM");	// 9
        th(   "BSNYAG");	// 9
        th(  "BSNYIGS");	// 9
        th(     "BSOR");	// 9
        th(   "BSTOBS");	// 9
        th(   "BSUNGS");	// 9
        th(    "BTAMS");	// 9
        th(     "BTOL");	// 9
        th(    "BTSAN");	// 9
        th(     "BTZE");	// 9
        th(       "BY");	// 9
        th(     "BYAL");	// 9
        th(     "BYIG");	// 9
        th(     "BYUS");	// 9
        th(    "BZHAS");	// 9
        th(    "BZHUt");	// 9
        th(     "BZUD");	// 9
        th(     "DAGN");	// 9
        th(  "D'ARPAN");	// 9
        th(    "DBE'I");	// 9
        th(       "dE");	// 9
        th(     "DHUt");	// 9
        th(     "DHVE");	// 9
        th(     "'DI'");	// 9
        th(      "DNG");	// 9
        th(   "DNGOSU");	// 9
        th(      "DOd");	// 9
        th( "DPA'CHEN");	// 9
        th(     "'DRO");	// 9
        th(    "DRO'O");	// 9
        th(   "'DRUBS");	// 9
        th(       "ES");	// 9
        th("GANG+G'AR");	// 9
        th(  "GANYDZI");	// 9
        th(   "GA-YUR");	// 9
        th(     "GCAR");	// 9
        th(     "GCED");	// 9
        th(     "GLAR");	// 9
        th(      "GOB");	// 9
        th(  "'GRONGS");	// 9
        th(   "GSHUNG");	// 9
        th(     "GTON");	// 9
        th(    "GTZAD");	// 9
        th(    "GTZER");	// 9
        th(   "G-YAGS");	// 9
        th(   "'GYONG");	// 9
        th(    "'GYRU");	// 9
        th(    "GZHER");	// 9
        th(    "GZHUR");	// 9
        th(     "GZUM");	// 9
        th(    "HRING");	// 9
        th(      "JAL");	// 9
        th(     "'JAN");	// 9
        th(     "'JEG");	// 9
        th(      "KAD");	// 9
        th(     "KH'I");	// 9
        th(    "KHOGS");	// 9
        th(  "'KHREGS");	// 9
        th(    "KHRUD");	// 9
        th(    "KHU'O");	// 9
        th(     "KLAM");	// 9
        th(   "KLU'AM");	// 9
        th(      "KOB");	// 9
        th(      "KOD");	// 9
        th(     "KRiT");	// 9
        th(    "KRiTA");	// 9
        th(      "KUn");	// 9
        th(       "KY");	// 9
        th(    "KYING");	// 9
        th(     "KYOD");	// 9
        th(     "KYON");	// 9
        th(    "KYONG");	// 9
        th(    "L'ANG");	// 9
        th(    "LDABS");	// 9
        th(     "LDUG");	// 9
        th(      "LIM");	// 9
        th(     "LO'U");	// 9
        th(    "LTAGS");	// 9
        th(     "LTIR");	// 9
        th(     "LTUG");	// 9
        th(      "LUN");	// 9
        th(   "MAnI'I");	// 9
        th(    "MCHUD");	// 9
        th(     "MDAB");	// 9
        th(     "MDAL");	// 9
        th(   "MDZE'I");	// 9
        th(      "MEG");	// 9
        th(    "MGNON");	// 9
        th(   "MKHYED");	// 9
        th(    "MNAMS");	// 9
        th(   "MNANGS");	// 9
        th(    "MNYAL");	// 9
        th(    "MNYER");	// 9
        th(   "MUnd'I");	// 9
        th(     "MYON");	// 9
        th(      "NA'");	// 9
        th(      "NA4");	// 9
        th(      "NA6");	// 9
        th(      "n'I");	// 9
        th(     "NI'U");	// 9
        th(      "NUG");	// 9
        th(     "NYOD");	// 9
        th(     "NYOM");	// 9
        th(    "PADME");	// 9
        th(   "PAKSHI");	// 9
        th(    "'PHOB");	// 9
        th(   "'PHRAG");	// 9
        th(  "'PHUNGS");	// 9
        th("PHU:SV'AH'A");	// 9
        th( "'PHYANGS");	// 9
        th(    "PHYOR");	// 9
        th(      "PID");	// 9
        th( "PINGKA'I");	// 9
        th(     "PROG");	// 9
        th(    "RADNA");	// 9
        th(      "RDI");	// 9
        th(     "RDUG");	// 9
        th(    "RDZAD");	// 9
        th(   "RDZI'U");	// 9
        th(    "RGUNG");	// 9
        th(    "RGYA:");	// 9
        th(    "RKA'I");	// 9
        th(    "RKENG");	// 9
        th(    "RKYAG");	// 9
        th(    "RNGOR");	// 9
        th(   "RNGUBS");	// 9
        th(    "RTZAN");	// 9
        th(     "RY'A");	// 9
        th(   "R+YA'I");	// 9
        th(   "SAMBHU");	// 9
        th("SAMBUDDHI");	// 9
        th(    "SBOGS");	// 9
        th(    "SBRAD");	// 9
        th(    "SBRAR");	// 9
        th(     "SBUL");	// 9
        th(     "SBYA");	// 9
        th(    "SBYED");	// 9
        th(  "SBYINGS");	// 9
        th(     "SDAG");	// 9
        th(     "SDUL");	// 9
        th(    "SGA'I");	// 9
        th(     "SGOD");	// 9
        th(    "SGRAN");	// 9
        th(     "SHAK");	// 9
        th("SHA'KYA'I");	// 9
        th(     "shAS");	// 9
        th(     "SHIL");	// 9
        th(  "SH'IshA");	// 9
        th(    "SHOOL");	// 9
        th(     "SHRU");	// 9
        th(     "shtA");	// 9
        th(   "SHUDHE");	// 9
        th(     "shVA");	// 9
        th(   "SINGGA");	// 9
        th(    "SKRES");	// 9
        th(    "SMONG");	// 9
        th(     "SNES");	// 9
        th(   "^SNYAN");	// 9
        th(     "SO'O");	// 9
        th(     "SPAN");	// 9
        th(     "SRES");	// 9
        th(     "ST'A");	// 9
        th(    "STA'I");	// 9
        th(     "STAM");	// 9
        th(    "SURYA");	// 9
        th(      "TA:");	// 9
        th(    "TANTA");	// 9
        th(   "TE'U'I");	// 9
        th(     "'THA");	// 9
        th(    "'THOL");	// 9
        th(    "tIKAR");	// 9
        th(     "TONG");	// 9
        th(    "TU:'O");	// 9
        th(     "TVAm");	// 9
        th(    "T+Y'A");	// 9
        th(  "TZAM-PA");	// 9
        th(    "TZAMS");	// 9
        th(   "TZAndA");	// 9
        th(  "TZAR+YA");	// 9
        th(   "TZINTA");	// 9
        th(   "TZUNDA");	// 9
        th( "TZUNDA'I");	// 9
        th(      "'UB");	// 9
        th(     "WAGS");	// 9
        th(     "WA'O");	// 9
        th(    "WARnA");	// 9
    }

    /** Tests the tsheg bars that appear [10, 99] times. */
    public void test10to99() {
        th(      "138");	// 10
        th(       "52");	// 10
        th(       "60");	// 10
        th(       "62");	// 10
        th(       "64");	// 10
        th(       "72");	// 10
        th(       "76");	// 10
        th(       "83");	// 10
        th(       "87");	// 10
        th(       "90");	// 10
        th(  "A'A:LAS");	// 10
        th(    "AARYA");	// 10
        th(  "A'AR+YA");	// 10
        th(    "A'I'O");	// 10
        th(     "AO'O");	// 10
        th(    "A'RYA");	// 10
        th(     "AU'I");	// 10
        th(      "AUN");	// 10
        th(      "AUT");	// 10
        th(   "AU-TPA");	// 10
        th(      "BA1");	// 10
        th(     "BCOm");	// 10
        th(     "BCOR");	// 10
        th(      "BDA");	// 10
        th(    "BDAMS");	// 10
        th(  "BDZRASA");	// 10
        th(     "BEBS");	// 10
        th(     "BHRi");	// 10
        th(   "BIKshU");	// 10
        th(    "BIRYA");	// 10
        th(      "BLE");	// 10
        th(     "BLOG");	// 10
        th(     "'BOS");	// 10
        th(   "'BRA'I");	// 10
        th(    "BRDUD");	// 10
        th(   "'BRI'O");	// 10
        th(   "BRiTTI");	// 10
        th(   "BRKO'I");	// 10
        th(   "BRKYAL");	// 10
        th(     "BRTE");	// 10
        th(     "BSED");	// 10
        th(    "BSE'I");	// 10
        th(    "BSGUR");	// 10
        th(   "BSHUBS");	// 10
        th(    "BSKAR");	// 10
        th(    "BSKAS");	// 10
        th(    "BSKUD");	// 10
        th(   "BSKYID");	// 10
        th(    "BSLAR");	// 10
        th(    "BSNAD");	// 10
        th(   "BSNYED");	// 10
        th(    "BSRAS");	// 10
        th(     "BTAS");	// 10
        th(    "BTUBS");	// 10
        th(     "BTUM");	// 10
        th(   "BU'I'O");	// 10
        th(     "BYAB");	// 10
        th(     "BYUD");	// 10
        th(    "'BYUG");	// 10
        th(    "BZENG");	// 10
        th(    "BZHIB");	// 10
        th(    "BZHID");	// 10
        th(      "BZI");	// 10
        th(     "BZLO");	// 10
        th(     "BZOG");	// 10
        th(     "CAGS");	// 10
        th(  "CHE'O'I");	// 10
        th(    "'CHOG");	// 10
        th(    "'DA'I");	// 10
        th(     "DAMN");	// 10
        th(     "DA'O");	// 10
        th(    "DBAGS");	// 10
        th(    "DBYAS");	// 10
        th(     "DEGS");	// 10
        th(   "DGANGS");	// 10
        th(     "'DID");	// 10
        th(      "DIL");	// 10
        th(   "DKROGS");	// 10
        th(      "dOG");	// 10
        th(    "DONGS");	// 10
        th(   "DPE'AM");	// 10
        th(      "DPU");	// 10
        th(    "'DRAD");	// 10
        th(     "DRAR");	// 10
        th(     "DRID");	// 10
        th(    "'DROG");	// 10
        th(     "DUGS");	// 10
        th(  "DU:-KHA");	// 10
        th(    "'DUNG");	// 10
        th(     "DY'A");	// 10
        th(     "DZA'");	// 10
        th(     "DZAN");	// 10
        th(   "GANDHA");	// 10
        th(     "GCIR");	// 10
        th(     "GCUN");	// 10
        th(    "GDOGS");	// 10
        th(   "GDONGS");	// 10
        th(     "GENG");	// 10
        th(     "GNES");	// 10
        th(   "GNYISU");	// 10
        th(    "'GRIN");	// 10
        th(     "'GRU");	// 10
        th(    "GRUMS");	// 10
        th(    "GSHUB");	// 10
        th(     "GTAG");	// 10
        th(     "GTIG");	// 10
        th(    "GTZOG");	// 10
        th(  "GUPTASA");	// 10
        th(     "GYOL");	// 10
        th(     "GYUD");	// 10
        th(    "G-YUG");	// 10
        th(     "GYUS");	// 10
        th(    "GZHAD");	// 10
        th(   "GZHIBS");	// 10
        th(   "GZHU'O");	// 10
        th(     "GZUt");	// 10
        th(     "HE'O");	// 10
        th(      "HIS");	// 10
        th(     "HRUG");	// 10
        th(      "HUG");	// 10
        th(        "K");	// 10
        th(   "KHARSA");	// 10
        th(     "KHEG");	// 10
        th(    "KHE'I");	// 10
        th(  "'KHRIMS");	// 10
        th(    "KHYAS");	// 10
        th(   "KHYU'I");	// 10
        th(     "KO'I");	// 10
        th(     "KRAR");	// 10
        th(  "KRIshnA");	// 10
        th(     "KRMA");	// 10
        th(    "KshUm");	// 10
        th(    "KUndA");	// 10
        th(    "KY'AR");	// 10
        th(     "KYOR");	// 10
        th(     "LABS");	// 10
        th(  "LENDRAR");	// 10
        th(       "LS");	// 10
        th(     "LTAM");	// 10
        th(    "LTO'I");	// 10
        th(     "LTON");	// 10
        th(    "LVANG");	// 10
        th(   "MA'I'O");	// 10
        th(  "MA'ONGS");	// 10
        th(    "MCHUN");	// 10
        th(     "MDUL");	// 10
        th(   "MI'ANG");	// 10
        th(     "MJA'");	// 10
        th("MOO-NGGAL");	// 10
        th(     "MTAN");	// 10
        th(   "MTHE'U");	// 10
        th(    "MTSAG");	// 10
        th(    "MUKTI");	// 10
        th(     "MYOG");	// 10
        th(     "NAmS");	// 10
        th(      "NRi");	// 10
        th(      "NUM");	// 10
        th(      "nYA");	// 10
        th(    "NYANT");	// 10
        th(     "NYUD");	// 10
        th(     "NYVA");	// 10
        th(    "PA'NG");	// 10
        th(    "PHAMS");	// 10
        th(   "PHRAGS");	// 10
        th(    "PHRAM");	// 10
        th(   "'PHROL");	// 10
        th(    "PHYUD");	// 10
        th(     "PR'A");	// 10
        th(      "PRO");	// 10
        th(       "PS");	// 10
        th(   "PUn+YE");	// 10
        th(     "PYED");	// 10
        th(     "PYIN");	// 10
        th(    "RA'AM");	// 10
        th(     "RBED");	// 10
        th(     "RDEL");	// 10
        th(    "RDZUL");	// 10
        th(     "REMS");	// 10
        th(    "RGE'I");	// 10
        th(     "RGUS");	// 10
        th(    "RGYUB");	// 10
        th(  "RJE'ANG");	// 10
        th(      "RLA");	// 10
        th(     "RLAM");	// 10
        th(    "RLING");	// 10
        th(     "RMEN");	// 10
        th(    "RNGAD");	// 10
        th(    "RNYAN");	// 10
        th(   "RNYENG");	// 10
        th(      "RTO");	// 10
        th(     "RTVA");	// 10
        th(  "RTZA'AM");	// 10
        th(   "RTZENG");	// 10
        th(        "s");	// 10
        th(      "SA'");	// 10
        th(     "SBOD");	// 10
        th(   "SBYUGS");	// 10
        th(     "SDEM");	// 10
        th(     "SDUB");	// 10
        th(    "SGRUG");	// 10
        th(    "SGRUN");	// 10
        th(    "SHE'O");	// 10
        th(    "SH'IS");	// 10
        th(     "SHON");	// 10
        th(  "SKRA'AM");	// 10
        th(  "SLA'ANG");	// 10
        th(    "SNAMS");	// 10
        th(    "SNGUN");	// 10
        th(      "SNI");	// 10
        th(     "SNOS");	// 10
        th(   "SNRUBS");	// 10
        th(      "SNU");	// 10
        th(    "SNYIN");	// 10
        th(   "SNYONG");	// 10
        th(   "SRANGS");	// 10
        th(       "ST");	// 10
        th(    "STE'I");	// 10
        th(    "SYA'I");	// 10
        th(     "tA'O");	// 10
        th(    "T'IKA");	// 10
        th(     "TI'U");	// 10
        th(      "TOO");	// 10
        th(     "TR'I");	// 10
        th(    "'TSES");	// 10
        th(     "TSIR");	// 10
        th(   "TY'A'I");	// 10
        th(   "TZAKLI");	// 10
        th("TZANDRASA");	// 10
        th(     "TZOD");	// 10
        th(     "TZ'U");	// 10
        th(     "TZUB");	// 10
        th(     "'UBS");	// 10
        th(      "'UL");	// 10
        th(      "'US");	// 10
        th(      "YAn");	// 10
        th(     "Y'AS");	// 10
        th(      "YIL");	// 10
        th(   "YONGSU");	// 10
        th(      "YUB");	// 10
        th(   "ZA'ANG");	// 10
        th(    "ZENGS");	// 10
        th(        "0");	// 11
        th(      "135");	// 11
        th(       "46");	// 11
        th(       "55");	// 11
        th(       "66");	// 11
        th(       "73");	// 11
        th(       "79");	// 11
        th(     "7BCU");	// 11
        th(    "7SHOG");	// 11
        th(       "89");	// 11
        th(       "AG");	// 11
        th(       "AM");	// 11
        th(  "AURGE'I");	// 11
        th(  "AU-T+PA");	// 11
        th(      "BA5");	// 11
        th(     "BCAM");	// 11
        th(    "BCHAD");	// 11
        th(     "BCIR");	// 11
        th(     "BCON");	// 11
        th(     "BDAB");	// 11
        th(     "BDAM");	// 11
        th(   "BDUNGS");	// 11
        th(    "BE'UR");	// 11
        th(  "BGRONGS");	// 11
        th(    "BGYON");	// 11
        th(    "BGYUR");	// 11
        th(  "BHANGGA");	// 11
        th(     "BH'I");	// 11
        th(    "BHI'I");	// 11
        th(     "BKUN");	// 11
        th(    "BLANK");	// 11
        th(     "BLOB");	// 11
        th(    "BNYID");	// 11
        th(     "BOBS");	// 11
        th(  "BRDZAGS");	// 11
        th(     "BRGA");	// 11
        th(  "BRGYABS");	// 11
        th(   "BRLOBS");	// 11
        th(    "BRNOG");	// 11
        th(  "BRTZEBS");	// 11
        th(   "BRTZEM");	// 11
        th(     "BSAS");	// 11
        th(   "BSGROM");	// 11
        th(    "BSGRU");	// 11
        th(     "BSHA");	// 11
        th( "BSKYINGS");	// 11
        th(    "BSNUB");	// 11
        th(     "BSRA");	// 11
        th(    "'BUBS");	// 11
        th(     "BYA'");	// 11
        th(    "BZHER");	// 11
        th(    "BZHIL");	// 11
        th(      "CIL");	// 11
        th(      "dAR");	// 11
        th(    "DBYUD");	// 11
        th(   "DBYUGS");	// 11
        th(      "DHO");	// 11
        th(     "DKOS");	// 11
        th(    "DMAGS");	// 11
        th(  "DNGO-SU");	// 11
        th(      "DOB");	// 11
        th(   "dOMBHI");	// 11
        th(      "dON");	// 11
        th(     "DPUS");	// 11
        th(  "d'U-RYA");	// 11
        th(    "DVGAS");	// 11
        th(    "'DZIG");	// 11
        th(    "DZOGS");	// 11
        th(     "DZUS");	// 11
        th(     "'GAM");	// 11
        th(      "GAn");	// 11
        th(   "GA-YAB");	// 11
        th(   "GA-YEN");	// 11
        th(    "GA-YU");	// 11
        th(     "GDUS");	// 11
        th(     "GHAS");	// 11
        th(    "GHUtA");	// 11
        th(   "GLANGS");	// 11
        th(    "'GO'U");	// 11
        th(     "GSIG");	// 11
        th(     "GSUN");	// 11
        th(  "GSUNGSO");	// 11
        th(    "GSUNS");	// 11
        th(    "GTE'U");	// 11
        th(      "GTO");	// 11
        th(     "G-YE");	// 11
        th(     "GZAG");	// 11
        th(    "GZHAM");	// 11
        th(    "GZI'I");	// 11
        th(     "HR'I");	// 11
        th(     "JANG");	// 11
        th(     "'JED");	// 11
        th(     "'JIB");	// 11
        th(      "JIN");	// 11
        th(     "'JUN");	// 11
        th(    "KALPA");	// 11
        th(    "KARKA");	// 11
        th(   "KARSHA");	// 11
        th(    "KHAmS");	// 11
        th(   "KHRE'U");	// 11
        th(   "KHRO'O");	// 11
        th(    "'KHUS");	// 11
        th(    "KHYUS");	// 11
        th(    "K'I'I");	// 11
        th(    "KLANG");	// 11
        th(    "KLOGS");	// 11
        th(    "KRi'I");	// 11
        th(     "KROR");	// 11
        th(     "KYED");	// 11
        th( "LANG-KAR");	// 11
        th(    "LCING");	// 11
        th(    "LDOGS");	// 11
        th(      "LIm");	// 11
        th(   "LJANGS");	// 11
        th(      "LOO");	// 11
        th(   "LOTZ'A");	// 11
        th(      "'MA");	// 11
        th(    "MALLI");	// 11
        th(   "MAND'A");	// 11
        th("MANGGALAm");	// 11
        th(   "M'ANTA");	// 11
        th(     "MDOD");	// 11
        th(     "MRAG");	// 11
        th(   "MTHENG");	// 11
        th(   "MTHO'O");	// 11
        th(     "MTSA");	// 11
        th(      "ndI");	// 11
        th(   "NGHA'I");	// 11
        th(      "NTE");	// 11
        th(       "OS");	// 11
        th(   "PADMOS");	// 11
        th(      "PEN");	// 11
        th(     "POGS");	// 11
        th(    "RDA'I");	// 11
        th(    "RDUBS");	// 11
        th(    "RDU'I");	// 11
        th(   "RGYUNG");	// 11
        th(    "RI'AM");	// 11
        th(      "RIt");	// 11
        th(      "RJO");	// 11
        th(    "RNGOD");	// 11
        th(    "RNYEN");	// 11
        th(  "RTZINGS");	// 11
        th(    "RTZUD");	// 11
        th(   "RU'ANG");	// 11
        th(    "SBROD");	// 11
        th(    "SBROG");	// 11
        th(    "SBU'I");	// 11
        th(    "SDUGS");	// 11
        th(  "SGRENGS");	// 11
        th(   "SGYONG");	// 11
        th(    "SGYOR");	// 11
        th(   "SHE'AM");	// 11
        th( "SH'KYA'I");	// 11
        th(  "SIDDHAm");	// 11
        th(     "SKRI");	// 11
        th(    "SKUNG");	// 11
        th(    "SKYAL");	// 11
        th(    "SLE'U");	// 11
        th(     "SMEE");	// 11
        th(     "SNED");	// 11
        th(     "SNUR");	// 11
        th(     "SNYA");	// 11
        th(    "SNYER");	// 11
        th(     "SPOL");	// 11
        th(    "SPRAM");	// 11
        th(     "SPRU");	// 11
        th(    "STAGS");	// 11
        th(     "STUN");	// 11
        th(      "SYE");	// 11
        th(    "tA'IK");	// 11
        th(     "T'AS");	// 11
        th(    "TE'AM");	// 11
        th(     "TEHG");	// 11
        th(      "TEM");	// 11
        th(    "THI'I");	// 11
        th(   "'THOGS");	// 11
        th(      "TNA");	// 11
        th(      "TOB");	// 11
        th(    "TRANG");	// 11
        th(     "TSId");	// 11
        th(     "TSUM");	// 11
        th(     "TSWA");	// 11
        th(     "TZA'");	// 11
        th(     "TZIS");	// 11
        th(     "TZOR");	// 11
        th(      "W'I");	// 11
        th(     "Y'AN");	// 11
        th(     "ZE'O");	// 11
        th(    "ZHONG");	// 11
        th(       "51");	// 12
        th(       "59");	// 12
        th(       "70");	// 12
        th(       "75");	// 12
        th(   "7GNYIS");	// 12
        th(      "AAn");	// 12
        th(    "A'ANG");	// 12
        th(    "AARHA");	// 12
        th(   "A'ATMA");	// 12
        th(      "AUG");	// 12
        th(     "AUTA");	// 12
        th(    "AUTTA");	// 12
        th(      "BA2");	// 12
        th(      "BA6");	// 12
        th(   "BANDHA");	// 12
        th(   "BANDHE");	// 12
        th(     "'BIG");	// 12
        th(     "BING");	// 12
        th(   "BLTABS");	// 12
        th(      "BOM");	// 12
        th(     "'BOM");	// 12
        th(    "BRDAG");	// 12
        th(   "BRDA'O");	// 12
        th(   "'BRI'I");	// 12
        th(    "BRING");	// 12
        th(     "BRIR");	// 12
        th(     "BROL");	// 12
        th(    "BRTOG");	// 12
        th(     "BSAR");	// 12
        th(    "BSDAL");	// 12
        th(  "BSGRANG");	// 12
        th(   "BSGRIN");	// 12
        th(    "BSHUM");	// 12
        th(   "BSKABS");	// 12
        th(    "BSKOD");	// 12
        th(   "BSKYAM");	// 12
        th(   "BSKYOM");	// 12
        th(   "BSKYOR");	// 12
        th(   "BSLONG");	// 12
        th(    "BSNGA");	// 12
        th(   "BSNYER");	// 12
        th(    "BSRAD");	// 12
        th(  "BSTANGS");	// 12
        th(    "BTZAD");	// 12
        th(     "'BUN");	// 12
        th(     "CE'I");	// 12
        th(    "'CHEG");	// 12
        th(    "CHEGS");	// 12
        th(    "'DENG");	// 12
        th(  "DGRA'AM");	// 12
        th(  "DHARMMA");	// 12
        th(     "DHRU");	// 12
        th(     "DKYI");	// 12
        th(     "DPYE");	// 12
        th(    "'DZUB");	// 12
        th(     "GCAG");	// 12
        th(   "GDINGS");	// 12
        th(      "GHU");	// 12
        th(     "GRAd");	// 12
        th(     "'GRI");	// 12
        th(     "GRIN");	// 12
        th(    "GSHIG");	// 12
        th(   "GTZAGS");	// 12
        th(    "GU'AM");	// 12
        th(      "GUB");	// 12
        th(      "GVA");	// 12
        th(    "'GYAD");	// 12
        th(    "'GYOR");	// 12
        th(   "GZHOMS");	// 12
        th(    "GZHOR");	// 12
        th(     "GZUS");	// 12
        th(     "HI'I");	// 12
        th(      "HUS");	// 12
        th(   "HYASTA");	// 12
        th(   "KARMAS");	// 12
        th(     "KE'I");	// 12
        th(  "'KHRI'I");	// 12
        th(    "KHROG");	// 12
        th(   "'KHYIG");	// 12
        th(     "KLOR");	// 12
        th(     "KYIM");	// 12
        th(    "LCI'I");	// 12
        th(     "LCIS");	// 12
        th(      "LDU");	// 12
        th(      "LEd");	// 12
        th(      "LHE");	// 12
        th(     "LHEB");	// 12
        th(    "LHUGS");	// 12
        th(      "LIG");	// 12
        th( "LNGA'ANG");	// 12
        th(      "LOM");	// 12
        th(     "LTES");	// 12
        th(     "MGUS");	// 12
        th(    "MTSUL");	// 12
        th(     "MUGS");	// 12
        th(      "nA:");	// 12
        th(     "n'Am");	// 12
        th(   "NA-YAS");	// 12
        th(    "NU'AM");	// 12
        th(     "NYOR");	// 12
        th(   "PAKshU");	// 12
        th(     "PHAd");	// 12
        th(   "'PHOGS");	// 12
        th(     "RBOD");	// 12
        th(     "RDOD");	// 12
        th(   "RKU'AM");	// 12
        th(     "RO'O");	// 12
        th(   "RTA'AM");	// 12
        th(     "RTOD");	// 12
        th(     "RTOt");	// 12
        th(    "RTZ'A");	// 12
        th(    "RTZIL");	// 12
        th(      "RVI");	// 12
        th(    "SBRUB");	// 12
        th(    "SBRUG");	// 12
        th(     "SBUS");	// 12
        th(    "SDAMS");	// 12
        th(     "SDOR");	// 12
        th(     "SGOm");	// 12
        th("SH'AMBA'I'I");	// 12
        th( "SH'ARI'I");	// 12
        th(   "SKRANG");	// 12
        th(  "SKU'ANG");	// 12
        th( "SKYE'ANG");	// 12
        th(     "SLUS");	// 12
        th(    "SNAGS");	// 12
        th(   "SNGOMS");	// 12
        th(    "SPABS");	// 12
        th(      "SPI");	// 12
        th(    "SRENG");	// 12
        th(     "SRIS");	// 12
        th(    "SRI'U");	// 12
        th(    "STEBS");	// 12
        th(   "SUMBHA");	// 12
        th(     "TAGS");	// 12
        th(      "TEG");	// 12
        th(    "THAmD");	// 12
        th(   "THINGS");	// 12
        th(       "ti");	// 12
        th( "t'IKKA'I");	// 12
        th(    "TSA'O");	// 12
        th(    "'TSEL");	// 12
        th(   "'TSE'O");	// 12
        th(  "TSO'ANG");	// 12
        th(    "'TSOM");	// 12
        th(     "TZOL");	// 12
        th(     "TZVA");	// 12
        th(  "WARMA'I");	// 12
        th(      "YEE");	// 12
        th(      "100");	// 13
        th(       "44");	// 13
        th(       "48");	// 13
        th(       "57");	// 13
        th(    "7DE'I");	// 13
        th(     "A'AR");	// 13
        th(      "A'O");	// 13
        th(   "ARGHAm");	// 13
        th(    "BDEBS");	// 13
        th(      "BEL");	// 13
        th(     "BGAL");	// 13
        th(    "BGROL");	// 13
        th(  "BGRUNGS");	// 13
        th(     "BHAR");	// 13
        th(   "BHR'Um");	// 13
        th(   "BHYOO:");	// 13
        th(      "BKU");	// 13
        th(   "BLDAGS");	// 13
        th(  "BRDA'AM");	// 13
        th(   "BRDZAS");	// 13
        th(    "BRLAR");	// 13
        th(   "BRNGAR");	// 13
        th(   "BRTABS");	// 13
        th(    "BSHAd");	// 13
        th(    "BSHIN");	// 13
        th(   "BSNYES");	// 13
        th(     "BSUM");	// 13
        th(    "'BYOD");	// 13
        th(        "C");	// 13
        th(     "'CHA");	// 13
        th(    "CHEBS");	// 13
        th(    "'CHED");	// 13
        th(    "'CHOD");	// 13
        th(        "d");	// 13
        th(     "dA'I");	// 13
        th(     "DBAS");	// 13
        th(   "DBYIGS");	// 13
        th(  "DBYUNGS");	// 13
        th(    "DGYER");	// 13
        th(     "dING");	// 13
        th(     "DKOD");	// 13
        th(     "'DOS");	// 13
        th(    "DPA'A");	// 13
        th(    "DPYAR");	// 13
        th(     "DV'I");	// 13
        th(   "'DZAGS");	// 13
        th(     "DZHA");	// 13
        th(    "DZI'I");	// 13
        th(     "DZIR");	// 13
        th(    "GCE'U");	// 13
        th(    "GNYED");	// 13
        th(     "GSED");	// 13
        th(      "GTU");	// 13
        th(   "GTZONG");	// 13
        th(      "GUP");	// 13
        th(    "GZHED");	// 13
        th(     "GZOB");	// 13
        th(    "GZO'I");	// 13
        th(     "HI'O");	// 13
        th(     "HO'I");	// 13
        th(      "JAR");	// 13
        th(      "JUR");	// 13
        th(    "'KHOG");	// 13
        th(  "'KHRANG");	// 13
        th(  "KHRI'UR");	// 13
        th(  "KHRO'AM");	// 13
        th(   "KHRONG");	// 13
        th(    "'KHUN");	// 13
        th(    "KHYID");	// 13
        th(    "KHYOG");	// 13
        th(    "KHYOS");	// 13
        th(      "KIN");	// 13
        th(      "KRE");	// 13
        th(   "KRiNYA");	// 13
        th(     "KUNG");	// 13
        th(     "KYAG");	// 13
        th( "LAKshM'I");	// 13
        th(     "LA'U");	// 13
        th(    "LCEBS");	// 13
        th(      "LDO");	// 13
        th(     "LENG");	// 13
        th(   "LNGANG");	// 13
        th(    "LNGOG");	// 13
        th(    "LO'AM");	// 13
        th(    "LTABS");	// 13
        th(     "LTEB");	// 13
        th(    "LTONG");	// 13
        th(    "MGRUB");	// 13
        th(    "MGU'I");	// 13
        th(    "MGYUR");	// 13
        th(    "M'I'I");	// 13
        th(    "MKHOB");	// 13
        th(    "MNANG");	// 13
        th(     "MNAS");	// 13
        th(    "MNGOR");	// 13
        th(    "MYUNG");	// 13
        th(    "NA'NG");	// 13
        th(     "NOGS");	// 13
        th(     "NU'O");	// 13
        th(    "PA'Am");	// 13
        th(   "PADM'A");	// 13
        th(     "PE'U");	// 13
        th(    "'PHAd");	// 13
        th(    "'PHAD");	// 13
        th(     "PHED");	// 13
        th(    "PHUBS");	// 13
        th(     "PHUS");	// 13
        th(   "'PHYED");	// 13
        th(   "PHYENG");	// 13
        th(    "PHYIG");	// 13
        th(      "PRE");	// 13
        th(      "PUL");	// 13
        th(    "PURnA");	// 13
        th(    "RDEBS");	// 13
        th(     "RDUD");	// 13
        th(    "RDZOS");	// 13
        th(    "RDZOt");	// 13
        th(    "RGA'I");	// 13
        th(     "RKAD");	// 13
        th(    "RTZAG");	// 13
        th(    "RTZEM");	// 13
        th( "RTZVA'AM");	// 13
        th(   "SA'ANG");	// 13
        th(     "S'AN");	// 13
        th(     "SBIS");	// 13
        th(   "SBRUGS");	// 13
        th(  "SENGGES");	// 13
        th(   "SGO'AM");	// 13
        th(       "SH");	// 13
        th(   "SHATAm");	// 13
        th(    "SHEGS");	// 13
        th(    "shthA");	// 13
        th(   "sht'I:");	// 13
        th(   "SHU'AM");	// 13
        th(     "SHVI");	// 13
        th(   "SKANGS");	// 13
        th(     "SKOG");	// 13
        th(    "SKOMS");	// 13
        th(    "SKYAN");	// 13
        th(   "SKYA'O");	// 13
        th(   "SKYIGS");	// 13
        th(    "SMRAD");	// 13
        th(   "SMYUGS");	// 13
        th(   "SNGONG");	// 13
        th(   "SNYOGS");	// 13
        th(    "SPYID");	// 13
        th(   "SPYOMS");	// 13
        th(     "SRIG");	// 13
        th(      "tAR");	// 13
        th(    "'THA'");	// 13
        th(     "THIB");	// 13
        th(     "TREE");	// 13
        th(    "'TSAB");	// 13
        th(    "'TSUL");	// 13
        th(     "TZAn");	// 13
        th(      "YOM");	// 13
        th(   "ZHINGS");	// 13
        th(     "ZHOD");	// 13
        th(    "ZHVAS");	// 13
        th(       "37");	// 14
        th(       "39");	// 14
        th(       "42");	// 14
        th(       "47");	// 14
        th(       "49");	// 14
        th(    "AANTA");	// 14
        th(      "'AS");	// 14
        th(     "BGAM");	// 14
        th(     "BHYO");	// 14
        th(    "BHYO:");	// 14
        th(    "BIRWA");	// 14
        th(    "BKA'O");	// 14
        th(    "BLOGS");	// 14
        th(   "BRKU'O");	// 14
        th(    "BRLAM");	// 14
        th(   "BRNGOD");	// 14
        th(     "BRUN");	// 14
        th(    "BSGOR");	// 14
        th(    "BSGRO");	// 14
        th(   "BSGRUL");	// 14
        th(    "BSKOM");	// 14
        th(   "BSKU'O");	// 14
        th(    "BSLUR");	// 14
        th(    "BSRAB");	// 14
        th(   "BSTIMS");	// 14
        th(  "BSTUNGS");	// 14
        th(    "BTUMS");	// 14
        th(     "'BUB");	// 14
        th(     "BUBS");	// 14
        th(      "BUm");	// 14
        th(     "BYOG");	// 14
        th(    "'BYOG");	// 14
        th(   "BZHANG");	// 14
        th(     "BZIN");	// 14
        th(     "CHAd");	// 14
        th(     "CHIR");	// 14
        th(     "DKAG");	// 14
        th(    "DNGAN");	// 14
        th(  "'DRI'AM");	// 14
        th(   "'DRONG");	// 14
        th(  "'DZANGS");	// 14
        th(  "GANG+GA");	// 14
        th( "GA-YANGS");	// 14
        th(   "GA-YOB");	// 14
        th(    "GCI'I");	// 14
        th(     "GCOS");	// 14
        th(    "GNYAS");	// 14
        th(    "GO'AM");	// 14
        th(   "GYENGS");	// 14
        th(    "GZAGS");	// 14
        th(    "GZHOB");	// 14
        th(     "'JEN");	// 14
        th(  "KARMA'I");	// 14
        th(    "KHABS");	// 14
        th(    "KHAGS");	// 14
        th(    "KHENG");	// 14
        th(    "KHRUM");	// 14
        th(    "KHYIL");	// 14
        th(    "KRA'I");	// 14
        th(    "KRI'I");	// 14
        th(     "KYIL");	// 14
        th(     "LCER");	// 14
        th(     "LCIR");	// 14
        th(     "LDOM");	// 14
        th( "LENDRA'I");	// 14
        th(      "L'i");	// 14
        th(     "LNGE");	// 14
        th(    "MAndA");	// 14
        th(    "MCHIG");	// 14
        th(   "MDZA'I");	// 14
        th(   "MI'ONG");	// 14
        th(     "MTHA");	// 14
        th(    "MTHOG");	// 14
        th(    "MTSOL");	// 14
        th(   "MTSO'O");	// 14
        th(   "MTSUGS");	// 14
        th(     "MYAN");	// 14
        th(     "MZAD");	// 14
        th(   "NANDAS");	// 14
        th(   "NGU'AM");	// 14
        th(     "n+YA");	// 14
        th(     "NY'A");	// 14
        th(     "NYOL");	// 14
        th(    "NYONG");	// 14
        th(   "PAD+MA");	// 14
        th(   "PAD-MO");	// 14
        th(     "PHEL");	// 14
        th(     "PHIB");	// 14
        th(    "PHRUM");	// 14
        th(     "PHYU");	// 14
        th(  "'PHYUNG");	// 14
        th(       "PR");	// 14
        th(     "RANM");	// 14
        th(     "RDAN");	// 14
        th(    "RDZID");	// 14
        th(    "RJOGS");	// 14
        th(   "RLINGS");	// 14
        th(    "RTZOG");	// 14
        th(    "SARPA");	// 14
        th(   "SBRA'I");	// 14
        th(     "SDAN");	// 14
        th(     "SDUR");	// 14
        th(    "SENGS");	// 14
        th(    "SGRES");	// 14
        th(   "SGRING");	// 14
        th(     "SHAB");	// 14
        th( "SH'AKYAS");	// 14
        th(  "shtR'I:");	// 14
        th(    "S'IRA");	// 14
        th(     "SNEL");	// 14
        th(   "SNGE'I");	// 14
        th(    "SPYED");	// 14
        th(     "SRON");	// 14
        th(   "STE'UR");	// 14
        th(     "SVOM");	// 14
        th(    "'THAL");	// 14
        th(     "THUd");	// 14
        th(      "tIG");	// 14
        th(   "t'IKAR");	// 14
        th(      "TRU");	// 14
        th(     "TRUG");	// 14
        th(    "'TSEG");	// 14
        th(  "'TSO'AM");	// 14
        th(     "TSUS");	// 14
        th(    "TU'AM");	// 14
        th(    "TZA'O");	// 14
        th(     "ZHAB");	// 14
        th(    "ZLA'O");	// 14
        th(       "40");	// 15
        th(       "45");	// 15
        th(       "53");	// 15
        th(      "AAD");	// 15
        th(  "AAMBHAN");	// 15
        th(     "AING");	// 15
        th(     "AONG");	// 15
        th(      "AOS");	// 15
        th(       "AS");	// 15
        th( "BADZRA'I");	// 15
        th(      "BAn");	// 15
        th(      "BAY");	// 15
        th(    "BKRAR");	// 15
        th(   "BKRONG");	// 15
        th(    "BLCAG");	// 15
        th(     "BRER");	// 15
        th(   "BRLOMS");	// 15
        th(   "BRNAMS");	// 15
        th(    "BSAHD");	// 15
        th(    "BSDOG");	// 15
        th(    "BSHER");	// 15
        th(   "BSNGAR");	// 15
        th(    "BSUBS");	// 15
        th(     "BTOG");	// 15
        th(     "BYAG");	// 15
        th(  "'BYANGS");	// 15
        th(      "CEG");	// 15
        th(     "CHIM");	// 15
        th(   "'CHONG");	// 15
        th(      "COM");	// 15
        th(    "CU'AM");	// 15
        th(  "DAmshtA");	// 15
        th(     "DBES");	// 15
        th(   "DBYING");	// 15
        th(    "DGOGS");	// 15
        th(     "DHAS");	// 15
        th(    "DHI'I");	// 15
        th(    "dOMBI");	// 15
        th(     "DRAB");	// 15
        th(      "DYU");	// 15
        th( "DZA:H'Um");	// 15
        th(   "DZAMBU");	// 15
        th(    "'DZIR");	// 15
        th(    "GANdI");	// 15
        th(   "GA-YER");	// 15
        th(    "GRENG");	// 15
        th(    "'GROM");	// 15
        th(     "GSLA");	// 15
        th(    "GTSIL");	// 15
        th(    "GTZAM");	// 15
        th(    "GTZAN");	// 15
        th(     "'GUR");	// 15
        th(     "GYAN");	// 15
        th(    "GZONG");	// 15
        th(      "HVO");	// 15
        th(   "'JINGS");	// 15
        th(      "JUG");	// 15
        th(      "KAG");	// 15
        th(    "KAMPA");	// 15
        th(   "KHRANG");	// 15
        th(     "KING");	// 15
        th(    "KshIm");	// 15
        th(     "KshU");	// 15
        th(      "KTA");	// 15
        th(     "KYAN");	// 15
        th(    "KYU'I");	// 15
        th(     "KYUR");	// 15
        th( "LANGKA'I");	// 15
        th(      "LCA");	// 15
        th(    "LCONG");	// 15
        th(    "LDA'I");	// 15
        th(     "LDIB");	// 15
        th(   "LO'ANG");	// 15
        th(      "LTU");	// 15
        th(  "MAN-D'A");	// 15
        th(    "MKHOR");	// 15
        th(    "MNYED");	// 15
        th(   "MTHUNG");	// 15
        th(   "MTSOGS");	// 15
        th(       "NG");	// 15
        th(     "NGHA");	// 15
        th(      "NID");	// 15
        th(   "N+YA'I");	// 15
        th(     "PHIN");	// 15
        th(    "PHONG");	// 15
        th(   "'PHO'O");	// 15
        th(   "'PHRAS");	// 15
        th(  "'PHRING");	// 15
        th(    "'PHUD");	// 15
        th(     "RGER");	// 15
        th(      "RGO");	// 15
        th(     "RKYE");	// 15
        th(    "RMONG");	// 15
        th(    "RNANG");	// 15
        th(   "RNYANG");	// 15
        th(    "SDENG");	// 15
        th(    "SDUNG");	// 15
        th(      "SES");	// 15
        th(     "SGAS");	// 15
        th(   "SGRANG");	// 15
        th(    "SGREL");	// 15
        th(    "SHA'O");	// 15
        th(    "shA'U");	// 15
        th(     "SHYA");	// 15
        th(      "SIB");	// 15
        th(    "SIDHA");	// 15
        th(    "SKYEB");	// 15
        th(     "SNER");	// 15
        th(   "SNYAGS");	// 15
        th(    "SNYEL");	// 15
        th(     "SPAD");	// 15
        th(     "SPUL");	// 15
        th(   "TE'ANG");	// 15
        th(   "THANGS");	// 15
        th(    "'THIB");	// 15
        th(    "THO'U");	// 15
        th(    "TILLI");	// 15
        th(   "'TSENG");	// 15
        th(      "TUM");	// 15
        th(     "TZER");	// 15
        th(     "'U'I");	// 15
        th(    "WARTA");	// 15
        th(     "YE'I");	// 15
        th(      "YOB");	// 15
        th(   "ZHVA'I");	// 15
        th(     "'ZIN");	// 15
        th(      "ZOB");	// 15
        th(       "38");	// 16
        th(       "41");	// 16
        th(       "43");	// 16
        th(       "92");	// 16
        th(      "AA:");	// 16
        th(  "AINDR'A");	// 16
        th(    "BDAGS");	// 16
        th(      "'BE");	// 16
        th(   "BE'U'I");	// 16
        th(   "B'IRYA");	// 16
        th(   "BLINGS");	// 16
        th(    "'BRAD");	// 16
        th(    "'BRED");	// 16
        th(    "BRLOB");	// 16
        th(   "BRTZOL");	// 16
        th(     "BRUG");	// 16
        th(     "BSHU");	// 16
        th(   "BSKYAS");	// 16
        th(     "BSTA");	// 16
        th(     "CA'I");	// 16
        th(      "CUG");	// 16
        th(     "'DAL");	// 16
        th(   "'DENGS");	// 16
        th(      "dHA");	// 16
        th( "DHARMA'I");	// 16
        th(     "'DIL");	// 16
        th(    "DNGES");	// 16
        th(      "dOM");	// 16
        th(     "DPYI");	// 16
        th(     "DRAD");	// 16
        th(     "DRVA");	// 16
        th(    "'DUMS");	// 16
        th(   "GANDHE");	// 16
        th( "GAnd'I'I");	// 16
        th(   "GA-YAG");	// 16
        th(   "GE'ANG");	// 16
        th(    "GLA'I");	// 16
        th(    "GSHIB");	// 16
        th(     "GSUR");	// 16
        th(      "GTE");	// 16
        th(    "GYUNG");	// 16
        th(    "GZHIB");	// 16
        th(      "HAB");	// 16
        th(    "HRENG");	// 16
        th(     "'JAS");	// 16
        th(     "KHEN");	// 16
        th(    "KHOMS");	// 16
        th(  "'KHRUNG");	// 16
        th(      "KOG");	// 16
        th(    "KULLE");	// 16
        th(     "KYAB");	// 16
        th(   "MARGAD");	// 16
        th(    "MKHEN");	// 16
        th(   "MOKshA");	// 16
        th( "MOONGGAL");	// 16
        th(     "nA'O");	// 16
        th(     "NGUL");	// 16
        th(      "NTA");	// 16
        th(      "'OM");	// 16
        th(    "'PHRA");	// 16
        th(    "PHYOt");	// 16
        th(     "PHYR");	// 16
        th(   "PO'I'O");	// 16
        th(     "PUNG");	// 16
        th(      "PYA");	// 16
        th(  "RATNA'I");	// 16
        th(     "RDUS");	// 16
        th(    "RDZOd");	// 16
        th(      "REL");	// 16
        th(      "RGE");	// 16
        th(  "RGYA'AM");	// 16
        th(      "RnA");	// 16
        th(    "RNASM");	// 16
        th("RONYAKsh'A");	// 16
        th(      "RUN");	// 16
        th(    "SBOMS");	// 16
        th(   "SDONGS");	// 16
        th(     "SGAB");	// 16
        th(     "SGON");	// 16
        th(   "SGRO'I");	// 16
        th(    "shA'I");	// 16
        th( "SH'AKYAR");	// 16
        th( "SH'AMB'I");	// 16
        th(     "SKUN");	// 16
        th(     "SMYU");	// 16
        th(    "SNA'O");	// 16
        th(    "SNE'I");	// 16
        th(    "SNYEM");	// 16
        th(    "SNYOD");	// 16
        th(    "SO'AM");	// 16
        th(    "STVAm");	// 16
        th(      "TES");	// 16
        th(      "T'O");	// 16
        th(    "'TSAR");	// 16
        th(    "TSVAS");	// 16
        th(     "TV'A");	// 16
        th(     "TZAm");	// 16
        th(   "TZANDA");	// 16
        th(     "TZAS");	// 16
        th(     "ZHUt");	// 16
        th("AUTPALA'I");	// 17
        th(     "BHAS");	// 17
        th(     "BHIR");	// 17
        th(    "BHRUm");	// 17
        th(    "BLTAN");	// 17
        th(   "BRANGS");	// 17
        th(   "BRKYAD");	// 17
        th(    "BRNYA");	// 17
        th( "BRTZENGS");	// 17
        th(    "BSDIG");	// 17
        th(    "BSGAM");	// 17
        th(    "BSGUG");	// 17
        th(    "BSKED");	// 17
        th(  "BSKYONG");	// 17
        th(     "BSUB");	// 17
        th(     "BYIL");	// 17
        th(     "CHEB");	// 17
        th(    "CHENG");	// 17
        th(    "CINGS");	// 17
        th(     "CONG");	// 17
        th(      "CUD");	// 17
        th(     "DEBS");	// 17
        th(      "DIS");	// 17
        th(      "DPO");	// 17
        th(     "DROL");	// 17
        th(    "'DRUB");	// 17
        th(    "'DZAR");	// 17
        th(     "DZYA");	// 17
        th(     "GANS");	// 17
        th(     "'GEB");	// 17
        th(     "GLUS");	// 17
        th(    "'GO'I");	// 17
        th(   "'GRING");	// 17
        th(    "GRVAS");	// 17
        th(     "GSUG");	// 17
        th(    "GTEGS");	// 17
        th(   "G-YABS");	// 17
        th(      "JIS");	// 17
        th(    "KHAYB");	// 17
        th(    "KLUGS");	// 17
        th(      "KUS");	// 17
        th(  "LANGKAR");	// 17
        th(     "LDAD");	// 17
        th(    "LTEBS");	// 17
        th(     "LTEM");	// 17
        th(      "LUd");	// 17
        th(      "LYA");	// 17
        th(      "MA'");	// 17
        th(   "MANTRA");	// 17
        th(    "MCHUR");	// 17
        th(    "MTH'A");	// 17
        th(  "MTSANGS");	// 17
        th(  "MTSO'AM");	// 17
        th(    "MTSUN");	// 17
        th(     "MYAL");	// 17
        th(      "MYO");	// 17
        th(  "NA-YA'I");	// 17
        th(      "nIS");	// 17
        th(     "NYIM");	// 17
        th(      "NYU");	// 17
        th(  "PAndITA");	// 17
        th(      "PER");	// 17
        th(    "PHA'O");	// 17
        th(     "PHEN");	// 17
        th(  "PHYI'AM");	// 17
        th(     "PRAL");	// 17
        th(   "PU'ANG");	// 17
        th(   "RAKshA");	// 17
        th(    "RDAMS");	// 17
        th(     "RDEB");	// 17
        th(      "REN");	// 17
        th(    "RGONG");	// 17
        th(   "RJE'AM");	// 17
        th(    "RTABS");	// 17
        th(     "RTHA");	// 17
        th(   "RTZABS");	// 17
        th(     "RVAR");	// 17
        th(    "SARWA");	// 17
        th(  "SBRINGS");	// 17
        th(     "SHIB");	// 17
        th(     "SHOB");	// 17
        th(     "SKOD");	// 17
        th(   "SKRAGS");	// 17
        th(    "SKYAG");	// 17
        th(      "SMI");	// 17
        th(      "SOO");	// 17
        th(     "STHI");	// 17
        th( "SV'AH'AS");	// 17
        th(      "TAB");	// 17
        th(      "TER");	// 17
        th(     "THES");	// 17
        th(     "tI'I");	// 17
        th(     "TRAm");	// 17
        th(      "TTA");	// 17
        th(    "tV'Am");	// 17
        th(     "TZES");	// 17
        th(      "WAL");	// 17
        th(   "WARMAS");	// 17
        th(      "ZUN");	// 17
        th(       "34");	// 18
        th(       "36");	// 18
        th(       "50");	// 18
        th(     "7RJE");	// 18
        th(      "AAL");	// 18
        th(      "AEL");	// 18
        th(      "BCA");	// 18
        th(     "BGED");	// 18
        th(    "BIDYA");	// 18
        th(   "BKVANG");	// 18
        th(    "BRDUL");	// 18
        th(    "BRJEN");	// 18
        th(  "BRTZOMS");	// 18
        th(  "BSDONGS");	// 18
        th(   "BSNGAD");	// 18
        th(    "BSONG");	// 18
        th(   "BSRE'I");	// 18
        th(   "BTANGS");	// 18
        th(   "BTZANG");	// 18
        th(    "'BYAD");	// 18
        th(     "DHIS");	// 18
        th(     "'DIN");	// 18
        th(    "DKYEL");	// 18
        th(    "DMANG");	// 18
        th(      "GAm");	// 18
        th(   "GANDHO");	// 18
        th(    "'GROG");	// 18
        th(   "G-YU'I");	// 18
        th(    "GZENG");	// 18
        th(    "GZHUD");	// 18
        th(      "HES");	// 18
        th(      "HRi");	// 18
        th(     "HVAN");	// 18
        th(     "'JAD");	// 18
        th(      "KAL");	// 18
        th(   "'KHANG");	// 18
        th(   "'KHREN");	// 18
        th( "KHRI'U'I");	// 18
        th(    "KHRUG");	// 18
        th(    "KHRUR");	// 18
        th(     "KHUB");	// 18
        th(   "KHYANG");	// 18
        th(    "Ksh'A");	// 18
        th(    "LA'NG");	// 18
        th(    "LLE'I");	// 18
        th(   "LNGAGS");	// 18
        th(     "LTUR");	// 18
        th(     "LU'U");	// 18
        th(    "MCHER");	// 18
        th(     "MJOD");	// 18
        th(     "MNOL");	// 18
        th(    "NA+YA");	// 18
        th(    "NYTZA");	// 18
        th(     "'O'I");	// 18
        th(   "PADMAR");	// 18
        th(   "P'URBA");	// 18
        th(     "PYIR");	// 18
        th(  "RDZANGS");	// 18
        th(    "RNGAL");	// 18
        th(    "RNGAS");	// 18
        th(    "RNGES");	// 18
        th(     "RTEG");	// 18
        th(      "RTZ");	// 18
        th(   "RTZI'O");	// 18
        th(     "RTZU");	// 18
        th(    "SBRUS");	// 18
        th(    "SH'AK");	// 18
        th("SH'AMB'I'I");	// 18
        th(      "shU");	// 18
        th(     "SKUM");	// 18
        th(    "SNGER");	// 18
        th(    "SPRID");	// 18
        th(    "^SPUG");	// 18
        th( "SPYI'ANG");	// 18
        th(     "STED");	// 18
        th(      "TAG");	// 18
        th(     "THIS");	// 18
        th(    "'THOM");	// 18
        th(     "T'IR");	// 18
        th(      "TOL");	// 18
        th(      "TPA");	// 18
        th( "TRAKSHAD");	// 18
        th(    "TR'Am");	// 18
        th(    "tVIKA");	// 18
        th(      "ZEN");	// 18
        th(     "ZENG");	// 18
        th(    "ZHI'O");	// 18
        th(        "+");	// 19
        th(       "35");	// 19
        th(      "AAB");	// 19
        th(      "AAt");	// 19
        th(      "AtA");	// 19
        th(     "'BAS");	// 19
        th(    "BDUNG");	// 19
        th(   "BGYING");	// 19
        th(     "BI'I");	// 19
        th(   "BKANGS");	// 19
        th(    "BKYED");	// 19
        th(   "BRGYUN");	// 19
        th(    "BRJAD");	// 19
        th(    "BRKOR");	// 19
        th(   "BSDANG");	// 19
        th(     "BSDE");	// 19
        th(    "BSHAG");	// 19
        th(   "BSHUGS");	// 19
        th(   "BSKROD");	// 19
        th(   "BSNGON");	// 19
        th(   "BTZEMS");	// 19
        th(    "'BYID");	// 19
        th(    "'CHIL");	// 19
        th(      "DA'");	// 19
        th(      "DE'");	// 19
        th(      "DEG");	// 19
        th(    "DHY'A");	// 19
        th(     "DO'O");	// 19
        th(    "'DRED");	// 19
        th(  "d'UR+YA");	// 19
        th(    "GCING");	// 19
        th(   "GNANGS");	// 19
        th(    "'GOMS");	// 19
        th(   "GSHOMS");	// 19
        th(     "GYUL");	// 19
        th(   "GZHING");	// 19
        th(    "'JAMS");	// 19
        th(     "'JOS");	// 19
        th(     "KHAm");	// 19
        th(    "KHRES");	// 19
        th(    "KHRUL");	// 19
        th(      "KIR");	// 19
        th(      "KKA");	// 19
        th(      "K'U");	// 19
        th(    "KVE'U");	// 19
        th(     "KYAD");	// 19
        th(   "KY'A'I");	// 19
        th(   "LANGKA");	// 19
        th(    "LBAGS");	// 19
        th(   "MAntAL");	// 19
        th(    "MCHIM");	// 19
        th(   "MGO'AM");	// 19
        th(     "MH'A");	// 19
        th(     "MIGS");	// 19
        th(     "M'IS");	// 19
        th(   "MNGA'I");	// 19
        th(    "MNGES");	// 19
        th( "MTHA'ANG");	// 19
        th(     "MU'I");	// 19
        th(     "MYOD");	// 19
        th(      "NES");	// 19
        th(      "N'O");	// 19
        th(      "PAI");	// 19
        th(     "PHAD");	// 19
        th(    "'PHRI");	// 19
        th(    "PHRUL");	// 19
        th(      "PUD");	// 19
        th(    "SBYOS");	// 19
        th(      "SED");	// 19
        th(    "SGYED");	// 19
        th(  "SHLOOKA");	// 19
        th(      "shO");	// 19
        th(    "SHTZA");	// 19
        th(      "SID");	// 19
        th(    "SKUGS");	// 19
        th(     "SLAS");	// 19
        th(     "SNOL");	// 19
        th(     "SPHA");	// 19
        th(     "STRA");	// 19
        th(     "S+YA");	// 19
        th(        "t");	// 19
        th(      "TAD");	// 19
        th(    "TADYA");	// 19
        th(     "T'AM");	// 19
        th(    "'THAM");	// 19
        th(    "THE'I");	// 19
        th(   "THUNGS");	// 19
        th(      "tOR");	// 19
        th(     "TR'A");	// 19
        th(     "TRON");	// 19
        th(      "TTI");	// 19
        th(     "TZAG");	// 19
        th(      "YA:");	// 19
        th(     "ZHOM");	// 19
        th(       "32");	// 20
        th(       "33");	// 20
        th(    "AANDA");	// 20
        th(    "AERTI");	// 20
        th(      "AOM");	// 20
        th(     "BDUS");	// 20
        th(     "BE'I");	// 20
        th(      "'BI");	// 20
        th(    "BIRBA");	// 20
        th(  "BLO'ANG");	// 20
        th(   "BRDUBS");	// 20
        th(    "BRDZE");	// 20
        th(    "'BRES");	// 20
        th(   "BRGYUR");	// 20
        th(     "BRIN");	// 20
        th(    "'BRIR");	// 20
        th(   "BRNGAN");	// 20
        th(   "BRNYEN");	// 20
        th(   "BTINGS");	// 20
        th(  "BUDDH'A");	// 20
        th(     "'BYO");	// 20
        th(   "'BYONG");	// 20
        th(     "BYOR");	// 20
        th(    "BZUGS");	// 20
        th(  "'CHINGS");	// 20
        th(      "COS");	// 20
        th(     "'DER");	// 20
        th(    "DGAGS");	// 20
        th(     "DGAN");	// 20
        th(     "^DGE");	// 20
        th(   "'DRE'O");	// 20
        th(     "DZAR");	// 20
        th( "GANG'A'I");	// 20
        th(     "GI'I");	// 20
        th(     "GING");	// 20
        th(   "GNYIGS");	// 20
        th(      "GRi");	// 20
        th(    "GTZOD");	// 20
        th(    "'GUMS");	// 20
        th(     "GU'O");	// 20
        th(  "'GYINGS");	// 20
        th(      "HAD");	// 20
        th(    "'JANG");	// 20
        th(     "'JUM");	// 20
        th(   "KHANGS");	// 20
        th(     "KHER");	// 20
        th(   "KHRU'I");	// 20
        th(    "KHRUN");	// 20
        th(    "KHYAG");	// 20
        th(  "KHYE'UR");	// 20
        th(     "KRIS");	// 20
        th(      "KRO");	// 20
        th(     "KYAS");	// 20
        th("LANG-KA'I");	// 20
        th(      "LLE");	// 20
        th(   "LTA'AM");	// 20
        th(     "LTOD");	// 20
        th(  "MANG+GA");	// 20
        th(    "MCHES");	// 20
        th(    "MCHUS");	// 20
        th(    "MDONG");	// 20
        th(    "MTHAD");	// 20
        th(    "NGE'O");	// 20
        th(    "PHYAS");	// 20
        th(  "'PHYE'I");	// 20
        th( "'PHYONGS");	// 20
        th(      "RJI");	// 20
        th(      "RLO");	// 20
        th(     "RNAB");	// 20
        th(   "RNYONG");	// 20
        th(     "RTAt");	// 20
        th(    "RTE'U");	// 20
        th(    "RTZEL");	// 20
        th(    "SBYIS");	// 20
        th(    "SHE'U");	// 20
        th(   "SH'KYA");	// 20
        th(    "SHRIS");	// 20
        th(   "SKONGS");	// 20
        th(    "SKYOG");	// 20
        th(    "SNYUG");	// 20
        th(    "STH'A");	// 20
        th(    "THE'U");	// 20
        th(    "T'I'I");	// 20
        th(   "TSONGS");	// 20
        th(     "TZIG");	// 20
        th(      "ZIG");	// 20
        th(       "31");	// 21
        th(   "AARDZU");	// 21
        th(      "AAS");	// 21
        th(     "BRAD");	// 21
        th(   "BRTZIR");	// 21
        th(     "BSLA");	// 21
        th(  "BSNYAMS");	// 21
        th(   "BTUNGS");	// 21
        th(    "BYO'I");	// 21
        th(    "BZHUG");	// 21
        th(    "'CHID");	// 21
        th(    "'CHIS");	// 21
        th(      "dAL");	// 21
        th(     "DDHA");	// 21
        th(     "DHA'");	// 21
        th(    "DNGON");	// 21
        th(     "DR'A");	// 21
        th(    "DRA'O");	// 21
        th(   "'DRI'I");	// 21
        th(       "dU");	// 21
        th(     "d'UR");	// 21
        th( "D'URYA'I");	// 21
        th(    "dY'AN");	// 21
        th(     "GDEG");	// 21
        th(     "GE'O");	// 21
        th(     "GI'U");	// 21
        th(   "GLINGS");	// 21
        th(     "GNOS");	// 21
        th(     "GNUR");	// 21
        th(   "GNYAGS");	// 21
        th(     "GRIR");	// 21
        th(     "GSAB");	// 21
        th(  "GSHONGS");	// 21
        th(    "GZANG");	// 21
        th(     "JO'I");	// 21
        th(     "KHEB");	// 21
        th(   "'KHONG");	// 21
        th(  "KHYE'US");	// 21
        th(    "KHYUG");	// 21
        th(   "LHONGS");	// 21
        th(   "MCHU'I");	// 21
        th(    "MDAGS");	// 21
        th(     "MKHA");	// 21
        th( "MKHYEEEN");	// 21
        th(    "MNYAm");	// 21
        th(    "NGAGS");	// 21
        th(     "NONG");	// 21
        th(    "PHRAS");	// 21
        th(     "PONG");	// 21
        th(    "RE'AM");	// 21
        th(      "R'i");	// 21
        th(      "RJA");	// 21
        th(     "RM'A");	// 21
        th(   "RMANGS");	// 21
        th(     "RMON");	// 21
        th(   "RNGU'I");	// 21
        th(    "RTEGS");	// 21
        th(    "RTI'I");	// 21
        th(      "RUD");	// 21
        th(     "RU'O");	// 21
        th(     "SEHS");	// 21
        th(     "SHAm");	// 21
        th(    "SKRAR");	// 21
        th(   "SNYENG");	// 21
        th(    "SNYOG");	// 21
        th(    "SPA'I");	// 21
        th(     "SRAG");	// 21
        th(    "STUNG");	// 21
        th(      "SUB");	// 21
        th(     "SY'A");	// 21
        th(    "THO'I");	// 21
        th(     "THOM");	// 21
        th(    "THRIG");	// 21
        th(     "TOGS");	// 21
        th(    "ZHA'I");	// 21
        th(     "AO'I");	// 22
        th(     "AYON");	// 22
        th(    "B'A'I");	// 22
        th(    "BDANG");	// 22
        th(   "BEEd'U");	// 22
        th(    "BENDE");	// 22
        th(   "BGRONG");	// 22
        th(   "BRTZEN");	// 22
        th(    "BSDEB");	// 22
        th(    "CHONG");	// 22
        th(   "'CHUGS");	// 22
        th(      "CIM");	// 22
        th(     "DBAN");	// 22
        th(      "DEM");	// 22
        th(      "DGA");	// 22
        th(      "DIG");	// 22
        th(     "DRIM");	// 22
        th(    "'DRUD");	// 22
        th(     "D'UR");	// 22
        th(    "'DZIM");	// 22
        th(   "'DZING");	// 22
        th(     "GAMS");	// 22
        th(     "GCIS");	// 22
        th(     "GCOM");	// 22
        th(      "GHI");	// 22
        th(      "GNA");	// 22
        th(     "GRUG");	// 22
        th(     "GZHE");	// 22
        th(     "HONG");	// 22
        th(   "KHYIMS");	// 22
        th(     "LDAB");	// 22
        th(    "LJONG");	// 22
        th(     "LTOG");	// 22
        th(     "LTUD");	// 22
        th(    "MCHAG");	// 22
        th(   "MDO'AM");	// 22
        th(    "MDO'O");	// 22
        th(    "MRGAD");	// 22
        th(        "n");	// 22
        th(      "nAm");	// 22
        th(  "'PHYONG");	// 22
        th(     "RDZO");	// 22
        th(   "RNYOGS");	// 22
        th(      "SA:");	// 22
        th(    "SAMBU");	// 22
        th( "S'ATZTSA");	// 22
        th(    "SBRID");	// 22
        th(     "SDUN");	// 22
        th(     "SGRU");	// 22
        th(   "SGRUNG");	// 22
        th(    "SGYEL");	// 22
        th(    "SHENG");	// 22
        th(    "SHOGS");	// 22
        th(      "SIS");	// 22
        th(   "SPRAGS");	// 22
        th(    "SPRIS");	// 22
        th(  "SPRO'AM");	// 22
        th(    "SPYUG");	// 22
        th(    "SROGS");	// 22
        th(     "SUNG");	// 22
        th(       "tE");	// 22
        th(     "THA'");	// 22
        th(   "'THONG");	// 22
        th(     "'THU");	// 22
        th(       "26");	// 23
        th(       "27");	// 23
        th(  "AITHI'I");	// 23
        th(  "AOdYAnA");	// 23
        th(    "BANGS");	// 23
        th(      "BIN");	// 23
        th(    "BODHI");	// 23
        th(  "BRNANGS");	// 23
        th(    "BSABS");	// 23
        th(   "BSLU'I");	// 23
        th(    "BZLUM");	// 23
        th(  "'DRA'AM");	// 23
        th(   "'DRE'I");	// 23
        th(      "D'U");	// 23
        th(      "'GA");	// 23
        th(  "GA-YO'I");	// 23
        th(    "GLEBS");	// 23
        th(    "GNYAR");	// 23
        th(    "GSHOR");	// 23
        th(      "'GU");	// 23
        th(    "GYA'I");	// 23
        th(   "GZANGS");	// 23
        th(    "GZHAS");	// 23
        th(      "H'I");	// 23
        th(      "HUm");	// 23
        th(     "'JOR");	// 23
        th(     "KHES");	// 23
        th(    "KHYAR");	// 23
        th(     "KYID");	// 23
        th(      "MDE");	// 23
        th(   "MKHRIG");	// 23
        th(     "MNOR");	// 23
        th(     "'OGS");	// 23
        th(   "PHRE'U");	// 23
        th(   "'PHRUG");	// 23
        th(   "'PHYIR");	// 23
        th(   "'PHYOR");	// 23
        th(    "RGYOB");	// 23
        th(  "RKYANGS");	// 23
        th(     "RNYE");	// 23
        th(    "RTA'O");	// 23
        th(     "RTIS");	// 23
        th(   "SBUNGS");	// 23
        th(      "SEM");	// 23
        th(    "SGRAL");	// 23
        th(    "SNYES");	// 23
        th(    "SNYOL");	// 23
        th(     "STAB");	// 23
        th(     "STHA");	// 23
        th(      "SUD");	// 23
        th(     "TENG");	// 23
        th(     "THAm");	// 23
        th(     "TRIG");	// 23
        th(      "'UD");	// 23
        th(      "YEN");	// 23
        th(      "YIB");	// 23
        th(     "ZABS");	// 23
        th(       "ZU");	// 23
        th(       "28");	// 24
        th(    "AAGNI");	// 24
        th(   "AAKshA");	// 24
        th(      "AIS");	// 24
        th(      "BID");	// 24
        th(    "BSTAD");	// 24
        th(    "'BUGS");	// 24
        th(     "BYOD");	// 24
        th(     "CHEL");	// 24
        th(   "CHU'AM");	// 24
        th(    "DBYIS");	// 24
        th(     "'DEG");	// 24
        th(     "'DEN");	// 24
        th(    "'DIZN");	// 24
        th(     "DKAS");	// 24
        th(     "DREN");	// 24
        th(   "'DRUMS");	// 24
        th(   "DU:KHA");	// 24
        th(  "GAndI'I");	// 24
        th(     "GCUS");	// 24
        th(   "'GONGS");	// 24
        th(     "GRES");	// 24
        th(     "GRIM");	// 24
        th(   "'GRONG");	// 24
        th(    "GRU'U");	// 24
        th(      "HUN");	// 24
        th(    "K'ARA");	// 24
        th(    "KHYUR");	// 24
        th(     "K'IS");	// 24
        th(    "KRITA");	// 24
        th(    "MCHON");	// 24
        th(    "MCHOR");	// 24
        th(     "MENG");	// 24
        th(   "MKHYAN");	// 24
        th(    "MNAGS");	// 24
        th(    "MTHAL");	// 24
        th(     "N'Am");	// 24
        th(   "PHRING");	// 24
        th(    "PHROL");	// 24
        th(    "PHYIM");	// 24
        th(   "'PHYIN");	// 24
        th(      "PRI");	// 24
        th(    "PUndA");	// 24
        th(    "RKU'O");	// 24
        th(    "RNA'I");	// 24
        th(  "RNYONGS");	// 24
        th(   "SARP'A");	// 24
        th(      "SGE");	// 24
        th( "SGRA'ANG");	// 24
        th(    "SGROD");	// 24
        th(    "SGUGS");	// 24
        th(      "SIN");	// 24
        th(    "SKYOS");	// 24
        th(  "SKYUNGS");	// 24
        th(    "SMANG");	// 24
        th(    "SNGAG");	// 24
        th(      "SRi");	// 24
        th(      "TRi");	// 24
        th(     "TSOd");	// 24
        th(    "TZE'I");	// 24
        th(        "U");	// 24
        th(      "ZUB");	// 24
        th(      "A'U");	// 25
        th(      "AUR");	// 25
        th(   "BKLUBS");	// 25
        th(   "BKRA'I");	// 25
        th(      "BOL");	// 25
        th(     "BRUB");	// 25
        th(    "'BRUB");	// 25
        th(   "BSGRON");	// 25
        th(   "BSGYEL");	// 25
        th(    "BSHUB");	// 25
        th(    "BSNAM");	// 25
        th(      "d'I");	// 25
        th(      "DMA");	// 25
        th(     "DMAD");	// 25
        th(     "DZ'I");	// 25
        th(    "'DZOL");	// 25
        th( "GANG+G'A");	// 25
        th(     "GLOR");	// 25
        th(     "GREL");	// 25
        th(     "GYED");	// 25
        th(    "GZING");	// 25
        th(      "HOS");	// 25
        th(      "HRI");	// 25
        th(      "KON");	// 25
        th(    "KRANG");	// 25
        th(     "LDES");	// 25
        th(     "LDUD");	// 25
        th(      "LOB");	// 25
        th(  "MCHUNGS");	// 25
        th(  "NGA'ANG");	// 25
        th(    "'PHAG");	// 25
        th(    "'PHAL");	// 25
        th(     "PHIR");	// 25
        th(     "PHOS");	// 25
        th(     "RMYA");	// 25
        th("RONYKsh'A");	// 25
        th(    "RTZID");	// 25
        th(     "R+YA");	// 25
        th(    "SAPTA");	// 25
        th(    "SHVAD");	// 25
        th(   "SMYANG");	// 25
        th(     "SNAN");	// 25
        th(    "SNGAL");	// 25
        th(    "SPRON");	// 25
        th(      "STU");	// 25
        th(        "T");	// 25
        th(     "THED");	// 25
        th(    "THEMS");	// 25
        th(    "THING");	// 25
        th(   "'THOMS");	// 25
        th(      "TIN");	// 25
        th(     "TRES");	// 25
        th(    "TRI'I");	// 25
        th(  "'TSENGS");	// 25
        th(    "TZI'I");	// 25
        th(       "23");	// 26
        th(     "B'AI");	// 26
        th(     "BHRA");	// 26
        th(    "BRKUR");	// 26
        th(   "BSGUGS");	// 26
        th(   "BSNGAS");	// 26
        th(    "'BUNG");	// 26
        th(     "BYEN");	// 26
        th(     "BZHE");	// 26
        th(    "BZONG");	// 26
        th(   "CHA'AM");	// 26
        th(     "CHON");	// 26
        th(    "DGYIS");	// 26
        th(    "DRONG");	// 26
        th(     "DU:-");	// 26
        th(      "DVI");	// 26
        th(     "DZIN");	// 26
        th(      "G'I");	// 26
        th(    "GLUNG");	// 26
        th(    "GNYIN");	// 26
        th(    "GSHAD");	// 26
        th(    "G-YEN");	// 26
        th(     "JING");	// 26
        th(     "KHA'");	// 26
        th(    "KHYEN");	// 26
        th(      "LIR");	// 26
        th(    "MANDA");	// 26
        th(   "MCHAMS");	// 26
        th(    "MDZAS");	// 26
        th(    "MJING");	// 26
        th(    "MTSAM");	// 26
        th(    "MTSOD");	// 26
        th(    "MTSOG");	// 26
        th(    "NYE'O");	// 26
        th(     "PHER");	// 26
        th(    "RDZOD");	// 26
        th(    "RDZOG");	// 26
        th(     "SDAS");	// 26
        th(    "SDING");	// 26
        th(     "SDOS");	// 26
        th(     "SGYE");	// 26
        th( "SHAPTA'I");	// 26
        th(      "shE");	// 26
        th(   "SHI'AM");	// 26
        th(    "SKRAD");	// 26
        th(     "SMRI");	// 26
        th(   "SRINGS");	// 26
        th(      "TIR");	// 26
        th(     "YAGS");	// 26
        th(       "21");	// 27
        th(       "24");	// 27
        th(     "BBAS");	// 27
        th(  "BEEd'UR");	// 27
        th(     "BHAN");	// 27
        th(    "BRE'I");	// 27
        th(    "BRENG");	// 27
        th(    "BSHED");	// 27
        th(   "BSHUNG");	// 27
        th(   "BSNYEG");	// 27
        th(      "CUN");	// 27
        th(     "DBER");	// 27
        th(   "'DI'AM");	// 27
        th(     "DRIR");	// 27
        th(   "GA-YEL");	// 27
        th(    "'GEMS");	// 27
        th(     "GO'O");	// 27
        th(    "GSHAR");	// 27
        th(   "'GYUNG");	// 27
        th(     "GZUd");	// 27
        th( "KHANG+GA");	// 27
        th(    "KHY'A");	// 27
        th(     "KLUR");	// 27
        th(   "KO'UdI");	// 27
        th(  "LING+GA");	// 27
        th(    "ME'AM");	// 27
        th(    "MTSES");	// 27
        th(     "NABS");	// 27
        th(    "NYA'O");	// 27
        th(   "PHYI'O");	// 27
        th(     "RKOS");	// 27
        th(     "RNAS");	// 27
        th(     "SBOR");	// 27
        th(    "^SBUG");	// 27
        th(      "SGU");	// 27
        th(  "SKAB-SU");	// 27
        th(    "SKRAS");	// 27
        th(     "SPON");	// 27
        th(     "SPUB");	// 27
        th(    "SUNGS");	// 27
        th(   "TZAKRA");	// 27
        th(     "ZHEG");	// 27
        th(       "20");	// 28
        th(       "22");	// 28
        th(   "7ZHABS");	// 28
        th(      "AAG");	// 28
        th(     "BCES");	// 28
        th(   "BKRAMS");	// 28
        th(   "BLO'AM");	// 28
        th(  "BRDZOGS");	// 28
        th(   "BRLAGS");	// 28
        th(    "BSKAD");	// 28
        th(    "BSLOB");	// 28
        th(   "BSNUBS");	// 28
        th(     "BSRI");	// 28
        th(   "BSTANG");	// 28
        th(      "BYO");	// 28
        th(      "BZE");	// 28
        th(   "'CHA'O");	// 28
        th(   "DBANGS");	// 28
        th(    "DBYIN");	// 28
        th(   "DE'I'O");	// 28
        th(     "DHRi");	// 28
        th(     "DKUR");	// 28
        th(     "DMIG");	// 28
        th(     "DOMS");	// 28
        th(    "'DRON");	// 28
        th(    "GCANG");	// 28
        th(     "GDAG");	// 28
        th(      "GLE");	// 28
        th(     "GRIL");	// 28
        th(      "GSE");	// 28
        th(     "GSEL");	// 28
        th(     "GYID");	// 28
        th(     "GZEG");	// 28
        th( "GZHI'ANG");	// 28
        th(      "KA'");	// 28
        th(    "KHRIM");	// 28
        th(     "KR'i");	// 28
        th(    "KRONG");	// 28
        th(    "KUNDA");	// 28
        th(   "LANDHA");	// 28
        th(    "LHONG");	// 28
        th(   "LHUNGS");	// 28
        th(  "MNGA'AM");	// 28
        th(   "MTSUNG");	// 28
        th(    "NANTA");	// 28
        th(    "NGE'I");	// 28
        th(    "'PHAM");	// 28
        th(   "'PHEBS");	// 28
        th(   "'PHRAN");	// 28
        th(     "PHRO");	// 28
        th(     "RKOR");	// 28
        th(     "RMOD");	// 28
        th(   "SAMBHA");	// 28
        th(    "SDE'U");	// 28
        th( "SH'ANTIS");	// 28
        th(   "SHAPTA");	// 28
        th(     "SMEL");	// 28
        th(    "SNRON");	// 28
        th(    "SNYIL");	// 28
        th(    "SRABS");	// 28
        th(     "TSEL");	// 28
        th(     "TSUG");	// 28
        th(     "TZAR");	// 28
        th(       "30");	// 29
        th(      "AGS");	// 29
        th("BEEd'URYA'I");	// 29
        th(    "BKAGS");	// 29
        th(     "BRAB");	// 29
        th(  "BRNYOGS");	// 29
        th(  "BRTZI'O");	// 29
        th(    "BTZOM");	// 29
        th(    "BYABS");	// 29
        th(   "BZLA'O");	// 29
        th(     "CHID");	// 29
        th(     "'CHO");	// 29
        th(   "DGU'AM");	// 29
        th(     "DNGO");	// 29
        th(    "DPYAS");	// 29
        th(   "GA-YA'");	// 29
        th(  "GA-YUNG");	// 29
        th(     "GCUG");	// 29
        th(     "'GEG");	// 29
        th(     "GLAG");	// 29
        th(     "'GUG");	// 29
        th(   "GUPTAS");	// 29
        th(   "G-YO'O");	// 29
        th(    "GZA'I");	// 29
        th(   "'KHRUS");	// 29
        th(    "KIRTI");	// 29
        th(     "KYAR");	// 29
        th(     "LDAM");	// 29
        th(    "MDE'U");	// 29
        th(      "NER");	// 29
        th(      "NGI");	// 29
        th(     "NYIR");	// 29
        th(      "PEE");	// 29
        th(    "PHYOd");	// 29
        th(    "RLANG");	// 29
        th(  "SBRENGS");	// 29
        th(    "SLABS");	// 29
        th(     "SPAG");	// 29
        th(    "STOGS");	// 29
        th(    "'THEB");	// 29
        th(   "tIKKAR");	// 29
        th(   "'TSUBS");	// 29
        th(      "TUG");	// 29
        th(     "TZAL");	// 29
        th(       "WU");	// 29
        th(     "YOGS");	// 29
        th(    "ZA'AM");	// 29
        th(     "ZHER");	// 29
        th(     "ZUGS");	// 29
        th(       "25");	// 30
        th(      "AEN");	// 30
        th(      "ANG");	// 30
        th(     "'BAM");	// 30
        th(   "BDE'AM");	// 30
        th(    "BDO'I");	// 30
        th(  "BKRONGS");	// 30
        th(    "BRDZU");	// 30
        th(    "BRTOD");	// 30
        th(  "BSGRE'O");	// 30
        th(   "BZHUNG");	// 30
        th(    "CHI'I");	// 30
        th(     "CHOR");	// 30
        th(    "'CHUN");	// 30
        th(      "DME");	// 30
        th(   "D'URYA");	// 30
        th(  "'GA'ANG");	// 30
        th(     "GSAM");	// 30
        th(    "GSUGS");	// 30
        th(   "GYI'AM");	// 30
        th(   "'GYOGS");	// 30
        th(    "GYU'I");	// 30
        th(    "GZHEN");	// 30
        th(      "HUB");	// 30
        th(     "KHYE");	// 30
        th(    "KHYIR");	// 30
        th(     "KROG");	// 30
        th(      "KUR");	// 30
        th(    "LDE'U");	// 30
        th(      "LUR");	// 30
        th(  "MKHYEEN");	// 30
        th(      "nAR");	// 30
        th(     "NYAD");	// 30
        th(     "PHEG");	// 30
        th(    "PHYOD");	// 30
        th(      "PRi");	// 30
        th(     "PU'O");	// 30
        th(     "RDZE");	// 30
        th(      "RMU");	// 30
        th(     "RMUG");	// 30
        th(    "RNGUS");	// 30
        th(    "SBUGS");	// 30
        th(   "SGRIMS");	// 30
        th(   "SHRI'I");	// 30
        th(   "SPRENG");	// 30
        th(    "SPROG");	// 30
        th(    "SU'AM");	// 30
        th(      "SUR");	// 30
        th(    "'THUR");	// 30
        th(    "'TSAG");	// 30
        th(   "TSE'AM");	// 30
        th(   "'TSE'I");	// 30
        th(       "tU");	// 30
        th("TZANDRA'I");	// 30
        th(      "YES");	// 30
        th(   "YI'ANG");	// 30
        th(       "17");	// 31
        th(       "19");	// 31
        th(    "BHYAS");	// 31
        th(      "BIM");	// 31
        th(     "'BRE");	// 31
        th(   "BRNOGS");	// 31
        th(     "BRUL");	// 31
        th(    "BTONG");	// 31
        th(    "BYI'I");	// 31
        th(  "CHA'ANG");	// 31
        th(      "dAm");	// 31
        th(    "'DANG");	// 31
        th(       "DD");	// 31
        th(    "DH'AR");	// 31
        th(   "GANGGA");	// 31
        th("GANG+GA'I");	// 31
        th(     "GLAS");	// 31
        th(   "GSHANG");	// 31
        th(    "GSHEN");	// 31
        th(    "G-YOD");	// 31
        th(      "GZE");	// 31
        th(      "HAM");	// 31
        th(     "HRAL");	// 31
        th(      "JUS");	// 31
        th(    "KARnA");	// 31
        th(    "KHRAS");	// 31
        th(   "KHRIGS");	// 31
        th(     "KVAN");	// 31
        th(    "KYE'I");	// 31
        th(     "LHOG");	// 31
        th(      "M'U");	// 31
        th(   "MYONGS");	// 31
        th(   "PHA'AM");	// 31
        th(     "REGS");	// 31
        th(     "RGVA");	// 31
        th(    "SGRAG");	// 31
        th(    "SGRUL");	// 31
        th(    "SHO'I");	// 31
        th(     "SKEG");	// 31
        th(    "SKYAD");	// 31
        th(   "SNYANG");	// 31
        th(      "SOt");	// 31
        th(     "SPOG");	// 31
        th(      "tAS");	// 31
        th(     "YA'O");	// 31
        th(       "18");	// 32
        th(       "29");	// 32
        th(     "A'AM");	// 32
        th(      "AIt");	// 32
        th(   "BRDZOD");	// 32
        th(   "BRLA'I");	// 32
        th(    "BSRID");	// 32
        th(    "BSRUB");	// 32
        th(     "BYAM");	// 32
        th(    "'CHUG");	// 32
        th(    "CHUGS");	// 32
        th(     "'DES");	// 32
        th(     "DZEE");	// 32
        th(     "GLUD");	// 32
        th(     "GNAN");	// 32
        th(     "GZHA");	// 32
        th(     "KHOB");	// 32
        th(   "KHRI'O");	// 32
        th(     "KHUD");	// 32
        th(     "LTAB");	// 32
        th(      "MOO");	// 32
        th(   "MTHU'O");	// 32
        th(   "PA'O'I");	// 32
        th(    "PHYAM");	// 32
        th(     "RI'O");	// 32
        th(     "RLIG");	// 32
        th(    "RLUBS");	// 32
        th(   "RTZONG");	// 32
        th("SENG+GE'I");	// 32
        th(     "SGUL");	// 32
        th(  "SKYANGS");	// 32
        th(  "SNGANGS");	// 32
        th(    "SNGOD");	// 32
        th(    "SNREL");	// 32
        th(    "THAmS");	// 32
        th(   "'THENG");	// 32
        th(     "ZHOS");	// 32
        th(    "'BANG");	// 33
        th(    "BKRAG");	// 33
        th(   "BLTA'I");	// 33
        th(    "'BRUS");	// 33
        th(    "BSDUM");	// 33
        th(   "BSGO'I");	// 33
        th(   "BSNGOR");	// 33
        th(   "BSTING");	// 33
        th(    "BTZEM");	// 33
        th(    "'BYOL");	// 33
        th(     "BYUL");	// 33
        th(     "DAMS");	// 33
        th(      "dOm");	// 33
        th( "DZAMBU'I");	// 33
        th(      "GCO");	// 33
        th(   "GTZUGS");	// 33
        th(      "GUN");	// 33
        th(    "'GYID");	// 33
        th(    "G-YOR");	// 33
        th(   "KHA'AM");	// 33
        th(  "'KHYOGS");	// 33
        th(      "KYO");	// 33
        th(    "LNGON");	// 33
        th(     "MJAD");	// 33
        th(    "MNYEL");	// 33
        th(    "MTSAD");	// 33
        th(  "NGU'ANG");	// 33
        th(    "PHOBS");	// 33
        th(    "PHRED");	// 33
        th(      "PON");	// 33
        th(     "POTA");	// 33
        th(      "RKA");	// 33
        th(      "RKE");	// 33
        th(     "RMAR");	// 33
        th(     "RTUN");	// 33
        th(   "RTZAMS");	// 33
        th(     "RTZO");	// 33
        th(      "SEG");	// 33
        th( "SHRADDHA");	// 33
        th(   "SKYUNG");	// 33
        th(    "SLU'I");	// 33
        th(    "SNUBS");	// 33
        th(     "SROS");	// 33
        th(    "STING");	// 33
        th(      "SUm");	// 33
        th(    "T'A'I");	// 33
        th(    "TSAMS");	// 33
        th(     "ZHUG");	// 33
        th(      "ZUL");	// 33
        th(       "A'");	// 34
        th(     "AA'I");	// 34
        th(      "AEE");	// 34
        th(     "BCUM");	// 34
        th(     "BDON");	// 34
        th(   "BRTZUN");	// 34
        th(   "BSHUMS");	// 34
        th(     "BZOR");	// 34
        th(      "'DA");	// 34
        th(   "DH'ARA");	// 34
        th(   "'DRANG");	// 34
        th(  "GATZTSA");	// 34
        th(    "GRUGS");	// 34
        th(    "GYOGS");	// 34
        th(     "HU'I");	// 34
        th(     "JU'I");	// 34
        th(   "LDANGS");	// 34
        th(     "LDAR");	// 34
        th(       "Li");	// 34
        th(   "MCHAGS");	// 34
        th(       "nE");	// 34
        th(  "'PHRIGS");	// 34
        th(     "RE'U");	// 34
        th(      "ROM");	// 34
        th(  "RTZANGS");	// 34
        th(  "RTZENGS");	// 34
        th(    "RYA'I");	// 34
        th(     "SDOG");	// 34
        th(     "SKAG");	// 34
        th(    "SKYIS");	// 34
        th(    "SNONG");	// 34
        th(      "SPE");	// 34
        th(    "SPRAN");	// 34
        th(   "SPYUGS");	// 34
        th(   "'THANG");	// 34
        th(    "THOBS");	// 34
        th(      "TUS");	// 34
        th(    "TZO'I");	// 34
        th(   "WENDRA");	// 34
        th(       "AR");	// 35
        th(      "BDU");	// 35
        th(    "BGRUB");	// 35
        th(    "BRI'O");	// 35
        th(  "BSNGOMS");	// 35
        th(    "'DAMS");	// 35
        th(     "DBUB");	// 35
        th(     "DE'U");	// 35
        th(      "DGO");	// 35
        th(     "GNYE");	// 35
        th(     "'GRA");	// 35
        th(    "'GRED");	// 35
        th(   "'GRIBS");	// 35
        th(   "GZHAMS");	// 35
        th(     "HRAG");	// 35
        th(     "'JIL");	// 35
        th(     "KHUN");	// 35
        th(    "KHYIS");	// 35
        th(     "KYES");	// 35
        th(    "PHYVA");	// 35
        th(     "RA'O");	// 35
        th(   "RDZE'U");	// 35
        th(     "RTUG");	// 35
        th(   "RTZEGS");	// 35
        th(    "SBOBS");	// 35
        th(     "SBUD");	// 35
        th(  "SHA'KYA");	// 35
        th(     "SRIL");	// 35
        th(     "tING");	// 35
        th(     "TO'I");	// 35
        th(      "WAR");	// 35
        th(       "WE");	// 35
        th(      "YED");	// 35
        th(     "YING");	// 35
        th(     "YI'O");	// 35
        th(     "YUGS");	// 35
        th(       "16");	// 36
        th(      "AAm");	// 36
        th(     "BCER");	// 36
        th(    "BKYAL");	// 36
        th(     "'BRA");	// 36
        th(    "BRDOS");	// 36
        th(    "BRNGA");	// 36
        th(   "BRTZAN");	// 36
        th(    "BYE'I");	// 36
        th(     "DBYA");	// 36
        th(    "'DE'I");	// 36
        th(     "DGYE");	// 36
        th(     "DI'I");	// 36
        th(        "E");	// 36
        th(     "GA'U");	// 36
        th(     "GDUD");	// 36
        th(     "GHAN");	// 36
        th(  "'GRENGS");	// 36
        th(    "G-YAM");	// 36
        th(     "GYAR");	// 36
        th(      "JIG");	// 36
        th(      "KIS");	// 36
        th(    "MCHEN");	// 36
        th(    "MTHAG");	// 36
        th(    "MTSEN");	// 36
        th(      "PHI");	// 36
        th(     "RDOB");	// 36
        th(    "RDZIG");	// 36
        th(     "RTAB");	// 36
        th(     "RTOd");	// 36
        th(    "SBRAS");	// 36
        th(    "SBROS");	// 36
        th(     "SPOD");	// 36
        th(    "SPROR");	// 36
        th(  "TIshthA");	// 36
        th(   "'TSO'O");	// 36
        th(     "ZHID");	// 36
        th(       "14");	// 37
        th(    "BHY'A");	// 37
        th(      "B'O");	// 37
        th(   "BSKYES");	// 37
        th(     "BYIR");	// 37
        th(   "BZHAGS");	// 37
        th(      "DAB");	// 37
        th(    "D'A'I");	// 37
        th(    "'DEMS");	// 37
        th(     "DH'I");	// 37
        th(    "DPONG");	// 37
        th(    "'DZIS");	// 37
        th(     "GLUR");	// 37
        th(     "GRON");	// 37
        th(     "HRIG");	// 37
        th(    "HVAGS");	// 37
        th(    "KVA'E");	// 37
        th(    "LHA'O");	// 37
        th(        "M");	// 37
        th(    "MTHOS");	// 37
        th(   "PHRA'I");	// 37
        th(    "PHYEN");	// 37
        th(    "RNYAD");	// 37
        th(    "RNYID");	// 37
        th(    "RTZAS");	// 37
        th(    "SGYUG");	// 37
        th(    "SHOLO");	// 37
        th(  "SPYI'AM");	// 37
        th(     "TSIS");	// 37
        th(     "TZAB");	// 37
        th(      "YAm");	// 37
        th(    "YANTU");	// 37
        th(     "ZHAD");	// 37
        th(      "BCI");	// 38
        th(   "BDZRAS");	// 38
        th(     "BHEE");	// 38
        th(     "BLUS");	// 38
        th(    "BRGYU");	// 38
        th(   "BSGROD");	// 38
        th(  "BSNYAGS");	// 38
        th(     "'BYE");	// 38
        th( "BZHI'ANG");	// 38
        th(   "CHE'AM");	// 38
        th(  "CHE'ANG");	// 38
        th(      "DRi");	// 38
        th(    "'DRUL");	// 38
        th(    "GA'AM");	// 38
        th(    "GNYOS");	// 38
        th(    "GRE'U");	// 38
        th(     "GYER");	// 38
        th(     "HU'm");	// 38
        th(     "'JUR");	// 38
        th(      "KEE");	// 38
        th(     "KHRE");	// 38
        th(     "LDAG");	// 38
        th(    "LDEBS");	// 38
        th(      "LHU");	// 38
        th(      "L'I");	// 38
        th(     "ME'O");	// 38
        th(     "MNGO");	// 38
        th(   "PUshPE");	// 38
        th(   "RDUNGS");	// 38
        th(   "RDZA'I");	// 38
        th(   "RDZI'I");	// 38
        th(     "RNYI");	// 38
        th(     "RTOL");	// 38
        th(    "SDE'O");	// 38
        th(    "SGRIN");	// 38
        th(      "shI");	// 38
        th(     "SH'O");	// 38
        th(  "SHR'ISA");	// 38
        th(    "SKEGS");	// 38
        th(      "t'A");	// 38
        th(   "'THAMS");	// 38
        th(      "AEM");	// 39
        th(      "AUS");	// 39
        th(    "BKRIR");	// 39
        th(      "BRi");	// 39
        th(     "BROD");	// 39
        th(     "BSAm");	// 39
        th(    "BSDUG");	// 39
        th(    "BSHAS");	// 39
        th(   "BSNYOD");	// 39
        th(    "BYE'U");	// 39
        th(    "CHU'O");	// 39
        th(     "DHYA");	// 39
        th(   "DPYANG");	// 39
        th(     "DRES");	// 39
        th(     "DZAm");	// 39
        th(      "GEN");	// 39
        th(     "GOGS");	// 39
        th(    "GONGS");	// 39
        th(    "'JUGS");	// 39
        th(     "KLOD");	// 39
        th(     "KLUS");	// 39
        th(    "LCIBS");	// 39
        th(   "MERGAN");	// 39
        th(   "NGA'AM");	// 39
        th(   "'PHYON");	// 39
        th(      "POT");	// 39
        th(    "RDZIS");	// 39
        th(      "REM");	// 39
        th(     "RMEG");	// 39
        th(      "RTE");	// 39
        th(    "SDAGS");	// 39
        th(    "SMRAN");	// 39
        th(     "SPOB");	// 39
        th(     "TSUB");	// 39
        th(    "TSUNG");	// 39
        th(      "ZED");	// 39
        th(   "AURGER");	// 40
        th(   "AUT-PA");	// 40
        th(     "BGAD");	// 40
        th(  "BRGYANG");	// 40
        th(    "BRTON");	// 40
        th(  "BSNYEMS");	// 40
        th(    "BUNGS");	// 40
        th(   "'CHUMS");	// 40
        th(   "DGA'SA");	// 40
        th(    "DH'I:");	// 40
        th(      "GSU");	// 40
        th(     "GTOG");	// 40
        th(      "GYE");	// 40
        th(    "'GYUS");	// 40
        th(      "HAm");	// 40
        th(    "KHA'U");	// 40
        th(    "'KHUM");	// 40
        th(   "MNYDZU");	// 40
        th(      "PAM");	// 40
        th(    "PHIBS");	// 40
        th(      "RA:");	// 40
        th(    "RGYUL");	// 40
        th(     "SHIR");	// 40
        th(     "SH'U");	// 40
        th(    "SKYOL");	// 40
        th(  "SMRA'AM");	// 40
        th(   "SNGANG");	// 40
        th(     "SRBA");	// 40
        th(    "SREGS");	// 40
        th(    "tI-KA");	// 40
        th(     "TUNG");	// 40
        th(       "13");	// 41
        th(    "BCIBS");	// 41
        th(    "BCOMS");	// 41
        th(      "BED");	// 41
        th(   "BZLUMS");	// 41
        th(     "'DAD");	// 41
        th(     "'DEB");	// 41
        th(     "DHAR");	// 41
        th(     "DMId");	// 41
        th(    "'GENG");	// 41
        th(     "GRUS");	// 41
        th(     "GYOG");	// 41
        th(   "KYI'AM");	// 41
        th(     "LDER");	// 41
        th(    "MANTA");	// 41
        th(    "MDZOS");	// 41
        th(    "NGH'A");	// 41
        th(     "NO'I");	// 41
        th(      "NUL");	// 41
        th(  "PANYTZA");	// 41
        th(    "PHYER");	// 41
        th(      "RId");	// 41
        th(    "RLUGS");	// 41
        th(    "SGE'U");	// 41
        th(   "SGYOGS");	// 41
        th(   "SGYU'I");	// 41
        th(     "SKAB");	// 41
        th(    "SKANG");	// 41
        th(    "'TSAD");	// 41
        th(       "12");	// 42
        th(    "BGAGS");	// 42
        th(   "BRDZIS");	// 42
        th(    "BRNAN");	// 42
        th(   "BRTOGS");	// 42
        th(   "BSDAGS");	// 42
        th(   "BSKYIL");	// 42
        th(   "BSKYIS");	// 42
        th(    "BTOGS");	// 42
        th(    "BTU'I");	// 42
        th(     "DBAG");	// 42
        th(    "d'IKA");	// 42
        th(     "DRUS");	// 42
        th(    "'GREM");	// 42
        th(     "GTUG");	// 42
        th(    "G-YOB");	// 42
        th(     "HE'I");	// 42
        th(     "'JIN");	// 42
        th(    "KHUMS");	// 42
        th(        "L");	// 42
        th(     "LDEB");	// 42
        th(      "MAG");	// 42
        th(     "MCHO");	// 42
        th(      "NIG");	// 42
        th(  "RDZINGS");	// 42
        th(    "RNAGS");	// 42
        th(     "RTAd");	// 42
        th(     "SEGS");	// 42
        th(    "SH'AN");	// 42
        th(      "SIG");	// 42
        th(     "SMES");	// 42
        th(     "SRUB");	// 42
        th(      "TYA");	// 42
        th(    "ZHU'O");	// 42
        th(     "ZI'I");	// 42
        th(    "ZLOGS");	// 42
        th(     "BGAG");	// 43
        th( "BRGYANGS");	// 43
        th(    "BRTZA");	// 43
        th(    "BSGOD");	// 43
        th(    "BTABS");	// 43
        th(    "BTZUM");	// 43
        th(     "BY'A");	// 43
        th(   "CHONGS");	// 43
        th(    "DBYED");	// 43
        th(      "GCU");	// 43
        th(    "GYENG");	// 43
        th(    "KHRIL");	// 43
        th(      "NIR");	// 43
        th(        "P");	// 43
        th(   "'PHRED");	// 43
        th(   "PHYANG");	// 43
        th(    "PHYID");	// 43
        th(    "PHYOG");	// 43
        th(     "PING");	// 43
        th( "RGYU'ANG");	// 43
        th(     "SBAM");	// 43
        th(     "SHUD");	// 43
        th(    "SNYID");	// 43
        th(      "thA");	// 43
        th(     "BCUN");	// 44
        th(     "BKUS");	// 44
        th(    "BSNUR");	// 44
        th(      "CID");	// 44
        th(     "DHVA");	// 44
        th(    "'DZUL");	// 44
        th(     "GLAD");	// 44
        th(   "GSANGS");	// 44
        th(    "GTIBS");	// 44
        th(  "KHRI'AM");	// 44
        th(     "KHUM");	// 44
        th(     "KshI");	// 44
        th(    "LDUGS");	// 44
        th(      "LES");	// 44
        th(    "LHOGS");	// 44
        th(    "MKHAL");	// 44
        th(   "MKHA'O");	// 44
        th(    "MOO'U");	// 44
        th(   "PHRANG");	// 44
        th(   "'PHREL");	// 44
        th(    "PHROG");	// 44
        th(    "PHYAL");	// 44
        th(      "REB");	// 44
        th(     "RMOG");	// 44
        th(      "ZOD");	// 44
        th(       "15");	// 45
        th(   "7BTZUN");	// 45
        th(  "BADZRAS");	// 45
        th(      "BEG");	// 45
        th(      "BIL");	// 45
        th(      "BRU");	// 45
        th(    "'BRUR");	// 45
        th(     "BTAD");	// 45
        th(    "BTZOD");	// 45
        th(      "DAm");	// 45
        th(    "DBYOD");	// 45
        th(      "DEL");	// 45
        th(      "DU:");	// 45
        th(    "DZNYA");	// 45
        th(     "DZVA");	// 45
        th(     "GCAM");	// 45
        th(     "GZIG");	// 45
        th(      "JUN");	// 45
        th(      "KAm");	// 45
        th(    "LUNGS");	// 45
        th(      "MAn");	// 45
        th(    "NYI'I");	// 45
        th(   "'PHYAG");	// 45
        th(    "RGYEN");	// 45
        th(    "RKONG");	// 45
        th(     "SBAD");	// 45
        th(  "SH'ANTA");	// 45
        th(    "SKRAN");	// 45
        th(    "SKYAB");	// 45
        th(     "SLON");	// 45
        th(      "TEN");	// 45
        th(  "'TSANGS");	// 45
        th(    "'TSOB");	// 45
        th(    "TZA'I");	// 45
        th( "TZANDRAS");	// 45
        th(      "ZOM");	// 45
        th(    "BKONG");	// 46
        th(     "BRAS");	// 46
        th(   "BSHIGS");	// 46
        th(     "CE'O");	// 46
        th(     "DBUG");	// 46
        th(     "DKAN");	// 46
        th(    "'DREL");	// 46
        th(    "GSOGS");	// 46
        th(     "'GUM");	// 46
        th(    "GZHUS");	// 46
        th(      "KAM");	// 46
        th(     "KHUS");	// 46
        th(     "KHYA");	// 46
        th(   "'KHYAR");	// 46
        th(   "'KHYOL");	// 46
        th(   "LE'U'O");	// 46
        th(    "MNGAN");	// 46
        th(     "MNON");	// 46
        th(    "MTHUR");	// 46
        th(   "'PHRIG");	// 46
        th(   "'PHYES");	// 46
        th(     "SBYO");	// 46
        th(     "SHEN");	// 46
        th( "SPRE'U'I");	// 46
        th(      "TUN");	// 46
        th(        "-");	// 47
        th(       "11");	// 47
        th(      "BIR");	// 47
        th(      "BOG");	// 47
        th(    "BONGS");	// 47
        th(    "BRGAN");	// 47
        th(     "BRTA");	// 47
        th(     "BRUD");	// 47
        th(     "BTAN");	// 47
        th(  "BYA'O'I");	// 47
        th(     "BZUR");	// 47
        th(     "GLON");	// 47
        th(     "KH'A");	// 47
        th(      "KIM");	// 47
        th(   "KLUNGS");	// 47
        th(     "KshE");	// 47
        th(  "KUNDA'I");	// 47
        th(    "PHAGS");	// 47
        th(     "RDIB");	// 47
        th(    "RNYIL");	// 47
        th(      "RUR");	// 47
        th(   "SGREGS");	// 47
        th(  "SHUDDHA");	// 47
        th(    "SPUBS");	// 47
        th(     "TA'O");	// 47
        th(     "TE'I");	// 47
        th(   "BSKYUD");	// 48
        th(     "JI'I");	// 48
        th(      "LBU");	// 48
        th(      "LIS");	// 48
        th(     "MI'O");	// 48
        th(     "MNAM");	// 48
        th(    "MNGOS");	// 48
        th(      "MNO");	// 48
        th(     "NE'I");	// 48
        th(      "P'E");	// 48
        th(    "PHIGS");	// 48
        th(    "'PHOG");	// 48
        th(    "SARGA");	// 48
        th(     "SDAD");	// 48
        th(     "SEmS");	// 48
        th(     "SESM");	// 48
        th(     "THEN");	// 48
        th(      "WAN");	// 48
        th(      "WEN");	// 48
        th(      "BHE");	// 49
        th(     "BRUS");	// 49
        th(   "BSDU'I");	// 49
        th(    "BSLAN");	// 49
        th(    "'BYER");	// 49
        th(   "BZHING");	// 49
        th(      "DBE");	// 49
        th(    "DPYAL");	// 49
        th(    "DZA'I");	// 49
        th(  "GANGG'A");	// 49
        th( "'GRO'ANG");	// 49
        th(   "GZHU'I");	// 49
        th(      "HAS");	// 49
        th(      "HUR");	// 49
        th( "K'ARsh'A");	// 49
        th(    "'KHAL");	// 49
        th(  "KRishnA");	// 49
        th(      "R'U");	// 49
        th(    "SBRAG");	// 49
        th(    "SHOMS");	// 49
        th(    "SHUBS");	// 49
        th(    "SNYEG");	// 49
        th(      "STO");	// 49
        th(    "ZHIGS");	// 49
        th(    "AURGE");	// 50
        th(   "BCU'AM");	// 50
        th(    "BCUNG");	// 50
        th( "BRDZANGS");	// 50
        th(   "BRGYAG");	// 50
        th(    "'BRUL");	// 50
        th(   "BSKONG");	// 50
        th(    "BSO'I");	// 50
        th(   "'BYING");	// 50
        th(   "BZHENG");	// 50
        th(      "CIN");	// 50
        th(     "CU'O");	// 50
        th(   "DBYI'O");	// 50
        th(     "DMAS");	// 50
        th(      "DRU");	// 50
        th("DZAM-BU'I");	// 50
        th(    "GNUBS");	// 50
        th(    "GTUMS");	// 50
        th(     "GYAS");	// 50
        th(     "KE'U");	// 50
        th(    "KHRAM");	// 50
        th(  "'KHYAGS");	// 50
        th(    "LINGS");	// 50
        th(  "LITZTSA");	// 50
        th(   "MCHING");	// 50
        th(     "RBAB");	// 50
        th(    "RNYIS");	// 50
        th(   "SPYI'U");	// 50
        th(    "SRUBS");	// 50
        th(    "THIMS");	// 50
        th(       "WI");	// 50
        th(     "YENG");	// 50
        th(       "AN");	// 51
        th(    "BDUGS");	// 51
        th(    "BSHOR");	// 51
        th(    "BTZUG");	// 51
        th(   "'CHAMS");	// 51
        th(     "DREG");	// 51
        th(     "'GOL");	// 51
        th(     "GSUS");	// 51
        th(   "GZHANG");	// 51
        th(     "'JAB");	// 51
        th(      "KHI");	// 51
        th(    "'KHRO");	// 51
        th(     "KR'I");	// 51
        th(      "LEG");	// 51
        th(     "LHUB");	// 51
        th(      "MAm");	// 51
        th(     "NYIL");	// 51
        th(     "PHEB");	// 51
        th(    "PHRAR");	// 51
        th(     "RKUB");	// 51
        th(     "RNAm");	// 51
        th(   "RTZOMS");	// 51
        th(    "SBRON");	// 51
        th(   "SNYUNG");	// 51
        th(     "THEL");	// 51
        th(    "'TSEM");	// 51
        th(   "TZ'A'I");	// 51
        th(     "BCIL");	// 52
        th(    "BCUGS");	// 52
        th(   "BKRABS");	// 52
        th(   "BRNAGS");	// 52
        th(   "BSHEGS");	// 52
        th(   "BSKYAD");	// 52
        th(    "BSTUD");	// 52
        th(      "DYA");	// 52
        th(  "DZAMBHA");	// 52
        th(     "GZUG");	// 52
        th(     "'JAR");	// 52
        th(   "'KHRAB");	// 52
        th(   "'KHYAG");	// 52
        th(   "PADMAS");	// 52
        th(   "'PHUNG");	// 52
        th(     "SDAM");	// 52
        th("SENG-GE'I");	// 52
        th(     "SHUB");	// 52
        th(    "SNYON");	// 52
        th(      "tAM");	// 52
        th(     "THUD");	// 52
        th(    "TYA'I");	// 52
        th(     "TZEE");	// 52
        th(     "ZHAM");	// 52
        th(    "BGO'O");	// 53
        th(    "BSU'I");	// 53
        th(    "BYI'U");	// 53
        th(     "DGUM");	// 53
        th(    "'DRAN");	// 53
        th(     "'GAB");	// 53
        th(      "GOG");	// 53
        th(   "GSHIBS");	// 53
        th(   "GTZO'O");	// 53
        th(     "GZIS");	// 53
        th(     "HANG");	// 53
        th(     "'KHO");	// 53
        th(   "KHYI'U");	// 53
        th( "'KHYONGS");	// 53
        th(      "LOD");	// 53
        th(     "MI'U");	// 53
        th(      "MRi");	// 53
        th(   "NYUNGS");	// 53
        th(     "PHOB");	// 53
        th(     "SBAG");	// 53
        th(     "SE'I");	// 53
        th(      "SLE");	// 53
        th(     "SPED");	// 53
        th(    "TSIMS");	// 53
        th(    "BLDAG");	// 54
        th(  "BRGYAGS");	// 54
        th(   "'BUNGS");	// 54
        th(    "'BYES");	// 54
        th(    "'BYUR");	// 54
        th(     "CHIL");	// 54
        th(    "GDA'O");	// 54
        th(    "GRA'I");	// 54
        th(   "'GRANG");	// 54
        th(   "GSUGNS");	// 54
        th(     "GZOS");	// 54
        th(      "KOS");	// 54
        th(  "MAndALA");	// 54
        th(    "NANDA");	// 54
        th(      "NIS");	// 54
        th(    "PHYAD");	// 54
        th(   "PHYE'O");	// 54
        th(   "'PHYUG");	// 54
        th(    "RJE'O");	// 54
        th(    "RKYAN");	// 54
        th(    "SBRIN");	// 54
        th(   "SPRUGS");	// 54
        th(     "STEG");	// 54
        th(    "THEGS");	// 54
        th(   "TZARYA");	// 54
        th(     "WA'I");	// 54
        th(     "ZHED");	// 54
        th(   "BSGRUN");	// 55
        th(   "DRONGS");	// 55
        th(   "'DZEMS");	// 55
        th(    "KA'AM");	// 55
        th(   "KARshA");	// 55
        th(    "'KHEL");	// 55
        th(  "K'IRTIS");	// 55
        th(    "MDZED");	// 55
        th(        "O");	// 55
        th(      "RUG");	// 55
        th(     "SHID");	// 55
        th(  "SKYE'AM");	// 55
        th(     "TE'U");	// 55
        th(   "TSUNGS");	// 55
        th(      "TZU");	// 55
        th(    "BLAGS");	// 56
        th(  "BRGYA'O");	// 56
        th(   "BSKYEL");	// 56
        th(    "'DAGS");	// 56
        th(    "DBUNG");	// 56
        th(      "DHE");	// 56
        th(     "DREL");	// 56
        th(     "GRAB");	// 56
        th(     "GRAR");	// 56
        th(  "'GRO'AM");	// 56
        th(    "KVA'I");	// 56
        th(     "LHAB");	// 56
        th(     "MDAR");	// 56
        th(    "MKHOS");	// 56
        th(   "MTSE'U");	// 56
        th(    "SNOMS");	// 56
        th(   "SPRO'I");	// 56
        th(      "S'U");	// 56
        th(    "ZE'AM");	// 56
        th(      "ZIM");	// 56
        th(   "BAKVAN");	// 57
        th(     "'BEN");	// 57
        th(     "BKON");	// 57
        th(     "DGER");	// 57
        th(    "DNGAS");	// 57
        th(    "GLU'I");	// 57
        th(      "GUL");	// 57
        th(     "HUNG");	// 57
        th(    "KHU'I");	// 57
        th(     "LUMS");	// 57
        th(    "MCHOS");	// 57
        th(  "MTHONGS");	// 57
        th(  "RANYDZA");	// 57
        th(   "SBRING");	// 57
        th(    "SKRUN");	// 57
        th(    "SNGOS");	// 57
        th(     "SNYI");	// 57
        th(    "'TSIR");	// 57
        th(      "ZIR");	// 57
        th(      "ZLO");	// 57
        th(     "BAGS");	// 58
        th(      "BIS");	// 58
        th(    "BKRIS");	// 58
        th(    "BSNOR");	// 58
        th(  "BSRANGS");	// 58
        th(      "BTU");	// 58
        th(    "BZA'I");	// 58
        th(     "DGOG");	// 58
        th(    "GNYES");	// 58
        th(    "'GOGS");	// 58
        th(    "GSHES");	// 58
        th(     "GTA'");	// 58
        th(    "G-YUR");	// 58
        th(   "KHAndA");	// 58
        th(   "'KHRIL");	// 58
        th(     "KI'I");	// 58
        th( "K'IRTI'I");	// 58
        th(    "MDANG");	// 58
        th(   "MO'ANG");	// 58
        th(    "RDZES");	// 58
        th(    "SATVA");	// 58
        th(  "SHAMBHA");	// 58
        th(  "SHR'I'I");	// 58
        th(     "SKON");	// 58
        th(    "BCA'O");	// 59
        th(   "BKRU'O");	// 59
        th(     "BSUN");	// 59
        th(  "BYA'ANG");	// 59
        th(   "'BYANG");	// 59
        th(     "BZAS");	// 59
        th(     "'CHU");	// 59
        th(      "DRE");	// 59
        th("GANG+G'A'I");	// 59
        th(  "MKHRANG");	// 59
        th(   "MKHYAB");	// 59
        th(   "PADMOR");	// 59
        th(    "RTZWA");	// 59
        th(     "SKRi");	// 59
        th(   "SNYAMS");	// 59
        th( "TZAN-DAN");	// 59
        th(     "ZHUB");	// 59
        th(  "7SKYABS");	// 60
        th(      "AOR");	// 60
        th( "BRGYA'AM");	// 60
        th( "BSGRENGS");	// 60
        th(   "BSNGAM");	// 60
        th(     "BYER");	// 60
        th(     "BYID");	// 60
        th(     "CHIB");	// 60
        th(   "DBYANG");	// 60
        th(      "DOS");	// 60
        th(    "'GANG");	// 60
        th(     "GRAM");	// 60
        th(    "GSHEG");	// 60
        th(    "GTAGS");	// 60
        th(   "'GYANG");	// 60
        th(    "'GYES");	// 60
        th(  "GYI'ANG");	// 60
        th(    "'JOGS");	// 60
        th(      "KER");	// 60
        th(      "MA:");	// 60
        th(    "NYE'I");	// 60
        th(   "RTZANG");	// 60
        th(     "RVAS");	// 60
        th(     "SGAL");	// 60
        th(   "SKYOGS");	// 60
        th(     "SLOD");	// 60
        th(     "SRIB");	// 60
        th(  "t'IKKAR");	// 60
        th(   "^ZHABS");	// 60
        th(     "BSEL");	// 61
        th(    "BZHUS");	// 61
        th(    "CE'AM");	// 61
        th(   "'CHI'O");	// 61
        th(    "MTHEB");	// 61
        th(      "PHE");	// 61
        th(     "RKUR");	// 61
        th(   "SGRAGS");	// 61
        th(    "SGYUS");	// 61
        th(    "SNYEN");	// 61
        th(     "SPRE");	// 61
        th(     "SREL");	// 61
        th(        ":");	// 62
        th(  "AARGHAm");	// 62
        th(   "BADZRE");	// 62
        th(     "'BAN");	// 62
        th(    "BHYA:");	// 62
        th(    "BLA'O");	// 62
        th(    "'BRID");	// 62
        th( "'DRA'ANG");	// 62
        th(   "'DRI'O");	// 62
        th( "d'URYA'I");	// 62
        th(      "DZO");	// 62
        th(      "NAL");	// 62
        th( "PAD-MA'I");	// 62
        th(      "P'O");	// 62
        th(      "POG");	// 62
        th(     "RJAS");	// 62
        th(     "BGOD");	// 63
        th(    "BLONG");	// 63
        th(      "'BO");	// 63
        th(   "BSDUMS");	// 63
        th(      "CAL");	// 63
        th(    "DBYID");	// 63
        th(    "'GRIL");	// 63
        th(   "'KHYOR");	// 63
        th(    "KLO'I");	// 63
        th(  "'PHYUGS");	// 63
        th(    "RDUGS");	// 63
        th(     "RE'O");	// 63
        th(    "SA-HO");	// 63
        th(  "SKRANGS");	// 63
        th(      "ZOG");	// 63
        th(   "7RGYAL");	// 64
        th("BEEd'URYA");	// 64
        th(    "BSDUN");	// 64
        th(    "BSENG");	// 64
        th(   "BSTONG");	// 64
        th(    "BZA'O");	// 64
        th(    "BZHOG");	// 64
        th(   "BZLOGS");	// 64
        th(     "CO'I");	// 64
        th(    "'DRIR");	// 64
        th(     "GTAR");	// 64
        th(     "GYOD");	// 64
        th(     "LTOR");	// 64
        th(     "MDAS");	// 64
        th(    "MNABS");	// 64
        th(     "PHOL");	// 64
        th(  "PHYUNGS");	// 64
        th(      "RAD");	// 64
        th(    "RGYAM");	// 64
        th(     "RKUS");	// 64
        th(     "RLOG");	// 64
        th(    "RLOMS");	// 64
        th(     "SBRO");	// 64
        th(   "SINGHA");	// 64
        th(    "SPUNG");	// 64
        th(     "BCAN");	// 65
        th(   "BSGO'O");	// 65
        th(    "BTZOG");	// 65
        th(    "CHOMS");	// 65
        th(      "d'U");	// 65
        th(     "DZAS");	// 65
        th(      "DZU");	// 65
        th(     "GROD");	// 65
        th(      "LBA");	// 65
        th(     "SAMS");	// 65
        th(    "SDUMS");	// 65
        th(      "BA'");	// 66
        th(      "BLU");	// 66
        th(     "BRED");	// 66
        th(  "BSTZAGS");	// 66
        th(     "BZHU");	// 66
        th(      "CER");	// 66
        th(    "DGE'O");	// 66
        th(  "DPYANGS");	// 66
        th(     "GCEN");	// 66
        th(    "GRO'I");	// 66
        th(  "KULLE'I");	// 66
        th(    "LNGAN");	// 66
        th(    "MCHIN");	// 66
        th(     "RGUR");	// 66
        th(      "ROR");	// 66
        th(     "SGRE");	// 66
        th(     "SRUN");	// 66
        th(     "TANG");	// 66
        th( "A'A:H'Um");	// 67
        th(    "BRDOL");	// 67
        th(  "BSKYUNG");	// 67
        th(    "BSNON");	// 67
        th(    "DGRAM");	// 67
        th(    "GNONG");	// 67
        th(     "GTZE");	// 67
        th(   "GTZO'I");	// 67
        th(     "'JOM");	// 67
        th(      "NIM");	// 67
        th(    "RTZON");	// 67
        th(      "RWA");	// 67
        th(    "SGRIM");	// 67
        th(    "'THOG");	// 67
        th(    "'CHES");	// 68
        th(     "CHIS");	// 68
        th(    "GSENG");	// 68
        th(    "GZHAR");	// 68
        th(   "MKHO'I");	// 68
        th(    "RTZEG");	// 68
        th(     "SAGS");	// 68
        th(    "SNGAN");	// 68
        th(      "SOd");	// 68
        th(     "SPUD");	// 68
        th(     "STOS");	// 68
        th(        "8");	// 69
        th(    "BZHEN");	// 69
        th(    "BZHOS");	// 69
        th(     "DBUM");	// 69
        th(  "DHARM'A");	// 69
        th(    "'DRER");	// 69
        th(     "GLAL");	// 69
        th(    "GRIMS");	// 69
        th(     "GTOS");	// 69
        th(  "G-YANGS");	// 69
        th(    "'JING");	// 69
        th(   "'JUNGS");	// 69
        th(    "LDONG");	// 69
        th(     "LJAN");	// 69
        th(     "LOBS");	// 69
        th(    "PAntI");	// 69
        th(     "SHRA");	// 69
        th(     "SPEN");	// 69
        th(     "SPUR");	// 69
        th(    "TZANG");	// 69
        th(      "BHO");	// 70
        th(   "BRJE'I");	// 70
        th(    "BSKYE");	// 70
        th(     "'GYU");	// 70
        th(    "'KHOL");	// 70
        th(   "KHRA'I");	// 70
        th(    "KHYOR");	// 70
        th(     "LJIN");	// 70
        th(      "SBO");	// 70
        th(  "SENG-GE");	// 70
        th(     "TU'I");	// 70
        th(      "TVA");	// 70
        th(     "TZUN");	// 70
        th(      "AUD");	// 71
        th(     "BHAm");	// 71
        th(   "BTZAMS");	// 71
        th(     "DBA'");	// 71
        th(    "'DU'O");	// 71
        th(   "GA'ANG");	// 71
        th(     "GCOG");	// 71
        th(     "GLOS");	// 71
        th(     "GNA'");	// 71
        th(   "'GRIMS");	// 71
        th(    "GSING");	// 71
        th(      "HO:");	// 71
        th(    "KLAGS");	// 71
        th(     "LO'O");	// 71
        th(      "RAm");	// 71
        th(    "RTZAB");	// 71
        th(   "SHANTI");	// 71
        th(    "SPRUG");	// 71
        th(     "STIS");	// 71
        th(   "THENGS");	// 71
        th(  "AUTPALA");	// 72
        th(     "BCOD");	// 72
        th(     "'BOL");	// 72
        th( "BSKYUNGS");	// 72
        th(    "'BYAM");	// 72
        th(    "GI'AM");	// 72
        th(    "'GRAL");	// 72
        th(    "'GROD");	// 72
        th(  "GZHI'AM");	// 72
        th(      "PAG");	// 72
        th(      "P'I");	// 72
        th(      "PIR");	// 72
        th(      "R'I");	// 72
        th(        "S");	// 72
        th(      "SDA");	// 72
        th(        "9");	// 73
        th(     "'BOR");	// 73
        th(   "BSKO'O");	// 73
        th(   "BSKYON");	// 73
        th(      "BUB");	// 73
        th(   "CHANGS");	// 73
        th(      "CON");	// 73
        th(    "DRUBS");	// 73
        th(   "GA-YAN");	// 73
        th(     "GTUB");	// 73
        th(    "GZHIL");	// 73
        th(     "KANG");	// 73
        th(    "MTHUD");	// 73
        th(        "N");	// 73
        th(     "NYOG");	// 73
        th(     "SE'U");	// 73
        th(   "SGRIBS");	// 73
        th(    "SNGAM");	// 73
        th(  "SPYANGS");	// 73
        th(       "10");	// 74
        th(   "BO'ANG");	// 74
        th(   "BSKUMS");	// 74
        th(    "GE'AM");	// 74
        th(    "GSHA'");	// 74
        th(    "NA'AM");	// 74
        th(   "'PHROR");	// 74
        th(    "RTZIB");	// 74
        th(      "SAB");	// 74
        th(   "SBANGS");	// 74
        th("SHRADDH'A");	// 74
        th(   "SKYIBS");	// 74
        th(   "SPYI'O");	// 74
        th(   "TSVA'I");	// 74
        th(    "BGRAD");	// 75
        th(    "BLTAB");	// 75
        th(     "BSER");	// 75
        th(    "BTZAG");	// 75
        th(     "BYAN");	// 75
        th(    "'CHEL");	// 75
        th(    "DBYES");	// 75
        th(     "'GOM");	// 75
        th(      "HYA");	// 75
        th(    "'JO'I");	// 75
        th(    "MKHUR");	// 75
        th(      "NUD");	// 75
        th(     "SBEL");	// 75
        th(   "SINDHU");	// 75
        th(   "'BRENG");	// 76
        th(  "BRGYUGS");	// 76
        th(   "BRTZES");	// 76
        th(     "BTAG");	// 76
        th(    "DBROG");	// 76
        th(    "DRA'I");	// 76
        th(        "G");	// 76
        th(     "GDOS");	// 76
        th(    "'KHOS");	// 76
        th(      "RDE");	// 76
        th(     "THAS");	// 76
        th(     "TY'A");	// 76
        th(     "ZAGS");	// 76
        th(     "BKAD");	// 77
        th(       "dA");	// 77
        th(    "'DZA'");	// 77
        th(      "KA:");	// 77
        th(     "KA'O");	// 77
        th(     "LDAS");	// 77
        th(      "NAm");	// 77
        th(     "shYA");	// 77
        th(    "SKEMS");	// 77
        th(      "SLO");	// 77
        th(     "SLOG");	// 77
        th(    "SNE'U");	// 77
        th(     "SPAL");	// 77
        th(      "TSI");	// 77
        th(    "BSDOM");	// 78
        th(     "DMOD");	// 78
        th(     "GYOS");	// 78
        th(    "KLUBS");	// 78
        th(     "SKAN");	// 78
        th(   "'THIBS");	// 78
        th(     "TSOG");	// 78
        th(     "'BAG");	// 79
        th(     "DPYA");	// 79
        th(     "GDAL");	// 79
        th(   "'KHYUG");	// 79
        th(  "MKHREGS");	// 79
        th(    "MTSER");	// 79
        th(   "'PHO'I");	// 79
        th(      "ROD");	// 79
        th(      "RUB");	// 79
        th(  "SKYENGS");	// 79
        th(   "SLONGS");	// 79
        th(     "tA'I");	// 79
        th(      "TAR");	// 79
        th(     "BDES");	// 80
        th(   "BRDZES");	// 80
        th(   "BSGRES");	// 80
        th(     "'DOG");	// 80
        th(     "DRED");	// 80
        th(    "GSO'I");	// 80
        th(   "GTZUBS");	// 80
        th(   "'PHYAN");	// 80
        th(     "RMOR");	// 80
        th(    "RNAmS");	// 80
        th(     "SBOS");	// 80
        th(    "SPOGS");	// 80
        th(    "7GONG");	// 81
        th(    "BKYON");	// 81
        th(    "BRDAB");	// 81
        th(   "BRNGAS");	// 81
        th(     "'BRO");	// 81
        th(     "BSDO");	// 81
        th(    "BSHUL");	// 81
        th(    "BZHAR");	// 81
        th(    "'CHIR");	// 81
        th(      "GEL");	// 81
        th(     "GO'U");	// 81
        th(     "GTUR");	// 81
        th(     "RNAR");	// 81
        th(   "RNGA'I");	// 81
        th(   "'THABS");	// 81
        th(    "THENG");	// 81
        th(    "tIKKA");	// 81
        th(     "DBAL");	// 82
        th(     "DBYU");	// 82
        th(      "DIR");	// 82
        th(     "'DUR");	// 82
        th( "GANGGA'I");	// 82
        th(   "GA-YEM");	// 82
        th(     "GYIR");	// 82
        th(    "LE'US");	// 82
        th(      "LVA");	// 82
        th(    "MNGAS");	// 82
        th(   "RKYONG");	// 82
        th(   "SKYANG");	// 82
        th(    "SONGS");	// 82
        th(     "SPUG");	// 82
        th(   "STENGS");	// 82
        th(    "THU'I");	// 82
        th(     "BDA'");	// 83
        th(    "DRI'O");	// 83
        th(    "DWAGS");	// 83
        th(     "DZA:");	// 83
        th(    "'DZUG");	// 83
        th(   "LHA'AM");	// 83
        th(     "MYUL");	// 83
        th(      "P'U");	// 83
        th(    "SLOBS");	// 83
        th(     "SMRi");	// 83
        th(     "SMUG");	// 83
        th(    "SNGOR");	// 83
        th(     "THEB");	// 83
        th(     "THOL");	// 83
        th(     "ZEGS");	// 83
        th(   "BA'I'O");	// 84
        th(     "BDAL");	// 84
        th(     "'DED");	// 84
        th(       "dI");	// 84
        th(   "GA-YAR");	// 84
        th(    "PHYES");	// 84
        th(   "RE'ANG");	// 84
        th(    "SBONG");	// 84
        th(   "SBYI'I");	// 84
        th(    "SBYON");	// 84
        th(     "SPAM");	// 84
        th(    "ZHOGS");	// 84
        th(   "BRGYUG");	// 85
        th(    "BSGUL");	// 85
        th(    "BSRAN");	// 85
        th(    "BSRUN");	// 85
        th(      "DBO");	// 85
        th(   "'DZI'I");	// 85
        th(     "HRUL");	// 85
        th(      "LAD");	// 85
        th(      "LER");	// 85
        th(      "M'I");	// 85
        th(    "PHOGS");	// 85
        th(     "TZOM");	// 85
        th(     "BLAS");	// 86
        th(   "BRTZOM");	// 86
        th(   "GZHUGS");	// 86
        th(    "KHRIS");	// 86
        th(    "MTSUR");	// 86
        th(     "nA'I");	// 86
        th(  "SGRA'AM");	// 86
        th(    "BSDOD");	// 87
        th(     "BZAR");	// 87
        th(   "'CHI'I");	// 87
        th(    "GAndI");	// 87
        th(     "GO'I");	// 87
        th(    "GZABS");	// 87
        th(    "'JEBS");	// 87
        th(      "RME");	// 87
        th(     "TSAM");	// 87
        th(  "TZ'ARYA");	// 87
        th(   "BIDY'A");	// 88
        th(    "BSHAR");	// 88
        th(   "'DZUMS");	// 88
        th(   "GARBHA");	// 88
        th(    "KHRAB");	// 88
        th(     "MO'U");	// 88
        th(      "PAL");	// 88
        th(   "RTZIGS");	// 88
        th(   "SIDDHI");	// 88
        th(     "SI'I");	// 88
        th(      "AER");	// 89
        th(     "BDAN");	// 89
        th(    "BLTAM");	// 89
        th(   "BYANGS");	// 89
        th(     "GTOL");	// 89
        th(    "GYANG");	// 89
        th(     "MDAG");	// 89
        th(     "MTSE");	// 89
        th(      "NEE");	// 89
        th(      "SKO");	// 89
        th(     "TSAS");	// 89
        th(     "BLAN");	// 90
        th(      "d'A");	// 90
        th(     "DBRI");	// 90
        th(     "GCAL");	// 90
        th(     "GRUM");	// 90
        th(     "NYUL");	// 90
        th(     "RLAG");	// 90
        th(    "SKYIN");	// 90
        th(      "YAD");	// 90
        th(     "BSKA");	// 91
        th(    "DE'AM");	// 91
        th(   "GTZIGS");	// 91
        th(      "HAL");	// 91
        th(     "'JUS");	// 91
        th(     "MNOG");	// 91
        th(     "RLAB");	// 91
        th(    "SDIGS");	// 91
        th(      "SDU");	// 91
        th(     "SHEG");	// 91
        th(   "TZITTA");	// 91
        th(    "BRTUN");	// 92
        th(   "BRTZOD");	// 92
        th(  "BSNGO'I");	// 92
        th(    "BZLOS");	// 92
        th(        "D");	// 92
        th(     "DBAR");	// 92
        th(      "GRE");	// 92
        th(    "GSHOM");	// 92
        th(     "GZUD");	// 92
        th(      "HIN");	// 92
        th(     "'JOL");	// 92
        th(     "KshA");	// 92
        th(    "RNGAM");	// 92
        th(     "SNUN");	// 92
        th(     "SPOR");	// 92
        th(   "SPYONG");	// 92
        th(     "THUS");	// 92
        th(     "TSEN");	// 92
        th(  "BSKYABS");	// 93
        th(    "DRE'U");	// 93
        th(      "DUN");	// 93
        th(   "KHRUMS");	// 93
        th(     "KYOG");	// 93
        th(      "NAB");	// 93
        th(     "RDOS");	// 93
        th(     "RGAD");	// 93
        th(  "RGYU'AM");	// 93
        th(    "SPYAR");	// 93
        th(    "BLO'O");	// 94
        th(      "'DO");	// 94
        th(    "GCONG");	// 94
        th(   "GI'ANG");	// 94
        th(    "GLONG");	// 94
        th(   "'GREMS");	// 94
        th(    "GTUBS");	// 94
        th(     "LHAD");	// 94
        th(     "NGOM");	// 94
        th(    "SDEBS");	// 94
        th(    "S'I'I");	// 94
        th(     "SNOM");	// 94
        th(      "SRO");	// 94
        th(      "AOO");	// 95
        th(        "B");	// 95
        th(    "BTZIR");	// 95
        th(   "BTZUMS");	// 95
        th(    "GUPTA");	// 95
        th(     "KHAN");	// 95
        th(    "LTANG");	// 95
        th(     "SDAR");	// 95
        th(     "'BAL");	// 96
        th(   "BSNYAL");	// 96
        th( "BSNYENGS");	// 96
        th(     "'GAN");	// 96
        th(     "GZEB");	// 96
        th(     "KHOD");	// 96
        th(   "'KHOGS");	// 96
        th(    "LHO'I");	// 96
        th(     "LKUG");	// 96
        th(     "NGER");	// 96
        th(      "RMO");	// 96
        th(   "SHR'IS");	// 96
        th(   "SPRO'O");	// 96
        th(    "STE'O");	// 96
        th(    "STE'U");	// 96
        th(    "'THAS");	// 96
        th(      "YOG");	// 96
        th(      "AIN");	// 97
        th(   "BRNYED");	// 97
        th(      "DKU");	// 97
        th(    "GSHAG");	// 97
        th(   "GZHIGS");	// 97
        th(     "LDEG");	// 97
        th(     "SLAB");	// 97
        th(    "SMYUG");	// 97
        th(     "DKRI");	// 98
        th(     "HA'I");	// 98
        th(     "SGUG");	// 98
        th(     "SRUL");	// 98
        th(     "DBOG");	// 99
        th(    "'DREG");	// 99
        th(     "GTUN");	// 99
        th(      "JAG");	// 99
        th(  "LOTZTSA");	// 99
        th(  "SBYONGS");	// 99
        th(     "SKEM");	// 99
    }

    /** Tests the tsheg bars that appear [100, 999] times. */
    public void test100to999() {
        th(    "BDE'O");	// 100
        th(    "BRDAL");	// 100
        th(   "BRLAMS");	// 100
        th(    "BSTAB");	// 100
        th(   "'KHRUD");	// 100
        th(   "KOOndI");	// 100
        th(    "NGU'O");	// 100
        th(    "RNGUL");	// 100
        th(     "SBUN");	// 100
        th(   "SNGA'I");	// 100
        th(    "SPYON");	// 100
        th(  "BSNYUNG");	// 101
        th(     "DGAL");	// 101
        th(     "DROR");	// 101
        th(     "GSOB");	// 101
        th(     "SBUR");	// 101
        th(   "SDANGS");	// 101
        th("SH'A-KYA'I");	// 101
        th(    "SHU'O");	// 101
        th(      "ZES");	// 101
        th(     "CHAM");	// 102
        th(   "'GANGS");	// 102
        th(   "'KHYOG");	// 102
        th(      "KUM");	// 102
        th(     "LCUG");	// 102
        th(    "MO'AM");	// 102
        th(    "MTHOD");	// 102
        th(   "'PHYIS");	// 102
        th(    "'PHYO");	// 102
        th(    "TRA'I");	// 102
        th(    "HR'I:");	// 103
        th(    "RMANG");	// 103
        th(    "SDOGS");	// 103
        th(    "SGO'O");	// 103
        th(     "SKED");	// 103
        th(    "'TSOG");	// 103
        th(      "ZON");	// 103
        th(      "BER");	// 104
        th(     "BSUR");	// 104
        th(    "DENGS");	// 104
        th(    "'GEBS");	// 104
        th(   "'KHROL");	// 104
        th(    "MDOMS");	// 104
        th(      "SKE");	// 104
        th(     "SMAG");	// 104
        th(      "YUR");	// 104
        th(   "BSNYAN");	// 105
        th(     "DRUD");	// 105
        th(     "GRUL");	// 105
        th(      "GUG");	// 105
        th(       "JU");	// 105
        th( "KHYE'U'I");	// 105
        th(    "LTOGS");	// 105
        th(    "MYAGS");	// 105
        th(     "RGUN");	// 105
        th(    "SHI'O");	// 105
        th(     "SNGE");	// 105
        th(      "TEE");	// 105
        th(   "BRDABS");	// 106
        th(   "BRDZUS");	// 106
        th(   "DKRIGS");	// 106
        th(    "GRU'I");	// 106
        th(    "MTHOB");	// 106
        th(    "BSOGS");	// 107
        th(     "BSTE");	// 107
        th(    "KHUGS");	// 107
        th(     "LJID");	// 107
        th(   "MANGGA");	// 107
        th(    "NA-YA");	// 107
        th(     "RBAD");	// 107
        th(    "SMRAR");	// 107
        th(      "WAS");	// 107
        th(     "BZHO");	// 108
        th(    "GRUNG");	// 108
        th(     "'JAG");	// 108
        th(     "JA'I");	// 108
        th(   "'KHRIS");	// 108
        th(     "SGOG");	// 108
        th(     "STES");	// 108
        th(    "'THUD");	// 108
        th(    "'TSUB");	// 108
        th(    "ZHU'I");	// 108
        th( "BSGYINGS");	// 109
        th(  "BSNGAMS");	// 109
        th( "BSNYUNGS");	// 109
        th(    "DBU'I");	// 109
        th(      "LDI");	// 109
        th(     "LDUM");	// 109
        th(    "SBANG");	// 109
        th(  "SENG+GE");	// 109
        th(      "WAm");	// 109
        th(   "BRLANG");	// 110
        th(    "BSKAM");	// 110
        th(      "TIS");	// 110
        th(      "CAM");	// 111
        th(    "DGA'O");	// 111
        th(     "DPEN");	// 111
        th(     "KHVA");	// 111
        th(     "LCES");	// 111
        th(   "PHYING");	// 111
        th(      "SDO");	// 111
        th(     "SMAR");	// 111
        th(     "BLAR");	// 112
        th(    "BSTON");	// 112
        th(      "CEN");	// 112
        th(     "GSEG");	// 112
        th(   "KHRUNG");	// 112
        th(   "KHYI'I");	// 112
        th(     "MGOS");	// 112
        th(   "MKHYUD");	// 112
        th(      "MUD");	// 112
        th(    "SNYIM");	// 112
        th(     "TRAS");	// 112
        th(     "TSAG");	// 112
        th(     "BSRO");	// 113
        th(     "DGUG");	// 113
        th(  "LTA'ANG");	// 113
        th(    "MTHOR");	// 113
        th(   "THONGS");	// 113
        th(      "A'I");	// 114
        th(     "DNAG");	// 114
        th(     "MNYE");	// 114
        th(    "PHANG");	// 114
        th(     "RTAS");	// 114
        th(      "RTI");	// 114
        th(     "S'IR");	// 114
        th(    "BTZUD");	// 115
        th(      "CAS");	// 115
        th(      "KLO");	// 115
        th(    "MRA'I");	// 115
        th(     "RLON");	// 115
        th(     "SHUS");	// 115
        th(   "SKYEMS");	// 115
        th(    "BSTIM");	// 116
        th(    "CHABS");	// 116
        th(     "CHIN");	// 116
        th(    "DPYIS");	// 116
        th(    "'DRIM");	// 116
        th(     "GCIN");	// 116
        th(    "GYONG");	// 116
        th(    "SGRIL");	// 116
        th(    "BCAGS");	// 117
        th(     "KHYO");	// 117
        th(    "KYA'I");	// 117
        th(      "LIN");	// 117
        th(   "BRTZER");	// 118
        th(    "CHA'O");	// 118
        th(      "DOL");	// 118
        th(  "'DRONGS");	// 118
        th(        "I");	// 118
        th(      "MEL");	// 118
        th(    "PHU'I");	// 118
        th(     "BDAS");	// 119
        th(     "'BUS");	// 119
        th(      "KRA");	// 119
        th(     "LDON");	// 119
        th(      "LUM");	// 119
        th(    "MTSAL");	// 119
        th(     "RMOS");	// 119
        th(      "SOD");	// 119
        th(   "BSGRIG");	// 120
        th(    "'DRID");	// 120
        th(    "'DRIN");	// 120
        th(      "G'A");	// 120
        th(   "K'IRTI");	// 120
        th(   "MTHU'I");	// 120
        th(   "PHYE'I");	// 120
        th(      "RON");	// 120
        th(      "RYA");	// 120
        th(    "SKAMS");	// 120
        th(     "TSEM");	// 120
        th(     "YUNG");	// 120
        th(  "BSNGOGS");	// 121
        th(      "DEN");	// 121
        th(    "DKROL");	// 121
        th(     "DZAM");	// 121
        th(  "'GYANGS");	// 121
        th(      "K'I");	// 121
        th(     "nI'I");	// 121
        th(   "'PHYUR");	// 121
        th(      "RDU");	// 121
        th(    "SGROM");	// 121
        th(    "SHAGS");	// 121
        th(      "ZUM");	// 121
        th(     "BCAB");	// 122
        th(    "BO'AM");	// 122
        th(   "BSNYUN");	// 122
        th(    "SBRAN");	// 122
        th(    "SPYOR");	// 122
        th(     "TSED");	// 122
        th(      "YUS");	// 122
        th(      "CUS");	// 123
        th(     "'DAN");	// 123
        th(      "DMU");	// 123
        th(     "GRUR");	// 123
        th(     "GSAG");	// 123
        th(    "G-YAN");	// 123
        th(   "G-YO'I");	// 123
        th(  "KYI'ANG");	// 123
        th(       "Ri");	// 123
        th(     "RMIG");	// 123
        th(      "SAL");	// 123
        th(    "'TSOD");	// 123
        th(     "BKAN");	// 124
        th(    "DKRUG");	// 124
        th(    "DPOGS");	// 124
        th(    "GRABS");	// 124
        th(    "MTHUG");	// 124
        th(      "n'A");	// 124
        th(    "NYANG");	// 124
        th(    "SGROS");	// 124
        th(    "'TSOR");	// 124
        th(    "BGYIR");	// 125
        th(     "BKUM");	// 125
        th(     "DH'U");	// 125
        th(  "DNGANGS");	// 125
        th(    "DPA'O");	// 125
        th("GANG-G'A'I");	// 125
        th(     "GA'O");	// 125
        th(    "GSONG");	// 125
        th(     "PHRI");	// 125
        th(  "RDZONGS");	// 125
        th(    "RJE'U");	// 125
        th(     "SHOL");	// 125
        th(    "SKYAS");	// 125
        th(     "ZHIS");	// 125
        th(      "BYU");	// 126
        th(   "GSHONG");	// 126
        th(   "'KHYAL");	// 126
        th(      "RID");	// 126
        th(   "SH'AKA");	// 126
        th(      "SRE");	// 126
        th(      "'DE");	// 127
        th(   "'DINGS");	// 127
        th(   "'GYING");	// 127
        th(     "LHOR");	// 127
        th(        "R");	// 127
        th(    "SMYIG");	// 127
        th(      "AAM");	// 128
        th(    "BRIMS");	// 128
        th(   "'BRONG");	// 128
        th(     "BSAB");	// 128
        th(     "BSOS");	// 128
        th(   "DNGANG");	// 128
        th(   "DPANGS");	// 128
        th(     "STAR");	// 128
        th(     "'BEL");	// 129
        th(    "BRTOL");	// 129
        th(    "BSHOS");	// 129
        th(     "'DUM");	// 129
        th(      "GAG");	// 129
        th(  "GA-YANG");	// 129
        th(     "'GEL");	// 129
        th(     "GSES");	// 129
        th(     "LTAN");	// 129
        th(     "SNAD");	// 129
        th(   "BSRUBS");	// 130
        th(      "DEB");	// 130
        th(  "'DZINGS");	// 130
        th(     "GSIL");	// 130
        th(     "KYUS");	// 130
        th(      "SIR");	// 130
        th(   "SLANGS");	// 130
        th(    "'THUL");	// 130
        th(    "'BABS");	// 131
        th(    "BRAGS");	// 131
        th(    "BSDUR");	// 131
        th(   "BSGRIB");	// 131
        th(   "BSRANG");	// 131
        th(     "CHEM");	// 131
        th(   "DBYE'O");	// 131
        th(    "G-YAG");	// 131
        th(     "GYIN");	// 131
        th(     "MDON");	// 131
        th(    "GCAGS");	// 132
        th(    "GCUNG");	// 132
        th(    "GNYIG");	// 132
        th(    "GZHIN");	// 132
        th(   "LDONGS");	// 132
        th(     "PHYA");	// 132
        th(    "ZHUNG");	// 132
        th(    "'DRAL");	// 133
        th(   "d'URYA");	// 133
        th(    "GTZUB");	// 133
        th(   "'KHUMS");	// 133
        th(    "'PHYI");	// 133
        th(     "SDON");	// 133
        th(   "SKYEGS");	// 133
        th(     "ZE'U");	// 133
        th(    "'BREG");	// 134
        th(    "BSGRE");	// 134
        th(    "BZHAN");	// 134
        th(     "DZ'A");	// 134
        th(   "GRONGS");	// 134
        th( "SENGGE'I");	// 134
        th(    "'BAGS");	// 135
        th(      "DZE");	// 135
        th(  "BTZONGS");	// 136
        th(      "COR");	// 136
        th(    "GSHOL");	// 136
        th(     "MDZO");	// 136
        th(      "PA'");	// 136
        th(     "PHON");	// 136
        th(      "RBA");	// 136
        th(     "BLUD");	// 137
        th(     "BSAN");	// 137
        th(      "DOM");	// 137
        th(    "NANGS");	// 137
        th(   "^RGYAL");	// 137
        th(    "RTZES");	// 137
        th(    "SBRAM");	// 137
        th(     "SPAS");	// 137
        th(    "7YANG");	// 138
        th(     "CHAL");	// 138
        th(      "MOL");	// 138
        th(     "BKYE");	// 139
        th(   "BRLING");	// 139
        th(    "DNGAR");	// 139
        th(     "PE'I");	// 139
        th(     "TI'O");	// 139
        th(      "TZO");	// 139
        th(     "BRID");	// 140
        th(      "DHU");	// 140
        th(    "DU'AM");	// 140
        th(   "'PHYAS");	// 140
        th(   "RTZING");	// 140
        th(    "SBRUM");	// 140
        th(    "SGREN");	// 140
        th(     "THUM");	// 140
        th(    "TSE'O");	// 140
        th(     "BLUG");	// 141
        th(  "BSNYEGS");	// 141
        th(     "DBUR");	// 141
        th(   "BSGRIM");	// 142
        th(     "DING");	// 142
        th(   "MNYAMS");	// 142
        th(    "ZHO'I");	// 142
        th(     "BCVO");	// 143
        th(    "BTZA'");	// 143
        th(    "MCHAD");	// 143
        th(  "RNYINGS");	// 143
        th(      "SME");	// 143
        th(     "CHUL");	// 144
        th(    "DGANG");	// 144
        th(     "STOB");	// 144
        th(     "YA'I");	// 144
        th(     "BRKO");	// 145
        th(      "KRI");	// 145
        th(   "LENDRA");	// 145
        th(     "SKOL");	// 145
        th(    "BGROS");	// 146
        th(   "BSGRIL");	// 146
        th(    "'DRIL");	// 146
        th(    "'GRIM");	// 146
        th(    "MDA'I");	// 146
        th(    "PHO'I");	// 146
        th(    "BRE'U");	// 147
        th(   "'GRE'O");	// 147
        th(   "RNGAMS");	// 147
        th(    "'THON");	// 147
        th(    "DBYAD");	// 148
        th(     "'GED");	// 148
        th(   "GSHAGS");	// 148
        th(    "MNGAG");	// 148
        th(    "RNGOG");	// 148
        th(    "SA'AM");	// 148
        th(   "SVASTI");	// 148
        th(    "BSHAN");	// 149
        th(   "CHUNGS");	// 149
        th(     "GRIS");	// 149
        th(     "LE'I");	// 149
        th(    "SNABS");	// 149
        th(     "SPUS");	// 149
        th(      "AAN");	// 150
        th(   "A'ARYA");	// 150
        th(     "BSUS");	// 150
        th(    "KHO'I");	// 150
        th(     "KHOS");	// 150
        th(    "RDUNG");	// 150
        th(     "SDUM");	// 150
        th(      "SGA");	// 150
        th(     "BHYA");	// 151
        th(   "BSDEBS");	// 151
        th(    "BSHAL");	// 151
        th(    "'CHOR");	// 151
        th(   "DGA'AS");	// 151
        th(     "MGOR");	// 151
        th(    "SKYUG");	// 151
        th(    "'GA'I");	// 152
        th(      "MRA");	// 152
        th(   "RLANGS");	// 152
        th(     "BCAL");	// 153
        th(     "DBYI");	// 153
        th(     "DRUB");	// 153
        th(      "'JI");	// 153
        th(  "LNGA'AM");	// 153
        th(    "SPYOS");	// 153
        th(     "'BEB");	// 154
        th(      "BUN");	// 154
        th(    "LA'AM");	// 154
        th(     "SKYI");	// 154
        th(    "DPE'O");	// 155
        th(    "GDANG");	// 155
        th(   "KHRO'I");	// 155
        th(     "KLAG");	// 155
        th(   "PA'I'O");	// 155
        th(   "PHYAGS");	// 155
        th(     "SPRA");	// 155
        th(    "BSKUM");	// 156
        th(    "DPRAL");	// 156
        th(      "GOD");	// 156
        th(   "RENDRA");	// 156
        th(  "RTZVA'I");	// 156
        th(     "YAMS");	// 156
        th(     "ZHUR");	// 156
        th(   "BSKYUR");	// 157
        th(    "DBRAL");	// 157
        th(      "DPA");	// 157
        th(      "HVA");	// 157
        th(    "RMA'I");	// 157
        th(     "SDUS");	// 157
        th(     "SROD");	// 157
        th(  "GZHUNGS");	// 158
        th(    "SLANG");	// 158
        th(      "BAm");	// 159
        th(    "'DZER");	// 159
        th(   "KHYOGS");	// 159
        th(     "ROGS");	// 159
        th(    "SBYAD");	// 159
        th(     "SHOM");	// 159
        th(     "SKOS");	// 159
        th(      "'BU");	// 160
        th(   "CHINGS");	// 160
        th(    "DRI'I");	// 160
        th(      "MEN");	// 160
        th(      "NEM");	// 160
        th(     "BREG");	// 161
        th(    "SGOMS");	// 161
        th(      "SYA");	// 161
        th(    "BRTAD");	// 162
        th(    "GZHES");	// 162
        th(      "N'I");	// 162
        th(    "'TSOS");	// 162
        th(      "GIN");	// 163
        th(    "MGO'I");	// 163
        th(  "'PHONGS");	// 163
        th(     "RDUM");	// 163
        th(    "RVA'I");	// 163
        th(      "BDO");	// 164
        th(     "BGRO");	// 164
        th(     "BH'A");	// 164
        th(   "BRGYUS");	// 164
        th(   "SGRA'O");	// 164
        th(     "BH'U");	// 165
        th(    "BSHUS");	// 165
        th(   "BSRABS");	// 165
        th(   "BU'ANG");	// 165
        th(     "HRIL");	// 165
        th(   "SNYEMS");	// 165
        th(       "'A");	// 166
        th(     "DGAB");	// 166
        th(  "'GRANGS");	// 166
        th(      "MEE");	// 166
        th(    "SGONG");	// 166
        th(      "S'I");	// 166
        th(    "STI'I");	// 166
        th(    "BRTAS");	// 167
        th(    "'DABS");	// 167
        th(    "G-YAB");	// 167
        th(    "RTZIR");	// 167
        th(   "GA-YOS");	// 168
        th(    "'CHAM");	// 169
        th(    "G-YA'");	// 169
        th(     "SHA'");	// 169
        th(      "SRU");	// 169
        th(     "ZHOL");	// 169
        th(    "BDZRA");	// 170
        th(     "GLOD");	// 170
        th(     "GSUm");	// 170
        th(     "KHEL");	// 170
        th(     "PHUB");	// 170
        th(     "RMEL");	// 170
        th(    "DANGS");	// 171
        th(      "DAS");	// 171
        th(   "GA-YON");	// 171
        th(    "'GRUL");	// 171
        th(     "NYED");	// 171
        th(     "'OBS");	// 171
        th(      "RGU");	// 171
        th(    "BRNAG");	// 172
        th(     "GDUB");	// 172
        th(    "KHRAL");	// 172
        th(  "TSE'ANG");	// 172
        th(    "SBYAN");	// 173
        th(     "GZUR");	// 174
        th(     "PHAt");	// 174
        th(      "RDA");	// 174
        th(     "ZHIR");	// 174
        th(      "AAR");	// 175
        th(    "'GRAS");	// 175
        th(     "LI'I");	// 175
        th(   "RGYUGS");	// 175
        th(      "SAN");	// 175
        th(    "SKYOD");	// 175
        th(     "MNA'");	// 176
        th(     "MO'O");	// 176
        th(       "PE");	// 176
        th(    "PHYAR");	// 176
        th(    "RGU'I");	// 176
        th(    "RGYAD");	// 176
        th(   "SENGGE");	// 176
        th(   "SKRA'I");	// 176
        th(    "'THUG");	// 176
        th(   "DNGAGS");	// 177
        th(     "'KHU");	// 177
        th(     "CHOL");	// 178
        th(    "DRAGS");	// 178
        th(      "GDU");	// 178
        th(    "GZHOG");	// 178
        th(   "MNGA'O");	// 178
        th(    "PHABS");	// 178
        th(     "SPYO");	// 178
        th(     "ZHIN");	// 178
        th(     "ZHOR");	// 178
        th(     "BYUR");	// 179
        th(      "YER");	// 179
        th(    "DGRAS");	// 180
        th(     "SING");	// 180
        th( "BRKYANGS");	// 181
        th(      "HAR");	// 181
        th(    "SKU'O");	// 181
        th(     "'DRU");	// 182
        th(     "KO'U");	// 182
        th(     "NE'U");	// 182
        th(     "RKAN");	// 182
        th(      "TRE");	// 182
        th(     "TSEG");	// 182
        th(     "BRES");	// 183
        th(   "BTZAGS");	// 183
        th(    "'DRAS");	// 183
        th(    "LHUMS");	// 183
        th(   "MDONGS");	// 183
        th(     "SNAG");	// 183
        th(  "SV'AH'A");	// 183
        th(     "ZLAR");	// 183
        th(    "'BRIM");	// 184
        th(  "BSKANGS");	// 184
        th(    "'GUGS");	// 184
        th(    "'JAGS");	// 184
        th(      "L'A");	// 184
        th(      "SBU");	// 184
        th(    "BSUNG");	// 185
        th(      "GUM");	// 185
        th(     "MNOD");	// 185
        th(     "SOMS");	// 185
        th(     "BRLA");	// 186
        th(     "NI'I");	// 186
        th(    "BTZAM");	// 187
        th(    "GZEGS");	// 187
        th(   "MYANGS");	// 187
        th(     "RMAS");	// 187
        th(    "RNGAN");	// 187
        th(     "RTAN");	// 187
        th(     "SMAS");	// 187
        th(      "T'I");	// 187
        th(    "BSDUD");	// 188
        th(  "MDZANGS");	// 188
        th(    "SHIGS");	// 188
        th(   "SKYOBS");	// 188
        th(     "SNAS");	// 188
        th(    "TSING");	// 188
        th(    "'TSIG");	// 189
        th(    "BRDA'");	// 190
        th(     "BSKU");	// 190
        th(      "YAM");	// 190
        th(     "DRON");	// 191
        th(     "KYIN");	// 191
        th(   "AINDRA");	// 192
        th(      "CUR");	// 192
        th(    "KHROL");	// 192
        th(      "ZAL");	// 192
        th(      "KYU");	// 193
        th(   "SNGOGS");	// 193
        th(    "'THEN");	// 193
        th(    "THIBS");	// 193
        th(   "BSKAMS");	// 194
        th(     "GAGS");	// 194
        th(     "MDZE");	// 194
        th(    "PHYUR");	// 194
        th(    "RTZEN");	// 194
        th(     "BOGS");	// 195
        th(    "RGYAG");	// 195
        th(      "SUG");	// 195
        th(    "'BIGS");	// 196
        th(      "'OR");	// 196
        th(  "'PHRANG");	// 196
        th(     "SBUG");	// 196
        th(    "SLA'I");	// 196
        th(   "BRDEGS");	// 197
        th(    "'BROS");	// 197
        th(    "DVANG");	// 197
        th(     "K'AR");	// 197
        th(    "KHROM");	// 197
        th(     "LCID");	// 197
        th(    "SBYIR");	// 197
        th(  "RGYANGS");	// 198
        th(     "SA'O");	// 198
        th(   "SPYANG");	// 198
        th(    "BRDZI");	// 199
        th(     "BRIM");	// 199
        th(     "NYAR");	// 199
        th(  "SKYONGS");	// 199
        th(      "GYU");	// 200
        th(     "KYEE");	// 200
        th(  "BSNGO'O");	// 201
        th(   "KA'ANG");	// 201
        th(    "RDZAB");	// 201
        th(   "GA-YUL");	// 202
        th(   "SNYEGS");	// 202
        th(      "SUL");	// 202
        th(    "BRJES");	// 203
        th(     "RENG");	// 203
        th(   "BZUNGS");	// 204
        th( "GA-YENGS");	// 204
        th(   "MTHO'I");	// 204
        th(    "SPYIL");	// 204
        th(    "'TSER");	// 204
        th(   "'TSOGS");	// 204
        th(  "SH'ANTI");	// 205
        th(      "KRi");	// 206
        th(    "RDO'I");	// 206
        th(      "TRI");	// 206
        th(      "BHU");	// 207
        th(      "B'I");	// 207
        th(    "BSKON");	// 207
        th(    "GNYAL");	// 207
        th(      "'OL");	// 207
        th(     "RNGO");	// 207
        th(   "SBRENG");	// 207
        th(     "SGUR");	// 207
        th(     "TZUG");	// 207
        th(     "RJED");	// 208
        th(      "tAm");	// 208
        th(    "BRKAM");	// 209
        th(    "'BRUM");	// 209
        th(    "DGONG");	// 209
        th(   "'DZEGS");	// 209
        th(      "GHA");	// 209
        th(      "ROS");	// 209
        th(    "SDOMS");	// 209
        th(     "BKAL");	// 210
        th(   "BRTZIG");	// 210
        th(     "GYES");	// 210
        th(     "SHRI");	// 210
        th(    "BSNOL");	// 211
        th(      "'JU");	// 211
        th(    "'KHUR");	// 211
        th(    "GDENG");	// 212
        th(     "NI'O");	// 213
        th(     "RNGU");	// 213
        th(   "BSGRAG");	// 214
        th(   "BSRING");	// 214
        th(  "'BYONGS");	// 214
        th(    "'DING");	// 214
        th(     "GDAM");	// 214
        th(    "GTZES");	// 214
        th(    "G-YER");	// 214
        th(    "'THUS");	// 214
        th(    "BSKUS");	// 215
        th(    "CHOGS");	// 215
        th(   "DH'A'I");	// 215
        th(    "'DU'I");	// 215
        th(     "NYAS");	// 215
        th(      "ZOL");	// 215
        th(     "NGAD");	// 216
        th(    "SBAGS");	// 216
        th(    "TSA'I");	// 216
        th(   "SKYUGS");	// 217
        th(      "THI");	// 217
        th(   "SPRANG");	// 218
        th(      "'UG");	// 218
        th(   "MTHA'O");	// 219
        th(     "NGON");	// 219
        th(      "NOG");	// 219
        th(     "KYIR");	// 220
        th(     "SLAN");	// 220
        th(      "RKO");	// 221
        th(   "TU'ANG");	// 221
        th(   "BRGYAB");	// 222
        th(      "L'U");	// 222
        th(      "RUM");	// 222
        th(     "BEMS");	// 223
        th(   "GA-YAS");	// 223
        th(    "'KHUL");	// 223
        th(   "'KHYAM");	// 223
        th(      "LDA");	// 223
        th(     "MTHE");	// 223
        th(   "'PHYAR");	// 223
        th(     "SV'A");	// 223
        th(      "KLA");	// 224
        th(  "'PHYANG");	// 224
        th(   "STZOGS");	// 224
        th(      "ZOR");	// 224
        th(    "BKREN");	// 225
        th(    "CHING");	// 225
        th(   "GRVA'I");	// 225
        th(   "'PHONG");	// 225
        th(    "SKYAR");	// 225
        th(   "BSNYEL");	// 226
        th(     "KLAS");	// 226
        th(    "NYA'I");	// 226
        th(   "CI'ANG");	// 227
        th(      "COL");	// 227
        th(    "MDZUB");	// 227
        th(     "SMYO");	// 227
        th(      "BRA");	// 228
        th(     "SHVA");	// 228
        th(      "SOL");	// 228
        th(      "ZAR");	// 228
        th(    "MI'AM");	// 229
        th(    "BGRES");	// 230
        th(  "BGYI'AM");	// 230
        th(   "BSKYAL");	// 230
        th(     "BSOG");	// 230
        th(   "G-YUNG");	// 230
        th(     "GZAS");	// 231
        th(     "BZOS");	// 232
        th(     "RJER");	// 232
        th(    "BKA'I");	// 233
        th(   "BTZONG");	// 233
        th(   "DGRA'I");	// 233
        th(   "GLENGS");	// 233
        th(   "SDINGS");	// 233
        th(    "SKYER");	// 233
        th(  "SPRINGS");	// 233
        th(     "'BOG");	// 234
        th(     "LCAM");	// 234
        th(    "LDENG");	// 234
        th(    "RGYAR");	// 234
        th(     "SKYU");	// 234
        th(    "DGU'O");	// 235
        th(      "LAB");	// 235
        th(     "SBYI");	// 235
        th(  "BSGRENG");	// 236
        th(    "MAH'A");	// 236
        th(     "SGER");	// 236
        th(   "MCHONG");	// 237
        th(     "BKAS");	// 238
        th(    "'GYEL");	// 238
        th(      "RIR");	// 238
        th(     "'DAR");	// 239
        th(  "'DI'ANG");	// 239
        th(   "PAD-MA");	// 239
        th(     "NYOS");	// 240
        th(    "SKYUR");	// 240
        th(     "MGAL");	// 241
        th(     "RKED");	// 241
        th(    "RTZAM");	// 241
        th(     "TSIL");	// 241
        th(     "A'A:");	// 242
        th(     "DKOR");	// 242
        th(   "GAnd'I");	// 242
        th(      "BSO");	// 243
        th(     "DGES");	// 243
        th(   "DRUNGS");	// 243
        th(     "KENG");	// 243
        th(     "LTAD");	// 243
        th(      "BUL");	// 244
        th(    "BZHIG");	// 244
        th(   "GZHOGS");	// 244
        th(   "'GENGS");	// 245
        th(     "KHAD");	// 245
        th(   "BRTZIS");	// 246
        th(    "RDE'U");	// 246
        th(     "SBRA");	// 246
        th(    "SGRIG");	// 246
        th(     "NYUG");	// 247
        th(    "'GONG");	// 248
        th(     "GZON");	// 248
        th(  "'KHRIGS");	// 248
        th(     "RLAN");	// 248
        th(      "BHI");	// 249
        th(     "TU'O");	// 249
        th(    "DPAGS");	// 250
        th(   "MCHIMS");	// 250
        th(      "SKA");	// 250
        th(   "SMYUNG");	// 250
        th(    "BSDOS");	// 251
        th(     "KY'A");	// 251
        th(    "BSKOL");	// 252
        th(    "RNGON");	// 253
        th(      "TAm");	// 253
        th(     "ZA'O");	// 253
        th(    "GCUGS");	// 254
        th(    "RKYAL");	// 254
        th(     "STOR");	// 254
        th(    "TSEMS");	// 254
        th(    "ZUNGS");	// 254
        th(     "KYAL");	// 255
        th(     "LCAG");	// 255
        th(   "RGYA'O");	// 255
        th(       "'U");	// 256
        th(     "ZHAS");	// 256
        th(    "BRKOS");	// 257
        th(     "'GOS");	// 257
        th(   "GZHONG");	// 257
        th(     "KLAD");	// 257
        th(    "'PHUL");	// 257
        th(     "SNON");	// 257
        th(      "PAm");	// 258
        th(     "RTAR");	// 258
        th(    "DRING");	// 259
        th(    "MCHID");	// 259
        th(     "N+YA");	// 259
        th(     "ZLAS");	// 259
        th(     "BCAG");	// 260
        th(    "BSHA'");	// 260
        th(   "BYA'AM");	// 260
        th(    "CHEMS");	// 260
        th(      "SIM");	// 260
        th(    "THAGS");	// 260
        th(   "BSDU'O");	// 261
        th(    "BSHAM");	// 261
        th(     "GZER");	// 261
        th(    "THA'I");	// 261
        th(    "G-YAR");	// 262
        th(     "SBUB");	// 262
        th(      "YOS");	// 262
        th(    "BSANG");	// 263
        th(     "G-YU");	// 264
        th(   "MTHING");	// 264
        th(     "PU'I");	// 265
        th("PRADZNY'A");	// 266
        th(     "RJEN");	// 266
        th(  "^SKYABS");	// 266
        th(    "'BYAR");	// 267
        th(      "MOG");	// 267
        th(  "PADMO'I");	// 267
        th(     "RMED");	// 267
        th(     "BTOD");	// 268
        th(     "GYAL");	// 269
        th(     "RDUB");	// 269
        th(     "SGAM");	// 269
        th(    "RDZUS");	// 270
        th(     "TE'O");	// 270
        th(    "BZO'I");	// 271
        th(    "SNGAS");	// 271
        th(     "TH'A");	// 271
        th(      "LAm");	// 272
        th(     "THAN");	// 272
        th(    "DGRAR");	// 273
        th(      "W'A");	// 273
        th(   "ZHE'AM");	// 273
        th(    "RNGUB");	// 274
        th(    "SHU'I");	// 275
        th(     "BKAB");	// 276
        th(    "GSHOG");	// 276
        th(   "RTZIBS");	// 276
        th(      "BUG");	// 277
        th(    "GRI'I");	// 277
        th(    "SPREL");	// 278
        th(   "SPRE'U");	// 278
        th(    "SPYIN");	// 278
        th(    "'THUN");	// 278
        th(    "YUNGS");	// 279
        th(     "SBAR");	// 280
        th(     "SNAL");	// 280
        th(   "KHRI'U");	// 281
        th(    "'PHYE");	// 281
        th(    "'DA'O");	// 282
        th(     "MJED");	// 282
        th(      "SRI");	// 283
        th(      "TZI");	// 283
        th(    "GTZOS");	// 284
        th(   "SGRIGS");	// 284
        th(      "Y'A");	// 284
        th(   "AUTPAL");	// 285
        th(     "'CHE");	// 285
        th(    "RTZAR");	// 286
        th(      "CHI");	// 287
        th(     "DH'A");	// 287
        th(    "'DZEG");	// 287
        th(     "'DZI");	// 287
        th(     "BSRE");	// 288
        th(     "LCOG");	// 288
        th(      "MUR");	// 288
        th(    "SGYID");	// 288
        th(    "GRVAR");	// 289
        th(    "LTA'O");	// 289
        th(     "STIR");	// 289
        th(     "BSAG");	// 290
        th(    "'GEGS");	// 290
        th(    "SKYEN");	// 290
        th(    "BCU'O");	// 291
        th(     "PHRU");	// 291
        th(    "RTZAD");	// 291
        th(     "SNYE");	// 291
        th(   "GDENGS");	// 292
        th(      "HUD");	// 292
        th(     "RU'I");	// 292
        th(    "BTEGS");	// 293
        th(     "GROG");	// 293
        th(    "BKROL");	// 294
        th(    "'DZAD");	// 294
        th(       "tA");	// 294
        th(      "TZE");	// 294
        th(      "NOS");	// 295
        th(        "6");	// 296
        th(   "BCANGS");	// 296
        th(    "DHA'I");	// 296
        th(  "MCHONGS");	// 296
        th(     "'GUL");	// 297
        th(   "GZINGS");	// 297
        th(    "'PHAN");	// 297
        th(    "SNYUN");	// 297
        th(     "SRAD");	// 297
        th(   "DBYE'I");	// 298
        th(      "SPA");	// 298
        th(      "YIR");	// 298
        th(      "SMA");	// 299
        th(    "BSHOL");	// 300
        th(      "shA");	// 300
        th(      "GHO");	// 301
        th(    "SNGOM");	// 301
        th(    "TSOMS");	// 301
        th(       "ZO");	// 301
        th(        "5");	// 302
        th(    "G-YEM");	// 302
        th(     "MGAR");	// 302
        th(    "YENGS");	// 302
        th(   "BSKYAR");	// 303
        th(   "DMANGS");	// 303
        th(    "'DZOM");	// 303
        th(     "GDOL");	// 304
        th(     "GSOR");	// 305
        th(    "MNYEN");	// 305
        th(    "'PHUR");	// 305
        th(    "THU'U");	// 305
        th(      "'UR");	// 305
        th(    "BKVAN");	// 306
        th(     "'BUR");	// 307
        th(     "MNAL");	// 307
        th(     "BKUG");	// 308
        th(   "MKHRIS");	// 308
        th(   "MTHANG");	// 308
        th(    "PHRAL");	// 308
        th(     "BRKU");	// 309
        th(   "SHANGS");	// 309
        th(     "DA'I");	// 311
        th(    "SPU'I");	// 311
        th(     "DGUS");	// 312
        th(     "SHUR");	// 312
        th(      "YAG");	// 312
        th(   "PHANGS");	// 313
        th(     "BZLA");	// 314
        th(       "KE");	// 314
        th(       "tI");	// 314
        th(  "TZANDRA");	// 314
        th(     "KHOL");	// 316
        th(   "BKLAGS");	// 317
        th(     "SKUL");	// 317
        th(     "DO'I");	// 318
        th(   "RTZA'I");	// 318
        th(     "SDEB");	// 318
        th(   "'BRU'I");	// 320
        th(     "DU'I");	// 320
        th(     "AANG");	// 322
        th(      "SAm");	// 322
        th( "SH'A-KYA");	// 322
        th(     "SU'O");	// 322
        th(    "RATNA");	// 323
        th(     "GROR");	// 324
        th(   "RAKshI");	// 324
        th(   "SPONGS");	// 326
        th(   "'GRIGS");	// 327
        th(    "'GRON");	// 327
        th(      "TAS");	// 328
        th(      "YOR");	// 328
        th(     "DROS");	// 330
        th(     "PANG");	// 330
        th(  "BRTZE'I");	// 331
        th(     "BCVA");	// 332
        th(  "BZHI'AM");	// 332
        th(     "GI'O");	// 332
        th(    "KHRON");	// 332
        th(    "MANGS");	// 332
        th(     "RDOG");	// 332
        th(    "RTING");	// 333
        th("GANGG'A'I");	// 334
        th(     "GSEB");	// 334
        th(      "SOM");	// 334
        th(    "TZTSA");	// 334
        th(     "LHUG");	// 335
        th(        "A");	// 336
        th(     "CHUN");	// 336
        th(    "LKUGS");	// 336
        th(    "'KHAR");	// 337
        th(       "HI");	// 338
        th(    "'KHRI");	// 339
        th(   "PO'ANG");	// 339
        th(    "SHI'I");	// 339
        th(   "BZHAMS");	// 340
        th(    "SHE'I");	// 340
        th(   "GA-YOG");	// 341
        th(     "NGUS");	// 341
        th(    "SHANG");	// 341
        th(    "G-YEL");	// 342
        th(     "BYOL");	// 343
        th(     "DMUS");	// 343
        th(  "BSGRIMS");	// 344
        th(  "TZANDAN");	// 344
        th(     "DBON");	// 346
        th(   "KHYAMS");	// 346
        th(     "BTZO");	// 347
        th(      "MUS");	// 347
        th(    "SPYIS");	// 347
        th(  "BSRINGS");	// 348
        th(    "LHANG");	// 348
        th(     "GDA'");	// 350
        th(      "GES");	// 351
        th(      "KVA");	// 351
        th(     "RDAL");	// 351
        th(        "4");	// 354
        th(      "D'I");	// 354
        th(    "'KHRU");	// 354
        th(    "DGROL");	// 355
        th(    "DPANG");	// 355
        th(     "KHUG");	// 356
        th(   "MA'ANG");	// 356
        th(   "SGRUBS");	// 356
        th(     "SHLO");	// 356
        th(    "WARMA");	// 356
        th(       "AO");	// 357
        th(    "DKA'I");	// 357
        th(     "KHON");	// 357
        th(    "BKRAS");	// 358
        th(    "SKROD");	// 358
        th(     "ZONG");	// 358
        th(     "BDAR");	// 359
        th(       "KI");	// 359
        th(    "SBREL");	// 359
        th(     "RO'I");	// 360
        th(     "KONG");	// 361
        th(     "SGEG");	// 361
        th(   "BUDDHA");	// 362
        th(     "CU'I");	// 363
        th(      "HAN");	// 363
        th(     "KROL");	// 363
        th(  "MANYDZU");	// 363
        th(    "'KHON");	// 365
        th(     "SPOM");	// 366
        th(     "ZHUN");	// 366
        th(      "RAR");	// 367
        th(    "GTAMS");	// 369
        th(     "'GRE");	// 370
        th(    "'GROL");	// 371
        th(      "ZAM");	// 372
        th(    "BYUGS");	// 374
        th(    "DVOGS");	// 375
        th(     "SNAR");	// 375
        th(   "'KHRUR");	// 376
        th(    "SGANG");	// 376
        th(     "GLAN");	// 377
        th(   "BRGYAL");	// 378
        th(     "GLEN");	// 378
        th(      "'JO");	// 378
        th(        "3");	// 379
        th(    "BZHUD");	// 379
        th(      "COD");	// 379
        th(   "STONGS");	// 379
        th(    "BCING");	// 380
        th(    "LTENG");	// 380
        th(      "MAD");	// 380
        th(      "GYO");	// 381
        th(     "LDEM");	// 381
        th(      "LEB");	// 381
        th(      "GUD");	// 382
        th(     "SU'I");	// 382
        th(  "BRDUNGS");	// 383
        th(   "BSDIGS");	// 383
        th(    "BTZOS");	// 383
        th(     "RAMS");	// 383
        th(     "MDOS");	// 384
        th(      "MON");	// 385
        th(     "MYUG");	// 385
        th(    "BRLAN");	// 386
        th(    "RUNGS");	// 386
        th(    "DRO'I");	// 388
        th(     "GYAD");	// 388
        th(       "HU");	// 389
        th(    "KHYUD");	// 389
        th(   "BLTAMS");	// 390
        th(     "GSHE");	// 390
        th(   "SGRENG");	// 390
        th(     "TSUD");	// 390
        th(    "'GRES");	// 391
        th(    "SMYAN");	// 391
        th(    "ZHE'O");	// 391
        th(      "KYA");	// 392
        th(     "THEM");	// 392
        th(     "SGOS");	// 393
        th(   "'BANGS");	// 395
        th(    "BKLAG");	// 395
        th(    "NGO'I");	// 395
        th(       "ZI");	// 396
        th(     "BREL");	// 398
        th(     "SPAR");	// 398
        th(   "BSANGS");	// 399
        th(     "BTUL");	// 400
        th(     "GZHU");	// 400
        th(    "BRNAB");	// 401
        th(   "BSNYON");	// 401
        th(      "LUD");	// 401
        th(      "SAG");	// 401
        th(    "PADMO");	// 402
        th(     "MDUD");	// 403
        th(    "SMYOS");	// 403
        th(    "SPAGS");	// 403
        th(   "BYINGS");	// 404
        th(     "SHUG");	// 404
        th(     "GZA'");	// 405
        th(    "BLA'I");	// 406
        th(     "BUNG");	// 406
        th(      "GCI");	// 406
        th(    "TSO'I");	// 406
        th(    "MDUNG");	// 407
        th(        "2");	// 408
        th(     "'GAS");	// 408
        th(   "BRTZAD");	// 410
        th(    "'DZEM");	// 410
        th(     "SMRE");	// 411
        th(      "KAN");	// 412
        th(   "PHRUGS");	// 413
        th(     "SLEB");	// 413
        th(    "'BROM");	// 414
        th(    "SGO'I");	// 414
        th(   "GDANGS");	// 415
        th(     "SHED");	// 415
        th(   "BSTABS");	// 417
        th(   "'DZOMS");	// 417
        th(    "GNYA'");	// 417
        th(     "KHAL");	// 419
        th(     "KHOG");	// 419
        th(      "SBA");	// 419
        th(  "GA-YENG");	// 420
        th(   "BSHANG");	// 421
        th(      "DVA");	// 422
        th(     "MCHE");	// 422
        th(        "1");	// 424
        th(    "BREGS");	// 426
        th(     "BLOR");	// 427
        th(     "'JA'");	// 427
        th(    "PHUGS");	// 427
        th(   "t'IKKA");	// 429
        th(    "NGOGS");	// 430
        th(     "DGAS");	// 431
        th(     "RMIS");	// 431
        th(    "BCA'I");	// 432
        th(     "BTEG");	// 432
        th(   "BSNAMS");	// 433
        th(     "LU'I");	// 433
        th(     "SDER");	// 433
        th(      "BRE");	// 434
        th(   "BSKYAB");	// 435
        th(   "LE'U'I");	// 435
        th(    "LPAGS");	// 435
        th(    "GYI'O");	// 436
        th(    "BSRES");	// 437
        th(     "DRAL");	// 437
        th(     "DRAS");	// 437
        th(       "'I");	// 437
        th(       "nI");	// 437
        th(   "RGYU'O");	// 438
        th(    "DGA'I");	// 439
        th(      "RAN");	// 439
        th(     "GZOD");	// 440
        th(     "KHUL");	// 440
        th(     "SKUS");	// 440
        th(      "BAD");	// 441
        th(     "TSVA");	// 441
        th(   "BRGYAS");	// 442
        th(   "MNGAGS");	// 443
        th(     "RDEG");	// 443
        th(    "BSDAM");	// 444
        th(   "DKRUGS");	// 444
        th(     "SKAS");	// 446
        th(    "THONG");	// 446
        th(    "'DZUD");	// 447
        th(  "G-YENGS");	// 447
        th(    "'DZAG");	// 448
        th(    "^GONG");	// 448
        th(       "BE");	// 449
        th(      "KHE");	// 449
        th(     "SRAN");	// 450
        th(    "SARBA");	// 451
        th(   "RGYA'I");	// 452
        th(       "YU");	// 452
        th(    "'BOGS");	// 453
        th(     "BYES");	// 453
        th(  "'THUNGS");	// 453
        th(  "BSKYANG");	// 454
        th(    "'BYON");	// 455
        th(  "PADMA'I");	// 455
        th(    "SMYON");	// 455
        th(      "GLA");	// 456
        th(      "GZU");	// 456
        th(   "KHRI'I");	// 456
        th(     "SBAL");	// 457
        th(     "BKRI");	// 458
        th(    "LJANG");	// 458
        th(    "NGA'O");	// 458
        th(    "PHRUG");	// 458
        th(      "RUL");	// 458
        th(    "SKYOR");	// 458
        th(   "BSNYIL");	// 459
        th(    "LCOGS");	// 461
        th(      "M'A");	// 461
        th(     "BROS");	// 462
        th(     "GZAB");	// 464
        th(     "BE'U");	// 468
        th(     "BTUB");	// 469
        th(      "SVA");	// 469
        th(     "SHUN");	// 470
        th(     "BDUG");	// 471
        th(     "BO'O");	// 471
        th(       "DI");	// 472
        th(     "GYEN");	// 473
        th(     "GZAR");	// 473
        th(    "KARMA");	// 473
        th(  "'PHROGS");	// 474
        th(    "GTZOR");	// 475
        th(   "GZENGS");	// 476
        th(    "MNGAR");	// 476
        th(     "SKUD");	// 477
        th(    "ZHAGS");	// 478
        th(    "'DRAR");	// 479
        th(    "SMROS");	// 479
        th(     "TZOG");	// 479
        th(     "DE'O");	// 480
        th(    "G'A'I");	// 480
        th(    "SGRAR");	// 480
        th(     "CHIG");	// 482
        th(   "SU'ANG");	// 488
        th(     "GZIR");	// 490
        th(    "DKYUS");	// 491
        th(       "PI");	// 491
        th(      "POD");	// 492
        th(     "sh'A");	// 492
        th(  "GA-YOGS");	// 493
        th(    "BSTAR");	// 496
        th(    "'KHOB");	// 496
        th(     "SRAB");	// 496
        th(   "SMRA'I");	// 497
        th(     "SNUM");	// 497
        th(    "ZHENG");	// 497
        th(      "H'A");	// 498
        th(    "BKRUS");	// 499
        th(     "'BUD");	// 499
        th(    "SBUBS");	// 499
        th(      "MID");	// 500
        th(     "tIKA");	// 501
        th(  "BGRANGS");	// 503
        th(     "BKAR");	// 503
        th(     "BGOM");	// 505
        th(    "GSHIS");	// 505
        th(   "BSGRAL");	// 507
        th(  "'KHYAMS");	// 507
        th(     "MCHU");	// 507
        th(      "GZO");	// 509
        th(     "MGUL");	// 509
        th(    "'BROG");	// 510
        th(     "CHAN");	// 510
        th(    "BLTAR");	// 511
        th(      "GAB");	// 511
        th(    "PHA'I");	// 512
        th(    "TSAGS");	// 512
        th(    "GNYAN");	// 514
        th(    "BSREL");	// 516
        th(    "BSGOS");	// 517
        th(    "GZHIG");	// 517
        th(   "MGYOGS");	// 517
        th(   "PHROGS");	// 517
        th(     "THOR");	// 517
        th(   "'KHYUD");	// 518
        th(      "BAN");	// 520
        th(   "DHARMA");	// 520
        th(    "KHEBS");	// 520
        th(     "NU'I");	// 520
        th(    "BSLUS");	// 522
        th(    "NONGS");	// 522
        th(   "SPRING");	// 522
        th(     "MNOS");	// 523
        th(    "SHONG");	// 525
        th(    "KLONG");	// 526
        th(    "MTHAS");	// 527
        th(     "TA'I");	// 527
        th(   "'TSONG");	// 528
        th(   "BRDUNG");	// 530
        th(    "RNYOG");	// 531
        th(    "STANG");	// 532
        th(      "TOR");	// 532
        th(     "ZHON");	// 532
        th(    "'THAG");	// 534
        th(      "TSU");	// 534
        th(      "NYO");	// 535
        th(    "RGYUG");	// 535
        th(      "DUB");	// 536
        th(     "RDOL");	// 536
        th(      "ZHA");	// 536
        th(     "GRAS");	// 538
        th(    "'CHOL");	// 539
        th(       "HO");	// 539
        th(    "G-YOS");	// 540
        th(     "GRAG");	// 542
        th(      "tIK");	// 542
        th(     "LING");	// 543
        th(     "SHAG");	// 543
        th(    "KHRIR");	// 545
        th(      "LDE");	// 545
        th(     "RDZI");	// 545
        th(     "STUG");	// 545
        th(     "ZA'I");	// 545
        th(     "BKRU");	// 546
        th(    "SHLOO");	// 546
        th(    "DGE'I");	// 547
        th(    "DKA'O");	// 547
        th(      "D'A");	// 548
        th(       "SI");	// 550
        th(    "BKRAM");	// 551
        th(      "DZI");	// 551
        th(     "GTUM");	// 551
        th(     "GZAN");	// 551
        th(     "THUL");	// 551
        th(    "MA'AM");	// 553
        th(     "BRJE");	// 554
        th(  "BRTZIGS");	// 554
        th(      "GOR");	// 554
        th(    "'TSED");	// 554
        th(      "GOM");	// 555
        th(    "RINGS");	// 555
        th(     "MOOD");	// 556
        th(   "GZHI'O");	// 557
        th(    "SPRAD");	// 558
        th(      "GER");	// 559
        th(    "'TSAM");	// 560
        th(    "SBYOD");	// 561
        th(    "SKYIL");	// 561
        th(    "BTZON");	// 563
        th(   "'KHYER");	// 564
        th(     "TSER");	// 564
        th(   "SBRAGS");	// 566
        th(    "BLUGS");	// 567
        th(      "TIL");	// 567
        th(     "BCAR");	// 570
        th(     "BKOL");	// 570
        th(      "DHI");	// 571
        th(      "YUD");	// 571
        th(        "'");	// 572
        th(        "^");	// 573
        th(      "B'A");	// 573
        th(     "RLOB");	// 574
        th(     "GZIM");	// 577
        th(       "AI");	// 578
        th(   "MCHI'O");	// 580
        th(    "KRUNG");	// 581
        th(   "'DZUGS");	// 582
        th(   "PHYUGS");	// 582
        th(     "SHOD");	// 582
        th(    "SLEBS");	// 583
        th(    "ZLA'I");	// 583
        th(    "GZHIS");	// 584
        th(     "PAGS");	// 585
        th(   "BADZRA");	// 589
        th(    "THIGS");	// 590
        th(    "BRLAG");	// 592
        th(      "LTE");	// 592
        th(   "RDZONG");	// 592
        th(    "CHE'O");	// 593
        th(      "GOL");	// 593
        th(     "BTUD");	// 594
        th(    "RTZED");	// 594
        th(   "DVANGS");	// 596
        th(    "BRDAS");	// 597
        th(   "MTSANG");	// 597
        th(      "SNE");	// 597
        th(    "'DEGS");	// 599
        th(      "MIS");	// 599
        th(     "CHUR");	// 601
        th(    "MTHOL");	// 603
        th(   "LNGA'O");	// 605
        th(   "BRTZAL");	// 608
        th(      "BRO");	// 614
        th(     "'GOR");	// 617
        th(    "LCE'I");	// 617
        th(     "GCAN");	// 618
        th(      "KOR");	// 619
        th(     "LDIR");	// 619
        th(    "SRING");	// 621
        th(      "COG");	// 624
        th(    "GZIMS");	// 624
        th(     "PHOD");	// 624
        th(      "BEM");	// 625
        th(    "'CHAG");	// 626
        th(    "MCHIL");	// 626
        th(      "GRA");	// 628
        th(     "PHAS");	// 629
        th(     "THER");	// 630
        th(    "BSNUN");	// 631
        th(     "GLOG");	// 632
        th(   "GDUNGS");	// 633
        th(    "'PHYA");	// 633
        th(     "BDOG");	// 636
        th(   "RDZING");	// 637
        th(   "BLTA'O");	// 638
        th(       "NE");	// 642
        th(     "LHAR");	// 644
        th(      "BEE");	// 645
        th(    "AUTPA");	// 647
        th(    "RMUGS");	// 647
        th(     "PHOR");	// 648
        th(     "GCER");	// 649
        th(     "RGAS");	// 649
        th(     "NYAG");	// 650
        th(     "THAM");	// 651
        th(     "BCUS");	// 653
        th(     "GSAD");	// 653
        th(    "GSHER");	// 653
        th(   "BSKRAD");	// 656
        th(     "STEN");	// 656
        th(    "'PHAR");	// 658
        th(      "ROG");	// 658
        th(   "BRDA'I");	// 659
        th(    "BRDAR");	// 660
        th(    "'DONG");	// 660
        th(     "DPAR");	// 661
        th(    "'GROS");	// 662
        th(     "GDUG");	// 663
        th(     "'JUD");	// 663
        th(   "BSKANG");	// 666
        th(      "PAN");	// 666
        th(   "SKYA'I");	// 666
        th(      "BSE");	// 667
        th(       "HE");	// 667
        th(     "KHOR");	// 667
        th(     "SPUN");	// 669
        th(   "RGYAGS");	// 670
        th(     "KHYU");	// 672
        th(    "RTZER");	// 672
        th(     "SGAR");	// 676
        th(     "ZLUM");	// 676
        th(     "BYOS");	// 678
        th(     "t'IK");	// 678
        th(    "BGEGS");	// 679
        th(      "PUS");	// 679
        th(     "KHOM");	// 680
        th(   "MDANGS");	// 680
        th(    "'BRUG");	// 681
        th(    "BANDE");	// 682
        th(    "G-YUL");	// 682
        th(    "SPRAS");	// 682
        th(    "KHYON");	// 683
        th(    "LDING");	// 683
        th(    "'DI'O");	// 687
        th(     "'GAR");	// 687
        th(      "GON");	// 688
        th(     "WANG");	// 691
        th(       "CO");	// 692
        th(    "DREGS");	// 692
        th(    "BZHON");	// 693
        th(     "SH'I");	// 693
        th(    "SMRIG");	// 693
        th(     "PHUD");	// 697
        th(       "CA");	// 698
        th(   "DZNY'A");	// 699
        th(    "BKRES");	// 701
        th(     "LTAG");	// 701
        th(     "'DAG");	// 709
        th(    "KYI'O");	// 714
        th(    "PO'AM");	// 714
        th(     "BDER");	// 715
        th(     "'BRI");	// 715
        th(     "LHOD");	// 715
        th(   "BSNYAD");	// 717
        th(    "RADVA");	// 717
        th(      "GAD");	// 720
        th(   "KHYUNG");	// 723
        th(    "DBYUG");	// 724
        th(      "YOL");	// 724
        th(    "BSHIG");	// 725
        th(    "SPROD");	// 725
        th(     "GDOD");	// 727
        th(    "KLU'I");	// 727
        th(     "DGUR");	// 728
        th(      "PRA");	// 732
        th(     "SHOR");	// 734
        th(    "'THOR");	// 734
        th(     "KHAG");	// 737
        th(      "SIL");	// 737
        th(    "SKYEL");	// 737
        th(      "GUR");	// 740
        th(    "ZHANG");	// 740
        th(     "'JIM");	// 745
        th(   "'TSO'I");	// 746
        th(     "'DOM");	// 748
        th(     "RNAG");	// 749
        th(    "NGOMS");	// 750
        th(    "ZHI'I");	// 750
        th(     "DGAR");	// 751
        th(     "RGUD");	// 751
        th(     "CHUG");	// 753
        th(     "THIM");	// 754
        th(    "SLA'O");	// 757
        th(     "KLAN");	// 759
        th(      "PUR");	// 764
        th(     "RA'I");	// 764
        th(        "7");	// 767
        th(     "THAB");	// 770
        th(    "DPYID");	// 771
        th(      "P'A");	// 773
        th(     "TZAN");	// 775
        th(   "KHENGS");	// 776
        th(     "SBED");	// 777
        th(      "A'A");	// 780
        th(      "R'A");	// 782
        th(      "GOO");	// 783
        th(    "SHA'I");	// 783
        th(    "CHIBS");	// 784
        th(     "ZHAN");	// 784
        th(     "BCOL");	// 786
        th(    "BRJID");	// 787
        th(   "BZHI'O");	// 788
        th(    "GLAGS");	// 788
        th(   "'PHRAD");	// 789
        th(     "DGUN");	// 791
        th(      "TIG");	// 792
        th(     "'DRE");	// 797
        th(      "'GO");	// 799
        th(   "BSDOMS");	// 800
        th(    "GSHED");	// 801
        th(    "SGROG");	// 805
        th(     "GRIB");	// 808
        th(     "LHAM");	// 809
        th(    "GSHAM");	// 813
        th(    "BSNAN");	// 814
        th(      "K'A");	// 816
        th(   "SKYE'I");	// 817
        th(    "t'IKA");	// 817
        th(     "GNAM");	// 818
        th(      "RNO");	// 818
        th(    "GZHOM");	// 826
        th(     "SDES");	// 830
        th(    "BRDEG");	// 831
        th(      "T'A");	// 831
        th(     "ZHVA");	// 832
        th(   "BSLABS");	// 834
        th(     "BSTI");	// 834
        th(    "BDE'I");	// 840
        th(     "'BOD");	// 840
        th(    "BKANG");	// 841
        th(    "BTZAS");	// 841
        th(     "SHAM");	// 842
        th(    "'CHA'");	// 843
        th(    "SNA'I");	// 843
        th(      "BRI");	// 846
        th(     "CHOM");	// 847
        th(    "MTHIL");	// 849
        th(    "'GRIG");	// 852
        th(    "LCANG");	// 852
        th(    "MCHAN");	// 854
        th(      "HOR");	// 857
        th(      "MIR");	// 859
        th(      "NUR");	// 860
        th(    "RLOBS");	// 860
        th(    "BTING");	// 863
        th(      "NOD");	// 863
        th(     "THUR");	// 865
        th(   "'KHRID");	// 867
        th(     "RKYA");	// 867
        th(      "SEN");	// 870
        th(    "BSREG");	// 871
        th(   "SPUNGS");	// 871
        th(     "SHAL");	// 872
        th(    "KHREL");	// 873
        th(   "'GRENG");	// 876
        th(  "BRGYA'I");	// 877
        th(    "MTSOS");	// 877
        th(    "TSE'I");	// 877
        th(    "'DRIS");	// 881
        th(     "RTON");	// 881
        th(     "PHUR");	// 882
        th(      "RER");	// 882
        th(       "LU");	// 883
        th(     "GSOS");	// 884
        th(   "BSREGS");	// 888
        th(     "TSOS");	// 888
        th(    "CHA'I");	// 892
        th(    "BCABS");	// 893
        th(   "G-YANG");	// 894
        th(   "'KHYIL");	// 894
        th(      "TUR");	// 895
        th(     "DMA'");	// 896
        th(     "PHOG");	// 896
        th(     "ZHOG");	// 902
        th(   "SMRA'O");	// 903
        th(     "ZHIM");	// 908
        th(       "JE");	// 910
        th(      "S'A");	// 914
        th(     "SHUL");	// 914
        th(    "TSABS");	// 917
        th(   "BSDAMS");	// 920
        th(     "BANG");	// 922
        th(    "RANGS");	// 926
        th(    "LJAGS");	// 927
        th(   "BRTZAM");	// 928
        th(    "GNYUG");	// 928
        th(    "PHRIN");	// 928
        th(      "TON");	// 929
        th(    "SNYOM");	// 932
        th(     "MGUR");	// 935
        th(     "GNAG");	// 936
        th(     "BLAG");	// 937
        th(     "GRAL");	// 941
        th(    "'GRAN");	// 941
        th(      "STA");	// 943
        th(     "'DAM");	// 944
        th(     "DPES");	// 945
        th(     "BCUR");	// 948
        th(      "DED");	// 950
        th(      "DOD");	// 950
        th(    "BSTUN");	// 951
        th(   "BZANGS");	// 951
        th("SH'AKYA'I");	// 953
        th(     "GTOD");	// 955
        th(   "'PHROD");	// 957
        th(    "GA-YO");	// 961
        th(     "KHAM");	// 961
        th(     "SKAM");	// 961
        th(    "KHRIG");	// 964
        th(    "KHA'I");	// 974
        th(     "RONG");	// 977
        th(      "GZI");	// 981
        th(     "NGUR");	// 981
        th(   "'TSAMS");	// 982
        th(     "SGOR");	// 983
        th(   "BSHAMS");	// 984
        th(   "SNYIGS");	// 984
        th(      "SOG");	// 986
        th(      "THU");	// 989
        th(      "TAM");	// 990
        th(    "ZANGS");	// 991
        th(   "BSTZAL");	// 993
        th(    "MTSOR");	// 993
        th(    "THEBS");	// 993
        th(      "SOS");	// 994
        th(      "DHA");	// 995
        th(    "DKRIS");	// 995
        th(   "'GRO'O");	// 997
    }


    /** Tests the tsheg bars that appear [1000, 9999] times. */
    public void test1000to9999() {
        th(      "GRI");	// 1002
        th(    "KHUNG");	// 1004
        th(  "BSGRIGS");	// 1012
        th(    "BCANG");	// 1014
        th(     "BLUN");	// 1015
        th(      "SHO");	// 1015
        th(    "BRTZI");	// 1016
        th(    "LE'UR");	// 1018
        th(    "MTHON");	// 1018
        th(    "'GRAM");	// 1019
        th(     "BYUG");	// 1021
        th(     "GU'I");	// 1021
        th(     "SMIG");	// 1021
        th(    "'DZUM");	// 1023
        th(      "GLU");	// 1023
        th(  "BSRUNGS");	// 1027
        th(     "KHYI");	// 1030
        th(      "NOM");	// 1031
        th(      "LOR");	// 1033
        th(     "GSON");	// 1036
        th(   "SRUNGS");	// 1042
        th(    "BTZAN");	// 1043
        th(      "TRA");	// 1048
        th(   "RTZE'I");	// 1051
        th(    "MGRON");	// 1059
        th(    "STZOL");	// 1063
        th(   "'KHRIG");	// 1071
        th(   "BSKRUN");	// 1074
        th(      "SAS");	// 1074
        th(     "BTON");	// 1076
        th(    "STABS");	// 1076
        th(    "SKONG");	// 1083
        th(   "BRGYAR");	// 1084
        th(  "BSLANGS");	// 1086
        th(    "BRKUS");	// 1090
        th(   "BTZUGS");	// 1093
        th(      "GAN");	// 1096
        th(      "N'A");	// 1101
        th(    "GTING");	// 1102
        th(    "BKRAL");	// 1107
        th(     "NING");	// 1107
        th(  "BSGRAGS");	// 1108
        th(    "RTZUB");	// 1108
        th(    "'CHOS");	// 1109
        th(    "KHROS");	// 1109
        th(      "RED");	// 1111
        th(     "BZAD");	// 1116
        th(   "BRNYAS");	// 1123
        th(   "G-YOGS");	// 1123
        th(   "MAndAL");	// 1125
        th(    "DGUNG");	// 1128
        th(     "KHRA");	// 1128
        th(     "ZLOS");	// 1129
        th(    "RTZAL");	// 1130
        th(    "G-YOG");	// 1131
        th(    "'DOMS");	// 1132
        th(    "GTANG");	// 1132
        th(     "MNAN");	// 1132
        th(      "RIL");	// 1133
        th(       "BI");	// 1136
        th(     "MKHO");	// 1136
        th(      "DUR");	// 1139
        th(      "DZA");	// 1140
        th(   "SGROGS");	// 1145
        th(    "DPE'I");	// 1147
        th(   "PHONGS");	// 1147
        th(      "PHU");	// 1155
        th(    "RTZIG");	// 1155
        th(     "H'Um");	// 1156
        th(      "ZUG");	// 1156
        th(   "BSNGOS");	// 1161
        th(    "TSEGS");	// 1162
        th(   "'GROGS");	// 1164
        th(      "SON");	// 1165
        th(     "GYON");	// 1168
        th(     "CHUS");	// 1170
        th(     "SHEL");	// 1170
        th(      "MAM");	// 1171
        th(       "nA");	// 1172
        th(     "KHRU");	// 1174
        th(    "NGU'I");	// 1189
        th(    "NYING");	// 1197
        th(      "ZAN");	// 1198
        th(     "PHAG");	// 1199
        th(     "PHUG");	// 1199
        th(   "LJONGS");	// 1203
        th(    "GZHOL");	// 1205
        th(     "THIG");	// 1205
        th(    "GDING");	// 1208
        th(     "RDZA");	// 1215
        th(      "GAS");	// 1217
        th(     "SKOM");	// 1218
        th(     "SHAD");	// 1219
        th(    "LNGAS");	// 1220
        th(      "LAR");	// 1222
        th(     "BRAN");	// 1230
        th(    "MGRIN");	// 1230
        th(   "'BYAMS");	// 1233
        th(    "GTUGS");	// 1235
        th(    "RENGS");	// 1238
        th(       "AU");	// 1241
        th(     "MYOS");	// 1241
        th(     "CHAB");	// 1242
        th(     "SBAS");	// 1242
        th(       "SE");	// 1242
        th(    "SKYID");	// 1243
        th(     "BGOS");	// 1245
        th(    "RTZVA");	// 1249
        th(   "'THUNG");	// 1250
        th(    "LNGAR");	// 1257
        th(     "TSAB");	// 1257
        th(     "BSIL");	// 1262
        th(   "SBRANG");	// 1267
        th(     "SHAN");	// 1271
        th(     "TSON");	// 1273
        th(      "MER");	// 1277
        th(      "LUG");	// 1280
        th(    "DBYIG");	// 1283
        th(      "CIS");	// 1284
        th(     "TI'I");	// 1284
        th(  "'BRANGS");	// 1285
        th(    "SHR'I");	// 1289
        th(     "SMOD");	// 1290
        th(      "SUS");	// 1292
        th(     "LHUR");	// 1294
        th(     "MDA'");	// 1298
        th(    "RTA'I");	// 1303
        th(      "KAS");	// 1304
        th(     "RNON");	// 1308
        th(     "DBAB");	// 1314
        th(     "RMAD");	// 1321
        th(     "RGAL");	// 1322
        th(   "BSLANG");	// 1325
        th(     "THOD");	// 1329
        th(    "'CHAB");	// 1330
        th(      "YUG");	// 1335
        th(   "BSDOGS");	// 1337
        th(     "BSKO");	// 1339
        th(    "GSHIN");	// 1343
        th(     "'DUD");	// 1349
        th(      "DUM");	// 1352
        th(     "RNGA");	// 1359
        th(    "DBYEN");	// 1364
        th(       "KO");	// 1364
        th(    "BSLAD");	// 1365
        th(      "TSO");	// 1365
        th(    "BSDAD");	// 1375
        th(      "BGO");	// 1376
        th(     "RTUL");	// 1381
        th(    "SPOBS");	// 1381
        th(      "ZHO");	// 1383
        th(     "RE'I");	// 1388
        th(      "DUL");	// 1389
        th(    "MKHAR");	// 1391
        th(    "MNGAL");	// 1393
        th(   "STANGS");	// 1393
        th(     "LJON");	// 1399
        th(     "NYEN");	// 1399
        th(    "LTA'I");	// 1402
        th(     "GCAD");	// 1406
        th(       "KU");	// 1408
        th(   "KHONGS");	// 1411
        th(    "BU'AM");	// 1414
        th(     "ZHAR");	// 1415
        th(    "MDZA'");	// 1428
        th(   "RKYANG");	// 1429
        th(     "TSIM");	// 1432
        th(      "BYI");	// 1433
        th(     "RDOR");	// 1434
        th(    "GANGS");	// 1435
        th(    "DGU'I");	// 1436
        th(    "'GRIB");	// 1441
        th(       "AE");	// 1443
        th(     "KHAB");	// 1443
        th(    "G-YON");	// 1444
        th(   "RTZI'I");	// 1449
        th(     "DPAS");	// 1452
        th(   "RNYING");	// 1452
        th(   "'KHRUG");	// 1454
        th(       "WA");	// 1458
        th(      "NON");	// 1461
        th(      "GYA");	// 1463
        th(     "GUNG");	// 1468
        th(    "SDE'I");	// 1470
        th(      "BZO");	// 1472
        th(     "NGAR");	// 1473
        th(     "LTAS");	// 1475
        th(     "DMAG");	// 1476
        th(    "GLEGS");	// 1479
        th(     "BLON");	// 1482
        th(    "MNYES");	// 1482
        th(     "DAGS");	// 1483
        th(    "GDONG");	// 1483
        th(    "'PHRO");	// 1489
        th(      "LTO");	// 1490
        th(  "BRTZEGS");	// 1493
        th(     "BCUD");	// 1495
        th(   "BGRANG");	// 1496
        th(     "THAD");	// 1500
        th(      "LCE");	// 1505
        th(     "MNAR");	// 1506
        th(      "CIR");	// 1508
        th(   "G-YENG");	// 1519
        th(    "'CHAL");	// 1522
        th(   "BGYI'O");	// 1523
        th(    "RTZIS");	// 1525
        th(      "LOS");	// 1537
        th(      "RIB");	// 1541
        th(     "ZANG");	// 1550
        th(      "YAL");	// 1553
        th(    "TZONG");	// 1554
        th(     "CHAS");	// 1559
        th(     "'ANG");	// 1562
        th(     "ZHUM");	// 1562
        th(     "'TSE");	// 1566
        th(    "TSUGS");	// 1567
        th(     "GDAB");	// 1572
        th(      "SPO");	// 1577
        th(     "MCHI");	// 1582
        th(     "ZING");	// 1583
        th(     "RKUN");	// 1592
        th(    "'TSOL");	// 1593
        th(   "'CHAGS");	// 1602
        th(   "'PHROS");	// 1605
        th(    "BZHAD");	// 1608
        th(    "KHEGS");	// 1609
        th(   "SKYE'O");	// 1614
        th(  "'PHRENG");	// 1621
        th(     "ZE'I");	// 1622
        th(   "BRNYES");	// 1624
        th(     "KLOG");	// 1625
        th(     "DGOD");	// 1632
        th(  "BSGRIBS");	// 1634
        th(    "BYING");	// 1635
        th(      "TZA");	// 1636
        th(      "AOm");	// 1639
        th(     "ZLOG");	// 1643
        th(    "BRLAB");	// 1644
        th(      "BHA");	// 1649
        th(      "STI");	// 1654
        th(     "NYAM");	// 1656
        th(    "'GROR");	// 1661
        th(     "SROL");	// 1661
        th(  "'KHRUGS");	// 1672
        th(     "DROD");	// 1674
        th(     "GCES");	// 1677
        th(    "BTUNG");	// 1684
        th(     "DRIL");	// 1684
        th(   "KHUNGS");	// 1690
        th(     "SRIN");	// 1694
        th(    "MTHUS");	// 1698
        th(    "SDONG");	// 1699
        th(     "BSLU");	// 1700
        th(    "BLTOS");	// 1705
        th(    "'THAB");	// 1707
        th(     "LO'I");	// 1709
        th(      "DAN");	// 1720
        th(    "BSKOS");	// 1728
        th(   "KHYE'U");	// 1732
        th(    "SRANG");	// 1737
        th(      "RMA");	// 1739
        th(    "SBRUL");	// 1739
        th(   "'PHRAL");	// 1741
        th(    "BRGAL");	// 1744
        th(     "KHAR");	// 1748
        th(    "LHAGS");	// 1748
        th(    "GDUGS");	// 1752
        th(      "MES");	// 1762
        th(    "SKYOB");	// 1764
        th(     "NYIS");	// 1771
        th(   "DU'ANG");	// 1772
        th(     "DONG");	// 1773
        th(     "STER");	// 1773
        th(    "RGYAB");	// 1774
        th(   "'PHROG");	// 1779
        th(    "BRJED");	// 1784
        th(      "KOO");	// 1784
        th(    "BRTUL");	// 1805
        th(    "PADMA");	// 1805
        th(      "RKU");	// 1811
        th(    "DBYER");	// 1812
        th(  "BZHENGS");	// 1813
        th(     "MA'O");	// 1813
        th(     "MDOG");	// 1816
        th(      "BSU");	// 1822
        th(     "THON");	// 1826
        th(  "BRKYANG");	// 1838
        th(    "'PHOS");	// 1839
        th(     "NGAL");	// 1843
        th(     "RLOM");	// 1849
        th(    "KHRUS");	// 1854
        th(   "BRGYAN");	// 1855
        th(    "BZHIS");	// 1855
        th(    "'KHOD");	// 1855
        th(    "'DZAM");	// 1856
        th(     "SPOS");	// 1856
        th(    "G-YAS");	// 1861
        th(   "PHYI'I");	// 1863
        th(    "DBUGS");	// 1865
        th(    "RDZUN");	// 1866
        th(     "LA'I");	// 1886
        th(    "DPUNG");	// 1887
        th(     "GSOG");	// 1889
        th(      "SPU");	// 1892
        th(       "LI");	// 1905
        th(    "THUNG");	// 1910
        th(      "LHO");	// 1915
        th(     "DPOG");	// 1924
        th( "BSKYANGS");	// 1929
        th(     "DBUL");	// 1938
        th(     "SANG");	// 1939
        th(   "'CHING");	// 1947
        th(    "'GYED");	// 1947
        th(   "BSKYOD");	// 1955
        th(    "CHANG");	// 1961
        th(   "RGYANG");	// 1962
        th(    "TSONG");	// 1963
        th(    "BZLAS");	// 1969
        th(      "GRU");	// 1976
        th(       "JA");	// 1978
        th(    "NGA'I");	// 1989
        th(     "SKAR");	// 1992
        th(    "KLUNG");	// 1994
        th(      "DRA");	// 1999
        th(     "'DAB");	// 2003
        th( "'KHRUNGS");	// 2018
        th(      "GRO");	// 2023
        th(     "'JAL");	// 2023
        th(      "BAL");	// 2033
        th(   "'DRA'O");	// 2034
        th(   "BA'ANG");	// 2037
        th(     "NGED");	// 2040
        th(    "CHU'I");	// 2046
        th(      "PAD");	// 2060
        th(     "GTER");	// 2064
        th(     "KHUR");	// 2072
        th(     "BABS");	// 2074
        th(      "YAR");	// 2080
        th(    "SKRAG");	// 2087
        th(      "t'I");	// 2097
        th(     "PHAB");	// 2126
        th(      "DRO");	// 2127
        th(     "GE'I");	// 2133
        th(   "DE'ANG");	// 2147
        th(    "DGYES");	// 2154
        th(     "GTAD");	// 2157
        th(  "'PHANGS");	// 2158
        th(      "KYE");	// 2163
        th(    "LDANG");	// 2163
        th(      "KHU");	// 2164
        th(    "MDO'I");	// 2170
        th(     "NAGS");	// 2181
        th(      "GIR");	// 2187
        th(    "GZHUG");	// 2208
        th(     "CHAG");	// 2211
        th(    "BLTAS");	// 2220
        th(      "SRA");	// 2220
        th(      "RGA");	// 2222
        th(    "DVAGS");	// 2225
        th(      "ZOS");	// 2230
        th(      "SAD");	// 2234
        th(  "SH'AKYA");	// 2236
        th(    "'GYOD");	// 2248
        th(      "MGU");	// 2248
        th(      "LCI");	// 2258
        th(      "RUS");	// 2263
        th(    "SGYUR");	// 2263
        th(     "'BRU");	// 2292
        th(    "SRONG");	// 2304
        th(   "'PHANG");	// 2324
        th(      "DAL");	// 2339
        th(      "RAL");	// 2342
        th(     "'BAB");	// 2346
        th(    "BZHIR");	// 2355
        th(     "DUNG");	// 2363
        th(    "LANGS");	// 2369
        th(      "MOR");	// 2369
        th(    "BSKUR");	// 2370
        th(     "SREG");	// 2371
        th(    "'BEBS");	// 2375
        th(     "STOD");	// 2384
        th(     "ZHIB");	// 2385
        th(     "TSUR");	// 2386
        th(     "DENG");	// 2394
        th(     "GEGS");	// 2396
        th(     "GTOR");	// 2406
        th(   "'TSANG");	// 2416
        th(   "SBYANG");	// 2418
        th(     "TSAR");	// 2431
        th(     "ME'I");	// 2435
        th(     "'DRI");	// 2454
        th(    "SGROL");	// 2467
        th(   "BCINGS");	// 2475
        th(    "BRANG");	// 2491
        th(     "SNGO");	// 2498
        th(     "SDOD");	// 2501
        th(    "BGROD");	// 2503
        th(     "STAG");	// 2507
        th(      "NYA");	// 2518
        th(   "BRLABS");	// 2545
        th(      "GLO");	// 2550
        th(   "MTSO'I");	// 2551
        th(     "SNAM");	// 2570
        th(      "RVA");	// 2582
        th(     "BLOS");	// 2585
        th(       "TI");	// 2590
        th(       "JO");	// 2592
        th(      "DUG");	// 2614
        th(     "SKYO");	// 2650
        th(    "DNGUL");	// 2658
        th(    "GNYID");	// 2663
        th(     "GA'I");	// 2693
        th(     "PO'O");	// 2704
        th(    "KHRAG");	// 2707
        th(    "MTSAR");	// 2707
        th(      "BAB");	// 2711
        th(     "BSGO");	// 2717
        th(      "DUD");	// 2718
        th(    "YANGS");	// 2722
        th(     "BTUS");	// 2728
        th(     "YONG");	// 2743
        th(     "'DOR");	// 2751
        th(     "SKRA");	// 2751
        th(    "SPRIN");	// 2759
        th(     "SPRO");	// 2772
        th(      "KAR");	// 2785
        th(      "PAn");	// 2787
        th(     "BSAD");	// 2791
        th(    "KHROD");	// 2791
        th(   "DBYIBS");	// 2795
        th(     "RGOD");	// 2846
        th(    "RGYUS");	// 2865
        th(     "TSOL");	// 2881
        th(     "GNON");	// 2894
        th(     "LKOG");	// 2903
        th(      "GTI");	// 2917
        th(     "LHUN");	// 2929
        th(     "GSAN");	// 2939
        th(   "MTHA'I");	// 2939
        th(    "BLO'I");	// 2943
        th(       "LE");	// 2951
        th(   "BRDZUN");	// 2954
        th(    "SKU'I");	// 2956
        th(      "SHE");	// 2960
        th(     "RABS");	// 2966
        th(    "BCU'I");	// 2970
        th(    "NYUNG");	// 2989
        th(     "LOGS");	// 3001
        th(     "NGAM");	// 3009
        th(    "BGYIS");	// 3022
        th(    "BTZAL");	// 3041
        th(     "MI'I");	// 3058
        th(      "MUG");	// 3062
        th(     "ZUNG");	// 3064
        th(      "ZIL");	// 3074
        th(     "RGAN");	// 3098
        th(      "LON");	// 3120
        th(    "PAndI");	// 3150
        th(     "SHOS");	// 3155
        th(    "'BRAL");	// 3156
        th(    "LCAGS");	// 3171
        th(     "GSOD");	// 3177
        th(  "BSNGAGS");	// 3181
        th(     "'PHO");	// 3194
        th(     "DU'O");	// 3201
        th(     "RDZU");	// 3227
        th(     "SPEL");	// 3228
        th(      "ZUR");	// 3245
        th(    "RTZOL");	// 3247
        th(     "NA'I");	// 3250
        th(   "LNGA'I");	// 3253
        th(      "NGU");	// 3263
        th(       "PU");	// 3263
        th(    "SGRON");	// 3265
        th(     "RJOD");	// 3291
        th(     "LHAS");	// 3293
        th(   "MKHA'I");	// 3295
        th(       "YO");	// 3312
        th(     "BONG");	// 3339
        th(      "YAB");	// 3357
        th(   "DBYUNG");	// 3360
        th(     "DRIS");	// 3374
        th(      "SLU");	// 3376
        th(      "RAG");	// 3379
        th(     "RTZI");	// 3391
        th(   "'GRO'I");	// 3393
        th(     "STAN");	// 3406
        th(     "DRIN");	// 3411
        th(     "RIMS");	// 3423
        th(    "'GAGS");	// 3426
        th(      "SLA");	// 3436
        th(    "'JOMS");	// 3447
        th(   "'BRANG");	// 3479
        th(    "'CHAR");	// 3482
        th(    "STEGS");	// 3485
        th(    "BZHES");	// 3486
        th(      "PHO");	// 3523
        th(   "PHYUNG");	// 3535
        th(       "ZE");	// 3545
        th(     "MJAL");	// 3550
        th(    "GRANG");	// 3551
        th(      "NAN");	// 3552
        th(   "'PHRIN");	// 3569
        th(    "BGYID");	// 3570
        th(     "GNAD");	// 3575
        th(    "GDUNG");	// 3579
        th(      "THO");	// 3615
        th(     "BRIS");	// 3637
        th(     "TSAL");	// 3637
        th(     "TSOD");	// 3656
        th(     "NYAL");	// 3661
        th(    "MYANG");	// 3665
        th(     "BZA'");	// 3679
        th(     "SHER");	// 3703
        th(    "NGANG");	// 3705
        th(    "SPROS");	// 3709
        th(     "NGAS");	// 3710
        th(    "BSKUL");	// 3720
        th(    "'PHEN");	// 3732
        th(     "PHAR");	// 3734
        th(     "BYIS");	// 3761
        th(      "MAL");	// 3761
        th(   "NA'ANG");	// 3763
        th(     "SDUD");	// 3766
        th(     "DMAR");	// 3783
        th(   "BRNYAN");	// 3787
        th(     "GTAM");	// 3797
        th(     "BCUG");	// 3800
        th(   "SPYI'I");	// 3800
        th(     "GRVA");	// 3803
        th(     "'BUM");	// 3816
        th(    "MNGA'");	// 3818
        th(     "LONG");	// 3840
        th(     "BZED");	// 3845
        th(      "RAM");	// 3855
        th(      "MUN");	// 3865
        th(    "SRUNG");	// 3870
        th(     "CANG");	// 3875
        th(      "RES");	// 3877
        th(    "BSNGO");	// 3882
        th(     "CHOD");	// 3885
        th(     "SLAD");	// 3886
        th(   "BSHAGS");	// 3896
        th(     "SKYA");	// 3897
        th(    "DBYAR");	// 3902
        th(       "HA");	// 3903
        th(     "BRDA");	// 3924
        th(     "KA'I");	// 3931
        th(      "RNA");	// 3931
        th(    "GLENG");	// 3934
        th(      "BOD");	// 3938
        th(       "NU");	// 3943
        th(   "GZUNGS");	// 3954
        th(      "SUN");	// 3955
        th(     "GDAN");	// 3960
        th(      "SAR");	// 3973
        th(     "SHIS");	// 3982
        th(    "GDAGS");	// 3992
        th(     "BKRA");	// 4007
        th(      "GAM");	// 4026
        th(    "PHRAN");	// 4040
        th(     "BYON");	// 4045
        th(      "KLU");	// 4045
        th(    "CHE'I");	// 4050
        th(     "SO'I");	// 4069
        th(     "NGOR");	// 4098
        th(      "RAS");	// 4109
        th(    "THANG");	// 4113
        th(     "RAGS");	// 4116
        th(     "'BAR");	// 4121
        th(    "KHRID");	// 4133
        th(     "KHRO");	// 4133
        th(      "BAM");	// 4135
        th(   "BSGOMS");	// 4146
        th(     "TZ'A");	// 4164
        th(    "BRTZE");	// 4183
        th(     "LE'U");	// 4195
        th(     "SBOM");	// 4207
        th(      "'OS");	// 4245
        th(     "SH'A");	// 4256
        th(     "'GOD");	// 4267
        th(   "SGRA'I");	// 4282
        th(   "RMONGS");	// 4292
        th(     "G-YO");	// 4319
        th(     "RI'I");	// 4352
        th(      "MGO");	// 4376
        th(  "SBYANGS");	// 4392
        th(   "'DRA'I");	// 4403
        th(    "RLABS");	// 4418
        th(    "GROGS");	// 4425
        th(  "DBYINGS");	// 4433
        th(    "RLUNG");	// 4438
        th(   "BSRUNG");	// 4443
        th(     "SA'I");	// 4453
        th(    "MNYAN");	// 4462
        th(     "TSES");	// 4482
        th(     "SRED");	// 4486
        th(      "NUB");	// 4495
        th(    "DMYAL");	// 4501
        th(      "CAR");	// 4514
        th(     "SENG");	// 4552
        th(    "DKYIL");	// 4558
        th(   "SKYONG");	// 4560
        th(       "YA");	// 4580
        th(     "MTHU");	// 4590
        th(      "YUN");	// 4601
        th(     "DRAG");	// 4656
        th(      "RIS");	// 4685
        th(     "TSAN");	// 4694
        th(     "'TSO");	// 4709
        th(      "YUM");	// 4718
        th(  "BSGRUBS");	// 4740
        th(      "SHU");	// 4749
        th(     "GDON");	// 4750
        th(     "PHAL");	// 4758
        th(     "MJUG");	// 4763
        th(     "'DON");	// 4771
        th(    "GZHIR");	// 4786
        th(     "'DA'");	// 4792
        th(    "BSAGS");	// 4808
        th(     "SKUR");	// 4811
        th(     "'BUL");	// 4846
        th(      "RMI");	// 4846
        th(  "BRTZAMS");	// 4851
        th(     "MTHO");	// 4870
        th(    "BSKOR");	// 4878
        th(      "BCO");	// 4907
        th(     "DBUS");	// 4911
        th(    "GZHON");	// 4912
        th(    "BA'AM");	// 4925
        th(    "MDZES");	// 4950
        th(     "BGYI");	// 4953
        th(    "BSTOD");	// 5061
        th(     "SKOR");	// 5067
        th(    "RTZOM");	// 5087
        th(    "GTZUG");	// 5097
        th(     "BYAD");	// 5139
        th(    "KHYER");	// 5149
        th(     "MDUN");	// 5152
        th(      "BUS");	// 5164
        th(    "PHRAD");	// 5173
        th(    "SHUGS");	// 5184
        th(    "SDANG");	// 5196
        th(    "DRUNG");	// 5208
        th(      "YAS");	// 5211
        th(       "CE");	// 5230
        th(    "RJE'I");	// 5242
        th(      "DOG");	// 5251
        th(   "DRANGS");	// 5282
        th(      "TSA");	// 5310
        th(      "YIG");	// 5310
        th(   "'BRING");	// 5314
        th(      "MYU");	// 5317
        th(      "DOR");	// 5323
        th(     "BRAM");	// 5340
        th(       "RU");	// 5361
        th(   "PHRENG");	// 5367
        th(     "GNYI");	// 5405
        th(    "'DREN");	// 5409
        th(    "GLANG");	// 5419
        th(    "LHUNG");	// 5448
        th(    "SGRAS");	// 5458
        th(   "GZHI'I");	// 5460
        th(    "RGYUR");	// 5490
        th(     "SHAS");	// 5492
        th(      "DAR");	// 5533
        th(     "SGRO");	// 5539
        th(      "NAG");	// 5551
        th(    "'DRES");	// 5588
        th(     "SHOG");	// 5591
        th(     "RGOL");	// 5603
        th(     "DBEN");	// 5621
        th(    "SPYIR");	// 5628
        th(     "'BAD");	// 5651
        th(       "AA");	// 5689
        th(    "'GRUS");	// 5707
        th(     "LANG");	// 5739
        th(     "LA'O");	// 5755
        th(   "'CHANG");	// 5764
        th(     "BU'O");	// 5785
        th(     "MYUR");	// 5802
        th(      "GUS");	// 5828
        th(   "BZHI'I");	// 5834
        th(     "GDUL");	// 5879
        th(     "DGON");	// 5889
        th(     "BKUR");	// 5895
        th(      "NAR");	// 5915
        th(     "BCA'");	// 5920
        th(     "SMAD");	// 5926
        th(      "MAN");	// 5935
        th(   "'PHRUL");	// 5981
        th(    "SPYAN");	// 5997
        th(    "SNYED");	// 6018
        th(    "SNYAN");	// 6035
        th(     "GROS");	// 6043
        th(   "GTZANG");	// 6052
        th(    "BSAMS");	// 6069
        th(     "BYAR");	// 6072
        th(     "BLTA");	// 6095
        th(    "GRONG");	// 6101
        th(     "SKAL");	// 6103
        th(     "'GAG");	// 6152
        th(    "PHYED");	// 6152
        th(     "ZHAG");	// 6152
        th(    "GNYER");	// 6163
        th(    "GDAMS");	// 6165
        th(   "RGYU'I");	// 6218
        th(      "GSO");	// 6293
        th(     "NGOS");	// 6303
        th(    "PHYUG");	// 6320
        th(    "PHEBS");	// 6323
        th(    "GZHAL");	// 6357
        th(     "NA'O");	// 6360
        th(     "DOGS");	// 6372
        th(     "GSAR");	// 6396
        th(     "'BA'");	// 6431
        th(    "GSANG");	// 6452
        th(    "'THOB");	// 6466
        th(     "BTAB");	// 6479
        th(     "SLAR");	// 6485
        th(     "CHUD");	// 6503
        th(   "PA'ANG");	// 6513
        th(     "KHRI");	// 6515
        th(      "BOS");	// 6549
        th(     "CHAR");	// 6558
        th(     "SMAN");	// 6571
        th(     "MO'I");	// 6592
        th(    "RKANG");	// 6598
        th(     "PHAM");	// 6605
        th(     "GOMS");	// 6614
        th(    "BSKAL");	// 6618
        th(    "'BYIN");	// 6662
        th(      "SER");	// 6663
        th(    "KHONG");	// 6673
        th(    "BRTAG");	// 6676
        th(   "LA'ANG");	// 6722
        th(     "DKON");	// 6748
        th(      "ZHU");	// 6753
        th(    "BSTEN");	// 6770
        th(      "GAR");	// 6774
        th(    "PHYIS");	// 6788
        th(     "CHOG");	// 6804
        th(       "CU");	// 6834
        th(  "DBYANGS");	// 6863
        th(    "LHA'I");	// 6865
        th(    "BZHED");	// 6885
        th(     "SGYU");	// 6925
        th(      "RTA");	// 6942
        th(    "BLANG");	// 6948
        th(      "TOG");	// 6949
        th(      "THE");	// 6951
        th(      "'AM");	// 6965
        th(    "DPYOD");	// 6966
        th(     "BO'I");	// 6970
        th(    "GLING");	// 6971
        th(      "NAD");	// 6982
        th(       "GU");	// 7001
        th(    "KHYED");	// 7011
        th(     "SHAR");	// 7050
        th(   "BRGYUD");	// 7093
        th(      "CHO");	// 7129
        th(    "PHRAG");	// 7162
        th(      "MOD");	// 7169
        th(    "MCHED");	// 7174
        th(     "TSOM");	// 7205
        th(    "'TSAL");	// 7210
        th(     "BDUD");	// 7234
        th(     "SNOD");	// 7258
        th(   "TSANGS");	// 7309
        th(     "RDUL");	// 7323
        th(   "BSGYUR");	// 7341
        th(      "BUD");	// 7344
        th(       "MU");	// 7357
        th(     "PHUN");	// 7358
        th(    "THOGS");	// 7486
        th(     "MDOR");	// 7496
        th(    "BSHES");	// 7534
        th(    "DRANG");	// 7715
        th(    "SPYAD");	// 7741
        th(    "MDZOD");	// 7745
        th(     "BSDU");	// 7829
        th(      "SHA");	// 7857
        th(     "DMAN");	// 7896
        th(      "SHI");	// 7920
        th(       "TA");	// 7920
        th(    "DPA'I");	// 7967
        th(     "TSUN");	// 8079
        th(     "CHED");	// 8112
        th(    "GTONG");	// 8135
        th(     "MGON");	// 8138
        th(     "BSAL");	// 8152
        th(    "BYAMS");	// 8184
        th(     "CUNG");	// 8187
        th(   "SNGAGS");	// 8191
        th(    "MCHIS");	// 8196
        th(       "RA");	// 8196
        th(     "PHUL");	// 8203
        th(      "SAM");	// 8203
        th(     "PHYE");	// 8359
        th(     "CHER");	// 8387
        th(    "ZHABS");	// 8436
        th(    "'DEBS");	// 8446
        th(       "GE");	// 8453
        th(     "MTSO");	// 8479
        th(    "GZIGS");	// 8490
        th(    "'DOGS");	// 8587
        th(     "SRAS");	// 8627
        th(      "ZAB");	// 8628
        th(     "RTZE");	// 8642
        th(     "ZHUS");	// 8646
        th(    "'PHEL");	// 8664
        th(      "BON");	// 8668
        th(     "ZHAL");	// 8727
        th(     "SDIG");	// 8766
        th(     "NYIN");	// 8815
        th(     "BKOD");	// 8857
        th(     "'JAM");	// 8935
        th(      "SEL");	// 8938
        th(     "GSER");	// 8949
        th(     "'DUL");	// 8996
        th(   "SKYABS");	// 9089
        th(      "ZAS");	// 9108
        th(     "BKAG");	// 9155
        th(      "DGU");	// 9249
        th(    "DPYAD");	// 9332
        th(     "'CHI");	// 9361
        th(     "RNAL");	// 9425
        th(    "'BYED");	// 9503
        th(     "SROG");	// 9504
        th(     "TSOR");	// 9515
        th(    "STZAL");	// 9517
        th(     "CHES");	// 9524
        th(      "MOS");	// 9529
        th(     "SHIG");	// 9555
        th(    "BZLOG");	// 9594
        th(      "CAG");	// 9756
        th(    "BTZUN");	// 9773
        th(    "BTANG");	// 9779
        th(    "'ONGS");	// 9794
        th(     "MONG");	// 9896
    }

    /** Tests the tsheg bars that appear [10000, 99999] times. */
    public void test10000to99999() {
        th(     "SMON");	// 10060
        th(     "NYER");	// 10081
        th(    "RGYAN");	// 10119
        th(     "PHRA");	// 10293
        th(    "PA'AM");	// 10356
        th(    "RTZOD");	// 10384
        th(     "DKAR");	// 10463
        th(      "DBU");	// 10466
        th(    "LONGS");	// 10480
        th(    "MTSON");	// 10696
        th(      "NGE");	// 10865
        th(   "BZHUGS");	// 10970
        th(      "NOR");	// 11039
        th(    "STENG");	// 11040
        th(      "YIS");	// 11051
        th(    "MKHA'");	// 11065
        th(     "GCOD");	// 11122
        th(   "BRTZON");	// 11166
        th(     "DKA'");	// 11212
        th(    "SPRUL");	// 11238
        th(      "MYA");	// 11259
        th(     "NAMS");	// 11260
        th(       "ZA");	// 11299
        th(     "THAG");	// 11357
        th(    "BSGOM");	// 11405
        th(     "'DIS");	// 11422
        th(     "THUB");	// 11440
        th(    "GTOGS");	// 11450
        th(     "THUG");	// 11451
        th(    "BRTAN");	// 11478
        th(      "BAG");	// 11516
        th(    "'JIGS");	// 11566
        th(   "BRTAGS");	// 11581
        th(     "TING");	// 11615
        th(     "GROL");	// 11634
        th(    "SGRIB");	// 11715
        th(     "LAGS");	// 11727
        th(     "THOG");	// 11801
        th(      "'OG");	// 11844
        th(      "REG");	// 11866
        th(   "SBYONG");	// 11878
        th(     "BCOS");	// 11918
        th(    "MKHAN");	// 11919
        th(   "MTSAMS");	// 11943
        th(     "CI'I");	// 11976
        th(    "TSANG");	// 12119
        th(   "BSNYEN");	// 12224
        th(    "'DI'I");	// 12241
        th(      "DPE");	// 12243
        th(     "ZHEN");	// 12256
        th(      "BOR");	// 12275
        th(     "THUN");	// 12456
        th(     "BZOD");	// 12489
        th(    "RDZOB");	// 12500
        th(     "'ONG");	// 12769
        th(     "DGRA");	// 12842
        th(      "BUR");	// 12879
        th(     "SMIN");	// 12940
        th(    "'GRUB");	// 13019
        th(    "KHANG");	// 13114
        th(    "GNANG");	// 13229
        th(    "KHYIM");	// 13452
        th(     "BCAD");	// 13475
        th(   "'KHRUL");	// 13592
        th(      "SNA");	// 13592
        th(      "NGA");	// 13605
        th(      "'OD");	// 13609
        th(     "LHAN");	// 13638
        th(   "SNYOMS");	// 13661
        th(       "RI");	// 13730
        th(    "'CHAD");	// 13829
        th(     "BRAG");	// 13883
        th(     "'DUG");	// 14027
        th(    "THUGS");	// 14031
        th(      "'ON");	// 14054
        th(     "BSOD");	// 14244
        th(      "DRI");	// 14261
        th(   "GZHUNG");	// 14313
        th(    "RNYED");	// 14440
        th(    "BYA'I");	// 14509
        th(    "BSLAB");	// 14640
        th(    "MYONG");	// 14665
        th(    "GZHAG");	// 14795
        th(     "SDOM");	// 14839
        th(    "SKYED");	// 14857
        th(   "KHRIMS");	// 15073
        th(     "GTZO");	// 15189
        th(    "SNYAD");	// 15327
        th(    "BRGYA");	// 15418
        th(      "NYI");	// 15426
        th(    "GRAGS");	// 15445
        th(       "'O");	// 15458
        th(      "RDO");	// 15473
        th(      "ZLA");	// 15565
        th(      "SOR");	// 15571
        th(     "SPYI");	// 15620
        th(    "BZUNG");	// 15674
        th(     "GONG");	// 15716
        th(     "'GOG");	// 15737
        th(     "SGOM");	// 15744
        th(      "GOS");	// 15778
        th(     "'JOG");	// 15833
        th(      "MAS");	// 15850
        th(     "RING");	// 15868
        th(    "PHYAG");	// 15877
        th(     "'GA'");	// 16026
        th(     "LTOS");	// 16074
        th(     "MING");	// 16092
        th(      "POS");	// 16181
        th(     "DPER");	// 16266
        th(    "SPONG");	// 16376
        th(    "GZUNG");	// 16520
        th(     "DPAL");	// 16577
        th(    "GNYEN");	// 16697
        th(    "GSUNG");	// 16729
        th(  "MTSUNGS");	// 16812
        th(    "BZHAG");	// 16909
        th(      "CHU");	// 16957
        th(     "DRAN");	// 17181
        th(    "MTHAR");	// 17216
        th(   "GRANGS");	// 17390
        th(     "DPON");	// 17427
        th(    "RGYUN");	// 17452
        th(    "MCHOD");	// 17469
        th(      "BYE");	// 17491
        th(    "SPANG");	// 17561
        th(    "MKHAS");	// 17613
        th(   "BLANGS");	// 17618
        th(     "BKA'");	// 17623
        th(     "DBYE");	// 17652
        th(       "DA");	// 17677
        th(    "THABS");	// 17920
        th(    "STOBS");	// 17922
        th(     "DPAG");	// 17995
        th(    "BTAGS");	// 18116
        th(     "THAR");	// 18224
        th(     "'DUS");	// 18249
        th(      "ZHI");	// 18412
        th(     "SMOS");	// 18571
        th(     "BDUN");	// 18585
        th(    "RKYEN");	// 18708
        th(    "PHUNG");	// 18749
        th(    "SKYON");	// 18841
        th(   "DGONGS");	// 18886
        th(      "MIG");	// 19084
        th(    "MNYAM");	// 19140
        th(    "RDZAS");	// 19158
        th(    "CHUNG");	// 19178
        th(      "SDE");	// 19232
        th(   "SPANGS");	// 19238
        th(    "'THAD");	// 19307
        th(     "SHIN");	// 19429
        th(    "BSDUS");	// 19466
        th(       "GA");	// 19504
        th(      "RIN");	// 19678
        th(     "NGAG");	// 19835
        th(    "TSIGS");	// 19878
        th(    "SNGAR");	// 19995
        th(    "KHAMS");	// 20144
        th(    "ZHUGS");	// 20188
        th(      "BUM");	// 20311
        th(    "SBYAR");	// 20319
        th(     "SONG");	// 20727
        th(     "BU'I");	// 20969
        th(   "GSHEGS");	// 21211
        th(      "LAN");	// 21274
        th(     "GSOL");	// 21319
        th(     "LUGS");	// 21386
        th(     "BYIN");	// 21535
        th(      "ZIN");	// 21542
        th(     "LDOG");	// 21775
        th(      "TAN");	// 21958
        th(     "NYAN");	// 21959
        th(     "DGAG");	// 22014
        th(     "GNOD");	// 22178
        th(   "BRGYAD");	// 22232
        th(       "YE");	// 22248
        th(     "BRAL");	// 22602
        th(       "ME");	// 22709
        th(      "MAR");	// 22714
        th(      "CHA");	// 22715
        th(     "'DUN");	// 23291
        th(     "SMRA");	// 23319
        th(   "SNYING");	// 23340
        th(      "LHA");	// 23477
        th(     "PHAN");	// 23588
        th(    "'GREL");	// 24160
        th(     "CHAD");	// 24225
        th(     "KHAS");	// 24378
        th(      "LOG");	// 24408
        th(      "YAN");	// 24442
        th(     "NYES");	// 24455
        th(    "NYAMS");	// 24582
        th(      "SKU");	// 24630
        th(     "NGAN");	// 24676
        th(   "BSGRUB");	// 24741
        th(    "'BYOR");	// 24807
        th(      "ZAD");	// 24892
        th(       "KA");	// 24999
        th(      "RIM");	// 25159
        th(    "MONGS");	// 25200
        th(    "SMRAS");	// 25448
        th(    "LTUNG");	// 25490
        th(    "SBYIN");	// 25800
        th(      "POR");	// 25853
        th(     "LEGS");	// 25954
        th(     "'GAL");	// 26029
        th(   "MKHYEN");	// 26054
        th(     "NANG");	// 26069
        th(      "NAM");	// 26429
        th(     "LUNG");	// 26597
        th(     "SLOB");	// 26621
        th(    "BZANG");	// 26706
        th(      "YON");	// 26791
        th(     "SNGA");	// 26801
        th(     "DGA'");	// 27013
        th(     "MANG");	// 27047
        th(     "'DRA");	// 27399
        th(    "BRTEN");	// 27666
        th(      "SGO");	// 27883
        th(       "TO");	// 27900
        th(     "RGYA");	// 28075
        th(     "DRUG");	// 28228
        th(     "THEG");	// 28272
        th(     "THOS");	// 28272
        th(    "SNGON");	// 28449
        th(      "'DU");	// 28560
        th(     "LHAG");	// 28760
        th(   "BSNGAL");	// 28785
        th(    "SNYAM");	// 28786
        th(   "'PHAGS");	// 28876
        th(   "BSKYED");	// 29122
        th(     "NYON");	// 29306
        th(      "MDO");	// 30152
        th(    "'BREL");	// 30707
        th(     "RTZA");	// 30760
        th(    "MCHOG");	// 31025
        th(     "BA'O");	// 31089
        th(     "DPA'");	// 31641
        th(    "RTAGS");	// 31740
        th(      "BLA");	// 31918
        th(      "LAG");	// 32281
        th(      "CHE");	// 32322
        th(      "LEN");	// 32330
        th(      "BCU");	// 32402
        th(     "SRID");	// 32452
        th(    "MTHA'");	// 32850
        th(     "LNGA");	// 33119
        th(      "ZER");	// 33348
        th(     "GSAL");	// 33528
        th(      "NYE");	// 33585
        th(      "PHA");	// 33747
        th(    "RGYUD");	// 34022
        th(      "ZAG");	// 34145
        th(     "GTAN");	// 34739
        th(     "SKAD");	// 34874
        th(      "SUM");	// 34941
        th(       "YI");	// 35252
        th(     "TSIG");	// 35406
        th(       "MO");	// 35624
        th(    "CHAGS");	// 35668
        th(     "SDUG");	// 35672
        th(     "PHYI");	// 35736
        th(     "MA'I");	// 35807
        th(   "RDZOGS");	// 35916
        th(     "'DIR");	// 36091
        th(      "RJE");	// 36307
        th(       "LO");	// 36547
        th(      "BDE");	// 36643
        th(      "MIN");	// 36833
        th(      "GAL");	// 36835
        th(    "MDZAD");	// 36836
        th(      "YID");	// 37061
        th(       "GO");	// 37096
        th(    "KHYOD");	// 37100
        th(    "SBYOR");	// 37179
        th(     "SGRA");	// 37707
        th(     "RUNG");	// 37720
        th(    "SKABS");	// 38097
        th(     "BSAM");	// 38500
        th(      "DER");	// 38969
        th(      "RIG");	// 39409
        th(     "STON");	// 39550
        th(    "MTHUN");	// 39567
        th(      "LUS");	// 39934
        th(       "RE");	// 40027
        th(     "GZHI");	// 40656
        th(      "BLO");	// 40696
        th(    "'KHOR");	// 40766
        th(      "NUS");	// 41451
        th(     "CING");	// 41455
        th(    "PHYIN");	// 41483
        th(     "THOB");	// 41642
        th(      "DAD");	// 42159
        th(     "'JIG");	// 42604
        th(    "SLONG");	// 42620
        th(    "SHING");	// 42659
        th(     "RTOG");	// 42684
        th(     "BCOM");	// 42910
        th(    "SPYOD");	// 43089
        th(      "ROL");	// 43237
        th(     "BDEN");	// 43429
        th(      "KHA");	// 43515
        th(    "KHYAB");	// 44753
        th(     "RTAG");	// 44907
        th(     "TSAD");	// 45509
        th(     "BZHI");	// 45748
        th(   "PHYOGS");	// 46373
        th(      "DUS");	// 46633
        th(     "PO'I");	// 46783
        th(       "CI");	// 47068
        th(    "TSOGS");	// 47285
        th(     "BCAS");	// 47881
        th(      "CES");	// 48059
        th(       "SA");	// 48220
        th(    "STONG");	// 48516
        th(    "YONGS");	// 48585
        th(     "'GRO");	// 48688
        th(    "SKYES");	// 48866
        th(    "GZUGS");	// 48978
        th(    "SNANG");	// 49182
        th(      "ZHE");	// 49341
        th(    "BYA'O");	// 49563
        th(    "KHYAD");	// 49875
        th(    "ZHING");	// 51139
        th(     "'JUG");	// 52077
        th(    "DMIGS");	// 52080
        th(      "KHO");	// 52173
        th(     "DGOS");	// 52458
        th(     "TZAM");	// 52653
        th(    "SANGS");	// 52683
        th(    "BYUNG");	// 53443
        th(    "SGRUB");	// 53481
        th(      "DAM");	// 53988
        th(     "CHUB");	// 54381
        th(      "YUL");	// 54802
        th(    "'BRAS");	// 55237
        th(    "RGYAL");	// 55752
        th(     "'DAS");	// 56415
        th(     "NGES");	// 57727
        th(    "BRJOD");	// 58571
        th(      "TSE");	// 58838
        th(      "DES");	// 58881
        th(   "'BYUNG");	// 59579
        th(     "RJES");	// 59593
        th(     "RTEN");	// 59696
        th(      "THA");	// 61532
        th(   "MTHONG");	// 61747
        th(      "CIG");	// 65681
        th(     "RIGS");	// 65876
        th(    "RTOGS");	// 66399
        th(    "BSHAD");	// 69779
        th(    "RGYAS");	// 69838
        th(     "TSUL");	// 70412
        th(       "JI");	// 70440
        th(    "MNGON");	// 71081
        th(       "DO");	// 71430
        th(     "PA'O");	// 72392
        th(    "'DZIN");	// 72661
        th(       "BO");	// 73564
        th(    "BYANG");	// 74668
        th(      "BAS");	// 75377
        th(     "RGYU");	// 75404
        th(      "KUN");	// 77923
        th(      "NGO");	// 78198
        th(    "DBANG");	// 78533
        th(     "ZHIG");	// 80445
        th(     "SKYE");	// 81691
        th(     "GYUR");	// 81880
        th(     "GCIG");	// 84136
        th(      "LAM");	// 84140
        th(     "GRUB");	// 84788
        th(      "GIS");	// 86287
        th(      "RAB");	// 87368
        th(     "THAL");	// 88079
        th(    "THAMS");	// 88235
        th(      "CAD");	// 88319
        th(   "GSUNGS");	// 88380
        th(    "DNGOS");	// 90450
        th(    "MTSAN");	// 93425
        th(     "CHEN");	// 98190
        th(     "BYAS");	// 99364
        th(     "GSUM");	// 99974
    }

    /** Tests the tsheg bars that appear 100,000 or more times. */
    public void test100000toInf() {
        th(      "DGE");	// 101072
        th(    "BSTAN");	// 102835
        th(     "GYIS");	// 102983
        th(       "BU");	// 103208
        th(       "RO");	// 109213
        th(      "LTA");	// 109408
        th(     "'DOD");	// 109829
        th(     "LDAN");	// 111697
        th(     "DE'I");	// 112492
        th(     "BDAG");	// 113531
        th(     "GNAS");	// 115153
        th(       "NO");	// 119787
        th(       "SO");	// 128280
        th(    "KYANG");	// 129121
        th(     "KYIS");	// 129642
        th(      "STE");	// 132942
        th(    "GZHAN");	// 145413
        th(     "LTAR");	// 150418
        th(    "GNYIS");	// 153072
        th(       "TU");	// 154024
        th(      "CAN");	// 159684
        th(     "SEMS");	// 160271
        th(    "BZHIN");	// 160585
        th(     "RANG");	// 161014
        th(      "YOD");	// 162259
        th(      "'DI");	// 164823
        th(    "'GYUR");	// 165910
        th(    "RNAMS");	// 170809
        th(      "DON");	// 175298
        th(     "SOGS");	// 184207
        th(     "RNAM");	// 184434
        th(     "GANG");	// 186547
        th(       "SU");	// 189979
        th(       "GI");	// 191035
        th(     "SHES");	// 195701
        th(     "CHOS");	// 198212
        th(     "BYED");	// 214395
        th(      "PAS");	// 223992
        th(      "DAG");	// 239535
        th(      "GYI");	// 239795
        th(      "NAS");	// 247143
        th(       "TE");	// 249145
        th(      "BAR");	// 256761
        th(     "BA'I");	// 280363
        th(      "LAS");	// 282308
        th(       "PO");	// 284576
        th(     "YANG");	// 296797
        th(      "BYA");	// 313689
        th(      "MED");	// 315073
        th(     "NYID");	// 318046
        th(       "MI");	// 338010
        th(     "ZHES");	// 354539
        th(    "PHYIR");	// 357923
        th(      "KYI");	// 359808
        th(       "DU");	// 432125
        th(       "NA");	// 486995
        th(      "YIN");	// 526603
        th(       "MA");	// 533020
        th(       "NI");	// 534609
        th(       "DE");	// 689355
        th(      "PAR");	// 739128
        th(       "BA");	// 791847
        th(       "LA");	// 827807
        th(     "DANG");	// 869141
        th(     "PA'I");	// 893696
        th(       "PA");	// 1954376
    }
}
// DLC FIXME: verify the most common tsheg bars by hand, carefully.
